//  QbTimeWidgets.h
//
// Time Edition Widgets
//
// November 2008 A Rochel
//

#ifndef QBTIME_WIDGETS_H
#define QBTIME_WIDGETS_H

#include <QObject>
#include "QdTime.h"     // QDOS
#include "KVDataBase.h"
#include "Viostream.h"

#include <qvalidator.h>
#include "qt_version.h"
#include <qtooltip.h>

#include <QDateTimeEdit>
#include <QLineEdit>
#include <QDropEvent>
#include <QLayout>
#include <QPushButton>
#include <QSpinBox>
#include <QInputDialog>

#include "qslot.h"

class QbDurationWidget:public QLineEdit
{
	Q_OBJECT

	public:
		QbDurationWidget(QWidget *);
		
		bool getDuration(double& );
		
		void setDuration(double);
		void setActive(bool);
		void setAltPalette(bool);
		
	signals:
		void gotFocus();
		
	protected slots:
		void focusInEvent(QFocusEvent *);
		
		void dropEvent(QDropEvent *event);
		
		void dragEnterEvent(QDragEnterEvent *event);
		
	protected:
		bool active;
		bool altPalette;
		QColor activeColor, inactiveColor;
		QColor activeColorAlt, inactiveColorAlt;
		QColor activeText, inactiveText;

		QRegExpValidator * validator;
};

class QbTimeValidator: public QValidator
{
	Q_OBJECT

	public:
		QbTimeValidator(QObject* parent);
		
		~QbTimeValidator();
		
		virtual void fixup(QString& input) const;
		
		virtual QValidator::State validate(QString& input, int& pos) const;
};

class QbDateTimeLineEdit:public QLineEdit
{
	Q_OBJECT
			
	public:
		QbDateTimeLineEdit ( QWidget * parent = 0 );

		
		QmjdTime time();
		
		void setTime(const QmjdTime&);
		
		void setActive(bool);
		void setAltPalette(bool);
		
		bool edited() {return hasBeenEdited;}
		
		void addToCurrent(int delta=0);
		bool active;
		bool altPalette;
	signals:
		void gotFocus();
		void valueChanged();
		
	protected slots:
		void focusInEvent(QFocusEvent *);
		void slotValueChanged();
		void slotValueInvalid();
		void keyPressEvent(QKeyEvent *);
		
		void wheelEvent(QWheelEvent *);
		
		void increaseCurrent()
		{
			addToCurrent(1);
		}
		
		void decreaseCurrent()
		{
			addToCurrent(-1);
		}
		
		void mouseDoubleClickEvent ( QMouseEvent * event );

	protected:
		QbTimeValidator* validator;
		
		QColor activeColor, inactiveColor;
		QColor activeText, inactiveText;
		QColor activeColorAlt, inactiveColorAlt;
		bool hasBeenEdited;
		QmjdTime previousTime;
};

class QbDateTimeWidget:public QAbstractSpinBox
{
	Q_OBJECT
	
	public:
		QbDateTimeWidget(QWidget* parent=0);
		
		void stepBy(int steps);
		
		QmjdTime time(){return editor->time();}
		
		void setTime(const QmjdTime& t){editor->setTime(t);}
		
		void setActive(bool a){editor->setActive(a);}
		void setAltPalette(bool a){editor->setAltPalette(a);}
		
		bool edited(){return editor->edited();}

		bool hasFocus() const
		{
			return (QAbstractSpinBox::hasFocus()||lineEdit()->hasFocus());
		}
	
	signals:
		void gotFocus();
		void valueChanged();
		void textEdited(const QString &);
		
	protected slots:
		void dropEvent(QDropEvent *event);
		void dragEnterEvent(QDragEnterEvent *event);
		void droppedObjStart();
		void droppedObjCentre();
		void droppedObjEnd();
		
	protected:
	
		QbDateTimeLineEdit* editor;
		QAbstractSpinBox::StepEnabled stepEnabled () const;
		QDOS::QdObject_var droppedObject;
	
};

#endif
