//  Qcalc.h
//
//  Qsas - Calculator Header File
//
//  Nora Bounaira
//  Dec 2003
//

#ifndef QSAS_QCALC_H_
#define QSAS_QCALC_H_

#include "qt_version.h"
#include <QObject.h>
#include <qsplitter.h>
#include <qtabwidget.h>
#include <qstatusbar.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qdatetime.h>
#include <qtextedit.h>


#include "qgu.h"

#include "qslot.h"
#include "qOperators.h"


#define Qcalc_VERSION  " V_2.11 [ Mar 2010 ]"


typedef enum QcalcAliasType_e
{
	ALIAS_FOUND=0,
	ALIAS_NOT_FOUND=1,
	ALIAS_VIRTUAL=2
} QcalcAliasType;

typedef enum QcalcAliasObjType_e
{
	ALIAS_IS_UNKNOWN=0,
	ALIAS_IS_VECTOR=1,
	ALIAS_IS_ARRAY=2,
	ALIAS_IS_OTHER=3
} QcalcAliasObjType;

/*
#define JOIN_MENU_ITEM_ID           0
#define ARITHMETIC_MENU_ITEM_ID     1
#define FILTER_MENU_ITEM_ID         2
#define TRIG_MENU_ITEM_ID           3
#define VECTOR_OPS_MENU_ITEM_ID     5
#define MODIFY_MENU_ITEM_ID         5

#define RENAME_ALIAS_RB_MENU_ITEM        7
#define RENAME_ALIAS_EDITMENU_MENU_ITEM  8
*/

#define MAX_N_OPERANDS  2

#define NUMBER_OPERAND  "NUMBER"

enum ItemTypeEnum   { WORKING_LIST_OBJECT = 0,  VIRTUAL_OBJECT, NONE_OBJECT  };

class QcalcWindow;
class QcalcItem;
class QcalcTab;
class QcalcInput;
class QcalcListView;
class QcalcListModel;

//Vostream& operator<<(   Vostream&,    QcalcItem& );
//Vostream& WriteCalcItem( Vostream&,   QcalcItem& );

Vistream& operator>>(   Vistream&,    QcalcItem& );
Vistream& ReadCalcItem( Vistream&,    QcalcItem& );

#define  DELETE_POINTER( PTR ) \
      if ( PTR )               \
	  {                        \
	       delete PTR;         \
		   PTR = NULL;         \
	  }



class QcalcItem : public QObject
{

public:

    QcalcItem( QcalcListView *parent= 0,
				QString      txt   = "",
				QString      WL    = "");

    ~QcalcItem();
    void setChoices( ); 

    QString GetNextAliaseName();
    bool    AliasAlreadyUsed( QString name );
    QdObject_var getDataObject();
    Dim_var getVirtualArrDim();
    
	QcalcListView* _view;
	QString _alias;
	QString _name;
	QString WLcontext;
	int with;
	ItemTypeEnum    m_eObjType;
	QdObject_var    m_qdObjJoinedFirst;
    QString			m_alias_cpy;  // used ONLY to hold alias while writing modified one to save file
    QString			m_qstrOperator;
    int				m_nOp;
    QdObject_var	m_qdObjResult;
    bool			m_bCalcDone;
    bool			m_bFilled;
    QcalcTab*		m_pcalcTab;
    QString			m_NameForSave;
    int				isIvlObj;
    int				isTagObj;
    bool			isVector;    // only used by WL objects
	Dim_var			virtualArrDim;

    bool setArrayFlags();
    void SetItemName( bool bEmitChange = true );

    QdObject_var getResultObject();
    
    void DoMathOperation();
    void DoOperation();
    bool CheckOperatorSelfLink( QString& rqstrAliase );
	int childNumber();
	int numChildren() const {return _children.size();}
	void moveTo(int pos);
	bool addChild(QcalcItem* newChild, int position);
	bool removeChild(QcalcItem* child, bool alsoDelete);
	bool removeChild(int posn, bool alsoDelete);

    void reNameAlias( const QString& NewVal );
    bool getIsVector( );
    bool getIsArray( );
	void setColour(bool OK);
	void setAlias(QString alias);
	void setName(QString alias);
	bool ancestorInList(QList <QcalcItem*> &list);
	bool isDescendentOf(QcalcItem* other);
	QDomElement toXMLElement(QDomDocument& doc) const;

	QcalcItem* getChildByAlias(const QString& alias);
	QcalcItem* getChildByName(const QString& name);
	QcalcItem* getChildByNumber(int i) const;
	QList<QcalcItem*> _children;
	QcalcItem* _parent;
};


class QcalcListModel : public QAbstractItemModel
{
	Q_OBJECT
	
//	friend Vostream& operator<<(Vostream&, QcalcListModel&);
	friend Vistream& operator>>(Vistream&, QcalcListModel&);
	
public:
	QcalcListModel(QcalcListView* view=0);
	~QcalcListModel();
	
	// Minimum set of functions
		
	QModelIndex index (int row, int column, const QModelIndex& parent=QModelIndex()) const;

	QModelIndex parent(const QModelIndex& index) const;
		
	int rowCount(const QModelIndex& parent=QModelIndex()) const;

	int columnCount(const QModelIndex& parent=QModelIndex()) const;
	
	QVariant data(const QModelIndex& index, int role=Qt::DisplayRole) const;
	
	QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
	
	Qt::ItemFlags flags(const QModelIndex &index) const	;
	
	bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::DisplayRole);
		
	bool removeRows(int position, int rows, const QModelIndex& parent= QModelIndex());
	
	bool insertColumns(int position, int columns, const QModelIndex& parent= QModelIndex());
	
	bool removeColumns(int position, int columns, const QModelIndex& parent= QModelIndex());
		
	// Drag and drop tools
	
	QStringList mimeTypes() const;
	
	QMimeData* mimeData(const QModelIndexList& indexes) const;
	bool dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex &parent);
	Qt::DropActions supportedDropActions() const;

	// Utils
	
	QModelIndex moveUp(QModelIndex& index);
	
	QModelIndex moveDown(QModelIndex& index);

	inline QcalcItem* itemFromIndex(const QModelIndex& index) const
	{
		if(index.isValid())
		{
			return static_cast<QcalcItem*>(index.internalPointer());
		}
		return NULL;
	}
	QModelIndex indexFromItem(QcalcItem* item, int column);
	
	inline void emitLayoutAboutToBeChanged(){emit layoutAboutToBeChanged();}
	inline void emitLayoutChanged(){emit layoutChanged();}
	
	void clear(QModelIndex& parent);
	bool deleteItem(QcalcItem* item);

	QcalcItem* _root;
	QcalcListView* _view;
	bool _replace;
	
};



class QcalcListView;

//Vostream& operator<<( Vostream&, QcalcListView& );
Vistream& operator>>( Vistream&, QcalcListView& );

class QcalcListView : public QTreeView
{

	typedef void ( QCheckBox::*CheckBoxFnPtr )( bool bOn );

	Q_OBJECT
	friend Vostream& operator<<(Vostream&,QcalcListView&);
	friend Vistream& operator>>(Vistream&,QcalcListView&);

public:
	QcalcWindow* owner;
	QString      m_qstrWidgetText;
	QdObject_var    targetD0;
	Dim_var selectedObjDim;
	int with;
	int dropRow;
	
	QcalcListView( QWidget *parent, QcalcWindow* QjW, const char *name=0 );
	~QcalcListView();
	QcalcItem * findAlias(QString alias);
	QcalcItem * findName(QString name);
	QList<QcalcItem*> listItems();
	QList<QcalcItem*> selectedItems();
	
	inline QcalcListModel* calcModel() const
	{
		return static_cast<QcalcListModel*>(model());
	}
	void showTab(QcalcTab* tab);

	void displayMenu(const QPoint& p);
	
	QcalcItem* insertEntry(const QString& context);

	virtual void getAliasArrDim(QString,Dim_var &);
	virtual QcalcAliasObjType getAliasObjectType(QString);
	virtual QcalcAliasType getAliasObject(QString,QdObject_var);
	virtual QdObject_var getAliasResultObject(QString);
	inline void emitListModified(){emit listModified();}

signals:
	void pasteDetected();
	void listItemDeleted();
	void listModified();


public slots:
	void deleteListItems();
	void deleteAllItems();

	void insertVirtualItem();
	void tabChanged( QWidget* qwdgt );
	void validateWLitems();
	void propagateSelection(const char *, const char *);
	void moveItemUp();
	void moveItemDown();
	void copy();
	void paste();

protected slots:
  void mouseReleaseEvent(  QMouseEvent*    event );

  void dragEnterEvent(QDragEnterEvent *event);
  void dragMoveEvent(QDragMoveEvent *event);

  void mousePressEvent(    QMouseEvent*    event );
	
private:

	QString       m_strAliseName;
	char          m_chChar;
	QMenu * RButtonMenu;
	QcalcListModel* _model;

};


class QcalcTab : public QWidget
{
Q_OBJECT

public:
   QcalcTab();
   ~QcalcTab();

   QGroupBox*       details_groupbox;  ///< Object name group box
   QGridLayout*		details_layout;
   QLabel*			objAlias_label;
   QGroupBox*		operation_groupbox;
   QGridLayout*		operation_layout;
   QGridLayout*		operand_layout;
   QGridLayout*		extras_layout;
   QLabel*			operator_label;
   QGroupBox*		join_groupbox;
   QGridLayout*		join_layout;
   QLabel*			algorithm_label;
   QLabel*			width_label;
   QLabel*			blank_label;
   QLabel*			blank2_label;
   QLabel*			gap_label;
   QLabel*			gapSize_label;
   QguWLinput*      objAliase_field;///< Alias widget. This can be changed directly from its parent tab
   QMenu*      arithmetic;
   QMenu*      trig;
   QMenu*      vectorOps;
   QMenu*      filter;
   QMenu*      modify;
   QcalcInput*      AngleUnits;
   QCheckBox*       saveWL_check;
   QguWLinput*      _nameWL_field;
   QcalcListView*   calcListview;
   QPushButton*     tb4;
   QMenu*      menu;
   vector <QcalcInput*>  operandStack;
   int              nSlots;
   QcalcItem *virtualItem;
   QguWLinput* _width_field;
   QComboBox*  _gap;
   QguWLinput* _gap_field;
   QComboBox*  _algorithm;
   bool WLnameUnset;
   bool opWasJoin;
 
   void createCalcTabWidgets(QcalcItem *VirtItem);
   void SetOperatorByName(const char*);
   void SetSlotsForOperator( const QString&  strOperator );
   bool isOpAllowed();
   bool AddValid();
   bool DivideValid();
   bool VectorProdValid();
   bool FilterValid();
   bool MultiplyValid();
   bool PiecewiseValid();
   bool PowerValid();
   
   QAction *actAdd;
   QAction *actSub;
   QAction *actMult;
   QAction *actDiv;
   QAction *actMod;
   QAction *actPower;
   QAction *actChSign;
   QAction *actPeicewise;
   QAction *actPwDiv;

   QAction *actCos;
   QAction *actSin;
   QAction *actTan;
   QAction *actCot;
   QAction *actACos;
   QAction *actASin;
   QAction *actATan;
   QAction *actATan2;
   QAction *actACot;
   QAction *actCosh;
   QAction *actSinh;
   QAction *actTanh;
   QAction *actCoth;
   QAction *actACosh;
   QAction *actASinh;
   QAction *actATanh;
   QAction *actACoth;

   QAction *actVecProd;
   QAction *actDotProd;

   QAction *actRemainder;
   QAction *actAbsValue;
   QAction *actLog;
   QAction *actLog10;
   QAction *actInvers;
   QAction *actSqrt;
   QAction *actExp;

   QAction *actUnits;
   QAction *actFills;
   
   QAction *actJoin;


private:

protected:

signals:

public slots:
  void reNameAliasTab();
  void ValidateAlgorithm( int nIndex );  // If algorithm is set to LINEAR then the average edit box is disabled
  void setGap(int nIndex);
  void setWidthField();
  void setGapField();
  void SetTab();
  void SetTabFeedback( );
  void confirmAliasSlot();
  void SetOperand( );
  void SetLink( const QString & opStr );  
  void toggleWLName();  
  void setNameForSave( ); // sets m_NameForSave which is used ONLY to keep name while save toggle is off
  void SetOperator(QAction*);

  void setThisSelected();
  void updateFlags();
  void reValidateSlot();
};


class QcalcWindow : public QguWindow {

Q_OBJECT

	QGridLayout*   _top_gl;
	QSplitter *    _top_splitter;
	QMenu*    options_menu;
	QAction*        toggleAction;
	QLabel* spacing_label;

public slots:
  void save();
  void saveAs();
  void saveSession(QString);
  void defaultAll();
  void verboseToggle();
  bool joinObjects( QdObjectSeq_var JoinedList, vector <string> &nameList, bool skipVirtual);
  bool joinSlotObjects(QcalcTab *qTab, vector <QdObject_var>   &returnList );
  void keepChoices();
  void applyChoices();
  void setTTsource(int);
  void setTTivl(int);
  void setTimeFeedback();
  void messageHandler( QString txt, bool bError = false );

  void launchManExport();
  void launchManJoin();
  void launchManual();
  void launchVersion();

   // Slot called when do button is pressed
  void DoCalculation();

	// Display end message
	void DisplayEndMessage();

signals:
  void ClearComboBox();
  void ProcessListView();

public:
  QcalcWindow( QWidget *parent=0, const char *name=0 );
  virtual ~QcalcWindow();
  void setDefaults();
  QcalcListView* _join_listview;
  
  QString SAVE_ALIAS_EXT;
   
  QdTimeInterval_var QcalcGetIntervalFromData();
  bool QcalcGetScalarIntervalFromData(double &start, double &end);
  
  QdTimeSeq_var      getTimeTags( );
  QdRScalarSeq_var getScalarTags( );
  
  QdTimeInterval_var QcalcGetInterval(  );
  bool QcalcGetScalarInterval(double &start, double &end );

  virtual void save( Vostream& );
  virtual void restore( Vistream& v_is, bool replace=true );

  QGroupBox  *buttons_groupbox;
  QPushButton *dismissBtn;
  QPushButton *WriteBtn;
  QPushButton *JoinBtn;
  QWidget*    _top_widget;
  QPushButton *newCalcBtn;

  QComboBox*  _TimeIvlSelect;
  QguWLinput* _DataSource;
  QguWLinput* _IvlSource;
  QComboBox*  _TimeTagSelect;
  QguWLinput* _TagSource;

  QLabel*     IvlSource_label;
  QLabel*     TagSource_label;
  QLabel*     Start_field;
  QLabel*     End_field;
  QLabel*     Ntags_field;
  QLabel*     spacing_field;


  //QMenu* menu;

  QCheckBox*  multiJn_check1;
  QCheckBox*  multiJn_check2;

  // Tab widget
  QTabWidget* tabWidget;

  QMenu* edit_menu;



  QTextEdit*      logArea;
  QString  _width_field_Def;
  int      _gap_Def;
  QString  _gap_field_Def;
  int      _algorithm_Def;
  QString  _IvlSource_Def;
  int      _TimeIvlSelect_Def;
  QString  _TagSource_Def;
  int      _TimeTagSelect_Def;

  QString  _width_field_cpy;
  int      _gap_cpy;
  QString  _gap_field_cpy;
  int      _algorithm_cpy;
  QString  _IvlSource_cpy;
  int      _TimeIvlSelect_cpy;
  QString  _TagSource_cpy;
  int      _TimeTagSelect_cpy;

  double   qcalc_version;
  bool     verbose;

  MapCalculator m_mapOps;

protected:
  void dragEnterEvent( QDragEnterEvent *event);
  void keyPressEvent(QKeyEvent * e );
 
private:
  void CreateJoinTabWindow();
  void CreateFeedBackGroupBox();
  void CreatePopUpMenus();

  void GetJoinOptionObject( QcalcItem* next, QdObject_var obj, double &gap_or_width, QdString &gap_method, QdString &join_method, string &name  );

  bool CheckSelfLinkedVirtualObjs();
  bool JoinFirst();
  bool JoinLast();
  bool disjointSeries( QdObject_var obj );

};


class QcalcSlot : public QguWLinput{

Q_OBJECT


public:
    QcalcSlot( QWidget *parent= 0);

public slots:
   void dropEvent(QDropEvent *e);

    
};


class QcalcInput : public QWidget{

Q_OBJECT


public:
    QcalcInput( QWidget *parent= 0, QString name="", QString label="", QcalcTab *qTab=0 );

    void setText(QString);
    void setAnglePulldown();
    void SetConstant( const QString& strConstant );
    
    QLabel *inputLabel;
    QcalcSlot *slot;
    QComboBox *pulldown;
};




QcalcWindow * QCALCmakeWindow( QWidget *parent, const char * Title );
void QcalcShowCalc();
void QcalcAppendTextDisplay ( const char*  text );
QString getOpName(int OpNum);
QString getOpChar(int OpNum);

extern QcalcWindow *QCALC_widget;
extern QcalcWindow *QCEX_widget;



#endif // QSAS_QCALC_H_

