#ifndef QCALC_CONTEXT_H
#define QCALC_CONTEXT_H

#include "qcalcCommon.h"
#include "qcalcOutputItem.h"
#include "qcalcJoinItem.h"
#include "qcalcView.h"
#include "qcalcTimeLine.h"

// This file defines all the context widgets.

// Everything here should be quite self-explanatory

class BlackBoxContextWidget;
class PluginContextWidget;
class OutputContextWidget;
class DefaultContextWidget;
class SelectionContextWidget;
class ExtractorContextWidget;
class JoinContextWidget;
class TimeLineWidget;

extern TimeLineHandler timeLineHander;


class ContextWidget: public QWidget
{
    Q_OBJECT

    public:
        ContextWidget(QWidget* parent, CalcView* view);

        virtual ~ContextWidget(){}
        QTabWidget* tabWidget;
		
    public slots:
		
		void setView(QList<QGraphicsItem*>& items);

		
		void newInput(const QString&);
		void closeEvent(QCloseEvent* ce){ce->ignore();}


    protected:
        BlackBoxContextWidget* bbView;
        PluginContextWidget* pluginView;
        OutputContextWidget* outView;
		JoinContextWidget* joinView;
		DefaultContextWidget* defaultView;
		SelectionContextWidget* selectionView;
		ExtractorContextWidget* extractorView;
		TimeLineWidget* timeLineView;
		QLabel* noViewLabel;
};

class TimeLineWidget: public QWidget
{
	Q_OBJECT
	
	public:
		TimeLineWidget(QWidget* parent=NULL);
		
		virtual ~TimeLineWidget(){};
		
		void setTimeLineTab(QList<VOutputItem*>& outList);

		void setTTivlTab(int i);
		
		void setTTsourceTab(int i);

		void refreshDefault();
		
		QList<VOutputItem*> outItems;

		
	public slots:
		
		void newSelection(const QString&);
		
		void addedTimeLine(const QString&);
		
		void deletedTimeLine(const QString&);
		
		void deleteSelected();
		
		void saveChanges();
		
		void saveAs();
		
		void setTTivl(int i);
		
		void setTTsource(int i);
		
		void setTimeLine();
		
	protected:
		
		QComboBox* tlBox;
		
		QPushButton* deleteButton;
		
		QComboBox*  _TimeIvlSelect;
        QguWLinput* _IvlSource;
        QComboBox*  _TimeTagSelect;
        QguWLinput* _TagSource;
        QLabel*     IvlSource_label;
        QLabel*     TagSource_label;
		QLabel *msg;

		
};

class BlackBoxContextWidget: public QScrollArea
{
    Q_OBJECT

    public:
        BlackBoxContextWidget(QWidget* parent, CalcView* view);

        virtual ~BlackBoxContextWidget(){}

        void setBlackBoxInfo(VOperatorItem* bb);

    protected slots:

        void updateName();
        void updateOutputTexts();
        void updateInputTexts();
    
    protected:

        VOperatorItem* blackBox;

        QWidget* inputWidget;
        QVBoxLayout* inputLayout;
        QWidget* outputWidget;
        QVBoxLayout* outputLayout;
        
        QVector<QTextEdit*> inputDescriptions;
        QVector<QTextEdit*> outputDescriptions;
        QVector<QLabel*> inputLabels;
        QVector<QLabel*> outputLabels;
        QVector<QGroupBox*> inputBoxes;
        QVector<QGroupBox*> outputBoxes;
		
		QLineEdit* nameEditor;
        
};

class PluginContextWidget: public QTabWidget
{
    Q_OBJECT
    public:
        PluginContextWidget(QWidget* parent=NULL);

        virtual ~PluginContextWidget(){}

        void setPluginInfo(VOperatorItem* plugin);

    protected:

        QTextEdit* ioWidget;
        QTextEdit* helpWidget;
        QTextBrowser* docWidget;
};

class JoinContextWidget: public QWidget
{
	Q_OBJECT
	
	public:
		
		JoinContextWidget(QWidget* parent=NULL);
		
		virtual ~JoinContextWidget(){}
		
		void setJoinParameters(QList<VInputItem*>& inputItems, QList<VJoinItem*>& joinItems);

		QList<VInputItem*> inputItems;
		QList<VJoinItem*> joinItems;
		
	public slots:
		
		void setKeepXrefs(bool);
		void setJoinStatus(int);
		void setUserJoinStatus();
        
        void setAlgorithm();
        void setWidthField();
        void setGap();
        void setGapField();
		void checkSequence(const QString& text);
		
	protected:
		QLabel* warningLabel;
		QCheckBox* _noJoinBox;
		QCheckBox* _keepXrefBox;
        QguWLinput* _width_field;
        QComboBox*  _gap;
        QguWLinput* _gap_field;
        QComboBox*  _algorithm;
		bool userJoinChecked;
		
};

class OutputContextWidget: public QWidget
{
    Q_OBJECT
    public:
        OutputContextWidget(QWidget* parent=NULL, CalcView * view=NULL);

        virtual ~OutputContextWidget(){}

        void setOutputParameters(QList<VOutputItem*>& items);
		
		void setTab(TimeLine* tl);

    public slots:
		
		void newSelection(const QString&);
		
		void addedTimeLine(const QString&);
		
		void deletedTimeLine(const QString&);
        
    protected:

		QLabel* warningLabel;
		QComboBox* tlBox;
        CalcView *_view;
		
		QLabel* fromDataLabel;
		QLabel* regularTTLabel;
};

class DefaultContextWidget: public QWidget
{
	Q_OBJECT
    public:
        DefaultContextWidget(QWidget* parent, CalcView* view);

        virtual ~DefaultContextWidget(){}
		
	public slots:

		void addInput();
		void addOutput();
		void addExtractor();
		void addJoin();
		void updateBlackBoxMenu();
		
    protected:
		
		CalcView* view;
		QMenu* bbOp;
};

class SelectionContextWidget: public QWidget
{
	Q_OBJECT
    public:
        SelectionContextWidget(QWidget* parent, CalcView* view);

        virtual ~SelectionContextWidget(){}


	protected:
		CalcView* view;
};

class ExtractorContextWidget: public QScrollArea
{
	Q_OBJECT
    public:
        ExtractorContextWidget(ContextWidget* parent);

        virtual ~ExtractorContextWidget(){}
		
		void setExtractorParameters(QList<VExtractionItem*>& items);
		
	public slots:

		void CheckChanged(bool);
		void selectedComponent(QAction*);
		void selectedEvent(QAction*);
		void selectedMeta(QAction*);
		void selectedTime(QAction*);
		void SuffixChanged(const QString&);
		void EventChanged(const QString&);
		void MetaChanged(const QString&);
		void userSetComponent(const char *comp, const char *type);
		void setTarget(int);
		
	protected:
		QList<VExtractionItem*> items;
		
		QLabel* warningLabel;
		
		QComboBox * target;

		
		QCheckBox * compCheckBox;
		QPushButton * ComponentButton;
		QPushButton * EventButton;
		QPushButton * MetaButton;
		QMenu* ComponentMenu0;
		QMenu* ComponentMenu1;
		QMenu* ComponentMenu2;
		QMenu* ComponentMenu3;
		QMenu* ComponentMenuText;
		QMenu* ComponentMenuTime;
		QMenu* ComponentMenuTimeIvl;
		QMenu* ComponentMenuMeta;
		QPushButton * TimeButton;
		QMenu* TimeMenu;
		QMenu* TimetagMenu;
		QLineEdit* SuffixComponent;
		QLineEdit* SuffixEvent;
		QLineEdit* SuffixMeta;
		QLineEdit* SuffixTime;
		QLabel* Suffix;
		
		QCheckBox * arrayCheckBox;
		QPushButton * arrayComponentButton;
		QPushButton * arrayTimeButton;
		QMenu* arrayTimeMenu;
		QLineEdit* arraySuffixComponent;
		QLineEdit* arraySuffixTime;
		QLabel* arraySuffix;
		QLabel *talkBack;
		ContextWidget *ctxtWidget;
};

#endif
