#ifndef QCALC_GRAPHIC_ELEMENTS
#define QCALC_GRAPHIC_ELEMENTS

#include "qcalcCommon.h"

// All the little items that are drawn in the calculator

class CalcToolTip: public QGraphicsRectItem
{
	public:
		
		CalcToolTip(CalcItem* parent);
		
		virtual ~CalcToolTip(){}
		
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*);
		
		inline virtual int type() const { return Type;}
		enum { Type = TYPE_CALCTOOLTIP };
};

class Connector: public QGraphicsLineItem
{
	public:
		// Creates a connector from "from", to either a slot (to) OR a point (toPos) in case the connector is being dragged
		Connector(OutSlot* from, InSlot* to=NULL, const QPointF& toPos=QPointF(), QGraphicsItem* parent=NULL);
		
		// removes itself from the scene
		// removes itself from the graphic slots it's connected to,
		// which cancels the connection between the underlying virtual slots
		virtual ~Connector();
		
		// Updates the graphics to the match the slots positions
		void updatePosition();
		
		// Moves the end, in case it's not a slot
		void setEnd(const QPointF& toPos);
		
		// Connects to an input slot
		// Checks if not closing a loop
		// Removes previous connections to the slot
		// Creates the underlying virtual connection
		void setEnd(InSlot* to);
		
		inline OutSlot* getFromSlot(){return from;}
		inline InSlot* getToSlot(){return to;}
		
		QRectF boundingRect() const;
		QPainterPath shape() const;
		
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget=NULL);
		
		inline virtual int type() const { return Type;}
		enum { Type = TYPE_TIMECONNECTOR };
		
		int page;
		
	protected:
		QPointF fromPos;
		QPointF toPos;
		OutSlot* from;
		InSlot* to;
		
		QPolygonF arrowHead;
};

class RotoZoomButton: public QGraphicsRectItem
{
	public:
		// Creates a Rotation/Zoom button for the "parent" item.
		// The parent has to set up its position
		RotoZoomButton(QGraphicsItem* parent);
		
		virtual ~RotoZoomButton(){}
		
		CalcItem* getItem();/* {return (CalcItem*)parentItem();}*/
		
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*);
		
		inline virtual int type() const { return Type;}
		enum { Type = TYPE_ROTOZOOMBUTTON };
	protected:
		void hoverEnterEvent(QGraphicsSceneHoverEvent*);
		void hoverLeaveEvent(QGraphicsSceneHoverEvent*);
		bool hasMouseFocus;
};

class CloseButton: public QGraphicsRectItem
{
	public:
		CloseButton(QGraphicsItem* parent);
		
		virtual ~CloseButton(){std::cout << "deleting CloseButton " << this << std::endl;}
		
		CalcItem* getItem();
		
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*);
		
		inline virtual int type() const { return Type;}
		enum { Type = TYPE_CLOSEBUTTON };
		
	protected:
		void hoverEnterEvent(QGraphicsSceneHoverEvent*);
		void hoverLeaveEvent(QGraphicsSceneHoverEvent*);
		bool hasMouseFocus;
};

class ExpandButton: public QGraphicsRectItem
{
	public:
		ExpandButton(QGraphicsItem* parent);
		
		virtual ~ExpandButton(){}
		
		CalcItem* getItem();
		
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget=NULL);
		
		inline virtual int type() const { return Type;}
		enum { Type = TYPE_EXPANDBUTTON };
		
	protected:
		void hoverEnterEvent(QGraphicsSceneHoverEvent*);
		void hoverLeaveEvent(QGraphicsSceneHoverEvent*);
		bool hasMouseFocus;

};

class ErrorButton: public QGraphicsRectItem
{
	public:
		ErrorButton(CalcItem* parent);
		
		virtual ~ErrorButton(){}
		
		CalcItem* getItem();
		
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget=NULL);
		
		inline virtual int type() const { return Type;}
		enum { Type = TYPE_ERRORBUTTON };
		
	protected:

};

#endif
