//  qjui.h
//
//  Qsas - join Header File
//
//  Tony Allen
//  Dec 2001
//

#ifndef QSAS_DATA_JUIW_H_
#define QSAS_DATA_JUIW_H_

#include "qt_version.h"
#include <QObject.h>
#include <qsplitter.h>
#include "qgu.h"
#include <QListWidget>
#include <QListWidgetItem>
#include <QLinkedList>
#include "qslot.h"

class QjuiWindow;

#define QJUI_VERSION  " V_1.3 [30 Jul 2009]"

class QjuiWidgetItem;

Vostream& operator<<(Vostream&,QjuiWidgetItem&);
Vistream& operator>>(Vistream&,QjuiWidgetItem&);

class QjuiWidgetItem : public QListWidgetItem
{
	public:
		QjuiWidgetItem(const QString & text, QListWidget * parent);

		QjuiWidgetItem(const QjuiWidgetItem& wi);

		virtual ~QjuiWidgetItem();

		QString I_width_field;
		QString I_gap_field;

		int I_algorithm;
		int I_gap;

		void setChoices(int w);
		
		QjuiWindow* owner;

		QString WLcontext;

		// The following field is only used at restore time
		// As old versions of qjui could have folders,
		// folder entries are now loaded, marked "toBeDeleted", and then deleted
		bool toBeDeleted;
		
        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
		
	protected:
};

class QjuiWidget;

Vostream& operator<<(Vostream&,QjuiWidget&);
Vistream& operator>>(Vistream&,QjuiWidget&);

class QjuiWidget : public QListWidget
{
	Q_OBJECT

	public:
		QjuiWidget(QWidget * parent, QjuiWindow * owner);

		~QjuiWidget();

		QjuiWindow* owner;
		void setItemChoices(int w);

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
        		
	protected:
		virtual void dragEnterEvent(QDragEnterEvent* event);

		virtual void dropEvent(QDropEvent* event);

		virtual void mousePressEvent(QMouseEvent* event);

		void displayMenu(const QPoint& p);
		QMimeData * mimeData(const QList<QListWidgetItem *> items) const;
		QStringList mimeTypes () const;
		QMenu* RButtonMenu;
	public slots:
		void setAlgorithm();
		void setWidthField();
		void setGap();
		void setGapField();
		void updateOnSelection();
		void update1OnSelection();
		void update2OnSelection();
		void update3OnSelection();
		void update4OnSelection();
		void openEditor(QListWidgetItem*);
		void deleteListItems();
		void renameSelectedItem();
		void copy();
		void paste();
		void moveItemUp();
		void moveItemDown();
		
};

   
class QjuiWindow : public QguWindow {

Q_OBJECT

  QSplitter * _top_splitter;
  QjuiWidget * _join_widget;
  QMenu* options_menu;
  QAction *verboseAction;
  
public slots:
  void save();
  void saveAs();
  void saveXML();
  void saveAsXML();
  void restoreXML(bool replace=true);
  bool restoreXML(QString path, bool replace=true);
  void saveSession(QString);
  void defaultAll();
  void deleteAllItems();
  void verboseToggle();
  bool joinObjects(QdObjectSeq_var JoinedList, vector <string> &nameList);
  void DoJoin();
  void writeFile();
  void keepChoices();
  void applyChoices();
  void setTTsource(int);
  void setTTivl(int);
  void setTimeFeedback();
  void setIntervalOption();
  void setTagsOption();
  void messageHandler(QString txt);
  bool alreadyJoined();
  
  void launchManExport();
  void launchManJoin();
  void launchManual();
  void launchVersion();
  
signals:

public:
  QjuiWindow( QWidget *parent=0, const char *name=0 );
  ~QjuiWindow();
  void setDefaults();
  QdTimeInterval_var QjuiGetIntervalFromData();
  QdObject_var getTimeTags();
  bool disjointSeries(QdObject_var);
  QdTimeInterval_var QjuiGetInterval();
  bool QjuiGetScalarInterval(double &start, double &end);
  bool QjuiGetScalarIntervalFromData(double &start, double &end);
  QdTimeSeq_var getTimeTagsFromObj();
  QdRScalarSeq_var getScalarTagsFromObj();
  virtual void save(Vostream&);
  virtual void restore(Vistream& v_is,bool replace=true);

  QBoxLayout *buttons_layout;
  QWidget * _buttons_widget;

  QPushButton *dismissBtn;
  QPushButton *WriteBtn;
  QPushButton *JoinBtn;
  QWidget*    _top_widget;

  QguWLinput* _width_field;
  QComboBox*  _gap;
  QguWLinput* _gap_field;
  QComboBox*  _algorithm;
  QComboBox*  _TimeIvlSelect;

  QguWLinput* _IvlSource;
  QComboBox*  _TimeTagSelect;
  QguWLinput* _TagSource;
  QLabel*     IvlSource_label;
  QLabel*     TagSource_label;
  QLabel* Start_field;
  QLabel* End_field;
  QLabel* Ntags_field;
  QLabel* spacing_field;
  
  QString  _width_field_Def;
  int  _gap_Def;
  QString  _gap_field_Def;
  int  _algorithm_Def;
  QString _IvlSource_Def;
  int  _TimeIvlSelect_Def;
  QString _TagSource_Def;
  int  _TimeTagSelect_Def;

  QString  _width_field_cpy;
  int  _gap_cpy;
  QString  _gap_field_cpy;
  int  _algorithm_cpy;
  QString _IvlSource_cpy;
  int  _TimeIvlSelect_cpy;
  QString _TagSource_cpy;
  int  _TimeTagSelect_cpy;
  
  bool verbose;

protected:
  void dragEnterEvent( QDragEnterEvent *event);

};


QjuiWindow * QJUICreatePopup(QWidget *parent, const char *Title);
void QJUImakeWindow(QWidget *parent, const char * Title);
void QEXPmakeWindow(QWidget *parent, const char * Title);
void QjuiShowJoin();
void QjuiShowExport();
extern QjuiWindow *QJUI_widget;
extern QjuiWindow *QJEX_widget;
                                                  
# endif // QSAS_DATA_JUIW_H_
