#ifndef QSAS_PSD_H_
#define QSAS_PSD_H_

#include <string.h>
#include <vector>

#include <QGroupBox>
#include <QMenu>
#include <QTreeView>
#include <QListWidget>
#include <QLineEdit>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qslider.h>
#include <qmenubar.h>
#include <qpushbutton.h>

#include "qt_version.h"

#include "qsas_version.h"
#include "Qdos.h"
#include "qsw_helpwindow.h"
#include "qgu.h"
#include "qdutil.h"
#include "qwui.h"
#include "floatspinbox.h"
#include "qpsd_utils.h"

class PSD_Window: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  PSD_Window( );
  virtual ~PSD_Window();
  void setObject(const char * WLctxt);
  void setObjectName();
  QdObject_var makeRotnMatrix();
  bool saveRotnMatrix(QdObject_var rtnMtx);
  bool evaluateMoments(PSD_Parameters &para);
  bool evaluateSlice(PSD_Parameters &para);
  void setEvalBtn(QString txt);

  QguWLinput *objField;
  QguWLinput *latField;
  QguWLinput *longField;
  QguWLinput *rotnField;
  QguWLinput *centreField;
  QguWLinput *dirnField;
  QguWLinput *potentialField;
  QguWLinput *thresholdField;
  QComboBox *Vparity;
  QguWLinput *name_field;
  QComboBox *transMethod;
  QComboBox *transTarget;
  QComboBox *processMethod;
  QComboBox *dirnChoice;
  QComboBox *centreChoice;
  QComboBox *potentialChoice;
  QComboBox *thresholdChoice;
  QComboBox *integrationMethod;
  QComboBox *massChoice;
  QguWLinput *gapSizeField;
  QLabel *methodLabl;
  QLabel *transLab;
  QLabel *latLabl;
  QLabel *longLabl;
  QLabel *rotnLabl;
  QLabel *centreLabl;
  QLabel *dirnLabl;
  QLabel *dirnChoiceLabl;
  QLabel *centreChoiceLabl;
  QLabel *potentialChoiceLabl;
  QLabel *potentialLabl;
  QLabel *thresholdChoiceLabl;
  QLabel *thresholdLabl;
  QLabel *massChoiceLabl;
  QLabel *massLabl;
  QLabel *chargeLabl;
  QLabel *binsLabl;
  QLabel *integrationLabl;
  QLabel *parityLabel;
  QSpinBox *numberBins;
  QSpinBox *chargeField;
  FloatSpinBox *massField;
  bool userSet;
  QCheckBox *allBtn;
  QCheckBox *dBtn;
  QCheckBox *vBtn;
  QCheckBox *pBtn;
  QCheckBox *hBtn;
  QLabel * percentage;
  bool running; // for interrupt

public slots:
  void evaluate();
  void setUserInput();
  void setNewName();
  void setMethod(const QString &);
  void setTarget(const QString &);
  void setProcess(const QString &);
  void setDirnChoice(const QString &);
  void setCentreChoice(const QString &);
  void setPotentialChoice(const QString &);
  void setThresholdChoice(const QString &);
  void setMassChoice(const QString &);
  void launchHelp();
  void setMomentsAll(int);


protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *saveBtn;
  QPushButton *dismissBtn;
};


# endif // QSAS_PSD_H_
