/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/
#ifndef _QWUI_H
#define _QWUI_H

#include <string.h>
#include <vector>
#include <qfiledialog.h>
#include <qstatusbar.h>

#include <qlayout.h>
#include <QObject.h>
#include "qt_version.h"
#include "Qdos.h"

#include "qsw_helpwindow.h"
#include "qgu.h"
#include "qbu.h"

#include "WLItem.h"
#include "WLModel.h"
#include "WLView.h"

#include "qtpl.h"
#include "qdutil.h"
#include "qprofile.h"

extern QFont defaultFont; // Initialised with default size at startup

class qwuiWindow;

   
class qwuiWindow : public QguWindow
{
   Q_OBJECT

public:
   qwuiWindow(QWidget *parent=0, const char *name=0);
   void makeWindows();
   QCheckBox *StripBadData;
   QCheckBox *StripFills;
   QCheckBox * overWrite;
   

   static QWidget *toggleBar;
   void Qwui_msgBox(const char *  );
   void addFolderToMenu(const char *folder, qtplMenu *menu, bool first = true);
   virtual void save(Vostream& v_os);
   virtual void restore(Vistream& v_is, bool replace=true);
   
//    QwlListView *WorkingList;
   WLView* wlview;
   qtplMenu * timeOps;
   qtplMenu * mathOps;
   qtplMenu * filterOps;
   qtplMenu * statOps;
   qtplMenu * unitOps;
   qtplMenu * eventTableOps;
   vector <qtplMenu *> pluginFolders;
   QString qsas_home;
   QswHelpWindow *help_window;
   QString _sessionFolder;
   void saveFolder(WLItem* item, std::ofstream &ofp);

signals:
  void saveSessionRequest(QString );
  void restoreSessionRequest(QString);
  void clearListRequest();
  void openQdaWindow(const char *);
  void runStaticPlugin(const char *);
   
public slots:
   void setPreferences();
   void launchManual();
   void launchManMainW();
   void launchManSampleW();
   void launchManDataH();
   void launchManSaveRestore();
   void launchVersion();
   void launchContact();
   void saveSession();
   void saveSessionAs();
   void restoreSession();
   void saveWL();
   void saveWLAs();

  
protected:
	void closeEvent( QCloseEvent* );
//	void dragMoveEvent( QDragMoveEvent *event);
	void dragEnterEvent( QDragEnterEvent *event);


private:
//   QWidget * grid;
   QWidget* centralW;
   QVBoxLayout * centralBox;
   
};
 

void  setStandardConstants();



class SessionFolderDialog : public QFileDialog {

Q_OBJECT

  
  QStatusBar* _status_bar;
  QMenuBar *_menu_bar;
  
public slots:
  QString getSessionFolder();
  
signals:
  
public:
  SessionFolderDialog( QWidget *parent=0, const char *name=0 );
  ~SessionFolderDialog(){;}
  void status(const QString& str)  {_status_bar->showMessage(str);}
  QMenuBar* menu_bar(){ return _menu_bar;}
  QLabel * label;
  
protected:

private slots:
  
};

// Qwui Utilities

// QString QwuiPutOnWL(QdObject_var obj, const char * objName, const char * List, bool replace=0);
QString QwuiPutOnWL(QdObject_var obj, const QString& path, bool replace=false);
QSize qsasGUISize();

#endif

/**********************************************************************/

