/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qar.h"
#include "qdutil.h"

extern "C" QplugReturnStatus QarDIVIDE  ( QplugArgList *);

QplugReturnStatus QarDIVIDE  ( QplugArgList *call_list )

{
QplugReturnStatus	status;
int qar_status;

// pointers for all possible input types

QdObject_var in1_ptr;
QdObject_var in2_ptr;
QdObject_var out_ptr;
QdObject_var inv_ptr;

string attr_ptr;
string string_buf;
char * out_name;

status = QPLUG_SUCCESS;


//	Unpack the input data objects

 in1_ptr = (*call_list)[0];

 in2_ptr = (*call_list)[1];


// Get name of output object

 char * slot_ptr = GetSlotText(0);
 if ( slot_ptr[0] != '\0' ) {
    out_name = QplugNewSTR(slot_ptr);
 }
 else{
    out_name = QplugNewSTR("Ratio");
    SetSlotText(0,out_name);
 }
  
// Test validity of inputs...

qar_status =  QarTestConformal(in1_ptr, in2_ptr);
  if(qar_status == QAR_ERR)   
  {
    QplugAppendTextDisplay ("Arithmetic>> input sequences not joined properly\n");
     return QPLUG_FAILURE; 
  } 

// trap divide by vector or array

 if( !QuIsScalar(in2_ptr) ) {
    QplugAppendTextDisplay (
        "Arithmetic>> Only divide by scalars or scalar sequences allowed\n");
    return QPLUG_FAILURE; 
 }
 
// do arithmetic...
  try{
    out_ptr = in1_ptr / in2_ptr;
  }
  catch(Exception &e){
    QplugAppendTextDisplay (e.msg().c_str());
    QplugAppendTextDisplay (
        "Arithmetic>> This Operation is not yet supported between these input types\n");
    return QPLUG_FAILURE;
  }
  
  if (out_ptr.is_nil()) {
    QplugAppendTextDisplay (
        "Arithmetic>> This Operation is not yet supported between these input types\n");
    return QPLUG_FAILURE; 
  }


 // Add timeline (or DEPEND_0) : Either timeline is safe if it exists
 QdObject_var qd0 = get_Depend_0( in1_ptr );
 if ( qd0.is_nil() ) qd0 = get_Depend_0( in2_ptr );
 if ( !qd0.is_nil() ) out_ptr->change_xref(DEPEND_0 , qd0 );


 // Set QSAS essential attriutes
 
  // find inverses first
  
  // SI_conversion
    attr_ptr = QarSIConv_inverse(in2_ptr);
    inv_ptr = in1_ptr->create(); // this is empty as we only need metadata
    
    if( !attr_ptr.empty() ){
       QuSetTxtAttr("SI_conversion", attr_ptr.c_str(), inv_ptr);
    }
 
 
 // UNITS
 
   attr_ptr = QuGetAttrText("UNITS", in2_ptr);
   if( !attr_ptr.empty() ){
     string_buf = "(";
     string_buf += attr_ptr;
     string_buf += ")^-1";
     QuSetTxtAttr("UNITS", string_buf.c_str(), inv_ptr);
   }
 
 // use inverse for attributes 
 
 // SI_conversion
 attr_ptr = QarSIConv_product(in1_ptr, inv_ptr);
 if ( !attr_ptr.empty() ) {
   QuSetTxtAttr("SI_conversion", attr_ptr.c_str(), out_ptr);
 }

 // FIELDNAM

 // ignore fieldnames as concatenating these rapidly gets too long

   QuSetTxtAttr("FIELDNAM", out_name, out_ptr);
   free (out_name);
 
 // Frame (must come from first object for divide)

 attr_ptr = QuGetAttrText("Frame", in1_ptr);
   QuSetTxtAttr("Frame", attr_ptr.c_str(), out_ptr);

 
 // UNITS
 attr_ptr = QarUNITS_product(in1_ptr, inv_ptr);
 if ( !attr_ptr.empty() ) {
   QuSetTxtAttr("UNITS", attr_ptr.c_str(), out_ptr);
 } 
 
 // LABLAXIS
	string labl_txt = QuGetAttrText( "LABLAXIS", in1_ptr );
	QString labl_s = QString(labl_txt.c_str());
	if(in2_ptr->xref_exists("LABLAXIS")){
		labl_s += "/";
		labl_s += QuGetAttrText( "LABLAXIS", in2_ptr ).c_str();
	}
	else if(in2_ptr->sequence_size() == 1){
		QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(in2_ptr);
		QdRScalar_var s = QdRScalar_var::narrow(in2_ptr);
		if(!ss.is_nil()) labl_s += QString(" / %1").arg(ss[0]);
		else if(!s.is_nil()) labl_s += QString(" / %1").arg(*s);
	}

	QuSetTxtAttr( "LABLAXIS", labl_s.toLatin1().data(), out_ptr );

 // DEPEND_i etc.
 
 bool OK = copyDimXrefs(in1_ptr, out_ptr);
 if (OK) QplugAppendTextDisplay ("Dimensional Xrefs copied to new object\n");
 
  //	..add  the output series to the list

  call_list->push_back(out_ptr);


  QplugAppendTextDisplay ("Divide: Done\n");
  
  
return status;

}

