/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qar.h"

extern "C" QplugReturnStatus QarMULTIPLY(QplugArgList *);

QplugReturnStatus QarMULTIPLY  ( QplugArgList *call_list )

{
QplugReturnStatus	status;
int qar_status;


QdObject_var in1_ptr;
QdObject_var in2_ptr;
QdObject_var  out_ptr;

string attr_ptr;
char * string_buf;
char * out_name;

status = QPLUG_SUCCESS;

 if(call_list->size() < 2) {
    QplugAppendTextDisplay ("Arithmetic>> This Operation requires 2 inputs\n");
    return QPLUG_FAILURE; 

 } 

//	Unpack the input data objects

 in1_ptr = (*call_list)[0];

 in2_ptr = (*call_list)[1];


// Get name of output object

 char * slot_ptr = GetSlotText(0);
 if ( slot_ptr[0] != '\0' ) {
    out_name = QplugNewSTR(slot_ptr);
 }
 else{
    out_name = QplugNewSTR("Product");
    SetSlotText(0,out_name);
 }
  
// Test validity of inputs...


qar_status =  QarTestConformal(in1_ptr, in2_ptr);
  if(qar_status == QAR_ERR)       
  {
    QplugAppendTextDisplay ("Arithmetic>> input sequences not joined properly\n");
     return QPLUG_FAILURE; 
  }


// do arithmetic...

  out_ptr = in1_ptr * in2_ptr;

  if (out_ptr.is_nil()) {
    QplugAppendTextDisplay (
        "Arithmetic>> This Operation is not yet supported between these input types\n");
    return QPLUG_FAILURE; 
  }


 // Add timeline (or DEPEND_0) : Either timeline is safe if it exists
 QdObject_var qd0 = get_Depend_0( in1_ptr );
 if ( qd0.is_nil() ) qd0 = get_Depend_0( in2_ptr );
 if ( !qd0.is_nil() ) out_ptr->change_xref(DEPEND_0 , qd0 );


 // Set QSAS essential attriutes
 
 // SI_conversion

 attr_ptr = QarSIConv_product(in1_ptr, in2_ptr);
 if ( !attr_ptr.empty()) {
   QuSetTxtAttr("SI_conversion", attr_ptr.c_str(), out_ptr);
 }

 // FIELDNAM

 // ignore fieldnames as concatenating these rapidly gets too long

   QuSetTxtAttr("FIELDNAM", out_name, out_ptr);
   free (out_name);
 
 // Frame

   attr_ptr = QarFrame_product(in1_ptr, in2_ptr);
   if(attr_ptr.find(">na") == string::npos) QuSetTxtAttr("Frame", attr_ptr.c_str(), out_ptr);// skip not applicable attributes
 
 
 // UNITS
 
 attr_ptr = QarUNITS_product(in1_ptr, in2_ptr);
 if ( !attr_ptr.empty() ) {
   QuSetTxtAttr("UNITS", attr_ptr.c_str(), out_ptr);

 } 
 
 // LABLAXIS
	string labl_txt = QuGetAttrText( "LABLAXIS", in1_ptr );
	QString labl_s = QString(labl_txt.c_str());
	if(in2_ptr->xref_exists("LABLAXIS")){
		labl_s += " x ";
		labl_s += QuGetAttrText( "LABLAXIS", in2_ptr ).c_str();
	}
	else if(in2_ptr->sequence_size() == 1){
		QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(in2_ptr);
		QdRScalar_var s = QdRScalar_var::narrow(in2_ptr);
		if(!ss.is_nil()) labl_s += QString(" x %1").arg(ss[0]);
		else if(!s.is_nil()) labl_s += QString(" x %1").arg(*s);
	}

	QuSetTxtAttr( "LABLAXIS", labl_s.toLatin1().data(), out_ptr );


 // DEPEND_i, Label_i  etc.
 
 innerProdDimXrefs(in1_ptr, in2_ptr, out_ptr);


  //	..add  the output series to the list

  call_list->push_back(out_ptr);

  QplugAppendTextDisplay ("Multiply: Done\n");
  
return status;

}

