/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qar.h"

extern "C" QplugReturnStatus QarPOWER(QplugArgList *);

QplugReturnStatus QarPOWER  ( QplugArgList *call_list )

{
QplugReturnStatus	status;
int qar_status;


QdObject_var in1_ptr;
QdObject_var in2_ptr;
QdObject_var out_ptr;

string attr_ptr;
string string_buf;
char * out_name;

status = QPLUG_SUCCESS;


//	Unpack the input data objects

 in1_ptr = (*call_list)[0];

 in2_ptr = (*call_list)[1];


// Get name of output object

 char * slot_ptr = GetSlotText(0);
 if ( slot_ptr[0] != '\0' ) {
    out_name = QplugNewSTR(slot_ptr);
 }
 else{
    out_name = QplugNewSTR("RaisedToPower");
    SetSlotText(0,out_name);
 }

// Test validity of inputs...

qar_status =  QarTestConformal(in1_ptr, in2_ptr);
  if(qar_status == QAR_ERR)     
  {
    QplugAppendTextDisplay ("Arithmetic>> input sequences not joined properly\n");
     return QPLUG_FAILURE; 
  } 

// get base SI for input data


  if( QarHasUnits(in2_ptr) ){
    // issue warning
    QplugAppendTextDisplay ("WARNING>> Exponent has units.\a \n");  
  }


// do arithmetic...

out_ptr = pow(in1_ptr, in2_ptr);

if (out_ptr.is_nil()) {
    QplugAppendTextDisplay (
        "Arithmetic>> This Operation is not yet supported between these input types\n");
    return QPLUG_FAILURE; 
}

 // Add timeline (or DEPEND_0) : Either timeline is safe if it exists
 QdObject_var qd0 = get_Depend_0( in1_ptr );
 if ( qd0.is_nil() ) qd0 = get_Depend_0( in2_ptr );
 if ( !qd0.is_nil() ) out_ptr->change_xref(DEPEND_0 , qd0 );


 // Set QSAS essential attributes
  
  QdRScalar_var B_ptr = QdRScalar_var::narrow(in2_ptr);

 if(!B_ptr.is_nil()){   // can only do these if power is unique
  
   // SI_conversion
   // the value for constant power is a global
   
   double constB = (double)(*B_ptr);
   attr_ptr = QarSIConv_power(in1_ptr, constB);
   if( !attr_ptr.empty() ){
     QuSetTxtAttr("SI_conversion", attr_ptr.c_str(), out_ptr);
   }
  
   // UNITS
   // the value for constant power is a global
   attr_ptr = QarUNITS_power(in1_ptr, constB);
   if( !attr_ptr.empty() ){
     QuSetTxtAttr("UNITS", attr_ptr.c_str(), out_ptr);
   }

  // LABLAXIS
	string labl_txt = QuGetAttrText( "LABLAXIS", in1_ptr );
	QString labl_s = QString(labl_txt.c_str());
	labl_s += QString(" ^ %1").arg(constB);
	QuSetTxtAttr( "LABLAXIS", labl_s.toLatin1().data(), out_ptr );

 }
 
 // FIELDNAM

 // ignore fieldnames as concatenating these rapidly gets too long

   QuSetTxtAttr("FIELDNAM", out_name, out_ptr);
   free (out_name);
 
 // Frame
 
   QuSetTxtAttr("Frame", "scalar>na", out_ptr);
 

  //	..add  the output series to the list

  call_list->push_back(out_ptr);

  QplugAppendTextDisplay ("Power: Done\n");
  
return status;

}

