/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qar.h"

extern "C" QplugReturnStatus QarSUBTRACT(QplugArgList *);

QplugReturnStatus QarSUBTRACT  ( QplugArgList *call_list )

{
QplugReturnStatus	status;
int qar_status;


QdObject_var in1_ptr;
QdObject_var in2_ptr;
QdObject_var  out_ptr;

string attr_ptr;
char * string_buf;
char * out_name;

status = QPLUG_SUCCESS;

//	Unpack the input data objects

  in1_ptr = (*call_list)[0];

  in2_ptr = (*call_list)[1];


// Get name of output object

 char * slot_ptr = GetSlotText(0);
 if ( slot_ptr[0] != '\0' ) {
    out_name = QplugNewSTR(slot_ptr);
 }
 else{
    out_name = QplugNewSTR("Difference");
    SetSlotText(0,out_name);
 }

// get base SI for input data

qar_status = QarAreUnitsSame(in1_ptr, in2_ptr);

  if(qar_status == QAR_ERR) {
      QplugAppendTextDisplay (
        "Arithmetic>> This Operation requires inputs in same units (edit SI_conversion)\n");
    return QPLUG_FAILURE; 
  }
  else if(qar_status == QAR_WARN){
    QplugAppendTextDisplay (
    "Arithmetic>> Warning: units unknown for at least one input\n");
    QplugAppendTextDisplay (
    "Proceed with caution!\n");
  }
  
// Test validity of inputs...

qar_status =  QarTestConformal(in1_ptr, in2_ptr);
  if(qar_status == QAR_ERR)      
  {
     QplugAppendTextDisplay ("Arithmetic>> input sequences not joined properly\n");
     return QPLUG_FAILURE; 
  }


qar_status = QarAreFramesSame(in1_ptr, in2_ptr);

  if(qar_status == QAR_ERR)  if(qar_status == QAR_ERR) {
    QplugAppendTextDisplay ("Arithmetic>> This Operation requires vectors in same Frame\n");
    return QPLUG_FAILURE;
  } 
  if(qar_status == QAR_WARN) {
    QplugAppendTextDisplay ("Arithmetic>> Warning: Frame unknown for at least one object\n");
  }
 


// do arithmetic ...


out_ptr = in1_ptr - in2_ptr;

if (out_ptr.is_nil()) {
    QplugAppendTextDisplay (
        "Arithmetic>> This Operation is not yet supported between these input types\n");
    return QPLUG_FAILURE; 
}

 // Add timeline (or DEPEND_0) : Either timeline is safe if it exists
 QdObject_var qd0 = get_Depend_0( in1_ptr );
 if ( qd0.is_nil() ) qd0 = get_Depend_0( in2_ptr );
 if ( !qd0.is_nil() ) out_ptr->change_xref(DEPEND_0 , qd0 );


 // Set QSAS essential attriutes
 // for subtraction they stay the same as inputs
 
 // SI_conversion
 
 attr_ptr = QuGetAttrText("SI_conversion", in1_ptr);
 if( !attr_ptr.empty() ){
   QuSetTxtAttr("SI_conversion", attr_ptr.c_str(), out_ptr);
 }
 else {
   attr_ptr = QuGetAttrText("SI_conversion", in2_ptr) ;
   QuSetTxtAttr("SI_conversion", attr_ptr.c_str(), out_ptr);
 }
  
 // FIELDNAM

 // ignore fieldnames as concatenating these rapidly gets too long

   QuSetTxtAttr("FIELDNAM", out_name, out_ptr);
   free (out_name);
 
 // Frame
 
 QuSetFrame(out_ptr, QuGetFrameAttr(  in1_ptr, in2_ptr ).c_str());
 
  
 // UNITS
 
 attr_ptr = QuGetAttrText("UNITS", in1_ptr);
 if( !attr_ptr.empty() ){
   QuSetTxtAttr("UNITS", attr_ptr.c_str(), out_ptr);
 }
 else  {
   attr_ptr = QuGetAttrText("UNITS", in2_ptr);
   QuSetTxtAttr("UNITS", attr_ptr.c_str(), out_ptr);
 }

 // LABLAXIS
	string labl_txt = QuGetAttrText( "LABLAXIS", in1_ptr );
	QString labl_s = QString(labl_txt.c_str());
	labl_s += " - ";
	labl_s += QuGetAttrText( "LABLAXIS", in2_ptr ).c_str();
	QuSetTxtAttr( "LABLAXIS", labl_s.toLatin1().data(), out_ptr );

 // DEPEND_i etc.
 
 bool OK = copyDimXrefs(in1_ptr, out_ptr);
 // only try other object if NO dimensioned xrefs found for first
 if(!OK) OK = copyDimXrefs(in2_ptr, out_ptr);
 if (OK) QplugAppendTextDisplay ("Dimensional Xrefs copied to new object\n");


  //	..add  the output series to the list

  call_list->push_back(out_ptr);

  QplugAppendTextDisplay ("Subtract: Done\n");
  
return status;

}

