#include "qOperators.h"

QdObject_var QdDiv( QString& rqstrErrorMsg, QdObject_var qdOperand1, QdObject_var qdOperand2 )
{
	QdObject_var        qdResult;
	if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
	{
		// The object will be empty - this should never happen
		// The qdResultt object is NULL at this point
		return ( qdResult );
	}

	QplugReturnStatus	status      = QPLUG_SUCCESS;;
	QdObject_var        qdInv;
	string  attrPtr;

 	// Test validity of inputs...
	int nQarStatus =  QarTestConformal( qdOperand1, qdOperand2 );
	if ( nQarStatus == QAR_ERR )
	{
		QcalcAppendTextDisplay( "Sequence size or join error: Notify csc_support" );
		return qdResult;
	}

	// do arithmetic...
	try
	{
		qdResult = qdOperand1 / qdOperand2;
	}
	catch( Exception &e )
	{
		QcalcAppendTextDisplay ( e.msg().c_str() );
		QcalcAppendTextDisplay( "This Operation Divide is not yet supported between these input types");
		rqstrErrorMsg = QString( "This Operation Divide is not yet supported between these input types");
		return ( qdResult );
	}

	if (qdResult.is_nil())
	{
	    QcalcAppendTextDisplay( "This Operation Divide is not yet supported between these input types" );
		rqstrErrorMsg = QString( "This Operation Divide is not yet supported between these input types");
		return ( qdResult );
	}

	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( qdOperand1 );
	if ( qd0.is_nil() ) qd0 = get_Depend_0( qdOperand2 );
	if ( !qd0.is_nil() ) qdResult->change_xref(DEPEND_0 , qd0 );

	// Set QSAS essential attriutes
	// find inverses first and set its attributes, then do for result

	// SI_conversion
	attrPtr = QarSIConv_inverse( qdOperand2 );
	qdInv   = qdOperand1->create(); // this is empty as we only need metadata

	if( !attrPtr.empty() )
	{
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), qdInv );
	}

	// UNITS
	attrPtr = QuGetAttrText( "UNITS", qdOperand2 );
	if( !attrPtr.empty() )
	{
		string stringBuf = "(";
		stringBuf += attrPtr.c_str();
		stringBuf += ")^-1";
		QuSetTxtAttr( "UNITS", stringBuf.c_str(), qdInv );
	}

	// use inverse for attributes for returned object
	// SI_conversion
	attrPtr = QarSIConv_product( qdOperand1, qdInv );
	if( !attrPtr.empty() )
	{
		QuSetTxtAttr("SI_conversion", attrPtr.c_str(), qdResult);
	}

	// Frame (must come from first object for divide)
	attrPtr = QuGetFrameAttr( qdOperand1 );
	if( !attrPtr.empty() )
	{
		QuSetFrame( qdResult, attrPtr.c_str() );
	}
	
	// UNITS
	attrPtr = QarUNITS_product( qdOperand1, qdInv );
	if( !attrPtr.empty() )
	{
		QuSetTxtAttr( "UNITS", attrPtr.c_str(), qdResult );
	}

	// LABLAXIS
	QString labl_s = QString(QuGetAttrText( "LABLAXIS", qdOperand1 ).c_str());
	if(qdOperand2->xref_exists("LABLAXIS")){
		labl_s += "/";
		labl_s += QuGetAttrText( "LABLAXIS", qdOperand2 ).c_str();
	}
	else if(qdOperand2->sequence_size() == 1){
		QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(qdOperand2);
		QdRScalar_var s = QdRScalar_var::narrow(qdOperand2);
		if(!ss.is_nil()) labl_s += QString(" / %1").arg(ss[0]);
		else if(!s.is_nil()) labl_s += QString(" / %1").arg(*s);
	}
	
	QuSetTxtAttr( "LABLAXIS", labl_s.toLatin1().data(), qdResult );

	// DEPEND_i
    if (qdOperand1->xref_exists(DEPEND_1)) 
    {
        QdObject_var d1_xref = qdOperand1->get_xref(DEPEND_1);
        qdResult->change_xref( DEPEND_1, d1_xref  );
    }
   
    if (qdOperand1->xref_exists(DEPEND_2)) 
    {
        QdObject_var d1_xref = qdOperand1->get_xref(DEPEND_2);
        qdResult->change_xref( DEPEND_2, d1_xref  );
    }

    if (qdOperand1->xref_exists(DEPEND_3)) 
    {
        QdObject_var d1_xref = qdOperand1->get_xref(DEPEND_3);
        qdResult->change_xref( DEPEND_3, d1_xref  );
    }
   
	return ( qdResult );

}





