#include "qOperators.h"
    
void setProductDepends(QdObject_var resultObj, QdObject_var obj1, QdObject_var obj2);
         
QdObject_var QdMult( QString& rqstrErrorMsg, QdObject_var qdOperand1, QdObject_var qdOperand2 )
{

    QdObject_var  qdResult;

    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
	// The qdResult object is NULL at this point
        return ( qdResult );
    }

 	// Test validity of inputs...
	int nQarStatus =  QarTestConformal( qdOperand1, qdOperand2 );
	if ( nQarStatus == QAR_ERR )
	{
		QcalcAppendTextDisplay( "Sequence size or join error: Notify csc_support" );
		return qdResult;
	}
	
	// Test Frames for vectors
	
	string  attrPtr;

	string flag1 = QuGetOrder(qdOperand1);
	string flag2 = QuGetOrder(qdOperand2);
	bool isV2 = false;
	bool isR1 = false;
	bool isR2 = false;

	if(QuToUpper(flag2).find("VECTOR") != string::npos) isV2 = true;
	if(QuToUpper(flag1).find("ROT") != string::npos) isR1 = true;
	if(QuToUpper(flag2).find("ROT") != string::npos) isR2 = true;
	
	if(isR1 && isV2){
	   string fr1 = QuGetFrame(qdOperand1);
	   string fr2 = QuGetFrame(qdOperand2);
	   if(fr1 != fr2) {
		QcalcAppendTextDisplay( "Frames for Rotation Matrix and Vector differ" );
		return qdResult;
	   }
	}
	if(isR1 && isR2){
	   string fr1 = QuGetFrame(qdOperand1);
	   string fr2 = QuGetToFrame(qdOperand2);
	   if(fr1 != fr2){
		QcalcAppendTextDisplay( "Frames for Rotation Matrices differ..." );
		QcalcAppendTextDisplay( "R1.Frame != R2.ToFrame" );
		return qdResult;
	   }
	   
	}
	
	// do arithmetic...
	try
	{
		qdResult = qdOperand1 * qdOperand2;
	}
	catch( Exception &e )
	{
		QcalcAppendTextDisplay ( e.msg().c_str() );
		QcalcAppendTextDisplay( "The operation Multiplication is not yet supported between these input types");
		rqstrErrorMsg = QString( "The operation Multiplication is not yet supported between these input types");
		return ( qdResult );
	}

	if ( qdResult.is_nil() )
	{
		QcalcAppendTextDisplay( "Arithmetic>> This Operation is not yet supported between these input types" );
		rqstrErrorMsg = QString( "Arithmetic>> This Operation is not yet supported between these input types" );
		return ( qdResult );
	}

	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( qdOperand1 );
	if ( qd0.is_nil() ) qd0 = get_Depend_0( qdOperand2 );
	if ( !qd0.is_nil() ) qdResult->change_xref(DEPEND_0 , qd0 );

	// Set QSAS essential attriutes
	// SI_conversion
	attrPtr = QarSIConv_product( qdOperand1, qdOperand2 );
	if ( !attrPtr.empty())
	{
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), qdResult );
	}

	// Frame
	attrPtr = QarFrame_product( qdOperand1, qdOperand2 );
	if(attrPtr.find(">na") == string::npos) QuSetFrame(qdResult, attrPtr.c_str() ); // skip not applicable attributes
	
	if(isR1 && isR2){
	  // new ToFrame from R1 
	  // Note Frame is from ToFrame of R2 and is handled by QarFrame_producet
	   string toFr = QuGetAttrText("ToFrame", qdOperand1);
	   QuSetTxtAttr( "ToFrame", toFr.c_str(), qdResult );
	}
	

	// UNITS
	attrPtr = QarUNITS_product( qdOperand1, qdOperand2 );
	if ( !attrPtr.empty())
	{
		QuSetTxtAttr("UNITS", attrPtr.c_str(), qdResult);
	}
	
	// LABLAXIS
	string labl_txt = QuGetAttrText( "LABLAXIS", qdOperand1 );
	QString labl_s = QString(labl_txt.c_str());
	if(qdOperand2->xref_exists("LABLAXIS")){
		labl_s += " x ";
		labl_s += QuGetAttrText( "LABLAXIS", qdOperand2 ).c_str();
	}
	else if(qdOperand2->sequence_size() == 1){
		QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(qdOperand2);
		QdRScalar_var s = QdRScalar_var::narrow(qdOperand2);
		if(!ss.is_nil()) labl_s += QString(" x %1").arg(ss[0]);
		else if(!s.is_nil()) labl_s += QString(" x %1").arg(*s);
	}
	QuSetTxtAttr( "LABLAXIS", labl_s.toLatin1().data(), qdResult );


	// DEPEND_i, Label_i  etc.
 
	innerProdDimXrefs(qdOperand1, qdOperand2, qdResult);

	
    return ( qdResult );
}

// *************************************************************************

void setProductDepends(QdObject_var resultObj, QdObject_var obj1, QdObject_var obj2){
   int dims1 = QuArrayDims(obj1);
   int dims2 = QuArrayDims(obj2);

   int thisDim=1;
   while( thisDim < dims1 ){ // this ignores last dim = first dim of obj2
     QString depName = QString("DEPEND_%1").arg(thisDim);
	 if (obj1->xref_exists(depName.toLatin1().data()))
     {
        	QdObject_var d_xref = obj1->get_xref(depName.toLatin1().data());
                resultObj->change_xref( depName.toStdString(), d_xref  );
	 }
     thisDim++;
   }
   
   int i=2;
   while( i <= dims2 ){ // this ignores first dim of obj2
     QString depNamei = QString("DEPEND_%1").arg(i);
     QString depName = QString("DEPEND_%1").arg(thisDim);
	 if (obj2->xref_exists(depNamei.toLatin1().data()))
     {
        	QdObject_var d_xref = obj2->get_xref(depNamei.toLatin1().data());

                resultObj->change_xref( depName.toStdString(), d_xref);
	 }
     thisDim++;
	 i++;
   }
}


