#include "qOperators.h"
             
QdObject_var QdMult_Pw( QString& rqstrErrorMsg, QdObject_var qdOperand1, QdObject_var qdOperand2 )
{

    QdObject_var  qdResult;

    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
	// The qdResult object is NULL at this point
        return ( qdResult );
    }

 	// Test validity of inputs...
	int nQarStatus =  QarTestConformal( qdOperand1, qdOperand2 );
	if ( nQarStatus == QAR_ERR )
	{
		QcalcAppendTextDisplay( "Sequence size or join error: Notify csc_support" );
		return qdResult;
	}
	
	
	// do arithmetic...
	try
	{
		// Not provided by qdos so handle explicitly all cases
		QdRMatrix_var mout;
		QdRMatrixSeq_var msout;
		
		QdRMatrix_var m2 = QdRMatrix_var::narrow(qdOperand2);
		QdRMatrixSeq_var ms2 = QdRMatrixSeq_var::narrow(qdOperand2);

		// Already tested that dims match
		Dim_var dim_ptr;
		QdRMatrix_var m1 = QdRMatrix_var::narrow(qdOperand1);
		if(!m1.is_nil())
		{
			// obj 1 is matrix obj
			dim_ptr = m1->dimvar();
			
			if(!m2.is_nil())
			{
				mout = Factory::clone(*m2);
				qdResult = mout;
				RMatrix::iterator itr=mout->begin();
				RMatrix::iterator itr1=m1->begin();
				while(itr!=mout->end() )
				{
					*itr *= *itr1;
					++itr;
					++itr1;
				}

			}
			else if (!ms2.is_nil())
			{
				msout = Factory::clone(*ms2);
				qdResult = msout;
				for(int j=0; j<ms2->sequence_size(); j++)
				{
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr1=m1->begin();
					while(itr!=msout[j].end() )
					{
						*itr *= *itr1;
						++itr;
						++itr1;
					}
				}
			}
			else return qdResult;
		}
		else
		{
			// obj 1 is matrix sequence
			QdRMatrixSeq_var ms1 = QdRMatrixSeq_var::narrow(qdOperand1);
			if(ms1.is_nil() || ms1->sequence_size() == 0) return qdResult;
			dim_ptr = ms1[0].dimvar();
			if(!m2.is_nil())
			{
				msout = Factory::clone(*ms1);
				qdResult = msout;
				for(int j=0; j<ms1->sequence_size(); j++)
				{
					// iterator for matrix
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr2=m2->begin();
					while(itr!=msout[j].end() )
					{
						*itr *= *itr2;
						++itr;
						++itr2;
					}
				}			
			}
			else if (!ms2.is_nil())
			{
				msout = Factory::clone(*ms1);
				qdResult = msout;
				for(int j=0; j<ms1->sequence_size(); j++)
				{
					// iterator for matrix
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr2=ms2[j].begin();
					while(itr!=msout[j].end() )
					{
						*itr *= *itr2;
						++itr;
						++itr2;
					}
				}
			}
			else return qdResult;
			
		}
		
		
	}
	catch( Exception &e )
	{
		QcalcAppendTextDisplay ( e.msg().c_str() );
		return ( qdResult );
	}

	if ( qdResult.is_nil() )
	{
		QcalcAppendTextDisplay( "Arithmetic>> This Operation is not supported between these input types" );
		rqstrErrorMsg = QString( "Arithmetic>> This Operation is not supported between these input types" );
		return ( qdResult );
	}
	if(qdOperand1->xref_exists(DEPEND_1) && qdOperand1->xref_exists(DEPEND_0)) qdResult->copy_xrefs_from(qdOperand1);
	else if(qdOperand2->xref_exists(DEPEND_1) && qdOperand2->xref_exists(DEPEND_0)) qdResult->copy_xrefs_from(qdOperand2);
	else 
	{
		if(qdOperand1->xref_exists(DEPEND_1) ) qdResult->copy_xrefs_from(qdOperand1);
		else if(qdOperand2->xref_exists(DEPEND_1) ) qdResult->copy_xrefs_from(qdOperand2);
		
		// Add timeline (or DEPEND_0): Either timeline is safe if it exists
		QdObject_var qd0 = get_Depend_0( qdOperand1 );
		if ( qd0.is_nil() ) qd0 = get_Depend_0( qdOperand2 );
		if ( !qd0.is_nil() ) qdResult->change_xref(DEPEND_0 , qd0 );
	
	}
	
	// Fix some attriutes
	
	string  attrPtr;
	
	// SI_conversion
	attrPtr = QarSIConv_product( qdOperand1, qdOperand2 );
	if ( !attrPtr.empty() )
	{
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), qdResult );
	}
	

	// UNITS
	attrPtr = QarUNITS_product( qdOperand1, qdOperand2 );
	if ( !attrPtr.empty() )
	{
		QuSetTxtAttr("UNITS", attrPtr.c_str(), qdResult);
	}

	
    return ( qdResult );
}

// ---------------------------------------------------------------------------------------------

             
QdObject_var QdDiv_Pw( QString& rqstrErrorMsg, QdObject_var qdOperand1, QdObject_var qdOperand2 )
{

    QdObject_var  qdResult;

    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
	// The qdResult object is NULL at this point
        return ( qdResult );
    }

 	// Test validity of inputs...
	int nQarStatus =  QarTestConformal( qdOperand1, qdOperand2 );
	if ( nQarStatus == QAR_ERR )
	{
		QcalcAppendTextDisplay( "Sequence size or join error: Notify csc_support" );
		return qdResult;
	}
	
	
	// do arithmetic...
	try
	{
		// Not provided by qdos so handle explicitly all cases
		QdRMatrix_var mout;
		QdRMatrixSeq_var msout;
		
		QdRMatrix_var m2 = QdRMatrix_var::narrow(qdOperand2);
		QdRMatrixSeq_var ms2 = QdRMatrixSeq_var::narrow(qdOperand2);

		// Already tested that dims match
		Dim_var dim_ptr;
		QdRMatrix_var m1 = QdRMatrix_var::narrow(qdOperand1);
		if(!m1.is_nil())
		{
			// obj 1 is matrix obj
			dim_ptr = m1->dimvar();
			
			if(!m2.is_nil())
			{
				mout = Factory::clone(*m1);
				qdResult = mout;
				RMatrix::iterator itr1=mout->begin();
				RMatrix::iterator itr2=m2->begin();
				while(itr1!=mout->end() )
				{
					*itr1 /= *itr2;
					++itr2;
					++itr1;
				}

			}
			else if (!ms2.is_nil())
			{
				msout = Factory::clone(*ms2);
				qdResult = msout;
				for(int j=0; j<ms2->sequence_size(); j++)
				{
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr1=m1->begin();
					while(itr!=msout[j].end() )
					{
						*itr = (*itr1)/(*itr);
						++itr;
						++itr1;
					}
				}
			}
			else return qdResult;
		}
		else
		{
			// obj 1 is matrix sequence
			QdRMatrixSeq_var ms1 = QdRMatrixSeq_var::narrow(qdOperand1);
			if(ms1.is_nil() || ms1->sequence_size() == 0) return qdResult;
			dim_ptr = ms1[0].dimvar();
			if(!m2.is_nil())
			{
				msout = Factory::clone(*ms1);
				qdResult = msout;
				for(int j=0; j<ms1->sequence_size(); j++)
				{
					// iterator for matrix
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr2=m2->begin();
					while(itr!=msout[j].end() )
					{
						*itr /= *itr2;
						++itr;
						++itr2;
					}
				}			
			}
			else if (!ms2.is_nil())
			{
				msout = Factory::clone(*ms1);
				qdResult = msout;
				for(int j=0; j<ms1->sequence_size(); j++)
				{
					// iterator for matrix
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr2=ms2[j].begin();
					while(itr!=msout[j].end() )
					{
						*itr /= *itr2;
						++itr;
						++itr2;
					}
				}
			}
			else return qdResult;
			
		}
		
		
	}
	catch( Exception &e )
	{
		QcalcAppendTextDisplay ( e.msg().c_str() );
		return ( qdResult );
	}

	if ( qdResult.is_nil() )
	{
		QcalcAppendTextDisplay( "Arithmetic>> This Operation is not supported between these input types" );
		rqstrErrorMsg = QString( "Arithmetic>> This Operation is not supported between these input types" );
		return ( qdResult );
	}
	if(qdOperand1->xref_exists(DEPEND_1) && qdOperand1->xref_exists(DEPEND_0)) qdResult->copy_xrefs_from(qdOperand1);
	else if(qdOperand2->xref_exists(DEPEND_1) && qdOperand2->xref_exists(DEPEND_0)) qdResult->copy_xrefs_from(qdOperand2);
	else 
	{
		if(qdOperand1->xref_exists(DEPEND_1) ) qdResult->copy_xrefs_from(qdOperand1);
		else if(qdOperand2->xref_exists(DEPEND_1) ) qdResult->copy_xrefs_from(qdOperand2);
		
		// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
		QdObject_var qd0 = get_Depend_0( qdOperand1 );
		if ( qd0.is_nil() ) qd0 = get_Depend_0( qdOperand2 );
		if ( !qd0.is_nil() ) qdResult->change_xref(DEPEND_0 , qd0 );
	
	}
	
	// Fix some attriutes
	
	
	string  attrPtr;
	
	// SI_conversion
	attrPtr = QarSIConv_inverse( qdOperand2 );
	if ( !attrPtr.empty() )
	{
		// set to inverse of 2nd operator
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), qdResult );
	}	
	attrPtr = QarSIConv_product( qdOperand1, qdResult );
	if ( !attrPtr.empty() )
	{
		// Then replace with correct ratio
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), qdResult );
	}
	

	// UNITS
	attrPtr = QuGetAttrText("UNITS", qdOperand2);
	if ( !attrPtr.empty() )
	{
		string string_buf = "(";
		string_buf += attrPtr;
		string_buf += ")^-1";
		QuSetTxtAttr("UNITS", string_buf.c_str(), qdResult);
	}
	attrPtr = QarUNITS_product( qdOperand1, qdResult ) ;
	if ( !attrPtr.empty() )
	{
		QuSetTxtAttr("UNITS", attrPtr.c_str(), qdResult);
	}

	
    return ( qdResult );
}


