#include "qOperators.h"


QdObject_var QdPower( QString& rqstrErrorMsg, QdObject_var qdOperand1, QdObject_var qdOperand2 )
{

    QdObject_var  qdResult;

    if ( qdOperand2.is_nil() ||  qdOperand1.is_nil())
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
        return ( qdResult );
    }

	string attrPtr;


	// do arithmetic..
	try
	{
		qdResult = pow(qdOperand1, qdOperand2);
		if (qdResult.is_nil())
		{
			QcalcAppendTextDisplay( "Arithmetic>> Power Operation is not yet supported between these input types" );
			rqstrErrorMsg = QString( "Arithmetic>>Power Operation is not yet supported between these input types" );
			return ( qdResult );
		}
	}
	catch( Exception & e)
    {
		e.print_msg();
		QcalcAppendTextDisplay( "Error>> Power operation - Exception thrown..." );
		QcalcAppendTextDisplay( e.msg().c_str() );
		rqstrErrorMsg = QString( "Error>> Power - Exception thrown..." );
		return ( qdResult );
    }

	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( qdOperand1 );
	if ( qd0.is_nil() ) qd0 = get_Depend_0( qdOperand2 );
	if ( !qd0.is_nil() ) qdResult->change_xref(DEPEND_0 , qd0 );

	// Set QSAS essential attributes
	QdRScalar_var qdrBPtr = QdRScalar_var::narrow( qdOperand2 );
	if ( !qdrBPtr.is_nil() )
	{
		// can only do these if power is unique
		// SI_conversion
		// the value for constant power is a global
		double dConstB = ( double )( *qdrBPtr );
		attrPtr = QarSIConv_power( qdOperand1, dConstB );
		if( !attrPtr.empty() )
		{
			QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), qdResult );
		}

		// UNITS
		// the value for constant power is a global
		attrPtr = QarUNITS_power( qdOperand1, dConstB ) ;
		if( !attrPtr.empty() )
		{
			QuSetTxtAttr( "UNITS", attrPtr.c_str(), qdResult );
		}
		
		// LABLAXIS
		QString labl_s = QString(QuGetAttrText( "LABLAXIS", qdOperand1 ).c_str());
		labl_s += QString(" ^ %1").arg(dConstB);
		QuSetTxtAttr( "LABLAXIS", labl_s.toLatin1().data(), qdResult );

	}


	// Frame
	QuSetFrame( qdResult, "scalar" );

    return ( qdResult );
}

