#include "qcalc.h"
#include "qwui.h"
#include "qdutil.h"

using namespace QSAS;

extern qwuiWindow* qsasWindow;


void QcalcWindow::save()
{

	if ( QuSlotDebug() )
	{
		printf( "QcalcWindow::save \n" ) ;
		fflush(stdout);
	}

	if ( _filename.isEmpty() ) return saveAs();

 	ofstream file;
	file.open(_filename.toLatin1().data(),ios::out);

	if(file.is_open())
	{
		QDomDocument doc("QSAS_calc1_save");

		QDomElement root=doc.createElement("QSAS_Calc1");
		doc.appendChild(root);
		string guiID;
		guiID=QSAS_VERSION_ID;

		QDomElement version=createTextXMLNode(doc, "QSAS_Version", guiID);
		root.appendChild(version);

		QDateTime  qdatetime = QDateTime::currentDateTime();
				
		// modify alias to make unique
		QList< QcalcItem *> lst;

		QList<QcalcItem*> items = _join_listview->listItems();
	
		// write each item, WL items first
		foreach(QcalcItem *item, items)
		{
			if ( item->m_eObjType == WORKING_LIST_OBJECT) root.appendChild(((QcalcItem*)item)->toXMLElement(doc));
		}
					
		foreach(QcalcItem *item, items)
		{
			if ( item->m_eObjType != WORKING_LIST_OBJECT) root.appendChild(((QcalcItem*)item)->toXMLElement(doc));
		}
					
   		root.appendChild(createTextXMLNode(doc, "Time_Ivl_Select",  _TimeIvlSelect->currentIndex() ));
   		root.appendChild(createTextXMLNode(doc, "Time_Ivl_Text",  _IvlSource->text() ));

   		root.appendChild(createTextXMLNode(doc, "Time_Tag_Select",  _TimeTagSelect->currentIndex() ));
   		root.appendChild(createTextXMLNode(doc, "Time_Tag_Text",  _TagSource->text() ));

   		root.appendChild(createTextXMLNode(doc, "Join_First",  multiJn_check1->isChecked() ));
   		root.appendChild(createTextXMLNode(doc, "Join_Last",  multiJn_check2->isChecked() ));

		QString xml = doc.toString();
		file << xml.toLatin1().data();
		file.close();
	}

  return;

 }


 void QcalcWindow::saveAs()
 {

 if ( QuSlotDebug() )
 {
   printf( "QcalcWindow::saveAs \n" );
   fflush(stdout);
 }

   QString _filter = QString( "save (*" );
   _filter        += ".qx_calc_old";
   _filter        += QString( ")" );
   QString fn = QFileDialog::getSaveFileName( this, "Select file - Save Join Settings", _filename, _filter ) ;

   if ( !fn.isEmpty() )
   {
     int pos = fn.lastIndexOf( ".qx_calc_old" );
     if ( pos < 0 )
     {
       pos = fn.lastIndexOf( "." );
       if ( pos > 0 )
       {
           fn.truncate( pos );
       }

       fn += ".qx_calc_old";
     }

     _filename = fn;
     save();
   }
   else
   {
     messageHandler( "saving aborted ", true );
   }

 }
      


void QcalcWindow::saveSession( QString folder )
{

 if ( QuSlotDebug() )
 {
	printf("QcalcWindow::saveSession \n");
	fflush(stdout);
 }

  if ( folder.isEmpty() )
  {
     return;
  }

  // keep original name
  QString fileN = QString( _filename.toLatin1().data() );

  // set session folder and save
  this->_filename  = QString( folder.toLatin1().data() );
  this->_filename += QString( "/saveSet" );
  this->_filename += ".qx_calc_old";
  save();

  // put file name back
  _filename = QString( fileN.toLatin1().data() );
}



void QcalcWindow::save(Vostream& v_os)
{
	
}
/*

 void QcalcWindow::saveAs()
 {

 if ( QuSlotDebug() )
 {
   printf( "QcalcWindow::saveAs %ld\n", (long)this );
   fflush(stdout);
 }

   QString _filter = QString( "save (*" );
   _filter        += SAVE_EXTENSION;
   _filter        += QString( ")" );
   QString fn = QFileDialog::getSaveFileName( this, "Select file - Save Join Settings", _filename, _filter ) ;

   if ( !fn.isEmpty() )
   {
     int pos = fn.lastIndexOf( SAVE_EXTENSION );
     if ( pos < 0 )
     {
       pos = fn.lastIndexOf( "." );
       if ( pos > 0 )
       {
           fn.truncate( pos );
       }

       fn += SAVE_EXTENSION;
     }

     _filename = fn;
     save();
   }
   else
   {
     messageHandler( "saving aborted ", true );
   }

 }
      

 void QcalcWindow::save()
 {

 if ( QuSlotDebug() )
 {
	printf( "QcalcWindow::save %ld\n", (long)this ) ;
	fflush(stdout);
 }


  _join_listview->with = 0;
  if ( _filename.isEmpty() )
  {
       saveAs();
       return;
  }

  try
  {
      VXDRostream xdr_os( _filename.toLatin1().data() );
      save(xdr_os);
  }
  catch ( Exception &e )
  {
       messageHandler( e.msg().c_str(), true );
       return;

  }

  messageHandler( _filename+" saved", false );
  return;

 }


void QcalcWindow::saveSession( QString folder )
{

 if ( QuSlotDebug() )
 {
	printf("QcalcWindow::saveSession %ld\n", (long)this);
	fflush(stdout);
 }

  if ( folder.isEmpty() )
  {
     return;
  }

  // keep original name
  QString fileN = QString( _filename.toLatin1().data() );

  // set session folder and save
  this->_filename  = QString( folder.toLatin1().data() );
  this->_filename += QString( "/saveSet" );
  this->_filename += this->SAVE_EXTENSION;
  save();

  // put file name back
  _filename = QString( fileN.toLatin1().data() );
}

*/

void QcalcWindow::restore( Vistream& v_is, bool replace )
{
	QcalcItem *tagItem = NULL;   
	QcalcItem *ivlItem = NULL;

	string guiID; 
	v_is >> guiID;
	char *ptr;
   
	if( QuSlotDebug() ) printf("QcalcWindow::restore set ID =  %s\n++++++++++++++++++++++\n", guiID.c_str());
  
	if( (ptr = (char*)strstr(guiID.c_str(), "QSASQcalcV")) == NULL)
	{
		// changed to QSAS_VERSION_ID at 2.3.04
		setRestoreVersion(guiID.c_str());
		qcalc_version = 2.1; // from here on use QSAS_VERSION_ID
	}
	else
	{
		qcalc_version = strtod( &(ptr[10]), NULL);
		// vs 1.1 change in isArray to virtualArrDim in calc at 2.0
		
		setRestoreVersion("QSAS 2.3.03"); // old saveset so ensure old Time class is used
	}
	  
	_join_listview->blockSignals( true );
   
	// clear list for true restore
	if(replace)  _join_listview->deleteAllItems();
      
	string aliasExt;
	v_is >>  aliasExt;
	SAVE_ALIAS_EXT = QString(aliasExt.c_str());

   	int cnt;
	v_is >> cnt;
        

	for ( int i = 0; i < cnt; i++ )
	{
		int entryType;
		v_is >>  entryType; // this ensures correct item type constructed
		int eObjType;
		v_is >> eObjType; // this ensures correct calc tab type constructed
		QString WL;
		if (eObjType == WORKING_LIST_OBJECT) WL = QString("WL");
		else WL = QString("");
		
		QcalcItem* item = NULL;
		_join_listview->calcModel()->emitLayoutAboutToBeChanged();
		item = new QcalcItem( _join_listview,  "Restored", WL );
		_join_listview->calcModel()->emitLayoutChanged();
		
		if ( item )
		{
			v_is >> *item;
			v_is >> item->isIvlObj;
			v_is >> item->isTagObj;
			if(item->isIvlObj == 1) ivlItem = item;
			if(item->isTagObj == 1) tagItem = item;
		}
		
	}

		
   // set copy to pre-restore values for recovery option

   _TimeIvlSelect_cpy = _TimeIvlSelect->currentIndex();
   int tIvl;
   v_is >> tIvl;
   _TimeIvlSelect->setCurrentIndex( tIvl );
   setTTivl(tIvl);

   _IvlSource_cpy = QString(_IvlSource->text().toLatin1().data());
   v_is >> *_IvlSource;

   _TimeTagSelect_cpy = _TimeTagSelect->currentIndex();
   int tTag;
   v_is >> tTag;
   _TimeTagSelect->setCurrentIndex( tTag );
   setTTsource( tTag );

   _TagSource_cpy = QString(_TagSource->text().toLatin1().data());

   v_is >> *_TagSource;
   
   // Joining options:
   bool bJoinFirst;
   bool bJoinLast;
   v_is >> bJoinFirst;
   v_is >> bJoinLast;

   multiJn_check1->setChecked( bJoinFirst );
   multiJn_check2->setChecked( bJoinLast );
   
   	// now all items have been restored to list reset aliases and references to them
	QList<QcalcItem*> items = _join_listview->listItems();	

	foreach(QcalcItem *item, items)
  	{
		if ( item && item->m_pcalcTab ){
			QString newName = item->_alias;
			int posn = newName.indexOf(SAVE_ALIAS_EXT);
 	 		if(posn != -1) {
				newName.truncate(posn);
				while ( _join_listview->findAlias(newName) ){
					newName+="_A";
				}
				item->reNameAlias(newName.toLatin1().data());
			}
			for(int i=0; i<item->m_pcalcTab->nSlots; i++){
				item->m_pcalcTab->operandStack[i]->slot->reValidate();
			}
		}
  	}

	// finally we can reset the tag and ivl slots to the new name if needed
	if( tagItem != NULL ) _TagSource->setText( tagItem->WLcontext.toLatin1().data());
	if( ivlItem != NULL ) _IvlSource->setText( ivlItem->WLcontext.toLatin1().data());
	_TagSource->reValidate();
	_IvlSource->reValidate();
	
	_join_listview->blockSignals( false );

	if(!replace) _join_listview->validateWLitems();
	
	setTimeFeedback();
	logArea->setFocus(); // allows selected item to be deleted as it makes window active
}

