#include "qcalc.h"
#include "qdutil.h"


using namespace QSAS;


bool QcalcWindow::joinSlotObjects(QcalcTab *qTab, vector <QdObject_var>   &returnList )
{
    returnList.clear();
 
    QdObjectSeq_var  qdToJoinList = new QdObjectSeq();
    vector <int> ToJoinIndx;
    QdObjectSeq_var qdJoinedList= new QdObjectSeq();
    vector <QdObject_var>   noActionList;
    vector <int> noActionIndx;
	vector <double>	  gap_or_width;
	QdStringSeq_var	  gap_method = new QdStringSeq();
	QdStringSeq_var	  join_method = new QdStringSeq();
	
    if ( qdToJoinList.is_nil() ||  qdJoinedList.is_nil() ) return true;


    for ( int nIndx = 0; nIndx < qTab->nSlots; nIndx++ )
    {
		QcalcItem*  qOperandItem = NULL;
    		if(qTab->virtualItem->m_qstrOperator == strOpJoin) 
		{
			if( !(qTab->saveWL_check->isChecked()) ) return false; // join only and not saved so don't join
			// use join options of this tab
			qOperandItem = qTab->virtualItem;
		}
		else
		{
			// get item slot refers to for join options
			QString Op_i = qTab->operandStack[nIndx]->slot->textObject();
			qOperandItem =  this->_join_listview->findAlias(Op_i);
		}
		QdObject_var  qdObj = qTab->operandStack[nIndx]->slot->getSlotObject();
        	if ( qdObj.is_nil()  ) return true; // can't get object at all
		
		// if object is already joined do nothing (e.g. if this object used as target tt or joined earlier)
		if( qOperandItem &&
		    qOperandItem->m_pcalcTab->_algorithm->currentText() != QString("boxcar average") && 
		    QuIsJoined(qdObj, (QdObject_var)_join_listview->targetD0) ) {
		    
				// pass back  as already joined	
				QString qstrNote = QString("Already joined: ") + qTab->operandStack[nIndx]->slot->text() ;
				QcalcAppendTextDisplay( qstrNote.toLatin1().data() );
				noActionList.push_back( qdObj );
				int OpID = nIndx;
				noActionIndx.push_back(OpID);
		}	
		// if object has DEPEND_0 add to join list
		else if ( qdObj->xref_exists(DEPEND_0) )
		{
			if( qOperandItem == NULL ) return true; // data object but no item REALLY shouldn't happen
			
			if ( qOperandItem->m_eObjType == WORKING_LIST_OBJECT )
			{
		
				bool bOK;
				// gap tolerance in seconds
				double gap_ptr = qOperandItem->m_pcalcTab->_gap_field->text().toDouble( &bOK );
				if(!bOK) gap_ptr = -1.0; // let QuMultijoin work it out

				// interpolation method
				QdString interp_ptr( (qOperandItem->m_pcalcTab->_algorithm->currentText()).toLatin1().data() );
				join_method->push_back( interp_ptr );

	 			if(interp_ptr == string(BOXCAR) ){
					// boxcar width in seconds
					gap_ptr = qOperandItem->m_pcalcTab->_width_field->text().toDouble( &bOK );
				}
				gap_or_width.push_back(gap_ptr);

				// gap handling option
				QdString gap_opt_ptr( (qOperandItem->m_pcalcTab->_gap->currentText()).toLatin1().data() );
				gap_method->push_back( gap_opt_ptr );

				qdToJoinList->push_back( qdObj );
				int OpID = nIndx;
				ToJoinIndx.push_back(OpID);
			}
		}
		else
		{
		 	// pass back unjoined, no tt either constant or text or data series			
			noActionList.push_back( qdObj );
			int OpID = nIndx;
			noActionIndx.push_back(OpID);
		}
	}

	if( qdToJoinList->size() > 0  )
	{	
		QuMultiJoin( qdToJoinList, _join_listview->targetD0, qdJoinedList, gap_or_width, gap_method, join_method );
		
	}

	// assemble return stack and add constants etc back
	int J_id=0;
	int nJ_id=0;
	for ( int nIndx = 0; nIndx < qTab->nSlots; nIndx++ )
	{
		if( ToJoinIndx.size() > J_id && ToJoinIndx[J_id] == nIndx)
		{
			// this was put on joined list
			returnList.push_back(qdJoinedList[J_id]);
			J_id++;
		}
		else if ( noActionIndx.size() > nJ_id && noActionIndx[nJ_id] == nIndx)  
		{
			// this was put on no ACtion list
			returnList.push_back(noActionList[nJ_id]);
			nJ_id++;
		}
		else return true;  // failed to resolve slot
	}
	
	return false;  // no errors found, returnList contains as many values as there were slots
}


void QcalcWindow::GetJoinOptionObject( QcalcItem* item, QdObject_var obj, double &gap_or_width, QdString &gap_method, QdString &join_method, string &name  )
{
	bool OK;
	// gap tolerance in seconds
	gap_or_width = item->m_pcalcTab->_gap_field->text().toDouble( &OK );


	// interpolation method
	join_method =  QdString( item->m_pcalcTab->_algorithm->currentText().toLatin1().data() );

	if(join_method == string(BOXCAR) ){
	// boxcar width in seconds
		gap_or_width =  item->m_pcalcTab->_width_field->text().toDouble( &OK );
	}
	
	// gap handling option
	gap_method =  QdString( item->m_pcalcTab->_gap->currentText().toLatin1().data() );

	name = string( item->_alias.toLatin1().data() );

	return;
}


bool QcalcWindow::joinObjects(QdObjectSeq_var         JoinedList,
							  vector <string>         &nameList,
						      bool                    bSkipVirtual )
{

	if  ( QuSlotDebug() )
		printf( "QcalcWindow::joinObjects %ld\n", (long)this );

	QdObjectSeq_var  ToJoinList = new QdObjectSeq();
	QdRScalar_var    fillval_ptr;

	vector <double>	  gap_or_width;
	QdStringSeq_var	  gap_method;
	QdStringSeq_var	  join_method;

	// For each item on Join List, set choices

	QList<QcalcItem*> items = _join_listview->listItems();
	foreach(QcalcItem *item, items)
	{
		bool OK;
		
		if(!item) continue;
		
		QdObject_var obj;
		if( item->m_eObjType == VIRTUAL_OBJECT && !bSkipVirtual)
		{
			obj = item->m_qdObjResult;
		}
		else if(item->m_eObjType == WORKING_LIST_OBJECT)
		{
			obj = item->m_pcalcTab->operandStack[0]->slot->getSlotObject();
		}
		
		QdObject_var qd0 = get_Depend_0( obj ); // safe against null obj
		
		if ( qd0.is_nil() )
		{
			continue;
		}

		if ( disjointSeries( obj ) )
		{
			QString ctxt = item->WLcontext;
			( void )QMessageBox::warning( this, "Join", "disjoint series - "+ctxt );
			qApp->processEvents();
		}
		
		double gap;
		QdString gap_opt;
		QdString join_opt;
		string name;
		
		GetJoinOptionObject(item, obj, gap, gap_opt, join_opt, name );
		gap_or_width.push_back(gap);
		gap_method->push_back(gap_opt);
		join_method->push_back(join_opt);
		nameList.push_back(name);
		
		ToJoinList->push_back( obj );

	}

	if ( ToJoinList->size() == 0 )  return ( true ); // no joining was needed

	// Do actual join
	
	messageHandler( "Performing join...", false );
	QuMultiJoin( ToJoinList, _join_listview->targetD0, JoinedList, gap_or_width, gap_method, join_method );
	

	return ( true );

}


bool QcalcWindow::disjointSeries( QdObject_var obj )
{
// checks whether obj is disjoint from target dependency Series.

	if ( obj.is_nil() ) return true;

	// fetch time interval associated with this object
	QdTimeInterval_var obj_ivl = get_timeinterval( obj );
	if ( obj_ivl.is_nil() ) {
		double start, end;
		if(get_ScalarInterval(obj, start, end) ){
			QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(_join_listview->targetD0);
			if(ss.is_nil()) return false;
			if ( start < end && (ss[ss->sequence_size()-1] < start || end < ss[0] ) ) return true;
			if ( start > end && (ss[ss->sequence_size()-1] > start || end > ss[0] ) ) return true;

		}
		else return false;
	}
	QdTimeInterval_var join_ivl = get_timeinterval( _join_listview->targetD0 );
	if ( join_ivl.is_nil() ) return false;
	
	if ( obj_ivl->int_intersect( *join_ivl ).empty() )
	{
		// are disjoint
		return true;
	}
	else return false;

}


bool QcalcWindow::QcalcGetScalarInterval(double &start, double &end )
{
	bool OK;
	if ( _TimeIvlSelect->currentIndex() == 0)
	{
		// interval from data
		OK = QcalcGetScalarIntervalFromData(start, end);
	}
	else
	{
		// interval from object (utility call get_timeinterval is safe against nil object
		QdObject_var slotObj = _IvlSource->getSlotObject() ;
		OK = get_ScalarInterval(slotObj, start, end );
	}

	if ( OK )
	{
		if(QuSlotDebug()){
			cout << "Scalar interval set: " << double_to_string(start) << " to " << double_to_string(end) << endl;
		}
	}
	
	return OK;
}



QdTimeInterval_var QcalcWindow::QcalcGetInterval( )
{
	QdTimeInterval_var ivl;

	if ( _TimeIvlSelect->currentIndex() == 0)
	{
		// interval from data
		ivl = QcalcGetIntervalFromData();
	}
	else
	{
		// interval from object (utility call get_timeinterval is safe against nil object
		QdObject_var slotObj = _IvlSource->getSlotObject() ;
		ivl = get_timeinterval(slotObj );
	}

	if ( !ivl.is_nil() )
	{
		if(QuSlotDebug()){
			cout << "Time interval set:-" << ivl->start().getISOstring(0) << " - " << ivl->end().getISOstring(0) << endl;
		}
	}
	
	return ivl;
}

QdTimeInterval_var QcalcWindow::QcalcGetIntervalFromData()
{
	QdTimeInterval_var ivl;
	QdTimeInterval_var ivlIntersect;

	QList<QcalcItem*> items = _join_listview->listItems();
	foreach(QcalcItem *item, items)
	{
		if ( item->m_eObjType == WORKING_LIST_OBJECT)
		{
			QdObject_var ivl_obj = QuGetObjByNameAndComp(item->m_pcalcTab->operandStack[0]->slot->text()); //QuGetObjByName( next->getWLcontext() );
		
			if ( !ivl_obj.is_nil() )
			{
				ivl = QdTimeInterval_var::narrow( ivl_obj );
				if ( ivl.is_nil() )
				{
					QdTimeSeq_var ts = get_timetags( ivl_obj );
					if ( !ts.is_nil() )
					{
						ivl = new QdTimeInterval( ts->get_interval_bounds() );
					}
				}
			}

			if ( ivlIntersect.is_nil() )
			{
				ivlIntersect = ivl;
			}
			else if ( !ivl.is_nil() )
			{
				QdTimeInterval_var ivlNew = new QdTimeInterval( ivlIntersect->int_intersect( *ivl ) );
				ivlIntersect = ivlNew;
			}
		}
	}

	return ivlIntersect;
}

bool QcalcWindow::QcalcGetScalarIntervalFromData(double &start, double &end)
{

	bool OK = false;
	double obj_start, obj_end;
	
	QList<QcalcItem*> items = _join_listview->listItems();
	foreach(QcalcItem *item, items)
	{
		if ( item->m_eObjType == WORKING_LIST_OBJECT)
		{
			QdObject_var ivl_obj = QuGetObjByNameAndComp(item->m_pcalcTab->operandStack[0]->slot->text()); //QuGetObjByName( next->getWLcontext() );
		
			if( get_ScalarInterval(ivl_obj, obj_start, obj_end ) )
			{		
				if(OK){	
					if(start < end){
						if(start > obj_start) start = obj_start;
						if(end < obj_end) end = obj_end;
					}
					else{
						if(start < obj_start) start = obj_start;
						if(end > obj_end) end = obj_end;
					}	
				}
				else{
					// first valid data	
					start = obj_start;
					end = obj_end;
				}
				OK = true;
				
			}
		}
	}

	return OK;
}


QdTimeSeq_var QcalcWindow::getTimeTags( )
{

	QdTimeSeq_var      tags;
	QdTimeInterval_var ivl;
	double             Spacing;
	double             Tinterval;
	
	if  ( QuSlotDebug() )
	 	printf( "QcalcWindow::getTimeTags %ld\n", (long)this );
		
	// set interval
	ivl = QcalcGetInterval( );

	if (ivl.is_nil() ) return tags;   // will be .is_nil()
	Tinterval = ivl->duration_abs();

	// set tags

	if ( _TimeTagSelect->currentIndex() == 0 )
	{
 		// create regular tags
		bool OK;
		Spacing = _TagSource->text().toDouble(&OK);
		if ( !OK || Spacing < 1.e-20 )
		{
			return tags;
		}

		int Ntags = (int) ( Tinterval / Spacing  +  1.0 );

		tags = new QdTimeSeq();
		tags->assign_regular( Ntags, ivl->start(), Spacing );

	}
	else
	{
		// tags from object, utility get_timetags() is safe against nil object
		QdTimeSeq_var  ts = get_timetags( _TagSource->getSlotObject());
		if ( ts.is_nil() )
		{
			messageHandler("Time tag source did not give valid tags", true);
			return tags;    // will be .is_nil()
		}

		// truncate to interval if necessary

		if (   ( ivl->start()  != ts->get_interval_bounds().start() )
			|| ( ivl->end()   != ts->get_interval_bounds().end()    )    )
		{
			SequenceMask msk( ts->get_mask( QdTimeSeq::InRange( ivl->start(),ivl->end() ) ) );
			tags = new QdTimeSeq();
			ts->get_subsequence( *tags, msk );
		}
		else
		{
			tags = new QdTimeSeq(*ts);
		}
	}

	return tags;
}

QdRScalarSeq_var QcalcWindow::getScalarTags( )
{

	QdRScalarSeq_var	stags;
	double 				start, end;
	double				Spacing;
	double 				Interval;
	
	if  ( QuSlotDebug() )
	 	printf( "QcalcWindow::getScalarTags %ld\n", (long)this );
		
	// set interval
	if( !QcalcGetScalarInterval(start, end) ) return stags;   // will be .is_nil()

	Interval = end - start;

	// set tags

	if ( _TimeTagSelect->currentIndex() == 0 )
	{
 		// create regular tags
		bool OK;
		Spacing = _TagSource->text().toDouble(&OK);
		if ( !OK || fabs(Spacing) < 1.e-20 )
		{
			return stags;
		}

		if(Interval < 0) Spacing = -fabs(Spacing); // force negative for decreasing seq
		int Ntags = (int) ( fabs(Interval/Spacing)  +  1.0 );

		stags = new QdRScalarSeq();
		stags->assign_regular( Ntags, start, Spacing );

	}
	else
	{
		// tags from object, utility get_timetags() is safe against nil object
		QdObject_var  D0 = get_Depend_0( _TagSource->getSlotObject());
		if ( D0.is_nil() )
		{
			messageHandler("Scalar tag source did not give valid tags", true);
			return stags;    // will be .is_nil()
		}
		
		QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(D0);
		if ( ss.is_nil() )
		{
			messageHandler("Scalar tag source did not give scalar tags", true);
			return stags;    // will be .is_nil()
		}
		
		// truncate to interval if necessary

		if ( start < end && (ss[0] < start || ss[ss->sequence_size()-1] > end) ) {

			SequenceMask msk(ss->get_mask(QdRScalarSeq::InRange(start,end)));
			stags = new QdRScalarSeq();
			ss->get_subsequence(*stags, msk);
		}
		else if ( start > end && (ss[0] > start || ss[ss->sequence_size()-1] < end) ) {

			SequenceMask msk(ss->get_mask(QdRScalarSeq::InRange(start,end)));
			stags = new QdRScalarSeq();
			ss->get_subsequence(*stags, msk);
		}
		else
		{
			stags = new QdRScalarSeq(*ss);
		}
	}

	return stags;
}


void QcalcWindow::messageHandler( QString txt, bool bError )
{
	if ( QuSlotDebug() )
		printf( "QcalcWindow::messageHandler %ld\n", (long)this );

	QPalette palette = QPalette(qApp->palette());
	
	if ( bError ){
		palette.setColor(QPalette::Active, QPalette::Text,  QPalette::Dark);
		palette.setColor(QPalette::Active, QPalette::Background,  QPalette::Link);
	}
	else
	{
		palette.setColor(QPalette::Inactive, QPalette::Text,  QPalette::Text );
		palette.setColor(QPalette::Inactive, QPalette::Background, QPalette::Background   );
	}

	this->msgBar->setPalette( palette );

	this->status( txt, 50000 );

}


QString getOpName(int OpNum){
 
  QString nextName = QString("Operand ") + getOpChar(OpNum) + QString("  ");
 
  return nextName;
  
}

QString getOpChar(int OpNum){
  // operand number starts at 0
  int charNum = OpNum + 65;
  
  if(charNum < 92) {
    return QString(QChar(charNum));
  }
  else
  {
    QString num;
    num.setNum(charNum);
    return num;
  }
  
}

