#include "qcalc.h"
#include "qwui.h"

using namespace QSAS;

extern qwuiWindow* qsasWindow;

// QcalcItem class
// -----------------------------


QcalcItem::QcalcItem( QcalcListView *calcView,
					  QString      txt   ,
					  QString      WL    )
{
	if ( QuSlotDebug() ) printf("QcalcItem::QcalcItem named %s \n", txt.toLatin1().data());

	if ( calcView == NULL ) return;
	_view = calcView;	
	_parent = NULL;
	m_pcalcTab = NULL;
	
	// initialise flag (virtualArrDim is a var ptr, so safe
	this->isVector = false; 
	
	m_qstrOperator    = QString("");
	m_qdObjResult     = NULL;
	m_bCalcDone       = FALSE;
	m_pcalcTab        = NULL;
	if(WL != QString("WL")) WLcontext = QString( WL.toLatin1().data() );
	else WLcontext=QString("");
	
	if(txt == "__root__") 
	{
		_alias = QString("");
		_name = QString("");
		return;
	}
	else if(txt == "Restored" )
	{
		_name = QString("");
	}
	else _name = txt;
	
	_parent = _view->calcModel()->_root;
	
	_parent->_children.push_back(this);
	
	QString qstrAlias = QString( this->GetNextAliaseName().toLatin1().data() );
	this->setAlias( qstrAlias );

	m_pcalcTab = new QcalcTab();

	if ( !m_pcalcTab ) return; // failed to allocate space

	m_pcalcTab->nSlots = 0;
	m_pcalcTab->calcListview = _view;
	m_pcalcTab->createCalcTabWidgets(this);
	m_pcalcTab->details_groupbox->setTitle( txt );
	_view->owner->tabWidget->addTab( this->m_pcalcTab, qstrAlias );
	m_pcalcTab->setObjectName( qstrAlias );
	
	if(WL != "") // object from working list
	{
		m_eObjType   = WORKING_LIST_OBJECT;
		m_pcalcTab->operandStack[0]->setText( WLcontext );

		m_pcalcTab->SetOperatorByName(strOpJoin);
			

		if(_view->underMouse()) _view->showTab(m_pcalcTab);
		QdObject_var dobj = this->getDataObject();
			
		if(!(dobj.is_nil()) ){
			if( QuIsThreeVector(dobj) ){
				this->isVector = true;
			}
			else{
				QdRMatrix_var Mat_obj = QdRMatrix_var::narrow(dobj);
				QdRMatrixSeq_var MatSeq_obj = QdRMatrixSeq_var::narrow(dobj);
				if ( !Mat_obj.is_nil()  ) {
					this->virtualArrDim = new Dim(Mat_obj->dimdata());
				}
				else if(!MatSeq_obj.is_nil() ){
					this->virtualArrDim = new Dim(MatSeq_obj[0].dimdata());
				}
				// else virtualArrDim is_nil()
			}
		}
		// if Time Tag object is not set, use this
		if(  ( _view->owner->_TagSource->text() == QString(" ") || _view->owner->_TagSource->text().isEmpty() )
		   &&  _view->owner->_TimeTagSelect->currentIndex() == 1) {

			QdObject_var D0 = get_Depend_0(dobj);
			if(!(D0.is_nil()) ){
				_view->owner->_TagSource->setText(WLcontext);
				_view->owner->_TagSource->reValidate(true);
			}
		}
	}
	else{
		m_eObjType   = VIRTUAL_OBJECT;
		WLcontext = QString( "" );
		m_pcalcTab->SetOperatorByName(strOpAdd);
		_view->showTab(m_pcalcTab);
		m_pcalcTab->saveWL_check->setChecked(true);
		m_pcalcTab->toggleWLName();
	}
	this->setChoices( );
		
}


QcalcItem::~QcalcItem()
{
		// remove tab
	if(m_pcalcTab != NULL) 
	{
		_view->owner->tabWidget->removeTab( _view->owner->tabWidget->indexOf(m_pcalcTab) );
		delete m_pcalcTab;
	}
	// remove from list of children in root
	if(_parent != NULL) _parent->_children.removeOne(this);
}

void QcalcItem::setColour(bool OK )
{
	// TODO ===
}

int QcalcItem::childNumber()
{
	if(_parent!=NULL)
	{
		return _parent->_children.indexOf(this);
	}
	return -1;
}

bool QcalcItem::ancestorInList(QList <QcalcItem*> &list)
{
	for(int i=0; i<list.size(); i++)
	{
		if(this != list[i] && this->isDescendentOf(list[i]) ) return true;	
	}
	return false;
}

bool QcalcItem::isDescendentOf(QcalcItem* other)
{
	if(this==other) return true;
	
	if(_parent==NULL) return false;
	
	return _parent->isDescendentOf(other);
}

void QcalcItem::setName(QString name )
{	
	_name = name;
	_view->calcModel()->setData(_view->calcModel()->indexFromItem(this, 0), name);
}

void QcalcItem::setAlias(QString alias)
{
	_alias = alias;
	_view->calcModel()->emitLayoutAboutToBeChanged();
	_view->calcModel()->setData(_view->calcModel()->indexFromItem(this, 1), alias);
	_view->calcModel()->emitLayoutChanged();
}

QcalcItem* QcalcItem::getChildByName(const QString& name)
{
	if(_children.isEmpty()) return NULL;
	for(QList<QcalcItem*>::const_iterator iter=_children.begin(); iter!=_children.end(); ++iter)
	{
		if((*iter)->_name==name) return *iter;
	}
	return NULL;
}

QcalcItem* QcalcItem::getChildByAlias(const QString& alias)
{
	if(_children.isEmpty()) return NULL;
	for(QList<QcalcItem*>::const_iterator iter=_children.begin(); iter!=_children.end(); ++iter)
	{
		if((*iter)->_alias==alias) return *iter;
	}
	return NULL;
}

QcalcItem* QcalcItem::getChildByNumber(int i) const
{
	if(i<0 || i>=_children.size()) return NULL;
	
	return _children.at(i);
}


void QcalcItem::moveTo(int posn)
{
	_view->calcModel()->emitLayoutAboutToBeChanged();
	_parent->removeChild(this, false);
	_view->calcModel()->emitLayoutChanged();
	qApp->processEvents();

	_view->calcModel()->emitLayoutAboutToBeChanged();
	_parent->addChild(this, posn);
	_view->calcModel()->emitLayoutChanged();
	qApp->processEvents();

}

void QcalcItem::setChoices(  )
{

	if ( _view == NULL ) return;
	
	m_pcalcTab->_algorithm->setCurrentIndex(_view->owner->_algorithm_Def);

	m_pcalcTab->_width_field->setText(_view->owner->_width_field_Def.toLatin1().data());

	m_pcalcTab->_gap->setCurrentIndex(_view->owner->_gap_Def);

	m_pcalcTab->_gap_field->setText(_view->owner->_gap_field_Def.toLatin1().data());
 
}

bool QcalcItem::removeChild(QcalcItem* child, bool alsoDelete)
{
	int i=_children.indexOf(child);
	
	if(i<0) return false;
	
	_children.removeAt(i);
	
	if(alsoDelete)
	{
		delete child;
	}
	return true;
}

bool QcalcItem::removeChild(int i, bool alsoDelete)
{
	if(i<0 || i>=_children.size())
		return false;
	
	if(alsoDelete)
	{
		QcalcItem* child=_children.takeAt(i);
		delete child;
	}
	else
	{
		_children.removeAt(i);
	}
	return true;
}

bool QcalcItem::addChild(QcalcItem* newChild, int position)
{
	if(newChild==NULL || _children.indexOf(newChild)>=0) return false;
	
	QString childName=newChild->_name;
	while(getChildByName(childName))
	{
		childName.append("A");
	}
		
	newChild->_parent=this;
	if(position<0 || position>=_children.size())
	{
		_children.push_back(newChild);
	}
	else
	{
		_children.insert(position, newChild);
	}
	newChild->_name=childName;

	
	return true;
}

bool QcalcItem::setArrayFlags(){

	// the flags specify the object type output by this item BEFORE component selection.
	// Use getIsVector() method to get the types as they are delivered by the item (WL items may have components selected)
	// return value indicates if the Array/Vector flags have changed
	
	if  ( QuSlotDebug() )
		printf("QcalcItem::setArrayFlags %ld \n", (long)this);

	if(_parent == NULL) return false;
	
    bool oldVec = this->isVector;
    Dim_var oldArr;
    Dim_var ArrDim0;
    Dim_var ArrDim1;
    if ( !(this->virtualArrDim.is_nil()) ) oldArr = new Dim(*(this->virtualArrDim));

    if ( this->m_eObjType == VIRTUAL_OBJECT )
    {
		bool isVector0;
		bool isVector1;
		// redetermine if output is vector or array for virtual item
		this->virtualArrDim = new Dim(); // empty dim
		this->isVector = false;

  		// try to resolve input types 

		QcalcItem * qItem = _parent->getChildByAlias(this->m_pcalcTab->operandStack[0]->slot->textObject() );
   		
		isVector0 = false;
		if( qItem) {
			isVector0 = qItem->getIsVector();
		
			if(isVector0){
				// allow for component selection in slot
				if(this->m_pcalcTab->operandStack[0]->slot->getComponent() != WL_OBJ) isVector0 = false;
			}
			else{
				// set array dim based on first input (so for no operator can change array dimensions)

				ArrDim0 = qItem->getVirtualArrDim();

			}
		}

	 	isVector1 = false;
		if (this->m_pcalcTab->nSlots > 1) {
			 qItem = _parent->getChildByAlias(this->m_pcalcTab->operandStack[1]->slot->textObject() );
			if( qItem) {
				isVector1 = qItem->getIsVector(); 
			
				if(isVector1) {
					// allow for component selection in slot
					if(this->m_pcalcTab->operandStack[1]->slot->getComponent() != WL_OBJ) isVector1 = false;
				}
				else{
					// re-set array dim based on second input (1st may be scalar)
					ArrDim1 = qItem->getVirtualArrDim();
				}
			}
		}
	
		// set isVector flag feedback

		if( !ArrDim0.is_nil() &&  !ArrDim1.is_nil() ) {
		  if( ArrDim0->size() > ArrDim1->size() ) this->virtualArrDim = ArrDim0;
		  else  this->virtualArrDim = ArrDim1;
		}
		else if( !ArrDim0.is_nil() ) this->virtualArrDim = ArrDim0;
		else if( !ArrDim1.is_nil() ) this->virtualArrDim = ArrDim1;
		
		// set operand feedback

		if(this->m_qstrOperator == strOpAdd || this->m_qstrOperator == strOpSub){
			if( isVector0 && isVector1 ) this->isVector = true; 
		}
		else if(this->m_qstrOperator == strOpMult){
			if( isVector0 ||  isVector1) this->isVector = true; 
			if( isVector0 &&  isVector1) this->isVector = false;  // undo true for disallowed op
			
			if( !ArrDim0.is_nil() &&  !ArrDim1.is_nil() ) {
				// construct dim for product
				Dim_var vDim;
				
				// get reduced array from slot else keep dim from list object
				vDim = this->m_pcalcTab->operandStack[0]->slot->getSlotArrDim();
				if ( !vDim.is_nil() ) ArrDim0 = new Dim(*vDim); 
				
				vDim = this->m_pcalcTab->operandStack[1]->slot->getSlotArrDim();
				if ( !vDim.is_nil() ) ArrDim1 = new Dim(*vDim); 
				
				vector <int> dimList;
				if (ArrDim0->rank() > 0 && ArrDim1->rank() > 0) {
					for(int i=0; i < ArrDim0->rank()-1; i++){
						// ist array loses last dimension
						int elems0 = (*ArrDim0)[i];
						dimList.push_back(elems0);
					}
					for(int i=1; i < ArrDim1->rank(); i++){
						// second array loses 1st dimension
						int elems1 = (*ArrDim1)[i];
						dimList.push_back(elems1);
					}
					if(dimList.size() > 0){
						vDim = new Dim(dimList);	
						this->virtualArrDim = vDim;
					}
					else{
						Dim_var emptyDim;
						this->virtualArrDim = emptyDim; // scalar return
					}
				}
				else if ( ArrDim0->rank() <= 0 && ArrDim1->rank() > 0){
					this->virtualArrDim = ArrDim1;
				}
				else if ( ArrDim1->rank() <= 0 && ArrDim0->rank() > 0){
					this->virtualArrDim = ArrDim0;
				}
				else{
					Dim_var emptyDim;
					this->virtualArrDim = emptyDim; // scalar return
				} 
			}
		}
		else if(this->m_qstrOperator == strOpDiv){
			// only numerator may be vector
			if( isVector0 ) this->isVector = true; 
		}
		else if(this->m_qstrOperator == strOpMod){
			// always scalar
			Dim_var emptyDim;
			this->virtualArrDim = emptyDim;
		}
		else if(this->m_qstrOperator == strOpPower){
			// always scalar
			Dim_var emptyDim;
			this->virtualArrDim = emptyDim;
		}
		else if(this->m_qstrOperator == strOpChSign){
			if( isVector0 ) this->isVector = true; 
		}
		else if(this->m_qstrOperator == strOpCos){
		}
		else if(this->m_qstrOperator == strOpSin){
		}
		else if(this->m_qstrOperator == strOpTan){
		}
		else if(this->m_qstrOperator == strOpCot){
		}
		else if(this->m_qstrOperator == strOpCosh){
		}
		else if(this->m_qstrOperator == strOpSinh){
		}
		else if(this->m_qstrOperator == strOptanh){
		}
		else if(this->m_qstrOperator == strOpCoth){
		}
		else if(this->m_qstrOperator == strOpACos){
		}
		else if(this->m_qstrOperator == strOpASin){
		}
		else if(this->m_qstrOperator == strOpATan){
		}
		else if(this->m_qstrOperator == strOpACot){
		}
		else if(this->m_qstrOperator == strOpACosh){
		}
		else if(this->m_qstrOperator == strOpASinh){
		}
		else if(this->m_qstrOperator == strOpAtanh){
		}
		else if(this->m_qstrOperator == strOpACoth){
		}
		else if(this->m_qstrOperator == strOpVecProd){
			this->isVector = true; 
		}
		else if(this->m_qstrOperator == strOpDotProd){
		}
		else if(this->m_qstrOperator == strOpRemainder){
		}
		else if(this->m_qstrOperator == strOPAbsValue){
		}
		else if(this->m_qstrOperator == strOpLog){
		}
		else if(this->m_qstrOperator == strOpLog10){
		}
		else if(this->m_qstrOperator == strOpInvers){
		}
		else if(this->m_qstrOperator == strOpSqrt){
		}
		else if(this->m_qstrOperator == strOpExp){
		}
		else if(this->m_qstrOperator == strOpUnits){
			// returns same dimension object as input
			if( !ArrDim0.is_nil()  )  this->virtualArrDim = ArrDim0;
			if( isVector0 ) this->isVector = true; 
		}
		else if(this->m_qstrOperator == strOpFills){
			// returns same dimension object as input
			if( !ArrDim0.is_nil()  )  this->virtualArrDim = ArrDim0;
			if( isVector0 ) this->isVector = true; 
		}
    }
    else{
	 	 // for WL items don't change setting from validation
		QdObject_var dobj =QuGetObjByName(this->m_pcalcTab->operandStack[0]->slot->textObject());
		if(dobj.is_nil()){
			// nothing has changed, no object
			return false;
		}
		else{

			this->isVector = QuIsThreeVector(dobj);
			
			if(  !(this->isVector) && QuArrayDims(dobj) > 0 )  {

				QdRMatrix_var Mat_obj = QdRMatrix_var::narrow(dobj);
				QdRMatrixSeq_var MatSeq_obj = QdRMatrixSeq_var::narrow(dobj);
				if ( !Mat_obj.is_nil()  ) {
					this->virtualArrDim = new Dim(Mat_obj->dimdata());
				}
				else if(!MatSeq_obj.is_nil() ){
					this->virtualArrDim = new Dim(MatSeq_obj[0].dimdata());
				}
				// else virtualArrDim is_nil()

			}
		}
		return true; // this method does not change WL item flags, but may need to propagate selection
    }

    if( oldVec != this->isVector ) return true;
    if( oldArr.is_nil() != this->virtualArrDim.is_nil() ) return true; //array status changed
    if( !(oldArr.is_nil()) &&  !(this->virtualArrDim.is_nil()) && oldArr->size() != this->virtualArrDim->size() ) return true; // size changed
    return false; // no changes made
}
    

void QcalcItem::SetItemName( bool bEmitChange )
{
    
	if ( QuSlotDebug() ) printf("QcalcItem::SetItemName %ld %d\n", (long)this, bEmitChange);

    if ( _view == NULL || _view->owner == NULL) return;

    if(!this->m_pcalcTab) return;
    
    int nNumOperands = this->m_pcalcTab->nSlots;
    QString                qstrVirtName = "";
	
	if(m_qstrOperator == strOpJoin)
	{
		qstrVirtName = QString( m_pcalcTab->operandStack[0]->slot->text().toLatin1().data() );
		if (qstrVirtName.isEmpty()) qstrVirtName = QString("??");
	}
	else if ( m_qstrOperator == "" ) 
	{
		qstrVirtName = QString(" operation not set ");
	}
	else
	{
		qstrVirtName   = QString( m_qstrOperator.toLatin1().data() );
		if(qstrVirtName == strOpUnits)
		{
			// special case of change units
			qstrVirtName = m_pcalcTab->operandStack[0]->slot->text();
			qstrVirtName += " (SIconv= ";
			qstrVirtName += m_pcalcTab->operandStack[1]->slot->text();
			qstrVirtName += ")";
		}
		else if(qstrVirtName == strOpFills)
		{
			// special case of change units
			qstrVirtName = m_pcalcTab->operandStack[0]->slot->text();
			qstrVirtName += " (FillVal= ";
			qstrVirtName += m_pcalcTab->operandStack[1]->slot->text();
			qstrVirtName += ")";
		}
		else
		{
			for(int i=0; i < this->m_pcalcTab->nSlots; i++)
			{
				// is this operand set?
				int nIndCh = qstrVirtName.lastIndexOf( getOpChar(i) ); // must search in reverse in case alias contains OpChar
				if ( nIndCh != -1 )
				{
					if ( m_pcalcTab->operandStack[i]->slot->text().isEmpty() ) 
					{
						qstrVirtName.replace( nIndCh, 1, "?" );
					}
					else
					{
						// use actual text (with component) in slot not base object name
						qstrVirtName.replace( nIndCh, 1, m_pcalcTab->operandStack[i]->slot->text().toLatin1().data() );
					}
				}
				// else don't know how to rename
			}
		}
	}

    if ( !qstrVirtName.isEmpty() )
    {

		// Rename this item within the current list view of objects
		this->setName( qstrVirtName );
		
		// Set the text within the selected object group box
		this->m_pcalcTab->details_groupbox->setTitle( qstrVirtName );

		if ( bEmitChange )  _view->emitListModified();
    }
}


void QcalcItem::DoOperation()
{
	if ( m_bCalcDone )  return;
		
	if(!_view) return;
		
	// Recursive call for virtual item to ensure inputs all exist first
	if ( m_eObjType == VIRTUAL_OBJECT )
	{  
		QcalcItem*  qOperandItem = NULL;
 		for ( int nIndx = 0; nIndx < this->m_pcalcTab->nSlots; nIndx++ )
 		{	
			// is this operand set?
			QString Op_i = m_pcalcTab->operandStack[nIndx]->slot->textObject();
			qOperandItem =  _view->findAlias(Op_i);

			// slot must contain either a constant or an alias 
			if ( qOperandItem == NULL ) continue; // trap for value don't need to do op

			qOperandItem->DoOperation();
  		}

	}
	
	if ( !(this->m_pcalcTab->isOpAllowed()) ){
		// not allowed, but don't keep trying to do it
		m_bCalcDone = true;
		char bufLog[ 1024 ];
		sprintf(  bufLog,"Input(s) NOT VALID for operation %s -> %s",  _alias.toLatin1().data(), _name.toLatin1().data() );
		QcalcAppendTextDisplay( bufLog );
		_view->owner->messageHandler( QString("Alias ")+_alias+QString(" Invalid Inputs to operation"), true );
		return;
	}

    this->DoMathOperation();	
    
}

bool QcalcItem::CheckOperatorSelfLink( QString& strAlias )
{
	bool                   bReturn      = true;
	QcalcItem*  operandItem = NULL;

	if ( this->m_eObjType == WORKING_LIST_OBJECT )
	{
		return ( true );
	}

	if ( strAlias.isEmpty() )
	{
		strAlias = QString( this->_alias );
	}

 	QcalcTab* qTab  = this->m_pcalcTab;
	if (qTab == NULL) return true; // if no tab there are  no links to check
	
	for ( int nIndex = 0; nIndex < qTab->operandStack.size(); nIndex++ )
	{
		QString linkStr = qTab->operandStack[nIndex]->slot->text();
		if(linkStr == strAlias)
		{
			return false;  // self link found here, cascade this failure up the chain
		}
		else
		{
			operandItem = _view->findAlias(linkStr);
			if ( operandItem )
			{
				// test links below on the chain
				bReturn = operandItem->CheckOperatorSelfLink( strAlias );
			}
		}
	}

	// this method does not test for unresolved links, only circular links
	return ( bReturn );
}

QdObject_var QcalcItem::getResultObject()
{
	QdObject_var nil_obj;
	if ( _view == NULL || _view->owner == NULL )  return nil_obj;
   
	if(this->m_eObjType == VIRTUAL_OBJECT)
	{
		// Virtual Object
		return this->m_qdObjResult;
	}
	else
	{
		// WL Object
		if ( _view->owner->multiJn_check1->isChecked() )
		{ 
	         return this->m_qdObjJoinedFirst;
		}
		else
		{
		    // get the selected component for this WL item
		    // note this slot does not itself accept an alias so this is not an infinite recursion
		    return this->m_pcalcTab->operandStack[0]->slot->getSlotObject();
		}
	}

}

void QcalcItem::DoMathOperation()
{
	QcalcTab* qTab  = NULL;
	qTab = this->m_pcalcTab;
	if (!qTab )  return;
	if ( qTab->nSlots == 0 ) return;

	if ( _view == NULL || _view->owner == NULL)  return;

	vector <QdObject_var>   qd_Operands;
	QdOperator      qdOpDetails;
	QString         qstrError       = QString("");
	bool            joinOnly        = false;
	int             numOps          = 0;
    
	char bufLog[ 1024 ];
	sprintf(  bufLog,"Start Operation %s -> %s", _alias.toLatin1().data(),  _name.toLatin1().data() );
	QcalcAppendTextDisplay( bufLog );
	
	sprintf(  bufLog,"End Operation %s -> %s", _alias.toLatin1().data(),  _name.toLatin1().data() );
	
	if (m_qstrOperator == strOpJoin) {
		joinOnly = true;
	}
	else  if (_view->owner->m_mapOps.FindOperator( m_qstrOperator, qdOpDetails ) )
	{
		numOps = qdOpDetails.GetNumOperands();
	}
	else
	{
		qstrError = QString("Error: When performing operation '") + _name + 
				  QString("', cannot be found within the internal operator map");
		QcalcAppendTextDisplay( qstrError.toLatin1().data() );
		QcalcAppendTextDisplay( bufLog );
		return;
	}
	
	// Get Operand Objects: either joining now or already joined
	qd_Operands.clear();

	// Check if pair joining is required

	if ( _view->owner->multiJn_check1->isChecked() )
	{ 
		// already joined
		
		if(joinOnly){
		 	qd_Operands.push_back(this->m_qdObjJoinedFirst); // case that this tab is WL item
		}
		else for ( int nIndx = 0; nIndx < qTab->nSlots; nIndx++ )
		{
			QdObject_var operand_i = qTab->operandStack[nIndx]->slot->getSlotObject();
			if ( operand_i.is_nil() )
			{
				qstrError = QString("Error: When performing operation '" ) + _name  + 
						  QString("' - Operand  (" + qTab->operandStack[nIndx]->slot->text() + ") is null");
				QcalcAppendTextDisplay( qstrError.toLatin1().data() );
				QcalcAppendTextDisplay( bufLog );
				return;
			}
			else
			{
				qd_Operands.push_back(operand_i);
			}
	
		} // end of for loop over slots
	}
	else
	{
		// not multi-joined first, join arguments of this tab
				
		if ( _view->owner->joinSlotObjects(qTab,  qd_Operands)  )
		{
			// join failed
			qstrError = QString("Error: When joining for operation '") + _name + QString("'") ;
			QcalcAppendTextDisplay( qstrError.toLatin1().data() );
			QcalcAppendTextDisplay( bufLog );
			return;
		}
	} // end over if else on joined
	if(qd_Operands.size() == 0) return;
	
	if(joinOnly) // if join or component from pulldown, just return first operand
	{
	     m_qdObjResult = qd_Operands[0];
		m_bCalcDone = true;
		return;
	}
	// Do the calculation
	OperatorFnPtr pOpRole    = qdOpDetails.GetOpRole();
	QString       qstrErrMsg = QString("");

	// Check if this is a trig operation
	if ( qdOpDetails.GetTrigFlag() )
	{
		int  nAngleUnit  = 2;  // default to hyperbolic (no units) if angle units slot hidden
		if ( m_pcalcTab )
		{
			if ( !(m_pcalcTab->AngleUnits->isHidden()) )
			{
				nAngleUnit = m_pcalcTab->AngleUnits->pulldown->currentIndex();
				if ( nAngleUnit == -1 )
				{
					nAngleUnit = 0; // default is degrees
				}
				
			}
		}
		// Construct operand B dynamically
		QdRScalar_var   qdUnit = new QdRScalar(nAngleUnit );
		if ( qdUnit.is_nil() )
		{
			// This should never happen - but ...
			qstrError = QString("Error: When performing operation '") +  _name + 
					  QString("' - Memory cannot be allocated for angle units");
			QcalcAppendTextDisplay( qstrError.toLatin1().data() );
			QcalcAppendTextDisplay( bufLog );
			return;
		}

		try
		{
			m_qdObjResult = ( *( pOpRole ) )( qstrErrMsg, qd_Operands[0],   qdUnit   );

		}
		catch( Exception &e )
		{
			qstrError = QString("Error: When performing operation '") + _name ;
			QcalcAppendTextDisplay( qstrError.toLatin1().data() );
			QcalcAppendTextDisplay ( e.msg().c_str() );
			return;
		}
		catch( ... )
		{
			qstrError = QString("Error: When performing operation '") + _name + 
					  QString("' - Unkown exception thrown");
			QcalcAppendTextDisplay( qstrError.toLatin1().data() );
			QcalcAppendTextDisplay( bufLog );
			return;
		}
	}
	else
	{
		try
		{
			if(numOps == 2)
			{
				m_qdObjResult = ( *( pOpRole ) )( qstrErrMsg, qd_Operands[0], qd_Operands[1]  );
			}
			else if (numOps == 1)
			{
				m_qdObjResult = ( *( pOpRole ) )( qstrErrMsg, qd_Operands[0],  qd_Operands[0]  ); // second operand arg is ignored
			}
			else return; // unary and binary only so far. Will migrate to using vector of operands
		}
		catch( Exception &e )
		{
			qstrError = QString("Error: When performing operation '") + _name ;
			QcalcAppendTextDisplay( qstrError.toLatin1().data() );
			QcalcAppendTextDisplay ( e.msg().c_str() );
			return;
		}
		catch( ... )
		{
			qstrError = QString("Error: When performing operation '") + _name +  QString("' - Unkown exception thrown");

			QcalcAppendTextDisplay( qstrError.toLatin1().data() );
			QcalcAppendTextDisplay( bufLog );
			return;
		}
	}
	if ( m_qdObjResult.is_nil() )
	{
		// Warning within the window status
		qstrError = QString("Error: When performing operation '") + _name + QString("' - Result is null");
		QcalcAppendTextDisplay( qstrError.toLatin1().data() );
	}

	m_bCalcDone = true;
	QcalcAppendTextDisplay( bufLog );
}

QString  QcalcItem::GetNextAliaseName()
{
	QString  qstrAlias  = QString("a");
	bool     bAliasUsed;
     int      nAliasChar = 97;
	int      nCounter = 0;
	
	// search for next unused alias starting from 'a' 
	// this allows us to reuse deleted aliases
	
	bAliasUsed = AliasAlreadyUsed( qstrAlias );
	
	while ( bAliasUsed )
	{
		if  ( nAliasChar > 122 /*90*/ )
		{
			nAliasChar = 97;
			nCounter++;
		}


		char chAlias = nAliasChar;
		if ( nCounter > 0 )
		{
			QString  qstrCounter;
			qstrAlias = QString( QChar( chAlias ) ) + qstrCounter.setNum( nCounter );
		}
		else
		{
			qstrAlias = QString( QChar( chAlias ) );
		}

		// keep incrementing until unused alias is found
		bAliasUsed = AliasAlreadyUsed( qstrAlias );
		nAliasChar++;
	}

	return ( qstrAlias );
}

bool QcalcItem::AliasAlreadyUsed( QString testAlias )
{
 	// alias name must be unique in whole calc list
	// go to list view and iterate over whole list

	if ( _parent == NULL) return false;

	QcalcItem * item = _parent->getChildByAlias(testAlias);
	
    if(item == NULL) return false;
	return true;
	
}


void QcalcItem::reNameAlias( const QString& NewVal )
{
	if  ( QuSlotDebug() )
		printf("QcalcItem::reNameAlias %ld of %s \n", (long)this, NewVal.toLatin1().data());
		
	QString qstrOldAlias = QString(this->_alias);
	QcalcTab* qTab = this->m_pcalcTab;
	
	if( !qTab || !_parent) return;
	
	this->setAlias( NewVal );

	if ( qTab )
	{
		if ( qTab->objAliase_field )
		{
			qTab->objAliase_field->setText( NewVal );
			qTab->objAliase_field->reValidate();
		}
		m_pcalcTab->calcListview->owner->tabWidget->setTabText( m_pcalcTab->calcListview->owner->tabWidget->indexOf(qTab), NewVal );
	}

	// Change all the objects depending on that alias
	
	for(QList<QcalcItem*>::iterator iter=_parent->_children.begin(); iter!=_parent->_children.end(); ++iter)
	{	
	
		if ( (*iter)->m_eObjType == VIRTUAL_OBJECT  )
		{
			qTab = (*iter)->m_pcalcTab;
			if(qTab) 
			{
				for(int nOp=0; nOp < qTab->nSlots; nOp++)
				{
					if (qTab->operandStack[nOp]->slot->textObject() == qstrOldAlias)
					{
				   	   	qTab->operandStack[nOp]->slot->setAliasKeepComp(NewVal);
					}
					qTab->SetTabFeedback();
				}
			}
		}
	}

	return ;
}
 
QdObject_var QcalcItem::getDataObject()
{
	// returns is_nil() object if not resolved
	// returns appropriate resultant object for this list item
	
	QdObject_var nil_obj;
	QdObject_var ret_obj;
	
	if ( this->m_pcalcTab == NULL ) return nil_obj;
	
	if(this->m_eObjType == WORKING_LIST_OBJECT)
	{
		if (m_qdObjJoinedFirst.is_nil())
		{
			ret_obj = (this->m_pcalcTab->operandStack[0])->slot->getSlotObject();
		}
		else
		{
			ret_obj = m_qdObjJoinedFirst;
		}
	}
	else if( this->m_eObjType == VIRTUAL_OBJECT)
	{
		ret_obj = m_qdObjResult;
	}
	
	return ret_obj;
}


bool QcalcItem::getIsVector( )
{
	if (this->m_eObjType == WORKING_LIST_OBJECT) {

		if ( this->isVector) {
			if(this->m_pcalcTab->operandStack[0]->slot->getComponent() == WL_OBJ
				|| this->m_pcalcTab->operandStack[0]->slot->getComponent() == SLICE_COMP){
			   return true;
			}
			else return false; // component selected
		}
	 	else return false; //  object on WL not a vector
	}
	else if (this->m_eObjType == VIRTUAL_OBJECT) return this->isVector;  // Virtual items don't deliver components

     else return false; 
}

bool QcalcItem::getIsArray( )
{
	bool isArray = false;
	if(!(this->virtualArrDim.is_nil()) && this->virtualArrDim->rank() > 0 ) isArray = true;

	if (this->m_eObjType == WORKING_LIST_OBJECT) {

		if ( isArray) {
			dimTripleStack arrTriple = this->m_pcalcTab->operandStack[0]->slot->parseArrayReduce();
			if(arrTriple.size() == 0)  return true;  // input object, not reduced
			else {
				for(int i=0; i<arrTriple.size(); i++){
					if(arrTriple[i][0] == 0) return true;  // elements option chosen for this dim, still an array
				}
				return false; // all dimensions sliced or summed, now a scalar
			}
		}
	 	else return false; //  object on WL not an array
	}
	else if (this->m_eObjType == VIRTUAL_OBJECT) return isArray;  // Virtual items don't deliver components

     else return false; 
}

Dim_var QcalcItem::getVirtualArrDim( )
{
	
	if (this->m_eObjType == WORKING_LIST_OBJECT) {
		Dim_var vDim;
		if ( !(this->virtualArrDim.is_nil()) && this->virtualArrDim->rank() > 0) {
			// construct dim for selection in ArrayReduce via text()

			vDim = this->m_pcalcTab->operandStack[0]->slot->getSlotArrDim();
		}
	 	if( vDim.is_nil() )return this->virtualArrDim;  // no sub-selection made from ArrayReduce
		else return vDim; 
	}
	
	else  return this->virtualArrDim;  // Virtual items don't deliver components

}

/*
Vostream& WriteCalcItem( Vostream& v_os,   QcalcItem& item )
{

	if ( item.m_eObjType == WORKING_LIST_OBJECT )
	{
       	v_os << string(item.WLcontext.toLatin1().data());
		item.m_pcalcTab->operandStack[0]->slot->with = item.with;
     	
	}
	v_os << string(item.m_qstrOperator.toLatin1().data());
	
	v_os << item.isVector;
	
	bool noArrayDim = item.virtualArrDim.is_nil();
	v_os << noArrayDim;
	if( !noArrayDim ){ 
		v_os << item.virtualArrDim;
	}
	
     v_os << item.m_pcalcTab->saveWL_check->isChecked();
	v_os << string(item.m_pcalcTab->_nameWL_field->text().toLatin1().data());
	v_os << item.m_pcalcTab->WLnameUnset;
	
	v_os << string(item.m_pcalcTab->tb4->text().toLatin1().data());
	
	v_os << string(item.m_pcalcTab->_width_field->text().toLatin1().data());
	v_os << item.m_pcalcTab->_algorithm->currentIndex();
	v_os << string(item.m_pcalcTab->_gap_field->text().toLatin1().data());
	v_os << item.m_pcalcTab->_gap->currentIndex();
	
	v_os << item.m_pcalcTab->nSlots;
	for(int i=0; i < item.m_pcalcTab->nSlots; i++)
	{
		v_os << *(item.m_pcalcTab->operandStack[i]->slot);
		v_os << false; // dummy for backward compatibility
	}
	v_os << *(item.m_pcalcTab->AngleUnits->slot);
	

   
	return ( v_os );
}
*/

QDomElement QcalcItem::toXMLElement(QDomDocument& doc) const
{
	QDomElement xml=doc.createElement("Calc_Item");

    xml.appendChild(createTextXMLNode(doc, "CalcObjType", (int) m_eObjType));
    xml.appendChild(createTextXMLNode(doc, "ItemName", _name));
    xml.appendChild(createTextXMLNode(doc, "ItemAlias", _alias));
	

	if ( m_eObjType == WORKING_LIST_OBJECT )
	{
	    xml.appendChild(createTextXMLNode(doc, "ItemWLcontext", WLcontext));     	
	}
	xml.appendChild(createTextXMLNode(doc, "ItemIsVector", isVector));     	

	int arrSize = 0;
	if(!virtualArrDim.is_nil()) arrSize = virtualArrDim->size();
	for(int i=0; i<arrSize; i++ ){ 
		xml.appendChild(createTextXMLNode(doc, "ItemArrDim", (int)(*virtualArrDim)[i]));     	
	}
	
	xml.appendChild(createTextXMLNode(doc, "ItemSaveWL", m_pcalcTab->saveWL_check->isChecked()));     	
	xml.appendChild(createTextXMLNode(doc, "ItemNameWL", m_pcalcTab->_nameWL_field->text()));     	
	xml.appendChild(createTextXMLNode(doc, "ItemNameUnset", m_pcalcTab->WLnameUnset));     	

	xml.appendChild(createTextXMLNode(doc, "ItemWidthFld", m_pcalcTab->_width_field->text()));     	
	xml.appendChild(createTextXMLNode(doc, "ItemAlgorithm", m_pcalcTab->_algorithm->currentIndex()));     	
	xml.appendChild(createTextXMLNode(doc, "ItemGapFld", m_pcalcTab->_gap_field->text()));     	
	xml.appendChild(createTextXMLNode(doc, "ItemGap", m_pcalcTab->_gap->currentIndex()));     	
			
	for(int i=0; i < m_pcalcTab->nSlots; i++)
	{
		xml.appendChild(createTextXMLNode(doc, "ItemSlot", m_pcalcTab->operandStack[i]->slot->text()));     	
	}
	xml.appendChild(createTextXMLNode(doc, "ItemAngleUnits", m_pcalcTab->AngleUnits->slot->text()));     		

	xml.appendChild(createTextXMLNode(doc, "ItemOperator", m_qstrOperator));     	
	xml.appendChild(createTextXMLNode(doc, "Itemtb4", m_pcalcTab->tb4->text()));     	


    return xml;

}

Vistream& ReadCalcItem( Vistream& v_is,   QcalcItem& item )
{

	if ( item.m_eObjType == WORKING_LIST_OBJECT )
	{
       	string name_str;
       	v_is >> name_str;
      	item.WLcontext = QString(name_str.c_str());
 	}
	
	// Operator
	string strOperator;
	v_is >> strOperator;
	item.m_qstrOperator = QString( strOperator.c_str() );

	v_is >>  item.isVector;  
	
	if(item._view->owner->qcalc_version > 1.1 ) {
		bool noArrayDim;
		v_is >> noArrayDim;
		if(!noArrayDim){
			Dim_var vArrDim = new Dim();
			v_is >> vArrDim;
			item.virtualArrDim = vArrDim;
		}
	}
	else {
		bool isArray;
		v_is >>  isArray;
		// not used, vitrualArrDim will be is_nil()
	}
	
	bool isChk;
	v_is >> isChk;
	item.m_pcalcTab->saveWL_check->setChecked(isChk);
	
	string nameWL;
	v_is >> nameWL;
	item.m_pcalcTab->_nameWL_field->setText(nameWL.c_str());
	item.m_NameForSave =  QString(nameWL.c_str());
	
	if(item._view->owner->qcalc_version > 1.0 ) v_is >> item.m_pcalcTab->WLnameUnset;
	
	string opTxt;
	v_is >> opTxt;

	item.m_pcalcTab->SetOperatorByName( opTxt.c_str() );
	string widthTxt;
	v_is >> widthTxt;
	item.m_pcalcTab->_width_field->setText(widthTxt.c_str());

	int alg;
	v_is >> alg;
	item.m_pcalcTab->_algorithm->setCurrentIndex(alg);
	 
	string gapTxt;
	v_is >> gapTxt;
	item.m_pcalcTab->_gap_field->setText(gapTxt.c_str());

	int gap;
	v_is >> gap;
	item.m_pcalcTab->_gap->setCurrentIndex(gap);
	 
	v_is >> item.m_pcalcTab->nSlots;
	for(int i=0; i < item.m_pcalcTab->nSlots; i++)
	{
		v_is >> *(item.m_pcalcTab->operandStack[i]->slot);
		bool dummy;
		v_is >> dummy;
	}
	v_is >> *(item.m_pcalcTab->AngleUnits->slot);
		
	return ( v_is );
}

/*
Vostream& operator<<( Vostream& v_os, QcalcItem& item )
{

	v_os << (int)item.m_eObjType;
	v_os << item.with;
	int isLocked = 0;
	v_os << isLocked; // dummy, no longer used
	int entryType = 0; // dummy, no longer used
	v_os << entryType;
	
	v_os << string(item._name.toLatin1().data());
	v_os << string(item._alias.toLatin1().data());
	

	WriteCalcItem( v_os, item );
	

	int cnt=0;
	v_os << cnt; // no folders in calc at present

	return v_os;
}
*/

Vistream& operator>>( Vistream& v_is, QcalcItem& item )
{   
	int obj_type;
	v_is >> obj_type;
	item.m_eObjType=(ItemTypeEnum)obj_type;
	v_is >> item.with;
	int nLock;
	v_is >> nLock; //  no longer used

     int entry_type;
     v_is >> entry_type; // no longer used

	string txt0;
	v_is >> txt0;
	item.setName(txt0.c_str());

	string txt1;
	v_is >> txt1;
	item.setAlias(txt1.c_str() );
		

	ReadCalcItem( v_is,   item );

	int cnt = 0;
	v_is >> cnt;
	for ( int i = 0; i < cnt; i++ )
	{
		QcalcItem* childItem = item._view->insertEntry("Restored");
		v_is >> *childItem;
		childItem->reNameAlias(childItem->_alias);
	}

	return v_is;

}

QcalcInput::QcalcInput( QWidget   *parent,  QString  name, QString label, QcalcTab *qTab )
	:QWidget(parent)
{
    if(qTab == 0) return;
    
    QHBoxLayout* layout=new QHBoxLayout;
#if QT_VERSION < 0x040300
    layout->setSpacing(QSAS_profiles->current()->GUImargins/2);
    layout->setMargin(QSAS_profiles->current()->GUImargins/2);
#else
    layout->setContentsMargins(0,0,0,0);
#endif
    
    inputLabel =  new QLabel( label+"  ", this);
    inputLabel->setAlignment(Qt::AlignRight);
    
    slot = new QcalcSlot( this);
    slot->setMinimumWidth( 300 );
    slot->setMaxLength( 200 );
    slot->setAcceptable(Qcondition(QS_WL_OBJ));
  //   slot->addAcceptable(Qcondition(QS_ALIAS));
   slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
    slot->aliasList = qTab->virtualItem->_view;  
    slot->setEnabled( TRUE );
    slot->setText( "" );
    
    pulldown = new QComboBox(this);
    pulldown->hide();
    this->setToolTip("Drop object from WL or calculator list, or type in value" );
    connect(  slot, SIGNAL( edited_OK() ), qTab,  SLOT( SetOperand() ) );
    connect(  slot, SIGNAL( validated_WL_OBJ( const QString & ) ), qTab,  SLOT( SetLink( const QString & ) ) );
    
    layout->addWidget(inputLabel, Qt::AlignVCenter);
    layout->addWidget(slot, Qt::AlignVCenter);
    layout->addWidget(pulldown, Qt::AlignVCenter);
    this->setLayout(layout);
    this->adjustSize();

}


void QcalcInput::setText(QString txt)
{
    slot->setText( txt );
}
 
void QcalcInput::setAnglePulldown()
{
  
  this->slot->hide();
  pulldown->insertItem(0, QString("degrees") );
  pulldown->insertItem(1, QString("radians") );
  this->pulldown->show();

}

// class  QcalcSlot
// ----------------

// drop slot overloaded from QguWLinput

QcalcSlot::QcalcSlot( QWidget *parent)
	: QguWLinput(parent, "")
{
	;
	
}


void QcalcSlot::dropEvent(QDropEvent *e)
{
	if(QuSlotDebug())
	{
		printf("QcalcSlot::dropEvent %ld \n", (long)this);
		fflush(stdout);
	}

	QcalcItem *qItem;

	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

	qApp->processEvents();

	if( aliasList != NULL)
	{
		QcalcListView *qview = dynamic_cast <QcalcListView*>(aliasList);

		if (e->mimeData()->hasText())
		{
			QString txt = e->mimeData()->text();
			// Check if this item is included within the local object list
			qItem = qview->findAlias(txt);
			if ( qItem == NULL )
			{
				// Item does not exist within the calculator list of objects
				// Check if this object is coming from the  working list
				WLItem* qWLitem=qsasWindow->wlview->wlmodel()->itemFromPath(txt);
				if ( qWLitem )
				{
					// find if it is already on calc list, and if not create it
					QcalcItem* item = qview->findName(qWLitem->getPath());
					if(item == NULL)
					{
						item = qview->insertEntry(qWLitem->getPath());
						// should now be there
					}
					
					if ( item ) txt = item->_alias;

				}
			}

			e->setAccepted( false );
			QString newText = QString( txt.toLatin1().data() );
			setTextKeepComp( newText );
		}
	}
	
	QApplication::restoreOverrideCursor();

	qApp->processEvents();
	lineBox->setFocus();
	emit hadDropEvent();
	reValidate(true); // causes edited_OK to be emitted on success
}

