#ifndef QCALC_COMMON_H
#define QCALC_COMMON_H


#include <QVector>
#include <QList>
#include <iostream>
#include <QString>

#include <QDomDocument>
#include <QGraphicsScene>

#include "qdutil.h"
#include "qwui.h"
#include "qtpl.h"


#include <QGraphicsItem>
#include <QGraphicsLineItem>
#include <QGraphicsRectItem>
#include <QPainterPath>
#include <QPainter>
#include <QWidget>

#include "qslot.h"

#define SLOT_SIZE 12.
#define HEADER_HEIGHT 20.
#define BASE_WIDTH 200.
#define BASE_HEIGHT HEADER_HEIGHT*2.
#define BASE_WIDTH_TIME 150.
#define BASE_HEIGHT_TIME HEADER_HEIGHT*5.
#define EXPANDED_SLOT_HEIGHT 40.

#define TYPE_INSLOT UserType+1
#define TYPE_OUTSLOT UserType+2
#define TYPE_INPUTITEM UserType+3
#define TYPE_OUTPUTITEM UserType+4
#define TYPE_OPERATORITEM UserType+5
#define TYPE_CONNECTOR UserType+6
#define TYPE_ROTOZOOMBUTTON UserType+7
#define TYPE_CLOSEBUTTON UserType+8
#define TYPE_EXPANDBUTTON UserType+9
#define TYPE_ERRORBUTTON UserType+10
#define TYPE_EXTRACTIONITEM UserType+11
#define TYPE_CALCTOOLTIP UserType+12
#define TYPE_TIMEINSLOT UserType+13
#define TYPE_TIMEOUTSLOT UserType+14
#define TYPE_TIMELINEITEM UserType+15
#define TYPE_TIMECONNECTOR UserType+16
#define TYPE_JOINITEM UserType+17

typedef enum QCalcOperation_e
{
 QCALC_ADD=0,
 QCALC_SUB=1,
 QCALC_MULT=2,
 QCALC_DIV=3,
 QCALC_MOD=4,
 QCALC_POWER=5,
 QCALC_OPP=6,
 QCALC_REMAINDER=7,
 QCALC_ABS=8,
 QCALC_LOG=9,
 QCALC_LOG10=10,
 QCALC_INVERSE=11,
 QCALC_SQRT=12,
 QCALC_EXP=13,
 QCALC_COS=14,
 QCALC_SIN=15,
 QCALC_TAN=16,
 QCALC_COT=17,
 QCALC_ACOS=18,
 QCALC_ASIN=19,
 QCALC_ATAN=20,
 QCALC_ATAN2=21,
 QCALC_ACOT=22,
 QCALC_COSH=23,
 QCALC_SINH=24,
 QCALC_TANH=25,
 QCALC_COTH=26,
 QCALC_ACOSH=27,
 QCALC_ASINH=28,
 QCALC_ATANH=29,
 QCALC_ACOTH=30,
 QCALC_VECTPROD=31,
 QCALC_DOTPROD=32,
 QCALC_UNITS=33,
 QCALC_FILLS=34,
 QCALC_PLUGIN=35,
 QCALC_BLACKBOX=36,
 QCALC_PIECEWISE=37,
 QCALC_PW_DIV=38,
 QCALC_TRACE=39,
 QCALC_DIFF3=40,
 QCALC_DIFF5=41,
 QCALC_ANGLE360=42,
 QCALC_ANGLE180=43,
 QCALC_TT_TO_SEC=44,
 QCALC_TT2_TT1=45,
 QCALC_T2_T1=46,
 QCALC_TIVL_TO_SEC=47,
 QCALC_LIN_REG=48,
 QCALC_INT_DEF=49,
 QCALC_INT_INDEF=50
} QCalcOperation;

class VInSlot;
class VOutSlot;
class VCalcItem;
class CalcItem;
class VOperatorItem;
class OperatorItem;
class VInputItem;
class InputItem;
class VOutputItem;
class OutputItem;
class VExtractionItem;
class ExtractionItem;
class VJoinItem;
class JoinItem;
class InSlot;
class OutSlot;
class TimeInSlot;
class TimeOutSlot;

class RotoZoomButton;
class CloseButton;
class ErrorButton;
class ExpandButton;
class Connector;

class QEvaluator;

extern QVector<VInputItem*> InputItems;

typedef struct
{
	bool noJoin;
    int algorithm;
    double width;
    int gap;
    double gapSize;
	
} JoinSettings;

#endif
