#include "qcalcEvaluator.h"


void QEvaluatorDivide::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string  attrPtr;
	string  stringBuf;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	// Test dimensionality of inputs

	if (!QuIsScalar(val2 ))
	{
		message=QString("Only divide by scalars or scalar sequences is allowed" );
		ok=false;
		return;
	}
	
	// Test validity of inputs...
	int nQarStatus=QarTestConformal(val1, val2);
	if ( nQarStatus == QAR_ERR )
	{
		message=QString("Non conformal inputs");
		ok=false;
		return;
	}
	
	// do arithmetic...
	try
	{
		result = val1 / val2;
	}
	catch( Exception &e )
	{
		message=QString(e.msg().c_str());
		message+=QString(" The operation Divide is not yet supported between these input types");
		ok=false;
		return;
	}

	if ( result.is_nil() )
	{
		message=QString("Arithmetic>> This Operation is not yet supported between these input types\n" );
		ok=false;
		return;
	}
	
	// get xrefs from inputs if requested
	result->use_xrefs_from(val1, val2);
	
	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( val1 );
	if ( qd0.is_nil() ) qd0 = get_Depend_0( val2 );
	if ( !qd0.is_nil() ) result->change_xref(DEPEND_0 , qd0 );
	
	// Set QSAS essential attriutes
	// find inverses first and set its attributes, then do for result

	// SI_conversion
	attrPtr = QarSIConv_inverse( val2 );
	QdObject_var qdInv   = val1->create(); // this is empty as we only need metadata

	if( !attrPtr.empty() )
	{
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), qdInv );
	}

	// UNITS
	attrPtr = QuGetAttrText( "UNITS", val2 );
	if( !attrPtr.empty() )
	{
		stringBuf =  "(";
		stringBuf += attrPtr;
		stringBuf += ")^-1" ;
		QuSetTxtAttr( "UNITS", stringBuf.c_str(), qdInv );
	}

	// use inverse for attributes for returned object
	// SI_conversion
	attrPtr = QarSIConv_product( val1, qdInv );
	if( !attrPtr.empty() )
	{
		QuSetTxtAttr("SI_conversion", attrPtr.c_str(), result);
	}

	// Frame (must come from first object for divide)
	attrPtr = QuGetFrameAttr( val1 );
	if( !attrPtr.empty() )
	{
		QuSetFrame( result, attrPtr.c_str() );
	}
	
	// UNITS
	attrPtr = QarUNITS_product( val1, qdInv );
	if( !attrPtr.empty() )
	{
		QuSetTxtAttr( "UNITS", attrPtr.c_str(), result );
	}

	// LABLAXIS
	QString labl_s = QuGetAttrText( "LABLAXIS", val1 ).c_str();
	if(val2->xref_exists("LABLAXIS")){
		labl_s += "/";
		labl_s += QuGetAttrText( "LABLAXIS", val2 ).c_str();
	}
	else if(val2->sequence_size() == 1){
		QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(val2);
		QdRScalar_var s = QdRScalar_var::narrow(val2);
		if(!ss.is_nil()) labl_s += QString(" / %1").arg(ss[0]);
		else if(!s.is_nil()) labl_s += QString(" / %1").arg(*s);
	}
	QuSetTxtAttr( "LABLAXIS", labl_s.toLatin1().data(), result );


/*	// DEPEND_i
	if (val1->xref_exists(DEPEND_1)) 
	{
		QdObject_var d1_xref = val1->get_xref(DEPEND_1);
		result->change_xref( DEPEND_1, d1_xref  );
	}
   
	if (val1->xref_exists(DEPEND_2)) 
	{
		QdObject_var d1_xref = val1->get_xref(DEPEND_2);
		result->schange_xref( DEPEND_2, d1_xref  );
	}

	if (val1->xref_exists(DEPEND_3)) 
	{
		QdObject_var d1_xref = val1->get_xref(DEPEND_3);
		result->change_xref( DEPEND_3, d1_xref  );
	}
*/
	
	out[0]->setValue(result, true);

}
