#include "qcalcEvaluator.h"


void QEvaluatorPow::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string  attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;

	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	bool isVector2=QuIsThreeVector(val2);
	if(isVector2)
	{
		message=QString("cannot raise to power of vector or array");
		ok=false;
		return;
	}
	
	if(QarHasUnits(val2))
	{
		// issue warning
		message=QString("WARNING! Operation Power - Exponent has units.");
		ok=false;
		return; //TODO warning rather than error
	}
	
	// do arithmetic..
	try
	{
		result = pow(val1, val2);
		if (result.is_nil())
		{
			message=QString("Arithmetic>> Power Operation is not yet supported between these input types");
			ok=false;
			return;
		}
	}
	catch( Exception & e)
	{
		message=QString("Error>> Power operation - Exception thrown..." );
		message+=QString(e.msg().c_str());
		ok=false;
		return;
	}

	// get xrefs from inputs if requested
	result->use_xrefs_from(val1);

	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( val1 );
	if ( qd0.is_nil() ) qd0 = get_Depend_0( val2 );
	if ( !qd0.is_nil() ) result->change_xref(DEPEND_0 , qd0 );

	// Set QSAS essential attributes
	QdRScalar_var qdrBPtr = QdRScalar_var::narrow(val2);
	if(!qdrBPtr.is_nil())
	{
		// can only do these if power is unique
		// SI_conversion
		// the value for constant power is a global
		double dConstB=(double)(*qdrBPtr);
		attrPtr = QarSIConv_power(val1, dConstB);
		if(!attrPtr.empty() )
		{
			QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), result );
		}

		// UNITS
		// the value for constant power is a global
		attrPtr = QarUNITS_power(val1, dConstB);
		if(!attrPtr.empty() )
		{
			QuSetTxtAttr( "UNITS", attrPtr.c_str(), result );
		}
	
		// LABLAXIS
		QString labl_s = QuGetAttrText( "LABLAXIS", val1 ).c_str();
		labl_s += QString(" ^ %1").arg(dConstB);
		QuSetTxtAttr( "LABLAXIS", labl_s.toLatin1().data(), result );
	}

	// Frame
	QuSetFrame(result, "scalar");

	out[0]->setValue(result, true);
}
