#include "qcalcView.h"
// #include "qcalcVirtualItems.h"
// #include "qcalcGUIItems.h"
#include "qcalcCalcItem.h"
#include "qcalcInputItem.h"
#include "qcalcOperatorItem.h"
#include "qcalcOutputItem.h"
#include "qcalcExtractionItem.h"
#include "qcalcJoinItem.h"
// #include "qcalcTimeLineItem.h"
#include <cmath>
#include <QActionGroup>

extern qwuiWindow* qsasWindow;


CalcView::CalcView(QGraphicsScene* scene):
	QGraphicsView(scene)
{
	RButtonMenu=new QMenu(this);
	QRectF rect=sceneRect();
	autoPosition=rect.center();
	centrePosition=autoPosition;
	positionSet=false;
	positionOK=false;
	setMouseTracking(true);

}

CalcView::~CalcView()
{
}

QList<VOutputItem*> CalcView::listSelectedOutputItems()
{
	QList<VOutputItem*> outList;
	QList<QGraphicsItem*> items=scene()->selectedItems();
	foreach(QGraphicsItem* item, items)
	{
		VOutputItem* outItem=dynamic_cast<VOutputItem*>(item);
		if(outItem) outList.push_back(outItem);
	}
	return outList;
}


void CalcView::dragEnterEvent(QDragEnterEvent* e)
{
	activateWindow();
	e->accept();

	QGraphicsView::dragEnterEvent(e);
}

void CalcView::dropEvent(QDropEvent* e)
{
	e->accept();

	QGraphicsView::dropEvent(e);
	
}

void CalcView::displayMenu(const QPoint& position)
{
	QList<QGraphicsItem*> items=scene()->selectedItems();
	bool isEmpty=items.empty();
	bool hasOperators=false;
	bool hasBlackBox=false;
	if(!isEmpty)
	{
		for(QList<QGraphicsItem*>::iterator iter=items.begin(); iter!=items.end() && !hasOperators; ++iter)
		{
			hasOperators=(qgraphicsitem_cast<OperatorItem*>(*iter)!=NULL);
		}
		if(hasOperators)
		{
			for(QList<QGraphicsItem*>::iterator iter=items.begin(); iter!=items.end() && !hasBlackBox; ++iter)
			{
				OperatorItem *temp=qgraphicsitem_cast<OperatorItem*>(*iter);
				if(temp!=NULL)
				{
					hasBlackBox=(temp->parentItem->operation==QCALC_BLACKBOX);
				}
			}
		}
	}
	
	RButtonMenu->clear();
	QMenu* addMenu=RButtonMenu->addMenu("Add");
	addMenu->addAction("Input", this, SLOT(addInputItem()));
	addMenu->addAction("Extractor", this, SLOT(addExtractionItem()));
	addMenu->addAction("Join Setup", this, SLOT(addJoinItem()));
	QMenu* addOp=addMenu->addMenu("Operator");
	QMenu* binOp=addOp->addMenu("Arith f(A,B)");
	binOp->addAction("A+B", this, SLOT(addOpAdd()));
	binOp->addAction("A-B", this, SLOT(addOpSub()));
	binOp->addAction("A*B", this, SLOT(addOpMul()));
	binOp->addAction("A/B", this, SLOT(addOpDiv()));
	binOp->addAction("mod(A)", this, SLOT(addOpMod()));
	binOp->addAction("A^B", this, SLOT(addOpPow()));
	binOp->addAction("A(i,j) * B(i,j) -> C(i,j)", this, SLOT(addOpPiecewise()));
	binOp->addAction("A(i,j) / B(i,j) -> C(i,j)", this, SLOT(addOpPwDiv()));
	QMenu* unOp=addOp->addMenu("Arith f(A)");
	unOp->addAction("A\%B", this, SLOT(addOpRemainder()));
	unOp->addAction("abs(A)", this, SLOT(addOpAbs()));
	unOp->addAction("ln(A)", this, SLOT(addOpLog()));
	unOp->addAction("log(A)", this, SLOT(addOpLog10()));
	unOp->addAction("1/A", this, SLOT(addOpInverse()));
	unOp->addAction("-A", this, SLOT(addOpOpp()));
	unOp->addAction("sqrt(A)", this, SLOT(addOpSqrt()));
	unOp->addAction("exp(A)", this, SLOT(addOpExp()));
	unOp->addAction("trace(A)", this, SLOT(addOpTrace()));
	QMenu* trigOp=addOp->addMenu("Trig");
	trigOp->addAction("cos(A)", this, SLOT(addOpCos()));
	trigOp->addAction("sin(A)", this, SLOT(addOpSin()));
	trigOp->addAction("tan(A)", this, SLOT(addOpTan()));
	trigOp->addAction("cot(A)", this, SLOT(addOpCot()));
	trigOp->addAction("acos(A)", this, SLOT(addOpAcos()));
	trigOp->addAction("asin(A)", this, SLOT(addOpAsin()));
	trigOp->addAction("atan(A)", this, SLOT(addOpAtan()));
	trigOp->addAction("atan2(A,B)", this, SLOT(addOpAtan2()));
	trigOp->addAction("acot(A)", this, SLOT(addOpAcot()));
	trigOp->addAction("cosh(A)", this, SLOT(addOpCosh()));
	trigOp->addAction("sinh(A)", this, SLOT(addOpSinh()));
	trigOp->addAction("tanh(A)", this, SLOT(addOpTanh()));
	trigOp->addAction("coth(A)", this, SLOT(addOpCoth()));
	trigOp->addAction("acosh(A)", this, SLOT(addOpAcosh()));
	trigOp->addAction("asinh(A)", this, SLOT(addOpAsinh()));
	trigOp->addAction("atanh(A)", this, SLOT(addOpAtanh()));
	trigOp->addAction("acoth(A)", this, SLOT(addOpAcoth()));
	trigOp->addAction("(-180,180) to (0,360)", this, SLOT(addOpAngle360()));
	trigOp->addAction("(0,360) to (-180,180)", this, SLOT(addOpAngle180()));
	QMenu* vectOp=addOp->addMenu("Vector Ops");
	vectOp->addAction("A.B", this, SLOT(addOpDotProd()));
	vectOp->addAction("AxB", this, SLOT(addOpVectProd()));
	QMenu* derivOp=addOp->addMenu("Derivative");
	derivOp->addAction("3pt derivative", this, SLOT(addOpDiff3()));
	derivOp->addAction("5pt derivative", this, SLOT(addOpDiff5()));
	QMenu* integOp=addOp->addMenu("Integral");
	integOp->addAction("definite integral", this, SLOT(addOpIntDef()));
	integOp->addAction("indefinite integral", this, SLOT(addOpIntIndef()));
	QMenu* modOp=addOp->addMenu("Filtering");
    addPluginMenu(modOp, "Filtering", true);
	QMenu* unitOp=addOp->addMenu("Change Units");
    addPluginMenu(unitOp, "UnitOperations", true);
	QMenu* timeOp=addOp->addMenu("Time Operations");
    addPluginMenu(timeOp, "TimeOperations", true);
	timeOp->addAction("Timetags to seconds from offset", this, SLOT(addOpTTtoS()));
	timeOp->addAction("Time Interval to seconds", this, SLOT(addOpTIVLtoS()));
	timeOp->addAction("Time2 - Time1 in sec", this, SLOT(addOpT2_T1()));
	timeOp->addAction("Timetags2 - Timetags1 in seconds", this, SLOT(addOpTT2_TT1()));
	QMenu* statsOp=addOp->addMenu("Statistics");
    addPluginMenu(statsOp, "Statistics", true);
	statsOp->addAction("Linear Regression", this, SLOT(addOpLinReg()));

	QMenu* eventsOp=addOp->addMenu("Event Tables");
    addPluginMenu(eventsOp, "EventTables", true);
    QMenu* plugOp=addOp->addMenu("Plug-ins");
    QMenu* geoOp=plugOp->addMenu("Geophysics");
    addPluginMenu(geoOp, "Geophysics", true);
    QMenu* anaOp=plugOp->addMenu("Analysis");
    addPluginMenu(anaOp, "Analysis", true);
	plugOp->addSeparator();
	plugOp->addAction("Browse...", this, SLOT(addPlugInBrowse()));
	
	addMenu->addAction("Output", this, SLOT(addOutputItem()));
	QMenu* bbOp=addOp->addMenu("Black Box from library");
	addBlackBoxMenu(bbOp);
    connect(bbOp, SIGNAL(triggered(QAction*)), this, SLOT(addBB(QAction*)));
	QMenu* templateOp=addMenu->addMenu("Template");
	templateOp->addAction("A+B", this, SLOT(addTemplateAdd()));
	templateOp->addAction("A-B", this, SLOT(addTemplateSub()));
	templateOp->addAction("A*B", this, SLOT(addTemplateMul()));
	templateOp->addAction("A/B", this, SLOT(addTemplateDiv()));
	templateOp->addAction("-A", this, SLOT(addTemplateOpp()));
	

	
	if(!isEmpty)
	{
		QMenu* aspectMenu=RButtonMenu->addMenu("Aspect");
		aspectMenu->addAction("Reset aspect", this, SLOT(resetAspect()));
		aspectMenu->addSeparator();
		aspectMenu->addAction("Horizontal", this, SLOT(horizontalAspect()));
		aspectMenu->addAction("Vertical", this, SLOT(verticalAspect()));
		aspectMenu->addSeparator();
		aspectMenu->addAction("Normal Size", this, SLOT(normalSize()));
		aspectMenu->addAction("Half size", this, SLOT(halfSize()));
		aspectMenu->addAction("Double size", this, SLOT(doubleSize()));
	}
	if(hasOperators)
	{
		QMenu* opMenu=RButtonMenu->addMenu("Change operator(s) for...");
		QMenu* binOp2=opMenu->addMenu("Arith f(A,B)");
		binOp2->addAction("A+B", this, SLOT(changeOpAdd()));
		binOp2->addAction("A-B", this, SLOT(changeOpSub()));
		binOp2->addAction("A*B", this, SLOT(changeOpMul()));
		binOp2->addAction("A/B", this, SLOT(changeOpDiv()));
		binOp2->addAction("mod(A)", this, SLOT(changeOpMod()));
		binOp2->addAction("A^B", this, SLOT(changeOpPow()));
		binOp2->addAction("A(i,j) * B(i,j) -> C(i,j)", this, SLOT(changeOpPiecewise()));
		binOp2->addAction("A(i,j) / B(i,j) -> C(i,j)", this, SLOT(changeOpPwDiv()));
		QMenu* unOp2=opMenu->addMenu("Arith f(A)");
		unOp2->addAction("A\%B", this, SLOT(changeOpRemainder()));
		unOp2->addAction("abs(A)", this, SLOT(changeOpAbs()));
		unOp2->addAction("ln(A)", this, SLOT(changeOpLog()));
		unOp2->addAction("log(A)", this, SLOT(changeOpLog10()));
		unOp2->addAction("1/A", this, SLOT(changeOpInverse()));
		unOp2->addAction("-A", this, SLOT(changeOpOpp()));
		unOp2->addAction("sqrt(A)", this, SLOT(changeOpSqrt()));
		unOp2->addAction("exp(A)", this, SLOT(changeOpExp()));
		unOp2->addAction("trace(A)", this, SLOT(changeOpTrace()));
		QMenu* trigOp2=opMenu->addMenu("Trig");
		trigOp2->addAction("cos(A)", this, SLOT(changeOpCos()));
		trigOp2->addAction("sin(A)", this, SLOT(changeOpSin()));
		trigOp2->addAction("tan(A)", this, SLOT(changeOpTan()));
		trigOp2->addAction("cot(A)", this, SLOT(changeOpCot()));
		trigOp2->addAction("acos(A)", this, SLOT(changeOpAcos()));
		trigOp2->addAction("asin(A)", this, SLOT(changeOpAsin()));
		trigOp2->addAction("atan(A)", this, SLOT(changeOpAtan()));
		trigOp2->addAction("atan2(A,B)", this, SLOT(changeOpAtan2()));
		trigOp2->addAction("acot(A)", this, SLOT(changeOpAcot()));
		trigOp2->addAction("cosh(A)", this, SLOT(changeOpCosh()));
		trigOp2->addAction("sinh(A)", this, SLOT(changeOpSinh()));
		trigOp2->addAction("tanh(A)", this, SLOT(changeOpTanh()));
		trigOp2->addAction("coth(A)", this, SLOT(changeOpCoth()));
		trigOp2->addAction("acosh(A)", this, SLOT(changeOpAcosh()));
		trigOp2->addAction("asinh(A)", this, SLOT(changeOpAsinh()));
		trigOp2->addAction("atanh(A)", this, SLOT(changeOpAtanh()));
		trigOp2->addAction("acoth(A)", this, SLOT(changeOpAcoth()));
		trigOp2->addAction("(-180,180) to (0,360)", this, SLOT(changeOpAngle360()));
		trigOp2->addAction("(0,360) to (-180,180)", this, SLOT(changeOpAngle180()));
		QMenu* vectOp2=opMenu->addMenu("Vector Ops");
		vectOp2->addAction("A.B", this, SLOT(changeOpDotProd()));
		vectOp2->addAction("AxB", this, SLOT(changeOpVectProd()));
		QMenu* derivOp2=opMenu->addMenu("Derivative");
		derivOp2->addAction("3pt derivative", this, SLOT(changeOpDiff3()));
		derivOp2->addAction("5pt derivative", this, SLOT(changeOpDiff5()));
		QMenu* integOp2=opMenu->addMenu("Integral");
		integOp2->addAction("definite integral", this, SLOT(changeOpIntDef()));
		integOp2->addAction("indefinite integral", this, SLOT(changeOpIntIndef()));
		QMenu* modOp2=opMenu->addMenu("Filtering");
    	addPluginMenu(modOp2, "Filtering", false);
		QMenu* unitOp2=opMenu->addMenu("Change Units");
    	addPluginMenu(unitOp2, "UnitOperations", false);
		QMenu* timeOp2=opMenu->addMenu("Time Operations");
    	addPluginMenu(timeOp2, "TimeOperations", false);
		timeOp2->addAction("Timetags to seconds from offset", this, SLOT(changeOpTTtoS()));
		timeOp2->addAction("Time Interval to seconds", this, SLOT(changeOpTIVLtoS()));
		timeOp2->addAction("Time2 - Time1 in sec", this, SLOT(changeOpT2_T1()));
		timeOp2->addAction("Timetags2 - Timetags1 in seconds", this, SLOT(changeOpTT2_TT1()));
		QMenu* statsOp2=opMenu->addMenu("Statistics");
    	addPluginMenu(statsOp2, "Statistics", false);
		statsOp2->addAction("Linear Regression", this, SLOT(changeOpLinReg()));
		QMenu* eventsOp2=opMenu->addMenu("Event Tables");
    	addPluginMenu(eventsOp2, "EventTables", false);
        QMenu* plugOp2=opMenu->addMenu("Plug-ins");
        QMenu* geoOp2=plugOp2->addMenu("Geophysics");
        addPluginMenu(geoOp2, "Geophysics", false);
        QMenu* anaOp2=plugOp2->addMenu("Analysis");
        addPluginMenu(anaOp2, "Analysis", false);
		plugOp2->addSeparator();
		plugOp2->addAction("Browse...", this, SLOT(changePlugInBrowse()));
	}
	if(!isEmpty)
	{
		RButtonMenu->addAction("Set as black box", this, SLOT(changeForBlackBox()));
	}
	if(hasBlackBox)
	{
		RButtonMenu->addAction("Unwrap black box", this, SLOT(unwrapBlackBox()));
		RButtonMenu->addAction("Save black box to library", this, SLOT(saveBlackBoxToLibrary()));
	}
	

	RButtonMenu->exec(position);
}

void CalcView::addBlackBoxMenu(QMenu* menu)
{

	const char * home = getenv("QSAS_HOME");
	QString menuPathQSAS = QString(home);
	menuPathQSAS += "/black_boxes";

	QDir dQS(menuPathQSAS);
    if(dQS.exists()){
	
	 	for(int i=0; i<dQS.count(); i++){
        	QString name = QString(dQS[i].toLatin1().data());
        	int posn = name.lastIndexOf(".qx_calc");
        	if(posn <0 ) continue;
        	name.truncate(posn);
        	QAction* action=menu->addAction(name);
        	QVariant data=menuPathQSAS+"/"+name;
        	action->setData(data);
		}
    }
	
	QString menuPath=QSAS_profiles->current()->BlackBoxPath;
	
	if(menuPath == menuPathQSAS) return;
	
	QDir d(menuPath);

    if(!d.exists()) return;
	
	 for(int i=0; i<d.count(); i++){
        QString name = QString(d[i].toLatin1().data());
        int posn = name.lastIndexOf(".qx_calc");
        if(posn <0 ) continue;
        name.truncate(posn);
        QAction* action=menu->addAction(name);
        QVariant data=menuPath+"/"+name;
        action->setData(data);
    }
}

void CalcView::addPluginMenu(QMenu* menu, const char* folder, bool isAdd)
{
    QString menuPath=QString(qsasWindow->qsas_home)+"/qtpl/"+folder;
    QDir d(menuPath);


    if(!d.exists())
        return;

	
    for(int i=0; i<d.count(); i++){
        QString name = QString(d[i].toLatin1().data());
        int posn = name.lastIndexOf(".qtpl");
        if(posn <0 ) continue;
        name.truncate(posn);
        QAction* action=menu->addAction(name);
        QVariant data=menuPath+"/"+name;
        action->setData(data);
		action->setStatusTip("plugin");
    }
  
  
  //Same thing for subfolders
    QStringList subFolders=d.entryList(QDir::Dirs/* | QDir::NoDotAndDotDot QT4 only*/);
    for(int i=0; i<subFolders.count(); i++)
    {
        if(!subFolders[i].startsWith("."))
        {
            QDir d2(QString(menuPath+"/"+subFolders[i]));
            QStringList filters;
			filters.push_back(QString("*.qtpl"));
            QStringList subFolders2=d2.entryList(filters, QDir::Files);
        
            if(subFolders2.count()>0)
            {
                qtplMenu * temp=new qtplMenu(menu);
                temp->menuPath=menuPath+"/"+subFolders[i];
                QMenu * tempMenu=menu->addMenu(subFolders[i]);
            
                for(int j=0; j<subFolders2.count(); j++)
                {
                    QString name = QString(subFolders2[j].toLatin1().data());
                    int posn = name.lastIndexOf(".qtpl");
                    if(posn <0 ) continue;
                    name.truncate(posn);
                    QAction* action=tempMenu->addAction(name);
                    QVariant data=temp->menuPath+"/"+name;
                    action->setData(data);
					action->setStatusTip("plugin");
                }
            }
        }
    }

    if(isAdd)
    {
        connect(menu, SIGNAL(triggered(QAction*)), this, SLOT(addPlugin(QAction*)));
    }
    else
    {
        connect(menu, SIGNAL(triggered(QAction*)), this, SLOT(changePlugin(QAction*)));
    }
}

void CalcView::autoSelectPosition()
{
// 	position=autoPosition;
// 	if(renewPosition)
// 	{
// 		autoPosition=autoPosition+QPointF(20,20);
// 		if(autoPosition.x()>centrePosition.x()+200)
// 		{
// 			centrePosition=centrePosition+QPointF(0., 20.);
// 			autoPosition=centrePosition;
// 		}
// 		renewPosition=false;
// 	}
positionSet=false;
positionOK=false;
}

void CalcView::wheelEvent(QWheelEvent* event)
{
	if(event->modifiers()==Qt::ControlModifier)
	{
		int numDegrees = event->delta() / 8;
		int numSteps = numDegrees / 15;
		scale(1.+(numSteps)/10., 1.+(numSteps)/10.);
	}
	else
		QGraphicsView::wheelEvent(event);
}

void CalcView::mousePressEvent(QMouseEvent* mouseEvent)
{
	if(mouseEvent->button()==Qt::RightButton)
	{
		if(((CalcScene*)scene())->tracking)
		{
			QGraphicsView::mousePressEvent(mouseEvent);
			return;
		}
		
		QPointF scenePoint=mapToScene(mouseEvent->pos());
		QGraphicsItem* gitem=scene()->itemAt(scenePoint, QTransform());
		if(gitem!=NULL)
		{
			if(!gitem->isSelected())
			{
				scene()->clearSelection();
				gitem->setSelected(true);
			}
		}
		position=scenePoint;
		positionSet=true;
		positionOK=true;
		displayMenu(mouseEvent->globalPos());
	}
	QGraphicsView::mousePressEvent(mouseEvent);
}

void CalcView::horizontalAspect()
{
	QList<QGraphicsItem*> items=scene()->selectedItems();
	foreach(QGraphicsItem* item, items)
	{
		OperatorItem* opItem=qgraphicsitem_cast<OperatorItem*>(item);
		InputItem* inItem=qgraphicsitem_cast<InputItem*>(item);
		OutputItem* outItem=qgraphicsitem_cast<OutputItem*>(item);
		ExtractionItem* extItem=qgraphicsitem_cast<ExtractionItem*>(item);
// 		TimeLineItem* timeItem=qgraphicsitem_cast<TimeLineItem*>(item);
		JoinItem* joinItem=qgraphicsitem_cast<JoinItem*>(item);
		if(opItem!=NULL || inItem!=NULL || outItem!=NULL || extItem!=NULL || /*timeItem!=NULL ||*/ joinItem!=NULL)
		{
			QMatrix m=item->matrix();
			double scale=sqrt(m.determinant());
			m.reset();
			m.scale(scale, scale);
			item->setMatrix(m);
			((CalcItem*)item)->updateConnectorsPositions();
		}
	}
}

void CalcView::verticalAspect() // TODO do nothing if operator expanded. Same thing for every aspect modifier
{
	QList<QGraphicsItem*> items=scene()->selectedItems();
	foreach(QGraphicsItem* item, items)
	{
		OperatorItem* opItem=qgraphicsitem_cast<OperatorItem*>(item);
		InputItem* inItem=qgraphicsitem_cast<InputItem*>(item);
		OutputItem* outItem=qgraphicsitem_cast<OutputItem*>(item);
		ExtractionItem* extItem=qgraphicsitem_cast<ExtractionItem*>(item);
// 		TimeLineItem* timeItem=qgraphicsitem_cast<TimeLineItem*>(item);
		JoinItem* joinItem=qgraphicsitem_cast<JoinItem*>(item);
		if(opItem!=NULL || inItem!=NULL || outItem!=NULL || extItem!=NULL || /*timeItem!=NULL ||*/ joinItem!=NULL)
		{
			QMatrix m=item->matrix();
			double scale=sqrt(m.determinant());
			m.reset();
			m.rotate(-90.);
			m.scale(scale, scale);
			item->setMatrix(m);
			((CalcItem*)item)->updateConnectorsPositions();
		}
	}
}

void CalcView::normalSize()
{
	QList<QGraphicsItem*> items=scene()->selectedItems();
	foreach(QGraphicsItem* item, items)
	{
		OperatorItem* opItem=qgraphicsitem_cast<OperatorItem*>(item);
		InputItem* inItem=qgraphicsitem_cast<InputItem*>(item);
		OutputItem* outItem=qgraphicsitem_cast<OutputItem*>(item);
		ExtractionItem* extItem=qgraphicsitem_cast<ExtractionItem*>(item);
// 		TimeLineItem* timeItem=qgraphicsitem_cast<TimeLineItem*>(item);
		JoinItem* joinItem=qgraphicsitem_cast<JoinItem*>(item);
		if(opItem!=NULL || inItem!=NULL || outItem!=NULL || extItem!=NULL || /*timeItem!=NULL || */joinItem!=NULL)
		{
			QMatrix m=item->matrix();
			double scale=sqrt(m.determinant());
			QMatrix m2;
			m2.reset();
			m2.scale(1./scale, 1./scale);
			item->setMatrix(m*m2);
			((CalcItem*)item)->updateConnectorsPositions();
		}
	}
}

void CalcView::halfSize()
{
	QList<QGraphicsItem*> items=scene()->selectedItems();
	foreach(QGraphicsItem* item, items)
	{
		OperatorItem* opItem=qgraphicsitem_cast<OperatorItem*>(item);
		InputItem* inItem=qgraphicsitem_cast<InputItem*>(item);
		OutputItem* outItem=qgraphicsitem_cast<OutputItem*>(item);
		ExtractionItem* extItem=qgraphicsitem_cast<ExtractionItem*>(item);
// 		TimeLineItem* timeItem=qgraphicsitem_cast<TimeLineItem*>(item);
		JoinItem* joinItem=qgraphicsitem_cast<JoinItem*>(item);
		if(opItem!=NULL || inItem!=NULL || outItem!=NULL || extItem!=NULL || /*timeItem!=NULL ||*/ joinItem!=NULL)
		{
			QMatrix m=item->matrix();
			double scale=sqrt(m.determinant())*2.;
			QMatrix m2;
			m2.reset();
			m2.scale(1./scale, 1./scale);
			item->setMatrix(m*m2);
			((CalcItem*)item)->updateConnectorsPositions();
		}
	}
}

void CalcView::doubleSize()
{
	QList<QGraphicsItem*> items=scene()->selectedItems();
	foreach(QGraphicsItem* item, items)
	{
		OperatorItem* opItem=qgraphicsitem_cast<OperatorItem*>(item);
		InputItem* inItem=qgraphicsitem_cast<InputItem*>(item);
		OutputItem* outItem=qgraphicsitem_cast<OutputItem*>(item);
		ExtractionItem* extItem=qgraphicsitem_cast<ExtractionItem*>(item);
// 		TimeLineItem* timeItem=qgraphicsitem_cast<TimeLineItem*>(item);
		JoinItem* joinItem=qgraphicsitem_cast<JoinItem*>(item);
		if(opItem!=NULL || inItem!=NULL || outItem!=NULL || extItem!=NULL || /*timeItem!=NULL ||*/ joinItem!=NULL)
		{
			QMatrix m=item->matrix();
			double scale=sqrt(m.determinant())/2.;
			QMatrix m2;
			m2.reset();
			m2.scale(1./scale, 1./scale);
			item->setMatrix(m*m2);
			((CalcItem*)item)->updateConnectorsPositions();
		}
	}
}

void CalcView::resetAspect()
{
	QList<QGraphicsItem*> items=scene()->selectedItems();
	foreach(QGraphicsItem* item, items)
	{
		OperatorItem* opItem=qgraphicsitem_cast<OperatorItem*>(item);
		InputItem* inItem=qgraphicsitem_cast<InputItem*>(item);
		OutputItem* outItem=qgraphicsitem_cast<OutputItem*>(item);
		ExtractionItem* extItem=qgraphicsitem_cast<ExtractionItem*>(item);
// 		TimeLineItem* timeItem=qgraphicsitem_cast<TimeLineItem*>(item);
		JoinItem* joinItem=qgraphicsitem_cast<JoinItem*>(item);
		if(opItem!=NULL || inItem!=NULL || outItem!=NULL || extItem!=NULL || /*timeItem!=NULL ||*/ joinItem!=NULL)
		{
			QMatrix m;
			m.reset();
			item->setMatrix(m);
			((CalcItem*)item)->updateConnectorsPositions();
		}
	}
}

void CalcView::addInputItem()
{

	if(!positionSet) positionCrosshair();
	
	if(!positionOK) return; // Cancelled
	VInputItem* item=new VInputItem;
	InputItem* item_gui=item->getGUI();
	scene()->addItem(item_gui);
	connect(item_gui, SIGNAL(newInput(const QString&)), ((CalcScene*)scene()), SIGNAL(newInput(const QString&)));
	item_gui->setPos(position);
// 	renewPosition=true;
	positionSet=false;
}

void CalcView::addJoinItem()
{
	if(!positionSet) positionCrosshair();
	if(!positionOK) return; // Cancelled
	VJoinItem* item=new VJoinItem;
	JoinItem* item_gui=item->getGUI();
	scene()->addItem(item_gui);
	item_gui->setPos(position);
	positionSet=false;
}

void CalcView::addOutputItem()
{
	if(!positionSet) positionCrosshair();
	if(!positionOK) return; // Cancelled
	VOutputItem* item=new VOutputItem;
	OutputItem* item_gui=item->getGUI();
	scene()->addItem(item_gui);
	item_gui->setPos(position);
// 	renewPosition=true;
	positionSet=false;
}

void CalcView::addExtractionItem()
{
	if(!positionSet) positionCrosshair();
	if(!positionOK) return; // Cancelled
	VExtractionItem* item=new VExtractionItem;
	ExtractionItem* item_gui=item->getGUI();
	scene()->addItem(item_gui);
	item_gui->setPos(position);
	connect(item_gui->edit, SIGNAL(componentSelected(const char *, const char * )), this,   SIGNAL(componentSelected(const char *, const char * )) ); 

// 	renewPosition=true;
	positionSet=false;
}

void CalcView::addOp(const QCalcOperation& op)
{
	if(!positionSet) positionCrosshair();
	if(!positionOK) return; // Cancelled
	VOperatorItem* item=new VOperatorItem(op);
	OperatorItem* item_gui=item->getGUI();
	scene()->addItem(item_gui);
	item_gui->setPos(position);
// 	renewPosition=true;
	positionSet=false;
}

void CalcView::changeOp(const QCalcOperation& op)
{
	QList<QGraphicsItem*> items=scene()->selectedItems();
	foreach(QGraphicsItem* item, items)
	{
		OperatorItem* opItem=qgraphicsitem_cast<OperatorItem*>(item);
		if(opItem!=NULL)
		{
			opItem->setOperation(op);
		}
	}
}

void CalcView::addBB(QAction* a)
{
// 	std::cout << "addBB" << std::endl;
// 	if(!positionSet) positionCrosshair();
// 	if(!positionOK) return; // Cancelled
	((CalcScene*)scene())->open(a->data().toString()+".qx_calc", false);
}

void CalcView::addPlugin(QAction* a)
{
	if(a->statusTip() != QString("plugin")) return; // allow non-plugins to be in same menu as plugins
	
	if(!positionSet) positionCrosshair();
	if(!positionOK) return; // Cancelled
    VOperatorItem* item=new VOperatorItem(QCALC_PLUGIN, a->data().toString());
    OperatorItem* item_gui=item->getGUI();
    scene()->addItem(item_gui);
    item_gui->setPos(position);
	positionSet=false;
}

void CalcView::changePlugin(QAction* a)
{
	if(a->statusTip() != QString("plugin")) return; // allow non-plugins to be in same menu as plugins

    QList<QGraphicsItem*> items=scene()->selectedItems();
    foreach(QGraphicsItem* item, items)
    {
        OperatorItem* opItem=qgraphicsitem_cast<OperatorItem*>(item);
        if(opItem!=NULL)
        {
            opItem->setOperation(QCALC_PLUGIN, a->data().toString());
        }
    }
    ((CalcScene*)scene())->emitSelectionChanged(); // To update the context view
}



bool itemsSortCriterion(QGraphicsItem* i1, QGraphicsItem* i2)
{
	QPointF p1=i1->scenePos();
	QPointF p2=i2->scenePos();
	return(p1.x()<p2.x() || (p1.x()==p2.x() && p1.y()<p2.y()));
}

void CalcView::changeForBlackBox()
{

	QList<QGraphicsItem*> items=scene()->selectedItems();
	qSort(items.begin(), items.end(), itemsSortCriterion);
	QList<VCalcItem*> blackBox;
	foreach(QGraphicsItem* item, items)
	{
		OperatorItem* opItem=qgraphicsitem_cast<OperatorItem*>(item);
		OutputItem* outItem=qgraphicsitem_cast<OutputItem*>(item);
		InputItem* inItem=qgraphicsitem_cast<InputItem*>(item);
		ExtractionItem* extItem=qgraphicsitem_cast<ExtractionItem*>(item);
		JoinItem* joinItem=qgraphicsitem_cast<JoinItem*>(item);
		if(opItem!=NULL)
		{
			blackBox.push_back(opItem->parentItem);
			opItem->parentItem->positionInBlackBox=opItem->pos()-position;
		}
		if(outItem!=NULL)
		{
			blackBox.push_back(outItem->parentItem);
			outItem->parentItem->positionInBlackBox=outItem->pos()-position;
		}
		if(inItem!=NULL)
		{
			blackBox.push_back(inItem->parentItem);
			inItem->parentItem->positionInBlackBox=inItem->pos()-position;
		}
		if(extItem!=NULL)
		{
			blackBox.push_back(extItem->parentItem);
			extItem->parentItem->positionInBlackBox=extItem->pos()-position;
		}
		if(joinItem!=NULL)
		{
			blackBox.push_back(joinItem->parentItem);
			joinItem->parentItem->positionInBlackBox=joinItem->pos()-position;
		}
	}
	
	VOperatorItem* item=new VOperatorItem;
	item->setAsBlackBox(blackBox);
	OperatorItem* item_gui=item->getGUI();
	scene()->addItem(item_gui);
	item_gui->setPos(position);
	item_gui->resetConnectors();
}

void CalcView::unwrapBlackBox()
{
	QList<QGraphicsItem*> items=scene()->selectedItems();
	QVector<InSlot*> savedConnectionsIn;
	QVector<OutSlot*> savedConnectionsOut;
	foreach(QGraphicsItem* item, items)
	{
		OperatorItem* opItem=qgraphicsitem_cast<OperatorItem*>(item);
		if(opItem!=NULL)
		{
			if(opItem->unwrap())
			{
				VOperatorItem* vopItem=opItem->parentItem;
				foreach(VInSlot* inSlot, vopItem->inputs)
				{
					VOutSlot* outSlot=inSlot->connectedSlot();
					if(outSlot!=NULL)
					{
						savedConnectionsIn.push_back(inSlot->getGUI());
						savedConnectionsOut.push_back(outSlot->getGUI());
					}
				}
				foreach(VOutSlot* outSlot, vopItem->outputs)
				{
					outSlot->openSlot = false;
					foreach(VInSlot* inSlot, outSlot->connectedSlots())
					{
						if(outSlot!=NULL)
						{
							savedConnectionsIn.push_back(inSlot->getGUI());
							savedConnectionsOut.push_back(outSlot->getGUI());
						}
					}
				}
				vopItem->removeGUI(); // This removes some connections
				delete vopItem;
			}
		}
	}
	((CalcScene*)scene())->addMissingConnectors();
	
	for(int i=0; i<savedConnectionsIn.size(); ++i)
	{
		if(savedConnectionsOut[i] == NULL )continue;
		Connector* connector=new Connector(savedConnectionsOut[i], savedConnectionsIn[i]);
		scene()->addItem(connector);
		savedConnectionsOut[i]->updateConnectorsPositions();
		savedConnectionsOut[i]->slot->connectTo(savedConnectionsIn[i]->slot);
	}
}

void CalcView::addUnaryTemplate(const QCalcOperation& op)
{
	if(!positionSet) positionCrosshair();
	if(!positionOK) return; // Cancelled
	VInputItem* vi=new VInputItem;
	InputItem* i=vi->getGUI();
	VOperatorItem* vo=new VOperatorItem(op);
	OperatorItem* o=vo->getGUI();
	VOutputItem* vou=new VOutputItem;
	OutputItem* ou=vou->getGUI();
	scene()->addItem(i);
	connect(i, SIGNAL(newInput(const QString&)), ((CalcScene*)scene()), SIGNAL(newInput(const QString&)));
	scene()->addItem(o);
	scene()->addItem(ou);
	ou->setOutputName(QString("Result_%1").arg(vou->output_id));
	
// 	QRectF rect=sceneRect();
// 	QPointF p=rect.center();
// 	autoSelectPosition();
	i->setPos(position-QPointF(0,100));
	o->setPos(position);
	ou->setPos(position+QPointF(0,100));
	
	VOutSlot* outSlot1=i->parentItem->outputs[0];
	VInSlot* inSlot1=o->parentItem->inputs[0];
	Connector* connector1=new Connector(outSlot1->getGUI(), inSlot1->getGUI());
	scene()->addItem(connector1);
	outSlot1->getGUI()->updateConnectorsPositions();
	outSlot1->connectTo(inSlot1);
	
	VOutSlot* outSlot2=o->parentItem->outputs[0];
	VInSlot* inSlot2=ou->parentItem->inputs[0];
	Connector* connector2=new Connector(outSlot2->getGUI(), inSlot2->getGUI());
	scene()->addItem(connector2);
	outSlot2->getGUI()->updateConnectorsPositions();
	outSlot2->connectTo(inSlot2);
	
	i->setSelected(true);
	o->setSelected(true);
	ou->setSelected(true);
}

void CalcView::addBinaryTemplate(const QCalcOperation& op)
{
	if(!positionSet) positionCrosshair();
	if(!positionOK) return; // Cancelled
	
	VInputItem* vi1=new VInputItem;
	InputItem* i1=vi1->getGUI();
	VInputItem* vi2=new VInputItem;
	InputItem* i2=vi2->getGUI();
	VOperatorItem* vo=new VOperatorItem(op);
	OperatorItem* o=vo->getGUI();
	VOutputItem* vou=new VOutputItem;
	OutputItem* ou=vou->getGUI();
	scene()->addItem(i1);
	scene()->addItem(i2);
	connect(i1, SIGNAL(newInput(const QString&)), ((CalcScene*)scene()), SIGNAL(newInput(const QString&)));
	connect(i2, SIGNAL(newInput(const QString&)), ((CalcScene*)scene()), SIGNAL(newInput(const QString&)));
	scene()->addItem(o);
	scene()->addItem(ou);
	ou->setOutputName(QString("Result_%1").arg(vou->output_id));
	
	i1->setPos(position-QPointF(110,100));
	i2->setPos(position-QPointF(-110,100));
	o->setPos(position);
	ou->setPos(position+QPointF(0,100));
	
	VOutSlot* outSlot1=i1->parentItem->outputs[0];
	VOutSlot* outSlot1b=i2->parentItem->outputs[0];
	VInSlot* inSlot1=o->parentItem->inputs[0];
	VInSlot* inSlot1b=o->parentItem->inputs[1];
	Connector* connector1=new Connector(outSlot1->getGUI(), inSlot1->getGUI());
	Connector* connector1b=new Connector(outSlot1b->getGUI(), inSlot1b->getGUI());
	scene()->addItem(connector1);
	scene()->addItem(connector1b);
	outSlot1->connectTo(inSlot1);
	outSlot1b->connectTo(inSlot1b);
	outSlot1->getGUI()->updateConnectorsPositions();
	outSlot1b->getGUI()->updateConnectorsPositions();
	
	VOutSlot* outSlot2=o->parentItem->outputs[0];
	VInSlot* inSlot2=ou->parentItem->inputs[0];
	Connector* connector2=new Connector(outSlot2->getGUI(), inSlot2->getGUI());
	scene()->addItem(connector2);
	outSlot2->getGUI()->updateConnectorsPositions();
	outSlot2->connectTo(inSlot2);
	
	i1->setSelected(true);
	i2->setSelected(true);
	o->setSelected(true);
	ou->setSelected(true);
}

void CalcView::addPlugInBrowse()
{
	QString pluginName = QFileDialog::getOpenFileName(this, "Select qtpl template", ".", "*.qtpl");
	if(!pluginName.isEmpty())
	{
		if(!positionSet) positionCrosshair();
		if(!positionOK) return; // Cancelled
		VOperatorItem* item=new VOperatorItem(QCALC_PLUGIN, pluginName);
		OperatorItem* item_gui=item->getGUI();
		scene()->addItem(item_gui);
		item_gui->setPos(position);
	}
}

void CalcView::changePlugInBrowse()
{
	QString pluginName = QFileDialog::getOpenFileName(this, "Select qtpl template", ".", "*.qtpl");
	if(!pluginName.isEmpty())
	{
		QList<QGraphicsItem*> items=scene()->selectedItems();
		foreach(QGraphicsItem* item, items)
		{
			OperatorItem* opItem=qgraphicsitem_cast<OperatorItem*>(item);
			if(opItem!=NULL)
			{
				opItem->setOperation(QCALC_PLUGIN,pluginName);
			}
		}
		((CalcScene*)scene())->emitSelectionChanged(); // To update the context view
	}
}

void CalcView::positionCrosshair()
{
	this->activateWindow();
	scene()->setFocus(Qt::OtherFocusReason);

	position=((CalcScene*)scene())->interactivePosition();

	positionOK=((CalcScene*)scene())->trackingSuccess;

}

void CalcView::saveBlackBoxToLibrary()
{
	QList<QGraphicsItem*> items=scene()->selectedItems();
	OperatorItem* bb=NULL;
	for(QList<QGraphicsItem*>::iterator iter=items.begin(); iter!=items.end() && bb==NULL; ++iter)
	{
		OperatorItem *temp=qgraphicsitem_cast<OperatorItem*>(*iter);
		if(temp!=NULL)
		{
			if(temp->parentItem->operation==QCALC_BLACKBOX)
			{
				bb=temp;
			}
		}
	}

	if(bb!=NULL)
	{
		QString name=QSAS_profiles->current()->BlackBoxPath+"/"+bb->parentItem->getName()+".qx_calc";
		QString path=QFileDialog::getSaveFileName(this, "Export as...", name, "QSAS calculator save (*.qx_calc)");
		if(!path.isEmpty())
		{
			// Save selection
			QList<QGraphicsItem*> backup=items;
			// select only this object
			foreach(QGraphicsItem* i, backup)
			{
				i->setSelected(i==bb);
			}
			
			QApplication::processEvents();
			
			((CalcScene*)scene())->exportAs(path);
			QApplication::processEvents();
			//restore selection
			foreach(QGraphicsItem* i, backup)
			{
				i->setSelected(true);
			}
			emit updateBlackBoxMenu();
			
		}
		// save black box
	}
}
