#include "qjui.h"
#include "qbu.h"

QjuiWidgetItem::QjuiWidgetItem(const QString & text, QListWidget * parent):
	QListWidgetItem(text, parent)
{
	if(parent!=NULL)
	{
		owner=((QjuiWidget*)parent)->owner;
	}
	WLcontext=text;
	setFlags(((QListWidgetItem*)this)->flags()|Qt::ItemIsEditable);
	setChoices(-1);
	toBeDeleted=false;
}

QjuiWidgetItem::QjuiWidgetItem(const QjuiWidgetItem& wi)
{
	owner=wi.owner;
	setText(wi.text());
	WLcontext=wi.WLcontext;
	setFlags(((QListWidgetItem*)this)->flags()|Qt::ItemIsEditable);
	I_width_field=wi.I_width_field;
	I_gap_field=wi.I_gap_field;
	I_algorithm=wi.I_algorithm;
	I_gap=wi.I_gap;
}

QjuiWidgetItem::~QjuiWidgetItem()
{
}

void QjuiWidgetItem::setChoices(int w)
{
	// if w = -1 update all selections
   // otherwise only update the one changed to avoid interlaced signals

	if(w == -1 || w == 0) {
		if(owner->_algorithm->currentIndex() == 2)
			this->I_algorithm =  owner->_algorithm_cpy;
		else
			this->I_algorithm =   owner->_algorithm->currentIndex();
	}

	if(w == -1 ) {
		if(owner->_width_field->text().isEmpty())
			this->I_width_field = owner->_width_field_cpy;
		else
			this->I_width_field = owner->_width_field->text();
	}
	if(w == 1 ) {
		this->I_width_field = owner->_width_field->text();
	}

	if(w == -1 || w == 2){ // Gap Handling
   
		if(owner == QJUI_widget){ // watch for stripping fills on WL
     
			if( owner->_gap->currentIndex() == 2){
         // issue warning as FILLVAL gap fill is selected
				owner->status("Uncheck Strip-Fill-Values toggle on main window",10000);
			}
			else if( owner->_gap->currentIndex() == 1){
         // issue warning if fillval is also zero
				QdObject_var dobj = QuGetObjByName(text()); //TODO
				if(!dobj.is_nil() && dobj->xref_exists("FILLVAL")) {
					QdRScalar_var xref = QdRScalar_var::narrow(dobj->get_xref("FILLVAL"));
					if( !xref.is_nil() && *xref == 0.)
						owner->status("WARNING Fill Value is also zero for this obj, uncheck Strip-Fill-Values on WL",10000);
				}
			}
			else{
				owner->status(" ",10000);
			}
		}
     
		if(owner->_gap->currentIndex() == 5)
			this->I_gap = owner->_gap_cpy;
		else
			this->I_gap = owner->_gap->currentIndex();
       
	}
   
	if(w == -1 ) {
		if(owner->_gap_field->text().isEmpty())
			this->I_gap_field = owner->_gap_field_cpy;
		else
			this->I_gap_field = owner->_gap_field->text();
	}
   
	if(w == 3 ) {
		this->I_gap_field =   owner->_gap_field->text();
	}
}





///////////////////// LIST

QjuiWidget::QjuiWidget(QWidget* parent, QjuiWindow* owner):
		QListWidget(parent)
{
	this->owner=owner;
	setDragEnabled(true);
	viewport()->setAcceptDrops(true);
	setDragDropMode(QAbstractItemView::InternalMove);
	setMovement(QListView::Free);
	
	setSelectionMode(QAbstractItemView::ExtendedSelection);
// 	QItemSelectionModel * select=selectionModel();
	connect(this, SIGNAL(itemSelectionChanged()), this, SLOT(updateOnSelection()));
	connect(this, SIGNAL(itemSelectionChanged()), this, SLOT(update1OnSelection()));
	connect(this, SIGNAL(itemSelectionChanged()), this, SLOT(update2OnSelection()));
	connect(this, SIGNAL(itemSelectionChanged()), this, SLOT(update3OnSelection()));
	connect(this, SIGNAL(itemSelectionChanged()), this, SLOT(update4OnSelection()));

	connect(this, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(openEditor(QListWidgetItem*)));

	setEditTriggers(QAbstractItemView::NoEditTriggers);

	RButtonMenu=new QMenu(this);

	RButtonMenu->addAction("Copy Settings <-", owner, SLOT(keepChoices()));
	RButtonMenu->addAction("Apply Settings ->", owner, SLOT(applyChoices()));
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Re-Name", this, SLOT(renameSelectedItem()));
// 	RButtonMenu->insertItem("&Edit WL Links", this, SLOT( editWLLink() ), Qt::CTRL+Qt::Key_E ); //TODO
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Copy", this, SLOT(copy()));
   	RButtonMenu->addAction("Paste", this, SLOT(paste()));
	RButtonMenu->addAction("select All", this, SLOT(selectAll()));
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Move Down", this, SLOT(moveItemDown()));
	RButtonMenu->addAction("Move Up", this, SLOT(moveItemUp()));
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Delete", this, SLOT(deleteListItems()));
	
	
}

QjuiWidget::~QjuiWidget()
{}

void QjuiWidget::displayMenu(const QPoint& p)
{
	RButtonMenu->exec(p);
}

void QjuiWidget::mousePressEvent(QMouseEvent* event)
{
	QListWidget::mousePressEvent(event);
	if(event->button()==Qt::RightButton)
	{
		displayMenu(event->globalPos());
	}
}


void QjuiWidget::dragEnterEvent(QDragEnterEvent *event)
{
	if(QuSlotDebug()) printf("QjuiWidget::dragEnterEvent %ld\n", (long)this);
	event->acceptProposedAction();
}

QStringList QjuiWidget::mimeTypes() const
{
	QStringList res;//=((QListWidget*)this)->mimeTypes();
	res.append("application/qjuiitems");
	return res;
}

QMimeData * QjuiWidget::mimeData(const QList<QListWidgetItem*> items) const
{
	QMimeData * data=new QMimeData;

	QStringList sl;
	foreach(QListWidgetItem * item, items)
	{
		QjuiWidgetItem* item_ok=dynamic_cast<QjuiWidgetItem*>(item);
		if(item_ok!=NULL)
		{
			QString descriptor=QString("%1//%2//%3//%4//%5//%6//%7").arg(item_ok->text()).arg(item_ok->WLcontext).arg(item_ok->I_width_field).arg(item_ok->I_gap_field).arg(item_ok->I_algorithm).arg(item_ok->I_gap).arg(row(item));
			sl.append(descriptor);
		}
	}
	QString dataString=sl.join("///");
	data->setData(QString("application/qjuiitems"), dataString.toLatin1());
	return data;
}

void QjuiWidget::dropEvent(QDropEvent* event)
{
	const QMimeData * data=event->mimeData();

	if(data->hasFormat("application/wlitems"))
	{
		QStringList itemList=QString(data->data("application/wlitems")).split("//");
		foreach(QString itemString, itemList)
		{
			if(!QuGetObjByName(itemString).is_nil())
			{
				new QjuiWidgetItem(itemString, this);
			}
		}
	}
	else
	{
		if(data->hasFormat("application/qjuiitems"))
		{
			QModelIndex index=indexAt(event->pos());
			int row=index.row();
			if(dropIndicatorPosition()==QAbstractItemView::BelowItem)
			{
				++row;
			}
			else if(dropIndicatorPosition()==QAbstractItemView::OnViewport)
			{
				row=count();
			}
			QStringList itemList=QString(data->data("application/qjuiitems")).split("///");
			foreach(QString itemString, itemList)
			{
				QStringList sl=itemString.split("//");
				if(sl.size()==7)
				{
					QjuiWidgetItem* newItem=(QjuiWidgetItem*)(item(sl[6].toInt()));
					if(newItem->text()==sl[0] && newItem->WLcontext==sl[1])
					{
						newItem=(QjuiWidgetItem*)takeItem(sl[6].toInt());
						insertItem(row, newItem);
					}
				}
			}
		}
	}
}

void QjuiWidget::setAlgorithm()
{
	if(QuSlotDebug())
	{
		printf("QjuiWidget::setAlgorithm %ld\n", (long)this);
		fflush(stdout);
	}
 
	setItemChoices(0);
}

void QjuiWidget::setWidthField()
{
	if(QuSlotDebug())
	{
		printf("QjuiWudget::setWidthField %ld\n", (long)this);
		fflush(stdout);
	}
	
	setItemChoices(1);
}

void QjuiWidget::setGap()
{
	if(QuSlotDebug())
	{
		printf("QjuiWidget::setGap %ld\n", (long)this);
		fflush(stdout);
	}
 
	setItemChoices(2);
}

void QjuiWidget::setGapField()
{
	if(QuSlotDebug())
	{
		printf("QjuiWidget::setGapField %ld\n", (long)this);
		fflush(stdout);
	}
 
	setItemChoices(3);
}

void QjuiWidget::updateOnSelection()
{
	if(QuSlotDebug())
	{
		printf("QjuiWidget::updateOnSelection %ld\n", (long)this);
		fflush(stdout);
	}
 
	
	QModelIndexList selected=selectedIndexes();
	if(selected.isEmpty()) return;
 
	QString WLcontxt_0 = QString("");

	foreach(QModelIndex index, selected)
	{
// 		QjuiWidgetItem* i=(QjuiWidgetItem*)(item(index.row()));
		QListWidgetItem* it=item(index.row());
		QjuiWidgetItem* i=dynamic_cast<QjuiWidgetItem*>(it);
		if(i!=NULL)
		{
			if (WLcontxt_0 == "") WLcontxt_0 = i->WLcontext;
			if (i->WLcontext != WLcontxt_0) return;
		}
	}
	
// 	for (int i=0; i<localList.size(); i++){
// 		item = localList[i];
// 		if (item == 0 || (QjuiListView *)item->listView() != this) return;
// 		if (item->getEntryType() != ListTypeEntry) continue;
// 		if (WLcontxt_0 == "") WLcontxt_0 = QString(item->WLcontext);
// 		if (item->WLcontext != WLcontxt_0) return;
// 	}
 
// 	owner->_DataSource->setEnabled(true);
// 	owner->_DataSource->setText(WLcontxt_0);
}

void QjuiWidget::update1OnSelection()
{
	if(QuSlotDebug())
	{
		printf("QjuiWidget::update1OnSelection %ld\n", (long)this);
		fflush(stdout);
	}
 
 
	if(owner->_algorithm->count() == 2)
		owner->_algorithm->insertItem(2," ");
	owner->_algorithm->setCurrentIndex(2);

	QModelIndexList selected=selectedIndexes();
	if(selected.isEmpty()) return;
 
	int I_alg_0 = -1;

	foreach(QModelIndex index, selected)
	{
		QjuiWidgetItem* i=(QjuiWidgetItem*)(item(index.row()));
		if(i!=NULL)
		{
			if (I_alg_0 == -1) I_alg_0 = i->I_algorithm;
			if (i->I_algorithm != I_alg_0) return;
		}
	}

 
	owner->_algorithm->setCurrentIndex(I_alg_0);
 
	if(owner->_algorithm->count() > 2)
		owner->_algorithm->removeItem(2);
       
}

void QjuiWidget::update2OnSelection()
{
	if(QuSlotDebug())
	{
		printf("QjuiWidget::update2OnSelection %ld\n", (long)this);
		fflush(stdout);
	}
 
// 	QjuiListItem * item;

	if(owner->_gap->count() == 5)
		owner->_gap->insertItem(5," ");
	owner->_gap->setCurrentIndex(5);
	
	QModelIndexList selected=selectedIndexes();
	
	if(selected.isEmpty()) return;
 
	int I_gap_0 = -1;

	foreach(QModelIndex index, selected)
	{
		QjuiWidgetItem* i=(QjuiWidgetItem*)(item(index.row()));
		if(i!=NULL)
		{
			if (I_gap_0 == -1) I_gap_0 = i->I_gap;
			if (i->I_gap != I_gap_0) return;
		}
	}
	
// 	for (int i=0; i<localList.size(); i++){
// 		item = dynamic_cast<QjuiListItem*> (localList[i]);
// 		if (item == 0 || (QjuiListView *)item->listView() != this) return;
// 		if (item->getEntryType() != ListTypeEntry) continue;
// 		if (I_gap_0 == -1) I_gap_0 = item->I_gap;
// 		if (item->I_gap != I_gap_0) return;
// 	}
	owner->_gap->setCurrentIndex(I_gap_0);
	if(owner->_gap->count() > 5)
		owner->_gap->removeItem(5);
}

void QjuiWidget::update3OnSelection()
{
	if(QuSlotDebug())
	{
		printf("QjuiWidget::update3OnSelection %ld\n", (long)this);
		fflush(stdout);
	}
 
// 	QjuiListItem * item;
 
	owner->_width_field->setText("");
	
	QModelIndexList selected=selectedIndexes();
	if(selected.isEmpty()) return;
 
	QString I_width_field_0 = QString("");

	foreach(QModelIndex index, selected)
	{
		QjuiWidgetItem* i=(QjuiWidgetItem*)(item(index.row()));
		if(i!=NULL)
		{
			if (I_width_field_0 == "") I_width_field_0 = QString(i->I_width_field);
			if (i->I_width_field != I_width_field_0) return;
		}
	}
// 	for (int i=0; i<localList.size(); i++){
// 		item = dynamic_cast<QjuiListItem*> (localList[i]);
// 		if (item == 0 || (QjuiListView *)item->listView() != this) return;
// 		if (item->getEntryType() != ListTypeEntry) continue;
// 		if (I_width_field_0 == "") I_width_field_0 = QString(item->I_width_field);
// 		if (item->I_width_field != I_width_field_0) return;
// 	}
	owner->_width_field->setText(I_width_field_0);
}

void QjuiWidget::update4OnSelection()
{
	if(QuSlotDebug())
	{
		printf("QjuiListView::update4OnSelection %ld\n", (long)this);
		fflush(stdout);
	}
 
// 	QjuiListItem * item;
 
	owner->_gap_field->setText("");
	
	QModelIndexList selected=selectedIndexes();
	if(selected.isEmpty()) return;
 
	QString I_gap_field_0 = QString("");

	foreach(QModelIndex index, selected)
	{
		QjuiWidgetItem* i=(QjuiWidgetItem*)(item(index.row()));
		if(i!=NULL)
		{
			if(I_gap_field_0 == "") I_gap_field_0 = QString(i->I_gap_field);
			if(i->I_gap_field != I_gap_field_0) return;
		}
	}
	
// 	for (int i=0; i<localList.size(); i++){
// 		item = dynamic_cast<QjuiListItem*> (localList[i]);
// 		if (item == 0 || (QjuiListView *)item->listView() != this) return;
// 		if (item->getEntryType() != ListTypeEntry) continue;
// 		if (I_gap_field_0 == "") I_gap_field_0 = QString(item->I_gap_field);
// 		if (item->I_gap_field != I_gap_field_0) return;
// 	}
	this->owner->_gap_field->setText(I_gap_field_0);
}

void QjuiWidget::setItemChoices(int w)
{
    
	if(count()==0)
	{
		owner->status("Drag items to join list", 10000);
		return;
	}

	static bool dontShow = false;
	QuMsgBar(" ", false);

	QModelIndexList selected=selectedIndexes();
	
    // if nothing is selected in this list
	if(selected.isEmpty() /*||
		   ((Q_LIST_VIEW_ITEM *)localList[0])->listView() != this*/) // I don't understand the second part...
	{
		if(dontShow)
		{
			QuMsgBar("No items selected", true);
			return;
		}
		else
		{
			switch( QMessageBox::information( this, "Settings",
				"No items selected", "Apply to All", "Don't Show Again", "Cancel") )
			{
				case 0:
					owner->keepChoices();
					selectAll();
					owner->applyChoices();
					return;
				case 1:
					dontShow = true;
					return;
				case 2:
					return;
			}
		}
	}

	// something selected so just apply new choice
	foreach(QModelIndex index, selected)
	{
		QjuiWidgetItem* i=(QjuiWidgetItem*)(item(index.row()));
		if(i!=NULL)
		{
			i->setChoices(w);
		}
	}

    
}

void QjuiWidget::openEditor(QListWidgetItem* item)
{
	QdObject_var dobj = QuGetObjByName(item->text());
	QdObject_var nil_obj;
	if(!dobj.is_nil())
	{
		QStringList decompName=item->text().split("/");
		QbuLaunchBrowse(nil_obj, 0, decompName[decompName.size()-1].toLatin1().data(), dobj, item->text());
	}
}

void QjuiWidget::deleteListItems()
{
	QModelIndexList selected=selectedIndexes();
	QLinkedList<QjuiWidgetItem*> list;
	foreach(QModelIndex index, selected)
	{
		QjuiWidgetItem * it=(QjuiWidgetItem*)(item(index.row()));
		if(it!=NULL)
		{
			list.append(it);
		}
	}
	foreach(QjuiWidgetItem* item, list)
	{
		delete item;
	}
}

void QjuiWidget::renameSelectedItem()
{
	QjuiWidgetItem* item=(QjuiWidgetItem*)currentItem();

	if(item!=NULL)
	{
		editItem(item);
	}
}

void QjuiWidget::copy()
{
	QClipboard *clipboard = QApplication::clipboard();
	QMimeData * data=new QMimeData;

	QStringList sl;
	QModelIndexList selected=selectedIndexes();
	foreach(QModelIndex index, selected)
	{
		QjuiWidgetItem * it=(QjuiWidgetItem*)(item(index.row()));
		if(it!=NULL)
		{
			QString descriptor=QString("%1//%2//%3//%4//%5//%6//%7").arg(it->text()).arg(it->WLcontext).arg(it->I_width_field).arg(it->I_gap_field).arg(it->I_algorithm).arg(it->I_gap).arg(index.row());
			sl.append(descriptor);
		}
	}
	QString dataString=sl.join("///");
	data->setData(QString("application/qjuiitems"), dataString.toLatin1());
	clipboard->setMimeData(data);
}

void QjuiWidget::paste()
{
	QClipboard *clipboard = QApplication::clipboard();
	const QMimeData * data=clipboard->mimeData();

	if(data->hasFormat("application/wlitems"))
	{
		QStringList itemList=QString(data->data("application/wlitems")).split("//");
		foreach(QString itemString, itemList)
		{
			if(!QuGetObjByName(itemString).is_nil())
			{
				new QjuiWidgetItem(itemString, this);
			}
		}
	}
	else if(data->hasFormat("application/qjuiitems"))
	{
		QStringList itemList=QString(data->data("application/qjuiitems")).split("///");
		foreach(QString itemString, itemList)
		{
			QStringList sl=itemString.split("//");
			if(sl.size()==7)
			{
				QjuiWidgetItem* newItem=new QjuiWidgetItem(sl[1], this);
				newItem->setText(sl[0]);
				newItem->I_width_field=sl[2];
				newItem->I_gap_field=sl[3];
				newItem->I_algorithm=sl[4].toInt();
				newItem->I_gap=sl[5].toInt();
			}
		}
	}
}

void QjuiWidget::moveItemUp()
{
	int row=currentRow();
	
	if(row>0)
	{
		QjuiWidgetItem* item=(QjuiWidgetItem*)takeItem(row);
		insertItem(row-1, item);
		setCurrentRow(row-1);
	}
}

void QjuiWidget::moveItemDown()
{
	int row=currentRow();
	
	if(row<count()-1)
	{
		QjuiWidgetItem* item=(QjuiWidgetItem*)takeItem(row);
		insertItem(row+1, item);
		setCurrentRow(row+1);
	}
}
