// -------------Derivative Window class ---------------------------------


#include "qpsd.h"
#include "qrtn.h"

extern qwuiWindow* qsasWindow;

int Facet3D::currentStatus=1;
int Facet4D::currentStatus=1;
int Point4D::next_p_ID=1; // used to identify each point

PSD_Window::PSD_Window() 
	: QMainWindow() 
{

	if(QuSlotDebug())
	{
		printf("PSD_Window::PSD_Window \n");
		fflush(stdout);
	}
  	this->setWindowTitle("PSD tool");
	
  	running = false;
	
  	QMenu * help = new QMenu( this );
	this->menuBar()->addMenu(help );
	help->addAction("Particle Tools", this, SLOT( launchHelp() ));
	help->setTitle(" Help ");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
//	setAttribute(Qt::WA_DeleteOnClose);
     
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QGridLayout * slotLayout=new QGridLayout;
	slotPanel->setTitle("Phase Space Distribution");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_ANY, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*80);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	slotLayout->addWidget(objField,  0, 0, 1, 4, Qt::AlignLeft);
	
	parityLabel = new QLabel("PSD coordinates in:");
	slotLayout->addWidget( parityLabel, 1, 0, Qt::AlignRight );
	 
	Vparity = new QComboBox(slotPanel);
	Vparity->addItem("Look Direction");
	Vparity->addItem("Flow Direction" );
	slotLayout->addWidget( Vparity, 1, 1, Qt::AlignLeft );
	
	QLabel *pad1 = new QLabel("   ");
	slotLayout->addWidget( pad1, 1, 2, Qt::AlignRight );
	QLabel *pad2 = new QLabel("   ");
	slotLayout->addWidget( pad2, 1, 3, Qt::AlignRight );

	massChoiceLabl = new QLabel("Species:");
	slotLayout->addWidget( massChoiceLabl, 2, 0, Qt::AlignRight );
	
	massChoice = new QComboBox(slotPanel);
	massChoice->addItem("Electron");
	massChoice->addItem("Proton");
	massChoice->addItem("He+");
	massChoice->addItem("He++");
	massChoice->addItem("O+");
	massChoice->addItem("Set Manually");
	
	slotLayout->addWidget( massChoice, 2, 1, Qt::AlignLeft );
	connect( massChoice,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setMassChoice(const QString &) ) );
	massChoice->setToolTip("Species or mass number)");
	
	massLabl = new QLabel("Atomic Mass Number");
	slotLayout->addWidget( massLabl, 2, 4, Qt::AlignLeft );
	massField = new FloatSpinBox(0.0, 50., 7, 1., NULL, "massField");
	massField->setToolTip("Atomic mass number ( p+n, 0 for electrons)");
	slotLayout->addWidget( massField, 2, 2, 1, 2, Qt::AlignRight );

	chargeLabl = new QLabel("Particle Charge");
	slotLayout->addWidget( chargeLabl, 1, 4, Qt::AlignLeft );
	chargeField = new QSpinBox();
	chargeField->setMinimum(0);
	chargeField->setMaximum(9);
	chargeField->setValue(1);
	chargeField->setToolTip("Charge number of ion");
	slotLayout->addWidget( chargeField, 1, 2, 1, 2, Qt::AlignRight );

	massChoice->setCurrentIndex(5);


	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// Reference Frame transformation panel
	
	QGroupBox * gse_panel=new QGroupBox(mainWidget);
	QGridLayout *gse_layout = new QGridLayout;
	gse_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	gse_layout->setMargin(QSAS_profiles->current()->GUImargins);	
	gse_panel->setTitle("Conversion from Instrument to Science coordinates");
	
	transLab = new QLabel("Source: ");
	gse_layout->addWidget( transLab, 0, 0, Qt::AlignRight );

	transMethod = new QComboBox(gse_panel);
	transMethod->addItem("Cluster PEACE ( SR2 )");
	transMethod->addItem("Cluster CIS ( ISR2 )" );
	transMethod->addItem("Other (use rotation matrix)" );
	gse_layout->addWidget( transMethod, 0, 1, Qt::AlignLeft );

	QLabel * targetLab = new QLabel("To:");
	gse_layout->addWidget( targetLab, 0, 2, Qt::AlignRight );
 	transTarget = new QComboBox(gse_panel);
	transTarget->addItem("GSE");
	transTarget->addItem("GSM" );
	transTarget->addItem("From Rotn Matrix" );
	transTarget->addItem("Do Not Rotate" );
	gse_layout->addWidget( transTarget, 0, 3, Qt::AlignLeft );

	methodLabl = new QLabel("  Caution, using PSD in supplied frame");
	gse_layout->addWidget( methodLabl, 0, 0, Qt::AlignLeft );
	methodLabl->hide();

	
	transMethod->adjustSize();
	int comboWidth = transMethod->width()-4;
	
	connect( transMethod,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setMethod(const QString &) ) );
	connect( transTarget,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setTarget(const QString &) ) );
	
	latLabl = new QLabel("SC spin axis Latitude: ");
	latField = new QguWLinput(gse_panel,"latField");
	latField->setToolTip(" in either GSE or GSM ");

	latField->setMinimumWidth(comboWidth);
	gse_layout->addWidget( latField, 1, 1, Qt::AlignLeft );
	gse_layout->addWidget( latLabl, 1, 0, Qt::AlignRight );
	latField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_ANY, QS_ANY));

	longLabl = new QLabel("SC spin axis Longitude: ");
	longField = new QguWLinput(gse_panel,"longField");
	longField->setToolTip(" in either GSE or GSM ");
	gse_layout->addWidget( longField, 2, 1, Qt::AlignLeft );
	gse_layout->addWidget( longLabl, 2, 0, Qt::AlignRight );
	longField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_ANY, QS_ANY));
	longField->setMinimumWidth(comboWidth);

	rotnLabl = new QLabel("Rotation Matrix: ");
	rotnField = new QguWLinput(gse_panel,"rotnField");
	gse_layout->addWidget( rotnField, 3, 1, Qt::AlignLeft );
	gse_layout->addWidget( rotnLabl, 3, 0, Qt::AlignRight );
	rotnField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_2D, QS_ANY, QS_ANY));
	rotnField->setMinimumWidth(comboWidth);
	rotnLabl->hide();
	rotnField->hide();
	
	gse_panel->setLayout(gse_layout);
	mainLayout->addWidget(gse_panel);

	// process panel
	
	QGroupBox * process_panel=new QGroupBox(mainWidget);
	QGridLayout * process_layout=new QGridLayout;
	process_panel->setTitle("Process PSD");
	
	processMethod = new QComboBox(process_panel);
	processMethod->addItem("Moments of PSD");
	processMethod->addItem("Slice PSD");
	processMethod->addItem("Save Rotation Matrix");
	process_layout->addWidget( processMethod, 0, 0, 1, 2, Qt::AlignLeft );
	connect( processMethod,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setProcess(const QString &) ) );
	
	// Slice Inputs
	
	centreChoiceLabl = new QLabel("Slice Centre:");
	process_layout->addWidget( centreChoiceLabl, 1, 0, Qt::AlignRight );
	centreChoiceLabl->hide();
	
	centreChoice = new QComboBox(process_panel);
	centreChoice->addItem("Use PSD Centre");
	centreChoice->addItem("Centre on Bulk Velocity");
	process_layout->addWidget( centreChoice, 1, 1, Qt::AlignLeft );
	connect( centreChoice,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setCentreChoice(const QString &) ) );
	centreChoice->hide();
	
	centreLabl = new QLabel("Typical Flow Velocity: ");
	process_layout->addWidget( centreLabl, 1, 4, Qt::AlignLeft );
	centreLabl->hide();
	centreField = new QguWLinput(process_panel,"centreField");
	centreField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_NOT_SEQ, QS_ANY));
	process_layout->addWidget( centreField, 1, 2, 1, 2, Qt::AlignLeft );
	centreField->setEnabled(false);
	centreField->hide();

	dirnChoiceLabl = new QLabel("Slice Normal:");
	process_layout->addWidget( dirnChoiceLabl, 2, 0, Qt::AlignRight );
	dirnChoiceLabl->hide();

	dirnChoice = new QComboBox(process_panel);
	dirnChoice->addItem("Z axis ");
	dirnChoice->addItem("Y axis ");
	dirnChoice->addItem("X axis ");
	dirnChoice->addItem("Along vector");
	process_layout->addWidget( dirnChoice, 2, 1, Qt::AlignLeft );
	connect( dirnChoice,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setDirnChoice(const QString &) ) );
	dirnChoice->hide();

	dirnLabl = new QLabel("Slice Normal vector");
	process_layout->addWidget( dirnLabl, 2, 4, Qt::AlignLeft );
	dirnLabl->hide();
	dirnField = new QguWLinput(process_panel,"dirnField");
	dirnField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_NOT_SEQ, QS_ANY));
	dirnField->setToolTip("Slice Normal Direction: (eg B field, normalisation not necessary) ");
	process_layout->addWidget( dirnField, 2, 2, 1, 2, Qt::AlignLeft );
	dirnField->setEnabled(false);
	dirnField->hide();
	
	
	binsLabl = new QLabel("Velocity bins: ");
	process_layout->addWidget( binsLabl, 3, 0, Qt::AlignRight );
	binsLabl->hide();
	
	numberBins = new QSpinBox();
	numberBins->setMinimum(2);
	numberBins->setMaximum(100);
	numberBins->setValue(50);
	numberBins->setToolTip("Number of bins on velocity axes");
	process_layout->addWidget( numberBins, 3, 1, Qt::AlignLeft );
	numberBins->hide();

	// Moments inputs
	
	potentialChoiceLabl = new QLabel("Spacecraft Potential:");
	process_layout->addWidget( potentialChoiceLabl, 1, 0, Qt::AlignRight );
	
	potentialChoice = new QComboBox(process_panel);
	potentialChoice->addItem("No Correction");
	potentialChoice->addItem("value in V");
	potentialChoice->addItem("object w/ SI_conversion");
	process_layout->addWidget( potentialChoice, 1, 1, Qt::AlignLeft );
	connect( potentialChoice,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setPotentialChoice(const QString &) ) );
	potentialChoice->setToolTip("Spacecraft potential for energy correction");
	
	potentialLabl = new QLabel("S/C Potential ");
	process_layout->addWidget( potentialLabl, 1, 4, Qt::AlignLeft );
	potentialField = new QguWLinput(process_panel,"potentialField");
	potentialField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	process_layout->addWidget( potentialField, 1, 2, 1, 2, Qt::AlignLeft );
	potentialField->setEnabled(false);

	thresholdChoiceLabl = new QLabel("Energy Threshold:");
	process_layout->addWidget( thresholdChoiceLabl, 2, 0, Qt::AlignRight );
	
	thresholdChoice = new QComboBox(process_panel);
	thresholdChoice->addItem("No Threshold");
	thresholdChoice->addItem("value in eV");
	thresholdChoice->addItem("object w/ SI_conversion");
	process_layout->addWidget( thresholdChoice, 2, 1, Qt::AlignLeft );
	connect( thresholdChoice,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setThresholdChoice(const QString &) ) );
	thresholdChoice->setToolTip("Threshold under which energies are not taken into account");
	
	thresholdLabl = new QLabel("");
	process_layout->addWidget( thresholdLabl, 2, 4, Qt::AlignLeft );
	thresholdField = new QguWLinput(process_panel,"thresholdField");
	thresholdField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	process_layout->addWidget( thresholdField, 2, 2, 1, 2, Qt::AlignLeft );
	thresholdField->setEnabled(false);

	integrationLabl = new QLabel("Integration Method:");
	process_layout->addWidget( integrationLabl, 3, 0, Qt::AlignRight );
	integrationMethod = new QComboBox(process_panel);
	integrationMethod->addItem("Sum over data bins");
	integrationMethod->addItem("Phase-space tetrahedra");
	process_layout->addWidget( integrationMethod, 3, 1, Qt::AlignLeft );

	
	dBtn = new QCheckBox("Density:", this);
	dBtn->setChecked(true);
	process_layout->addWidget( dBtn, 4, 0, Qt::AlignLeft );

	vBtn = new QCheckBox("Velocity:", this);
	process_layout->addWidget( vBtn, 4, 1, Qt::AlignLeft );

	pBtn = new QCheckBox("Pressure (&Temp):", this);
	process_layout->addWidget( pBtn, 4, 2, Qt::AlignLeft );

	hBtn = new QCheckBox("Heat Flux:", this);
	process_layout->addWidget( hBtn, 4, 3, Qt::AlignLeft );
	
	allBtn = new QCheckBox("All:", this);
	process_layout->addWidget( allBtn, 4, 4, Qt::AlignLeft );
	allBtn->setChecked(false);
	connect( allBtn,  SIGNAL( stateChanged(int) ), this, SLOT( setMomentsAll(int) ) );

	
	int rowHeight = processMethod->sizeHint().height();
	process_layout->setRowMinimumHeight(0, rowHeight);
	process_layout->setRowMinimumHeight(1, rowHeight);
	process_layout->setRowMinimumHeight(2, rowHeight);
	process_layout->setRowMinimumHeight(3, rowHeight);
	process_layout->setRowMinimumHeight(4, rowHeight);

	process_panel->setLayout(process_layout);
	mainLayout->addWidget(process_panel);

// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Name stem for save ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*45);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Evaluate");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(evaluate()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);
	saveBtn->setDefault(true);

	percentage = new QLabel("    ");
	buttons_layout->addWidget(percentage);
	
	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());

}

PSD_Window::~PSD_Window() 
{
}
// -------------


void PSD_Window::setEvalBtn(QString txt) 
{
	saveBtn->setText(txt);
}
// -------------

void PSD_Window::launchHelp() 
{
  	qsasWindow->help_window->manEntry("QSAS_analysis.html","#particles");
}
// -------------

void PSD_Window::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}

// -------------

void PSD_Window::setMethod(const QString & text){
    
	if( text.contains("PEACE") ) {
		latField->show();
		longField->show();
		rotnField->hide();
		latLabl->show();
		longLabl->show();
		rotnLabl->hide();
		methodLabl->hide();
	}
	else if( text.contains("CIS") ) {
		latField->show();
		longField->show();
		rotnField->hide();
		latLabl->show();
		longLabl->show();
		rotnLabl->hide();
		methodLabl->hide();

	}
	else {
		latField->hide();
		longField->hide();
		rotnField->show();
		latLabl->hide();
		longLabl->hide();
		rotnLabl->show();		
		methodLabl->hide();
	}
}

// -------------

void PSD_Window::setTarget(const QString & text){
    
	if( text.contains("GSE") ) {
		transLab->show();
		transMethod->show();
		methodLabl->hide();
		setMethod(transMethod->currentText());
	}
	else if( text.contains("GSM") ) {
		transLab->show();
		transMethod->show();
		methodLabl->hide();
		setMethod(transMethod->currentText());
	}
	else if( text.contains("Rotn") ) {
		transLab->show();
		transMethod->show();
		methodLabl->hide();
		transMethod->setCurrentIndex(2);
		rotnField->show();
		rotnLabl->show();		
	}
	else  {
		transMethod->hide();
		transLab->hide();
		methodLabl->show();
		latField->hide();
		longField->hide();
		rotnField->hide();
		latLabl->hide();
		longLabl->hide();
		rotnLabl->hide();
	}
	
}

// -------------

void PSD_Window::setProcess(const QString & text){
    
	if( text.contains("Moments") ) {
		centreField->hide();
		centreChoice->hide();
		centreChoiceLabl->hide();
		centreLabl->hide();
		dirnField->hide();
		dirnChoice->hide();
		dirnChoiceLabl->hide();
		dirnLabl->hide();
		binsLabl->hide();
		numberBins->hide();	
		potentialField->show();
		potentialChoice->show();
		potentialChoiceLabl->show();
		potentialLabl->show();		
		thresholdField->show();
		thresholdChoice->show();
		thresholdChoiceLabl->show();
		thresholdLabl->show();	
		integrationMethod->show();
		integrationLabl->show();
		massField->show();
		chargeField->show();
		massChoice->show();		
		massChoiceLabl->show();
		massLabl->show();	
		chargeLabl->show();	
		objField->show();
		parityLabel->show();
		Vparity->show();
		allBtn->show();
		dBtn->show();
		vBtn->show();
		pBtn->show();
		hBtn->show();
		if(!userSet) setObjectName();
	}
	else if( text.contains("Slice") ) {
		centreField->show();
		centreChoice->show();
		centreChoiceLabl->show();
		centreLabl->show();
		dirnField->show();
		dirnChoice->show();
		dirnChoiceLabl->show();
		dirnLabl->show();
		binsLabl->show();
		numberBins->show();	
		potentialField->hide();
		potentialChoice->hide();
		potentialChoiceLabl->hide();
		potentialLabl->hide();			
		thresholdField->hide();
		thresholdChoice->hide();
		thresholdChoiceLabl->hide();
		thresholdLabl->hide();		
		integrationMethod->hide();
		integrationLabl->hide();
		massField->hide();
		chargeField->hide();
		massChoice->hide();		
		massChoiceLabl->hide();
		massLabl->hide();
		chargeLabl->hide();
		objField->show();
		parityLabel->show();
		Vparity->show();
		allBtn->hide();
		dBtn->hide();
		vBtn->hide();
		pBtn->hide();
		hBtn->hide();
		if(!userSet) setObjectName();
	}
	else if( text.contains("Rotation Matrix") ){
		centreField->hide();
		centreChoice->hide();
		centreChoiceLabl->hide();
		centreLabl->hide();
		dirnField->hide();
		dirnChoice->hide();
		dirnChoiceLabl->hide();
		dirnLabl->hide();
		binsLabl->hide();
		numberBins->hide();	
		potentialField->hide();
		potentialChoice->hide();
		potentialChoiceLabl->hide();
		potentialLabl->hide();			
		thresholdField->hide();
		thresholdChoice->hide();
		thresholdChoiceLabl->hide();
		thresholdLabl->hide();			
		integrationMethod->hide();
		integrationLabl->hide();
		massField->hide();
		chargeField->hide();
		massChoice->hide();		
		massChoiceLabl->hide();
		massLabl->hide();
		chargeLabl->hide();
		objField->hide();
		parityLabel->hide();
		Vparity->hide();
		allBtn->hide();
		dBtn->hide();
		vBtn->hide();
		pBtn->hide();
		hBtn->hide();
		if(!userSet) name_field->setText("Rxyz");
	}
}

// -------------

void PSD_Window::setDirnChoice(const QString & text){
    
	if( text.contains("Z") ) {
		dirnField->setEnabled(false);
	}
	else if( text.contains("Y") ) {
		dirnField->setEnabled(false);
	}
	else if( text.contains("X") ) {
		dirnField->setEnabled(false);
	}
	else {
		dirnField->setEnabled(true);
	}
}

// -------------

void PSD_Window::setPotentialChoice(const QString & text){
    
	if( text.contains("No") ) {
		potentialField->setEnabled(false);
	}
	else if( text.contains("eV") ) {
		potentialField->setEnabled(true);
	}
	else {
		potentialField->setEnabled(true);
	}
}

// -------------

void PSD_Window::setMomentsAll(int on){

	dBtn->setChecked(on);
	vBtn->setChecked(on);
	pBtn->setChecked(on);
	hBtn->setChecked(on);
	
}

// -------------

void PSD_Window::setThresholdChoice(const QString & text){
    
	if( text.contains("No") ) {
		thresholdField->setEnabled(false);
	}
	else if( text.contains("eV") ) {
		thresholdField->setEnabled(true);
	}
	else {
		thresholdField->setEnabled(true);
	}
}
// -------------

void PSD_Window::setMassChoice(const QString & text){
    
	if( text.contains("Electron") ) {
		massField->setEnabled(false);
		chargeField->setEnabled(false);
		massField->setValue(0.0005447);
		chargeField->setValue(1);
	}
	else if( text.contains("Proton") ) {
		massField->setEnabled(false);
		chargeField->setEnabled(false);
		massField->setValue(1);
		chargeField->setValue(1);
	}
	else if( text.contains("He++") ) {
		// must test He++ first as He+ also tests true
		massField->setEnabled(false);
		chargeField->setEnabled(false);
		massField->setValue(4);
		chargeField->setValue(2);
	}
	else if( text.contains("He+") ) {
		massField->setEnabled(false);
		chargeField->setEnabled(false);
		massField->setValue(4);
		chargeField->setValue(1);
	}
	else if( text.contains("O+") ) {
		massField->setEnabled(false);
		chargeField->setEnabled(false);
		massField->setValue(16);
		chargeField->setValue(1);
	}
	else {
		massField->setEnabled(true);
		chargeField->setEnabled(true);
		massField->setValue(1.);
		chargeField->setValue(1);
	}

	
}

// -------------

void PSD_Window::setCentreChoice(const QString & text){
    
	if( text.contains("PSD") ) {
		centreField->setEnabled(false);
	}
	else {
		centreField->setEnabled(true);
	}
}


// -------------

void PSD_Window::setNewName(){
     
	if(QuSlotDebug())
	{
		printf("PSD_Window::setNewName \n");
		fflush(stdout);
	}
	
	// Look for PEACE or CIS
	QdObject_var dobj = objField->getSlotObject();
	QdObject_var name_obj;

	bool isPEACE = false;
	bool isCIS = false;
	if(dobj->xref_exists(OBJECT_NAME) ){
		QString name_str = QString(QuGetAttrText(OBJECT_NAME, dobj).c_str());
		if( name_str.contains("CIS", Qt::CaseInsensitive) ) isCIS = true;
		else if( name_str.contains("PEA", Qt::CaseInsensitive) ) isPEACE = true;
	}
	
	bool isGSE = false;
	bool isGSM = false;
	QString attr_str = QString(QuGetFrame(dobj).c_str());
	if( attr_str.contains("GSE", Qt::CaseInsensitive) ) isGSE = true;
	else if( attr_str.contains("GSM", Qt::CaseInsensitive) ) isGSM = true;

	if(isGSE || isGSM) statusBar()->showMessage("PSD already in GSE or GSM frame",0);
	else statusBar()->showMessage(" ",0);
	

	if(dobj->xref_exists("ENTITY") ){
		QString entity_str=QString(QuGetAttrText("ENTITY", dobj).c_str()).toLower();
		if (entity_str=="electron") {
			massChoice->setCurrentIndex(0);
			setMassChoice("Electron");
		}
		else if (entity_str=="proton") {
			massChoice->setCurrentIndex(1);
			setMassChoice("Proton");
		}
		else if (entity_str=="helium+") {
			massChoice->setCurrentIndex(2);
			setMassChoice("He+");
		}
		else if (entity_str=="helium++") {
			massChoice->setCurrentIndex(3);
			setMassChoice("He++");
		}
		else if (entity_str=="alpha") {
			massChoice->setCurrentIndex(3);
			setMassChoice("He++");
		}
		else if (entity_str=="oxygen+") {
			massChoice->setCurrentIndex(4);
			setMassChoice("O+");
		}
		else {
			massChoice->setCurrentIndex(5);
			massField->setValue(1);
			chargeField->setValue(1);
		}
	}
	else{
		massChoice->setCurrentIndex(5);
	}
	
	if(isPEACE){
		Vparity->setCurrentIndex(0);
		transMethod->setCurrentIndex(0);
		massChoice->setCurrentIndex(0);
	}
	else if(isCIS){
		Vparity->setCurrentIndex(1);
		transMethod->setCurrentIndex(1);
	}
			
	// set output name default
	if (!userSet) setObjectName();

	this->adjustSize();
}

// -----------------

void PSD_Window::setObjectName()
{  
	QString objName = QString(this->objField->textObject().toLatin1().data());
   
	if(!objName.isEmpty()){
		QString newName = QString(objName);
       
		int ptr = newName.lastIndexOf("/");
		if (ptr >= 0) newName.remove(0, ptr+1);
		ptr = newName.indexOf(" [");
		if (ptr > 0) newName.truncate(ptr);

		this->name_field->setText(newName);
	}
	else{
		this->name_field->setText("PSD");
	}
   
}
// -------------

void PSD_Window::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("PSD_Window::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) evaluate();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

// -------------

void PSD_Window::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("PSD_Window::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate(true);
   
}


// -------------

void PSD_Window::evaluate(){
	
	static PSD_Parameters *para_ptr=NULL;
	
	if(QuSlotDebug())
	{
		printf("PSD_Window::evaluate \n");
		fflush(stdout);
	}
	if(running){
		running = false;
		// reset GUI
		setEvalBtn("Evaluate");
		percentage->setText(" " );
		
		if(para_ptr != NULL) para_ptr->stop = true;
		this->statusBar()->showMessage("Interrupt: output data truncated", 20000);
		
		return;

	}
	
	running = true;
	
	setEvalBtn(" Interrupt ");

	qApp->processEvents();

	
	if (processMethod->currentText().contains("Rotation Matrix")){
		// don't need PSD
		QdObject_var RotnMtx = this->makeRotnMatrix();
		
		if( !RotnMtx.is_nil() ) this->saveRotnMatrix(RotnMtx);
		else this->statusBar()->showMessage("Need Lat and Long", 10000);
		
		// reset GUI
		setEvalBtn("Evaluate");
		percentage->setText(" " );
		running = false;
		return;
	}
	
	QdObject_var data = objField->getSlotObject();
	int direction;
	if( Vparity->currentText().contains("Look") ) direction = -1;
	else  direction = 1;
	
    PSD_Parameters para(data, direction);
	para_ptr = &para;

    if(!para.init_ok)
    {
		this->statusBar()->showMessage(para.message, 20000);
		// reset GUI
		setEvalBtn("Evaluate");
		percentage->setText(" " );
		running = false;
        return;
    }
	
	// set the mass and charge
	
	para.init_species(massField->value(), chargeField->value());
		
	// set rotation matrix

	QdObject_var rtnMtx = this->makeRotnMatrix();

   	if(rtnMtx.is_nil()) {
		this->statusBar()->showMessage("Failed to create rotation matrix", 20000);
		// reset GUI
		setEvalBtn("Evaluate");
		percentage->setText(" " );
		running = false;
		return; 
	}
	
	if( para.init_rotn(rtnMtx) == false) {
		this->statusBar()->showMessage(para.message, 20000);
		// reset GUI
		setEvalBtn("Evaluate");
		percentage->setText(" " );
		running = false;
		return;
	}

	// Save Slice or Moments
	
	if (processMethod->currentText().contains("Slice PSD")){
	
		// set up parameters
		
		// centre of velocity to subtract off
		QdObject_var centreOfPSD; // will be is_nil() if PSD centre required
		
		if(centreChoice->currentText().contains("Bulk") ){
			// user provided velocity centre
			centreOfPSD = centreField->getSlotObject();
			
		} 
		if( !para.init_PSDcentre(centreOfPSD) ) {
			this->statusBar()->showMessage(para.message, 20000);
			// reset GUI
			setEvalBtn("Evaluate");
			percentage->setText(" " );
			running = false;
			return;
		}
		
		// direction of normal
		QdObject_var normal; 
		
		if(dirnChoice->currentText().contains("Z") ){
			normal = QuGetObjByName("Constants/Iz");
		}
		else if(dirnChoice->currentText().contains("Y") ){
			normal = QuGetObjByName("Constants/Iy");
		}
		else if(dirnChoice->currentText().contains("X") ){
			normal = QuGetObjByName("Constants/Ix");
		}
		else if(dirnChoice->currentText().contains("Along") ){

			// user provided dircection
			normal = dirnField->getSlotObject();
		} 
		para.init_sliceNormal(normal);
		
		// use zero SC potential 
    	QdObject_var pot_var=new QdRScalar(0.);
		if( !para.init_potential(pot_var) ) {
			this->statusBar()->showMessage(para.message, 20000);
			// reset GUI
			setEvalBtn("Evaluate");
			percentage->setText(" " );
			running = false;
			return;
		}
		
		// use zero for energy threshold
    	QdObject_var threshold_var=new QdRScalar(0.);
		if( !para.init_threshold(threshold_var) ) {
			this->statusBar()->showMessage(para.message, 20000);
			// reset GUI
			setEvalBtn("Evaluate");
			percentage->setText(" " );
			running = false;
			return;
		}
	
		// set up memory for quantities used often
		para.init_cartesian();

		para.resolution = numberBins->value();

		evaluateSlice(para);
		
	
	}
	else if (processMethod->currentText().contains("Moments of PSD")){
	
		// set SC potential 
		
		QdObject_var pot_var;
		if(potentialChoice->currentText().contains("No Correction")){
			
			QdRScalar_var pot = new QdRScalar(0.0);
			pot_var = pot;

		}
		else {
			pot_var = potentialField->getSlotObject();
		}
		
		if( !para.init_potential(pot_var) ) {
			this->statusBar()->showMessage(para.message, 20000);
			// reset GUI
			setEvalBtn("Evaluate");
			percentage->setText(" " );
			running = false;
			return;
		}

		// set threshold energy 
		 
		QdObject_var threshold_var;
		if(thresholdChoice->currentText().contains("No Threshold")){
			
			QdRScalar_var thr = new QdRScalar(0.0);
			threshold_var = thr;

		}
		else {
			threshold_var = thresholdField->getSlotObject();
		}
		
		if( !para.init_threshold(threshold_var) ) {
			this->statusBar()->showMessage(para.message, 20000);
			// reset GUI
			setEvalBtn("Evaluate");
			percentage->setText(" " );
			running = false;
			return;
		}

		para.init_cartesian();
		

		evaluateMoments(para);

	}

	// reset GUI
	setEvalBtn("Evaluate");
	percentage->setText(" " );
	running = false;


}
//-----------------

bool PSD_Window::evaluateSlice(PSD_Parameters &para){

	QdRMatrixSeq_var PSD_slice;

	para.getSlice(PSD_slice, percentage);
		
	QString newName = name_field->text();
	newName += "_slice";
	QwuiPutOnWL(PSD_slice, newName.toLatin1().data());
	
    return true;

}

//-----------------

bool PSD_Window::evaluateMoments(PSD_Parameters &para){

	QdRScalarSeq_var density;
	QdRMatrixSeq_var velocity;
	QdRMatrixSeq_var pressure;
	QdRMatrixSeq_var heat;
	QdRScalarSeq_var temperature;
	
	int flags = MOM_DENSITY|MOM_VELOCITY; // p, T and h need n & v
	if(pBtn->isChecked()) flags = flags|MOM_PRESSURE;
	if(hBtn->isChecked()) flags = flags|MOM_HEATFLUX;
	
	if(integrationMethod->currentText().contains("Sum") ){
		// Sum over bins
		
		para.getMomentsSummed(flags, density, velocity, pressure, heat, temperature, percentage);		
	}
	else{
		// Integrate using Delauney triangulation
		
		para.getMomentsIntegrated(flags, density, velocity, pressure, heat, temperature, percentage);
		
	}
	 
	if( dBtn->isChecked() && !density.is_nil() ){		

		QString newName = name_field->text();
		newName += "_den";
		QwuiPutOnWL(density, newName.toLatin1().data());
	}
	if( vBtn->isChecked() && !velocity.is_nil() ){		

		QString newName = name_field->text();
		newName += "_vel";
		QwuiPutOnWL(velocity, newName.toLatin1().data());
	}
	if( pBtn->isChecked() && !pressure.is_nil() ){		

		QString newName = name_field->text();
		newName += "_pres";
		QwuiPutOnWL(pressure, newName.toLatin1().data());
	}
	if( hBtn->isChecked() && !heat.is_nil() ){		

		QString newName = name_field->text();
		newName += "_heatFlux";
		QwuiPutOnWL(heat, newName.toLatin1().data());
	}
	if( pBtn->isChecked() && !temperature.is_nil() ){		

		QString newName = name_field->text();
		newName += "_T";
		QwuiPutOnWL(temperature, newName.toLatin1().data());
	}
    return true;
}


//-----------------

bool PSD_Window::saveRotnMatrix(QdObject_var rtnMtx){

	if(rtnMtx.is_nil()){
		this->statusBar()->showMessage("Rotation Matrix is empty", 10000);
		return false;
	}
	
	QString name = name_field->text();
	if(transTarget->currentIndex() == GSE) name += "_gse";
	else if(transTarget->currentIndex() == GSM) name += "_gsm";
	
	QwuiPutOnWL(rtnMtx, name.toLatin1().data());
	return true;
}

//-----------------

QdObject_var PSD_Window::makeRotnMatrix(){
	
	QdObject_var rtnMtx;
	QdTimeSeq_var times;
	bool isToGSE = true;
	bool isToGSM = false;
	int n_elements;

	if(transMethod->currentText().contains("use rotation")){
		rtnMtx = rotnField->getSlotObject();
		if(rtnMtx.is_nil()){
			this->statusBar()->showMessage("No Rotation Matrix ", 10000);
			return rtnMtx;
		}
		times=get_timetags(rtnMtx);
		n_elements = rtnMtx->sequence_size();
		
		if(rtnMtx->xref_exists("ToFrame") ){
			QString toframe = QuGetAttrText("ToFrame", rtnMtx).c_str();		
			
			if(toframe.contains("gsm", Qt::CaseInsensitive) ) { 
				isToGSM = true; 
				isToGSE = false;
			}
		}
	}
	else {
	
	
		QdObject_var lat_temp = latField->getSlotObject();
    	QdObject_var lon_temp = longField->getSlotObject();

    	QdRScalarSeq_var lat_seq=QdRScalarSeq_var::narrow(lat_temp);
    	QdRScalarSeq_var lon_seq=QdRScalarSeq_var::narrow(lon_temp);
    	QdRScalar_var lat_const=QdRScalar_var::narrow(lat_temp);
    	QdRScalar_var lon_const=QdRScalar_var::narrow(lon_temp);

    	double conv2rad_lat=M_PI/180.;
    	double conv2rad_lon=M_PI/180.;
    
		if(lat_seq.is_nil() && lat_const.is_nil())
		{
        	this->statusBar()->showMessage("Wrong data type for Latitude", 10000);
        	return rtnMtx;
		}
    
    	if(lon_seq.is_nil() && lon_const.is_nil())
    	{
			this->statusBar()->showMessage("Wrong data type for Longitude", 10000);
        	return rtnMtx;
    	}
    
    	if(!lat_temp.is_nil())
    	{
			QString units=QuGetAttrText("UNITS", lat_temp).c_str();
        	if(units.contains("rad", Qt::CaseInsensitive)) conv2rad_lat=1.;
    	}
    	
    	if(!lon_temp.is_nil())
    	{
        	QString units=QuGetAttrText("UNITS", lon_temp).c_str();
			if(units.contains("rad", Qt::CaseInsensitive)) conv2rad_lon=1.;
    	}
   
    
    	if(!lat_seq.is_nil() || !lon_seq.is_nil())
    	{
			QString frame;
			if(!lat_seq.is_nil()) frame = QuGetFrame(lat_seq).c_str();		
			else  frame = QuGetFrame(lon_seq).c_str();	
			
			if(frame.contains("gsm", Qt::CaseInsensitive) ) { 
				isToGSM = true; 
				isToGSE = false;
			}
			
        	if(!lat_seq.is_nil() && !lon_seq.is_nil())
        	{
           		int qar_status=QarTestJoined(lat_seq, lon_seq);
            	if(qar_status==QAR_ERR)
            	{
     			   	this->statusBar()->showMessage("Latitude and Longitude not joined", 10000);
                	return rtnMtx;
            	}
			}
        
        	if(!lat_seq.is_nil())
        	{
            	times=get_timetags(lat_seq);
        		n_elements=lat_seq->sequence_size();
        	}
        	else
        	{
            	times=get_timetags(lon_seq);
        		n_elements=lon_seq->sequence_size();
        	}
        
        
        	QdRMatrixSeq_var result=new QdRMatrixSeq(n_elements, Dim(3,3));
 
		       
			double lat, lon;
			double SCx, SCz, SCy, alpha;
 
			for(int i=0; i<n_elements; i++)
			{
            	if(!lat_seq.is_nil())
				 	lat = lat_seq->elt(i)*conv2rad_lat;
            	else lat = (*lat_const)*conv2rad_lat;
            
            	if(!lon_seq.is_nil())
                 	lon = lon_seq->elt(i)*conv2rad_lon;
            	else lon = (*lon_const)*conv2rad_lon;
           
        		SCx = cos(lat)*cos(lon);
				SCz = sin(lat);
				SCy = cos(lat)*sin(lon);
        		alpha = sqrt(1.-SCx*SCx);

				if(transMethod->currentText().contains("PEA")){
   		         	result->elt(i)(0, 0)=alpha;
        	    	result->elt(i)(0, 1)=0.;
        	    	result->elt(i)(0, 2)=SCx;
         		   	result->elt(i)(1, 0)=-SCx*SCy/alpha;
         	   		result->elt(i)(1, 1)=SCz/alpha;
            		result->elt(i)(1, 2)=SCy;
            		result->elt(i)(2, 0)=-SCx*SCz/alpha;
            		result->elt(i)(2, 1)=-SCy/alpha;
            		result->elt(i)(2, 2)=SCz;
            	}
				else if(transMethod->currentText().contains("CIS")){
					result->elt(i)(0, 0)=alpha;
            		result->elt(i)(0, 1)=0.;
            		result->elt(i)(0, 2)=-SCx;
            		result->elt(i)(1, 0)=-SCx*SCy/alpha;
            		result->elt(i)(1, 1)=-SCz/alpha;
            		result->elt(i)(1, 2)=-SCy;
            		result->elt(i)(2, 0)=-SCx*SCz/alpha;
            		result->elt(i)(2, 1)=SCy/alpha;
            		result->elt(i)(2, 2)=-SCz;
				}
            
        	}
            if(!times.is_nil()) set_timetags(result, times);
			rtnMtx = result;
    	}
    	else
    	{
        	QdRMatrix_var result=new QdRMatrix(Dim(3,3));
        	n_elements = 1;
			
        	double lat  = (*lat_const)*conv2rad_lat;
			double lon = (*lon_const)*conv2rad_lon;
        	double SCx = cos(lat)*cos(lon);
			double SCz = sin(lat);
			double SCy = cos(lat)*sin(lon);
        	double alpha = sqrt(1.-SCx*SCx);

			QString frame = QuGetFrame(lat_const).c_str();		
			if( frame.isEmpty() )  frame = QuGetFrame(lon_const).c_str();	
			
			if(frame.contains("gsm", Qt::CaseInsensitive) ) { 
				isToGSM = true; 
				isToGSE = false;
			}
			times = get_timetags(lat_const);
			if(times.is_nil()) times = get_timetags(lon_const);


			if(transMethod->currentText().contains("PEA")){
        		(*result)(0, 0)=alpha;
        		(*result)(0, 1)=0.;
        		(*result)(0, 2)=SCx;
        		(*result)(1, 0)=-SCx*SCy/alpha;
        		(*result)(1, 1)=SCz/alpha;
        		(*result)(1, 2)=SCy;
        		(*result)(2, 0)=-SCx*SCz/alpha;
        		(*result)(2, 1)=-SCy/alpha;
        		(*result)(2, 2)=SCz;
        	}
			else if(transMethod->currentText().contains("CIS")){
        		(*result)(0, 0)=alpha;
				(*result)(0, 1)=0.;
        		(*result)(0, 2)=-SCx;
        		(*result)(1, 0)=-SCx*SCy/alpha;
        		(*result)(1, 1)=-SCz/alpha;
        		(*result)(1, 2)=-SCy;
        		(*result)(2, 0)=-SCx*SCz/alpha;
        		(*result)(2, 1)=SCy/alpha;
        		(*result)(2, 2)=-SCz;
			}
			
			rtnMtx = result;
			
    	}
    
	} // else on known instrument
	
	// rotate to GSM?
	QrVector vec;
    
	if(transTarget->currentText().contains("GSM") && isToGSE ){
		if( times.is_nil() ) {
			this->statusBar()->showMessage("No timetags for GSE to GSM rotation", 10000);
			if(transMethod->currentIndex() == 0) QuSetFrame(rtnMtx, "rot>SR2PEA_xyz");
			else if(transMethod->currentIndex() == 1) QuSetFrame(rtnMtx, "rot>ISR2CIS_xyz");
		
			vec.QrSetToFrameAttr(GSE, rtnMtx); // ToFrame is GSE as rotn didn't happen

		}
		else{
			
			vec.n_recs = n_elements;
			vec.in_frame = GSE;
			vec.out_frame = GSM;
			vec.t = times;
			vec.IsPositionVector = false;

			QdRMatrixSeq_var gse2gsm = vec.QrGetRotnMatx();
		
			QdRMatrix_var rtnMtxConst = QdRMatrix_var::narrow(rtnMtx);
			QdRMatrixSeq_var rtnMtxSeq = QdRMatrixSeq_var::narrow(rtnMtx);
			try {
		
				if(!rtnMtxConst.is_nil()) *rtnMtxConst = gse2gsm[0] * (*rtnMtxConst);
				else if(!rtnMtxSeq.is_nil() && rtnMtxSeq->sequence_size() == gse2gsm->sequence_size()) {
					for(int i=0; i<times->sequence_size(); i++) rtnMtxSeq[i] = gse2gsm[i] * rtnMtxSeq[i];
				}	

			}
			catch (Exception &e) {
       			e.print_msg();
       			return rtnMtx;
			}
		}
	}
    
	
	// rotate to GSE?
	if(transTarget->currentText().contains("GSE") && isToGSM ){
		if( times.is_nil() ) {
			this->statusBar()->showMessage("No timetags for GSM to GSE rotation", 10000);
			if(transMethod->currentIndex() == 0) QuSetFrame(rtnMtx, "rot>SR2_xyz");
			else if(transMethod->currentIndex() == 1) QuSetFrame(rtnMtx, "rot>ISR2_xyz");
		
			vec.QrSetToFrameAttr(GSM, rtnMtx); // ToFrame is GSM as rotn didn't happen
		}
		else{
		
			vec.n_recs = n_elements;
			vec.in_frame = GSM;
			vec.out_frame = GSE;
			vec.t = times;
			vec.IsPositionVector = false;

			QdRMatrixSeq_var gsm2gse = vec.QrGetRotnMatx();
			
			QdRMatrix_var rtnMtxConst = QdRMatrix_var::narrow(rtnMtx);
			QdRMatrixSeq_var rtnMtxSeq = QdRMatrixSeq_var::narrow(rtnMtx);
			try {
		
				if(!rtnMtxConst.is_nil()) *rtnMtxConst = gsm2gse[0] * *rtnMtxConst;
				else if(!rtnMtxSeq.is_nil() && rtnMtxSeq->sequence_size() == gsm2gse->sequence_size()) {
					for(int i=0; i<times->sequence_size(); i++) rtnMtxSeq[i] = gsm2gse[i] * rtnMtxSeq[i];
				}
			}
			catch (Exception &e) {
       			e.print_msg();
       			return rtnMtx;
			}
		}
	}
	
	// set ToFrame attribute
	
	if(transMethod->currentIndex() == 0) QuSetFrame(rtnMtx, "rot>SR2_xyz");
	else if(transMethod->currentIndex() == 1) QuSetFrame(rtnMtx, "rot>ISR2_xyz");
		
	if(transTarget->currentIndex() == GSM) vec.QrSetToFrameAttr(GSM, rtnMtx);
	else vec.QrSetToFrameAttr(GSE, rtnMtx);

	return rtnMtx;

}


