/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/
/*************************************************************************
* PROJECT: Cluster UK CDHF
*
* COMPONENT:    Plug In library
* 
* MODULE:	ComplexProduct.cc
*
* LANGUAGE:	C++
*
* FUNCTION:     QSAS ComplexProduct wrapper
*
* PURPOSE:      Wrapper for NR 'spctrm' power spectrum.
* 
* DESCRIPTION:  Plug In module checking input parameters.
*
* ARGUMENTS:
*    IN:      
*
*   OUT:
*         
* GLOBALS:
*
* RETURN:
*    TYPE:    QplugReturnStatus
*    VALUES:
*
* FUNCTIONS_USED:	
*
* ALGORITHM:  
*
* HISTORY: created  15/3/2001 Markus Fraenz 
*
***********************************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

#include "qplug_if.h"
#include "qdutil.h"
#include "Qdos.h"

#include "qar.h"

extern "C" QplugReturnStatus	ComplexProd( QplugArgList *);

using namespace QSAS;



QplugReturnStatus	ComplexProd( QplugArgList *call_list )
{

// define some C variables

int	        attribute_ok,list_length,vectorset=0;
unsigned long	Ndata;
int qar_status;

char str_out[150];
string attr_ptr;

// define QSAS data objects


QdRScalarSeq_var  out_im_series,out_re_series;


//
// ...first, unpack the input data objects
//

// ...first item is input Series

QplugAppendTextDisplay("ComplexProduct 1.0 11 Apr 2001\n");

QdObject_var Re1_in = (* call_list)[0];

QdRScalarSeq_var Re1 = QdRScalarSeq_var::narrow(Re1_in);
if(Re1.is_nil() ){
      QplugAppendTextDisplay ("ComplexProduct>> ERROR - Re1 sequence type not handled\n");
      return QPLUG_FAILURE;  
}

Ndata = Re1->sequence_size();

QdObject_var Im1_in = (* call_list)[1];

QdRScalarSeq_var Im1 = QdRScalarSeq_var::narrow(Im1_in);
if(Im1.is_nil() ){
      QplugAppendTextDisplay ("ComplexProduct>> ERROR - Im1 sequence type not handled\n");
      return QPLUG_FAILURE;  
}


QdObject_var Re2_in = (* call_list)[2];

QdRScalarSeq_var Re2 = QdRScalarSeq_var::narrow(Re2_in);
if(Re2.is_nil() ){
      QplugAppendTextDisplay ("ComplexProduct>> ERROR - Re2 sequence type not handled\n");
      return QPLUG_FAILURE;  
}


QdObject_var Im2_in = (* call_list)[3];

QdRScalarSeq_var Im2 = QdRScalarSeq_var::narrow(Im2_in);
if(Im2.is_nil() ){
      QplugAppendTextDisplay ("ComplexProduct>> ERROR - Im2 sequence type not handled\n");
      return QPLUG_FAILURE;  
}


// Test validity of inputs...


qar_status =  QarTestConformal(Re1_in, Im1_in);
  if(qar_status == QAR_ERR)      
  {
    QplugAppendTextDisplay ("Arithmetic>> input sequences not joined properly - Re1, Im1\n");
     return QPLUG_FAILURE; 
  }
qar_status =  QarTestConformal(Re1_in, Re2_in);
  if(qar_status == QAR_ERR)     
  {
    QplugAppendTextDisplay ("Arithmetic>> input sequences not joined properly - Re1, Re2\n");
     return QPLUG_FAILURE; 
  } 
qar_status =  QarTestConformal(Re1_in, Im2_in);
  if(qar_status == QAR_ERR)     
  {
    QplugAppendTextDisplay ("Arithmetic>> input sequences not joined properly - Re1, Im2\n");
     return QPLUG_FAILURE; 
  }


QplugAppendTextDisplay ("ComplexProduct>> Calculating.\n");

//      (notice that NR routines start indices with 1!!) 



out_re_series  = new QdRScalarSeq(Ndata);
out_im_series = new QdRScalarSeq(Ndata);

for (long i =0; i<Ndata; i++) { 
 out_re_series[i]  = Re1[i]*Re2[i] + Im1[i]*Im2[i];
 out_im_series[i] = Re1[i]*Im2[i] - Im1[i]*Re2[i];
}

	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( Re1_in );
	if(qd0.is_nil()) qd0 = get_Depend_0( Re2_in );
	if ( !qd0.is_nil() ) {
		out_re_series->change_xref(DEPEND_0 , qd0 );
		out_im_series->change_xref(DEPEND_0 , qd0 );
	}



//	...set some attributes

 // SI_conversion
 
 attr_ptr = QarSIConv_product(Re1_in, Re2_in);
 if ( !attr_ptr.empty() ) {
   QuSetTxtAttr("SI_conversion", attr_ptr.c_str(), out_re_series);
   QuSetTxtAttr("SI_conversion", attr_ptr.c_str(), out_im_series);
 }
 attr_ptr = QarUNITS_product(Re1_in, Re2_in);
 if ( !attr_ptr.empty() ) {
   QuSetTxtAttr("UNITS", attr_ptr.c_str(), out_re_series);
   QuSetTxtAttr("UNITS", attr_ptr.c_str(), out_im_series);
 } 

  QuSetTxtAttr("LABLAXIS", "ReCProd", (QdObject_var) out_re_series);
  QuSetTxtAttr("FIELDNAM", "ReCProd", (QdObject_var) out_re_series);
  QuSetFrame( (QdObject_var) out_re_series, "scalar");

  QuSetTxtAttr("LABLAXIS", "ImCProd", (QdObject_var) out_im_series);
  QuSetTxtAttr("FIELDNAM", "ImCProd", (QdObject_var) out_im_series);
  QuSetFrame((QdObject_var) out_im_series, "scalar");

//	..add them to the list

  call_list->push_back( (QdObject_var) out_re_series);    
  call_list->push_back( (QdObject_var) out_im_series);     


//	..some messages to the analysis interface window..

sprintf(str_out, "ComplexProduct>> %ld data points\n", Ndata);
QplugAppendTextDisplay (str_out);
QplugAppendTextDisplay ("ComplexProduct>> finished...\n");


//	..clean up and exit


return QPLUG_SUCCESS;


}



