/* PGSpectro 2-D spectrogram and PGWheel routines using Pgplot
*  Also simple XY traces
*
*  Handles both Y-X and Y-time arrays for spectrograms
*
*  Language:  C
*
*  History:
*  July 2000 SJS First Draft
*  25 April 2001 - up-dated
*  1 Oct 2001 - fixed bug in calculating colour from zval and zrange by
*               adding cast to float and extra parenthesis to prevent
*               early truncating cast to int
*    Oct 2001 - added XYtrace routines
*  Jan 2002   - minor bug fix (double -> int for cpgslw)
*  Mar 2002   - add missing values for PGSpectro and time for traces
*  Sep 2008   - A. Rochel: translated to plplot
* 
*  See header file for description of parameters and defaults
*
*  Use:
*
*    Allocate PGSpectro or PGWheel structure
*    Fill with defaults
*    Fill with actual arrays, strings for labels
*    Set options
*    Pass to plot routine
*
*    Notes: Pgplot works entirely in floats
*           Time values should be in seconds of the day
*/

// #ifdef __cplusplus
// extern "C"{
// #endif
// 
// #include "PGSpectro.h"
// 
// /*****************************************************************/
// void set_pgp_rainbow(PGSpectro_col color_table){
// /*****************************************************************/
// /* set pgplot colours to a rainbow, with red at max of col index,
// *  purple at min, or to gray with white as either low or high.
// *  Returns number of colors in table
// *
// *  Need to do more than once if cpggray is used, as it re-defines
// *  the col table
// *
// */
// 
// 	 PLFLT *h, *s, *l, *i;
// 
// 	 switch(color_table)
// 	 {
// 		 case PGSPECTRO_GRAY_WHITE_HIGH:
// 			 h=new PLFLT[2]; h[0]=0.; h[1]=0.;
// 			 s=new PLFLT[2]; s[0]=0.; s[1]=0.;
// 			 l=new PLFLT[2]; l[0]=0.; l[1]=1.;
//            
// 			 i=new PLFLT[2]; i[0]=0.; i[1]=1.;
//            
// 			 plscmap1l(0, 2, i, h, l, s, NULL);
//            
// 			 delete[] h;
// 			 delete[] s;
// 			 delete[] l;
// 			 delete[] i;
// 			 break;
//             
// 		 case PGSPECTRO_GRAY_WHITE_LOW:
// 			 h=new PLFLT[2]; h[0]=0.; h[1]=0.;
// 			 s=new PLFLT[2]; s[0]=0.; s[1]=0.;
// 			 l=new PLFLT[2]; l[0]=1.; l[1]=0.;
//            
// 			 i=new PLFLT[2]; i[0]=0.; i[1]=1.;
//            
// 			 plscmap1l(0, 2, i, h, l, s, NULL);
//            
// 			 delete[] h;
// 			 delete[] s;
// 			 delete[] l;
// 			 delete[] i;
// 			 break;
// 				 
// 		 case PGSPECTRO_RAINBOW:
// 		 default:
// 			 h=new PLFLT[2]; h[0]=300.; h[1]=-10.;
// 			 s=new PLFLT[2]; s[0]=1.; s[1]=1.;
// 			 l=new PLFLT[2]; l[0]=.5; l[1]=.5;
//            
// 			 i=new PLFLT[2]; i[0]=0.; i[1]=1.;
//             
// 			 plscmap1l(0, 2, i, h, l, s, NULL);
//             
// 			 delete[] h;
// 			 delete[] s;
// 			 delete[] l;
// 			 delete[] i;
//             
// 			 break;
// 	 }
// 
// }
// 
// /*****************************************************************/
// void spectro_wedge(PGSpectro_wedge wedge, float zscalemin,
//            float zscalemax, PGSpectro_linlog lin_log) {
// /*****************************************************************/
// /* sjs routine to draw spectrogram key to the right of the current viewport */
// 
// PLFLT x1vp_save, x2vp_save, y1vp_save, y2vp_save;
// PLFLT x1win_save, x2win_save, y1win_save, y2win_save; /* world coords */
// PLINT ci;  /* color index */
// PLFLT x1,x2,y1,y2;
// PLFLT chsize_ndc,dummy;
// 
// PLINT i,col, icollow, icolhigh, ncolors;  /* see set_rainbow for algorithm */
// PLFLT zstep, zrange;
// 
// plgvpw(&x1win_save, &x2win_save, &y1win_save, &y2win_save); /* save window */
// 
// plgvpd(&x1vp_save, &x2vp_save, &y1vp_save, &y2vp_save);/* get vport in ndc*/
// 
// plgchr(&chsize_ndc,&dummy);
// chsize_ndc/=2500.;
// 
// x1 = x2vp_save + wedge.disp*chsize_ndc;
// x2= x1 + wedge.width*chsize_ndc;
// 
// plvpor(x1,x2,y1vp_save, y2vp_save);
// plwind(0.,1.,0., 1.);
// 
// zrange = zscalemax - zscalemin;
// PLFLT rect_x[4], rect_y[4];
// // for (i = 0; i<ncolors; i++){
// //    x1=0.0; x2=1.0;
// //    y1= zscalemin + ((float)i-0.5)*zstep;
// //    y2=y1+zstep;
// //    col = icollow + i;
// //    cpgsci(col);
// //    cpgrect(x1,x2,y1,y2);
// // }
// for(i=0; i<40; i++)
// {
// 	plcol1(((PLFLT)i+0.5)/40.);
// 	rect_x[0]=rect_x[3]=0.;
// 	rect_x[1]=rect_x[2]=1.;
// 	rect_y[0]=rect_y[1]=((PLFLT)i/40.);
// 	rect_y[2]=rect_y[3]=(((PLFLT)i+1.)/40.);
// 	plfill(4, rect_x, rect_y);
// }
// plwind(0.,1.,zscalemin, zscalemax);
// plcol0(0);
// switch (lin_log) {
//     case PGSPECTRO_LOG:
//       plbox("bc",0.0, 0, "bcmtslv", 0.0, 0);
//       break;
//     case PGSPECTRO_LIN:
//       plbox("bc",0.0, 0, "bcmtsv", 0.0, 0);
//       break;
//     default:
//       printf("Unknown PGSpectro_linlog in spectro_wedge\n");
//       break;
// }
// plmtex("l",0.5, 0.5, 0.5, wedge.label);
// 
// /* restore pgsettings for user */
// 
// plvpor(x1vp_save, x2vp_save, y1vp_save, y2vp_save); /* restore vport in ndc */
// plwind(x1win_save, x2win_save, y1win_save, y2win_save);  /* restore window */
// 
// return ;
// }
// /*****************************************************************/
// void PGSpectro_set_defaults(PGSpectro *spectro){
// /*****************************************************************/
// 
// /* set default values for as much as possible */
// 
// spectro->nx = 0;            spectro->ny = 0;
// spectro->zscalemin = 1.0;   spectro->zscalemax = 10.0;
// spectro->skip_missing = NO; spectro->zmissing = -1.e31;
// spectro->xlabel = " ";      spectro->ylabel = " ";
// spectro->title = " ";       spectro->subtitle = " ";
// 
// spectro->color_table = PGSPECTRO_RAINBOW;
// PGSpectro_domain_set_defaults(  &(spectro->domain) );
// spectro->wedge.disp = 1.5;
// spectro->wedge.width = 1.5;
// spectro->wedge.label = " ";
// spectro->xlinlog =  PGSPECTRO_LIN;  spectro->ylinlog =  PGSPECTRO_LIN;
// spectro->zlinlog =  PGSPECTRO_LIN;
// spectro->xnums = YES;               spectro->ynums = YES;
// spectro->ylabel_vert = YES;
// spectro->draw_wedge = YES;
// spectro->timex = NO;
// spectro->datestart = " ";
// spectro->dateend = " ";
// spectro->char_size = PGSPECTRO_DEFAULT_CHAR_SIZE;
// 
// return;
// }
// 
// 
// void PGSpectro_domain_set_defaults (PGSpectro_domain *domainp){
// 
// domainp->xworldmin = 1.0;   domainp->xworldmax = 10.0;
// domainp->yworldmin = 1.0;   domainp->yworldmax = 10.0;
// domainp->xndcmin   = 0.1;   domainp->xndcmax   = 0.85;
// domainp->yndcmin   = 0.1;   domainp->yndcmax   = 0.9;
// 
// return;
// }
// 
// /*****************************************************************/
// PGSpectro_ret PGSpectro_Plot(PGSpectro *spectro){
// /*****************************************************************/
// 
// //int ncolours,ncolminusone;
// int col, icollow=LOWEST_COLOR_INDEX;
// int pgid;
// int i, j, k;
// PLFLT zrange,zmin,zmax, zval;
// int skip;
// PLFLT skip_val;
// PLFLT xworld1, xworld2, yworld1, yworld2;
// PLFLT ztemp;
// PLFLT *xlo, *xhi, *ylo, *yhi, *z;  /* local pointers */
// PLINT nx, ny;  /* local copies */
// char xopt[20], yopt[20];
// int iopt=0, jopt=0;
// 
// //cpgiden(); TODO
// //cpgsitf(0); TODO? /* 0=lin, 1 = log, 2 = sqrt; do log manually, hence always lin */
// plvpor(spectro->domain.xndcmin, spectro->domain.xndcmax,
//           spectro->domain.yndcmin, spectro->domain.yndcmax);
// 
// /*ncolours = */set_pgp_rainbow(spectro->color_table);
// // ncolminusone = ncolours -1;
// if (spectro->zscalemax > spectro->zscalemin ){       /* no inverted scales */
//    zmin = spectro->zscalemin;
//    zmax = spectro->zscalemax;
// } else {
//    zmin = spectro->zscalemax;
//    zmax = spectro->zscalemin;
// }
// 
// nx = spectro->nx;
// ny = spectro->ny;
// skip = (spectro->skip_missing == YES);
// skip_val = spectro->zmissing;
// 
// if (spectro->zlinlog == PGSPECTRO_LOG){
//    zmin = SafeLog10 (zmin);
//    zmax = SafeLog10 (zmax);
//    z = (PLFLT *)malloc(nx*ny*sizeof(PLFLT));
//    for (i=0; i<nx; i++){
//      for(j=0; j<ny; j++){
//        k=ny*i+j;
//        ztemp = spectro->z[k];
//        if (skip && ztemp == skip_val){
//            z[k]=SKIP_LOG_VALUE;
//        } else {
//            z[k]=SafeLog10(ztemp);
//        }
//      }
//    }
//    skip_val = SKIP_LOG_VALUE; /* now tests are done, re-define to plot */
//    /*printf("PGSpectro: zmax = %f, zmin = %f \n", zmax, zmin);*/
// } else {
//    z = spectro->z;
// }
// 
// zrange = zmax- zmin;
// 
// /* x-axis. Ignore log if time axes to be used */
// if(spectro->xlinlog == PGSPECTRO_LOG && spectro->timex == NO){
//    xlo = (PLFLT *)malloc(spectro->nx * sizeof(PLFLT));
//      if (xlo == NULL) return PGSPECTRO_MALLOC_FAILURE;
//    xhi = (PLFLT *)malloc (spectro->nx * sizeof(PLFLT));
//      if (xhi == NULL) return PGSPECTRO_MALLOC_FAILURE;
//    for (i=0; i<nx; i++){
//      xlo[i]  = SafeLog10(spectro->xbinlow[i]);
//      xhi[i]  = SafeLog10(spectro->xbinhigh[i]);
//    }
//    xworld1 = SafeLog10(spectro->domain.xworldmin);
//    xworld2 = SafeLog10(spectro->domain.xworldmax);
// } else {
//    xlo  = spectro->xbinlow;
//    xhi  = spectro->xbinhigh;
//    xworld1 = spectro->domain.xworldmin;
//    xworld2 = spectro->domain.xworldmax;
// }
// 
// if(spectro->ylinlog == PGSPECTRO_LOG){
//    ylo = (PLFLT *)malloc(spectro->ny * sizeof(PLFLT));
//      if (ylo == NULL) return PGSPECTRO_MALLOC_FAILURE;
//    yhi = (PLFLT *)malloc (spectro->ny * sizeof(PLFLT));
//      if (yhi == NULL) return PGSPECTRO_MALLOC_FAILURE;
//    for (i=0; i<ny; i++){
//      ylo[i] = SafeLog10(spectro->ybinlow[i]);
//      yhi[i]  = SafeLog10(spectro->ybinhigh[i]);
//    }
//    yworld1 = SafeLog10(spectro->domain.yworldmin);
//    yworld2 = SafeLog10(spectro->domain.yworldmax);
// 
// } else {
//    ylo  = spectro->ybinlow;
//    yhi  = spectro->ybinhigh;
//    yworld1 = spectro->domain.yworldmin;
//    yworld2 = spectro->domain.yworldmax;
// }
// 
// plwind(xworld1, xworld2, yworld1, yworld2);
// 
// PLFLT fill_x[4], fill_y[4];
// for (i=0; i<nx; i++){
//   for(j=0; j<ny; j++){
//     k = ny*i+j;
//     zval = z[k];
//     if (skip && zval == skip_val){  /* fill with background */
//     } else {
// 	    	zval = (zval< zmin)? zmin : zval;
// 	zval = (zval > zmax) ? zmax : zval;
// 	plcol1((zval-zmin)/zrange);
// 	fill_x[0]=xlo[i]; fill_x[1]=xlo[i]; fill_x[2]=xhi[i]; fill_x[3]=xhi[i];
// 	fill_y[0]=ylo[j]; fill_y[1]=yhi[j]; fill_y[2]=yhi[j]; fill_y[3]=ylo[j];
// 	plfill(4, fill_x, fill_y);
//     }
// 
//   }
// }
// 
// plcol0(0);  /* reset default colour */
// plschr(40., spectro->char_size);
// 
// strcpy(xopt,"bcts");   strcpy(yopt, "bcts");
// iopt = strlen(xopt);   jopt = strlen(yopt);
// 
// if (spectro->xlinlog ==PGSPECTRO_LOG && spectro->timex == NO)xopt[iopt++] = 'l';
// if (spectro->ylinlog == PGSPECTRO_LOG) yopt[jopt++] = 'l';
// if (spectro->xnums   == YES          ) xopt[iopt++] = 'n';
// if (spectro->ynums   == YES          ) yopt[jopt++] = 'n';
// 
// if (spectro->timex == YES){                     /* time axis labelling, etc. */
//   xopt[iopt++] = 'd'; 
//   pltimefmt("%H:%M");
// }
// 
// xopt[iopt] = '\0';                              /* terminate options strings */
// yopt[jopt] = '\0';
// 
// plbox(xopt, 0.0, 0, yopt , 0.0, 0);  /* box after to see ticks! */
// if (spectro->draw_wedge != NO){
//   spectro_wedge(spectro->wedge,zmin, zmax, spectro->zlinlog);
// }
// plmtex("b", 4.0, 0.5, 0.5, spectro->xlabel);
// plmtex("t", 2.0, 0.5, 0.5, spectro->title);
// if(spectro->ylabel_vert ==NO){
// //   pllab(spectro->xlabel,spectro->ylabel,spectro->title);
// 	plmtex("l", 2.0, 0.5, 0.5, spectro->ylabel);
// } else {
// //   pllab(spectro->xlabel, "", spectro->title);
// //   plmtex("lv", 1.0, 0.5, 1.0, spectro->ylabel);
// 	plmtex("lv", 1.0, 1.05, 0.5, spectro->ylabel);
// }
// plschr(40., 0.75*spectro->char_size);
// plmtex("t", 0.5, 0.5,0.5, spectro->subtitle);
// plschr(40., spectro->char_size);
// 
// 
// if (spectro->timex == YES){
//    plmtex("b", 2.5, 0.0, 0.5, spectro->datestart);
//    if ( strcmp(spectro->datestart,spectro->dateend) ){
//      plmtex("b", 2.5, 1.0, 1.0, spectro->dateend);
//    }
// }
// 
// if (spectro->xlinlog == PGSPECTRO_LOG && spectro->timex == NO) {
//    free(xlo); free (xhi);
// }
// if (spectro->ylinlog == PGSPECTRO_LOG) {
//    free(ylo); free (yhi);
// }
// if (spectro->zlinlog == PGSPECTRO_LOG) free (z);
// 
// return PGSPECTRO_OK;
// }
// 
// /*****************************************************************/
// void PGWheel_set_defaults(PGWheel *wheel){
// /*****************************************************************/
// 
// /* set default values for as much as possible */
// 
// wheel->nr = 0;          wheel->nth = 0;
// wheel->ncircles = 0;
// wheel->zscalemin = 1.0; wheel->zscalemax = 10.0;
// wheel->zmissing = -1.e31; wheel->skip_missing = NO;
// wheel->xlabel = " "; wheel->ylabel = " "; wheel->title = " ";
// wheel->subtitle = " "; wheel->subsubtitle = " ";
// 
// wheel->color_table = PGSPECTRO_RAINBOW;
// PGWheel_domain_set_defaults(  &(wheel->domain) );
// wheel->wedge.disp = 1.5;
// wheel->wedge.width = 1.5;
// wheel->wedge.label = " ";
// wheel->rlinlog =  PGSPECTRO_LIN;
// wheel->zlinlog =  PGSPECTRO_LIN;
// wheel->rnums = YES;
// wheel->title_just = 0.5;
// 
// return;
// }
// 
// 
// void PGWheel_domain_set_defaults (PGWheel_domain *domainp){
// 
// domainp->rmin = 0.0;   domainp->rmax = 10.0;
// domainp->xndcmin   = 0.15;   domainp->xndcmax   = 0.8;
// domainp->yndcmin   = 0.15;   domainp->yndcmax   = 0.8;
// 
// return;
// }
// 
// /*****************************************************************/
// PGSpectro_ret PGWheel_Plot(PGWheel *wheel){
// /*****************************************************************/
// 
// //int ncolours,ncolminusone;
// int col, icollow=LOWEST_COLOR_INDEX;
// int pgid;
// int i, j, k;
// PLFLT zrange,zmin,zmax, zval;
// int skip;
// PLFLT skip_val;
// PLFLT r1, r2, r2a;
// PLFLT ztemp;
// PLFLT *rlo, *rhi, *thlo, *thhi, *z;  /* local pointers */
// int nr, nth;  /* local copies */
// char ropt[20];
// PLFLT rcirc;
// 
// cpgqid(&pgid);
// if (pgid == 0) return PGSPECTRO_NO_DEVICE;   /* bale out if no pgplot device */
// 
// cpgiden();
// cpgsitf(0);  /* 0=lin, 1 = log, 2 = sqrt; do log manually, hence always lin */
// cpgsvp(wheel->domain.xndcmin, wheel->domain.xndcmax,
//           wheel->domain.yndcmin, wheel->domain.yndcmax);
// 
// set_pgp_rainbow(wheel->color_table);
// 
// if (wheel->zscalemax > wheel->zscalemin ){       /* no inverted scales */
//    zmin = wheel->zscalemin;
//    zmax = wheel->zscalemax;
// } else {
//    zmin = wheel->zscalemax;
//    zmax = wheel->zscalemin;
// }
// 
// nr  = wheel->nr;
// nth = wheel->nth;
// skip = (wheel->skip_missing == YES);
// skip_val = wheel->zmissing;
// 
// if (wheel->zlinlog == PGSPECTRO_LOG){
//    zmin = SafeLog10 (zmin);
//    zmax = SafeLog10 (zmax);
//    z = (PLFLT *)malloc(nr*nth*sizeof(PLFLT));
//    for (i=0; i<nr; i++){
//      for(j=0; j<nth; j++){
//        k=nth*i+j;
//        ztemp = wheel->z[k];
//        if (skip && ztemp == skip_val){
//           z[k] = SKIP_LOG_VALUE;
//        } else {
//           z[k]=SafeLog10(ztemp);
//        }
//      }
//    }
//    skip_val = SKIP_LOG_VALUE;  /* re-define now tests are done */
// } else {
//    z = wheel->z;
// }
// 
// zrange = zmax- zmin;
// 
// /* r-arrays. */
// if(wheel->rlinlog == PGSPECTRO_LOG ){
//    r1  = SafeLog10(wheel->domain.rmin);     /* "origin" in log r scale */
//    r2a =  SafeLog10(wheel->domain.rmax) ;   /* for axes */
//    r2  = r2a- r1;                           /* for dist from origin */
//    rlo = (PLFLT *)malloc(wheel->nr * sizeof(PLFLT));
//      if (rlo == NULL) return PGSPECTRO_MALLOC_FAILURE;
//    rhi = (PLFLT *)malloc (wheel->nr * sizeof(PLFLT));
//      if (rhi == NULL) return PGSPECTRO_MALLOC_FAILURE;
//    for (i=0; i<nr; i++){
//      rlo[i]  = SafeLog10(wheel->rbinlow[i]) - r1;
//      rhi[i]  = SafeLog10(wheel->rbinhigh[i])  - r1;
//    }
// } else {
//    rlo  = wheel->rbinlow;
//    rhi  = wheel->rbinhigh;
//    r1 = wheel->domain.rmin;
//    r2 = wheel->domain.rmax;
// }
// 
// thlo = wheel->thbinlow;
// thhi = wheel->thbinhigh;
// 
// cpgwnad(-r2, r2, -r2, r2);   /* window with equal x and y axes */
// 
// 
// for (i=0; i<nr; i++){
//   for(j=0; j<nth; j++){
//     k = nth*i+j;
//     zval = z[k];
//     if (skip && zval == skip_val){  /* fill with background */
//        cpgsci(0);
//     } else {
// //        zval = (zval< zmin)? zmin : zval;
// //        zval = (zval > zmax) ? zmax : zval;
// //        col = icollow+(int)( ((PLFLT)ncolminusone*(zval- zmin) )/zrange);
// //        cpgsci(col); //TODO
//     }
//     if (rhi[i] > 0.0){   /* at least part of arc has positive r */
//        if (rlo[i] < 0.0) rlo[i] = 0.0;   /* clip any negative portion */
//        arc(rlo[i],rhi[i],thlo[j],thhi[j]);/* draw an arc;*/
//     }
//   }
// }
// 
// cpgsci(1);  /* reset default colour */
// 
// spectro_wedge(wheel->wedge,zmin, zmax, wheel->zlinlog);
// /* cpglab(wheel->xlabel,wheel->ylabel," "); */
// cpgmtxt("B",4.0, 0.5, 0.5, wheel->xlabel);
// cpgmtxt("L",4.0, 0.5, 0.5, wheel->ylabel);
// cpgsch(1.0);
// cpgmtxt("T",2.0, 0.5,wheel->title_just, wheel->title);
// cpgsch(0.75);
// cpgmtxt("T", 1.5, 0.5,wheel->title_just, wheel->subtitle);
// cpgmtxt("T", 0.5, 0.5,wheel->title_just, wheel->subsubtitle);
// cpgsch(1.0);
// 
// if (wheel->rlinlog ==PGSPECTRO_LIN )  {
// 
//    if (wheel->rnums   == YES )  strcpy(ropt,"ABCTSN"); else strcpy(ropt,"ABCTS");
// 
//    cpgtbox(ropt, 0.0, 0, ropt , 0.0, 0);  /* box after to see ticks!      */
//                                           /* Then zero axes:              */
//    cpgaxis("", 0.0, -r2, 0.0, r2, -r2, r2, 0.0, 1, 0.5, 0.5, 0.5, 1.0, 0);
//    cpgaxis("", -r2, 0.0,  r2,0.0, -r2, r2, 0.0, 1, 0.5, 0.5, 0.5, 1.0, 0);
// 
// } else if (wheel->rlinlog == PGSPECTRO_LOG ) {
//                                    /* for log r need to draw axis segments*/
//     if (wheel->rnums == YES) strcpy(ropt,"LN"); else strcpy(ropt, "L");
// /* bottom axis */
//     cpgaxis(ropt, -r2, -r2, 0.0, -r2, r2a, r1, 0.0, 1, 0.5, 0.0, 0.5, 1.0,320);
//     cpgaxis(ropt, 0.0, -r2, r2,  -r2, r1, r2a, 0.0, 1, 0.5, 0.0, 0.5, 1.0, 320);
// /* top axis */
//     cpgaxis("L" , -r2,  r2, 0.0,  r2, r2a, r1, 0.0, 1, 0.0, 0.5, 0.5, 1.0, 0);
//     cpgaxis("L" , 0.0,  r2,  r2,  r2, r1, r2a, 0.0, 1, 0.0, 0.5, 0.5, 1.0, 0);
// /* horiz zero axis */
//     cpgaxis("L" , -r2, 0.0, 0.0, 0.0, r2a, r1, 0.0, 1, 0.5, 0.5, 0.5, 1.0, 0);
//     cpgaxis("L" , 0.0, 0.0,  r2, 0.0, r1, r2a, 0.0, 1, 0.5, 0.5, 0.5, 1.0, 0);
// /* left axis */
//     cpgaxis(ropt, -r2, -r2, -r2, 0.0, r2a, r1, 0.0, 1, 0.0, 0.5, 0.5, -1.0, 40);
//     cpgaxis(ropt, -r2, 0.0, -r2,  r2, r1, r2a, 0.0, 1, 0.0, 0.5, 0.5, -1.0, 40);
// /* right axis */
//     cpgaxis("L" ,  r2, -r2,  r2, 0.0, r2a, r1, 0.0, 1, 0.5, 0.0, 0.5, 1.0, 0);
//     cpgaxis("L" ,  r2, 0.0,  r2,  r2, r1, r2a, 0.0, 1, 0.5, 0.0, 0.5, 1.0, 0);
// /* vertical zero axis */
//     cpgaxis("L" , 0.0, -r2, 0.0, 0.0, r2a, r1, 0.0, 1, 0.5, 0.5, 0.5, 1.0, 0);
//     cpgaxis("L" , 0.0, 0.0, 0.0,  r2, r1, r2a, 0.0, 1, 0.5, 0.5, 0.5, 1.0, 0);
// }
// 
// /* Draw reference circles */
// cpgsave();   /* save attributes */
// cpgsci(0);   /* circles in background colour */
// cpgsls(2); /* dashed */
// cpgsfs(2); /* outline circles */
// 
// for (i=0; i< wheel->ncircles; i++){
//    if (wheel->rlinlog == PGSPECTRO_LOG){
//                 rcirc = SafeLog10(wheel->rcircles[i]) - r1;
//    }
//    else rcirc = wheel->rcircles[i];
//    if (rcirc >= 0.0) cpgcirc(0.0, 0.0, rcirc); /* manual clipping at origin */
// }
// 
// cpgunsa(); /* restore attributes */
// 
// /* Cleanup */
// if (wheel->rlinlog == PGSPECTRO_LOG) {
//    free(rlo); free (rhi);
// }
// if (wheel->zlinlog == PGSPECTRO_LOG) free (z);
// 
// return PGSPECTRO_OK;
// }
// 
// /******************************************************************/
// void arc(float rlo, float  rhi, float thlo,float thhi){
// /******************************************************************/
// 
// /* draw a pseudo-arc with limits as given using current fill properties
// *     r in units of plot, e.g., already converted to logs if log scales
// *     th in degrees
// *
// *     present routine uses one theta at same radii in addition to limits */
// 
// float x[6], y[6];
// 
// x[0] = rlo*cos(DEG2RAD*thlo); y[0] = rlo*sin(DEG2RAD*thlo);
// x[1] = rhi*cos(DEG2RAD*thlo); y[1] = rhi*sin(DEG2RAD*thlo);
// x[5] = rlo*cos(DEG2RAD*0.5*(thlo+thhi));
//                               y[5] = rlo*sin(DEG2RAD*0.5*(thlo+thhi));
// x[2] = rhi*cos(DEG2RAD*0.5*(thlo+thhi));
//                               y[2] = rhi*sin(DEG2RAD*0.5*(thlo+thhi));
// x[4] = rlo*cos(DEG2RAD*thhi); y[4] = rlo*sin(DEG2RAD*thhi);
// x[3] = rhi*cos(DEG2RAD*thhi); y[3] = rhi*sin(DEG2RAD*thhi);
// 
// cpgpoly(6,x,y);
// 
// return;
// 
// }
// 
// /****************************************************************/
// PGSpectro_ret PGXYtrace_plot(PGXYtrace *t){
// /****************************************************************/
// /* plot a single XY trace, including (optionally) box and legend
// *  For multiple traces, make multiple calls to this routine, changing
// *  options as necessary. In particular, setting YoNbox to NO skips all
// *  axes, labels, etc. apart from in the legend */
// 
// char xopt[50],yopt[50], xlabopt[5], ylabopt[5];
// int iopt, jopt;
// float xworld1, xworld2, yworld1, yworld2;
// PLFLT *x, *y;
// float datestrcoord;
// int i;
// 
// pladv(0);
// 
// //cpgsave();  /* save pgstate and restore on return */ 
// //cpgsci(1);  /* set color to default foreground */
// plcol0(0);
// 
// /* check if either axis log; if so, calc logs of limits and data */
// if (t->xlinlog != PGSPECTRO_LOG || t->timex == YES) { /*ignore log for time */
//   x =  t->x;
//   xworld1 = t->plot_domain.xworldmin;  xworld2 = t->plot_domain.xworldmax;
// } else {
//     x = (PLFLT *)malloc (t->npts * sizeof (PLFLT) );
//     for (i=0; i<t->npts; i++) x[i] = SafeLog10(t->x[i]);
//     xworld1 = SafeLog10(t->plot_domain.xworldmin);
//     xworld2 = SafeLog10(t->plot_domain.xworldmax);
// }
// 
// if (t->ylinlog != PGSPECTRO_LOG) {
//   y =  t->y;
//   yworld1 = t->plot_domain.yworldmin;  yworld2 = t->plot_domain.yworldmax;
// } else {
//     y = (PLFLT *)malloc (t->npts * sizeof (PLFLT) );
//     for (i=0; i<t->npts; i++) y[i] = SafeLog10(t->y[i]);
//     yworld1 = SafeLog10(t->plot_domain.yworldmin);
//     yworld2 = SafeLog10(t->plot_domain.yworldmax);
// }   
// 
// /* viewport etc for plot */
// 
// // cpgsvp(t->plot_domain.xndcmin, t->plot_domain.xndcmax,
// //        t->plot_domain.yndcmin, t->plot_domain.yndcmax);
// plvpor(t->plot_domain.xndcmin, t->plot_domain.xndcmax,
//        t->plot_domain.yndcmin, t->plot_domain.yndcmax);
// 
// // cpgswin(xworld1, xworld2, yworld1, yworld2);
// plwind(xworld1, xworld2, yworld1, yworld2);
// 
// /* box and axes  - we make 2 calls to cpgbox. The first draws the box and
// *  axes and ticks, so we can control their length and thickness. The 2nd
// *  only puts the numerica labels, so these can be different size and
// *  line thickness */
// 
// if (t->YoNbox == YES){
// //    cpgslw(t->box_thickness);
// 	plwid(t->box_thickness);
// //    cpgsch(t->tick_length);
// 	plschr(40, t->tick_length);
//    strcpy(xopt,"bcts");     strcpy(yopt,"bctsv");
//    iopt = strlen(xopt);     jopt = strlen(yopt);
//    if (t->xlinlog     == PGSPECTRO_LOG && t->timex == NO) xopt[iopt++] = 'l';
//    if (t->ylinlog     == PGSPECTRO_LOG) yopt[jopt++] = 'l';
//    if (t->timex == YES){                  /* time axis labelling, etc. */
//      xopt[iopt++] = 'd';
//      pltimefmt("%H:%M");
//    }
//    xopt[iopt] = '\0';       yopt[jopt]     = '\0';   
//    plbox(xopt,0.0,0,yopt,0.0,0);  /* bare box with ticks, no tick labels */
//    plwid(t->text_line_width);
//    plschr(40., t->label_char_size); 
//    if (t->YoNXnumlab  == YES) {
//       if (t->Xnumlab_bot == NO ) {
//         xopt[iopt++] = 'm';} else {xopt[iopt++] = 'n';}
//    }
//    if (t->YoNYnumlab  == YES) {
//       if (t->Ynumlab_left == NO ) {
//         yopt[jopt++] = 'm';} else {yopt[jopt++] = 'n';}
//    }
//    xopt[iopt] = '\0';       yopt[jopt]     = '\0';
//    plbox(&(xopt[2]), 0.0, 0,&(yopt[2]), 0.0, 0); /* chop "BC" => tick lab */
// }
// 
// /* titles and labels */
// 
// if (t->YoNbox == YES){
//     plschr(40., t->title_char_size);
//     plwid(t->text_line_width);
//     plmtex("t", t->title_disp, 0.5, t->title_just, t->title);
//     plschr(40.,t->label_char_size);
//     plmtex("t", t->subtitle_disp, 0.5, t->title_just, t->subtitle);
//     plmtex("t", t->subsubtitle_disp, 0.5, t->title_just, t->subsubtitle);
//     if (t->YoNXnumlab == YES){
// 	if (t->Xtextlab_bot == NO){ strcpy(xlabopt,"t");
// 	} else { strcpy(xlabopt,"b");
// 	}
// 	plmtex(xlabopt,4.0, 0.5, 0.5, t->xlabel);  /* 2 char heights */
//     }
//     if (t->YoNYnumlab == YES){
// 	if (t->Ytextlab_left == NO){ strcpy(ylabopt,"rv");
// 	} else { strcpy(ylabopt,"lv");
// 	}
// 	plmtex(ylabopt,1.0, 1.05, 0.5, t->ylabel);
//     }
//     if (t->timex == YES){
//        datestrcoord = (xworld1<=xworld2) ? 0.0 : 1.0; /* decr time axes ok */
//        plmtex("b", 2.5, datestrcoord, 0.5*(1.0+datestrcoord), t->datestart);
//        if ( strcmp(t->datestart,t->dateend) ){
// 	 plmtex("b", 2.5, 1.0-datestrcoord,1.0-0.5*datestrcoord,t->dateend);
//        }
//     }
// }
// 
// /* data - this is the easy bit! */
// 
// plwid(t->line_width);
// pllsty(t->linestyle);
// plcol0(t->colour);
// plschr(40., t->symbol_size);
// if (t->YoNline == YES)plline(t->npts, x, y);
// if (t->YoNsymbol == YES) plpoin(t->npts, x, y, t->symbol);
// 
// /* legend */
// 
// if (t->YoNlegend == YES) PGXYtrace_legend(t);
// 
// if (t->xlinlog == PGSPECTRO_LOG && t->timex == NO) free(x);
// if (t->ylinlog == PGSPECTRO_LOG) free(y);
// // cpgunsa();  /* restore state before call */
// return PGSPECTRO_OK;
// }
// 
// /*******************************************************************/
// PGSpectro_ret PGXYtrace_legend(PGXYtrace *t){
// /*******************************************************************/
// /* routine to draw the legend for an XY trace plot */
// 
// float x[2], y[2], marg, len, xch, ych;
// 
// cpgsave();
// 
// cpgsci(1);; /* black */
// cpgsls(QPGSOLID);
// cpgsch(t->legend_text_char_size);
// 
// cpgsvp(t->legend_domain.xndcmin, t->legend_domain.xndcmax,
//        t->legend_domain.yndcmin, t->legend_domain.yndcmax);
// 
// cpgswin(0.0, 1.0,0.0, 1.0);
// 
// if (t->YoNlegendbox == YES) {
//    cpgslw(t->box_thickness);
//    cpgbox("BC",0.0,0,"BC",0.0,0);
// }
// 
// y[0]= y[1] = t->legend_just;
// len = t->legend_line_length_frac;
// marg = 0.1*len;
// x[0] = marg;  x[1]=marg+len;
// 
// cpgslw(t->line_width);
// cpgsls(t->linestyle);
// cpgsci(t->colour);
// if (t->YoNline == YES)cpgline(2, x, y);
// if (t->YoNsymbol == YES) {
//   cpgsch(t->symbol_size);
//   cpgpt1( 0.5*(x[0]+x[1]), y[0], t->symbol);
// }
// cpgsci(1);
// cpgslw(t->text_line_width);
// cpgsch(t->legend_text_char_size);
// cpgqcs(4,&xch,&ych);    /* get ch height in units of world coords */
// cpgptxt(x[1]+marg, y[1]- 0.4*ych, 0.0, 0.0, t->legend_text);
// 
// cpgunsa();
// return PGSPECTRO_OK;
// }
// 
// /********************************************************************/
// PGSpectro_ret PGXYtrace_set_defaults(PGXYtrace *trace){
// /********************************************************************/
// /* set defaults for as much as possible. Should be safe enough so that
// *  will not crash if nothing is set other than data */
// 
// float ndcwidth;   /* will need this to make room for legend */
// 
// trace->npts=0;
// PGSpectro_domain_set_defaults(&(trace->plot_domain));
// PGSpectro_domain_set_defaults(&(trace->legend_domain));
// ndcwidth =trace->plot_domain.xndcmax - trace->plot_domain.xndcmin;
// trace->plot_domain.xndcmax = trace->plot_domain.xndcmin + 0.6*ndcwidth;
// trace->legend_domain.xndcmin = trace->plot_domain.xndcmax + 0.1*ndcwidth;
// 
// trace->colour = QPGBLACK;
// trace->linestyle = QPGSOLID;
// trace->line_width = 1;
// trace->YoNline = YES;
// trace->YoNsymbol = NO;
// trace->symbol = QPGFILLEDCIRCLE;
// trace->symbol_size = 1;
// 
// trace->subtitle = " ";
// trace->subsubtitle = " ";
// trace->xlabel = " ";
// trace->ylabel = " ";
// trace->title = " ";
// trace->Xtextlab_bot = YES;
// trace->Ytextlab_left = YES;
// trace->title_just = 0.5;
// trace->title_disp = 2.0;
// trace->subtitle_disp = 1.5;
// trace->subsubtitle_disp = 0.5;
// 
// trace->label_char_size = 1.0;
// trace->title_char_size = 1.5;
// trace->text_line_width = 1;     /* cpgslw expects an int */
// 
// trace->YoNXnumlab = YES;
// trace->YoNYnumlab = YES;
// trace->Ynumlab_left = YES;
// trace->Xnumlab_bot = YES;
// trace->YoNbox = YES;
// trace->xlinlog = PGSPECTRO_LIN;
// trace->ylinlog = PGSPECTRO_LIN;
// trace->tick_length = 1;
// trace->box_thickness = 1;       /* cpgslw expects an int */
// 
// trace->timex = NO;
// trace->datestart = " ";
// trace->dateend   = " ";
// 
// trace->legend_text_char_size = 0.7;
// trace->YoNlegend = YES;
// trace->YoNlegendbox = YES;
// trace->legend_just  = 0.5;
// trace->legend_line_length_frac = 0.2;
// trace->legend_text = " ";
// 
// return PGSPECTRO_OK;
// }
// 
// /*****************************************************************/
// double SafeLog10 (double x) {  /* from QppLog10 in qpp_panel.cc */
// /*****************************************************************/
//   return (x < 1.0e-33) ? -33.0 : log10(x);
// }
// 
// #ifdef __cplusplus
// }
// #endif
// 
// 
// /* end of PGSpectro.c file */
