/* featherPlot.cc Jeremy Mitchell 10/08/2009 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "qplug_if.h"
#include "qdutil.h"
#include "Qdos.h"
#include "qar.h"
#include "Xrefs.h"

extern "C" QplugReturnStatus featherPlot(QplugArgList *);

using namespace QSAS;

QplugReturnStatus featherPlot(QplugArgList *call_list) {
  // Unpack input data
  
  QdObject_var pos1_in   = (* call_list)[0];
  QdObject_var deltax_in = (* call_list)[1];
  QdObject_var scale_in  = (* call_list)[2];
  QdObject_var step_in  = (* call_list)[3];
  QdObject_var boxCar_in  = (* call_list)[4];
  QdObject_var head_in   = (* call_list)[5];
  QdObject_var hScale_in   = (* call_list)[6];
  
  //********************** Test Inputs ***********************************
  
  QdRMatrixSeq_var pos1_qd = QdRMatrixSeq_var::narrow(pos1_in);
  if (pos1_qd.is_nil()) {
    QplugAppendTextDisplay("featherPlot error: Position 1 is not a Matrix\n");
    return QPLUG_FAILURE;
  }
  if (!pos1_in->xref_exists(FRAME)) {
    QplugAppendTextDisplay("featherPlot error: Position 1 is not a Vector\n");
    return QPLUG_FAILURE;
  }
  
  bool dxseries = true;
  
  QdRMatrixSeq_var deltax_qd = QdRMatrixSeq_var::narrow(deltax_in);
  QdRMatrix_var deltax_single_qd = QdRMatrix_var::narrow(deltax_in);
  if (deltax_qd.is_nil()) {
    if (deltax_single_qd.is_nil()) {
      QplugAppendTextDisplay("featherPlot error: delta x is not a Matrix\n");
      return QPLUG_FAILURE;
    } else {
      dxseries = false;
      QplugAppendTextDisplay("featherPlot: Delta X is not a series, all arrows the same\n");
    }
  }
  if (!deltax_in->xref_exists(FRAME)) {
    QplugAppendTextDisplay("featherPlot error:delta x is not a Vector\n");
    return QPLUG_FAILURE;
  }
  
  QdRScalar_var scale_qd = QdRScalar_var::narrow(scale_in);
  if (scale_qd.is_nil()) {
    QplugAppendTextDisplay("featherPlot error: Scale is not a Scalar\n");
    return QPLUG_FAILURE;
  }
  
  QdRScalar_var step_qd = QdRScalar_var::narrow(step_in);
  if (step_qd.is_nil()) {
    QplugAppendTextDisplay("featherPlot error: Step is not a Scalar\n");
    return QPLUG_FAILURE;
  }
  int step = (int)(* step_qd);
  if (step <= 0) {
    QplugAppendTextDisplay("featherPlot error: step must be > 0\n");
    return QPLUG_FAILURE;
  }
  QdRScalar_var boxCar_qd = QdRScalar_var::narrow(boxCar_in);
  if (step_qd.is_nil()) {
    QplugAppendTextDisplay("featherPlot error: Step is not a Scalar\n");
    return QPLUG_FAILURE;
  }
  int boxCar = (int)(* boxCar_qd);
  if (boxCar < 0) {
    QplugAppendTextDisplay("featherPlot error: boxCar must be >= 0\n");
    return QPLUG_FAILURE;
  }
  
  QdRScalar_var head_qd = QdRScalar_var::narrow(head_in);
  if (head_qd.is_nil()) {
    QplugAppendTextDisplay("featherPlot error: Draw Head is not a Scalar\n");
    return QPLUG_FAILURE;
  }
  int head = (int)(* head_qd);
  int arrowLength = 7;
  if ((head < 0) || (head > 3)) {
    QplugAppendTextDisplay("featherPlot error: Draw Head must be between 0 and 3\n");
    return QPLUG_FAILURE;
  }
  if (head == 0)
    arrowLength = 3;
  
  QdRScalar_var hScale_qd = QdRScalar_var::narrow(hScale_in);
  if (hScale_qd.is_nil()) {
    QplugAppendTextDisplay("featherPlot error: Width is not a Scalar\n");
    return QPLUG_FAILURE;
  }
  float hScale = (float)(* hScale_qd);
  if ((hScale < 0)) {
    QplugAppendTextDisplay("featherPlot WARNING: Width is less than 0\n\tArrow may point wrong way!");
  }
  
  //***************** step stuff ***************************
  
  // DETERMINE INFORMATION ABOUT THE TIME STEPS
  
  bool timeStep = false;
  
  // Determine if step is in space or time:
  //   IF step UNITS = seconds AND pos1 has time_tags, then TIME step
  //   First, check units of Step:
  //   NB!!! IF timeStep = true, but arrows have no time_tags, then only consider the first arrow
  if (step_in->xref_exists(UNITS)) {
    char stepUnits[20];
    strcpy(stepUnits, QuGetAttrText(UNITS, step_in).c_str() );
    
    // if step not in seconds, then space step
    if ( (strcmp("s", stepUnits)) ||  (strcmp("S", stepUnits)) || (strncmp("sec", stepUnits, 3)) ||
      (strncmp("Sec", stepUnits, 3)) || (strncmp("SEC", stepUnits, 3)) ) {
      
      // check pos1 for time_tags
      timeStep = true;
      if ( /*(!pos1_in->xref_exists(TIME_TAGS)) && */(!pos1_in->xref_exists(DEPEND_0)) ) {
	QplugAppendTextDisplay("featherPlot Warning: Pos1 has no TIME_TAGS/DEPEND_0 => Spatial Step!\n");
	timeStep = false;
      }
      // IF timeStep = true, but arrows have no time_tags, then only consider the first arrow
      if ( /*(!deltax_in->xref_exists(TIME_TAGS)) && */(!deltax_in->xref_exists(DEPEND_0)) && (dxseries) ) {
        QplugAppendTextDisplay("featherPlot WARNING: Delta X has no TIME_TAGS/DEPEND_0 => All arrows the same!\n");
	dxseries = false;
	(* deltax_single_qd)[0] = deltax_qd[0][0];
	(* deltax_single_qd)[1] = deltax_qd[0][1];
	(* deltax_single_qd)[2] = deltax_qd[0][2];
      }
    }
  }
  
  //***************** Find Drawn Arrow positions ***************************
  
  int nout,nStep;		// sized of final array and number of drawn arrows, respectively
  QdRMatrixSeq_var outArr;	// output array, with arrows
  QdRMatrixSeq_var arrowArr;	// array with the vectors showing arrows that will be drawn
  QdTimeSeq_var ttout;		// time_tags for output array (if applicable)
  int initNstep = pos1_qd->sequence_size();	// size of initial pos1 array
  int *arrowPlace;	// array containing position of drawn arrows are in outArr
  

  if ( !timeStep ) {
    // SPATIAL TIMESTEP STUFF
    //-----------------------
    
    QplugAppendTextDisplay("featherPlot: Spatial steps chosen");
    if (dxseries) // for spatial series, pos1 and deltax must be same length (unless only one arrow is used)
      if (initNstep != deltax_qd->sequence_size()) {
        QplugAppendTextDisplay("featherPlot error: pos1 and delta x series not the same size!\n");
        return QPLUG_FAILURE;
      }
    
    float* path = new float[initNstep];
    path[0] = 0.;
    for (int i=1; i < initNstep; i++) {
      float px = (float)(pos1_qd[i][0]) - (float)(pos1_qd[i-1][0]);
      float py = (float)(pos1_qd[i][1]) - (float)(pos1_qd[i-1][1]);
      float pz = (float)(pos1_qd[i][2]) - (float)(pos1_qd[i-1][2]);
      path[i] = path[i-1] + sqrt(px*px + py*py + pz*pz);
    }
    
    nStep = (int)( floorf(path[initNstep-1]/step) );
    nout = initNstep + arrowLength * nStep;
    
    outArr = new QdRMatrixSeq(nout, Dim(3));
    arrowArr = new QdRMatrixSeq(nStep, Dim(3));
    
    arrowPlace = new int[nStep];
    
    int k=0;
    for (int i=0; i<nStep; i++) {
      float pos = step * i;
      
      while (path[k+1] < pos) {
    	outArr[k + i*arrowLength][0] = pos1_qd[k][0];
    	outArr[k + i*arrowLength][1] = pos1_qd[k][1];
    	outArr[k + i*arrowLength][2] = pos1_qd[k][2];
    	k++;
		if(k >= initNstep) {
        	QplugAppendTextDisplay("pos error\n");
        	return QPLUG_FAILURE;
		}
      }
      
      arrowPlace[i] = k + i*arrowLength;
      float wt1 = (pos - path[k])/(path[k+1] - path[k]), wt2 = (pos - path[k+1])/(path[k] - path[k+1]);
      outArr[k + i*arrowLength][0] = wt1*(float)(pos1_qd[k][0]) + wt2*(float)(pos1_qd[k+1][0]);
      outArr[k + i*arrowLength][1] = wt1*(float)(pos1_qd[k][1]) + wt2*(float)(pos1_qd[k+1][1]);
      outArr[k + i*arrowLength][2] = wt1*(float)(pos1_qd[k][2]) + wt2*(float)(pos1_qd[k+1][2]);
     
      if ( !dxseries ) {
    	// all arrows the same
		arrowArr[i][0] = (* deltax_single_qd)[0];
    	arrowArr[i][1] = (* deltax_single_qd)[1];
    	arrowArr[i][2] = (* deltax_single_qd)[2];
      } else if ( boxCar == 0) {
        // Linear interpolation of arrows
		arrowArr[i][0] = wt1*(float)(deltax_qd[k][0]) + wt2*(float)(deltax_qd[k+1][0]);
        arrowArr[i][1] = wt1*(float)(deltax_qd[k][1]) + wt2*(float)(deltax_qd[k+1][1]);
        arrowArr[i][2] = wt1*(float)(deltax_qd[k][2]) + wt2*(float)(deltax_qd[k+1][2]);
      } else {
        // Boxcar average of arrows
	int l=k, kmin=k, kmax=k;
	while ((pos - path[l] <= boxCar/2.) && (l >= 0)) l--;
	kmin = l+1; l = k;
	while ((path[l] - pos <= boxCar/2.) && (l <= initNstep)) l++;
	kmax = l-1;
	
	arrowArr[i][0] = 0.; arrowArr[i][1] = 0.; arrowArr[i][2] = 0.;
	for (l=kmin; l <= kmax; l++) {
	  arrowArr[i][0] += deltax_qd[l][0] / (kmax - kmin + 1);
	  arrowArr[i][1] += deltax_qd[l][1] / (kmax - kmin + 1);
	  arrowArr[i][2] += deltax_qd[l][2] / (kmax - kmin + 1);
	}
      }
    }

    for (int m=k + nStep*arrowLength; m < nout; m++) {
      outArr[m][0] = pos1_qd[m - nStep*arrowLength][0];
      outArr[m][1] = pos1_qd[m - nStep*arrowLength][1];
      outArr[m][2] = pos1_qd[m - nStep*arrowLength][2];
    }
    delete [] path;

  } else {
    // timestep is true:
    
    QplugAppendTextDisplay("feathPlot: Time steps chosen");
    QdTimeSeq_var ttin = get_timetags(pos1_in);
    QmjdTime tt0 = (* ttin)[0];
    QmjdTime ttf = (* ttin)[initNstep-1];
    
    nStep = (int)( floorf((ttf.getDiffSecs(tt0)) / step) );
    nout = initNstep + arrowLength * nStep;
    
    outArr = new QdRMatrixSeq(nout, Dim(3));
    arrowArr = new QdRMatrixSeq(nStep, Dim(3));
    
    arrowPlace = new int[nStep];
    
    QdTimeSeq_var ttjoin = new QdTimeSeq(nStep);
    ttout = new QdTimeSeq(nout);
    for (int i=0; i<nStep; i++)
      ttjoin[i] = ttin[0] + i * step;
    
	QdRMatrixSeq_var outTmp;
    if (boxCar == 0) {
		outTmp = QdRMatrixSeq_var::narrow( QuLinearJoin(pos1_in, ttjoin, step+10., string(ZERO_FILL_GAP)) );
    } else {
		outTmp = QdRMatrixSeq_var::narrow( QuBoxcarJoin(pos1_in, ttjoin, boxCar, string(ZERO_FILL_GAP)) );
    }
    
    
    if (dxseries) {
	    if (boxCar == 0) {
			arrowArr = QdRMatrixSeq_var::narrow( QuLinearJoin(deltax_in, ttjoin, step+10., string(ZERO_FILL_GAP)) );
	    } else {
			arrowArr = QdRMatrixSeq_var::narrow( QuBoxcarJoin(deltax_in, ttjoin, boxCar, string(ZERO_FILL_GAP)) );
	    }
	
    } else for (int i=0; i<nStep; i++) {
      arrowArr[i][0] = (* deltax_single_qd)[0];
      arrowArr[i][1] = (* deltax_single_qd)[1];
      arrowArr[i][2] = (* deltax_single_qd)[2];
    }
   
    int k=0,i=0;
    while( i < initNstep) {
      if ( (ttin[i].getDiffSecs(tt0) <= step*k) || (k == nStep) ) {
	outArr[i+k*arrowLength][0] = pos1_qd[i][0];
        outArr[i+k*arrowLength][1] = pos1_qd[i][1];
        outArr[i+k*arrowLength][2] = pos1_qd[i][2];
	ttout[i+k*arrowLength] = ttin[i];
	i++;
      } else if (k < nStep) {
	outArr[i+k*arrowLength][0] = outTmp[k][0];
        outArr[i+k*arrowLength][1] = outTmp[k][1];
        outArr[i+k*arrowLength][2] = outTmp[k][2];
	arrowPlace[k] = i+k*arrowLength;
	for (int m=0; m < arrowLength; m++)
	  ttout[i+k*arrowLength+m] = ttjoin[k];
	//i += arrowLength;
	k++;
      }
    }
  }
      QplugAppendTextDisplay("do arrow\n");

  //******************** calculate *************************
  
  for (int k = 0; k < nStep; k++) {
    
    int api = arrowPlace[k];
    
    float delx = (float)(arrowArr[k][0]) * (float)(* scale_qd);
    float dely = (float)(arrowArr[k][1]) * (float)(* scale_qd);
    float delz = (float)(arrowArr[k][2]) * (float)(* scale_qd);
    outArr[api+1][0] = (float)(outArr[api][0]) + delx;
    outArr[api+1][1] = (float)(outArr[api][1]) + dely;
    outArr[api+1][2] = (float)(outArr[api][2]) + delz;
    
    QdRScalarSeq_var dep0;
    
    if (head != 0) {
      float phi, xarr, yarr, zarr, width;
      
      switch (head) {
    	case 1:
    	  width = hScale*sqrt(delx*delx + dely*dely);
    	  phi = atan2(dely, delx);
    	  xarr = -sin(phi)*width;
    	  yarr = cos(phi)*width;
  	  zarr = 0;
  	  break;
    	case 2:
    	  width = hScale*sqrt(delx*delx + delz*delz);
    	  phi = atan2(delz, delx);
    	  xarr = -sin(phi)*width;
    	  yarr = 0;
  	  zarr = cos(phi)*width;
  	  break;
    	case 3:
    	  width = hScale*sqrt(dely*dely + delz*delz);
    	  phi = atan2(dely, delz);
    	  xarr = 0;
    	  yarr = cos(phi)*width;
  	  zarr = -sin(phi)*width;
  	  break;
      }
      
      float onemw = 1. - hScale;
      outArr[api+2][0] = (float)(outArr[api][0]) + onemw*delx + xarr;
      outArr[api+2][1] = (float)(outArr[api][1]) + onemw*dely + yarr;
      outArr[api+2][2] = (float)(outArr[api][2]) + onemw*delz + zarr;
      
      outArr[api+3][0] = outArr[api+1][0];
      outArr[api+3][1] = outArr[api+1][1];
      outArr[api+3][2] = outArr[api+1][2];
      
      outArr[api+4][0] = (float)(outArr[api][0]) + onemw*delx - xarr;
      outArr[api+4][1] = (float)(outArr[api][1]) + onemw*dely - yarr;
      outArr[api+4][2] = (float)(outArr[api][2]) + onemw*delz - zarr;
      
      outArr[api+5][0] = outArr[api+1][0];
      outArr[api+5][1] = outArr[api+1][1];
      outArr[api+5][2] = outArr[api+1][2];
      
      outArr[api+6][0] = outArr[api+0][0];
      outArr[api+6][1] = outArr[api+0][1];
      outArr[api+6][2] = outArr[api+0][2];
    } else {
      outArr[api+2][0] = outArr[api+0][0];
      outArr[api+2][1] = outArr[api+0][1];
      outArr[api+2][2] = outArr[api+0][2];
    }
  }  
  QdRScalarSeq_var dep0 = new QdRScalarSeq(nout);
  int d0=0;
  if (head == 3)
    d0 = 1;
  for (int i=0; i<nout; i++) dep0[i] = outArr[i][d0];
  
  // Add attributes:
  
  QdObject_var outArr_out = (QdObject_var) outArr;
  QdObject_var dep0_out = (QdObject_var) dep0;
  
  outArr_out->copy_xrefs_from(pos1_in);
  dep0_out->copy_xrefs_from(pos1_in);
  // Narrow type to a scalar
  if (dep0_out->xref_exists("TIME_TAGS")) {
    dep0_out->delete_xref("TIME_TAGS");
    outArr_out->delete_xref("TIME_TAGS");
  }
  if (dep0_out->xref_exists(DEPEND_0)) {
      dep0_out->delete_xref(DEPEND_0);
      outArr_out->delete_xref(DEPEND_0);
  }
  if (dep0_out->xref_exists(DEPEND_0)) {
    dep0_out->delete_xref(DEPEND_0);
    outArr->delete_xref(DEPEND_0);
    QplugAppendTextDisplay("Deleting arrow DEPEND_0\n");
  }
  if (dep0_out->xref_exists("TIME_TAGS")) {
    dep0_out->delete_xref("TIME_TAGS");
    outArr->delete_xref("TIME_TAGS");
    QplugAppendTextDisplay("Deleting arrow TIME_TAGS\n");
  }
  if (timeStep) {
    set_timetags(outArr, ttout);
    set_timetags(dep0_out, ttout);
    outArr->set_xref("DEPEND_0", ttout);
    dep0_out->set_xref("DEPEND_0", ttout);
  }
  
  if (dep0_out->xref_exists(DEPEND_1)) {
    dep0_out->delete_xref(DEPEND_1);
    outArr_out->delete_xref(DEPEND_1);
    QplugAppendTextDisplay("Deleting outArr DEPEND_1\n");
  }
  QuSetTxtAttr(FRAME, "scalar>na", dep0_out);
  outArr->set_xref("DEPEND_1", dep0_out);
  
    
  //***********************************************************
  
  // Assign data to output:
  
  delete [] arrowPlace;
    
  call_list->push_back( outArr_out );
  
  QplugAppendTextDisplay ("featherPlot>> finished...\n");
  
  return QPLUG_SUCCESS;
}
