// -----------------------------------------------------------------------------------------
//
// Performs a nested generic residue analysis
//
// SEH, Sep 16, 2003
// 
// Inputs :
//     B = magnetic field   (n x 3) 
//     V = plasma velocity  (n x 3) 
//     E = electric field   (n x 3)
//     P = pressure tensor  (n x 3 x 3)
//     Q = heat flux vec    (n x 3)
//     N = density          (n)
//     nestsize = number of points in data sets
//     min_nest = minimum nest size
//     constrain = 1 for Bn=0, 2 for (<V> - U)n etc - see below.
//     alpha = slope of walen correlation coeff (used for Alfvenic constrain)
//
// SEH, 16 Sep 2003
//      14 May 2005, added alpha - the Walenslope for constraining
//      31 May 2006, added MVA dB/dt
//
// -----------------------------------------------------------------------------------------

#include "GenericMinVar.h"
#include "GenericResidue.h"

int NestedAnalysis(QdRMatrixSeq_var V_, QdRMatrixSeq_var B_, QdRMatrixSeq_var dB_, QdRMatrixSeq_var P_, 
		   QdRMatrixSeq_var E_, QdRScalarSeq_var N_, QdRMatrixSeq_var H_, 
		   QdTimeSeq_var ttJoined,  QdRMatrix_var ExWeights,
		   int constrained, double alpha, int min_nest, char *SCc_ID,
		   bool doType[10],int nestsize)
{

  int i,j,k,nloops=0;
  QdRMatrixSeq_var  B = new QdRMatrixSeq();                                       // B = Magnetic field series
  QdRMatrixSeq_var  dB = new QdRMatrixSeq();                                      // dB = d/dt magnetic field series
  QdRMatrixSeq_var  V = new QdRMatrixSeq();                                       // V = Velocity series
  QdRMatrixSeq_var  E = new QdRMatrixSeq();                                       // E = Electric field
  QdRMatrixSeq_var  P = new QdRMatrixSeq();                                       // P = Pressure tensor
  QdRMatrixSeq_var  H = new QdRMatrixSeq();                                       // H = heat flux
  QdRScalarSeq_var  N = new QdRScalarSeq();                                       // N = Plasma density

  QdRMatrix_var ProjMx  = new QdRMatrix(Dim(3,3), 0.0);
  QdRMatrix_var Q  = new QdRMatrix(Dim(3,3), 0.0);
  QdRMatrix_var Q_COM  = new QdRMatrix(Dim(3,3), 0.0);
  QdRMatrix_var Evec = new QdRMatrix(Dim(3,3), 0.0);  
  QdRMatrix_var EvecMVAB = new QdRMatrix(Dim(3,3), 0.0);     
  QdRMatrix_var Evalues = new QdRMatrix(Dim(3), 0.0);
  QdRMatrix_var Dummy =  new QdRMatrix(Dim(3), 0.0);  
  QdRMatrix_var U = new QdRMatrix(Dim(3), 0.0);        
  QdTimeSeq_var tt_out=new QdTimeSeq();                                           
  char buf[255];
  double un=0.0, ucis=0.0, deltat=0.0,normfac;

  QuIsTSRegular(ttJoined,&deltat);

  int max_numnest = (nestsize - min_nest + 3) / 2;                               // number of nest to study

  printf("Nested : nestsize = %d\n",nestsize);
  printf("Nested : min_nest = %d\n",min_nest);
  printf("Nested : n loopse = %d\n",max_numnest);
  fflush(stdout);

  QDateTime dt = QDateTime::currentDateTime();
  QdRMatrixSeq_var q_mvab  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3,3), 0.00));
  QdRMatrixSeq_var q_mvadb  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3,3), 0.00));
  QdRMatrixSeq_var q_mfr  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3,3), 0.00));
  QdRMatrixSeq_var q_mmr  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3,3), 0.00));
  QdRMatrixSeq_var q_mer  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3,3), 0.00));
  QdRMatrixSeq_var q_mter  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3,3), 0.00));
  QdRMatrixSeq_var q_mlmr  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3,3), 0.00));
  QdRMatrixSeq_var q_mvav  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3,3), 0.00));
  QdRMatrixSeq_var q_com  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3,3), 0.00));

  QdRMatrixSeq_var U_mva  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));
  QdRMatrixSeq_var U_mvadb  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));
  QdRMatrixSeq_var U_mmr  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));
  QdRMatrixSeq_var U_mfr  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));
  QdRMatrixSeq_var U_mer  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));
  QdRMatrixSeq_var U_mter  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));
  QdRMatrixSeq_var U_mlmr  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));
  QdRMatrixSeq_var U_mvav  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));
  QdRMatrixSeq_var U_com  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));

  QdRMatrixSeq_var B_nestave  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));
  QdRMatrixSeq_var V_nestave  = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));
  QdRMatrixSeq_var Va_nestave = new QdRMatrixSeq(max_numnest,RMatrix(Dim(3), 0.00));
  QdRMatrix_var B_doubleave = new QdRMatrix(Dim(3), 0.0);
  QdRMatrix_var V_doubleave = new QdRMatrix(Dim(3), 0.0);
  QdRMatrix_var Va_doubleave = new QdRMatrix(Dim(3), 0.0);

  QdRMatrixSeq_var Valf; //  this is initialized in Q_MVAB_calc

  //  ------ loop over nestsize ------

  QdTime t_start = (*ttJoined)[0];
  QdTime t_stop = (*ttJoined)[nestsize-1];
  QdTime t_center = t_start + (t_stop - t_start) / 2;
  double wl=0.0, cc=0.0, weight_k=0.0, weight_sum=0.0;

  sprintf(buf,"#Date performed : %s\n#",dt.toString().toLatin1().data());
  QplugAppendTextDisplay(buf);
  sprintf(buf,"#nloops = %2d  initial t = %s <-- %s --> %s (%2d samples)\n",
	    max_numnest, t_start.iso_srep().c_str()+11,t_center.iso_srep().c_str()+11,t_stop.iso_srep().c_str()+11,nestsize);
  QplugAppendTextDisplay(buf);



  //  while (nestsize > min_nest) {
  for (nloops=0; nloops<max_numnest; nloops++) {
    SequenceMask msk(ttJoined->get_mask(QdTimeSeq::InRange(t_start, t_stop)));
    ttJoined->get_subsequence(*tt_out, msk);
       

    sprintf(buf,"\n\n#loop = %2d of %2d  t = %s <-- %s --> %s (%2d samples)\n",
	    nloops+1,max_numnest, t_start.iso_srep().c_str()+11,t_center.iso_srep().c_str()+11,t_stop.iso_srep().c_str()+11,nestsize);
    printf("%s",buf);
	fflush(stdout);


    // ---- do MVAB to get the eigenvectors from unconstrained MVAB ------

    bool doV = !V_.is_nil();
    bool doN = !N_.is_nil();

    if (doV) V_->get_subsequence(*V,msk);
    if (doN) N_->get_subsequence(*N,msk);

    if (doType[0]) {
      printf("doing initial MVAB... doV = %d doN = %d\n",(int)doV, (int)doN);
	  fflush(stdout);
      B_->get_subsequence(*B,msk);
      nestsize = B->sequence_size();
      Valf = new QdRMatrixSeq(nestsize,RMatrix(Dim(3), 0.0));  
      Q = Q_MVAB_calc(B,V,U,Valf,N, &wl, &cc, nestsize, FALSE);
      try{
	GenericMinVar minvar(0,*Q,*ProjMx);                                               // unconstrained
	*EvecMVAB = minvar.eigenvectors().duplicate(SHALLOW);
      }
      catch(Exception& e) {
	printf("Exception in Nested Generic, initial MVAB\n");
	return 1;
      }
      printf("Initial MVAB done..\n");
	  fflush(stdout);
    }
    


    // ---- 0,2 : do MVAB (HT if doType==2) with selected constrain ------

    if (doType[0]) {
      printf("doing MVAB..\n");
	  fflush(stdout);
      Q = Q_MVAB_calc(B,V,U,Valf,N, &wl, &cc,nestsize, doType[2]);
      ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained);

      try{
	GenericMinVar minvar(constrained,*Q,*ProjMx);  
	*Evec = minvar.eigenvectors().duplicate(SHALLOW);
	*Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
      }
      catch(Exception& e) {
	printf("exception in NestedGeneric L123\n");
	return 1; 
      }
      printf(" in nested MVAB\n");
	  fflush(stdout);

      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*q_mvab)[nloops][i][j] = (*Evec)[0][i] * (*Evec)[0][j];
	}
	(*U_mva)[nloops][i] = (*U)[i];
      }

      weight_k =  (*ExWeights)[0]  * weight_k_calc(Evalues);
      printf("weight_MVAB = %10.4f\n",weight_k);
	  fflush(stdout);
      weight_sum = weight_k; 
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*Q_COM)[i][j] = weight_k * (double)(*Q)[i][j] / ( (double)(*Q)[0][0] +  (double)(*Q)[1][1] + (double)(*Q)[2][2]);         // NB not += here 
	}
	(*U_com)[nloops][i] += weight_k * (double) (*U)[i];
      }

 
      // ---- do the double-averages -----------------

      printf("double avg...%d  %d \n", V->sequence_size(), B->sequence_size());
	  fflush(stdout);
      try{
	for (k=0; k<nestsize; k++) {
	  for (i=0; i<3; i++) {      
	    (*B_nestave)[nloops][i] += (double)(*B)[k][i] / (double) nestsize;;
	  }
	}

      } catch(Exception& e) {
	printf("exception in NestedGeneric L123\n");
	return 1; 
      }
      printf("MVAB done...\n");
	  fflush(stdout);
    }



    // ------- 1 : do MVAdB/dt with this constrain --------

    if (doType[1]) {
      printf("doing MVABdBdt..\n");
	  fflush(stdout);
      dB_->get_subsequence(*dB,msk);
      QdRMatrixSeq_var DummyValf = new QdRMatrixSeq(nestsize,RMatrix(Dim(3), 0.0));  
      
      Q = Q_MVAB_calc(dB,V,U,DummyValf,N, &wl, &cc,nestsize,doType[2]);
      ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained);
      
      try{
	GenericMinVar minvar(constrained,*Q,*ProjMx);  
	*Evec = minvar.eigenvectors().duplicate(SHALLOW);
	*Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
      }
      catch(Exception& e) {
	printf("exception in NestedGeneric for MVA dB/dt\n");
	return 1; 
      }
      
      
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*q_mvadb)[nloops][i][j] = (*Evec)[0][i] * (*Evec)[0][j];
	}
	(*U_mvadb)[nloops][i] = (*U)[i];
      }
      
      weight_k =  (*ExWeights)[1]  * weight_k_calc(Evalues);
      printf("weight_MVAB = %10.4f\n",weight_k);
	  fflush(stdout);
      weight_sum = weight_k; 
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*Q_COM)[i][j] = weight_k * (double)(*Q)[i][j] / ( (double)(*Q)[0][0] +  (double)(*Q)[1][1] + (double)(*Q)[2][2]);         // NB not += here 
	}
	(*U_com)[nloops][i] += weight_k * (double) (*U)[i];
      }

    }


    // ------- 3: do MFR with this constrain ----

    if (doType[3]) {
      printf("Doing MFR...\n");
	  fflush(stdout);
      V_->get_subsequence(*V,msk);
      Q = Q_MFR_calc(B,V,U,nestsize);
      ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize,constrained);
      try{
	GenericMinVar minvar(constrained,*Q,*ProjMx);  
	*Evec = minvar.eigenvectors().duplicate(SHALLOW);
	*Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
      }
      catch(Exception& e) {
	printf("exception in NestedGeneric L123\n");
	return 1; 
      }
 
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*q_mfr)[nloops][i][j] = (*Evec)[0][i] * (*Evec)[0][j];
	}
	(*U_mfr)[nloops][i] = (*U)[i];
      }

      weight_k =  (*ExWeights)[2]  * weight_k_calc(Evalues);
      printf("weight_MFR = %10.4f\n",weight_k);
	  fflush(stdout);
      weight_sum += weight_k;
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*Q_COM)[i][j] += weight_k * (double)(*Q)[i][j] / ( (double)(*Q)[0][0] +  (double)(*Q)[1][1] + (double)(*Q)[2][2]);
	}
	(*U_com)[nloops][i] += weight_k * (double) (*U)[i];
      }
    }



    // ---- 4 : do MMR with selected constrain ------
 
    if (doType[4]) {
      Q = Q_MMR_calc(V, N, U, nestsize);
      ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained);
      try{
	GenericMinVar minvar(constrained,*Q,*ProjMx);  
	*Evec = minvar.eigenvectors().duplicate(SHALLOW);
	*Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
      }
      catch(Exception& e)  {return 1; }
    
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*q_mmr)[nloops][i][j] = (*Evec)[0][i] * (*Evec)[0][j];
	}
	(*U_mmr)[nloops][i] = (*U)[i];
      }

      weight_k =  (*ExWeights)[3]  * weight_k_calc(Evalues);
      printf("weight_MMR = %10.4f\n",weight_k);
	  fflush(stdout);
      weight_sum += weight_k;
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*Q_COM)[i][j] += weight_k * (double)(*Q)[i][j] / ( (double)(*Q)[0][0] +  (double)(*Q)[1][1] + (double)(*Q)[2][2]);
	}
	(*U_com)[nloops][i] += weight_k * (double) (*U)[i];
      }

      // ---- do the double-averages for N (Va) -----------------

      try{
	for (k=0; k<nestsize; k++) {
	  for (i=0; i<3; i++) {      
	    (*Va_nestave)[nloops][i] += (alpha*(double)(*Valf)[k][i] / (double) nestsize);
	  }
	}
      } catch(Exception& e) {
	printf("exception in NestedGeneric double avg MMR\n");
	return 1; 
      }
    }

  

    

    // ---- 5: do MLMR with selected constrain ------

    if (doType[5]) {
      printf("doing MLMR..\n");
	  fflush(stdout);
      P_->get_subsequence(*P,msk);
      Q =  Q_MLMR_calc(V,B,P,N,U,nestsize);
      ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained);
      try{
	GenericMinVar minvar(constrained,*Q,*ProjMx);  
	*Evec = minvar.eigenvectors().duplicate(SHALLOW);
	*Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
      }
      catch(Exception& e)  {return 1; }

      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*q_mlmr)[nloops][i][j] = (*Evec)[0][i] * (*Evec)[0][j];
        }
	(*U_mlmr)[nloops][i] = (*U)[i];
      }

      weight_k =  (*ExWeights)[4]  * weight_k_calc(Evalues);
      printf("weight_MLMR = %10.4f\n",weight_k);
	  fflush(stdout);
      weight_sum += weight_k;
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*Q_COM)[i][j] += weight_k * (double)(*Q)[i][j] / ( (double)(*Q)[0][0] +  (double)(*Q)[1][1] + (double)(*Q)[2][2]);
	}
	(*U_com)[nloops][i] += weight_k * (double) (*U)[i];
      }
    }


  // ---- 6: do MER with selected constrain ------
 
    if (doType[6]) {
      printf("doing MER..\n");
	  fflush(stdout);
      Q =  Q_MER_calc(V,B,P,N,U,nestsize);
      ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained);
      try{
	GenericMinVar minvar(constrained,*Q,*ProjMx);  
	*Evec = minvar.eigenvectors().duplicate(SHALLOW);
	*Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
      }
      catch(Exception& e) {return 1; }
      
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*q_mer)[nloops][i][j] = (*Evec)[0][i] * (*Evec)[0][j];
	}
	(*U_mer)[nloops][i] = (*U)[i];
      }
      
      
      weight_k =  (*ExWeights)[6]  * weight_k_calc(Evalues);
      weight_sum += weight_k;
      printf("weight_MER = %10.4f\n",weight_k);
	  fflush(stdout);
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*Q_COM)[i][j] += weight_k * (double)(*Q)[i][j] / ( (double)(*Q)[0][0] +  (double)(*Q)[1][1] + (double)(*Q)[2][2]);
	}
	(*U_com)[nloops][i] += weight_k * (double) (*U)[i];
      }
    }



    // ---- 7: do MTER with selected constrain ------

    if (doType[7]) {
     E_->get_subsequence(*E,msk);
     H_->get_subsequence(*H,msk);
     Q =  Q_MTER_calc(V,B,P,E,N,H,U,nestsize);
      ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained);
      try{
	GenericMinVar minvar(constrained,*Q,*ProjMx);  
	*Evec = minvar.eigenvectors().duplicate(SHALLOW);
	*Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
      }
      catch(Exception& e) {return 1; }
    
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*q_mter)[nloops][i][j] = (*Evec)[0][i] * (*Evec)[0][j];
	}
	(*U_mter)[nloops][i] = (*U)[i];
      }
 
      weight_k =  (*ExWeights)[5]  * weight_k_calc(Evalues);
      weight_sum += weight_k;
      printf("weight_MTER = %10.4f\n",weight_k);
	  fflush(stdout);
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*Q_COM)[i][j] += weight_k * (double)(*Q)[i][j] / ( (double)(*Q)[0][0] +  (double)(*Q)[1][1] + (double)(*Q)[2][2]);
	}
	(*U_com)[nloops][i] += weight_k * (double) (*U)[i];
      }
    }

  

    // ---- 8 : do MVAV with selected constrain ------ 

    if (doType[8]) {
      printf("Doing MVAV..\n");
	  fflush(stdout);
      Q =  Q_MVAV_calc(V,U,nestsize);
      ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained);
      try{
	GenericMinVar minvar(constrained,*Q,*ProjMx);  
	*Evec = minvar.eigenvectors().duplicate(SHALLOW);
	*Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
      }
      catch(Exception& e) {return 1; }
      
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*q_mvav)[nloops][i][j] = (*Evec)[0][i] * (*Evec)[0][j];
	}
	(*U_mvav)[nloops][i] = (*U)[i];
      }
      
      
      weight_k =  (*ExWeights)[7]  * weight_k_calc(Evalues);
      weight_sum += weight_k;
      printf("weight_MVAV = %10.4f\n",weight_k);
	  fflush(stdout);
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	  (*Q_COM)[i][j] += weight_k * (double)(*Q)[i][j] / ( (double)(*Q)[0][0] +  (double)(*Q)[1][1] + (double)(*Q)[2][2]);
	}
	(*U_com)[nloops][i] += weight_k * (double) (*U)[i];
      }

      // ---- do the double-for V -----------------

      try{
	for (k=0; k<nestsize; k++) {
	  for (i=0; i<3; i++) {      
	    (*V_nestave)[nloops][i] += (double)(*V)[k][i];	    
	  }
	}
	for (i=0; i<3; i++) {
	  (*V_nestave)[nloops][i] /= (double) nestsize;
	}
      } catch(Exception& e) {
	printf("exception in NestedGeneric double avgs for MFR\n");
	return 1; 
      }	      
    }




    // ---- do COM with selected constrain ------

    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*Q_COM)[i][j] /= weight_sum;
      }
    }


    ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained);
    try{
      GenericMinVar minvar(constrained,*Q_COM,*ProjMx);  
      *Evec = minvar.eigenvectors().duplicate(SHALLOW);
    }
    catch(Exception& e) {return 1; }
    for (i=0; i<3; i++) {
       for (j=0; j<3; j++) {
	 (*q_com)[nloops][i][j] = (*Evec)[0][i] * (*Evec)[0][j];
       }
       (*U_com)[nloops][i] /= weight_sum;
    }

    // ---- add up time -----------------

    t_start += 0.99*deltat;
    t_stop -= 0.99*deltat;
  }



  // ---- now, calc doubleaverages and do a Siscoe analysis of the normals -----------

  double nestfac = (double) max_numnest;
  for (k=0; k<max_numnest; k++) {
    for (i=0; i<3; i++) {
      (*B_doubleave)[i] += (double)(*B_nestave)[k][i];
      (*V_doubleave)[i] += (double)(*V_nestave)[k][i];
      (*Va_doubleave)[i] += (double)(*Va_nestave)[k][i];
    }
  }

  for (i=0; i<3; i++) {
    (*B_doubleave)[i] /= nestfac;
    (*V_doubleave)[i] /= nestfac;
    (*Va_doubleave)[i] /= nestfac;
  }
  


  if (doType[0]) ReportSiscoe(q_mvab, U_mva,     B_doubleave, V_doubleave, Va_doubleave, nloops, "MVAB", constrained,alpha, SCc_ID);
  if (doType[1]) ReportSiscoe(q_mvadb, U_mvadb,  B_doubleave, V_doubleave, Va_doubleave, nloops, "MVADB", constrained,alpha, SCc_ID);
  if (doType[3]) ReportSiscoe(q_mfr,  U_mfr,     B_doubleave, V_doubleave, Va_doubleave, nloops, "MFR",  constrained,alpha, SCc_ID);
  if (doType[4]) ReportSiscoe(q_mmr,  U_mmr,     B_doubleave, V_doubleave, Va_doubleave, nloops, "MMR",  constrained,alpha, SCc_ID);
  if (doType[5]) ReportSiscoe(q_mlmr, U_mlmr,    B_doubleave, V_doubleave, Va_doubleave, nloops, "MLMR", constrained,alpha, SCc_ID);
  if (doType[6]) ReportSiscoe(q_mer,  U_mer,     B_doubleave, V_doubleave, Va_doubleave, nloops, "MER",  constrained,alpha, SCc_ID);
  if (doType[7]) ReportSiscoe(q_mter, U_mter,    B_doubleave, V_doubleave, Va_doubleave, nloops, "MTER", constrained,alpha, SCc_ID);
  if (doType[8]) ReportSiscoe(q_mvav, U_mvav,     B_doubleave, V_doubleave, Va_doubleave, nloops, "MVAV",  constrained,alpha, SCc_ID);
  ReportSiscoe(q_com,  U_com,     B_doubleave, V_doubleave, Va_doubleave, nloops, "COM",  constrained,alpha, SCc_ID);

  return (0);
}




