/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qrtn.h"
#include <qregexp.h>

/* Globals */

 int verbose;

  /* used for MAH routine disassembly */


double year_0;               
int MaxTerm;
static int Qrtn_skip = 0;
char last_err[240]= {""};    /* message area for last error */

  /* MAH common blocks */
 
int n_sys;                        /* number of coord systems handled (=NSYS) */
char sys_name[MAX_N_SYS][SYS_NAME_LEN+1];/* full names of systems (=TSYS) */
static int sys_to_gse[MAX_N_SYS][N_ROTNS+1]; /* rtns to gse for each system (=DEFS) */
                                        /* sys_to_gse[k-1][j-1] = DEFS[j][k] */
                                        /* static initialises to zero */
// field dipole
double model_year[N_YEAR_MODELS];
double G01[N_YEAR_MODELS];
double G11[N_YEAR_MODELS];
double H11[N_YEAR_MODELS];
double G01_datum, G11_datum, H11_datum;

int   Lterms[N_ROTNS];                  /* (=LTERMS) sequence of rotations */
int   m_max;                            /* (=MMAX) */
double M_pole_long;                      /* (=MLONG) */
double M_pole_colat;                     /* (=MCOLAT) */
double rotations[12];                    /* (=ROT) standard rotations */
int rot_axis[12]={2,2,0,0,1,1,2,2,0,2,1,2}; /* array of twelve ints = 0,1 or 2 for x, y, z */
                                        /* axis appropriate to rotations */
  /* constants */

static char R_STR_NULL[]=""; 
 double value_of_pi;
 double deg2rad;
 double rad2deg;

/***********************************************************************/

/* get rotation matrix, but don't use it */
/* from MAH transys and trancart */

// This function has been modified to return usual
// pre-multiplying matrix for qpsd

QdRMatrixSeq_var QrGetRotnMatx(QrVector *vec)

{
	double ut;
	int MJDdate, MJDdate_last;
	int n;
	double T[3][3];
 
	QdRMatrixSeq_var T_ptr = new QdRMatrixSeq(vec->n_recs);
	QdRMatrixSeq_var Nil_ptr;
	QdString_var to_frame;
	
	if(Qrtn_skip != 1){
		 /* initialise MAH frame methods */
		if (QrInitQRTN() != QRTN_OK) return Nil_ptr;
	}
   
	verbose = QSAS_profiles->current()->QrtnDebug;
	if(verbose){
		printf("Rotation Matrix \n from frame %s \n   to frame %s\n",
		sys_name[vec->in_frame],
		sys_name[vec->out_frame]);
	}
	/* short stop returns */
	if(vec->in_frame == UNKNOWN) return Nil_ptr;  
	if(vec->in_frame == vec->out_frame) return Nil_ptr;

	/* Get the field model for date using MAH DEFMOD2 */

	MJDdate_last = 0;  /* initialise to ensure first call */
 
 
	for (n=0; n<vec->n_recs; n++){
 
		if(QrEpochToMJD_UT((vec->t)[n].getCDFepoch(), &MJDdate, &ut) != QRTN_OK) {
			if(verbose){
				printf("Failed to turn Epoch into MJD time\n");
				return Nil_ptr;
			}
		}
 
		if( MJDdate_last != MJDdate){   /* repeat only if day changes */
			MJDdate_last = MJDdate;
     
			/* Calculate required years from modified Julian date. 
			 * Calculation is referenced to Epoch 2000.0 */
 
			double yr = 2000.0 + ((double)(MJDdate) - 51544.5)/365.25;
	
			/* Exclude epochs outside model range */
 
			if (yr < model_year[0]) {
				sprintf(last_err,"No magnetic model available for year %f\n", yr);
				return Nil_ptr;
      		}
      		if (yr >= model_year[N_YEAR_MODELS-1]) {
         		printf("WARNING: Extrapolating after last magnetic model available: year %f after %f\n", yr, model_year[N_YEAR_MODELS-1]);
      		}        
 
      		if(QrGetModel(yr) != QRTN_OK) {
				if(verbose){
					printf("Failed to get Magnetic model coefficients\n");
					return Nil_ptr;
         		}
      		}
   		}
 
   		 /* repeat every record */
   		if(QrUpdateModel(MJDdate, ut) != QRTN_OK) {
			if(verbose){
				printf("Failed to update mag model\n");
				return Nil_ptr;
			}
   		}  

   		if(QrMakeMatx(vec->in_frame, vec->out_frame, &T) != QRTN_OK) {
     		if(verbose){
       			printf("Failed to make rotation matrix\n");
				return Nil_ptr;
     		}
   		} 
  
  		QdRMatrix_var R = new QdRMatrix(Dim(3,3)); 
//		*R = T[0][0],T[0][1],T[0][2],T[1][0],T[1][1],T[1][2],T[2][0],T[2][1],T[2][2]; //qrtn native mtx (post mult)
		*R = T[0][0],T[1][0],T[2][0],T[0][1],T[1][1],T[2][1],T[0][2],T[1][2],T[2][2];
		T_ptr[n] = *R;
   
  
   
	} /* end for over records */	
  

	// Frame

	QrSetFrameAttr(vec->in_frame, T_ptr);
	QrSetToFrameAttr(vec->out_frame, T_ptr);
	
  	
	return T_ptr;   
  
} /* end QrGetRotnMatx */

/***********************************************************************/

/* perform rotation requested */
/* from MAH transys and trancart */

long QrDoRotation(QrVector *vec)

{
 long check = QRTN_OK;
 double ut;
 int MJDdate, MJDdate_last;
 int n;
 double T[3][3];
 
 QdRMatrix_var T_ptr = new QdRMatrix(Dim(3,3));
 
   if(Qrtn_skip != 1){
     check = QrInitQRTN();  /* initialise MAH frame methods */
     if (check != QRTN_OK) return QRTN_INIT_ERROR;
   }
   
   verbose = QSAS_profiles->current()->QrtnDebug;
   if(verbose){
     printf("Rotate vector \n from frame %s \n   to frame %s\n",
             sys_name[vec->in_frame],
             sys_name[vec->out_frame]);
   }
   /* short stop returns */
   if(vec->in_frame == UNKNOWN) return UNKNOWN_FRAME;  
   if(vec->in_frame == vec->out_frame) return SAME_FRAME;

/* Get the field model for date using MAH DEFMOD2 */

 MJDdate_last = 0;  /* initialise to ensure first call */
 
  // ensure input is in xyz cartesian
 QdObject_var vec_xyz = QuGetVectorXYZ((QdObject_var)vec->ptr);
 vec->ptr = QdRMatrixSeq_var::narrow(vec_xyz);

 if ( vec->ptr.is_nil() ) return CANNOT_GET_XYZ;
 
 for (n=0; n<vec->n_recs; n++){
 
   check = QrEpochToMJD_UT((vec->t)[n].getCDFepoch(), &MJDdate, &ut);
   if(check != QRTN_OK) {
     if(verbose){
       printf("Failed to turn Epoch into MJD time\n");
       break;
     }
   }
 
   if( MJDdate_last != MJDdate){   /* repeat only if day changes */
      MJDdate_last = MJDdate;
     
	 /* Calculate required years from modified Julian date. 
       * Calculation is referenced to Epoch 2000.0 */
 
      double yr = 2000.0 + ((double)(MJDdate) - 51544.5)/365.25;
	
      /* Exclude epochs outside model range */
 
      if (yr < model_year[0]) {
         sprintf(last_err,"No magnetic model available for year %f\n", yr);
         return QRTN_ERROR;
      }
      if (yr >= model_year[N_YEAR_MODELS-1]) {
         printf("WARNING: Extrapolating after last magnetic model available: year %f after %f\n", yr, model_year[N_YEAR_MODELS-1]);
      }        
 
      check = QrGetModel(yr);
      if(check != QRTN_OK) {
         if(verbose){
           printf("Failed to get Magnetic model coefficients\n");
           break;
         }
      }
        
   }
 
   check = QrUpdateModel(MJDdate, ut);   /* repeat every record */
   if(check != QRTN_OK) {
     if(verbose){
       printf("Failed to update mag model\n");
       break;
     }
   }  

   check = QrMakeMatx(vec->in_frame, vec->out_frame, &T); 
   if(check != QRTN_OK) {
     if(verbose){
       printf("Failed to make rotation matrix\n");
       break;
     }
   } 
     
   if( vec->IsPositionVector ){
       printf("Warning - position vector: origin shift not supported\n");
   }
  
  *(T_ptr) = T[0][0],T[0][1],T[0][2],T[1][0],T[1][1],T[1][2],T[2][0],T[2][1],T[2][2];
   
  (vec->ptr)[n] *= *T_ptr;
  
   
 } /* end for over records */	
  
  // copy required attributes across from input series to output series

// Frame

  QrSetFrameAttr(vec->out_frame, vec->ptr);
  
   return check;   
  
} /* end QrDoRotation */

/***********************************************************************/

/* perform MAH initialisation tasks on launch of window */
/* based on MAH tran.f */

long QrInitQRTN()
{

 long status = QRTN_OK;

 /* set up global value of Pi */
 
 value_of_pi = (double) 4.0 * atan(1.0);
 deg2rad = value_of_pi / 180.0;
 rad2deg = 180.0 / value_of_pi;
 
// set model field data as globals
// G01, G11, H11 are hard coded for fiducial years
// from IAGA
// 11th Generation International Geomagnetic Reference Field Schmidt semi-normalised spherical harmonic coefficients


 model_year[0] = 1900.;
        G01[0] = -31543.;
	   G11[0] = -2298.;
	   H11[0] = 5922.;
	   
 model_year[1] = 1905.;
        G01[1] = -31464.;
	   G11[1] = -2298.;
	   H11[1] = 5909.;
	   
 model_year[2] = 1910.;
        G01[2] = -31354.;
	   G11[2] = -2297.;
	   H11[2] = 5898.;
	      
 model_year[3] = 1915.;
        G01[3] = -31212.;
	   G11[3] = -2306.;
	   H11[3] = 5875.;
	      
 model_year[4] = 1920.;
        G01[4] = -31060.;
	   G11[4] = -2317.;
	   H11[4] = 5845.  ;
	       
 model_year[5]= 1925.;
        G01[5]= -30926.;
	   G11[5]= -2318.;
	   H11[5]= 5817.;
	       
 model_year[6] = 1930.;
        G01[6] = -30805.;
	   G11[6] = -2316.;
	   H11[6] = 5808.;
	         
 model_year[7] = 1935.;
        G01[7] = -30715.;
	   G11[7] = -2306.;
	   H11[7] = 5812.;
	         
 model_year[8] = 1940.;
        G01[8] = -30654.;
	   G11[8] = -2292.;
	   H11[8] = 5821.;
	   	         
 model_year[9] = 1945.;
        G01[9] = -30594.;
	   G11[9] = -2285.;
	   H11[9] = 5810.;
	   	         
 model_year[10] = 1950.;
        G01[10] = -30554.;
	   G11[10] = -2250.;
	   H11[10] = 5815.;
	   	         
 model_year[11] = 1955.;
        G01[11] = -30500.;
	   G11[11] = -2215.;
	   H11[11] = 5820.;
	   
 model_year[12] = 1960.;
        G01[12] = -30421.;
	   G11[12] = -2169.;
	   H11[12] = 5791.;
	   
 model_year[13] = 1965.;
        G01[13] = -30334.;
	   G11[13] = -2119.;
	   H11[13] = 5776.;
		   
 model_year[14] = 1970.;
        G01[14] = -30220.;
	   G11[14] = -2068.;
	   H11[14] = 5737.;
	   
 model_year[15] = 1975.;
        G01[15] = -30100.;
	   G11[15] = -2013.;
	   H11[15] = 5675.;
	   
 model_year[16] = 1980.;
        G01[16] = -29992.;
	   G11[16] = -1956.;
	   H11[16] = 5604.;
	   
 model_year[17] = 1985.;
        G01[17] = -29873.;
	   G11[17] = -1905.;
	   H11[17] = 5500.;
	   
 model_year[18] = 1990.;
        G01[18] = -29775.;
	   G11[18] = -1848.;
	   H11[18] = 5406.;
	   
 model_year[19] = 1995.;
	    G01[19] = -29692.;
	    G11[19] = -1784.;
	    H11[19] = 5306.;
	   
 model_year[20] = 2000.;
	    G01[20] = -29619.4;
	    G11[20] = -1728.2;
	    H11[20] = 5186.1;
	   
 model_year[21] = 2005.;
	    G01[21] = -29554.63;
	    G11[21] = -1669.05;
	    H11[21] = 5077.99;

 model_year[22] = 2010.;
	    G01[22] = -29496.5;
	    G11[22] = -1585.9;
	    H11[22] = 4945.1;
	   
// NOTE: THIS IS A PREDICTED MODEL - REPLACE WITH ACTUAL WHEN NEXT MODEL IS ADDED	   
 model_year[23] = 2015.;
	    G01[23] = -29439.5;
	    G11[23] = -1502.4;
	    H11[23] = 4801.1;
	   

 
 /* Specification for coordinate systems  */

 QrGetTerms(); /* hard wired frame mapping, no conditional statements */

 /* set flag to skip future calls */
 
 Qrtn_skip = 1;
 
 return status;

} /* end QrInitQRTN */

/***********************************************************************/

int QrGetFrameFromAttr(QdObject_var dobj){
 
   int frame_flag;
 
   /*  Note GSE is sub-str of GSEQ, so use strcmp not less safe strstr */
   /* so we need a copy to get frame sub-string on its own */

   string ptr = QuGetFrame(dobj);
   /* we now have frame sub-str */
   
   if(QuToUpper(ptr).compare(0, 4, "GSEQ") == 0){
     frame_flag = GSEQ;
   }
   else if (QuToUpper(ptr).compare(0, 4, "GEOC") == 0){
     frame_flag = GEOC;
   }
   else if (QuToUpper(ptr).compare(0, 4, "MAGD") == 0){
     frame_flag = MAGD;
   }
   else if (QuToUpper(ptr).compare(0, 4, "HEEQ") == 0){
     frame_flag = HEEQ;
   }
   else if (QuToUpper(ptr).compare(0, 3, "GSE") == 0){
     frame_flag = GSE;
   }
   else if (QuToUpper(ptr).compare(0, 3, "GSM") == 0){
     frame_flag = GSM;
   }
   else if (QuToUpper(ptr).compare(0, 3, "GEI") == 0){
     frame_flag = GEI;
   }
   else if (QuToUpper(ptr).compare(0, 3, "MAG") == 0){
     frame_flag = MAGD;
   }
   else if (QuToUpper(ptr).compare(0, 3, "HAE") == 0){
     frame_flag = HAE;
   }
   else if (QuToUpper(ptr).compare(0, 3, "HEE") == 0){
     frame_flag = HEE;
   }
   else if (QuToUpper(ptr).compare(0, 2, "SM") == 0){
     frame_flag = SM;
   } 
   else {
     frame_flag = UNKNOWN;
   } 

 return frame_flag;
 
} /* end  QrGetFrameFromAttr */

/***********************************************************************/

void QrSetFrameAttr(int frame_flag,
                    QdObject_var dobj){

  string Attribute_str;
  string inF;
  string outF;
 
  inF = QuGetFrame(dobj);

  Attribute_str = "vector>";
  
  switch(frame_flag){
    case GSE:
      Attribute_str += "gse";
	 outF = "gse";
      break;
    case GSM:
      Attribute_str += "gsm";
	 outF = "gsm";
      break;
    case GEOC:
      Attribute_str += "geoc";
	 outF = "geoc";
      break;
    case SM:
      Attribute_str += "sm";
	 outF = "sm";
      break;
    case GEI:
      Attribute_str += "gei";
	 outF = "gei";
      break;
    case MAGD:
      Attribute_str += "magd";
	 outF = "magd"; // "mag" will occur in lots of attributes, so we prefer magd
      break;
    case HAE:
      Attribute_str += "hae";
	 outF = "hae";
      break;
    case HEE:
      Attribute_str += "hee";
	 outF = "hee";
      break;
    case HEEQ:
      Attribute_str += "heeq";
	 outF = "heeq";
      break;
    case GSEQ:
      Attribute_str += "gseq";
	 outF = "gseq";
      break;
    default:
      Attribute_str += "???";
	 outF = "???";
      break;
  }
  Attribute_str += "_xyz";
    
    // remove inframe from all attributes if known
  if( !inF.empty() ){
    vector<string> sl;
    dobj->list_xref_names(sl);
    for(int i=0;i<sl.size();i++)
    {
        if(sl[i][0]==CACHE_XREF_CHAR) continue;
        QdObject_var xref_obj=dobj->get_xref(sl[i]);
	   QdString_var text_xref = QdString_var::narrow(xref_obj);
	   if(!text_xref.is_nil()){
	     QString xrefTxt = text_xref->c_str();
	         xrefTxt.replace(QRegExp(inF.c_str(),Qt::CaseInsensitive),QString(outF.c_str())); //  not case sensitive
		 QdString_var text_obj = new QdString(xrefTxt.toLatin1().data());
		 dobj->change_xref(sl[i],text_obj);
	   }	
	   else{
	     // string sequence
	     QdStringSeq_var text_xref_seq = QdStringSeq_var::narrow(xref_obj);
	     QdStringSeq_var text_obj_seq = new QdStringSeq();
	      if(!text_xref_seq.is_nil()){
		   for(int j=0; j<text_xref_seq->sequence_size(); j++){
	             QString xrefTxt = text_xref_seq[j].c_str();
         	     xrefTxt.replace(QRegExp(inF.c_str(),Qt::CaseInsensitive),QString(outF.c_str())); //  not case sensitive
		      QdString_var text_obj = new QdString(xrefTxt.toLatin1().data());
			  text_obj_seq->push_back(*text_obj);
		   }
             dobj->change_xref(sl[i],text_obj_seq);
	      }
		 
		 // string matrix
		 QdSMatrix_var text_xref_mat = QdSMatrix_var::narrow(xref_obj);
	      if(!text_xref_mat.is_nil()){
		   QdSMatrix_var text_obj_mat = Factory::clone( text_xref_mat);
	           SMatrix::iterator itr = text_obj_mat->begin();
		   while(itr != text_obj_mat->end()){
	           QString xrefTxt = itr->c_str();
		   xrefTxt.replace(QRegExp(inF.c_str(),Qt::CaseInsensitive),QString(outF.c_str())); //  not case sensitive
		  
		   QdString_var text_obj = new QdString(xrefTxt.toLatin1().data());
	           *itr = *text_obj;
		   itr++;
		   }
             dobj->change_xref(sl[i],text_obj_mat);
	      }
		 
	   }
	} 
    }
   // reset Frame explicitly, now cartesian
   QuSetFrame(dobj, Attribute_str.c_str());
   
 
  return;
  
} /* end  QrSetFrameAttr */

/***********************************************************************/

void QrSetToFrameAttr(int frame_flag,
                    QdObject_var dobj){

  QdString_var Attribute_str = new QdString("vector>");
 
  switch(frame_flag){
    case GSE:
      *Attribute_str += "gse";
      break;
    case GSM:
      *Attribute_str += "gsm";
      break;
    case GEOC:
      *Attribute_str += "geoc";
      break;
    case SM:
      *Attribute_str += "sm";
      break;
    case GEI:
      *Attribute_str += "gei";
      break;
    case MAGD:
      *Attribute_str += "magd";
      break;
    case HAE:
      *Attribute_str += "hae";
      break;
    case HEE:
      *Attribute_str += "hee";
      break;
    case HEEQ:
      *Attribute_str += "heeq";
      break;
    case GSEQ:
      *Attribute_str += "gseq";
      break;
    default:
      *Attribute_str += "???";
      break;
  }
  *Attribute_str += "_xyz";
   
   dobj->change_xref("ToFrame", Attribute_str);
 
  return;
  
} /* end  QrSetToFrameAttr */

/***********************************************************************/

 char * QrErrorMessage(long err_flag){
  
 static char message[240];
 
 switch(err_flag){
   case QRTN_OK:
     strcpy(last_err, "Qrtn: no error\n");
     break;
   case QRTN_WARNING:
     strcat(last_err, "\n Qrtn: non-specific warning\n");
     break;
   case QRTN_ERROR:
     strcat(last_err, "\n Qrtn: Rotation not performed\n");
     break;
   case QRTN_INIT_ERROR:
     strcat(last_err, "\n Qrtn: env variable MAG_MODELS_DIR not set\n");
     break;
   case NOT_YET_IMPLEMENTED:
     strcat(last_err, "\n Qrtn: Frame not yet handled\n");
     break;
   case UNKNOWN_FRAME:
     strcat(last_err, "\n Qrtn: Frame not recognised\n");
     break;
   case CANNOT_GET_XYZ:
     strcat(last_err, "\n Qrtn: Cannot get vector in Cartesians, is Frame set?\n");
     break;
   case SAME_FRAME:
     strcat(last_err, "\n Qrtn: Already in this frame\n");
     break;
   case BAD_MDT_FILE:
     strcat(last_err, "\n Qrtn: trouble with model.mdt file\n");
     break;
   case BAD_EPOCH:
     strcat(last_err, "\n Qrtn: bad epoch variable \n");
     break;
   case BAD_AXIS:
     strcat(last_err, "\n Qrtn: Bad axis for rotation: report to cscsupport \n");
     break;
   case CANNOT_ROTATE:
     strcat(last_err, "\n Qrtn: This frame not supported \n");
     break;
   default:
     strcat(last_err, "\n unknown error\n");
 }
  
 strcpy(message, last_err); /* defensive programming...            */
 strcpy(last_err, "");      /* avoid risk of accumulating messages */
 
 return &(message[0]);
  
} /* end  QrErrorMessage */

/***********************************************************************/
/*********** M A Hapgood Routines translated to C  *********************/
/***********************************************************************/



long QrGetModel(double yr){

/* Use coefficients for the spherical harmonic model of the magnetic 
   field to extrapolate to epoch of interest: from GETMDL  */

 long check;
 int model;
 
 /* globals...
    G01, G11, H11 are the g and h terms,  
    model_year is the year of the model 
    N_YEAR_MODELS is the maximum number of models set  
    C arrays start at index 0                               
    
    Extrapolate coefficients to date of interest */

    // find which model to use
    for(int i=0; i < N_YEAR_MODELS-1; i++){ // last model is extrapolated and used only to estimate deltas
      model = i;
	 if (yr < model_year[i+1] ) break;
    }

    // note if date is off the end of models defined we still use last pair of models
    // This will extrapolate and will get progressively unreliable beyond the last model
    
	double delta = (yr - model_year[model])/(model_year[model+1] - model_year[model]);
	double one_minus_dt = 1.0 - delta;
	
	G01_datum = G01[model+1]*one_minus_dt + G01[model]*delta;
	G11_datum = G11[model+1]*one_minus_dt + G11[model]*delta;
	H11_datum = H11[model+1]*one_minus_dt + H11[model]*delta;

/* Find the dipole magnetic pole */

 check = QrGetMagPole();       
 
    return QRTN_OK;

} /* end QrGetModel */

/***********************************************************************/

long QrUpdateModel(int MJDdate, double ut)
{

/* update models for time changes */
/* based on MAH tran.f */

 long check;

/* Calculate rotation angles for transformations */

	check = QrGetRotAngles(MJDdate, ut);
	
  if(verbose){
    printf("Greenwich Sideral Time %f\n", (rad2deg*rotations[0]) );
    printf("Solar Longitude %f\n", (rad2deg*rotations[1]) );
    printf("Obliquity of the ecliptic %f\n", (rad2deg*rotations[2]) );
    printf("Rotation from GSE to GSM %f\n", (-rad2deg*rotations[3]) );
    printf("Dipole tilt (GSM to SM) %f\n", (-rad2deg*rotations[4]) );
    printf("Latitude of the dipole pole %f\n", ((rad2deg*rotations[5])+90.0) );
    printf("Longitude of the dipole pole %f\n", (rad2deg*rotations[6]) );
    printf("Ascending node of solar equ. %f\n", (rad2deg*rotations[7]) );
    printf("Inclination of solar equator %f\n", (rad2deg*rotations[8]) );
    printf("Solar longitude of Earth %f\n", (rad2deg*rotations[9]) );
    printf("Solar latitude of Earth %f\n", (rad2deg*rotations[10]) );
    printf("XY inversion %f\n", (rad2deg*rotations[11]) );
  }
    return check;
	  
} /* end QrUpdateModel */

/*********************************************************/

long QrGetRotAngles(int MJDdate, double ut){

 /* from MAH CALANGLE */
     
/* Calculate all 12 rotation angles need for the transformations
   given time as MJD and UT hours. Output is via globals rotations 
   Values of rotations are in degrees. */
  
  long check;
  double latitude, longitude; 
  double JulianC;
  double psi, mu;
  double theta, phi;

/*  position of the magnetic pole calculated by a previous call to QrGetMagPole */
              
/* Convert MJD into time in Julian centuries from Epoch 2000 */

       JulianC = ((double)(MJDdate) - 51544.5) / 36525.0;

/* Greenwich Sideral Time, Solar Longitude and the obliquity of the ecliptic */

       rotations[0] = QrGST(JulianC, ut);
       rotations[1] = QrSunLong(JulianC, ut);
       rotations[2] = QrObliquity(JulianC);
       
/*  latitude and longitude of the dipole pole and convert to rotation angles */

       latitude = value_of_pi/2 - M_pole_colat;
       longitude = M_pole_long;
       rotations[5] = - M_pole_colat;
       rotations[6] = longitude;
       
/* rotation angles from GSE to GSM (PSI) and GSM to SM (MU= dipole tilt). 
   Note this needs dipole latitude and longitude explicitly. 
   It also implictly needs rotation[0] through [2] via internal calls in 
   subroutine GSMGSE. Thus we must calculate these angles last. */
   
       QrGSM_to_GSE(&psi, &mu, latitude, longitude);
       rotations[3] = -psi;
       rotations[4] = -mu;
       
/* ecliptic longitude of the ascending node of the solar
   equator and the inclination of that equator to the ecliptic */
   
       rotations[7] = QrSunEqator(MJDdate);
       rotations[8] = 7.25 * deg2rad;
       
/* heliographic longitude (THETA) and latitude (PHI) of the centre of the disc 
   as seen from the Earth. This is also the heliographic position of the Earth. */
   
       check = QrSunEarth(&theta, &phi);
       rotations[9]  = theta;
       rotations[10] = phi;
       
/* 180 degree rotation needed for XY inversion */
       rotations[11] = value_of_pi;

   return QRTN_OK;

} /* end QrGetRotAngles */

/*********************************************************/
 
 long QrSunEarth(double *theta , double *phi){
 
/* Calculate the heliographic longitude (THETA) and latitude (PHI) of
   the centre of the disc as seen from the Earth. */
   
   double incl, smo;
   double sunlong, omega;
   
/* Pull out angle required */

   incl = rotations[8];
   sunlong = rotations[1];
   omega = rotations[7];
   
/* Working variable */

   smo = sunlong - omega;
   
/* Compute longitude in range -pi/2, pi/2 */

   *theta = atan(cos(incl) * tan(smo));
   
/* Convert THETA to range (0 to pi). */

   if (*theta < 0.0) *theta += value_of_pi;

/* Assign value of THETA in full range (0 to 2 pi). THETA lies in 
   the opposite quadrant to SMO, so if SMO < pi  (SIN(SMO) +ve)
   add pi to THETA  */

   if( sin(smo) > 0.0 ) *theta += value_of_pi;
       
/* Compute latitude */

   *phi = asin(sin(smo)*sin(incl));
   
 return QRTN_OK;
 
} /* end QrSunEarth */

/*********************************************************/

double QrSunEqator(int MJDdate){
 
 double suneq;
 
 /* from MAH SUNEQ */
 
 /* ecliptic longitude of the ascending node of the Sun's equator */

 suneq = 73.6667 + 0.013958*(MJDdate+3242)/365.25;
 suneq *= deg2rad;
       
 return suneq;
 
} /* end QrSunEqator */
 
/*********************************************************/

 long QrGSM_to_GSE(double *psi,
                   double *mu,
                   double latitude,
                   double longitude){

/* from MAH GSMGSE */

 long check;
 double UnitVec[3];
 int list_rtns[N_ROTNS];
 double T[3][3];
  
  /* Calculate rotation angles PSI and MU between GSM and GSE, and between
   GSM and SM, coordinates respectively. */
   
   /* initialise rotation matrix to NULL as trap */
   
   QrNullMatrix(&T);
   
  /* Define the dipole axis as a unit vector Q in geographic coordinates */

       UnitVec[0] = cos(latitude) * cos(longitude);
       UnitVec[1] = cos(latitude) * sin(longitude);
       UnitVec[2] = sin(latitude);
       
  /* Define and calculate transformation matrix T from geographic to
   GSE coordinates by way of GEI. */
   
       list_rtns[0] = -1;
       list_rtns[1] =  3;
       list_rtns[2] =  2;
       
  check = QrBuildTran(&T, 3, list_rtns);
  
/* Apply matrix T to vector Q to get a new unit vector describing
   the dipole axis in GSE coordinates. */
   
  check = QrMultVecbyMatx(&UnitVec, T );
  
/* Derive angles PSI and MU from vector UnitVec */

 *psi = atan2(UnitVec[1],UnitVec[2]);
 *mu  = atan2(UnitVec[0], sqrt(UnitVec[1]*UnitVec[1]+UnitVec[2]*UnitVec[2]) );
       
 return QRTN_OK;
 
 } /* end QrGSM_to_GSE  */
 
/*********************************************************/

 double QrObliquity(double JulianC){

/* from MAH OBLIQ */
 
 double obliq;
 
/* Calculate the obliquity of the ecliptic given JulianC */

  obliq = 23.439 - 0.013*JulianC;
  obliq *= deg2rad;
  
  return obliq;

} /* end QrObliquity */

/*********************************************************/

double QrSunLong(double JulianC, double ut){

 /* from MAH SLONG */

 double sun_long;
 double m, lambda;
 
 /* Calculate the Sun's ecliptic longtitude given JulianC and UT */
 
       m =        QrDegToRadMod2pi(357.528 + 35999.050*JulianC + 0.04107*ut);
       lambda =   QrDegToRadMod2pi(280.460 + 36000.772*JulianC + 0.04107*ut);
       sun_long = QrDegToRadMod2pi(rad2deg*lambda + 
                                (1.915 - 0.0048*JulianC)*sin(m) +
                                0.020*sin(2*m) );
       
  return sun_long;
      
} /* end QrSunLong */

/*********************************************************/

double QrGST(double JulianC, double ut){

 double gst;
 
 /* from MAH GST */
 
 /* Calculate the Greenwich Sideral Time given JulianC and UT */

  gst = QrDegToRadMod2pi(100.461 + 36000.770*JulianC + 15.04107*ut);

  return gst;
  
} /* end QrGST */
  
/*********************************************************/

double QrDegToRadMod2pi(double angle){
 
 double mod_angle;
 int i;
 
 /* Reduce an angle in degrees to the principal value range 0 to 2 pi */
 
        i = (int)(angle/360.0);
        mod_angle = angle - 360.0*(double)(i);
        if (mod_angle < 0.0) mod_angle += 360.0;
        
        mod_angle *= deg2rad;

 return mod_angle;
 
} /* end QrDegToRadMod2pi */

/*********************************************************/

long QrGetMagPole(){

 /* from MAH GETMPL 
    Calculate position of north magnetic pole using dipole (n=1)
    terms of the spherical harmonic model   */
   
 double temp1, temp2;
 

/* Calculate colatitude and longitude of pole */

	M_pole_long  = atan2(H11_datum,G11_datum);
	M_pole_long  += value_of_pi;
	M_pole_colat = atan((G11_datum*cos(M_pole_long)+H11_datum*sin(M_pole_long))/G01_datum); 

  if(verbose){
    temp1 = M_pole_long*rad2deg;
    temp2 = M_pole_colat*rad2deg;	
    printf("pole longitude=%f colatitude=%f radians\n", 
                           M_pole_long, M_pole_colat);
    printf("pole longitude=%f colatitude=%f degrees\n", temp1, temp2);
  }
    return QRTN_OK;
} /* end QrGetMagPole */

/*********************************************************/

long QrEpochToMJD_UT(double epoch, int * MJDdate, double * ut){

 int year, month, day, hour, minute;
 double second;
  if( epoch <= 0.0) return BAD_EPOCH;
  QmjdTime t;
  t.setFromCDFepoch(epoch);
  t.breakDownMJD(&year, &month, &day, &hour, &minute, &second);
  
  if(verbose){
     printf("year %d, month %d, day %d\n",year, month, day);
     printf("hour %d, minute %d, second %f\n",hour, minute, second);
  }
                   
  *MJDdate = QrMJD(year, month, day);
  
  *ut = (double)( hour + ((double) minute)/ 60.0 
                     + ((double) second)/ 3600.0 );
  
  return QRTN_OK;

} /* end QrEpochToMJD_UT */

/*********************************************************/

int QrMJD(int year, int month, int day){
        
 /* Modified Julian Date from MJD(YEAR,MONTH,DAY) */
 
  int mjd;
  double wy, wm;
                   
  if (month > 2) {
     wy = (double) year;
     wm = (double) month + 1.0;
  }
  else{
     wy = (double) year - 1.0;
     wm = (double) month + 13.0;
  }      
  
  mjd = (int) (365.25*wy) + (int) (30.6001*wm) + day - 679019;

  return mjd;
 
} /* end QrMJD */

/****************************************************/

long QrGetTerms ()

{
 
 /* from MAH GETTERMS */
 
/* Set up the data which defines the transformations. 
*
* "n_sys" is the number of systems
* 
* "sys_name" contains the full name of each system
*
* "sys_to_gse" specifies the transformation from each systemto GSE. 
* All other transformations can be derived from this.
* Each definition consists a line of integers. The first value gives
* the number of valid terms following. The other terms indicate
* how the fundamental rotations should be applied for the given
* transformation. If the first value is -1, a rotational transformation
* is not possible; this allows for systems like Geodetic. 
*
* These are hard wired as they SHOULD NOT CHANGE unless a serious error
* is found. This replaces data from TERMS2.DAT file. */
  
 n_sys = 10;
 
 strcpy(sys_name[0], "Geocentric Solar Ecliptic");
  /* null rotation to GSE, leave Sys_to_gse = 0 */

 strcpy(sys_name[1], "Geocentric Solar Magnetospheric");
      sys_to_gse[1][0] = 1;
      sys_to_gse[1][1] = -4;

 strcpy(sys_name[2], "Geographic (geocentric)");
      sys_to_gse[2][0] = 3;
      sys_to_gse[2][1] = -1;
      sys_to_gse[2][2] = 3;
      sys_to_gse[2][3] = 2;
   
 strcpy(sys_name[3], "Solar Magnetic");
      sys_to_gse[3][0] = 2;
      sys_to_gse[3][1] = -5;
      sys_to_gse[3][2] = -4;
 
 strcpy(sys_name[4], "Geocentric Equatorial Inertial");
      sys_to_gse[4][0] = 2;
      sys_to_gse[4][1] = 3;
      sys_to_gse[4][2] = 2;

 strcpy(sys_name[5], "Geomagnetic");
      sys_to_gse[5][0] = 5;
      sys_to_gse[5][1] = -6;
      sys_to_gse[5][2] = -7;
      sys_to_gse[5][3] = -1;
      sys_to_gse[5][4] = 3;
      sys_to_gse[5][5] = 2;

 strcpy(sys_name[6], "Heliocentric Aries Ecliptic");
      sys_to_gse[6][0] = 1;
      sys_to_gse[6][1] = 2;

 strcpy(sys_name[7], "Heliocentric Earth Ecliptic");
      sys_to_gse[7][0] = 1;
      sys_to_gse[7][1] = 12;

 strcpy(sys_name[8], "Heliocentric Earth Equatorial");
      sys_to_gse[8][0] = 4;
      sys_to_gse[8][1] = 2;
      sys_to_gse[8][2] = -8;
      sys_to_gse[8][3] = -9;
      sys_to_gse[8][4] = -10;

 strcpy(sys_name[9], "Geocentric Solar Equatorial");
      sys_to_gse[9][0] = 6;
      sys_to_gse[9][1] = 2;
      sys_to_gse[9][2] = -8;
      sys_to_gse[9][3] = -9;
      sys_to_gse[9][4] = -10;
      sys_to_gse[9][5] = 11;
      sys_to_gse[9][6] = 12;

  return QRTN_OK; 
       
} /* end QrGetTerms */

/****************************************************/

long QrBuildTran(double (*T)[3][3],
                int n_terms,
                int *list_rtns){
 
 /* from MAH MAKET */
                
/* Calculate the transformation matrix T defined by the sequence
   of rotations specified in list_rtns. 
  
   Each transformation is broken down into sequence of simple
   rotations, which are specified by a series of values in integer
   array list_rtns. There are twelve different types of rotation which
   are specified by the integers 1 to 12 as shown below. An inverse
   rotation is specified by adding a negative sign. n_terms gives the
   number of values to be used in list_rtns.
  
   1.  Greenwich Sideral Time      
   2.  Solar Longitude             
   3.  Obliquity of the ecliptic   
   4.  Rotation from GSE to GSM    
   5.  Dipole tilt (GSM to SM)     
   6.  Latitude of the dipole pole 
   7.  Longitude of the dipole pole
   8.  Ascending node of solar equator
   9.  Inclination of solar equator
   10. Solar longitude of Earth   
   11. Solar latitude of Earth    
   12. XY inversion  
  
   For example, a transformation from geographic to GSE requires
   3 rotations: an inverse of 1 followed by 3 then 2. Thus n_terms = 3
   and list_rtns = -1 3 2 0 0 0 0              */
  
  long check;
  double S[3][3];
  double angle;
  int axis;
  int i, j;
    
/* Initalise to unit matrix */
       
     QrUnitMatrix(T);

/* If there is more than one rotation, extract each rotation code, 
   identify its angle and axis, then calculate its transformation 
   matrix S and finally multiply it with the existing transformation
   matrix T. Put the result in T and repeat as necessary. At end T
   is the required matrix.   */

   for(j=0; j<n_terms; j++){
      i = list_rtns[j];
      angle = QrGetAngle(i);
      axis  = QrGetAxis(i);
      check = QrMakeMatrix(angle, axis, &S);
      check = QrMultMtxByMtx(T, S);
   }

   return check;
 
 } /* end QrBuildTran */

/****************************************************/

long QrMultVecbyMatx(double (*XX)[3], double T[3][3]){

  /* from MAH  MULTMTX  */
 
  /* Multiply vector XX by matrix T to obtain new vector XX */
 
  double YY[3];
  int j;
        
  for(j=0; j<3; j++){
     YY[j] = (*XX)[0]*T[0][j] + (*XX)[1]*T[1][j] + (*XX)[2]*T[2][j];
  }

  for(j=0; j<3; j++){
    (*XX)[j] = YY[j];
  }

  return QRTN_OK;
 
} /* end QrMultVecbyMatx */
 
/****************************************************/

  long QrMultMtxByMtx(double (*T)[3][3], double S[3][3]){
  
 /* Multiply matrix T by matrix S to obtain new matrix T    */
 
  double YY[3][3];
  int k,j;
     
  for(k=0; k<3; k++){
    for(j=0; j<3; j++){
      YY[k][j] = (*T)[k][0]*S[0][j] + (*T)[k][1]*S[1][j] + (*T)[k][2]*S[2][j];
    }
  }
  for(k=0; k<3; k++){
     for(j=0; j<3; j++){
       (*T)[k][j] = YY[k][j];
     }
  }
 
  return QRTN_OK;
  
} /* end  */

/****************************************************/

 long QrMakeMatrix(double theta, int axis, double (*T)[3][3]){
 
 /* Make a simple rotation matrix     */
 /* from MAH   MAKEMTX */

 double st, ct;
 int I,J,K;
 
 /* Sort out which columns contain which values     */  
       if (axis == 2) {
         I=0;
         J=1;
         K=2;
       }
       else if (axis == 0){
         I=1;
         J=2;
         K=0;
       }
       else if (axis == 1) {
         I=0;
         J=2;
         K=1;
       }
       else{
         return BAD_AXIS;
       }   
       
  /* make the trig functions  */     
       st = sin(theta);
       ct = cos(theta);
       
  /* Make the matrix   */     
       (*T)[I][I] = ct;
       (*T)[J][J] = ct;
       (*T)[K][K] = 1.0;
       (*T)[I][J] = -st;
       (*T)[J][I] = st;
       (*T)[J][K] = 0.0;
       (*T)[K][I] = 0.0;
       (*T)[I][K] = 0.0;
       (*T)[K][J] = 0.0;

  return QRTN_OK;
  
} /* end QrMakeMatrix */

/****************************************************/
 
  int  QrGetAxis(int i){
  
  int rtn_axis;
  
/* Gets the rotation axis for rotation code I. Pulls the index (0,1, 2) of the 
   axis out of global array axis indexed by the absolute value of i. 
   The sign of i is ignored.   */
   
   if (i < 0) i = -i;
    
   rtn_axis = rot_axis[i-1];
       
  return rtn_axis;
 
 } /* end QrGetAxis */    
       
/****************************************************/

 double QrGetAngle(int i){
 
/* Gets the rotation angle for rotation code i. Pulls the value of the 
   angle out of array rotations indexed by the absolute value of i-1 and gives
   it the same sign as i.  */
   
   double angle;
   int sign;
   

/* Separate out use of i as an INDEX to the angle and as its SIGN   */
       
   if (i < 0) {
     sign = -1;
     i = -i;
   }
   else{
     sign = 1;
   }
       
/* Get the angle */        
   angle = sign * rotations[i-1]; /* C index starts at zero */
      
  return angle;
  
} /* end QrGetAngle */
       
/****************************************************/

void QrUnitMatrix(double (*T)[3][3] ){
 
  /* Return a unit rotation matrix */
 int j, k;
 
 for(j=0; j<3; j++){
    for(k=0; k<3; k++){
       if(j == k) (*T)[j][j] = 1.0;
       else (*T)[j][k] = 0.0;
    }
 } 
 return;
  
} /* end QrUnitMatrix */

/****************************************************/

void QrNullMatrix(double (*T)[3][3] ){
 
  /* Return a zero matrix as error trap */

 int j, k;
 
 for(j=0; j<3; j++){
    for(k=0; k<3; k++){
      (*T)[j][k] = 0.0;
    }
 } 
 return;
  
} /* end QrNullMatrix */

/****************************************************/

long QrMakeMatx(int in_frame,
                int out_frame,
                double (*T)[3][3]){

int j;
int Nt, NtTotal;
long check=QRTN_OK;
int LastRot;

/* Calculate the transformation matrix T corresponding to a 
   transformation between INSYS and OUTSYS. Construct array Lterms 
   indicating the sequence of simple rotations, call MAKET to
   generate T.  From MAH MAKTRAN.    */
       
// Initialize all elements of Lterms to zero
  for(j=0; j<20; j++){
    Lterms[j]=0;
  }

/* Pull out the relevant list of tranformations from definition array 
   'sys_to_gse', and copy the list into array 'Lterms'. 
   Nt is the number of terms in this list.   */
   
    Nt = sys_to_gse[in_frame][0];
    if(Nt < 0)  return CANNOT_ROTATE;
    
    for(j=1; j<=Nt; j++){ 
      Lterms[j-1] = sys_to_gse[in_frame][j];
    }
    NtTotal = Nt;
      
/* Pull out the relevant list of tranformations from definition array 
   sys_to_gse, and copy the list into array Lterms in reverse order. 
   Nt is the number of terms in this list.  */
   
    Nt = sys_to_gse[out_frame][0];
    if(Nt < 0)  return CANNOT_ROTATE;

/* add rotations to list removing cancelling terms */
    
    for(j=1; j<=Nt; j++){ 
      /* if there is no rotation in list set a large number to avoid match */
      if(NtTotal > 0) {
        LastRot = Lterms[NtTotal-1];
      }
      else{
        LastRot = 1000;
      }

      if( LastRot == sys_to_gse[out_frame][Nt+1-j]){
        NtTotal--;  /* cancel matching rotation */
      }
      else{
        /* add inverse rotation to list */
        Lterms[NtTotal] = - sys_to_gse[out_frame][Nt+1-j];
        NtTotal++;
      }
    }
       
/* Calculate transformation matrix */

    check = QrBuildTran(T, NtTotal, Lterms); 
    return check;
       
} /* end QrMakeMatx */

/****************************************************/

int QrIsPositionVector( QdObject_var dobj){

 int IsPosn;
 size_t ptr;
 
  string attr = QuGetAttrText("SI_conversion", dobj);

  /* if we don't have SI_conversion, return false as best guess */
  
  if( attr.empty() ) return 0;

 /* find delimiter */ 
 ptr = attr.find(">");

 if(ptr == string::npos) return 0;
 ptr++;
 /* read from after delimiter to end, 
    if only m and white space found then this is position */
    
 IsPosn = 0; 
 while( ptr < attr.size() ){


   if(attr.at(ptr) == ' ') continue; /* ignore white space */
 
   if(attr.at(ptr) == 'm'){
     /* set 'position' flag, but keep looking, e.g. "m s^-1" */
     IsPosn = 1;     
   }
   else{
     /* anything else and it cannot be position */
     IsPosn = 0;     
     break;
   }
	ptr++;
 }

 return IsPosn;

} /* end QrIsPositionVector */

/****************************************************/

int QrOriginsDiffer(int in_frame, int out_frame){

 if(in_frame == 6 || in_frame == 7 || in_frame == 8 ){
   if (out_frame != 6 && out_frame != 7 && out_frame != 8 ) return 1;
 }
 else if(out_frame == 6 || out_frame == 7 || out_frame == 8 ) return 1;
 
 return 0;

} /* end QrOriginsDiffer */
