#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

#include "qplug_if.h"
#include "qdutil.h"
#include "Qdos.h"
#include "qar.h"
#include "Xrefs.h"  

using namespace QSAS;

// SCALAR value_in
QdObject_var        GetRemainder(    QdRScalar_var qVal,           QdRScalar_var    qDivider       );
QdObject_var        GetRemainder(    QdRScalar_var qVal,           QdRScalarSeq_var qSeqDivider    );
QdObject_var        GetRemainder(    QdRScalar_var qVal,           QdRMatrix_var    qMatDivider    );
QdObject_var        GetRemainder(    QdRScalar_var qVal,           QdRMatrixSeq_var qMatSeqDivider );

// SEQUENCE OF SCALARS value_in
QdObject_var  GetRemainder(       QdRScalarSeq_var qSeqVal,     QdRScalar_var    qDivider        );
QdObject_var  GetRemainder(       QdRScalarSeq_var qSeqVal,     QdRScalarSeq_var qSeqDivider     );
QdObject_var  GetRemainder(       QdRScalarSeq_var qSeqVal,     QdRMatrix_var    qMatDivider     );
QdObject_var  GetRemainder(       QdRScalarSeq_var qSeqVal,     QdRMatrixSeq_var qMatSeqDivider  );

// MATRIX OF SCALARS value_in
QdObject_var        GetRemainder(    QdRMatrix_var qMatVal,        QdRScalar_var    qDivider );
QdObject_var        GetRemainder(    QdRMatrix_var qMatVal,        QdRScalarSeq_var qSeqDivider );
QdObject_var        GetRemainder(    QdRMatrix_var qMatVal,        QdRMatrix_var    qMatDivider );
QdObject_var        GetRemainder(    QdRMatrix_var qMatVal,        QdRMatrixSeq_var qMatSeqDivider );

// SEQUENCE OF MATRIX OF SCALARS value_in
QdObject_var  GetRemainder(       QdRMatrixSeq_var qMatSeqVal,  QdRScalar_var    qDivider );
QdObject_var  GetRemainder(       QdRMatrixSeq_var qMatSeqVal,  QdRScalarSeq_var qSeqDivider );
QdObject_var  GetRemainder(       QdRMatrixSeq_var qMatSeqVal,  QdRMatrix_var    qMatDivider );
QdObject_var  GetRemainder(       QdRMatrixSeq_var qMatSeqVal,  QdRMatrixSeq_var qMatSeqDivider );


template <class T> QdObject_var      ProcessRemainder( T                rVal, 
                                                       QdRScalar_var    rDivider,
                                                       QdRScalarSeq_var rSeqDivider,
                                                       QdRMatrix_var    rMatDivider,
                                                       QdRMatrixSeq_var rMatSeqDivider );
					       


// Make the function accessible from external modules 
extern "C" QplugReturnStatus QarRemainder( QplugArgList *call_list );

QplugReturnStatus	QarRemainder( QplugArgList *call_list )
{

   // Define some variables
   int                     qar_status;
   char                    *outName = NULL;
   QplugReturnStatus       qStatus  = QPLUG_SUCCESS;
   
 
   // Display initial message in QpluGui window
   QplugAppendTextDisplay( "Remainder Plugin 1.0 Oct 2003\n" );
   
   // ( 1 ) Unpack the input data objects
   // This will consist of two series
   // Serie 1: List of values to be phased
   // Serie 2: List of phases

   QdObject_var value_in     = (* call_list)[0]; // List of QdObject_var objects 
   QdObject_var divider_in   = (* call_list)[1];
   QdObject_var qReminder_out;
   
   // ( 2 ) Get name of output objects:
   char  *slotPtr = GetSlotText(0);
   if ( slotPtr[0] != '\0' ) 
   {
      outName = QplugNewSTR( slotPtr );
   }
   else
   {
      outName = QplugNewSTR( "Remainder" );
      SetSlotText( 0, outName );
   }

   // ( 3 ) Check whether input conformal and Time series joined
   qar_status = QarTestConformal( value_in, divider_in );
   if( qar_status == QAR_ERR )
   { 
      QplugAppendTextDisplay("Remainder ERROR: Input conformity test failed\n");
      return QPLUG_FAILURE;
   } 

   // ( 4 ) Display what is happening
   QplugAppendTextDisplay ("Remainder>> Calculating.\n");
   
   // Value_in
   QdRScalar_var    rVal              = QdRScalar_var::narrow(    value_in );
   QdRScalarSeq_var rSeqVal           = QdRScalarSeq_var::narrow( value_in );
   QdRMatrix_var    rMatVal           = QdRMatrix_var::narrow(    value_in );
   QdRMatrixSeq_var rMatSeqVal        = QdRMatrixSeq_var::narrow( value_in );
   
   // Divider_in
   QdRScalar_var    rDivider          = QdRScalar_var::narrow(    divider_in );
   QdRScalarSeq_var rSeqDivider       = QdRScalarSeq_var::narrow( divider_in );
   QdRMatrix_var    rMatDivider       = QdRMatrix_var::narrow(    divider_in );
   QdRMatrixSeq_var rMatSeqDivider    = QdRMatrixSeq_var::narrow( divider_in );
   
      
   // ( 5 ) Value_in is a scalar (float)
   if ( !rVal.is_nil() )
   {
       // Float - value_in
       
       qReminder_out = ProcessRemainder( rVal, 
                                         rDivider,
                                         rSeqDivider,
                                         rMatDivider,
                                         rMatSeqDivider );
					 
       if ( qReminder_out.is_nil() )
       {
           return ( QPLUG_FAILURE );
       }

   }
   else if ( !rSeqVal.is_nil() )
   {
       // Series of floats - value_in
       
       qReminder_out = ProcessRemainder( rSeqVal, 
                                         rDivider,
                                         rSeqDivider,
                                         rMatDivider,
                                         rMatSeqDivider );
       if ( qReminder_out.is_nil() )
       {
           return ( QPLUG_FAILURE );
       }

   } 
   else if ( !rMatVal.is_nil() )
   {
       // Matrix of scalars - value_in
       
       qReminder_out = ProcessRemainder( rMatVal, 
                                         rDivider,
                                         rSeqDivider,
                                         rMatDivider,
                                         rMatSeqDivider );
       if ( qReminder_out.is_nil() )
       {
           return ( QPLUG_FAILURE );
       }

   }
   else if ( !rMatSeqVal.is_nil() )
   {
      // Series of matrixes of scalars - value_in
        qReminder_out = ProcessRemainder( rMatSeqVal, 
                                          rDivider,
                                          rSeqDivider,
                                          rMatDivider,
                                          rMatSeqDivider );
       if ( qReminder_out.is_nil() )
       {
           return ( QPLUG_FAILURE );
       }
 
   }
   else 
   {
        // Type not being supported within this plug-in
	QplugAppendTextDisplay ("Remainder>> ERROR - input object types for value_in not handled \n");
        return QPLUG_FAILURE;
   }
   
   
	// ( 6 ) Add timeline: Either timeline is safe if it exists
	QplugAppendTextDisplay("Remainder>> Get the DEPEND_0\n");
	QdObject_var qd0 = get_Depend_0( value_in );
	if ( !qd0.is_nil() ) qd0 = get_Depend_0( divider_in );
	if ( !qd0.is_nil() ) qReminder_out->change_xref(DEPEND_0 , qd0 );
	
 
   // ( 7 ) Display what is happening
   QplugAppendTextDisplay ("Remainder>> Setting Essential attributes\n");
  
   // ( 8 ) Set QSAS essential attributes
   qReminder_out->copy_xrefs_from( value_in ); 

   // ( 8.1 ) FRAME
   if(QuIsThreeVector(value_in))
   QplugAppendTextDisplay ("WARNING: Remainder treats each component independently");

  
   // ( 8.4 ) FIELDNAME
     QuSetTxtAttr( FIELDNAM, "Remainder",     qReminder_out );
   

   // ( 9 ) Add output object to  QSAS working list
   call_list->push_back( qReminder_out );
   
   
   QplugAppendTextDisplay( "Remainder: Calculation completed\n" );  
  
   return ( qStatus );
}

//////////////////////////////////SCALAR VALUE_IN///////////////////////////////////
QdObject_var     GetRemainder(    QdRScalar_var qVal, 
                                  QdRScalar_var qDivider 
			       )
{
    QdObject_var  rObjNull;
    QdObject_var qReminder_out;
    
    if ( qVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The scalar value for value_in is NULL\n");
          return ( rObjNull );
    }
     
     
    if ( qDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The scalar value for the divider_in is NULL\n");
          return ( rObjNull );
    }
     
    float  fVal       = (float)(*qVal );
    float  fDivider   = (float)(*qDivider);
    float  fRemainder = 0.;
    
    QdRScalar_var  qRemainder;
    
    if ( fDivider != 0. )
    {
        qRemainder = new QdRScalar;
	if ( qRemainder.is_nil() )
	{
	     QplugAppendTextDisplay( "Remainder>>Error Cannot allocate memory for remainder scalar object\n");
             return ( rObjNull );
	}
	
        fRemainder = fmod( fVal, fDivider );
	*qRemainder = fRemainder;
    }
    else
    {
         QplugAppendTextDisplay( "Remainder>>Error The divider value is set to zero (0)\n");
         return ( rObjNull );
    }
 
    qReminder_out = ( QdObject_var )qRemainder;
    return ( qReminder_out );
}



QdObject_var  GetRemainder( QdRScalar_var    qVal,    
                            QdRScalarSeq_var qSeqDivider 
			       )
{
     QdObject_var  rObjNull;
     QdObject_var  qReminder_out;

     if ( qVal.is_nil() )
     {
          QplugAppendTextDisplay( "Remainder>> The scalar value for value_in is NULL\n");
          return ( rObjNull );
     }
     
     
     if ( qSeqDivider.is_nil() )
     {
          QplugAppendTextDisplay( "Remainder>> The sequence of scalars for divider_in is NULL\n");
          return ( rObjNull );
     }
     
     
     float  fVal                     = (float)(*qVal );
     int               nData         = qSeqDivider->sequence_size();
     QdRScalarSeq_var  rSeqRemainder = new QdRScalarSeq( nData );
     float    fDivider = 0.;
     
     if ( rSeqRemainder.is_nil() )
     {
         QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar sequence\n");
         return ( rObjNull );
     }
     
     for ( long i = 0; i < nData; i++ ) 
     {  
	 fDivider = ( float )qSeqDivider[ i ];
	   
	 if ( fDivider == 0. )
	 {
	      // We have to make sure that the divider is not equal to 0
	      rSeqRemainder[ i ] = 0.;
	 }
	 else
	 {
              rSeqRemainder[ i ] = fmod( *qVal , qSeqDivider[ i ] );
	 }
     }
     
     qReminder_out = ( QdObject_var )rSeqRemainder;
     return ( qReminder_out );	
}

QdObject_var     GetRemainder(    QdRScalar_var    qVal,    
                                  QdRMatrix_var    qMatDivider
			      )
{
     QdObject_var qMatNullObj;
     QdObject_var qReminder_out;

     if ( qVal.is_nil() )
     {
          QplugAppendTextDisplay( "Remainder>> The scalar value for value_in is NULL\n");
          return ( qMatNullObj );
     }
     
     
     if ( qMatDivider.is_nil() )
     {
          QplugAppendTextDisplay( "Remainder>> The matrix of scalars for divider_in is NULL\n");
          return ( qMatNullObj );
     }
     
     
     
     int nRankSize = (qMatDivider->dims()).size();
     cout << "rank size = " << nRankSize << endl; 
  
     
     float  fVal                     = (float)(*qVal );
     float  fDivider      = 0.;
     
    
       
     QdRMatrix_var qMatRemainder = new QdRMatrix( RMatrix( qMatDivider->dims() ) );
     if ( qMatRemainder.is_nil() )
     {
          QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar matrix\n");
          return ( qMatNullObj );
     } 
     
     QdRMatrix           mat    = *qMatDivider;
     QdRMatrix::iterator itrRes = (*qMatRemainder).begin();
     for ( QdRMatrix::iterator itr = mat.begin(); itr != mat.end(); itr++ )
     {
         float fDivider = (float)(*itr);
	 float fRemainder = 0.;
	 
	 if ( fDivider == 0. )
	 {
	     // We should make sure that the divider is not set to zero (0)
	     *itrRes = 0.;
	 }
	 else
	 {
	      // Calculate the remainder
	      *itrRes = fmod( *qVal, *itr );
	 }
	 
	 itrRes++;
     }

     qReminder_out = ( QdObject_var )qMatRemainder;
     return ( qReminder_out );	
     
}

QdObject_var  GetRemainder( QdRScalar_var     qVal, 
                            QdRMatrixSeq_var  qMatSeqDivider )
{
     QdObject_var qMatNullObj;
     QdObject_var qReminder_out;

     if ( qVal.is_nil() )
     {
          QplugAppendTextDisplay( "Remainder>> The scalar value for value_in is NULL\n");
          return ( qMatNullObj );
     }
     
     
     if ( qMatSeqDivider.is_nil() )
     {
          QplugAppendTextDisplay( "Remainder>> The series of matrix of scalars for divider_in is NULL\n");
          return ( qMatNullObj );
     }
     
     int nRankSize = ( qMatSeqDivider[ 0 ].dims() ).size();
          
        
     float  fVal                     = (float)(*qVal );
     float  fDivider                 = 0.;
     float  fRemainder               = 0.;
     int    nData                    = qMatSeqDivider->sequence_size();
     
     
     // NOTE: We do assume that all the matrixes do have the same dimension within a 
     //       sequence. Hence, only the first element of the sequence is used
     //       to instantiate the sequence of matrixes object.
     
     QdRMatrixSeq_var qMatSeqRemainder = new QdRMatrixSeq( nData, 
                                                           RMatrix( qMatSeqDivider[ 0 ].dims() )
							  );
     if ( qMatSeqRemainder.is_nil() )
     {
         // This should never happen - but it could happen
         QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar matrix sequence\n");
         return ( qMatNullObj );
     }
    
     for ( long nSeq = 0; nSeq < nData; nSeq++ )
     {
         QdRMatrix           mat    = ( qMatSeqDivider[   nSeq ] );
         QdRMatrix::iterator itrRes = ( qMatSeqRemainder[ nSeq ] ).begin();
	 
         for ( QdRMatrix::iterator itr = mat.begin(); 
	       itr != mat.end(); 
	       itr++ 
	     )
         {
	      float fDivider = (float)(*itr);
	      if ( fDivider == 0. )
	      {
	         // We should make sure that the divider is not set to zero ( 0 )
	         *itrRes = 0.;
	      }
	      else
	      {
	         // Calculate the remainder
	         *itrRes = fmod( *qVal, *itr );
	      }
	 
	 
	      itrRes++;
	 }
     }
   
     qReminder_out = ( QdObject_var )qMatSeqRemainder;
     return ( qReminder_out );	
}


//////////////////////////SCALAR SEQ VALUE_IN/////////////////////////////////////
QdObject_var  GetRemainder(    QdRScalarSeq_var qSeqVal,  
                               QdRScalar_var    qDivider )
{

    QdObject_var  rObjNull;
    QdObject_var qReminder_out;

    if ( qSeqVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The sequence of scalar value for value_in is NULL\n");
          return ( rObjNull );
    }
     
     
    if ( qDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The scalar value for divider_in is NULL\n");
          return ( rObjNull );
    }
    
    float  fDivider = (float)(*qDivider);
    if ( fDivider == 0. )
    {
          QplugAppendTextDisplay( "Remainder>> The scalar value for divider_in is set to zero (0)\n");
          return ( rObjNull );
    }
    
    
    int    nData    = qSeqVal->sequence_size();
        
    QdRScalarSeq_var  rSeqRemainder = new QdRScalarSeq( nData );
    if ( rSeqRemainder.is_nil() )
    {
        QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar series\n");
        return ( rObjNull );
    }
     
    
    try
    {      
	for ( long i = 0; i < nData; i++ ) 
	{  
	   rSeqRemainder[ i ] = fmod( qSeqVal[ i ] , *qDivider );
	}	
     }
     catch( Exception &e )
     {
          e.print_msg();
          QplugAppendTextDisplay( "Remainder>> Exception thrown\n");
          return ( rObjNull );   
     }
     
     qReminder_out = ( QdObject_var )rSeqRemainder;
     return ( qReminder_out );	
     
}



QdObject_var  GetRemainder( 
                                             QdRScalarSeq_var qSeqVal, 
                                             QdRScalarSeq_var qSeqDivider 
			       )
{
     char   bufLog[ 100 ];
     int    nData    = qSeqVal->sequence_size();
     float  fDivider = 0.;
     
     sprintf( bufLog, "Remainder>> Size of sequence %d\n", nData );
     QplugAppendTextDisplay( bufLog ); 

     QdRScalarSeq_var  rSeqRemainder = new QdRScalarSeq( nData );
     QdObject_var  rObjNull;
     QdObject_var qReminder_out;
   
     
     try
     {      
	for ( long i = 0; i < nData; i++ ) 
	{  
	   fDivider = (float)qSeqDivider[ i ];
	   
	   if ( fDivider == 0. )
	   {
	      // We have to make sure that the divider is not equal to 0
	      rSeqRemainder[ i ] = 0.;
	   }
	   else
	   {
              rSeqRemainder[ i ] = fmod( qSeqVal[ i ] , qSeqDivider[ i ] );
	   }
	}	
     }
     catch( Exception &e )
     {
          e.print_msg();
          QplugAppendTextDisplay( "Remainder>> Exception thrown\n");
          return ( rObjNull );   
     }
     
     qReminder_out = ( QdObject_var )rSeqRemainder;
     return ( qReminder_out );	
     
}

QdObject_var  GetRemainder( QdRScalarSeq_var qSeqVal,  
                            QdRMatrix_var    qMatDivider )
{
     
    QdObject_var qMatNullObj;   
    
    if ( qSeqVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The sequence of scalar value for value_in is NULL\n");
          return ( qMatNullObj );
    }
     
     
    if ( qMatDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The scalar matrix for divider_in is NULL\n");
          return ( qMatNullObj );
    }
    
    
    int          nData         = qSeqVal->sequence_size();
    QdObject_var qReminder_out;
    
    
        
    QdRMatrixSeq_var qMatSeqRemainder = new QdRMatrixSeq( nData, 
                                                          RMatrix( qMatDivider->dims() ) 
							 );
    if ( qMatSeqRemainder.is_nil() )
    {
        QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar matrix sequence\n");
        return ( qMatNullObj );
    }
    		
    QdRMatrix           mat    = *qMatDivider;
    for ( long nSeq = 0; nSeq < nData; nSeq++ )
    {
    
       QdRMatrix::iterator itrRes = qMatSeqRemainder[ nSeq ].begin();
       for ( 
             QdRMatrix::iterator itr = mat.begin(); 
             itr != mat.end(); 
	     itr++
	    )
       {
         float fDivider = (float)(*itr);
	 
	 if ( fDivider == 0. )
	 {
	     // We should make sure that the divider is not set to zero (0)
	     *itrRes = 0.;
	 }
	 else
	 {
	      // Calculate the remainder
	      *itrRes = fmod( qSeqVal[ nSeq ], *itr );
	 }
	 
	 itrRes++;
       }

    } 

 
    qReminder_out = ( QdObject_var )qMatSeqRemainder;
    return ( qReminder_out );	
     
}

QdObject_var GetRemainder( QdRScalarSeq_var qSeqVal,  
                           QdRMatrixSeq_var qMatSeqDivider  )
{
    QdObject_var qMatNullObj;
    
    if ( qSeqVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The sequence of scalar value for value_in is NULL\n");
          return ( qMatNullObj );
    }
     
     
    if ( qMatSeqDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The sequence of scalar matrix for divider_in is NULL\n");
          return ( qMatNullObj );
    }
        
    int          nData         = qSeqVal->sequence_size();
    QdObject_var qReminder_out;
    
    
        
    QdRMatrixSeq_var qMatSeqRemainder = new QdRMatrixSeq( nData, 
                                                          RMatrix( qMatSeqDivider[0].dims() ) 
							 );
    if ( qMatSeqRemainder.is_nil() )
    {
        QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar matrix sequence\n");
        return ( qMatNullObj );
    }
    
    for ( long nSeq = 0; nSeq < nData; nSeq++ )
    {
         QdRMatrix           mat    = ( qMatSeqDivider[   nSeq ] );
         QdRMatrix::iterator itrRes = ( qMatSeqRemainder[ nSeq ] ).begin();
	 
         for ( QdRMatrix::iterator itr = mat.begin(); 
	       itr != mat.end(); 
	       itr++ 
	     )
         {
	      float fDivider = (float)(*itr);
	      if ( fDivider == 0. )
	      {
	         // We should make sure that the divider is not set to zero ( 0 )
	         *itrRes = 0.;
	      }
	      else
	      {
	         // Calculate the remainder
	         *itrRes = fmod( qSeqVal[ nSeq ], *itr );
	      }
	 
	 
	      itrRes++;
	 }
     }
   
     qReminder_out = ( QdObject_var )qMatSeqRemainder;
     return ( qReminder_out );	
}




///////////////////////////SCALAR MATRIX VALUE_IN/////////////////////////////////

QdObject_var     GetRemainder(     QdRMatrix_var    qMatVal,   
                                   QdRScalar_var    qDivider )
{
    QdObject_var      qMatNullObj;
    if ( qMatVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The matrix of scalar value for value_in is NULL\n");
          return ( qMatNullObj );
    }
     
     
    if ( qDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The scalar value for divider_in is NULL\n");
          return ( qMatNullObj );
    }
    
    QdObject_var      qReminder_out;
    
    float fDivider = (float)(*qDivider);
    if ( fDivider == 0. )
    {
        QplugAppendTextDisplay( "Remainder>> Error divider scalar value is set to zero (0)\n");
        return ( qMatNullObj );
    }
    
    QdRMatrix_var qMatRemainder = new QdRMatrix( RMatrix( qMatVal->dims() ) );
    if ( qMatRemainder.is_nil() )
    {
        QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar matrix\n");
        return ( qMatNullObj );
    }
    
    QdRMatrix           mat    =   *qMatVal;
    QdRMatrix::iterator itrRes = ( *qMatRemainder ).begin();
    for ( 
             QdRMatrix::iterator itr = mat.begin(); 
             itr != mat.end(); 
	     itr++
	)
    {
         // Calculate the remainder
	 *itrRes = fmod( *itr, *qDivider );
	 itrRes++;
    }

    qReminder_out = ( QdObject_var )qMatRemainder;
    return ( qReminder_out );	 
}


QdObject_var  GetRemainder( QdRMatrix_var    qMatVal,    
                            QdRScalarSeq_var qSeqDivider )
{
    QdObject_var      qMatNullObj;
    if ( qMatVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The matrix of scalar value for value_in is NULL\n");
          return ( qMatNullObj );
    }
     
     
    if ( qSeqDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The scalar sequence for divider_in is NULL\n");
          return ( qMatNullObj );
    }
    
    float        fDivider     = 0.;
    int          nData = qSeqDivider->sequence_size();
    QdObject_var qReminder_out;
        
    QdRMatrixSeq_var qMatSeqRemainder = new QdRMatrixSeq( nData, RMatrix( qMatVal->dims() ) );
    if ( qMatSeqRemainder.is_nil() )
    {
        QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar matrix sequence\n");
        return ( qMatNullObj );
    }
    
    QdRMatrix           mat    =   *qMatVal;
    for ( long nSeq = 0; nSeq < nData; nSeq++ )
    {
    
        QdRMatrix::iterator itrRes = qMatSeqRemainder[ nSeq ].begin();
	
        for ( 
             QdRMatrix::iterator itr = mat.begin(); 
             itr != mat.end(); 
	     itr++
	    )
        {
             // Calculate the remainder
	     fDivider = (float)qSeqDivider[ nSeq ];
	     if ( fDivider == 0. )
	     {
	        *itrRes = 0;
	     
	     }
	     else
	     {
	     
	         *itrRes = fmod( *itr, qSeqDivider[ nSeq ] );
             }
	     
	     itrRes++;
        }
	
    }
    
      
    qReminder_out = ( QdObject_var )qMatSeqRemainder;
    return ( qReminder_out );	 
    
}

QdObject_var     GetRemainder( QdRMatrix_var qMatVal, 
                               QdRMatrix_var qMatDivider )
{
       
    float        fDivider     = 0.;
    QdObject_var qReminder_out;
    QdObject_var qMatNullObj;
    
    if ( qMatVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The matrix of scalar value for value_in is NULL\n");
          return ( qMatNullObj );
    }
     
     
    if ( qMatDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The matrix of scalar for divider_in is NULL\n");
          return ( qMatNullObj );
    }
    
    
    if ( !( *qMatVal ).samesize( ( *qMatDivider ) ) )
    {
          QplugAppendTextDisplay( "Remainder>>Error the matrixes for value_in and divider_in do not have the same number and dimension size" );
	  return ( qMatNullObj );
    }
    
  
    QdRMatrix_var qMatRemainder = new QdRMatrix( RMatrix( qMatVal->dims() ) );
    if ( qMatRemainder.is_nil() )
    {
        QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar matrix\n");
        return ( qMatNullObj );
    }
    
    
    QdRMatrix           matVal    =   *qMatVal;
    QdRMatrix           matDiv    =   *qMatDivider;
    
    QdRMatrix::iterator  itrRes = ( *qMatRemainder ).begin();
    QdRMatrix::iterator  itrVal;
    QdRMatrix::iterator  itrDiv;

   for ( 
             itrVal  = matVal.begin(), itrDiv  = matDiv.begin(); 
             itrVal != matVal.end() && itrDiv != matDiv.end() ; 
	     itrVal++, itrDiv++
        )
   {
             // Calculate the remainder
	     fDivider = (float)( *itrDiv );
	     if ( fDivider == 0. )
	     {
	        *itrRes = 0;
	     }
	     else
	     {
	        *itrRes = fmod( *itrVal, *itrDiv );
             }
	     
	     itrRes++;

    }
   
  
    qReminder_out = ( QdObject_var )qMatRemainder;
    return ( qReminder_out );	
     
}

QdObject_var  GetRemainder( QdRMatrix_var    qMatVal,    
                            QdRMatrixSeq_var qMatSeqDivider )
{
    QdObject_var qMatNullObj;
    
    if ( qMatVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The matrix of scalar value for value_in is NULL\n");
          return ( qMatNullObj );
    }
     
     
    if ( qMatSeqDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The sequence of matrix of scalar for divider_in is NULL\n");
          return ( qMatNullObj );
    }
    
    
    int          nData        = qMatSeqDivider->sequence_size();
    float        fDivider     = 0.;
    QdObject_var qReminder_out;
        
        
    if ( ! (*qMatVal).samesize( qMatSeqDivider[0] ) )
    {
        QplugAppendTextDisplay( "Remainder>>Error - The matrixes for value_in and divider_in do not have the same number and dimension size" );
	return ( qMatNullObj );
    }
    
    
    QdRMatrixSeq_var qMatSeqRemainder = new QdRMatrixSeq( nData, 
                                                          RMatrix( qMatVal->dims() ) );
    if ( qMatSeqRemainder.is_nil() )
    {
        QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder sequence of scalar matrix\n");
        return ( qMatNullObj );
    }
							  
    QdRMatrix           matVal    =   *qMatVal;
    for ( long nSeq = 0; nSeq < nData; nSeq++ )
    {
    
        QdRMatrix::iterator itrRes    = qMatSeqRemainder[ nSeq ].begin();
	QdRMatrix           matDiv    = qMatSeqDivider[ nSeq ];
	QdRMatrix::iterator itrDiv;
	
        for ( 
             QdRMatrix::iterator itrVal = matVal.begin(), itrDiv = matDiv.begin(); 
             itrVal != matVal.end() && itrDiv != matDiv.end() ; 
	     itrVal++, itrDiv++
	    )
        {
             // Calculate the remainder
	     fDivider = (float)( *itrDiv );
	     if ( fDivider == 0. )
	     {
	        *itrRes = 0;
	     
	     }
	     else
	     {
	     
	         *itrRes = fmod( *itrVal, *itrDiv );
             }
	     
	     itrRes++;
        }
	
    }

    qReminder_out = ( QdObject_var )qMatSeqRemainder;
    return ( qReminder_out );	
    
}



/////////////////////SCALAR MATRIX SEQUENCE VALUE_IN///////////////////////////////
QdObject_var  GetRemainder(    QdRMatrixSeq_var qMatSeqVal,  
                               QdRScalar_var    qDivider )
{
    QdObject_var qMatNullObj;
    
    if ( qMatSeqVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The sequence of matrix of scalar value for value_in is NULL\n");
          return ( qMatNullObj );
    }
     
     
    if ( qDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The scalar value for divider_in is NULL\n");
          return ( qMatNullObj );
    }
    
    int          nData        = qMatSeqVal->sequence_size();
    float        fDivider     = 0.;
    QdObject_var qReminder_out;
    
    fDivider = (float)(*qDivider);
    if ( fDivider == 0. )
    {
        QplugAppendTextDisplay( "Remainder>>The scalar value of the divider is set to zero (0)" );
	return ( qMatNullObj );
    }
    
    
    QdRMatrixSeq_var qMatSeqRemainder = new QdRMatrixSeq( nData, 
                                                          RMatrix( qMatSeqVal[0].dims() ) 
							 );
    if ( qMatSeqRemainder.is_nil() )
    {
        QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar matrix sequence\n");
        return ( qMatNullObj );
    }
    
    for ( long nSeq = 0; nSeq < nData; nSeq++ )
    {
    
        QdRMatrix::iterator itrRes    = qMatSeqRemainder[ nSeq ].begin();
	QdRMatrix           matVal    = qMatSeqVal[ nSeq ];
		
        for ( 
                QdRMatrix::iterator itrVal = matVal.begin(); 
                itrVal != matVal.end()                     ; 
	        itrVal++
	    )
        {
             // Calculate the remainder
	     *itrRes = fmod( *itrVal, *qDivider );
       	     itrRes++;
        }
    }
      
    qReminder_out = ( QdObject_var )qMatSeqRemainder;
    return ( qReminder_out );	
}



QdObject_var  GetRemainder( QdRMatrixSeq_var qMatSeqVal,  
                            QdRScalarSeq_var qSeqDivider )
{
    QdObject_var qMatNullObj;
    if ( qMatSeqVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The sequence of matrix of scalar value for value_in is NULL\n");
          return ( qMatNullObj );
    }
     
     
    if ( qSeqDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The sequence of scalars for divider_in is NULL\n");
          return ( qMatNullObj );
    }
    
    int          nData        = qMatSeqVal->sequence_size();
    float        fDivider     = 0.;
    QdObject_var qReminder_out;
 
    
    QdRMatrixSeq_var qMatSeqRemainder = new QdRMatrixSeq( nData, 
                                                          RMatrix( qMatSeqVal[0].dims() ) );
    
    if ( qMatSeqRemainder.is_nil() )
    {
        QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar matrix sequence\n");
        return ( qMatNullObj );
    }
      
    for ( long nSeq = 0; nSeq < nData; nSeq++ )
    {
        QdRMatrix::iterator itrRes    = qMatSeqRemainder[ nSeq ].begin();
	QdRMatrix           matVal    = qMatSeqVal[ nSeq ];
		
        for ( 
                QdRMatrix::iterator itrVal = matVal.begin(); 
                itrVal != matVal.end()                     ; 
	        itrVal++
	    )
        {
             // Calculate the remainder
	     fDivider = (float)qSeqDivider[ nSeq ];
	     if ( fDivider == 0. )
	     {
	        *itrRes = 0.;
	     }
	     else
	     {
	        *itrRes = fmod( *itrVal, qSeqDivider[ nSeq ] );
	     }
       	     itrRes++;
        }

     }
     
     qReminder_out = ( QdObject_var )qMatSeqRemainder;
     return ( qReminder_out );	
    
}



QdObject_var  GetRemainder(    QdRMatrixSeq_var qMatSeqVal,  
                               QdRMatrix_var    qMatDivider )
{
    QdObject_var qMatNullObj;
    if ( qMatSeqVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The sequence of matrix of scalar value for value_in is NULL\n");
          return ( qMatNullObj );
    }
     
     
    if ( qMatDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The matrix of scalars for divider_in is NULL\n");
          return ( qMatNullObj );
    }
    
    int   nData        = qMatSeqVal->sequence_size();
    float fDivider     = 0.;
    QdObject_var qReminder_out;
    
    if ( !qMatSeqVal[ 0 ].samesize( (*qMatDivider) ) )
    {
        QplugAppendTextDisplay( "Remainder>>Error - The matrixes for value_in and divider_in do not have the same number and dimension size" );
	return ( qMatNullObj );
    }
   
    QdRMatrixSeq_var qMatSeqRemainder = new QdRMatrixSeq( nData, 
                                                          RMatrix( qMatDivider->dims() ) );
    if ( qMatSeqRemainder.is_nil() )
    {
        QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar matrix sequence\n");
        return ( qMatNullObj );
    }
    
    QdRMatrix           matDiv    = ( *qMatDivider );
    QdRMatrix::iterator itrDiv;
    
    for ( long nSeq = 0; nSeq < nData; nSeq++ )
    {
        QdRMatrix::iterator itrRes    = qMatSeqRemainder[ nSeq ].begin();
	QdRMatrix           matVal    = qMatSeqVal[ nSeq ];
		
        for ( 
                QdRMatrix::iterator itrVal = matVal.begin(), itrDiv = matDiv.begin(); 
                itrVal != matVal.end() && itrDiv != matDiv.end()                    ; 
	        itrVal++, itrDiv++
	    )
        {
             // Calculate the remainder
	     fDivider = (float)( *itrDiv );
	     if ( fDivider == 0. )
	     {
	        *itrRes = 0.;
	     }
	     else
	     {
	        *itrRes = fmod( *itrVal, *itrDiv );
	     }
	     
       	     itrRes++;
        }

    }

    qReminder_out = ( QdObject_var )qMatSeqRemainder;
    return ( qReminder_out );	
     
}




QdObject_var  GetRemainder(   QdRMatrixSeq_var   qMatSeqVal, 
                              QdRMatrixSeq_var   qMatSeqDivider )
{

    // We do assume that all the dimension are the same accross each element
    // of the sequence of matrixes for both input objects
    QdObject_var qMatNullObj;
    if ( qMatSeqVal.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The sequence of matrix of scalar value for value_in is NULL\n");
          return ( qMatNullObj );
    }
     
     
    if ( qMatSeqDivider.is_nil() )
    {
          QplugAppendTextDisplay( "Remainder>> The sequence of scalars matrix for divider_in is NULL\n");
          return ( qMatNullObj );
    }
    
    int          nData        = qMatSeqVal->sequence_size();
    float        fDivider     = 0.;
    QdObject_var qReminder_out;
    
   
    if (  !qMatSeqVal[ 0 ].samesize( qMatSeqDivider[ 0 ] ) )
    {
        QplugAppendTextDisplay( "Remainder>>Error - The matrixes for value_in and divider_in do not have the same number and dimension size" );
	return ( qMatNullObj );
    }
    
   
    QdRMatrixSeq_var qMatSeqRemainder = new QdRMatrixSeq( nData, 
                                                          RMatrix( qMatSeqVal[0].dims() ) );
							  
    if ( qMatSeqRemainder.is_nil() )
    {
        QplugAppendTextDisplay( "Remainder>> Cannot allocate memory for remainder scalar matrix sequence\n");
        return ( qMatNullObj );
    }
  
    for ( long nSeq = 0; nSeq < nData; nSeq++ )
    {
        QdRMatrix::iterator itrRes    = qMatSeqRemainder[ nSeq ].begin();
	QdRMatrix           matVal    = qMatSeqVal[ nSeq ];
	QdRMatrix           matDiv    = qMatSeqDivider[ nSeq ];
        QdRMatrix::iterator itrDiv;
		
        for ( 
                QdRMatrix::iterator itrVal = matVal.begin(), itrDiv = matDiv.begin(); 
                itrVal != matVal.end() && itrDiv != matDiv.end()                    ; 
	        itrVal++, itrDiv++
	    )
        {
             // Calculate the remainder
	     fDivider = (float)( *itrDiv );
	     if ( fDivider == 0. )
	     {
	        *itrRes = 0.;
	     }
	     else
	     {
	        *itrRes = fmod( *itrVal, *itrDiv );
	     }
	     
       	     itrRes++;
        }

    }
    						  
    qReminder_out = ( QdObject_var )qMatSeqRemainder;
    return ( qReminder_out );	
     
}



template <class T >
QdObject_var      ProcessRemainder( T                rVal, 
                                    QdRScalar_var    rDivider,
                                    QdRScalarSeq_var rSeqDivider,
                                    QdRMatrix_var    rMatDivider,
                                    QdRMatrixSeq_var rMatSeqDivider )
{
       QdObject_var qReminder_out;
       
       if ( !rDivider.is_nil() )
       {
           // ( 5.1 ) Divider is a scalar
	   // SCALAR OBJECTS
           qReminder_out = GetRemainder( rVal, rDivider );
           if ( qReminder_out.is_nil() )
           {
               QplugAppendTextDisplay ("Remainder>> ERROR - divider_in ( scalar value ) is set to zero\n");
               return ( qReminder_out );
           }
      
                  
       }
       else if ( !rSeqDivider.is_nil() )
       {
           // ( 5.2 ) Divider is a series of scalars
	   qReminder_out = GetRemainder( rVal, rSeqDivider );
	   if ( qReminder_out.is_nil() )
           {
               QplugAppendTextDisplay ("Remainder>> ERROR - Calculating remainder of scalar by a scalar series\n");
               return ( qReminder_out );
           }
      
           	
       }
       else if ( !rMatDivider.is_nil() )
       {
           // ( 5.3 ) Divider is a matrix of scalars
	   qReminder_out = GetRemainder( rVal, rMatDivider );
	   if ( qReminder_out.is_nil() )
           {
               QplugAppendTextDisplay ("Remainder>> ERROR - Calcualting remainder of a scalar by a matrix\n");
               return ( qReminder_out );
           }
      
           
       
       }
       else if ( !rMatSeqDivider.is_nil() )
       {
           // ( 5.2 ) Divider is a series of scalar matrixes
	   qReminder_out = GetRemainder( rVal, rMatSeqDivider );
	   if ( qReminder_out.is_nil() )
           {
               QplugAppendTextDisplay ("Remainder>> ERROR - Calculating remainder for a sequence of matrix\n");
               return ( qReminder_out );
           }
      
           
       }
       else
       {
           // Type not being supported within this plug-in
	   QplugAppendTextDisplay ("Remainder>> ERROR - input object types for divider not handled \n");
           return ( qReminder_out );
       }
       
       return ( qReminder_out );
}

			       



