/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/
/* PROJECT: Cluster UK CDHF
*
* COMPONENT:    makeVectorXYZ
* 
* MODULE:	QSAS construct Vector Time Series from 3 scalars
*
* LANGUAGE:	C++
*
* FUNCTION:
*
* PURPOSE: 
* 
* DESCRIPTION:
*
* ARGUMENTS:
*    IN:
*     SCALAR_TIME_SERIES    x_in                 input x time series 
*     SCALAR_TIME_SERIES    y_in                 input y time series 
*     SCALAR_TIME_SERIES    z_in                 input z time series 
*     STRING               Frame_in          Frame attribute  
*    OUT:
*        VECTOR_TIME_SERIES ts_out
* GLOBALS:
*
* RETURN:
*    TYPE:     QplugReturnStatus
*    VALUES:   see qplug_if.h
*
* FUNCTIONS_USED:	 
*
* ALGORITHM:  
*
* HISTORY:	Tony Allen Oct 2005
*
***********************************************************************/


#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "Qdos.h"
#include "qdutil.h"
#include "makeVectorXYZ.h"

using namespace QSAS;

QplugReturnStatus  makeVectorXYZ  ( QplugArgList *call_list )

{

QdRScalarSeq_var  x_in;
QdRScalarSeq_var  y_in;
QdRScalarSeq_var  z_in;
QdString_var  frame;

string string_ptr;
char fieldnam[150];                  // attributes
char siconversion[150];
char units[150];

//
//	...first, unpack the input data objects ready to join
//

 QdObjectSeq_var toJoin  = new QdObjectSeq(3);
 QdObjectSeq_var joined  = new QdObjectSeq(3);
 QdStringSeq_var joinMethod = new QdStringSeq(3);
 vector <double> gapValue(3);
 QdStringSeq_var gapHandler = new QdStringSeq(3);

//	..first 3 items are input Time Series

x_in = QdRScalarSeq_var::narrow((*call_list)[0]);
if( x_in.is_nil() ) {
   QplugAppendTextDisplay ("makeVectorXYZ>> ERROR - input x not a Scalar Time Series!\n");
	return QPLUG_FAILURE; 
}

toJoin[0] = x_in;
joined[0]  = x_in->create();
QdTimeSeq_var x_tt = get_timetags(x_in);
joinMethod[0] = QdString(LINEAR);
gapHandler[0] = QdString(LINEAR);

y_in = QdRScalarSeq_var::narrow((*call_list)[1]);
if( y_in.is_nil() ) {
   QplugAppendTextDisplay ("makeVectorXYZ>> ERROR - input y not a Scalar Time Series!\n");
	return QPLUG_FAILURE; 
}
toJoin[1] = y_in;
joined[1]  = y_in->create();
QdTimeSeq_var y_tt = get_timetags(y_in);
joinMethod[1] = QdString(LINEAR);
gapHandler[1] = QdString(LINEAR);


z_in = QdRScalarSeq_var::narrow((*call_list)[2]);
if( z_in.is_nil() ) {
   QplugAppendTextDisplay ("makeVectorXYZ>> ERROR - input z not a Scalar Time Series!\n");
	return QPLUG_FAILURE; 
}
toJoin[2] = z_in;
joined[2]  = z_in->create();
QdTimeSeq_var z_tt = get_timetags(z_in);
joinMethod[2] =  QdString(LINEAR);
gapHandler[2] =  QdString(LINEAR);


// Compare units (SI_conversion)

string y_SIc = string(QuGetAttrText("SI_conversion", y_in));
string z_SIc = string(QuGetAttrText("SI_conversion", z_in));
string x_SIc = string(QuGetAttrText("SI_conversion", x_in));

if( x_SIc != y_SIc || x_SIc != z_SIc){
   QplugAppendTextDisplay ("makeVectorXYZ>> ERROR - input SIconversions differ\n");
   return QPLUG_FAILURE; 
}


// Compare time tags and join

 double deltat, avg_width;
 QdTimeSeq_var tt_target = Factory::clone(x_tt); 
 QuIsTSRegular(tt_target, &deltat); // don't care about regular, but need estimate of spacing
 avg_width = 1.5 * deltat;
 gapValue[0] = avg_width;
 gapValue[1] = avg_width;
 gapValue[2] = avg_width;
 
try {
   QuMultiJoin(toJoin, tt_target, joined, gapValue, gapHandler, joinMethod);
 }
 catch(Exception &e) {
   printf("Exception : avg_width = %f\n", avg_width);
   e.print_msg();
   fflush(stdout);
   return QPLUG_FAILURE;
 }

frame = QdString_var:: narrow((*call_list)[3]);
if( frame.is_nil() ) {
   QplugAppendTextDisplay ("makeVectorXYZ>> ERROR - input frame not a String!\n");
	return QPLUG_FAILURE; 
}


// Make vector 

int sz = tt_target->size();
QdRMatrixSeq_var  ts_out = new QdRMatrixSeq(sz, Dim(3));
QdRScalarSeq_var x_out = QdRScalarSeq_var::narrow(joined[0]);
QdRScalarSeq_var y_out = QdRScalarSeq_var::narrow(joined[1]);
QdRScalarSeq_var z_out = QdRScalarSeq_var::narrow(joined[2]);

if( x_out.is_nil() || y_out.is_nil() || z_out.is_nil() ){
   QplugAppendTextDisplay ("Join of input scalar sequences failed\n");
	return QPLUG_FAILURE; 
}

for(register int i=0; i<sz; i++){
   ts_out[i][0] = x_out[i];
   ts_out[i][1] = y_out[i];
   ts_out[i][2] = z_out[i];
}

// add tt
 
 set_timetags(ts_out, tt_target);
 
// Set attributes

   string_ptr = QuGetAttrText("UNITS", x_in);
   strcpy(units,string_ptr.c_str());



// load all the attributes, and the half spacing/sample period:

 QuSetTxtAttr("FIELDNAM", "composed vector", ts_out);
 QuSetFrame(ts_out, frame->c_str());
 QuSetTxtAttr("SI_conversion", x_SIc.c_str(), ts_out);
 QuSetTxtAttr("UNITS", units, ts_out);

//	..add  the output series to the list

  call_list->push_back(ts_out);     

// Clean up:


QplugAppendTextDisplay ("makeVectorXYZ>> data object created - exiting successfully\n");

return QPLUG_SUCCESS;

}
