#include "QbTimeEditors.h"


void QbEditor::updateForRec(){

	if(record > maxRec) return;

	if (editorType == "QbScalarSeqEditor") ((QbScalarSeqEditor *)this)->updateForRec();
	else if (editorType == "QbMatrixSeqEditor") ((QbMatrixSeqEditor *)this)->updateForRec();
	else if (editorType == "QbTextSeqEditor") ((QbTextSeqEditor *)this)->updateForRec();
	else if (editorType == "QbTtagEditor") ((QbTtagEditor *)this)->updateForRec();
	else if (editorType == "QbEventTableEditor") ((QbEventTableEditor *)this)->updateForRec();

}

QbTimeIntervalEditor::QbTimeIntervalEditor( QdObject_var dobj, QWidget *parent, const char *name )
	:QbEditor(dobj, parent, name)
{
	setTitle("Time Interval");
	
	interval_edit=new TimeIntervalWidget(this, 1);
	editorType = "QbTimeIntervalEditor";
	
	setData(dobj);
	
	this->layout()->addWidget(interval_edit);
	connect(interval_edit, SIGNAL(intervalUpdated()), this, SLOT(updateTI()));
}

void QbTimeIntervalEditor::setReadOnly(bool readOnly)
{
	
	valueField->setReadOnly(readOnly);
	timeField->setReadOnly(readOnly);
	
	interval_edit->setReadOnly(readOnly);
	
}

void QbTimeIntervalEditor::updateTI()
{
	_time_interval_var->set(interval_edit->getStart(), interval_edit->getEnd());
		
	edited_var = _time_interval_var;

}

void QbTimeIntervalEditor::setData(QdObject_var dobj)
{
	showDep0Field(false);
	showValueField(false);

	interval_edit->setInterval(dobj);

	if( dobj.is_nil() )
	{ 
		QmjdTime end_time;
		end_time.set_today();
		QmjdTime start_time(end_time-86400.);
		_time_interval_var=new QdTimeInterval(start_time, end_time);
		return;
	}
	
	QdTimeInterval_var tint=QdTimeInterval_var::narrow(dobj);
   
	if( !tint.is_nil() ){
   
		_time_interval_var = Factory::clone(tint);
		_time_interval_var->copy_xrefs_from(dobj);

		string objectName;
		if(dobj->xref_exists(OBJECT_NAME))
		{
			QdString_var objectName_var=QdString_var::narrow(dobj->get_xref(OBJECT_NAME));
			if(!objectName_var.is_nil())
				objectName="Time Intervals/"+*objectName_var;
		}
	}
}

void QbTimeIntervalEditor::save(Vostream& v_os)
{
	QdTimeInterval_var TIobj=QdTimeInterval_var::narrow(Factory::clone(*_time_interval_var));
	v_os << TIobj;

}
void QbTimeIntervalEditor::restore(Vistream& v_is)
{
	QdTimeInterval_var TIobj;
	v_is >> TIobj;
	this->set_time_interval(*TIobj);
}

void QbTimeIntervalEditor::set_time_interval(const QmjdTimeInterval& tintv, bool setViewRangeToo)
{
// 	QdTimeInterval_var temp=new QdTimeInterval(tintv);
	interval_edit->setInterval(tintv, setViewRangeToo);
}

bool QbTimeIntervalEditor::get_time_interval( QmjdTimeInterval& tintv ) const
{
	 tintv=interval_edit->getInterval();
	return true;
}

void QbTimeIntervalEditor::set_view_range(const QmjdTimeInterval& tintv)
{
    interval_edit->setViewRange(tintv);
}

double QbTimeIntervalEditor::get_duration()
{
	return (interval_edit->getDuration());
}



// ----------------------------------------------------------



QbEventTableEditor::QbEventTableEditor( QdObject_var dobj, QWidget *parent, QString name )
	:QbEditor(dobj, parent, name.toLatin1().data())
{
	setTitle(name);
	
	interval_edit = new TimeIntervalWidget(this, 2);
	
	setData(dobj);
	interval_edit->dobjPath = name;

	this->layout()->addWidget(interval_edit);
	
	editorType="QbEventTableEditor";
}


void QbEventTableEditor::setData(QdObject_var dobj)
{
	showDep0Field(false);
	showValueField(false);
	if(dobj.is_nil()) return;

	QdTimeIntervalSeq_var seq=QdTimeIntervalSeq_var::narrow(dobj);
	
	if(!seq.is_nil() )
	{
		interval_edit->setInterval(seq);
		interval_edit->setEventTable(seq);
		maxRec=seq->sequence_size()-1;

		_event_table_var=Factory::clone(seq);
		_event_table_var->copy_xrefs_from(dobj);
		
		return;
	}
	else
		QuMsgBar("Data object not recognised", true);
}



void QbEventTableEditor::set_view_range(const QmjdTimeInterval& tintv)
{
    interval_edit->setViewRange(tintv);
}

bool QbEventTableEditor::get_time_interval( QmjdTimeInterval& tintv ) const
{
    tintv=interval_edit->getInterval();
    return true;
}

void QbEventTableEditor::updateForRec()
{
	if(!_event_table_var.is_nil())
	{
		interval_edit->setInterval(_event_table_var[record], false);
	}
}

void QbEventTableEditor::setReadOnly(bool readOnly)
{
	
	valueField->setReadOnly(readOnly);
	timeField->setReadOnly(readOnly);
	
	interval_edit->setReadOnly(readOnly);
	
}

void QbuLineEdit::focusOutEvent(QFocusEvent *e)
{
	const QValidator *v=validator();
	int curpos=cursorPosition();
	QString str=text();
	bool changed=str!=_old_text;
	if(changed)
	{
		if(v)
		{
			if(v->validate(str,curpos)==QValidator::Acceptable)
			{
				if(curpos!=cursorPosition())
					setCursorPosition(curpos);
				if(str!=text())
					setText(str);
			}
			else 
			{
				if(curpos!=cursorPosition())
					setCursorPosition(curpos);
				v->fixup(str);
				if(str!=text())
					setText(str);
			}
		}
		else
		{
			if(curpos!=cursorPosition())
				setCursorPosition(curpos);
			if(str!=text())
				setText(str);
		}
	}
	QLineEdit::focusOutEvent(e);
	if(changed) 
		emit focusOut();
}


