//  qgu.cc
//
// GUI Utility Widgets
//
// Mar 2010: Tony Allen
//


#include "qgu.h"
#include "qwui.h"

#include <stdio.h>

extern qwuiWindow* qsasWindow;




// QguWindow class
//-----------------
DataNamesTable_var QguWindow::_saved_objects=new DataNamesTable();
DataNamesTable_var QguWindow::_restored_objects=new DataNamesTable();

/// \fn QguWindow::QguWindow( QWidget *parent, const char *name )
/// \brief Constructor of the QguWindow class
/// \param     parent of type QWidget *
/// \param     name   of type const char *
/// \exception None
/// \return    None
///
  QguWindow::QguWindow( QWidget *parent, const char *name )
          :QMainWindow(0) // not QMainWindow(parent,name) any more. If so, the children windows are always *over* the main window on Windows, which feels weird.
{
  msgBar=statusBar();
  _menu_bar=menuBar();
  
  _name = QString(name);
  
  QString userFolder = QSAS_profiles->current()->SaveRestorePath;

  if (userFolder.isEmpty()) this->_filename = QString(".");
  else                    this->_filename = userFolder;

  this->SAVE_EXTENSION = QString(".qs_qgu_default");
  fromSession=false;

}

QguWindow::~QguWindow()
{
 // QguWindow destructor
}

/// \fn void QguWindow::bringToFront()
/// \brief Method that brings the current main window to the foreground and activates it.
/// \param     None
/// \exception None
/// \return    None
///
void QguWindow::bringToFront(){
  this->activateWindow();
  this->raise();
}


/// \fn void QguWindow::save()
/// \brief Method saves the content of the window to a local file.  Last used save file name is used if it exists, or default.
/// \param     None
/// \exception None
/// \return    None
///
 void QguWindow::save()
 {

	if(QuSlotDebug())
 	{
		printf("QgWindow::save %ld\n", (long)this);
		fflush(stdout);
 	}

    if(_filename.isEmpty())
    {
       saveAs();
       return;
    }

	if(_filename.contains(".qx_") ){
		saveAsXML(_filename);
	}

	QString fileNameCopy = _filename;
	fileNameCopy.replace(".qx_", ".qs_"); // now do old XDR save as well

    QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
    qApp->processEvents();

    try
    {
      VXDRostream xdr_os(fileNameCopy.toLatin1().data());
      save(xdr_os);
    }
    catch(Exception &e)
    {
       status(e.msg().c_str(),10000);
       return;
    }

    QApplication::restoreOverrideCursor();
    qApp->processEvents();

    status(_filename+" saved",10000);
    return;
 }


/// \fn void QguWindow::saveAs()
/// \brief Method saves the content of the working list within a local file using the filename as provided by the user
/// \param     None
/// \exception None
/// \return    None
///
void QguWindow::saveAs()
{
   QString _filter = QString("save (*");
   QString extn = SAVE_EXTENSION;
   extn.replace(".qx_", ".qs_"); // this is old xdr save 
   _filter += extn;
   _filter += QString(")");
   QString xdr_file = _filename;
   xdr_file.replace(".qx_", ".qs_"); 

   QString fn=QFileDialog::getSaveFileName(this, tr("Select file - Save Settings"), xdr_file, _filter);
   if(!(fn.isEmpty()) )
   {
     QFileInfo fn_fi(fn);
     QString tmp=fn_fi.fileName();
     int pos = tmp.lastIndexOf(extn);
     if (pos < 0) {
       pos = tmp.lastIndexOf(".");
       if (pos > 0) tmp.truncate(pos);
       tmp += extn;
     }
     _filename=fn_fi.path()+"/"+tmp;
     save();
   }
   else
     status("saving aborted ",10000);
 }


  
 /// \fn void QguWindow::saveWithOut()
/// \brief Method that saves the content of the current main window without the data using the previously provided filename
/// \param     None
/// \exception None
/// \return    None
///
void QguWindow::saveWithOut()
 { with=0; save();}
 
  
/// \fn void QguWindow::saveWithOutAs()
/// \brief Method that saves the content of the current main window without the data prompting the user to enter a filename
/// \param     None
/// \exception None
/// \return    None
///
void QguWindow::saveWithOutAs()
 { with=0; saveAs();}
 

void QguWindow::include()
{
  restore(false);
}


/// \fn void QguWindow::restore(bool replace)
/// \brief Method that restores the content of the current window using the file selected by the user (or from session restore)
/// \param     bool replace
/// \exception None
/// \return    None
///
//
//
void QguWindow::restore(bool replace)
{
	if(QuSlotDebug())
	{
		printf("QguWindow::restore %ld replace = %d name = %s\n", (long)this, replace, _name.toLatin1().data());
		fflush(stdout);
	}

   QString fn;
   if(!fromSession){
      fn=QFileDialog::getOpenFileName(this, tr("select file to open"), _filename, _saveset_filter);
      _filename=fn;
   }
   fromSession = false; // reset for save from GUI

   if(!(_filename.isEmpty()) )
   {
		QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
		qApp->processEvents();
		
		if(_filename.contains(".qx_")) {
			// XML saveset
			bool xml_ok = restoreXML(_filename, replace);
			if(xml_ok){
				QApplication::restoreOverrideCursor();
      			qApp->processEvents();
				return;
			}
			else {
				// try old style saveset
				_filename.replace(".qx_", ".qs_");
			}

		}
		
		try
		{
        	VXDRistream xdr_is(_filename.toLatin1().data());
        	restore(xdr_is,replace);
	    	status(_filename+" restored",10000);
		}
		catch(Exception &e)
		{
         	QApplication::restoreOverrideCursor();
         	qApp->processEvents();
         	status(e.msg().c_str(),10000);
         	return;
		}

		QApplication::restoreOverrideCursor();
		qApp->processEvents();

   }
   else
		QuMsgBar("Empty file name", true);
 }


/// \fn void QguWindow::saveSession(QString folder)
/// \brief Method that saves the content of this window when saving the current QSAS session
/// \param     folder of type QString
/// \exception None
/// \return    None
///
 void QguWindow::saveSession(QString folder)
 {
  if(folder.isEmpty()) return;
  // keep original name
  QString fileN = QString(_filename.toLatin1().data());

  // set session folder and save
     this->_filename = QString(folder.toLatin1().data());
     this->_filename += QString("/saveSet");
     this->_filename += this->SAVE_EXTENSION;

	// if(this->SAVE_EXTENSION.contains(".qx_")) saveAsXML(this->_filename);
	saveWithOut(); // will save both XML and XDR

  // put file name back
  _filename = QString(fileN.toLatin1().data());
}


/// \fn void QguWindow::restoreSession(QString folder)
/// \brief Method that restores the content of this window when restoring a session
/// \param     folder of type QString 
/// \exception None
/// \return    None
///
void QguWindow::restoreSession(QString folder){

	if ( QuSlotDebug() )
	{
		printf( "QguWindow::restoreSession %ld\n", (long)this );
		fflush(stdout);
	}
    
	if(folder.isEmpty()) return;
	// keep original name
	QString fileN = QString(_filename.toLatin1().data());
  
	// insert session folder and save
	this->_filename = QString(folder.toLatin1().data());
	this->_filename += QString("/saveSet");
	this->_filename += this->SAVE_EXTENSION;
	this->fromSession = true;
	
	restore(true);
  
	// put file name back
	_filename = QString(fileN.toLatin1().data());
}

   /// \fn void status(const QString& str)
  /// \brief Function that displays the message str within the current window message bar.
  /// \param     str of const QString& type
  /// \exception None
  /// \return    None
  ///
 void QguWindow::status(const QString& str)
  {
      msgBar->showMessage(str);
	  if(qApp) qApp->processEvents();
  }

  /// \fn void status(const QString& str,int ms)
  /// \brief Function that displays the message str within the current window message bar
  /// \param     str of const QString& type
  /// \param     ms  of int            type
  /// \exception None
  /// \return    None
  ///
  void QguWindow::status(const QString& str,int ms)
  {
    msgBar->showMessage(str,ms);
    if(qApp) qApp->processEvents();
  }
