#include "qt_version.h"

#include "PlotConfig2.h"

extern ColourTablePalette importedPalette;

PlotConfig2::PlotConfig2()
{
    QGridLayout * mainLayout=new QGridLayout(this);
    
    
    QGroupBox * fontsGroup=new QGroupBox("Fonts in plots", this);
    QGridLayout* fontsLayout=new QGridLayout;
    fontsLayout->setSpacing(QSAS_profiles->current()->GUImargins);
    
    QLabel * warning=new QLabel("Selected profile determines the font used in a screen plot. PDF and EPS plots embed the chosen fonts.", fontsGroup);
    warning->setWordWrap(true);
    fontsLayout->addWidget(warning, 0, 0, 1, 3);
    
    fontsLayout->addWidget(new QLabel("Sans font", fontsGroup), 1, 0);
    fontsLayout->addWidget(new QLabel("Serif font", fontsGroup), 2, 0);
    fontsLayout->addWidget(new QLabel("Monotype font", fontsGroup), 3, 0);
    fontsLayout->addWidget(new QLabel("Decorative font", fontsGroup), 4, 0);
    
    sansFontCheck=new QCheckBox("Default", fontsGroup);
    fontsLayout->addWidget(sansFontCheck, 1, 1);
    connect(sansFontCheck, SIGNAL(stateChanged(int)), this, SLOT(sansCheckChanged()));
    sansFontCombo=new QFontComboBox(fontsGroup);
    fontsLayout->addWidget(sansFontCombo, 1, 2);
    
    serifFontCheck=new QCheckBox("Default", fontsGroup);
    fontsLayout->addWidget(serifFontCheck, 2, 1);
    connect(serifFontCheck, SIGNAL(stateChanged(int)), this, SLOT(serifCheckChanged()));
    serifFontCombo=new QFontComboBox(fontsGroup);
    fontsLayout->addWidget(serifFontCombo, 2, 2);
    
    monotypeFontCheck=new QCheckBox("Default", fontsGroup);
    fontsLayout->addWidget(monotypeFontCheck, 3, 1);
    connect(monotypeFontCheck, SIGNAL(stateChanged(int)), this, SLOT(monotypeCheckChanged()));
    monotypeFontCombo=new QFontComboBox(fontsGroup);
    fontsLayout->addWidget(monotypeFontCombo, 3, 2);
    
    decorativeFontCheck=new QCheckBox("Default", fontsGroup);
    fontsLayout->addWidget(decorativeFontCheck, 4, 1);
    connect(decorativeFontCheck, SIGNAL(stateChanged(int)), this, SLOT(decorativeCheckChanged()));
    decorativeFontCombo=new QFontComboBox(fontsGroup);
    fontsLayout->addWidget(decorativeFontCombo, 4, 2);
    
    fontsGroup->setLayout(fontsLayout);
    
    mainLayout->addWidget(fontsGroup, 0, 0);
    
#if QT_VERSION >= 0x040300
    svgFontsCheck=new QCheckBox("Vectorise fonts in SVG, EPS or PDF export", this);
    mainLayout->addWidget(svgFontsCheck, 1, 0);
#endif
    
    QGroupBox * colourTableGroup=new QGroupBox("Custom colour tables", this);
    QVBoxLayout * colourTableLayout=new QVBoxLayout;
    colourTableLayout->setSpacing(QSAS_profiles->current()->GUImargins);
    
//     colourTableLayout->addWidget(new QLabel("See online help for syntax (""Profiles"" page). Undefined behaviour for wrong syntaxes."));
//     colourTableEdit=new QLineEdit(this);
//     colourTableLayout->addWidget(colourTableEdit);

    paletteEdit=new ColourTableList;
    colourTableLayout->addWidget(paletteEdit);

    QHBoxLayout* paletteButtonsLayout=new QHBoxLayout;

    QPushButton* addButton=new QPushButton("Add new");
    connect(addButton, SIGNAL(clicked()), paletteEdit, SLOT(add()));
    paletteButtonsLayout->addWidget(addButton);
    QPushButton* importButton=new QPushButton("Add imported");
    connect(importButton, SIGNAL(clicked()), this, SLOT(showImportedTables()));
    paletteButtonsLayout->addWidget(importButton);
    QPushButton* mergeButton=new QPushButton("Add from profile");
    connect(mergeButton, SIGNAL(clicked()), this, SLOT(showProfilesMenu()));
    paletteButtonsLayout->addWidget(mergeButton);
    QPushButton* deleteButton=new QPushButton("Delete");
    connect(deleteButton, SIGNAL(clicked()), paletteEdit, SLOT(deleteSelected()));
    paletteButtonsLayout->addWidget(deleteButton);
    QPushButton* editButton=new QPushButton("Edit");
    connect(editButton, SIGNAL(clicked()), paletteEdit, SLOT(editSelected()));
    paletteButtonsLayout->addWidget(editButton);
    
    colourTableLayout->addLayout(paletteButtonsLayout);
    
    colourTableGroup->setLayout(colourTableLayout);
    
    mainLayout->addWidget(colourTableGroup);
    
    
    QGroupBox * plotEltsGroup=new QGroupBox("Defaults for Plot Elements", this);
    QGridLayout* plotEltsLayout=new QGridLayout;
    plotEltsLayout->setSpacing(QSAS_profiles->current()->GUImargins);
    
    plotEltsLayout->addWidget(new QLabel("Default line width."), 0, 0, 1, 2);
    lineWidthBox=new QDoubleSpinBox(this);
    lineWidthBox->setRange(0.5, 10.);
    lineWidthBox->setDecimals(2);
    lineWidthBox->setSingleStep(0.1);
    plotEltsLayout->addWidget(lineWidthBox, 0, 2);
    
    plotEltsLayout->addWidget(new QLabel("Base font size (for labels, +0.3 for titles, +0.1 for subtitles)."), 1, 0, 1, 2);
    baseFontSizeBox=new QDoubleSpinBox(this);
    baseFontSizeBox->setRange(0.1, 10.);
    baseFontSizeBox->setDecimals(2);
    baseFontSizeBox->setSingleStep(0.1);
    plotEltsLayout->addWidget(baseFontSizeBox, 1, 2);

    plotEltsLayout->addWidget(new QLabel("Default Gap threshold"), 2, 0);

    gapToleranceTypeBox=new QComboBox(this);
    gapToleranceTypeBox->addItem("No gap detection");
    gapToleranceTypeBox->addItem("In seconds:");
    gapToleranceTypeBox->addItem("x median data spacing:");
    plotEltsLayout->addWidget(gapToleranceTypeBox, 2, 1);
    
    gapToleranceBox=new QDoubleSpinBox(this);
    gapToleranceBox->setRange(0.1, 100.);
    gapToleranceBox->setDecimals(2);
    gapToleranceBox->setSingleStep(0.2);
    plotEltsLayout->addWidget(gapToleranceBox, 2,2);
    
    plotEltsLayout->addWidget(new QLabel("Default Axis scales from"), 3, 0);
    axisScaleBox=new QComboBox(this);
    axisScaleBox->addItem("From Metadata");
    axisScaleBox->addItem("Autoscale");
    plotEltsLayout->addWidget(axisScaleBox, 3, 1);
	
			
    plotEltsGroup->setLayout(plotEltsLayout);
    
    mainLayout->addWidget(plotEltsGroup);
}

void PlotConfig2::updateProfile(QProfile * p)
{
//  p->SansFont=sansFontEdit->text();
    if(sansFontCheck->isChecked())
    {
        p->SansFont="";
    }
    else
    {
        p->SansFont=sansFontCombo->currentFont().family();
    }
    
    if(serifFontCheck->isChecked())
    {
        p->SerifFont="";
    }
    else
    {
        p->SerifFont=serifFontCombo->currentFont().family();
    }

    if(monotypeFontCheck->isChecked())
    {
        p->MonotypeFont="";
    }
    else
    {
        p->MonotypeFont=monotypeFontCombo->currentFont().family();
    }

    if(decorativeFontCheck->isChecked())
    {
        p->DecorativeFont="";
    }
    else
    {
        p->DecorativeFont=decorativeFontCombo->currentFont().family();
    }
    
#if QT_VERSION >= 0x040300
    p->VectoriseSVGFonts=svgFontsCheck->isChecked();
#endif
//     p->ColourTable=colourTableEdit->text().replace(' ', "");
    paletteEdit->getColourTables(p->palette);
    p->LineWidth=lineWidthBox->value();
    p->BaseFontSize=baseFontSizeBox->value();
    p->gapTolerance=gapToleranceBox->value();
    switch(gapToleranceTypeBox->currentIndex())
    {
        case 1:
            p->gapToleranceType=0;
            break;

        case 2:
            p->gapToleranceType=1;
            break;

        default:
            p->gapToleranceType=-1;
            break;
    }
    switch(axisScaleBox->currentIndex())
    {
        case 0:
            p->axisScaleType=0;
            break;

        case 1:
            p->axisScaleType=1;
            break;

        default:
            p->axisScaleType=0;
            break;
    }
}

void PlotConfig2::setSelectedProfile(QProfile * p)
{

    int index=sansFontCombo->findText(p->SansFont);
    QFont font;
    font.setStyleHint(QFont::SansSerif);
    font.setFamily(p->SansFont);
    if(index<0)
    {
        sansFontCheck->setChecked(true);
        sansFontCombo->setEnabled(false);
    }
    else
    {
        sansFontCheck->setChecked(false);
        sansFontCombo->setEnabled(true);
        sansFontCombo->setCurrentIndex(index);
    }
    sansFontCombo->setCurrentFont(font);
    

    font.setStyleHint(QFont::Serif);
    font.setFamily(p->SerifFont);
    index=serifFontCombo->findText(p->SerifFont);
    if(index<0)
    {
        serifFontCheck->setChecked(true);
        serifFontCombo->setEnabled(false);
    }
    else
    {
        serifFontCheck->setChecked(false);
        serifFontCombo->setEnabled(true);
        serifFontCombo->setCurrentIndex(index);
    }
    serifFontCombo->setCurrentFont(font);

    font.setStyleHint(QFont::TypeWriter);
    font.setFamily(p->MonotypeFont);
    index=monotypeFontCombo->findText(p->MonotypeFont);
    if(index<0)
    {
        monotypeFontCheck->setChecked(true);
        monotypeFontCombo->setEnabled(false);
    }
    else
    {
        monotypeFontCheck->setChecked(false);
        monotypeFontCombo->setEnabled(true);
        monotypeFontCombo->setCurrentIndex(index);
    }
    monotypeFontCombo->setCurrentFont(font);

    font.setStyleHint(QFont::Decorative);
    font.setFamily(p->DecorativeFont);
    index=decorativeFontCombo->findText(p->DecorativeFont);
    if(index<0)
    {
        decorativeFontCheck->setChecked(true);
        decorativeFontCombo->setEnabled(false);
    }
    else
    {
        decorativeFontCheck->setChecked(false);
        decorativeFontCombo->setEnabled(true);
        decorativeFontCombo->setCurrentIndex(index);
    }
    decorativeFontCombo->setCurrentFont(font);

#if QT_VERSION >= 0x040300
    svgFontsCheck->setChecked(p->VectoriseSVGFonts);
#endif
//     colourTableEdit->setText(p->ColourTable);
    paletteEdit->setColourTables(p->palette);
    lineWidthBox->setValue(p->LineWidth);

    baseFontSizeBox->setValue(p->BaseFontSize);
    gapToleranceBox->setValue(p->gapTolerance);

    gapToleranceTypeBox->setCurrentIndex(p->gapToleranceType+1);
    axisScaleBox->setCurrentIndex(p->axisScaleType);

}

void PlotConfig2::sansCheckChanged()
{
    sansFontCombo->setEnabled(!sansFontCheck->isChecked());
}

void PlotConfig2::serifCheckChanged()
{
    serifFontCombo->setEnabled(!serifFontCheck->isChecked());
}

void PlotConfig2::monotypeCheckChanged()
{
    monotypeFontCombo->setEnabled(!monotypeFontCheck->isChecked());
}

void PlotConfig2::decorativeCheckChanged()
{
    decorativeFontCombo->setEnabled(!decorativeFontCheck->isChecked());
}

void PlotConfig2::showImportedTables()
{
    QMenu menu;

    
    if(!importedPalette.isEmpty())
    {
            
        for(int i=0; i<importedPalette.size(); ++i)
        {
            QAction* action=menu.addAction(importedPalette[i].icon, importedPalette[i].name);
            action->setData(QVariant(i));
        }

        connect(&menu, SIGNAL(triggered(QAction*)), this, SLOT(addImportedTable(QAction*)));
    }
    else
    {
        menu.addAction("No palette imported in current session");
    }
    
    menu.exec(QCursor::pos());
}

void PlotConfig2::showProfilesMenu()
{
    QMenu menu;

    std::vector<QString> names;
    QSAS_profiles->names(names);
    
    for(int i=0; i<names.size(); ++i)
    {
        QAction* action=menu.addAction(names[i]);
        action->setData(QVariant(names[i]));
    }

    connect(&menu, SIGNAL(triggered(QAction*)), this, SLOT(mergeProfileTables(QAction*)));
    
    menu.exec(QCursor::pos());
}

void PlotConfig2::addImportedTable(QAction* action)
{
    int index=action->data().toInt();
    if(index>=0 && index<importedPalette.size())
    {
        ColourTablePalette current;
        paletteEdit->getColourTables(current);
        current.push_back(importedPalette[index]);
        paletteEdit->setColourTables(current);
    }
}

void PlotConfig2::mergeProfileTables(QAction* action)
{
    QString name=action->data().toString();

    QProfile* prof=QSAS_profiles->fromName(name);
    if(prof!=NULL)
    {
        ColourTablePalette current;
        paletteEdit->getColourTables(current);
        foreach(ColourTable table, prof->palette)
        {
            current.push_back(table);
        }
        paletteEdit->setColourTables(current);
    }
}
