

#include "qslot.h"
#include "qwui.h"
#include "qdutil.h"
// #include "TimeUtils.h"

#include "qgu.h"


 indexSlider::indexSlider(int atNum, QString title,  int maxN, QWidget *parent, const char * name)
: QWidget(parent)
{

  if(QuSlotDebug())
  {
	printf(" indexSlider::indexSlider \n");
	fflush(stdout);
  }
   
   maxNum = maxN-1;
   num = atNum;
   
   QGridLayout * layout=new QGridLayout(this);
   
   layout->setSpacing(0);
   
   echoLabel=new QLabel(title, this);

   layout->addWidget(echoLabel, 0, 0, 1, 3, Qt::AlignCenter);

   QToolButton * leftSlide = new QToolButton(this);
   leftSlide->setArrowType(Qt::LeftArrow);
   leftSlide->setFixedSize(20,21);
   layout->addWidget(leftSlide, 1, 0, Qt::AlignLeft);
   numField = new QguWLinput(this,"numField");
   numField->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
   numField->setEnabled(true);
   numField->setText(QString("%1").arg(atNum+1));
   numField->setFixedWidth(40);
   numField->setAlignment(Qt::AlignHCenter);
   layout->addWidget(numField, 1, 1);
   
   QToolButton * rightSlide = new QToolButton(this);
   rightSlide->setArrowType( Qt::RightArrow);
   rightSlide->setFixedSize(20,21);
   layout->addWidget(rightSlide, 1, 2, Qt::AlignLeft);
   
   connect( rightSlide, SIGNAL(clicked()), this, SLOT(incRec()) );
   connect( leftSlide,  SIGNAL(clicked()), this, SLOT(decRec()) );
   connect( numField,   SIGNAL(edited_OK()), this, SLOT(adjustRec()) );
   
}

indexSlider::~indexSlider(){
  delete numField;
 // delete sliderBox;
}

void indexSlider::incRec(){
  if(QuSlotDebug())
  {
	printf("indexSlider::incRec %ld\n", (long)this);
	fflush(stdout);
  }

    num += 1;
    numField->setText(tr("%1").arg(num+1));
    adjustRec(); 
}

void indexSlider::decRec(){
  if(QuSlotDebug())
  {
	printf("indexSlider::decRec %ld\n", (long)this);
	fflush(stdout);
  }

    num -= 1;
    if(num < 0) num += 1; // don't go to zero as this is not caught by adjustRec due to empty field option
    numField->setText(tr("%1").arg(num+1));
    adjustRec(); 
}

void indexSlider::adjustRec(){
  if(QuSlotDebug())
  {
   printf("indexSlider::adjustRec %ld\n", (long)this);
   fflush(stdout);
  }

  bool OK;
  num = numField->text().toInt(&OK);

  if(num == 0){ // zero or empty field
    // set red
    numField->setColour(COL_NOT_OK);
    return;
  }

  num -= 1; // as feedback goes 1 to N while code goes 0 to N-1

  if(OK){
    if(num < 0) num = 0;
    if(num > maxNum) num = maxNum;
    numField->setText(tr("%1").arg(num+1));
    emit indexChanged();
  }
  else
    numField->setText("1");
    emit indexChanged();
}

// class rangeSlider
//-------------------
 rangeSlider::rangeSlider(int idx,  int MaxN, QdObject_var dep_i, QWidget *parent, char * name)
	: QGroupBox(parent)
{
	if(QuSlotDebug())
	{
		printf("rangeSlider::rangeSlider \n");
		fflush(stdout);
	}
	
	QHBoxLayout * mainLayout=new QHBoxLayout;
	
	binTextSeq = new QdStringSeq();
	QdObject_var index_label;
	QdObject_var index_units;
	
	if( !dep_i.is_nil())
	{
		if(dep_i->xref_exists("LABLAXIS") )
			index_label = dep_i->get_xref("LABLAXIS");
		if(index_label.is_nil() && dep_i->xref_exists("FIELDNAM") )
			index_label = dep_i->get_xref("FIELDNAM");

		if(dep_i->xref_exists("UNITS") )
			index_units = dep_i->get_xref("UNITS");

		// scalar 1-D array

		QdRMatrixSeq_var depS = QdRMatrixSeq_var::narrow(dep_i);
		QdRMatrix_var dep;
		// if time dependent take first matrix, else take matrix itself
		if ( !depS.is_nil() ) dep = new QdRMatrix(depS[0]);
		else dep = QdRMatrix_var::narrow(dep_i);

		if( !dep.is_nil())
		{
			for (int i=0; i<dep->size(); i++){
				QString nextBin = QString("%1").arg((*dep)[i]);
				binTextSeq->push_back( QdString(nextBin.toLatin1().data()) );
			}
		}
		else
		{
			QdSMatrix_var lab_i = QdSMatrix_var::narrow(dep_i);

			if( !lab_i.is_nil() )
			{
				for (int i=0; i<lab_i->size(); i++){
					binTextSeq->push_back( (*lab_i)[i] );
				}
			}
		}
	}
	
	QString title;

	if( !index_label.is_nil() ) {
		QdString_var indexString = QdString_var::narrow(index_label);
		if( !indexString.is_nil() ) title = QString(indexString->c_str());
		title += QString(",   ");
	}
	else{
		title = QString("index %1,   ").arg(idx+1);
	}
	
	if( !index_units.is_nil() ) {
		QdString_var unitsString = QdString_var::narrow(index_units);
		if( !unitsString.is_nil() ) {
			title += QString(unitsString->c_str());
			title += QString(",   ");
		}
	}
	
	title += QString(" %1 bins").arg(MaxN );
	this->setTitle(title);
	
	// Combo box
	actionChoice = new QComboBox(this);
	actionChoice->insertItem( 0, "elements");
	actionChoice->insertItem( 1, "sum" );
	actionChoice->insertItem( 2, "slice");
	actionChoice->insertItem( 3, "average");
	actionChoice->insertItem( 4, "alias to time");
	actionChoice->resize( actionChoice->sizeHint() );
	mainLayout->addWidget(actionChoice, 1, Qt::AlignLeft);

	fromLabel = new QLabel("   from", this);
	fromLabel->resize( fromLabel->sizeHint() );
	mainLayout->addWidget(fromLabel, 1, Qt::AlignRight);

	QString fromBinTxt;
	QString toBinTxt;
	if( binTextSeq->sequence_size() > 0 ){
		fromBinTxt = QString( (binTextSeq[0]).c_str() );
		toBinTxt = QString( (binTextSeq[ binTextSeq->sequence_size()-1]).c_str() );
	}

	fromSlide = new indexSlider(0, fromBinTxt, MaxN, this, "fromSlider" );
   	mainLayout->addWidget(fromSlide, 1, Qt::AlignLeft);
	
	toLabel = new QLabel("   to", this);
	toLabel->resize( toLabel->sizeHint() );
	mainLayout->addWidget(toLabel, 1, Qt::AlignRight);
   
	toSlide = new indexSlider(MaxN-1, toBinTxt, MaxN, this, "toSlider" );
   	mainLayout->addWidget(toSlide, 1, Qt::AlignLeft);
	
	QLabel *pad = new QLabel("  ");
   	mainLayout->addWidget(pad, 10, Qt::AlignLeft);
	
	connect( fromSlide, SIGNAL(indexChanged()), this, SLOT(updateOnIdx()) );
	connect( toSlide, SIGNAL(indexChanged()), this, SLOT(updateOnIdx()) );
	connect( actionChoice, SIGNAL(activated(int)), this,SLOT(updateRangeWidget(int)) );
	this->setEnabled(true);
	
	setLayout(mainLayout);
}



rangeSlider::~rangeSlider(){
}

void rangeSlider::updateRangeWidget(int idx)
{
  if(QuSlotDebug())
  {
	printf("rangeSlider::updateRangeWidget %ld\n", (long)this);
	fflush(stdout);
  }
   
  if(idx == 2){
     fromLabel->setText("       at");
     toSlide->hide();
	toLabel->setText("                -------------     ");

  }
  else{
     fromLabel->setText("   from");
     toSlide->show();
	toLabel->setText("   to");
  }

}

void rangeSlider::updateOnIdx()
{
  if(QuSlotDebug())
  {
	printf("rangeSlider::updateOnIdx %ld\n", (long)this);
	fflush(stdout);
  }

   int k;
   bool ok;
   // set label feedback
   if (binTextSeq.is_nil() || binTextSeq->sequence_size() == 0) return;

   // from Label
   k = fromSlide->numField->text().toLong(&ok);
   if(ok) {
     k--; // feedback is from 1 to N
	if (k < 0 || k >= binTextSeq->sequence_size() )
	{
	   fromSlide->setLabel( QString( "out of range" ) );
	}
	else
	{
	   fromSlide->setLabel( QString( (binTextSeq[k]).c_str() ) );
	}
   }
   else  fromSlide->setLabel( QString( "" ) );

   // to Label
   k = toSlide->numField->text().toLong(&ok);
   if(ok) {
     k--; // feedback is from 1 to N
	if (k < 0 || k >= binTextSeq->sequence_size() )
	{
	   toSlide->setLabel( QString( "out of range" ) );
	}
	else
	{
	   toSlide->setLabel( QString( (binTextSeq[k]).c_str() ) );
	}
   }
   else toSlide->setLabel( QString( "" ) );

}

// ---------------------------------------------------

ArrayReduceWindow::ArrayReduceWindow(QString baseName, Dim &arrayDim,  QWidget *parent, const char *name, bool modal )
  : QDialog(/*parent*/NULL)
{
  if(QuSlotDebug())
  {
	printf("ArrayReduceWindow::ArrayReduceWindow \n");
	fflush(stdout);
  }
	this->setModal(true);
	
  isSequence = FALSE;

  setAttribute(Qt::WA_DeleteOnClose);

  QVBoxLayout *array_window_top_layout=new QVBoxLayout(this);
//   array_window_top_layout->setSpacing(8);

  QMenuBar * menu_bar = new QMenuBar(this);
  array_window_top_layout->addWidget(menu_bar);

  QMenu * help = new QMenu( this );
  menu_bar->addMenu(help);
  help->setTitle(" Help");
  
  help->addAction(tr("QSAS &Manual"), this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
  help->addSeparator();
  help->addAction("...Array Reduction", this, SLOT( launchManArrayReduce() ));
  help->addSeparator();
  help->addAction("Array Module Version", this, SLOT( launchVersion() ));
  menu_bar->setFixedSize(menu_bar->sizeHint());

  this->setWindowTitle(baseName);

//   QFrame * underline = new QFrame(this, "underline");
//   underline->setFrameStyle(QFrame::HLine | QFrame::Raised);
//   underline->setLineWidth(1);
//   array_window_top_layout->addWidget(underline);

   // set up sliders for each dimension
   QdObject_var dobj = QuGetObjByName(baseName);
   for (int i=0; i < arrayDim.rank(); i++){
       QdObject_var dep_i;
	   QString dep_i_name = QString("DEPEND_%1").arg(i+1);
	   QString lab_i_name = QString("LABEL_%1").arg(i+1);
       if( !dobj.is_nil() ){
	     if( dobj->xref_exists( dep_i_name.toLatin1().data()) ) dep_i = dobj->get_xref( dep_i_name.toLatin1().data() );
		 else if ( dobj->xref_exists( lab_i_name.toLatin1().data()) ) dep_i = dobj->get_xref( lab_i_name.toLatin1().data() ); // use label_i instead
		 
	     if(dobj->sequence_size() > 1 ) isSequence = TRUE;
	   }
	   // Note that rangeSlider class can cope with dep_i.is_nil()
       rangeSlider *slider_i = new rangeSlider(i,  arrayDim[i], dep_i, this);
	   sliderStack.push_back(slider_i);
	   connect(slider_i->actionChoice, SIGNAL(activated(int)), this, SLOT(updateFeedback()) );
	   array_window_top_layout->addWidget(slider_i);
	   slider_i->setMinimumSize(slider_i->sizeHint());
   }
   
   if( !dobj.is_nil() ) {
      array_window_top_layout->addSpacing(10);
      slicer = new timeSlider(dobj, true, true, this);
      array_window_top_layout->addWidget(slicer);
   }
   else slicer = 0;
   
   

   buttons_groupbox=new QGroupBox(this);
   QHBoxLayout * buttons_layout=new QHBoxLayout;

   QPushButton *acceptBtn = new QPushButton( "Accept");
   acceptBtn->setMinimumSize(acceptBtn->sizeHint());
   connect( acceptBtn, SIGNAL(clicked()), this, SLOT(acceptSelection()) );
   acceptBtn->setFocusPolicy(Qt::StrongFocus);
   acceptBtn->setDefault(true);
   buttons_layout->addWidget(acceptBtn);

   QPushButton *setDefaultBtn = new QPushButton( "Reset");
   setDefaultBtn->setMinimumSize(setDefaultBtn->sizeHint());
   connect( setDefaultBtn, SIGNAL(clicked()), this, SLOT(setDefaults()) );
   buttons_layout->addWidget(setDefaultBtn);

   dismissBtn = new QPushButton( "Cancel", buttons_groupbox );
   dismissBtn->setMinimumSize(dismissBtn->sizeHint());
   connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
   buttons_layout->addWidget(dismissBtn);

   QString feedbackMsg = QString("Input is %1-D array").arg(arrayDim.rank());
   buttons_groupbox->setAlignment(Qt::AlignHCenter);
   buttons_groupbox->setTitle(feedbackMsg);
   array_window_top_layout->addWidget(buttons_groupbox);
   
   buttons_groupbox->setLayout(buttons_layout);

   this->setMinimumSize(this->sizeHint());

   arrHelp_window = new QswHelpWindow(this, "arrHelpWindow");
}


void ArrayReduceWindow::updateFeedback(){

  if(QuSlotDebug())
	{
		printf("ArrayReduceWindow::updateFeedback %ld\n", (long)this);
		fflush(stdout);
	}

   int nDims=0;
   for (int i=0; i<sliderStack.size(); i++){
      if( sliderStack[i]->actionChoice->currentIndex() == 0) nDims++;
   }
   QString newTitle;
   
   if(isSequence){
      if( nDims > 0)
         newTitle = QString("  Output is %1-D array sequence").arg(nDims) ;
	 else
         newTitle =  QString("  Output is a scalar sequence") ;
   }
   else{
      if( nDims > 0)
         newTitle =  QString("  Output is %1-D array").arg(nDims) ;
	 else
         newTitle =  QString("  Output is a scalar") ;
   }
   
   if (slicer != 0){
     if(slicer->sliceChoice->currentIndex() == slicer->Isubset){
	   int nrec = slicer->toSlider->record - slicer->fromSlider->record + 1;
	    newTitle += QString(", subset of %1 records  ").arg(nrec);
	 }
     if(slicer->sliceChoice->currentIndex() == slicer->Iaverage){
	   int nrec = slicer->toSlider->record - slicer->fromSlider->record + 1;
	    newTitle += QString(", averaged over %1 records to single value  ").arg(nrec);
	 }
	 if(slicer->sliceChoice->currentIndex() == slicer->Islice){
	    newTitle += QString(", value at record %1  ").arg(slicer->atSlider->record);
	 }
   }
   else  newTitle += "  ";
   
   buttons_groupbox->setTitle( newTitle);

}

void ArrayReduceWindow::dismiss(){
  if(QuSlotDebug())
  {
	printf("ArrayReduceWindow::dismiss %ld\n", (long)this);
	fflush(stdout);
  }

  close();
}

void ArrayReduceWindow::setDefaults(){
  if(QuSlotDebug())
  {
	printf("ArrayReduceWindow::setDefaults %ld\n", (long)this);
	fflush(stdout);
  }


     // set up sliders for each dimension
   for (int i=0; i<sliderStack.size(); i++){
      sliderStack[i]->actionChoice->setCurrentIndex(0);
      sliderStack[i]->updateRangeWidget(0);
      sliderStack[i]->fromSlide->numField->setText("-1");      // force reset and cleanup
	  sliderStack[i]->toSlide->numField->setText(QString("%1").arg(sliderStack[i]->toSlide->maxNum+1) );
	  sliderStack[i]->fromSlide->adjustRec();
	  sliderStack[i]->toSlide->adjustRec();
   }
   
   if (slicer != 0){
         slicer->sliceChoice->setCurrentIndex(slicer->Isubset); 
         slicer->fromSlider->adjustRec(0);
         slicer->toSlider->adjustRec(slicer->toSlider->maxRecord);
		 
         slicer->fromSlider->show();
	  	 slicer->toSlider->show();
         slicer->atSlider->hide();
   }

}

void ArrayReduceWindow::setToLastChoice(){

  if(QuSlotDebug())
  {
	printf("ArrayReduceWindow::setToLastChoice %ld\n", (long)this);
	fflush(stdout);
  }

   dimTripleStack dim_stack = parentSlot->parseArrayReduce();
   dimTriple sliceTriple = parentSlot->parseTimeSlice();

   if(dim_stack.size() == sliderStack.size()) {
   
      // set up sliders for each dimension
      for (int i=0; i<dim_stack.size(); i++){
         sliderStack[i]->actionChoice->setCurrentIndex(dim_stack[i][0]);
         sliderStack[i]->fromSlide->numField->setText(QString("%1").arg(dim_stack[i][1]) );      // force reset and cleanup
	     sliderStack[i]->fromSlide->adjustRec();
	     if(dim_stack[i][0] != 2)  {
	       sliderStack[i]->toSlide->numField->setText(QString("%1").arg(dim_stack[i][2]) );
	       sliderStack[i]->toSlide->adjustRec();
	     }
	     sliderStack[i]->updateRangeWidget(dim_stack[i][0]);
      }
   }

   if(sliceTriple.size() == 3 && slicer != 0){

      if(sliceTriple[0] == 0 || sliceTriple[0] == 3) {
	     slicer->sliceChoice->setCurrentIndex(slicer->Isubset);
         slicer->fromSlider->recBar->setValue(sliceTriple[1]);
         slicer->toSlider->recBar->setValue(sliceTriple[2]);
	  }
	  else if(sliceTriple[0] == 1) {
	     slicer->sliceChoice->setCurrentIndex(slicer->Iaverage);
		 slicer->fromSlider->recBar->setValue(sliceTriple[1]);
		 slicer->toSlider->recBar->setValue(sliceTriple[2]);
	  }
	  else if(sliceTriple[0] == 2) {
	     slicer->sliceChoice->setCurrentIndex(slicer->Islice);
		 slicer->atSlider->recBar->setValue(sliceTriple[1]);
		 slicer->updateTimeSlider(slicer->Islice);
	  }
   }
}

void ArrayReduceWindow::acceptSelection(){
  
  if(QuSlotDebug())
  {
	printf("ArrayReduceWindow::acceptSelection %ld\n", (long)this);
	fflush(stdout);
  }
  
  if(slicer != 0 && slicer->sliceChoice->currentIndex() != slicer->Islice){
  	if(slicer->fromSlider->record > slicer->toSlider->record) {
		( void )QMessageBox::warning( this, "Time Subset", "Start time after End Time, action cancelled" );
		return;
	}
  }
  
  dismissBtn->setEnabled(false); // prevent dismiss deleting window before signals complete
  
  QString newText = parentSlot->textObject();
  
  bool sliceTaken = false;
  
  for (int i=0; i<sliderStack.size(); i++){
       if( sliderStack[i]->actionChoice->currentIndex() != 0) {
	      sliceTaken=true;
		  break;
	   }
	   if( sliderStack[i]->fromSlide->num != 0){
	      sliceTaken=true;
		  break;
	   }
	   if( sliderStack[i]->toSlide->num != sliderStack[i]->toSlide->maxNum){
	      sliceTaken=true;
		  break;
	   }
  }
  
  QString cmpText="";
  if( sliceTaken ){
    cmpText = " [ ";
    for (int i=0; i<sliderStack.size(); i++){
     
		int op = sliderStack[i]->actionChoice->currentIndex();
		QString start = sliderStack[i]->fromSlide->numField->text();
		QString end = sliderStack[i]->toSlide->numField->text();      
  	   if(op == 0 && start != end){
	 	  cmpText += "(";
	 	  cmpText += start;
	 	  cmpText += ",";
	 	  cmpText += end;
	 	  cmpText += ") ";
   	   }
	   else if (op == 1 && start != end){
	 	  cmpText += "Sum(";
	 	  cmpText += start;
	 	  cmpText += ",";
	 	  cmpText += end;
	 	  cmpText += ") ";
	   }
	   else if (op == 3 && start != end){
	 	  cmpText += "Ave(";
	 	  cmpText += start;
	 	  cmpText += ",";
	 	  cmpText += end;
	 	  cmpText += ") ";
	   }
	   else if (op == 4 && start != end){
	 	  cmpText += "AtT(";
	 	  cmpText += start;
	 	  cmpText += ",";
	 	  cmpText += end;
	 	  cmpText += ") ";
	   }
	   else
	   {
	 	  cmpText += "(";
	 	  cmpText += start;
	 	  cmpText += ") ";
	   }
    }
    cmpText += "]";
  }
  
  if(slicer != 0 && !(slicer->isDefault()) && slicer->sliceChoice->currentIndex() != slicer->Inothing){

     cmpText += " < ";
	 
     // Time slice/subset available
	 if(slicer->sliceChoice->currentIndex() == slicer->Isubset){
		QString start = slicer->fromSlider->recField->text();      
		QString end   = slicer->toSlider->recField->text();
	 	cmpText += start;
	 	cmpText += ",";
	 	cmpText += end;
	 }
	 else if(slicer->sliceChoice->currentIndex() == slicer->Iaverage){
		QString start = slicer->fromSlider->recField->text();      
		QString end   = slicer->toSlider->recField->text();
	 	cmpText += "Ave ";
	 	cmpText += start;
	 	cmpText += ",";
	 	cmpText += end;
	 }
	 else if(slicer->sliceChoice->currentIndex() == slicer->Islice){
		QString atT = slicer->atSlider->recField->text();      
	 	cmpText += atT;
	 }
	 
     cmpText += " >";
  }
  
  newText += cmpText;
  parentSlot->setText(newText);
  parentSlot->reValidate(true);

  parentSlot->emitCmpSelected(cmpText.toLatin1().data(), "ARRAY");

  close();
}

void ArrayReduceWindow::launchManual(){ 

 if(QuSlotDebug())
 {
	printf("ArrayReduceWindow::launchManual %ld\n", (long)this);
	fflush(stdout);
 }
 
   arrHelp_window->manEntry("index.html");
}

void ArrayReduceWindow::launchManArrayReduce(){ 

 if(QuSlotDebug())
 {
	printf("ArrayReduceWindow::launchManArrayReducer %ld\n", (long)this);
	fflush(stdout);
 }
 
  
   arrHelp_window->manEntry("QSAS_array_reduction.html");
}


void ArrayReduceWindow::launchVersion(){

 if(QuSlotDebug())
 {
	printf("ArrayReduceWindow::launchVersion %ld\n", (long)this);
	fflush(stdout);
 }
 
  
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n Array Reduction  ";
  text += QString(QSLOT_VERSION);
  QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" );

}

selectMetadataWindow::selectMetadataWindow(QdObject_var dobj,  QWidget *parent)
	: QMainWindow(/*parent*/NULL) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
	if(QuSlotDebug())
	{
		printf("selectMetadataWindow::selectMetadataWindow \n");
		fflush(stdout);
	}
	this->setWindowTitle("Select Metadata to Use");
	
	QSizePolicy fixed(QSizePolicy::Fixed,QSizePolicy::Fixed);

	setAttribute(Qt::WA_DeleteOnClose);

	menuBar()->addSeparator();
	QMenu * help = new QMenu( this );
	menuBar()->addMenu(help );
	help->setTitle( tr(" Help"));
	help->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->addSeparator();


	//  Main widget
	QGroupBox *metaVB = new QGroupBox( this );
	QVBoxLayout *metaVBLayout=new QVBoxLayout;
	
	// XRef list
	
	xrefList = new QListWidget(metaVB);
	xrefList->setAcceptDrops(false);
	xrefList->setMinimumHeight(150);
	xrefList->setMinimumWidth(160);
	xrefList->clear();
	metaVBLayout->addWidget(xrefList);

	vector<string> xrefs;
	dobj->list_xref_names(xrefs);
	for(int i=0; i<xrefs.size(); i++){
		xrefList->addItem(QString(xrefs[i].c_str()));
	}
	
	// Buttons
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	QPushButton *selectBtn = new QPushButton( "Select");
	selectBtn->setMinimumSize(selectBtn->sizeHint());
	connect( selectBtn, SIGNAL(clicked()), this, SLOT(selectAndClose()) );
	selectBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(selectBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	buttons_layout->addWidget(dismissBtn);
	
	metaVBLayout->addLayout(buttons_layout);
	
	//Done
	
	metaVB->setLayout(metaVBLayout);
	setCentralWidget(metaVB);
	
	this->setMinimumSize(this->sizeHint());

	metaHelp_window = new QswHelpWindow(this, "metaHelpWindow");
}

selectMetadataWindow::~selectMetadataWindow(){
 if(QuSlotDebug())
 {
	printf("selectMetadataWindow::~selectMetadataWindow %ld\n", (long)this);
	fflush(stdout);
 }

 // delete arrHelp_window;
}

void selectMetadataWindow::dismiss(){
  
  if(QuSlotDebug())
  {
	printf("selectMetadataWindow::dismiss \n");
	fflush(stdout);
  }

  close();
}

void selectMetadataWindow::selectAndClose(){
  
  if(QuSlotDebug())
  {
	printf("selectMetadataWindow::selectAndClose \n");
	fflush(stdout);
  }
  dismissBtn->setEnabled(false); // prevent dismiss deleting window before signals complete
  
  QList<QListWidgetItem*> items = this->xrefList->selectedItems();
  foreach(QListWidgetItem* item, items){
     QString xrefStr=" [=> "+item->text()+" ]";
     QString txt = baseObjectText(parentSlot->text());
	 txt += xrefStr;

     parentSlot->setText(txt);
     parentSlot->reValidate(true);
     parentSlot->emitCmpSelected(xrefStr.toLatin1().data(), "XREF");
  }


  close();
}

void selectMetadataWindow::launchManual(){

 if(QuSlotDebug())
 {
	printf("selectMetadataWindow::launchManual %ld\n", (long)this);
	fflush(stdout);
  }

   metaHelp_window->manEntry("index.html");
}

// Class RecordSlider

// used in QDI and QGU (Array) and 2D/3D plot


RecordSlider::RecordSlider(const QString & title, QWidget * parent, const char * name )
	: QGroupBox(title, parent)
{
  
	QHBoxLayout * mainLayout=new QHBoxLayout;
	mainLayout->setSpacing(3);
	mainLayout->setContentsMargins(1,1,1,1);
	
	titleBase = QString(title.toLatin1().data()); // deep copy
   
	this->record = 0; // init to be safe, should be reset by parent
	this->maxRecord = 0; // init to be safe, should be reset by parent
   
	QToolButton * leftEnd = new QToolButton( this);
	leftEnd->setText("|");
	leftEnd->setFixedSize(15,21);
	mainLayout->addWidget(leftEnd);
	
	QToolButton * leftSlide = new QToolButton(this);
	leftSlide->setArrowType(Qt::LeftArrow);
	leftSlide->setFixedSize(20,21);
	mainLayout->addWidget(leftSlide);
   
	recBar = new QSlider(Qt::Horizontal, this);
	recBar->setMinimumWidth(220);
  	mainLayout->addWidget(recBar);
	
	QToolButton * rightSlide = new QToolButton(this);
	rightSlide->setArrowType(Qt::RightArrow);
	
	rightSlide->setFixedSize(20,21);
	mainLayout->addWidget(rightSlide);
	
	QToolButton * rightEnd = new QToolButton( this);
	rightEnd->setText("|");
	rightEnd->setFixedSize(15,21);
	mainLayout->addWidget(rightEnd);
	
	recBar->setTracking(false);
	connect( rightSlide, SIGNAL(clicked()), this, SLOT(incRec()) );
	connect( leftSlide,  SIGNAL(clicked()), this, SLOT(decRec()) );
	connect( rightEnd, SIGNAL(clicked()), this, SLOT(lastRec()) );
	connect( leftEnd,  SIGNAL(clicked()), this, SLOT(firstRec()) );
	connect( recBar,   SIGNAL(valueChanged(int)), this, SLOT(sliderChanged(int)) );
   
	recField = new QguWLinput(this,"recField");
	recField->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
	recField->setEnabled(true);
	recField->setFixedWidth(60);
	recField->setAlignment(Qt::AlignHCenter);
	recField->setText("0");
	connect( recField,   SIGNAL(edited_OK()), this, SLOT(setFromTyping()) );
	mainLayout->addWidget(recField);

	valueField = new QguWLinput(this,"valueField");
	
	valueField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
	valueField->setEnabled(true);
	valueField->setFixedWidth(200);
	valueField->setAlignment(Qt::AlignHCenter);
	connect( valueField,   SIGNAL(edited_OK()), this, SLOT(setFromValueFld()) );
	mainLayout->addWidget(valueField);

	recBar->setValue(0);
	
	setLayout(mainLayout);

}

void RecordSlider::setAcceptScalar(QdRScalarSeq_var ss){
	valueField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	stags = ss;
}

void RecordSlider::setTime(QmjdTime tobj){

	if(ttags.is_nil()) return;
	
	for(int i=0; i<ttags->sequence_size(); i++){
		if(ttags[i] >= tobj) {
			adjustRec(i);
			return;
		}
	}
	// if tobj after ttags range set to last tag
	lastRec();
}

void RecordSlider::setValue(double val){

	if(stags.is_nil()) return;
	
	for(int i=0; i<stags->sequence_size(); i++){
		if(stags[i] >= val) {
			adjustRec(i);
			return;
		}
	}
	// if tobj after ttags range set to last tag
	lastRec();
}


void RecordSlider::sliderChanged(int pos){
	if(record != recBar->value()) {
		record = recBar->value();
		setRecText(pos);
  		emit recordSliderChanged();
	}
}


void RecordSlider::setRecText(int rec){
	
	// sets text feedback only while sliding
	// does not emit signal
	
	recField->setText(tr("%1").arg(rec));
	updateTitle(rec);
	
}


void RecordSlider::updateTitle(int rec){
  
	if(QuSlotDebug())
	{
		printf("RecordSlider::updateTitle \n");
		fflush(stdout);
	}
	
	if( !ttags.is_nil() && rec < ttags->sequence_size()){
		StringRep string_rep;
		this->setTitle(titleBase + QString(" time %1").arg(ttags[rec].getISOstring().c_str()) );
		valueField->blockSignals(true);
		valueField->setText(ttags[rec].getISOstring().c_str());
		valueField->blockSignals(false);
	}
	else if( !stags.is_nil() && rec < stags->sequence_size()){
		StringRep string_rep;
		this->setTitle(titleBase + QString(" %1").arg(stags[rec]) );
		valueField->blockSignals(true);
		valueField->setText(QString(" %1").arg(stags[rec]));
		valueField->blockSignals(false);

	}
	else this->setTitle(titleBase + QString(" record %1").arg(rec) );

}

void RecordSlider::updateTitleEnd(){
  
	if(QuSlotDebug())
	{
		printf("RecordSlider::updateTitleEnd \n");
		fflush(stdout);
	}
	
	updateTitle(maxRecord);

}


void RecordSlider::setFromTyping(){

  if( !(recField->lineBox->hasFocus()) ) return;

  if(QuSlotDebug())
  {
	printf("recordSlider::setFromTyping %ld\n", (long)this);
	fflush(stdout);
  }

  bool OK;
  int newValue = recField->text().toInt(&OK);
  if (!OK) return;

  if(newValue > maxRecord){
    newValue = maxRecord;
  }
  else if(newValue < 0) {
    newValue = 0;
  }
  
  recBar->setValue(newValue);
}

void RecordSlider::setFromValueFld(){

	if( !(valueField->lineBox->hasFocus()) ) return;

	if(QuSlotDebug())
	{
		printf("recordSlider::setFromValueFld %ld\n", (long)this);
		fflush(stdout);
	}
	int posn = valueField->lineBox->cursorPosition();
	
	QdObject_var dobj = valueField->getSlotObject();
	QdTime_var tobj = QdTime_var::narrow(dobj);
	QdRScalar_var sobj = QdRScalar_var::narrow(dobj);
	
	if( !tobj.is_nil() && !ttags.is_nil()) {
		if(*tobj < ttags[0]) return; // wait for completed edit
		setTime(*tobj);
	}
	if( !sobj.is_nil() && !stags.is_nil()) {
		if(*sobj < stags[0]) return; // wait for completed edit
		if( valueUnchanged() ) return; // allow typing decimal pt etc
		setValue(*sobj);
	}
	valueField->lineBox->setCursorPosition(posn);
 }

bool RecordSlider::valueUnchanged(){
	QString newTxt = valueField->lineBox->text();
	int ptr = newTxt.indexOf(".");
	if( ptr < 0) return false;
	
	for(int i=ptr+1; i<newTxt.length(); i++){
		if(newTxt[i] != '0') return false;
	}
	
	return true;
}


void RecordSlider::incRec(){

	if(QuSlotDebug())
	{
		printf("recordSlider::incRec %ld\n", (long)this);
		fflush(stdout);
	}

	if(record < maxRecord){
		record +=1;
	}
	else{
    	record = maxRecord;
	}
  	adjustRec(record);
}

void RecordSlider::decRec(){

 if(QuSlotDebug())
 {
	printf("recordSlider::decRec %ld\n", (long)this);
	fflush(stdout);
 }
 
	if(record > 0){
		record -= 1;
	}
	else{
		record = 0;
  	}
  	adjustRec(record);
}

void RecordSlider::lastRec(){

	if(QuSlotDebug())
	{
		printf("recordSlider::lastRec %ld\n", (long)this);
		fflush(stdout);
	}
	
  	adjustRec(maxRecord);
 
}

void RecordSlider::firstRec(){

 	if(QuSlotDebug())
 	{
		printf("recordSlider::firstRec %ld\n", (long)this);
		fflush(stdout);
 	}
	
 	adjustRec(0);
}

void RecordSlider::adjustRec(int rec){

 if(QuSlotDebug())
 {
	printf("recordSlider::adjustRec %ld %d\n", (long)this, rec);
	fflush(stdout);
 }
 
  record = rec;
  recBar->setValue(rec);
  setRecText(rec);
  
  emit recordSliderChanged();
}

// Class TimeIntervalSlider

// used in  QGU (time slices)
// provides just the slider for event number.


TimeIntervalSlider::TimeIntervalSlider(const QString & title, QWidget * parent, const char * name )
	: QGroupBox(title, parent)
{
  
	if(QuSlotDebug())
	{
		printf("TimeIntervalSlider::TimeIntervalSlider \n");
		fflush(stdout);
	}

	obj=0;
    
	QHBoxLayout * mainLayout=new QHBoxLayout;
	
	titleBase = QString(title.toLatin1().data()); // deep copy
   
	this->record = 0; // init to be safe, should be reset by parent
	this->maxRecord = 0; // init to be safe, should be reset by parent
   
	QToolButton * leftEnd = new QToolButton( this);
	leftEnd->setText("|");
	leftEnd->setFixedSize(15,21);
	mainLayout->addWidget(leftEnd);
	
	QToolButton * leftSlide = new QToolButton(this);
	leftSlide->setArrowType(Qt::LeftArrow);
	leftSlide->setFixedSize(20,21);
	mainLayout->addWidget(leftSlide);
   
	recBar = new QSlider(Qt::Horizontal, this);
	mainLayout->addWidget(recBar);
  
	QToolButton * rightSlide = new QToolButton(this);
	rightSlide->setArrowType(Qt::RightArrow);
	rightSlide->setFixedSize(20,21);
	mainLayout->addWidget(rightSlide);
	
	QToolButton * rightEnd = new QToolButton( this);
	rightEnd->setText("|");
	rightEnd->setFixedSize(15,21);
	mainLayout->addWidget(rightEnd);
	
	connect( rightSlide, SIGNAL(clicked()), this, SLOT(incRec()) );
	connect( leftSlide,  SIGNAL(clicked()), this, SLOT(decRec()) );
	connect( rightEnd, SIGNAL(clicked()), this, SLOT(lastRec()) );
	connect( leftEnd,  SIGNAL(clicked()), this, SLOT(firstRec()) );
	connect( recBar,   SIGNAL(valueChanged(int)), this, SLOT(adjustRec(int)) );
   
	recField = new QguWLinput(this,"recField");
	recField->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
	recField->setEnabled(true);
	recField->setFixedWidth(60);
	recField->setAlignment(Qt::AlignHCenter);
	recField->setText("0");
	connect( recField,   SIGNAL(edited_OK()), this, SLOT(setFromTyping()) );
	mainLayout->addWidget(recField);

	recBar->setValue(0);
    
	setBaseSize(200, 30);
	
	setLayout(mainLayout);

}

void TimeIntervalSlider::updateTitle(){
  
    if(QuSlotDebug())
    {
		printf("TimeIntervalSlider::updateTitle \n");
		fflush(stdout);
	}

    if(obj!=NULL)
    {
        QString text=titleBase + " ";
        text+=obj[record].start().getISOstring().c_str();
        text+=" / ";
        text+=obj[record].end().getISOstring().c_str();
     this->setTitle(text);
    }
}

void TimeIntervalSlider::setFromTyping(){

    if(QuSlotDebug())
    {
		printf("TimeIntervalSlider::setFromTyping %ld\n", (long)this);
		fflush(stdout);
	}
   
    bool OK;
    int newValue = recField->text().toInt(&OK);
    if (!OK) return;

    if(newValue > maxRecord){
        newValue = maxRecord;
        recField->setText(tr("%1").arg(newValue));
    }
    else if(newValue < 0) {
        newValue = 0;
        recField->setText("0");
    }
  
    recBar->setValue(newValue);
    this->updateTitle();
}

void TimeIntervalSlider::incRec(){

    if(QuSlotDebug())
    {
		printf("TimeIntervalSlider::incRec %ld\n", (long)this);
		fflush(stdout);
	}

    if(record < maxRecord){
        record +=1;
    }
    else{
        record = maxRecord;
    }
    recBar->setValue(record);
    this->updateTitle();
}

void TimeIntervalSlider::decRec(){

    if(QuSlotDebug())
    {
		printf("TimeIntervalSlider::decRec %ld\n", (long)this);
		fflush(stdout);
	}
 
    if(record > 0){
        record -= 1;
    }
    else{
        record = 0;
    }
    recBar->setValue(record);
    this->updateTitle();
}

void TimeIntervalSlider::lastRec(){

    if(QuSlotDebug())
    {
		printf("TimeIntervalSlider::lastRec %ld\n", (long)this);
		fflush(stdout);
	}
 
    record = maxRecord;
    recBar->setValue(record);
    this->updateTitle();
}

void TimeIntervalSlider::firstRec(){

    if(QuSlotDebug())
    {
		printf("TimeIntervalSlider::firstRec %ld\n", (long)this);
		fflush(stdout);
	}
 
    record = 0;
    recBar->setValue(record);
    this->updateTitle();
}

void TimeIntervalSlider::adjustRec(int rec){

    if(QuSlotDebug())
	{
		printf("TimeIntervalSlider::adjustRec %ld %d\n", (long)this, rec);
		fflush(stdout);
	}
 
    record = rec;
    recField->setText(tr("%1").arg(rec));
    this->updateTitle();
  
    emit sliderChanged();
}

// class timeSlider
//-------------------

// Provides subset or time selection sliders

 timeSlider::timeSlider(QdObject_var dobj, bool acceptTSlice, bool acceptTSubset, QWidget *parent, char * name)
	: QGroupBox(parent)
{
	if(QuSlotDebug())
	{
		printf("timeSlider::timeSlider \n");
		fflush(stdout);
	}
	
	mainLayout=new QVBoxLayout;
	
	mainLayout->setMargin(QSAS_profiles->current()->GUImargins);
	this->setTitle(" Time Options ");
	int i=0;
	Isubset = -1;
	Iaverage = -1;
	Islice = -1;
	Inothing = -1;
	
	 // Combo box
	sliceChoice = new QComboBox(this);
	
	if(acceptTSubset)  {
		Isubset = i;
		sliceChoice->insertItem( i++, "Subset over time range");
	}
	else{
		Inothing = i;
		sliceChoice->insertItem( i++, "Keep as Time Series");
	}
	if(acceptTSlice)  {     
		Iaverage = i;
		sliceChoice->insertItem( i++, "Average over time range");

		Islice = i;
		sliceChoice->insertItem( i++, "Slice at time");
	}
	sliceChoice->resize( sliceChoice->sizeHint() );
	connect( sliceChoice, SIGNAL(activated(int)), this, SLOT(updateTimeSlider(int)) );
	
	mainLayout->addWidget(sliceChoice);
	
	//
	
	mainLayout->addSpacing(10);
	
	// Sliders
	fromSlider = new  RecordSlider("From ", this, "fromSlider");
	mainLayout->addWidget(fromSlider);
	toSlider =   new  RecordSlider("To ", this, "toSlider");
	mainLayout->addWidget(toSlider);
	atSlider =   new  RecordSlider("At ", this, "atSlider");
	mainLayout->addWidget(atSlider);
	fromSlider->resize(fromSlider->sizeHint());
	toSlider->resize(toSlider->sizeHint());
	atSlider->resize(atSlider->sizeHint());
	
	QLabel *dropLabel = new QLabel(" ->  Drop Interval object or Event List here  <- ");
	mainLayout->addWidget(dropLabel, 0, Qt::AlignHCenter);
	
	
	int seqLength = dobj->sequence_size();
	fromSlider->recBar->setRange(0, seqLength-1);
	toSlider->recBar->setRange(0, seqLength-1);
	atSlider->recBar->setRange(0, seqLength-1);
   
	fromSlider->maxRecord = seqLength-1;
	toSlider->maxRecord = seqLength-1;
	atSlider->maxRecord = seqLength-1;
	toSlider->recBar->setValue(toSlider->maxRecord);
	
	// get time line and keep globally for this file
	QdObject_var d0 = get_Depend_0(dobj);
	QdTimeSeq_var tt = QdTimeSeq_var::narrow(d0);
	QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(d0);
	if( !tt.is_nil() ){
		fromSlider->ttags = tt;
		toSlider->ttags = tt;
		atSlider->ttags = tt;
	}
	else if( !ss.is_nil() ){
		fromSlider->setAcceptScalar(ss);
		toSlider->setAcceptScalar(ss);
		atSlider->setAcceptScalar(ss);
	}
  
   	// now ttags are set, add time feedback to title
	fromSlider->updateTitle(0);
	toSlider->updateTitle(toSlider->maxRecord);
	atSlider->updateTitle(0);

	if(acceptTSubset){
		fromSlider->show();
		toSlider->show();
		atSlider->hide();
	}
	else{
		fromSlider->hide();
		toSlider->hide();
		atSlider->hide();
	}
   
	this->setMinimumHeight( (int)(sliceChoice->sizeHint().height() +  atSlider->sizeHint().height()*2.5));
	this->setEnabled(true);
	setLayout(mainLayout);
	setAcceptDrops(true);
}



void timeSlider::updateTimeSlider(int opChoice){

  if (opChoice == Isubset){
    fromSlider->show();
    toSlider->show();
    atSlider->hide();
  }
  else if(opChoice == Iaverage){
    fromSlider->show();
    toSlider->show();
    atSlider->hide();
  }
  else if(opChoice == Islice){
    fromSlider->hide();
    toSlider->hide();
    atSlider->show();
  }
  else if(opChoice == Inothing){
    fromSlider->hide();
    toSlider->hide();
    atSlider->hide();
  }
  else {
    cout << " Slice option not available " << endl;
  }
}

bool timeSlider::isDefault(){

 if(QuSlotDebug())
 {
	printf("timeSlider::isDefault \n");
	fflush(stdout);
 }

  if (sliceChoice->currentIndex() == Isubset){
    if(fromSlider->record == 0 && toSlider->record == toSlider->maxRecord) return true;
  }
  return false;
}

void timeSlider::dragEnterEvent(QDragEnterEvent *event)
{
     if (event->mimeData()->hasFormat("text/plain"))
         event->acceptProposedAction();
}
 
void timeSlider::dropEvent(QDropEvent * event)
{
	QdObject_var droppedObject  = QuGetObjByName(event->mimeData()->text());

	if(droppedObject.is_nil()) return;
		
	if(sliceChoice->currentText() == "Slice at time")
	{
		// Time Slice actions
		QdTime_var time_var=QdTime_var::narrow(droppedObject);
	
		if(!time_var.is_nil()) 
		{
			atSlider->setTime(*time_var);
			return;
		}

		QdTimeInterval_var int_var=QdTimeInterval_var::narrow(droppedObject);
	
		if(!int_var.is_nil() )
		{
			QMenu * selectPopup=new QMenu;
			QAction *start = selectPopup->addAction("Start");
			QAction *centre = selectPopup->addAction("Centre");
			QAction *end = selectPopup->addAction("End");
			QAction *cancel = selectPopup->addAction("Cancel");

			QAction *res = selectPopup->exec(QCursor::pos());
			if(res == start) atSlider->setTime(int_var->start());
			if(res == centre) atSlider->setTime(int_var->centre());
			if(res == end) atSlider->setTime(int_var->end());
			
			return;
		}

		QdTimeSeq_var ts_var=get_timetags(droppedObject);
	
		if(!ts_var.is_nil() )
		{
			QMenu * selectPopup=new QMenu;
			QAction *start = selectPopup->addAction("Start");
			QAction *centre = selectPopup->addAction("Centre");
			QAction *end = selectPopup->addAction("End");
			QAction *cancel = selectPopup->addAction("Cancel");

			QAction *res = selectPopup->exec(QCursor::pos());
			if(res == start) atSlider->setTime(ts_var[0]);
			if(res == centre) atSlider->setTime(ts_var[(ts_var->sequence_size()-1)/2]);
			if(res == end) atSlider->setTime(ts_var[ts_var->sequence_size()-1]);
			
			return;
		}
	}
	else {

		// Interval actions

		QdTimeInterval_var int_var=QdTimeInterval_var::narrow(droppedObject);
	
		if( !int_var.is_nil() )
		{
			fromSlider->setTime(int_var->start());
			toSlider->setTime(int_var->end());

			return;
		}
		
		QdTimeSeq_var ts_var=get_timetags(droppedObject);
	
		if( !ts_var.is_nil() )
		{
			fromSlider->setTime(ts_var[0]);
			toSlider->setTime(ts_var[ts_var->sequence_size()-1]);

			return;
		}
		
		// use popup to select from event list
		QdTimeIntervalSeq_var tivs_var=QdTimeIntervalSeq_var::narrow(droppedObject);
		if( !tivs_var.is_nil())
		{
			QguWLinput * temp = new QguWLinput(this,"temp");
			temp->hide();
			temp->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
			temp->setText(event->mimeData()->text());
			SelectTivlWindow * selectWindow = new SelectTivlWindow(event->mimeData()->text(), this, "tSliceWindow", true);
			selectWindow->setPalette(qApp->palette());

			selectWindow->parentSlot = temp;
			selectWindow->exec();
			
			QdObject_var tiobj = temp->getSlotObject();
			QdTimeInterval_var ti=QdTimeInterval_var::narrow(tiobj);
				
			if( !ti.is_nil() )
			{
				fromSlider->setTime(ti->start());
				toSlider->setTime(ti->end());
			}
			qApp->processEvents();

			delete temp;
		}
		
	}
	return;
}


