#include "WLItem.h"

WLItem::WLItem(const QString& name, WLItem* parent, bool isFolder):
	_name(name), _isFolder(isFolder)
{
	if(parent!=NULL)
	{
		parent->addChild(this, parent->numChildren());
	}
	else
	{
		_path=_name;
		_parent = NULL;
	}
	_startTime = 1.e30;
	_objType = -1;
}

WLItem::~WLItem()
{
	QuDeleteObject(_path);
	emit deleted(_path);
	qDeleteAll(_children);
	_children.clear();
	_parent=NULL;
}


double WLItem::getTime()
{
	if(_isFolder){
		double tStart = 1.e30;
		QList <WLItem*> childList = getChildren();
		tStart = childList[0]->getTime();

		for(int i=1; i<childList.size(); i++){

			double tNext = childList[i]->getTime();

			if(tNext < tStart) tStart = tNext;
		} 
		_startTime = tStart;
	}			
	else if(_startTime > 1.e29){
		QdObject_var dobj = QuGetObjByName(_path);
		if(!dobj.is_nil()){
			QdTimeInterval_var ti = get_timeinterval(dobj);
			if(!ti.is_nil()) _startTime = ti->start().getEpoch2000();
		}
	}
	
	return _startTime;
}

double WLItem::getType()
{
	if(_isFolder) _objType = 0;	
	else if(_objType < 0){
		QdObject_var dobj = QuGetObjByName(_path);
		if(!dobj.is_nil()){
			QdRScalarSeq_var sseq = QdRScalarSeq_var::narrow(dobj);
			if(!sseq.is_nil()) {
				_objType = 1;
				return _objType;
			}
			QdRMatrixSeq_var mseq = QdRMatrixSeq_var::narrow(dobj);
			if(!mseq.is_nil()) {
				if(QuIsThreeVector(dobj)){
					_objType = 2;
					return _objType;
				}
				else{
					_objType = 3;
					return _objType;
				}
			}
			QdRScalar_var s = QdRScalar_var::narrow(dobj);
			if(!s.is_nil()) {
				_objType = 4;
				return _objType;
			}
			QdRMatrix_var m = QdRMatrix_var::narrow(dobj);
			if(!m.is_nil()) {
				if(QuIsThreeVector(dobj)){
					_objType = 5;
					return _objType;
				}
				else{
					_objType = 6;
					return _objType;
				}
			}
			QdTimeInterval_var ti = QdTimeInterval_var::narrow(dobj);
			if(!ti.is_nil()) {
				_objType = 7;
				return _objType;
			}
			QdTimeIntervalSeq_var tiseq = QdTimeIntervalSeq_var::narrow(dobj);
			if(!tiseq.is_nil()) {
				_objType = 8;
				return _objType;
			}
			QdTimeSeq_var tseq = QdTimeSeq_var::narrow(dobj);
			if(!tseq.is_nil()) {
				_objType = 9;
				return _objType;
			}
			QdTime_var t = QdTime_var::narrow(dobj);
			if(!t.is_nil()) {
				_objType = 10;
				return _objType;
			}
			QdStringSeq_var strseq =  QdStringSeq_var::narrow(dobj);
			if(!strseq.is_nil()) {
				_objType = 11;
				return _objType;
			}
			QdString_var str =  QdString_var::narrow(dobj);
			if(!str.is_nil()) {
				_objType = 12;
				return _objType;
			}
			else{
				_objType = 13;
				return _objType;
			}
			
		}
	}
	
	return _objType;
}

bool WLItem::setName(const QString& newName)
{
	if(newName.isEmpty()) return false;
	
	if(_parent==NULL)
	{
		_name=newName;
		refreshPath();
		return true;
	}
	
	QString tempName(newName);
	WLItem* item;
	item=_parent->getChildByName(tempName);
	while(item!=NULL && item!=this)
	{
		tempName.append("A");
		item=_parent->getChildByName(tempName);
	}
	if(tempName!=_name)
	{
		_name=tempName;
		refreshPath();
	}
	
	return true;
}

bool WLItem::setFolder(bool isFolder)
{
	if(isFolder)
	{
		clear();
	}
	_isFolder=isFolder;
	
	return true;
}

bool WLItem::isParentOf(WLItem* other)
{
	//return other->isDescendentOf(this);
	if(this==other) return true;
	
	if(!_isFolder) return false;
	
	bool res=false;
	for(QList<WLItem*>::iterator iter=_children.begin(); iter!=_children.end() && !res; ++iter)
	{
		res=(*iter)->isParentOf(other);
	}
	return res;
}

bool WLItem::isDescendentOf(WLItem* other)
{
	if(this==other) return true;
	
	if(_parent==NULL) return false;
	
	return _parent->isDescendentOf(other);
}

bool WLItem::ancestorInList(QList <WLItem*> &list)
{
	for(int i=0; i<list.size(); i++)
	{
		if(this != list[i] && this->isDescendentOf(list[i]) ) return true;	
	}
	return false;
}


WLItem* WLItem::getChildByName(const QString& name) const
{
	if(!_isFolder || _children.isEmpty()) return NULL;
	for(QList<WLItem*>::const_iterator iter=_children.begin(); iter!=_children.end(); ++iter)
	{
		if((*iter)->_name==name) return *iter;
	}
	return NULL;
}

WLItem* WLItem::getChildByPath(const QString& path) const
{
	QString tempPath(path);
	while(tempPath.startsWith("/")) tempPath.remove(0,1);
	if(path.contains("/"))
	{
		QStringList splitList=path.split("/", QString::SkipEmptyParts);
		
		WLItem* child=getChildByName(splitList.at(0));
		if(child)
		{
			splitList.removeFirst();
			return child->getChildByPath(splitList.join("/"));
		}
	}
	else
	{
		return getChildByName(path);
	}
	
	return NULL;
}

WLItem* WLItem::getChildByNumber(int i) const
{
	if(i<0 || i>=_children.size()) return NULL;

	return _children.at(i);
}

bool WLItem::removeChild(WLItem* child, bool alsoDelete)
{
	int i=_children.indexOf(child);
	
	if(i<0) return false;
	
	_children.removeAt(i);
	
	if(alsoDelete)
	{
		delete child;
	}
	return true;
}

bool WLItem::removeChild(int i, bool alsoDelete)
{
	if(i<0 || i>=_children.size())
		return false;
	
	if(alsoDelete)
	{
		WLItem* child=_children.takeAt(i);
		delete child;
	}
	else
	{
		_children.removeAt(i);
	}
	return true;
}

bool WLItem::addChild(WLItem* newChild, int position)
{
	if(_children.indexOf(newChild)>=0) return false;
		
	QString childName=newChild->_name;
	while(getChildByName(childName))
	{
		childName.append("A");
	}
	
	newChild->_parent=this;
	if(position<0 || position>_children.size())
	{
		_children.push_back(newChild);
	}
	else
	{
		_children.insert(position, newChild);
	}
	newChild->_name=childName;
	newChild->refreshPath();
	
	return true;
}

bool WLItem::reparent(WLItem* newParent, int position)
{
	if(isParentOf(newParent)) return false;

	if(_parent!=NULL)
	{
		_parent->removeChild(this, false);
	}
	if(newParent!=NULL)
	{
		newParent->addChild(this, position);
	}
	
	return true;
}

void WLItem::refreshPath()
{
	if(_parent==NULL)
	{
		if(_path!=_name)
		{
			QString oldPath=_path;
			_path=_name;
			emit pathChanged(oldPath, _path);
		}
	}
	else
	{
		QString oldPath=_path;
		_path=_parent->_path+"/"+_name;
		if(_path.startsWith("/")) _path=_path.remove(0,1);
		if(_path!=oldPath)
		{
			QdObject_var tmp=QuGetObjByName(oldPath);
			if(!tmp.is_nil())
			{
				QuRenameObject(oldPath, _path);
			}
			emit pathChanged(oldPath, _path);
		}
		foreach(WLItem* child, _children)
		{
			child->refreshPath();
		}
	}
}

void WLItem::clear()
{
	qDeleteAll(_children);
	_children.clear();
}

void WLItem::display(int offset) const
{
	for(int i=0; i<offset; ++i)
	{
		std::cout << "  ";
	}
	
	std::cout << _name.toLatin1().data() << " - " << _path.toLatin1().data() << " - " << (_parent!=NULL ? _parent->_name.toLatin1().data(): "") << std::endl;
	
	foreach(WLItem* child, _children)
	{
		child->display(offset+1);
	}
}

// I/O streaming

QDOS::Vostream& operator<<(QDOS::Vostream& v_os ,WLItem& item)
{
	v_os << string(item.getName().toLatin1().data());
	if(item.isFolder())
	{
		v_os << ListTypeGroup;
	}
	else
	{
		v_os << ListTypeEntry;
		v_os << string(item.getPath().toLatin1().data());
		QdObject_var obj=QuGetObjByName(item.getPath());
		string type=obj->object_type();
		v_os << type;
		v_os << obj;
	}
	
	int cnt=item.numChildren();
	v_os << cnt;
	for(int i=0; i<cnt; ++i)
	{
		v_os << *(item.getChildByNumber(i));
	}
	
	return v_os;
}

QDOS::Vistream& operator>>(QDOS::Vistream& v_is, WLItem& item)
{
	std::string text;
	v_is >> text;
	
	item.setName(text.c_str());
	
	int type=ListTypeNull;
	v_is >> type;


	if((QguListType)type==ListTypeEntry)
	{
 
		item.setFolder(false);
		
		string name_ctxt;
		v_is >> name_ctxt;
		
		string obj_type;
		v_is >> obj_type;

		QdObject_var obj=QdObject::create(obj_type);
		v_is >> obj;

		if(!obj.is_nil() && obj->xref_exists("TIME_TAGS"))
		{
			QdObject_var tt = obj->get_xref("TIME_TAGS");
			obj->change_xref(DEPEND_0, tt);
			obj->delete_xref("TIME_TAGS");
		}
		QuSetObjByName(obj, item.getPath());
		
	}
	else if((QguListType)type==ListTypeGroup)
	{
		item.setFolder(true);
	}
	
	int cnt=0;
	v_is >> cnt;
	for(int i=0; i<cnt; ++i)
	{
		WLItem *childItem=new WLItem("", &item);
		v_is >> *childItem;
	}
	
	return v_is;
}
