#include "WLView.h"
#include "qwui.h"
#include "qpsd.h"
#include <QRadioButton>


WLEditor::WLEditor(QObject *parent):
	QItemDelegate(parent)
{
	
}

QWidget * WLEditor::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
	QLineEdit *e=new QLineEdit(parent);
	e->setFrame(false);
	return e;
}

void WLEditor::setEditorData(QWidget *editor, const QModelIndex &index) const
{
	QString value = index.model()->data(index, Qt::DisplayRole).toString();

	QLineEdit *edit = static_cast<QLineEdit*>(editor);
	edit->setText(value);
}
		
void WLEditor::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
	QLineEdit *edit = static_cast<QLineEdit*>(editor);
	QString value = edit->text();

	model->setData(index, value, Qt::EditRole);
}

void WLEditor::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &/*index*/) const
{
	editor->setGeometry(option.rect);
}



WLView::WLView(QWidget* parent):
	QTreeView(parent)
{

	setSelectionMode(QAbstractItemView::ExtendedSelection);

	setDragEnabled(true);
	setAcceptDrops(true);
	viewport()->setAcceptDrops(true);
	setDropIndicatorShown(true);
	header()->hide();
	setDragDropMode(QAbstractItemView::DragDrop);
	// Set Right Button Menu
	
	RButtonMenu=new QMenu(this);
	
#if QT_VERSION >= 0x040300
	setEditTriggers(QAbstractItemView::NoEditTriggers);
#endif
	
	connect(this, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(browseObject(const QModelIndex&)));
	
	objTypeSelector=NULL;
	sortTypeSelector=NULL;
	
	delegate=new WLEditor(parent);
	setItemDelegate(delegate);
	
	sortReverse = false;
	sortOn = "name";
}

// Slots

void WLView::newFolder()
{
	QModelIndexList indexes;
	foreach(QModelIndex index, selectionModel()->selectedRows())
	{
		WLItem* item=wlmodel()->itemFromIndex(index);
		if(item!=NULL && item!=wlmodel()->_root && item!=wlmodel()->_data && item!=wlmodel()->_time && item!=wlmodel()->_constants && item!=wlmodel()->_temp)
		{
			indexes.push_back(index);
		}
	}
	
	QModelIndex folderIndex=wlmodel()->newFolder(indexes);
		
	edit(folderIndex);
	
}

void WLView::moveUp()
{
	QModelIndexList indexes=selectionModel()->selectedRows();
	QModelIndex temp;
	
	foreach(QModelIndex index, indexes)
	{
		selectionModel()->select(index, QItemSelectionModel::Deselect);
		temp=wlmodel()->moveUp(index);
		if(temp.isValid())
		{
			selectionModel()->select(temp, QItemSelectionModel::Select);
		}
	}
}

void WLView::moveDown()
{
	QModelIndexList indexes=selectionModel()->selectedRows();
	QModelIndex temp;
	
	foreach(QModelIndex index, indexes)
	{
		selectionModel()->select(index, QItemSelectionModel::Deselect);
		temp=wlmodel()->moveDown(index);
		if(temp.isValid())
		{
			selectionModel()->select(temp, QItemSelectionModel::Select);
		}
	}
}

void WLView::sortWLSelection()
{
	QList <WLItem*> itemList = selectedItems();
	wlmodel()->sortWLSelection(itemList, sortOn, sortReverse);
	
}

void WLView::sortWL()
{
	selectAllData();
	QList<WLItem*> itemList = selectedItems();
	wlmodel()->sortWLSelection(itemList, sortOn, sortReverse);
	
}

void WLView::setSortWLname(bool on)
{
	sortOn = "name";
}

void WLView::setSortWLtime(bool on)
{
	sortOn = "time";
}

void WLView::setSortWLtype(bool on)
{
	sortOn = "object";
}

void WLView::setSortRev(bool on)
{
	sortReverse = on;
}




void WLView::clear()
{
	QModelIndexList indexes=selectionModel()->selectedRows();
	
	foreach(QModelIndex index, indexes)
	{
		wlmodel()->clear(index);
	}
}

void WLView::displayMenu(const QPoint& p)
{
	QModelIndex currIndex=currentIndex();
	QModelIndexList indexes=selectionModel()->selectedRows();
	WLItem* currItem=wlmodel()->itemFromIndex(currIndex);
	if(currItem==NULL) return;
	
	RButtonMenu->clear();
	
	if(currItem==wlmodel()->_data || currItem==wlmodel()->_time || currItem==wlmodel()->_constants || currItem==wlmodel()->_temp) 
	{
		RButtonMenu->addAction("New Object", this, SLOT(newObject()));
		RButtonMenu->exec(p);
		return;
	}
	
	bool oneObject=(indexes.size()==1);
	
	RButtonMenu->addAction("Move items to new folder", this, SLOT(newFolder()));
	if(oneObject)
		RButtonMenu->addAction("Re-Name", this, SLOT(rename()));
	if(oneObject)
		RButtonMenu->addAction("Browse Object", this, SLOT(browseObject()));
	if(oneObject)
		RButtonMenu->addAction("Edit Object", this, SLOT(editObject()));
	RButtonMenu->addAction("New Object", this, SLOT(newObject()));
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Duplicate", this, SLOT(duplicateObj()));
	if(oneObject)
	{
		RButtonMenu->addAction("Subsample/Extract", this, SLOT(sliceObj()));
		RButtonMenu->addAction("Get Trace of Array", this, SLOT(arrayTrace()));
		RButtonMenu->addAction("Make Array into Vector", this, SLOT(vect2array()));
	}
	if(oneObject && currItem->isFolder())
		RButtonMenu->addAction("Time Interval Folder to Event Table", this, SLOT(tif2ev()));
	if(oneObject /*and event table*/)
	{
		QdTimeIntervalSeq_var obj=QdTimeIntervalSeq_var::narrow(QuGetObjByName(currItem->getPath()));
		if(!obj.is_nil())
		{
			RButtonMenu->addAction("Event Table to Time Interval Folder", this, SLOT(ev2tif()));
			RButtonMenu->addAction("Export Event Table", this, SIGNAL(exportET()));
		}
	}
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Copy", this, SLOT(copy()));
	RButtonMenu->addAction("Move Up", this, SLOT(moveUp()), Qt::CTRL+Qt::Key_Up);
	RButtonMenu->addAction("Move Down", this, SLOT(moveDown()), Qt::CTRL+Qt::Key_Down);
	RButtonMenu->addSeparator();
	RButtonMenu->addAction(style()->standardIcon(QStyle::SP_TrashIcon, 0, this), "Delete", this, SLOT(deleteObject()));
	//if at least one object has time tags
	bool hasTimeTags=false;
	foreach(QModelIndex index, indexes)
	{
		WLItem* item=wlmodel()->itemFromIndex(index);
		if(item!=NULL)
		{
			QdObject_var dobj=QuGetObjByName(item->getPath());
			if(!dobj.is_nil()) hasTimeTags=dobj->xref_exists(DEPEND_0);
		}
		if(hasTimeTags)
		{
			continue;
		}
	}
	if(hasTimeTags)
	{
		RButtonMenu->addAction("Remove Time Tags", this, SLOT(removeTimeTags()));
	}
	
	RButtonMenu->exec(p);
}

void WLView::mousePressEvent(QMouseEvent* event)
{
	QTreeView::mousePressEvent(event);
	if(event->button()==Qt::RightButton)
	{
		displayMenu(event->globalPos());
	}
}

void WLView::insert()
{
	QModelIndexList indexes=selectionModel()->selectedRows();
	
	foreach(QModelIndex index, indexes)
	{
		wlmodel()->insertRow(index.row(), index.parent());
	}
}

void WLView::rename()
{
	if(wlmodel()->flags(currentIndex()) & Qt::ItemIsEditable) // Just to avoid seeing the "editing failed" message if not editable
	{	
		setCurrentIndex(currentIndex()); // See Qt doc for this precaution
		edit(currentIndex());
	}
}

void WLView::browseObject(const QModelIndex& index)
{
	WLItem* item;
	DataBrowseWindow * browser;
	
	if(!index.isValid())
	{
		item=wlmodel()->itemFromIndex(currentIndex());
	}
	else
	{
		item=wlmodel()->itemFromIndex(index);
	}
	
	if(item==wlmodel()->_data || item==wlmodel()->_time || item==wlmodel()->_constants || item==wlmodel()->_temp) return;

	if(item->isFolder())
	{
		return;
	}
	else
	{
		QdObject_var obj=QuGetObjByName(item->getPath());

		QdTimeInterval_var ti = QdTimeInterval_var::narrow(obj);
		QdTimeIntervalSeq_var tis = QdTimeIntervalSeq_var::narrow(obj);
		if(ti.is_nil() && tis.is_nil())
			browser = QbuLaunchBrowse(QdObject_var(), this, item->getName().toLatin1().data(), obj, item->getPath(), false);
		else
			browser = QbuLaunchBrowse(QdObject_var(), this, item->getName().toLatin1().data(), obj, item->getPath(), true);

		if(browser) connect(browser, SIGNAL(objChanged(const QString&)), wlmodel(), SIGNAL(objChanged(const QString&)));
	}
}
void WLView::editObject(const QModelIndex& index)
{
	WLItem* item;
	if(!index.isValid())
	{
		item=wlmodel()->itemFromIndex(currentIndex());
	}
	else
	{
		item=wlmodel()->itemFromIndex(index);
	}
	
	if(item==wlmodel()->_data || item==wlmodel()->_time || item==wlmodel()->_constants || item==wlmodel()->_temp) return;

	if(item->isFolder())
	{
		return;
	}
	else
	{
		QdObject_var obj=QuGetObjByName(item->getPath());
		DataBrowseWindow * browser=QbuLaunchBrowse(QdObject_var(), this, item->getName().toLatin1().data(), obj, item->getPath(), true);

		if(browser) connect(browser, SIGNAL(objChanged(const QString&)), wlmodel(), SIGNAL(objChanged(const QString&)));
	}
}

void WLView::newObject()
{

	if(QuSlotDebug())
	{
		printf("createObject::createObject %ld\n", (long)this);
		fflush(stdout);
	}
	
	static QPushButton *Sbtn;
	static QPushButton *Vbtn;
	static QPushButton *Abtn;
	static QPushButton *tbtn;
	static QPushButton *tibtn;
	static QPushButton *Txtbtn;
	static QPushButton *TxtArrbtn;
	static QWidget * typeBox;
	
	if(objTypeSelector==NULL)
	{
		objTypeSelector = new QDialog(0);
		typeBox = new QWidget(objTypeSelector);
		QVBoxLayout* typeBoxLayout=new QVBoxLayout;
		typeBoxLayout->setMargin(4);
		typeBoxLayout->setSpacing(6);
	
		Txtbtn = new QPushButton(" Text Constant  ", typeBox); typeBoxLayout->addWidget(Txtbtn);
		TxtArrbtn = new QPushButton(" Text Array   ", typeBox); typeBoxLayout->addWidget(TxtArrbtn);
		tbtn = new QPushButton(" Time value ", typeBox); typeBoxLayout->addWidget(tbtn);
		tibtn = new QPushButton(" Time Interval  ", typeBox); typeBoxLayout->addWidget(tibtn);
		Sbtn = new QPushButton(" Scalar Constant  ", typeBox); typeBoxLayout->addWidget(Sbtn);
		Vbtn = new QPushButton(" Vector Constant  ", typeBox); typeBoxLayout->addWidget(Vbtn);
		Abtn = new QPushButton(" Array Constant  ", typeBox); typeBoxLayout->addWidget(Abtn);
		connect(Txtbtn,SIGNAL(clicked()), this, SLOT(createText()));
		connect(TxtArrbtn,SIGNAL(clicked()), this, SLOT(createTextArr()));
		connect(tbtn,SIGNAL(clicked()), this, SLOT(createTime()));
		connect(tibtn,SIGNAL(clicked()), this, SLOT(createTInterval()));
		connect(Sbtn,SIGNAL(clicked()), this, SLOT(createConstant()));
		connect(Vbtn,SIGNAL(clicked()), this, SLOT(createVConstant()));
		connect(Abtn,SIGNAL(clicked()), this, SLOT(createArray()));
		typeBox->setLayout(typeBoxLayout);
		typeBox->adjustSize();
	}
	objTypeSelector->show();
}
void WLView::sortWLPrefs()
{
	if(sortTypeSelector==NULL)
	{
		
		sortTypeSelector = new QDialog(0);
		
		QWidget *typeBox = new QWidget(sortTypeSelector);
		QVBoxLayout* typeBoxLayout=new QVBoxLayout(typeBox);
		typeBoxLayout->setMargin(10);
		typeBoxLayout->setSpacing(6);
		
		QLabel *title = new QLabel("  Working List Sort Preferences  ");
		typeBoxLayout->addWidget(title);

		QLabel *titleSpace = new QLabel("  ");
		typeBoxLayout->addWidget(titleSpace);

		QCheckBox *revTickBox = new QCheckBox(" Sort in Reverse order", typeBox);
		typeBoxLayout->addWidget(revTickBox);
		revTickBox->setChecked(false);
		
		QLabel *spacer = new QLabel("  ");
		typeBoxLayout->addWidget(spacer);

		QRadioButton *nameBtn = new QRadioButton(" Sort on Name  ", typeBox); 
		typeBoxLayout->addWidget(nameBtn);
		nameBtn->setChecked(true);
		
		QRadioButton *timeBtn = new QRadioButton(" Sort on Start Time ", typeBox); 
		typeBoxLayout->addWidget(timeBtn);
		
		QRadioButton *typeBtn = new QRadioButton(" Sort on Object Type ", typeBox); 
		typeBoxLayout->addWidget(typeBtn);

		QLabel *spacer2 = new QLabel("  ");
		typeBoxLayout->addWidget(spacer2);

		
		connect(nameBtn,SIGNAL(clicked(bool)), this, SLOT(setSortWLname(bool)));
		connect(timeBtn,SIGNAL(clicked(bool)), this, SLOT(setSortWLtime(bool)));
		connect(typeBtn,SIGNAL(clicked(bool)), this, SLOT(setSortWLtype(bool)));
		connect(revTickBox,SIGNAL(clicked(bool)), this, SLOT(setSortRev(bool)));
		
		typeBox->setLayout(typeBoxLayout);
		typeBox->adjustSize();
	}
	sortTypeSelector->show();
}

void WLView::createText()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::createText %ld\n", (long)this);
		fflush(stdout);
	}
	WLItem* currItem=wlmodel()->itemFromIndex(currentIndex());
	QString folderName;
	if(currItem!=NULL)
	{
		folderName=(currItem->isFolder() ? currItem->getPath(): currItem->parent()->getPath());
	}

	QdObject_var nil_obj;
	DataBrowseWindow *QBROWSER_widget;
	QdObject_var dobj;
	QdString_var sobj = new QdString(" ");

	dobj = sobj;

	objTypeSelector->close();
	QString path =  folderName+QString("/newText");

	QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, "", dobj, path);
	QBROWSER_widget->name_field->setText("newText"); // set here so that it doesn't think object exists on WL
	QBROWSER_widget->allowEdit();

}

void WLView::createTextArr()
{

	if(QuSlotDebug())
	{
		printf("mainMenu::createTextArr %ld\n", (long)this);
		fflush(stdout);
	}
	WLItem* currItem=wlmodel()->itemFromIndex(currentIndex());
	QString folderName;
	if(currItem!=NULL)
	{
		folderName=(currItem->isFolder() ? currItem->getPath(): currItem->parent()->getPath());
	}
 
	objTypeSelector->close();
    
	arrTypeSelector = new arrDialog(this, "arrTypeSelector", 0, true);
	
	arrTypeSelector->arrType = QB_STRING;
	arrTypeSelector->folderPath = folderName;
	arrTypeSelector->show();
}

void WLView::createTime()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::createTime %ld\n", (long)this);
		fflush(stdout);
	}
	
	WLItem* currItem=wlmodel()->itemFromIndex(currentIndex());
	QString folderName;
	if(currItem!=NULL)
	{
		folderName=(currItem->isFolder() ? currItem->getPath(): currItem->parent()->getPath());
	}
 
	QdTime_var dobj;
	QdObject_var nil_obj;
	DataBrowseWindow *QBROWSER_widget;
  
	dobj = new QdTime("2001-09-05 00:00:00.000");
	objTypeSelector->close();
  
	QdString_var SI_ptr = new QdString("1>s");
	dobj->change_xref(SI_CONVERSION, SI_ptr);
	QString path =  folderName+QString("/new");
	QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, "", dobj, path);
	QBROWSER_widget->name_field->setText("new"); // set here so that it doesn't think object exists on WL
	QBROWSER_widget->allowEdit();
}

void WLView::createTInterval()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::createTInterval %ld\n", (long)this);
		fflush(stdout);
	}
	
	WLItem* currItem=wlmodel()->itemFromIndex(currentIndex());
	QString folderName;
	if(currItem!=NULL)
	{
		folderName=(currItem->isFolder() ? currItem->getPath(): currItem->parent()->getPath());
	}

	objTypeSelector->close();

	QdTimeInterval_var dobj = new QdTimeInterval;

	dobj->set( QmjdTime(2001, 9, 4), QmjdTime(2001, 9, 5) );

	QdObject_var nil_obj;
	DataBrowseWindow *QBROWSER_widget;
	QString path =  folderName+QString("/new");
	QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, "", dobj, path);
	QBROWSER_widget->name_field->setText("new"); // set here so that it doesn't think object exists on WL
	QBROWSER_widget->allowEdit();
}

void WLView::createConstant()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::createConstant %ld\n", (long)this);
		fflush(stdout);
	}
	
	WLItem* currItem=wlmodel()->itemFromIndex(currentIndex());
	QString folderName;
	if(currItem!=NULL)
	{
		folderName=(currItem->isFolder() ? currItem->getPath(): currItem->parent()->getPath());
	}

	QdRScalar_var dobj;
	QdObject_var nil_obj;
	DataBrowseWindow *QBROWSER_widget;

	dobj = new QdRScalar(0.0);
	objTypeSelector->close();

	QdString_var SI_ptr = new QdString("1>?");
	dobj->change_xref(SI_CONVERSION, SI_ptr);
	QString path =  folderName+QString("/newScalar");
	QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, "", dobj, path);
	QBROWSER_widget->name_field->setText("newScalar"); // set here so that it doesn't think object exists on WL
	QBROWSER_widget->allowEdit();
}

void WLView::createVConstant()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::createVConstant %ld\n", (long)this);
		fflush(stdout);
	}
	
	WLItem* currItem=wlmodel()->itemFromIndex(currentIndex());
	QString folderName;
	if(currItem!=NULL)
	{
		folderName=(currItem->isFolder() ? currItem->getPath(): currItem->parent()->getPath());
	}
 
	Dim mDim = Dim(3);
  
	QdObject_var nil_obj;
	DataBrowseWindow *QBROWSER_widget;
  
	QdRMatrix_var dobj = new QdRMatrix(mDim, 0);
  
	QdString_var Frame_ptr = new QdString("vector>gse_xyz");
	dobj->change_xref(FRAME, Frame_ptr);
  
	QdString_var SI_ptr = new QdString("1>?");
	dobj->change_xref(SI_CONVERSION, SI_ptr);
	QString path =  folderName+QString("/newVector");  
	QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, "", dobj, path);
	QBROWSER_widget->name_field->setText("newVector"); // set here so that it doesn't think object exists on WL
	QBROWSER_widget->allowEdit();
	objTypeSelector->close();
}

void WLView::createArray()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::createArray %ld\n", (long)this);
		fflush(stdout);
	}
	
	WLItem* currItem=wlmodel()->itemFromIndex(currentIndex());
	QString folderName;
	if(currItem!=NULL)
	{
		folderName=(currItem->isFolder() ? currItem->getPath(): currItem->parent()->getPath());
	}
  
	objTypeSelector->close();
  
	arrTypeSelector = new arrDialog(this, "arrayDimChooser", 0, true);
	
	arrTypeSelector->arrType = QB_DOUBLE;
	arrTypeSelector->folderPath = folderName;
	arrTypeSelector->move(qsasGUISize().width(),30);
	arrTypeSelector->show();
}

void WLView::sliceObj()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchSlice %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=currentIndex();
	WLItem* item=wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	DataSliceWindow *SliceWindow = new DataSliceWindow(0);
	SliceWindow->setObject(path.toLatin1().data());
	SliceWindow->move(qsasGUISize().width(),30);
	SliceWindow->show();
}


void WLView::duplicateObj()
{
	QModelIndex index=currentIndex();
	WLItem* currItem=wlmodel()->itemFromIndex(index);
	if(wlmodel()->notEditable(currItem)) return;

	QString path=currItem->getPath();
	QdObject_var dobj=QuGetObjByName(path);
	QdObject_var newObj = Factory::clone((QdObject_var) dobj);
	newObj->copy_xrefs_from(dobj);
	
	QwuiPutOnWL(newObj, path+"_copy");

}

void WLView::tif2ev()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchSlice %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=currentIndex();
	WLItem* item=wlmodel()->itemFromIndex(index);
	if(wlmodel()->notEditable(item)) return;
	
	if(!item->isFolder()) return;
	
	QdTimeIntervalSeq_var itv(new QdTimeIntervalSeq());
		
	for(int i=0; i<item->numChildren(); ++i)
	{
		WLItem* currentItem=item->getChildByNumber(i);
		if(!currentItem->isFolder())
		{
			QdObject_var temp=QuGetObjByName(currentItem->getPath());
			QdTimeInterval_var temp_ti=QdTimeInterval_var::narrow(temp);
				
			if(!temp_ti.is_nil())
			{
				itv->push_back(*temp_ti);
			}
			else
			{
				QdTimeIntervalSeq_var temp_tis=QdTimeIntervalSeq_var::narrow(temp);
				if(!temp_tis.is_nil())
				{
					for(vector<QmjdTimeInterval>::iterator iter=temp_tis->begin(); iter!=temp_tis->end(); iter++)
					{
						QdTimeInterval foo(*iter);
						itv->push_back(foo);
					}
				}
			}
		}
	}
		
	QString name=QString(item->getName())+QString(" (evt table)");
		
	QwuiPutOnWL((QdObject_var)itv, "Time Intervals/"+name);
}

void WLView::ev2tif()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchSlice %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=currentIndex();
	WLItem* item=wlmodel()->itemFromIndex(index);
	if(item==NULL) return;
	if(item->isFolder()) return;

	QdObject_var obj=QuGetObjByName(item->getPath());
	QdTimeIntervalSeq_var dobj=QdTimeIntervalSeq_var::narrow(obj);

	if(dobj.is_nil())
	{
		return;
	}

	QString folderPath=item->getPath();
	folderPath.append("_folder");
	WLItem* folder=wlmodel()->createItem(folderPath);
	folder->setFolder();
	
	int i=dobj->sequence_size()-1;
	for(QdTimeIntervalSeq::reverse_iterator iter=dobj->rbegin(); iter!=dobj->rend(); iter++, i--)
	{
		QdTimeInterval_var temp=new QdTimeInterval(*iter);
		QString name=folderPath+QString("/evt")+QString().setNum(i);
		QString context=QwuiPutOnWL(temp, name);
	}
		
}

QList<WLItem*> WLView::selectedItems()
{
	QModelIndexList indexes=selectionModel()->selectedRows();
	
	QList<WLItem*> res;
	res.clear();
	
	foreach(QModelIndex index, indexes)
	{
		WLItem* item=wlmodel()->itemFromIndex(index);
		if(item!=NULL && !res.contains(item))
		{
			res.push_back(item);
		}
	}
	return res;
}

void WLView::setCurrentItem(WLItem* i)
{
	QModelIndex index=wlmodel()->indexFromItem(i);
	if(index.isValid())
	{
		selectionModel()->setCurrentIndex(/*currentIndex()*/index, QItemSelectionModel::SelectCurrent);
	}
}

void WLView::deleteObjectFromPath(const QString& path)
{
    WLItem* item=wlmodel()->itemFromPath(path);
    if(item!=NULL)
    {
        wlmodel()->deleteItem(item);
    }
}

void WLView::deleteObject()
{
	QModelIndexList indexes=selectionModel()->selectedRows();
	QList<WLItem*> itemList;
	itemList.clear();

	foreach(QModelIndex index, indexes)
	{
		if(index != rootIndex()){
			WLItem *item = wlmodel()->itemFromIndex(index);
			itemList.push_back(item);
		}
	}
	clearSelection();

	// select current to something never deleted
	selectionModel()->setCurrentIndex(rootIndex(), QItemSelectionModel::NoUpdate );

 	// now remove children of items on list
	int i=0;
	while(i<itemList.size())
	{
		WLItem* item=itemList[i];	
		// remove any item whose ancestor is also selected
		if( item->ancestorInList(itemList) )  itemList.removeAt(i);
		else i++;

	}
	
	// now finally safe to start deleting
	for(int i=0; i<itemList.size(); i++)
	{
		WLItem* item=itemList[i];	
		wlmodel()->deleteItem(item);
	
	}
	
}

void WLView::removeTimeTags()
{
	QModelIndexList indexes=selectionModel()->selectedRows();
	
	foreach(QModelIndex index, indexes)
	{
		WLItem* item=wlmodel()->itemFromIndex(index);
		if(item!=NULL)
		{
			QdObject_var dobj=QuGetObjByName(item->getPath());
			
			if (dobj->xref_exists(DEPEND_0)) dobj->delete_xref(DEPEND_0);  
		}
		QuMsgBar("Time Tags removed: ", item->getPath().toLatin1().data(), false);

	}
}

void WLView::selectAllData()
{
	QList <QModelIndex> dList = wlmodel()->getDataList();
	for(int i=0; i<dList.size(); i++)
	{
		selectionModel()->select(dList[i], QItemSelectionModel::Select);
	}
}

void WLView::deleteAllDataItems()
{
	wlmodel()->deleteAllDataItems();
}

void WLView::deleteAllTimeIntervalItems()
{
	wlmodel()->deleteAllTimeIntervalItems();
}

void WLView::deleteAllConstantItems()
{
	wlmodel()->deleteAllConstantItems();
}

void WLView::clearCacheSelected()
{
	QModelIndexList indexes=selectionModel()->selectedRows();
	
	foreach(QModelIndex index, indexes)
	{
		QdObject_var dobj = QuGetObjByName(wlmodel()->itemFromIndex(index)->getPath());
		if(!dobj.is_nil())
		{
			vector<string> sl;
			dobj->list_xref_names(sl);
			for(int i=0; i<sl.size(); i++)
			{
				if( sl[i][0] == '_') dobj->delete_xref(sl[i].c_str());
			}
			QuMsgBar("Cache cleared for ", wlmodel()->itemFromIndex(index)->getPath().toLatin1().data(), false);
		}
	}

}

void WLView::clearCache()
{
	selectAll();
	clearCacheSelected();
	clearSelection();
	QuMsgBar("Cache cleared", false);
}

void WLView::newSelectedIndexes(const QModelIndexList& l)
{
	QModelIndexList indexes=selectionModel()->selectedRows();

	foreach(QModelIndex index, indexes)
	{
		selectionModel()->select(index, QItemSelectionModel::Deselect);
	}
	foreach(QModelIndex index, l)
	{
		selectionModel()->select(index, QItemSelectionModel::Select);
	}
	
}

void WLView::drawRow(QPainter * painter, const QStyleOptionViewItem& option, const QModelIndex& index) const
{
	QStyleOptionViewItem newOption=option;

	WLItem* item=wlmodel()->itemFromIndex(index);
	
	if(item==NULL) return;
	
#if QT_VERSION >= 0x040300
	if(item==wlmodel()->_data || item==wlmodel()->_time || item==wlmodel()->_constants || item==wlmodel()->_temp ) 
	{
		newOption.displayAlignment=Qt::AlignCenter;
		newOption.font.setBold(true);
		
		QLinearGradient grad(newOption.rect.topLeft(), newOption.rect.bottomLeft());
		grad.setColorAt(0, QColor(255, 255, 255));
		grad.setColorAt(1, QColor(230, 230, 230));
		QBrush brush(grad);
		QBrush oldBrush=painter->brush();
		QPen oldPen=painter->pen();
		painter->setBrush(brush);
		QPen pen(QColor(180, 180, 180));
		painter->setPen(pen);
		
		painter->drawRect(newOption.rect);
		painter->setBrush(oldBrush);
		painter->setPen(oldPen);
	}
	else if(item->isFolder())
	{
		newOption.font.setBold(true);
	}
#else
	if(item->isFolder())
	{
		newOption.font.setBold(true);
	}
#endif
	QTreeView::drawRow(painter, newOption, index);
}

void WLView::dragEnterEvent(QDragEnterEvent *event)
{
	if(QuSlotDebug()) printf("WLView::dragEnterEvent %ld\n", (long)this);
	event->acceptProposedAction();
	activateWindow();
	raise();
}

void WLView::copy()
{
	QModelIndexList indexes=selectionModel()->selectedRows();

	QMimeData* data=wlmodel()->mimeData(indexes);

	QApplication::clipboard()->setMimeData(data);
}
