//  qdi.cc
//
// Direct Import Widget
//
// Sept 2002: Tony Allen
//

#include <stdlib.h>
#include "qcheckbox.h"

#include "qbu.h"
#include "qdi.h"

#include "qie_qsas.h"
#include "qie_caa.h"
#include "qwui.h"

extern qwuiWindow* qsasWindow;

vector <int> importedVarNos;
vector <QdObject_var> importedVars;

// -----------------------------------------------------------------------

void setAsSeparator(QListWidgetItem* item)
{
    QFont font=item->font();
    font.setBold(true);
    item->setFont(font);

    if(item->listWidget()!=NULL)
    {
        QRect rect=item->listWidget()->visualItemRect(item);

        QLinearGradient grad(QPoint(0,0), QPoint(0, rect.height()));
        grad.setColorAt(0, QColor(255, 255, 255));
        grad.setColorAt(1, QColor(230, 230, 230));
        QBrush brush(grad);
        item->setBackground(brush);
    }

    item->setTextAlignment(Qt::AlignCenter);
    QBrush brush2(Qt::black);
    item->setForeground(brush2);
    
    item->setFlags(Qt::NoItemFlags);
}

DirectImportWindow::DirectImportWindow( QWidget *parent, const char *name ) 
  : QMainWindow(parent)
{

	// Status bar
	
	msgBar = statusBar();
	
	// Menu
	
	menuBar()->addSeparator();
	QMenu * help =menuBar()->addMenu(" Help");
	help->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->addSeparator();
	help->addAction("...Data Import", this, SLOT( launchManDirectImport() ));
	help->addSeparator();
	help->addAction("Module Version", this, SLOT( launchVersion() ));
	
	// Main widget
	
	QGroupBox * topWidget=new QGroupBox(this);
	QVBoxLayout * topLayout=new QVBoxLayout;
	
	// List panel
	
	QHBoxLayout * listLayout=new QHBoxLayout;
    
    QVBoxLayout* varLayout=new QVBoxLayout;
    QLabel* varLabel=new QLabel("Variables");
    varLabel->setFixedWidth(200);
    varLabel->setAlignment(Qt::AlignCenter);
    varLayout->addWidget(varLabel);
    varList=new QListWidget;
    varList-> setSortingEnabled(false);
    varList->setFixedWidth(200);
    connect( varList, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(clearGattrList()) );
    connect( varList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(varDoubleClick(QListWidgetItem*)) );
    activeList = varList;
    varLayout->addWidget(varList);
    listLayout->addLayout(varLayout);

    QVBoxLayout* GattrLayout=new QVBoxLayout;
    QLabel* GattrLabel=new QLabel("Global Attributes");
    GattrLabel->setFixedWidth(200);
    GattrLabel->setAlignment(Qt::AlignCenter);
    GattrLayout->addWidget(GattrLabel);
    GattrList=new QListWidget;
    GattrList-> setSortingEnabled(true);
    GattrList->setFixedWidth(200);
    connect( GattrList, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(clearVarList()) );
    connect( GattrList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(GattrDoubleClick(QListWidgetItem*)) );
    GattrLayout->addWidget(GattrList);
    listLayout->addLayout(GattrLayout);
	
	topLayout->addLayout(listLayout);
	
	// Subset panel
	
	QGroupBox *subset_panel=new QGroupBox(topWidget);
	QVBoxLayout * subset_layout=new QVBoxLayout;
	
	fromSlider = new  RecordSlider("From ", subset_panel, "fromSlider");
	subset_layout->addWidget(fromSlider);
	
	toSlider =   new  RecordSlider("To ", subset_panel, "toSlider");
	subset_layout->addWidget(toSlider);
	
	subset_panel->setLayout(subset_layout);
	topLayout->addWidget(subset_panel);
	
	// Buttons
	
	QHBoxLayout * button_layout=new QHBoxLayout;
	
	getVarBtn = new QPushButton( "Get Data");
	getVarBtn->setMinimumSize(getVarBtn->sizeHint());
	getVarBtn->setDefault(TRUE);
	connect( getVarBtn, SIGNAL(clicked()), this, SLOT(getData()) );
	button_layout->addWidget(getVarBtn);

	browseVarBtn = new QPushButton( "Browse Data");
	browseVarBtn->setMinimumSize(browseVarBtn->sizeHint());
	connect( browseVarBtn, SIGNAL(clicked()), this, SLOT(browseData()) );
	button_layout->addWidget(browseVarBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	button_layout->addWidget(dismissBtn);

	
	topLayout->addLayout(button_layout);
	
	
	topWidget->setLayout(topLayout);
	setCentralWidget(topWidget);
	
	QieOpt = (QiOptions *) NULL;
	QieContents = (QiCDFContents *) NULL;
	setAttribute(Qt::WA_DeleteOnClose);
}

void DirectImportWindow::keyPressEvent(QKeyEvent * e )
{  
  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if(   getVarBtn->isDefault()) {
		 getVarBtn->setDefault(false);
		 browseVarBtn->setDefault(true);
	  }
	  else if(browseVarBtn->isDefault()) {
		 browseVarBtn->setDefault(false);
		 dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault())  {
		 dismissBtn->setDefault(false);
		 getVarBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(getVarBtn->isDefault()) getData();
	 else if (browseVarBtn->isDefault()) browseData();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void DirectImportWindow::launchManual(){ 

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::launchManual %ld\n", (long)this);
  fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("index.html");
}

void DirectImportWindow::launchManDirectImport(){ 

 if(QuSlotDebug())
 {
	printf("DirectImportWindow::launchManDirectImport %ld\n", (long)this);
	fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}


void DirectImportWindow::launchVersion(){

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::launchVersion %ld\n", (long)this);
  fflush(stdout);
  }
 
  
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n Direct Import:  ";
  text += QString(QIE_VERSION);
  QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" ); 

}


void DirectImportWindow::getFileContents(const char *file){

char * ptr;
char * ptr2;
long error;
char *fnamecpy;
int openType;

  QieOpt = QiFreeOptionsObj(QieOpt);                 // always safe
  QieContents = QiFreeCDFContentsObj(QieContents);   // always safe

  QieOpt = (QiOptions *) QiMakeOptionsObj();
  QieContents = (QiCDFContents *) QiMakeCDFContentsObj();

   /* keep global copy of path to add as xref later */
   
    Selected_File_Path = QString(file);
 
    /* parse copy of file name into path, name, extn */
    fnamecpy = QiNewStr(file);

    if( (ptr = strrchr(fnamecpy, '.')) != NULL) {
	  if( strcmp(ptr, ".gz") == 0){
	     // find real extension
	     if( (ptr2 = strstr(fnamecpy, ".cef.gz")) != NULL) ptr = ptr2;
	  }
      QieContents->io_f_extn = QiNewStr(ptr);
      ptr[0] = '\0';
    }
    if( (ptr = strrchr(fnamecpy, '/')) != NULL) {
      QieContents->io_f_name = QiNewStr(&(ptr[1]));
      ptr[0] = '\0';
      QieContents->io_f_path = QiNewStr(fnamecpy);
    }
    else{
      QieContents->io_f_name = QiNewStr(fnamecpy);
      QieContents->io_f_path = QiNewStr("./");
    }
    if( !QistrNULL(fnamecpy) ) free (fnamecpy);

    /* set up options object */


    QieOpt->fp_display = stdout;

    if(QuSlotDebug())
       QieOpt->debug_choice = 'f';
    else
       QieOpt->debug_choice = 'n';
	  
    // check for a recognised file extension and read in data to Contents 

    if (strcmp(QieContents->io_f_extn, ".cdf") == 0 || 
        strcmp(QieContents->io_f_extn, ".CDF") == 0){
        openType = TYPE_CDF;
        QieOpt->f_type = UNSET;  // cdf interface does not use this
    }
    else if (strcmp(QieContents->io_f_extn, ".qft") == 0){
      openType = TYPE_FLAT;
      QieOpt->f_type = TABULAR;
    }
    else if (strcmp(QieContents->io_f_extn, ".cef") == 0 || strcmp(QieContents->io_f_extn, ".cef.gz") == 0){
      openType = TYPE_FLAT;
      int ask = -1; // this tells cefRead2() we are asking not setting flag
      if(cefRead2(ask)) 
         QieOpt->f_type = CAA;
      else
         QieOpt->f_type = EXCHANGE;
    }
    else if (strcmp(QieContents->io_f_extn, ".qfd") == 0){
      openType = TYPE_FLAT;
      QieOpt->f_type = DELIMITED;
    }
    else if (strcmp(QieContents->io_f_extn, ".qfh") == 0){
      openType = TYPE_HEADER;
      QieOpt->f_type = UNSET;
    }
    else{
      openType = TYPE_FLAT;
      QieOpt->f_type = UNSET;

    }

    switch(openType){
    
      case TYPE_CDF:
     
     	error=QiGetCSDSgenCDF(file, QieContents);
        if(error != QMW_OK){
          QuMsgBar( "ERROR reading file ", file, true);
          msgBar->showMessage(QString(QiErrStr(error)) );
	  return;
        }     
      
        break;

      case TYPE_FLAT:
        
        if(QieOpt->f_type == CAA ) initQIE(true);
	    else initQIE(false);

        error=QiGetCSDSgenFlat(QieContents, QieOpt);

        if( error != QMW_OK){
          QuMsgBar( "ERROR reading file ", file, true);
          msgBar->showMessage(QString(QiErrStr(error)) );
	      return;
        }

        // strip out any incomplete variables 

        if (QiMakeSafe(QieContents) == NULL) {
          QuMsgBar( "ERROR reading file ", file, true);
          msgBar->showMessage(QString(QiErrStr(error)) );
	      return;
        }
        
        break;


      case TYPE_HEADER:
        error=QiGetCSDSgenFlat(QieContents, QieOpt);
        if(error != QMW_OK){
			QuMsgBar( "ERROR reading file ", file, true);
			msgBar->showMessage(QString(QiErrStr(error)) );
			return;
        }
        
        break;

      default:

        QuMsgBar( "Unknown file type ", file, true);
        msgBar->showMessage(QString(QiErrStr(error)) );
	return;
    }

    // Set up display if file read OK
    bool first = true;
    GattrList->clear();
    for(int i=0; i<QieContents->num_g_attrs; i++){
      QListWidgetItem* newGattrItem=new QListWidgetItem(QieContents->g_attr[i]->name, GattrList);
      QSize size=newGattrItem->sizeHint();
      size.setWidth(180);
      size.setHeight(20);
      newGattrItem->setSizeHint(size);
    }
//     vector <QListWidgetItem*> metaList;
//     vector <QListWidgetItem*> dataList;
    QLinkedList<QListWidgetItem*> metaList;
    QLinkedList<QListWidgetItem*> dataList;
    varList->clear();

    QListWidgetItem* topItem=new QListWidgetItem("Data", varList);
    setAsSeparator(topItem);

    for(int i=0; i<QieContents->n_vars; i++){
      bool isData = true;
      if(QieContents->vardata[i] == NULL) continue;
      for (int j=0; j< QieContents->vardata[i]->num_v_attrs; j++){
	   if( strcmp(QieContents->vardata[i]->attribute[j].name, "PARAMETER_TYPE") == 0) {   //  CAA standard
	     if (QuStrCmp( (char *)QieContents->vardata[i]->attribute[j].data, "Support_Data")) {
		  isData = false;
		  break;
		}
	   }
	   if( strcmp(QieContents->vardata[i]->attribute[j].name, "VAR_TYPE") == 0) {   // ISTP/CSDS standard
	     if (QuStrCmp( (char *)QieContents->vardata[i]->attribute[j].data, "metadata") ) {
		  isData = false;
		  break;
	     }
	     else if (QuStrCmp( (char *)QieContents->vardata[i]->attribute[j].data, "support_data") ) { // added for Themis
		  isData = false;
		  break;
	     }
	   }
	 }

      QListWidgetItem* newItem=new QListWidgetItem(QieContents->vardata[i]->name, NULL);
      QSize size=newItem->sizeHint();
      size.setWidth(180);
      size.setHeight(20);
      newItem->setSizeHint(size);
      if(isData){
        dataList.push_back(newItem);
	 }
	 else{
        metaList.push_back(newItem);
	 }
    }
    
 
    // now move metadata

    
//     for(int i=0; i<dataList.size(); ++i){
//        varList->addItem(dataList[i]);
//     }
    foreach(QListWidgetItem* i, dataList)
    {
        varList->addItem(i);
    }

    QListWidgetItem* sepItem=new QListWidgetItem("Support Data", varList);
    setAsSeparator(sepItem);

//     for(int i=0; i<metaList.size(); ++i){
//         varList->addItem(metaList[i]);
//     }
    foreach(QListWidgetItem* i, metaList)
    {
        varList->addItem(i);
    }
    

  // get time line and keep globally for this file

  int d0_var_num = QieContents->d0_var_num;
  if( d0_var_num >= 0){
	QdObject_var d0 =  (QdiVarToObj ( QieContents, d0_var_num ));
    QdTimeSeq_var tt =  QdTimeSeq_var::narrow(d0);
    QdRScalarSeq_var ss =  QdRScalarSeq_var::narrow(d0);
	if(!tt.is_nil()){
    	fromSlider->ttags = tt;
    	toSlider->ttags = tt;
	}
	else if(!ss.is_nil()){
    	fromSlider->stags = ss;
    	toSlider->stags = ss;
	}
  }

  if(QieContents->n_recs < 1) {
     fromSlider->recBar->setRange(0, 0);
     toSlider->recBar->setRange(0, 0);
     getVarBtn->setEnabled(false);
	 browseVarBtn->setEnabled(false);
     fromSlider->maxRecord = 0;
     toSlider->maxRecord = 0;
	 msgBar->showMessage("WARNING: No records were read, empty file?" );
  }
  else if(QieContents->n_recs == 1) {
     fromSlider->recBar->setRange(0, 0);
     toSlider->recBar->setRange(0, 0);
     getVarBtn->setEnabled(false);
	 browseVarBtn->setEnabled(false);
     fromSlider->maxRecord = 0;
     toSlider->maxRecord = 0;
  }
  else{
     fromSlider->recBar->setRange(0, QieContents->n_recs-1);
     toSlider->recBar->setRange(0, QieContents->n_recs-1);
     fromSlider->maxRecord = QieContents->n_recs-1;
     toSlider->maxRecord = QieContents->n_recs-1;
     toSlider->recBar->setValue(toSlider->maxRecord);
     fromSlider->recBar->setValue(0);
  }
  
 
}

void DirectImportWindow::clearVarList(){

    if(QuSlotDebug())
    {
        printf("DirectImportWindow::clearVarList %ld\n", (long)this);
        fflush(stdout);
    }
 

//     Q_LIST_VIEW_ITEM * next = varList->firstChild();
//     while(next){
//         varList->setSelected(next, false);
//         next->listView()->update();
//         next = next->nextSibling();
//     }
  
    varList->clearSelection();
    activeList = GattrList;
}

void DirectImportWindow::clearGattrList(){

    if(QuSlotDebug())
    {
        printf("DirectImportWindow::clearGattrList %ld\n", (long)this);
        fflush(stdout);
    }
 

//     Q_LIST_VIEW_ITEM * next = GattrList->firstChild();
//     while(next){
//         GattrList->setSelected(next, false);
//         next->listView()->update();
//         next = next->nextSibling();
//     }
  
    GattrList->clearSelection();
    activeList = varList;
}


void DirectImportWindow::browseData()
{

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::browseData %ld\n", (long)this);
  fflush(stdout);
 }
 

  QdObject_var  dobj;
  QdObject_var  nil_obj;
  long vnum;
  DataBrowseWindow *QBROWSER_widget;

  // browse either list 
  
  if( activeList == varList) {

      QListWidgetItem* item=varList->currentItem();
     if(item == 0){
       msgBar->showMessage( "No variable selected");
       return;
     }   
    vnum = QdiFindVarNum(QieContents, item->text().toLatin1().data() );
  
    if(vnum == -1 || QieContents->vardata[vnum]->max_rec_num < 0) {
      msgBar->showMessage( "No Data Object selected to browse");
      return;
    }

    dobj = qie2qdos(vnum);
     
    //    open browser
    if( !dobj.is_nil() ){
    
      QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, 
                        QieContents->vardata[vnum]->name, dobj, ""); 
      if(QBROWSER_widget != 0) {
         QBROWSER_widget->disallowSaves = true;
	 if(QBROWSER_widget->browser != 0)
	     QBROWSER_widget->editMenu->setEnabled(false);
	     QBROWSER_widget->browser->parentsBtn->setEnabled(false);
      }
    }
  }
  else if( activeList == GattrList){
      QListWidgetItem* item=GattrList->currentItem();
     if(item == 0){
       msgBar->showMessage( "No G Attr selected");
       return;
     }
    
     // open G attr
     long Anum = QdiFindGattrNum(QieContents, item->text().toLatin1().data() );
     dobj = QdiGattrToObj(QieContents, Anum);

     //    open browser
     if( !dobj.is_nil() ){
        QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, 
                        QieContents->g_attr[Anum]->name, dobj, "");
        if(QBROWSER_widget != 0) {
	  QBROWSER_widget->disallowSaves = true;
	  if(QBROWSER_widget->browser != 0)
	     QBROWSER_widget->editMenu->setEnabled(false);
	     QBROWSER_widget->browser->parentsBtn->setEnabled(false);
	}
     }
  }
  

}


void DirectImportWindow::varDoubleClick(QListWidgetItem* item){

    if(QuSlotDebug())
    {
        printf("DirectImportWindow::varDoubleClick %ld, item %ld\n", (long)this, (long)item);
        fflush(stdout);
    }

    if(item == 0) return;
    activeList = varList;
    varList->setCurrentItem(item);
    getData();

}

void DirectImportWindow::GattrDoubleClick(QListWidgetItem* item){

    if(QuSlotDebug())
    {
        printf("DirectImportWindow::GattrDoubleClick %ld, item %ld\n", (long)this, (long)item);
        fflush(stdout);
    }
 

    if(item == 0) return;
    activeList = GattrList;
    GattrList->setCurrentItem(item);
    browseData();

}

void DirectImportWindow::getData()
{

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::getData %ld\n", (long)this);
  fflush(stdout);
 }

  QdObject_var  dobj;
  long vnum;

  // import from either list 
  
  if( activeList == varList) {
  
      QListWidgetItem* item=varList->currentItem();
    if(item == 0) {
      msgBar->showMessage( "No variable selected");
      return;
    }
    vnum = QdiFindVarNum(QieContents, item->text().toLatin1().data() );
    if(vnum == -1 || QieContents->vardata[vnum]->max_rec_num < 0) {
      msgBar->showMessage( "No Data Object selected");
      return;
    }
    
     dobj = qie2qdos(vnum);
     
//    Add object name to WL
    if( !dobj.is_nil() ){
		QdTimeInterval_var ti = QdTimeInterval_var::narrow(dobj);
		QdTimeIntervalSeq_var tis = QdTimeIntervalSeq_var::narrow(dobj);

		if(ti.is_nil() && tis.is_nil()){
      		string sanitizedVarName = QuSanitizeName(QieContents->vardata[vnum]->name);
      		QwuiPutOnWL((QdObject_var) dobj, QString("Data/")+QString(sanitizedVarName.c_str()));
		}
		else{
      		string sanitizedVarName = QuSanitizeName(QieContents->vardata[vnum]->name);
      		QwuiPutOnWL((QdObject_var) dobj, QString("Time Intervals/")+QString(sanitizedVarName.c_str()));
		}
    } 
  }
  else if(activeList == GattrList){

      QListWidgetItem* item=GattrList->currentItem();
     if(item == 0){
       msgBar->showMessage( "No G Attr selected");
       return;
     }
    
     // open G attr
     long Anum = QdiFindGattrNum(QieContents, item->text(/*Obj_Name_*/).toLatin1().data() );
     dobj = QdiGattrToObj(QieContents, Anum);

     //   put G attr on WL
     if( !dobj.is_nil() ){
		QdTimeInterval_var ti = QdTimeInterval_var::narrow(dobj);
		QdTimeIntervalSeq_var tis = QdTimeIntervalSeq_var::narrow(dobj);
		if(ti.is_nil() && tis.is_nil()){
       		string sanitizedGattrName = QuSanitizeName(QieContents->g_attr[Anum]->name);
       		QwuiPutOnWL((QdObject_var) dobj, QString("Data/")+QString(sanitizedGattrName.c_str()));
		}
		else{
       		string sanitizedGattrName = QuSanitizeName(QieContents->g_attr[Anum]->name);
       		QwuiPutOnWL((QdObject_var) dobj, QString("Time Intervals/")+QString(sanitizedGattrName.c_str()));
		}
     }

  }

}



QdObject_var DirectImportWindow::qie2qdos(long vnum)
{
  QdObject_var  dobj;
  QdObject_var  nil_obj;
     
  importedVarNos.clear();  // clear list of xref vars read
  importedVars.clear();  // clear list of xref vars read
  
  try{

    dobj =  QdiVarToObj ( QieContents, vnum );  

    if ( is_empty(dobj) ) {
      msgBar->showMessage( "Failed to read variable");
      return nil_obj;
    }
 
  }
  catch(Exception &e){
    QString Serr = QString(e.msg().c_str());
    Serr += QString(" variable number %1").arg(vnum);
    msgBar->showMessage(Serr );
    return nil_obj;
  }
  // add to list of variables read in this request
  importedVarNos.push_back(vnum);
  importedVars.push_back(dobj);
  
  // get time line (will be needed by some attributes, so set now)
  
  //   QcvGetTimeVar returns ...
  //   epoch if DEPEND_0 exists and points to an epoch
  //   -2 if DEPEND_0 points to a non-epoch var
  //   -1 if DEPEND_0 does NOT exist but QieContents->d0_var_num exists
  //   -2 if varnum is itself an epoch 
  //   -2 if this variable is not record varying (e.g. most xrefs)
  //   -1 if var is record varying, but time tags not explicitly set

  int  d0_var_num = QdiGetDep0Var(QieContents, vnum);
  if( d0_var_num == -1) d0_var_num = QieContents->d0_var_num; // use first tt from file
  if( d0_var_num >= 0 ){

      // get time line for object (based on Depend_0, not epoch in file) subset records (fromRec, toRec)
      QdObject_var d0_for_var =  QdiVarToObj ( QieContents, d0_var_num );
      if( !d0_for_var.is_nil() ){
	    // add time tags object to list of vars read in
         importedVarNos.push_back(d0_var_num);
 	    importedVars.push_back(d0_for_var);
	    dobj->change_xref(DEPEND_0, d0_for_var);
	    // set xrefs for time tags
	    QdiSet_VAttrs(d0_for_var, QieContents, d0_var_num);
      }
  }


  // Add Xrefs to V attributes for dobj itself
  // NOTE: attributes cannot be set in QdiVarToObj() as this can result in infinite loops for
  // metadata that refers back to data (e.g. time dependent delta plus for time tags)

  QdiSet_VAttrs(dobj, QieContents, vnum);
  
  // Add full file name and path to allow browsing global metadata

  QdStringSeq_var file_seq = new QdStringSeq(1);
  file_seq[0] = QdString(Selected_File_Path.toLatin1().data());
  dobj->change_xref("Parents", (QdObject_var) file_seq); 
    
     
  return dobj;
 
}

void DirectImportWindow::close(){

 if(QuSlotDebug())
 {
  printf("DirectImportWindow::close %ld\n", (long)this);
  fflush(stdout);
 }
 

  QieOpt = QiFreeOptionsObj(QieOpt);            // always safe
  QieContents = QiFreeCDFContentsObj(QieContents);   // always safe
 
  QMainWindow::close();
}

//*************************************************************

QdObject_var  DirectImportWindow::QdiVarToObjSingle( QiCDFContents* info, int vnum, int atRec)
{
	QdObject_var dobj;
	
	// test for Data Object is TimeTags object

	if( info->vardata[vnum]->data_type == CDF_EPOCH ) {

		if(  info->vardata[vnum]->num_dims == 0 || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1 ))
		{ 
			// Time 
 
			QdTime_var to = new QdTime();
			double *ep = (double *)info->vardata[vnum]->data;
			to->setFromCDFepoch(ep[atRec]);
			
			// This is a time sequence object so broaden to the QdObject
			dobj = to;
		}
		else if(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 2){
   
			// assume to be time intervals
	     
			double *ep = (double *)info->vardata[vnum]->data;
			QdTimeInterval_var ti = new QdTimeInterval(); 

			QmjdTime iv_start; 
			iv_start.setFromCDFepoch(ep[2*atRec]);
			QmjdTime iv_end; 
			iv_end.setFromCDFepoch(ep[2*atRec+1]);
			ti->set(iv_start, iv_end );
			
			// This is a time object so broaden to the QdObject
			dobj = ti;
		}
		else{
			// qsas does not handle time arrays
			printf("QdiVarToObjSingle: cannot handle time array\n");
			fflush(stdout);
			return  dobj;
		} // end if on dimensions

	}
	else if( info->vardata[vnum]->data_type == CDF_EPOCH16 ) {

		if(  info->vardata[vnum]->num_dims == 0 || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1 ))
		{ 
			// Time 
 
			QdTime_var to = new QdTime();
			double *ep = (double *)info->vardata[vnum]->data;
			to->setFromCDFepoch16(&(ep[2*atRec]));
			
			// This is a time object so broaden to the QdObject
			dobj = to;
		}
		else if(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 2){
   
			// assume to be time intervals
	     
			double *ep = (double *)info->vardata[vnum]->data;
			QdTimeInterval_var ti = new QdTimeInterval(); 

			QmjdTime iv_start; 
			iv_start.setFromCDFepoch16(&(ep[4*atRec]));
			QmjdTime iv_end; 
			iv_end.setFromCDFepoch16(&(ep[4*atRec+2]));

			ti->set(iv_start, iv_end );
			
			// This is a time sequence object so broaden to the QdObject
			dobj = ti;
		}
		else{
			// qsas does not handle time arrays
			printf("QdiVarToObjSingle: cannot handle time array\n");
			fflush(stdout);
			return  dobj;
		} // end if on dimensions

	}
	else if( info->vardata[vnum]->data_type == ISO_TIME ) {
		int iso_str_len = info->vardata[vnum]->num_elems;
		
		if(	info->vardata[vnum]->num_dims == 0  || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1))
		{
  
			QdTime_var to = new QdTime();
			char *sp = (char *)info->vardata[vnum]->data;

			double epochValue;
			// skip strings up to  record
			int s_ptr = atRec*iso_str_len;
   
			to->setFromISOstring( &(sp[s_ptr]));
   
			// This is a time sequence object so broaden to the QdObject
			dobj = to;
		}
		else if(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 2)
		{
			// assume to be a time interval sequence
		    
			char *sp = (char *)info->vardata[vnum]->data;

			// skip strings up to start record
			int s_ptr = 2*atRec*iso_str_len;

			QdTimeInterval_var ti = new QdTimeInterval(); 
		
			char sp_0[iso_str_len+1];
			char sp_1[iso_str_len+1];
			strncpy(sp_0, &(sp[s_ptr]), iso_str_len);
			s_ptr += iso_str_len;
			strncpy(sp_1, &(sp[s_ptr]), iso_str_len);
			s_ptr += iso_str_len;
	 
			int i = iso_str_len-1;
			while(sp_0[i] == ' ' && i>0) {sp_0[i] = '\0'; i--;} // remove white space
			QmjdTime iv_start(sp_0);	 
			i = iso_str_len-1;
			while(sp_1[i] == ' ' && i>0) {sp_1[i] = '\0'; i--;} // remove white space
			QmjdTime iv_end(sp_1);

			ti->set(iv_start, iv_end); 

			// This is a time sequence object so broaden to the QdObject
			dobj = ti;
		}
		else{ 
			// qsas does not handle time arrays
			printf("QdiVarToObjSingle: cannot handle time array\n");
			fflush(stdout);
			return  dobj;
		}
	}
	else if( info->vardata[vnum]->data_type == ISO_TIME_RANGE ) {
		int iso_str_len = info->vardata[vnum]->num_elems;
		
		if(	info->vardata[vnum]->num_dims == 0  || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1))
		{
  
			//  time interval 
		    
			char *sp = (char *)info->vardata[vnum]->data;

			// skip strings up to start record
			int s_ptr = atRec*iso_str_len;

			QdTimeInterval_var ti = new QdTimeInterval(); 

			ti->setFromISOstring( &(sp[s_ptr])); 

			// This is a time sequence object so broaden to the QdObject
			dobj = ti;
		}
		else{ 
			// qsas does not handle time range arrays
			printf("QdiVarToObjSingle: cannot handle time array\n");
			fflush(stdout);
			return  dobj;
		}
	}

	else if(info->vardata[vnum]->data_type == CDF_CHAR ||
			info->vardata[vnum]->data_type == CDF_UCHAR) {
    
		int n_bytes_elem = (int) info->vardata[vnum]->num_elems;
   
		const char* p = (const char*)info->vardata[vnum]->data;

		if(	info->vardata[vnum]->num_dims == 0  || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1))
		{
			// string 
			p += atRec*n_bytes_elem;
			
			QdString_var so = new QdString();
							
			int actualLen=1;
			for (int kk=1; kk<n_bytes_elem; kk++) {
				if( *(p+kk) != '\0') actualLen++;
			}
			// strip trailing spaces 
			while (*(p+actualLen-1) == ' ')  actualLen--;
				
			*so = string(p, actualLen);

			dobj = so;

		}
		else
		{
			// string matrix 
			
			// find matrix dims and n elements
			vector<int> dims;
			int n_elems=1; 
			for(int i=0; i<info->vardata[vnum]->num_dims; i++)
			{
				int sz=info->vardata[vnum]->dim_sizes[i];
				dims.push_back(sz);
				n_elems*=sz;
			}				
			
			p += atRec*n_bytes_elem*n_elems;

			
			// fill matrix  
			QdSMatrix_var sm = new QdSMatrix(Dim(dims));
			
			vector <string> str_elements(n_elems);
			for(int i=0; i<n_elems; i++)
			{
				int actualLen=1;
				for (int kk=1; kk<n_bytes_elem; kk++) {
					if( *(p+i*n_bytes_elem+kk) != '\0') actualLen++;
				}
				// strip trailing spaces 
				while (*(p+i*n_bytes_elem+actualLen-1) == ' ')  actualLen--;
				
				str_elements[i] = string(p+i*n_bytes_elem, actualLen);
			}
			sm->assign_vector(str_elements);
			
			dobj = sm;
			
		} // end if string array vs string	
	}
	else
	{
		// cast all other data types to double 

		// create new matrix sequence object with dimensionality:  info->vardata[vnum]->num_dims
		//  based on array of sizes info->vardata[vnum]->dim_sizes
     
		if(	info->vardata[vnum]->num_dims == 0  || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1))
		{
			// scalar sequence
			
			QdRScalar_var so = new QdRScalar();
        
			int next = atRec;
    
			void *p = info->vardata[vnum]->data;
    

			// for numeric types use element by element static casting
			//   and let = (double) do the conversion to double

			switch( info->vardata[vnum]->data_type )
			{
				case CDF_BYTE:
				case CDF_INT1:
				{
					*so = *( (char *)p+next);
					next++;
					break;
				}
				case CDF_UINT1:
				{
					*so = *( (unsigned char *)p+next);
					next++;
					break;
				}
				case CDF_INT2:
				{
					*so = *( (short *)p+next);
					next++;
					break;
				}
				case CDF_UINT2:
				{
					*so = *( (unsigned short *)p+next);
					next++;
					break;
				}
				case CDF_INT4:
				{
					*so = *( (int *)p+next);
					next++;
					break;
				}
				case CDF_UINT4:
				{
					*so = *( (unsigned int *)p+next);
					next++;
					break;
				}
				case CDF_REAL4:
				case CDF_FLOAT:
				{
					*so = *( (float *)p+next);
					next++;
					break;
				}
				case CDF_REAL8:
				case CDF_DOUBLE:
				{
					*so = *((double *)p+next);
					next++;
					break;
				}
				default:
					printf("QdiVarToObjSingle: unknown scalar data type\n");
					fflush(stdout);
					break;
				}

				dobj = so;
		}
		else
		{
			// matrix 
			vector<int> dims;
		    int n_length = 1;
			for(int j=0; j<info->vardata[vnum]->num_dims; j++){
				n_length *= (int) info->vardata[vnum]->dim_sizes[j];
				dims.push_back( info->vardata[vnum]->dim_sizes[j] );
			}   
    
			QdRMatrix_var mo = new QdRMatrix(Dim(dims));
        
			int next = atRec * n_length;
    
			void *p = info->vardata[vnum]->data;
    
			vector <double> d_elements(n_length);

			// for numeric types use element by element static casting
			//   and let = (double) do the conversion to double

			switch( info->vardata[vnum]->data_type )
			{
				case CDF_BYTE:
				case CDF_INT1:
				{
					for(int i=0; i<n_length; i++){
						d_elements[i] = *( (char *)p+next);
						next++;
					}
					break;
				}
				case CDF_UINT1:
				{
					for(int i=0; i<n_length; i++){
						d_elements[i] = *( (unsigned char *)p+next);
						next++;
					}
					break;
				}
				case CDF_INT2:
				{
					for(int i=0; i<n_length; i++){
						d_elements[i] = *( (short *)p+next);
						next++;
					}
					break;
				}
				case CDF_UINT2:
				{
					for(int i=0; i<n_length; i++){
						d_elements[i] = *( (unsigned short *)p+next);
						next++;
					}
					break;
				}
				case CDF_INT4:
				{
					for(int i=0; i<n_length; i++){
						d_elements[i] = *( (int *)p+next);
						next++;
					}
					break;
				}
				case CDF_UINT4:
				{
					for(int i=0; i<n_length; i++){
						d_elements[i] = *( (unsigned int *)p+next);
						next++;
					}
					break;
				}
				case CDF_REAL4:
				case CDF_FLOAT:
				{
					for(int i=0; i<n_length; i++){
						d_elements[i] = *( (float *)p+next);
						next++;
					}
					break;
				}
				case CDF_REAL8:
				case CDF_DOUBLE:
				{
					for(int i=0; i<n_length; i++){
						d_elements[i] = *((double *)p+next);
						next++;
					}
					break;
				}
				default:
					printf("QdiVarToObjSingle: unknown data type\n");
					fflush(stdout);
					break;
			}
					
			mo->assign_vector(d_elements);

			dobj = mo;

		} // end if on scalar vs matrix
		
	} // end else on data type

	return dobj;

}

//*************************************************************

QdObject_var  DirectImportWindow::QdiVarToObj ( QiCDFContents* info, int vnum){

	int  i,j,k;
	int seq_size;
	QdObject_var  dobj;
	
	if(info->n_recs <= 0) return dobj;

	if (vnum > info->n_vars - 1 || vnum < 0) return dobj;
	if(info->vardata[vnum] == 0) return dobj;

	bool intOK;
	int fromRec = fromSlider->recField->text().toInt(&intOK);
	if(!intOK) fromRec = 0;
  
	int toRec = toSlider->recField->text().toInt(&intOK);
	if(!intOK) toRec = toSlider->maxRecord;

	if(info->vardata[vnum]->rec_vary == NOVARY){
		toRec = 0;
		fromRec = 0;
	}
	if(fromRec == toRec) return QdiVarToObjSingle(info,  vnum,  toRec);
      
  
	// test for Data Object is TimeTags object

	if( info->vardata[vnum]->data_type == CDF_EPOCH ) {

		if(  info->vardata[vnum]->num_dims == 0 || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1 ))
		{ 
			// Time sequence
 
			QdTimeSeq_var ts = new QdTimeSeq(toRec-fromRec+1);
			double *ep = (double *)info->vardata[vnum]->data;
			for (int i = fromRec; i<= toRec; i++){
				ts[i-fromRec].setFromCDFepoch(ep[i]);
			}
			// This is a time sequence object so broaden to the QdObject
			dobj = ts;
		}
		else if(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 2){
   
			// assume to be time intervals
	     
			double *ep = (double *)info->vardata[vnum]->data;
			QdTimeIntervalSeq_var tis = new QdTimeIntervalSeq(toRec-fromRec+1); 

			for (int i = fromRec; i<= toRec; i++){
				QmjdTime iv_start; 
				iv_start.setFromCDFepoch(ep[2*i]);
				QmjdTime iv_end; 
				iv_end.setFromCDFepoch(ep[2*i+1]);
				tis[i-fromRec].set(iv_start, iv_end );
			}
			// This is a time sequence object so broaden to the QdObject
			dobj = tis;
		}
		else{
			// qsas does not handle time arrays
			printf("QdiVarToObj: unknown data type\n");
			fflush(stdout);
			return  dobj;
		} // end if on dimensions

	}
	else if( info->vardata[vnum]->data_type == CDF_EPOCH16 ) {

		if(  info->vardata[vnum]->num_dims == 0 || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1 ))
		{ 
			// Time sequence
 
			QdTimeSeq_var ts = new QdTimeSeq(toRec-fromRec+1);
			double *ep = (double *)info->vardata[vnum]->data;
			for (int i = fromRec; i<= toRec; i++){
				ts[i-fromRec].setFromCDFepoch16(&(ep[2*i]));
			}
			// This is a time sequence object so broaden to the QdObject
			dobj = ts;
		}
		else if(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 2){
   
			// assume to be time intervals
	     
			double *ep = (double *)info->vardata[vnum]->data;
			QdTimeIntervalSeq_var tis = new QdTimeIntervalSeq(toRec-fromRec+1); 

			for (int i = fromRec; i<= toRec; i++){
				QmjdTime iv_start; 
				iv_start.setFromCDFepoch16(&(ep[4*i]));
				QmjdTime iv_end; 
				iv_end.setFromCDFepoch16(&(ep[4*i+2]));

				tis[i-fromRec].set(iv_start, iv_end );
			}
			// This is a time sequence object so broaden to the QdObject
			dobj = tis;
		}
		else{
			// qsas does not handle time arrays
			printf("QdiVarToObj: unknown data type\n");
			fflush(stdout);
			return  dobj;
		} // end if on dimensions

	}
	else if( info->vardata[vnum]->data_type == ISO_TIME ) {
		int iso_str_len = info->vardata[vnum]->num_elems;

		if(	info->vardata[vnum]->num_dims == 0  || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1))
		{
  
			QdTimeSeq_var ts = new QdTimeSeq(toRec - fromRec + 1);
			char *sp = (char *)info->vardata[vnum]->data;

			double epochValue;
			// skip strings up to start record
			int s_ptr = fromRec*iso_str_len;
   
			for (int i = fromRec; i <= toRec; i++){
				ts[i-fromRec].setFromISOstring( &(sp[s_ptr]));
				s_ptr += iso_str_len;
			}
   
			// This is a time sequence object so broaden to the QdObject
			dobj = ts;
		}
		else if(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 2)
		{
			// assume to be a time interval sequence
		    
			char *sp = (char *)info->vardata[vnum]->data;

			// skip strings up to start record
			int      s_ptr = 2*fromRec*iso_str_len;

			QdTimeIntervalSeq_var tis = new QdTimeIntervalSeq(toRec - fromRec + 1); 
		
			for (int i = fromRec; i <= toRec; i++){

				char sp_0[iso_str_len+1];
				char sp_1[iso_str_len+1];
				strncpy(sp_0, &(sp[s_ptr]), iso_str_len);
				s_ptr += iso_str_len;
				strncpy(sp_1, &(sp[s_ptr]), iso_str_len);
				s_ptr += iso_str_len;
	 
				int k = iso_str_len-1;
				while(sp_0[k] == ' ' && k>0) {sp_0[k] = '\0'; k--;} // remove white space
				QmjdTime iv_start(sp_0);	 
				k = iso_str_len-1;
				while(sp_1[k] == ' ' && k>0) {sp_1[k] = '\0'; k--;} // remove white space
				QmjdTime iv_end(sp_1);

				tis[i].set(iv_start, iv_end); 
			}	 
			// This is a time sequence object so broaden to the QdObject
			dobj = tis;
		}
		else{ 
			// qsas does not handle time arrays
			printf("QdiVarToObj: unknown data type\n");
			fflush(stdout);
			return  dobj;
		}
	}
	else if( info->vardata[vnum]->data_type == ISO_TIME_RANGE ) {
		int iso_str_len = info->vardata[vnum]->num_elems;

		if(	info->vardata[vnum]->num_dims == 0  || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1))
		{
  
			// time interval sequence
		    
			char *sp = (char *)info->vardata[vnum]->data;
			// skip strings up to start record
			int   s_ptr = fromRec*iso_str_len;

			QdTimeIntervalSeq_var tis = new QdTimeIntervalSeq(toRec - fromRec + 1); 
		
			for (int i = fromRec; i <= toRec; i++){

				tis[i-fromRec].setFromISOstring( &(sp[s_ptr]));
				s_ptr += iso_str_len;
			}	 
			// This is a time sequence object so broaden to the QdObject
			dobj = tis;
		}
		else{ 
			// qsas does not handle time range arrays
			printf("QdiVarToObj: unknown data type\n");
			fflush(stdout);
			return  dobj;
		}
	}

	else if(info->vardata[vnum]->data_type == CDF_CHAR ||
			info->vardata[vnum]->data_type == CDF_UCHAR) {
    
		// if an array set up matrix with dimensions
		int n_bytes_elem = (int) info->vardata[vnum]->num_elems;
   
		const char* p = (const char*)info->vardata[vnum]->data;
    
		if(	info->vardata[vnum]->num_dims == 0  || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1))
		{
			// string sequence
			
			QdStringSeq_var ss = new QdStringSeq( toRec - fromRec + 1);
			
			for( int r=fromRec; r <= toRec; r++ )
			{
				const char* next = p + r*n_bytes_elem;
				
				int actualLen=1;
				for (int kk=1; kk<n_bytes_elem; kk++) {
					if( *(next+kk) != '\0') actualLen++;
				}
				// strip trailing spaces 
				while (*(next+actualLen-1) == ' ')  actualLen--;
				
				ss[i] = string(next, actualLen);
			}
			dobj = ss;

		}
		else
		{
			// string matrix sequence
			
			// find matrix dims and n elements
			vector<int> dims;
			int n_elems=1; 
			for(int i=0; i<info->vardata[vnum]->num_dims; i++)
			{
				int sz=info->vardata[vnum]->dim_sizes[i];
				dims.push_back(sz);
				n_elems*=sz;
			}
			
			// fill matrix sequence 
			QdSMatrixSeq_var sms = new QdSMatrixSeq( toRec - fromRec + 1, Dim(dims));
			
			for( int r=fromRec; r <= toRec; r++ )
			{
				vector <string> str_elements(n_elems);
				const char* next = p + r*n_bytes_elem*n_elems;
				for(int i=0; i<n_elems; i++)
				{
					int actualLen=1;
					for (int kk=1; kk<n_bytes_elem; kk++) {
						if( *(next+i*n_bytes_elem+kk) != '\0') actualLen++;
					}
					// strip trailing spaces 
					while (*(next+i*n_bytes_elem+actualLen-1) == ' ')  actualLen--;
				
					str_elements[i] = string(next+i*n_bytes_elem, actualLen);
				}
				sms[r].assign_vector(str_elements);
			}
			dobj = sms;
		} // end if on string vs S matrix	
	}
	else
	{
		// cast all other data types to double 

		// create new matrix sequence object with dimensionality:  info->vardata[vnum]->num_dims
		//  based on array of sizes info->vardata[vnum]->dim_sizes
     
		if(	info->vardata[vnum]->num_dims == 0  || 
			(info->vardata[vnum]->num_dims == 1 && info->vardata[vnum]->dim_sizes[0] == 1))
		{
			// scalar sequence
			
			QdRScalarSeq_var ss = new QdRScalarSeq(toRec - fromRec + 1);
        
			int next = fromRec;
    
			void *p = info->vardata[vnum]->data;
    
			for( int r = fromRec; r <= toRec; r++ )
			{
				// for numeric types use element by element static casting
				//   and let = (double) do the conversion to double

				switch( info->vardata[vnum]->data_type )
				{
					case CDF_BYTE:
					case CDF_INT1:
					{
						(*ss)[r] = *( (char *)p+next);
						next++;
						break;
					}
					case CDF_UINT1:
					{
						(*ss)[r] = *( (unsigned char *)p+next);
						next++;
						break;
					}
					case CDF_INT2:
					{
						(*ss)[r] = *( (short *)p+next);
						next++;
						break;
					}
					case CDF_UINT2:
					{
						(*ss)[r] = *( (unsigned short *)p+next);
						next++;
						break;
					}
					case CDF_INT4:
					{
						(*ss)[r] = *( (int *)p+next);
						next++;
						break;
					}
					case CDF_UINT4:
					{
						(*ss)[r] = *( (unsigned int *)p+next);
						next++;
						break;
					}
					case CDF_REAL4:
					case CDF_FLOAT:
					{
						(*ss)[r] = *( (float *)p+next);
						next++;
						break;
					}
					case CDF_REAL8:
					case CDF_DOUBLE:
					{
						(*ss)[r] = *((double *)p+next);
						next++;
						break;
					}
					default:
						printf("QdiVarToObj: unknown scalar data type\n");
						fflush(stdout);
						break;
				}
					
			}  // loop over records

			dobj = ss;
		}
		else
		{
			// matrix sequence;
			vector<int> dims;
		    int n_length = 1;
			for( j=0; j<info->vardata[vnum]->num_dims; j++){
				n_length *= (int) info->vardata[vnum]->dim_sizes[j];
				dims.push_back( info->vardata[vnum]->dim_sizes[j] );
			}   
    
			QdRMatrixSeq_var ms = new QdRMatrixSeq(  toRec - fromRec + 1, Dim(dims));
        
			int next = fromRec * n_length;
    
			void *p = info->vardata[vnum]->data;
    
			for( int r = fromRec; r <= toRec; r++ )
			{
				vector <double> d_elements(n_length);

				// for numeric types use element by element static casting
				//   and let = (double) do the conversion to double

				switch( info->vardata[vnum]->data_type )
				{
					case CDF_BYTE:
					case CDF_INT1:
					{
						for(int i=0; i<n_length; i++){
							d_elements[i] = *( (char *)p+next);
							next++;
						}
						break;
					}
					case CDF_UINT1:
					{
						for(int i=0; i<n_length; i++){
							d_elements[i] = *( (unsigned char *)p+next);
							next++;
						}
						break;
					}
					case CDF_INT2:
					{
						for(int i=0; i<n_length; i++){
							d_elements[i] = *( (short *)p+next);
							next++;
						}
						break;
					}
					case CDF_UINT2:
					{
						for(int i=0; i<n_length; i++){
							d_elements[i] = *( (unsigned short *)p+next);
							next++;
						}
						break;
					}
					case CDF_INT4:
					{
						for(int i=0; i<n_length; i++){
							d_elements[i] = *( (int *)p+next);
							next++;
						}
						break;
					}
					case CDF_UINT4:
					{
						for(int i=0; i<n_length; i++){
							d_elements[i] = *( (unsigned int *)p+next);
							next++;
						}
						break;
					}
					case CDF_REAL4:
					case CDF_FLOAT:
					{
						for(int i=0; i<n_length; i++){
							d_elements[i] = *( (float *)p+next);
							next++;
						}
						break;
					}
					case CDF_REAL8:
					case CDF_DOUBLE:
					{
						for(int i=0; i<n_length; i++){
							d_elements[i] = *((double *)p+next);
							next++;
						}
						break;
					}
					default:
						printf("QdiVarToObj: unknown data type\n");
						fflush(stdout);
						break;
				}
					
				ms[r].assign_vector(d_elements);

			}  // loop over records

			dobj = ms;

		} // end if on scalar vs matrix
		
	} // end else on data type


	return dobj;
 
}


// Utility call for qwui
//----------------------

 void QdiLaunchWindow(QString file, QWidget *parent)
{
  DirectImportWindow *QDI_widget = new DirectImportWindow(parent, "import");
  QSize qsasGUISize = qsasWindow->frameSize();
  QDI_widget->move(qsasGUISize.width(),20);  

  QDI_widget->show();
  
  if ( !file.isNull() ){  // open file and get data
 
     QDI_widget->setWindowTitle(file);

     // open file
     QDI_widget->getFileContents(file.toLatin1().data());
     

  }
  else QDI_widget->msgBar->showMessage("no file selected?");

  return;
}

// interface functions
//--------------------------

long QdiGetDep0Var(QiCDFContents *info, int vnum){

 char * depend;
 long i, ii;
 long d0_var;
 
 // time variables don't take time lines...

 if(vnum == info->d0_var_num) return -2; // don't attach tt to tt
 
 // if NRV doesn't take time line
 if( info->vardata[vnum]->rec_vary == NOVARY ) return -2;

 // set default in case no Depend_0 found
 
 if (info->d0_var_num >=0) d0_var = -1;  // tt exist for file
 
 // look for Depend_0
 
 for(i=0; i<info->vardata[vnum]->num_v_attrs; i++){
	if(strcmp(((info->vardata[vnum]->attribute)+i)->name, "DEPEND_0") == 0){
		/* find var pointed to */
		depend = (char *)((info->vardata[vnum]->attribute)+i)->data;
  
		// find this variable
		for (ii=0; ii<info->n_vars; ii++){
			if(info->vardata[ii] == 0) continue;
			if( strncmp(depend, info->vardata[ii]->name,  strlen(info->vardata[ii]->name)) == 0){
				d0_var = ii;
			}       
		}  // end for on vars  
	} 
 } // end for on attrs
 
 return d0_var;   /*  may be -1 or -2 flag */
 
}

//*************************************************************

QdObject_var  QdiGattrToObj ( QiCDFContents* info, int Anum){

	int seq_size;
	QdObject_var  dobj;
	long n_length;

	if (Anum > info->num_g_attrs - 1 || Anum < 0) return dobj;	

	seq_size = (int)(info->g_attr[Anum]->num_entries );
	if(seq_size <= 0) return dobj;

	if(info->g_attr[Anum]->entry[0].num_elems < 1) return dobj;
    
	if(seq_size == 1) return  QdiGattrToObjSingle (info, Anum);
	
	// Sequence of objects

	if( info->g_attr[Anum]->entry[0].data_type == CDF_CHAR ||
		info->g_attr[Anum]->entry[0].data_type == CDF_UCHAR) {
		
		QdStringSeq_var ss = new QdStringSeq(seq_size);
		
		for(int r=0; r<seq_size; r++)
		{
			const char *ptr = (const char *) info->g_attr[Anum]->entry[r].data;
			int n_elems =  (int)info->g_attr[Anum]->entry[r].num_elems;

			ss[r] = string(ptr, n_elems);
		}   
		dobj = ss;
	}
	else if(info->g_attr[Anum]->entry[0].data_type == ISO_TIME) {
      
		QdTimeSeq_var ts = new QdTimeSeq(seq_size);
		for (int r = 0; r<seq_size; r++){
			char *ptr = (char *) info->g_attr[Anum]->entry[r].data;
			ts[r].setFromISOstring( ptr);
		}
   
		// This is a time sequence object so broaden to the QdObject
		dobj = ts;
	}
	else if(info->g_attr[Anum]->entry[0].data_type == ISO_TIME_RANGE) {
      
		QdTimeIntervalSeq_var tis = new QdTimeIntervalSeq(seq_size);
		for (int r = 0; r<seq_size; r++){
			char *ptr = (char *) info->g_attr[Anum]->entry[r].data;
			tis[r].setFromISOstring( ptr);
		}
   
		// This is a time sequence object so broaden to the QdObject
		dobj = tis;
	}
	else if(info->g_attr[Anum]->entry[0].data_type == CDF_EPOCH) {
        if(info->g_attr[Anum]->entry[0].num_elems == 1){
			QdTimeSeq_var ts = new QdTimeSeq(seq_size);
			for (int r = 0; r<seq_size; r++){
				double *ptr = (double *) info->g_attr[Anum]->entry[r].data;
				ts[r].setFromCDFepoch( *ptr);
			}
   		
			// This is a time sequence object so broaden to the QdObject
			dobj = ts;
		}
		else if(info->g_attr[Anum]->entry[0].num_elems == 2){
			QdTimeIntervalSeq_var tis = new QdTimeIntervalSeq(seq_size);
			for (int r = 0; r<seq_size; r++){
				double *ptr = (double *) info->g_attr[Anum]->entry[r].data;
				QdTime t1, t2; 
				t1.setFromCDFepoch(ptr[0]);
				t2.setFromCDFepoch(ptr[1]);
				tis[r].set( t1, t2);
			}
   		
			// This is a time sequence object so broaden to the QdObject
			dobj = tis;
		
		}
	}
	else if(info->g_attr[Anum]->entry[0].data_type == CDF_EPOCH16) {
        if(info->g_attr[Anum]->entry[0].num_elems == 1){
			QdTimeSeq_var ts = new QdTimeSeq(seq_size);
			for (int r = 0; r<seq_size; r++){
				double *ptr = (double *) info->g_attr[Anum]->entry[r].data;
				ts[r].setFromCDFepoch16( ptr);
			}
   		
			// This is a time sequence object so broaden to the QdObject
			dobj = ts;
		}
		else if(info->g_attr[Anum]->entry[0].num_elems == 2){
			QdTimeIntervalSeq_var tis = new QdTimeIntervalSeq(seq_size);
			for (int r = 0; r<seq_size; r++){
				double *ptr = (double *) info->g_attr[Anum]->entry[r].data;
				QdTime t1, t2; 
				t1.setFromCDFepoch16(ptr);
				t2.setFromCDFepoch16(&(ptr[2]));
				tis[r].set( t1, t2);
			}
   		
			// This is a time sequence object so broaden to the QdObject
			dobj = tis;
		
		}
	}
	else{
		// cast all other data types to double 

		if(info->g_attr[Anum]->entry[0].num_elems == 1)
		{			
			// scalar sequence
			
			QdRScalarSeq_var ss = new QdRScalarSeq(seq_size);
			
			for( int r=0; r<seq_size; r++ )
			{
				void *p =  info->g_attr[Anum]->entry[r].data;

				switch( info->g_attr[Anum]->entry[r].data_type )
				{
					case CDF_BYTE:
					case CDF_INT1:
					{
						(*ss)[r] = *( (char *)p);
						break;
					}
					case CDF_UINT1:
					{
						(*ss)[r] = *( (unsigned char *)p);
						break;
					}
					case CDF_INT2:
					{
						(*ss)[r] = *( (short *)p);
						break;
					}
					case CDF_UINT2:
					{
						(*ss)[r] = *( (unsigned short *)p);
						break;
					}
					case CDF_INT4:
					{
						(*ss)[r] = *( (int *)p);
						break;
					}
					case CDF_UINT4:
					{
						(*ss)[r] = *( (unsigned int *)p);
						break;
					}
					case CDF_REAL4:
					case CDF_FLOAT:
					{
						(*ss)[r] = *( (float *)p);
						break;
					}
					case CDF_REAL8:
					case CDF_DOUBLE:
					{
						(*ss)[r] = *( (double *)p);
						break;
					}
				}
			}  // loop over records
			
			dobj = ss;
		}
		else
		{
			// if an array set up matrix with dimensions
			
			QdRMatrixSeq_var ms = new QdRMatrixSeq(seq_size, Dim(info->g_attr[Anum]->entry[0].num_elems));
			
			for( int r=0; r<seq_size; r++ )
			{
				n_length =  info->g_attr[Anum]->entry[r].num_elems;
				void *p =  info->g_attr[Anum]->entry[r].data;
				vector <double> d_elements(n_length);

				switch( info->g_attr[Anum]->entry[r].data_type )
				{
					case CDF_BYTE:
					case CDF_INT1:
					{
						for(int i=0; i<n_length; i++) d_elements[i] = *( (char *)p+i);
						break;
					}
					case CDF_UINT1:
					{
						for(int i=0; i<n_length; i++) d_elements[i] = *( (unsigned char *)p+i);
						break;
					}
					case CDF_INT2:
					{
						for(int i=0; i<n_length; i++) d_elements[i] = *( (short *)p+i);
						break;
					}
					case CDF_UINT2:
					{
						for(int i=0; i<n_length; i++) d_elements[i] = *( (unsigned short *)p+i);
						break;
					}
					case CDF_INT4:
					{
						for(int i=0; i<n_length; i++) d_elements[i] = *( (int *)p+i);
						break;
					}
					case CDF_UINT4:
					{
						for(int i=0; i<n_length; i++) d_elements[i] = *( (unsigned int *)p+i);
						break;
					}
					case CDF_REAL4:
					case CDF_FLOAT:
					{
						for(int i=0; i<n_length; i++) d_elements[i] = *( (float *)p+i);
						break;
					}
					case CDF_REAL8:
					case CDF_DOUBLE:
					{
						for(int i=0; i<n_length; i++) d_elements[i] = *( (double *)p+i);
						break;
					}
					
					ms[r].assign_vector(d_elements);
				}
			}  // loop over records
			
			dobj = ms;
			
		} // end if on num elements
		
	} // end else on data type
    
	return dobj;
 
}
//*************************************************************

QdObject_var  QdiGattrToObjSingle( QiCDFContents* info, int Anum){

	QdObject_var  dobj;
	long n_length;

	// Single object

	if( info->g_attr[Anum]->entry[0].data_type == CDF_CHAR ||
		info->g_attr[Anum]->entry[0].data_type == CDF_UCHAR) {
		
		QdString_var so = new QdString();

		const char *ptr = (const char *) info->g_attr[Anum]->entry[0].data;
		int n_elems =  (int)info->g_attr[Anum]->entry[0].num_elems;

		*so = string(ptr, n_elems);
		
		dobj = so;
	}
	else if(info->g_attr[Anum]->entry[0].data_type == CDF_EPOCH) {
        if(info->g_attr[Anum]->entry[0].num_elems == 1){
			QdTime_var to = new QdTime();
			double *ptr = (double *) info->g_attr[Anum]->entry[0].data;
			to->setFromCDFepoch( *ptr);
   
			// This is a time sequence object so broaden to the QdObject
			dobj = to;
		}
		else if(info->g_attr[Anum]->entry[0].num_elems == 2){
				
			double *ptr = (double *) info->g_attr[Anum]->entry[0].data;
			QdTime t1, t2; 
			t1.setFromCDFepoch(ptr[0]);
			t2.setFromCDFepoch(ptr[1]);
			QdTimeInterval_var ti = new QdTimeInterval(t1, t2);
			
   		
			// This is a time sequence object so broaden to the QdObject
			dobj = ti;
		
		}	
	}
	else if(info->g_attr[Anum]->entry[0].data_type == CDF_EPOCH16) {
        if(info->g_attr[Anum]->entry[0].num_elems == 1){
			QdTime_var to = new QdTime();
			double *ptr = (double *) info->g_attr[Anum]->entry[0].data;
			to->setFromCDFepoch16( ptr);
			// This is a time sequence object so broaden to the QdObject
			dobj = to;
		}
		else if(info->g_attr[Anum]->entry[0].num_elems == 2){
				
			double *ptr = (double *) info->g_attr[Anum]->entry[0].data;
			QdTime t1, t2; 
			t1.setFromCDFepoch16(ptr);
			t2.setFromCDFepoch16(&(ptr[2]));
			QdTimeInterval_var ti = new QdTimeInterval(t1, t2);
			
   		
			// This is a time sequence object so broaden to the QdObject
			dobj = ti;
		
		}
	}
	else if(info->g_attr[Anum]->entry[0].data_type == ISO_TIME) {
      
		QdTime_var to = new QdTime();
		char *ptr = (char *) info->g_attr[Anum]->entry[0].data;
		to->setFromISOstring( ptr);
   
		// This is a time sequence object so broaden to the QdObject
		dobj = to;
	}
	else if(info->g_attr[Anum]->entry[0].data_type == ISO_TIME_RANGE) {
      
		QdTimeInterval_var ti = new QdTimeInterval();
		char *ptr = (char *) info->g_attr[Anum]->entry[0].data;
		ti->setFromISOstring( ptr);
   
		// This is a time sequence object so broaden to the QdObject
		dobj = ti;
	}
	else
	{
		// cast all other data types to double 

		if(info->g_attr[Anum]->entry[0].num_elems == 1)
		{
			QdRScalar_var so = new QdRScalar();

			void *p =  info->g_attr[Anum]->entry[0].data;

			switch( info->g_attr[Anum]->entry[0].data_type )
			{
				case CDF_BYTE:
				case CDF_INT1:
				{
					*so = *( (char *)p);
					break;
				}
				case CDF_UINT1:
				{
					*so = *( (unsigned char *)p);
					break;
				}
				case CDF_INT2:
				{
					*so = *( (short *)p);
					break;
				}
				case CDF_UINT2:
				{
					*so = *( (unsigned short *)p);
					break;
				}
				case CDF_INT4:
				{
					*so = *( (int *)p);
					break;
				}
				case CDF_UINT4:
				{
					*so = *( (unsigned int *)p);
					break;
				}
				case CDF_REAL4:
				case CDF_FLOAT:
				{
					*so = *( (float *)p);
					break;
				}
				case CDF_REAL8:
				case CDF_DOUBLE:
				{
					*so = *( (double *)p);
					break;
				}
			}
			
			dobj = so;
		}
		else
		{
			// if an array set up matrix with dimensions
			
			QdRMatrix_var mo = new QdRMatrix();
			
			n_length =  info->g_attr[Anum]->entry[0].num_elems;
			vector <double> d_elements(n_length);
			void *p =  info->g_attr[Anum]->entry[0].data;

			switch( info->g_attr[Anum]->entry[0].data_type )
			{
				case CDF_BYTE:
				case CDF_INT1:
				{
					for(int i=0; i<n_length; i++) d_elements[i] = *( (char *)p+i);
					break;
				}
				case CDF_UINT1:
				{
					for(int i=0; i<n_length; i++) d_elements[i] = *( (unsigned char *)p+i);
					break;
				}
				case CDF_INT2:
				{
					for(int i=0; i<n_length; i++) d_elements[i] = *( (short *)p+i);
					break;
				}
				case CDF_UINT2:
				{
					for(int i=0; i<n_length; i++) d_elements[i] = *( (unsigned short *)p+i);
					break;
				}
				case CDF_INT4:
				{
					for(int i=0; i<n_length; i++) d_elements[i] = *( (int *)p+i);
					break;
				}
				case CDF_UINT4:
				{
					for(int i=0; i<n_length; i++) d_elements[i] = *( (unsigned int *)p+i);
					break;
				}
				case CDF_REAL4:
				case CDF_FLOAT:
				{
					for(int i=0; i<n_length; i++) d_elements[i] = *( (float *)p+i);
					break;
				}
				case CDF_REAL8:
				case CDF_DOUBLE:
				{
					for(int i=0; i<n_length; i++) d_elements[i] = *( (double *)p+i);
					break;
				}
					
				mo->assign_vector(d_elements);
			}
			
			dobj = mo;
			
		} // end if on num elements
		
	} // end else on string vs double
    
	return dobj;
 
}

//*******************************************************************

void   DirectImportWindow::QdiSet_VAttrs(QdObject_var dobj, QiCDFContents * info, int vnum)
{
 char * ptr_name;
 int ptr_num;
 
 QiCDFVariable * var =  info->vardata[vnum];

 for(int i=0; i<var->num_v_attrs; i++){

 // get either the attribute or the var pointed to by _ptr type attrs
 
  if( ( ptr_num=Qdi_DistinctVarExists( info, vnum, i) ) >= 0 ) {

      bool readAlready = false;
  	 for (int j=0; j < importedVarNos.size(); j++){
	     if(ptr_num == importedVarNos[j]) {
	        readAlready = true;
		   ptr_num = j;
	     }
	 }

     // we've found a variable pointed to by this attribute
	QdObject_var a_obj;
	if(readAlready){
	   a_obj = importedVars[ptr_num];
	}
	else{

		a_obj = QdiVarToObj(info, ptr_num);

		if( a_obj.is_nil() ){
			continue;
		}
		importedVarNos.push_back(ptr_num);
		importedVars.push_back(a_obj);
	   
		// set dep 0 for xref object (may differ from parent)
		int  d0_var_num = QdiGetDep0Var(QieContents, ptr_num);
		QdObject_var d0_for_xref;
        if( d0_var_num >= 0 ){
			// has this time var already been read in
			bool AlreadyRead = false;
			for(int j=0; j<importedVarNos.size(); j++){
		    	if(d0_var_num == importedVarNos[j]) {
					AlreadyRead = true;
					d0_var_num = j;
			  		break;
				}
			}
			if(AlreadyRead){
				d0_for_xref = importedVars[d0_var_num];
				if( !d0_for_xref.is_nil() ) a_obj->change_xref(DEPEND_0, d0_for_xref);
			}
			else{
				// get dep 0 for object (based on Depend_0, not epoch in file) subset records (fromRec, toRec)
				d0_for_xref =  QdiVarToObj ( QieContents, d0_var_num );
				if( !d0_for_xref.is_nil() ){
					a_obj->change_xref(DEPEND_0, d0_for_xref);
					importedVarNos.push_back(d0_var_num);
					importedVars.push_back(d0_for_xref);
					// set xrefs for time tags
					QdiSet_VAttrs(d0_for_xref, QieContents, d0_var_num);
 				}
			}
		}
		else if(d0_var_num == -1 && a_obj->sequence_size() == dobj->sequence_size()){
			// no Depend_0 so use parent object DEP
			d0_for_xref = get_Depend_0(dobj);
			if( !d0_for_xref.is_nil() ) a_obj->change_xref(DEPEND_0,  d0_for_xref);
		}

        // xref attributes were not set by QdiVarToObj
		QdiSet_VAttrs(a_obj, info, ptr_num);

        QuSetTxtAttr (PTR_VAR_NAME,  (char *) (var->attribute[i]).data, a_obj); 
	}
	
	try{
		dobj->change_xref( QdiAttrName((var->attribute[i]).name) , a_obj);  
	}
	catch(Exception &e){
		QuMsgBar( e.msg().c_str(), (var->attribute[i]).name, true ); 
	}
     

  }
  else if(ptr_num == -1) { // this really is an attribute, so add it as Xref
     try{
        QdiAttrToXref(dobj, var->attribute[i]);
     }
     catch(Exception &e){
       QuMsgBar( e.msg().c_str(), (var->attribute[i]).name, true ); 
     }

  } 
  // else ptr_num == -2 is DEPEND_0 which is skipped
   
 } // end for over attributes by number

  return;
}


//*******************************************************************
     
int Qdi_DistinctVarExists(QiCDFContents *info, int vnum, int attr) {
     
   // if attribute is text, find if it points to a variable
   
   if( (info->vardata[vnum]->attribute[attr]).data_type != CDF_CHAR && 
       (info->vardata[vnum]->attribute[attr]).data_type != CDF_UCHAR)
           return -1;

   int n_elems = (int) strlen( (char *)(info->vardata[vnum]->attribute[attr]).data);
      
   char * ptr = (char *) (info->vardata[vnum]->attribute[attr]).data;
   char entry[n_elems+1];
   strncpy(entry, ptr, n_elems);
   entry[n_elems] = '\0';

   if ( strcmp((info->vardata[vnum]->attribute[attr]).name, "DEPEND_0") == 0)
   return -2;   // epoch done already      

   // find the var referred to 
   for (int k=0; k<info->n_vars; k++){
     if(info->vardata[k] == 0) continue;
     if (strcmp((info->vardata[k])->name, entry) == 0) {
       if( k == vnum ) return -1; // ignore if the same variable
       return k;
     }
   }
   return -1;
}


//*******************************************************************

char * QdiAttrName(char * in_name){
 static char attr_name[ATTR_NAME_LEN+1];

 char * ptr;
 
 // this function is benign if attribute name is not
 //  of ptr type and can safely be called for all attributes
 
 strncpy(attr_name, in_name, ATTR_NAME_LEN); // ensure we return something
 
 // look for ISTP/CSDS names and clean up 
 if (strcmp("LABL_PTR_1", in_name) == 0){
   strcpy(attr_name, "LABLAXIS");
   return &attr_name[0];
 }
 if (strcmp("UNIT_PTR", in_name) == 0){
   strcpy(attr_name, "UNITS");
   return &attr_name[0];
 }
 if (strcmp("FORM_PTR", in_name) == 0){
   strcpy(attr_name, "FORMAT");
   return &attr_name[0];
 }
 
 // look for generic _ptr names, i.e. SI_conversion_ptr
 
 ptr = strstr (attr_name, "_ptr");
 if (ptr == NULL)  ptr = strstr (attr_name, "_PTR");
 if (ptr != NULL) ptr[0] = '\0'; // truncate name
 
 // if no ptr type name found return original name
 
 return &attr_name[0];
}


//*******************************************************************


void QdiAttrToXref(QdObject_var dobj, QiSVarAttribute attribute){

	QdObject_var a_obj;
  
    // switch on data type
    // either string, ISO_TIME or cast to double
    
    if( attribute.data_type == CDF_CHAR || 
       attribute.data_type == CDF_UCHAR){

		if( attribute.num_elems == 1)
		{
			// single string
			      
			char *p = ( char*) attribute.data;
			int n_elems = (int)strlen(p);

			QdString_var so  = new QdString(string(p, n_elems));
			a_obj = so;
		}
		else
		{
			// string matrix
			int n_length = (int) attribute.num_elems;
        
			char *p = ( char*) attribute.data;
			vector <string> str_elements(n_length);
			
			for(int i=0; i<n_length; i++){
         
				int n_elems = (int)strlen(p);
				str_elements[i] = string(p, n_elems);
				p += n_elems + 1;       // jump string and terminator       
	 
			}
			QdSMatrix_var sm = new QdSMatrix(Dim(n_length), str_elements);
			a_obj = sm;
		}
    }
    else if( attribute.data_type == ISO_TIME) {
      
		char *sp = (char *)attribute.data;
		QdTime_var tv = new QdTime(sp);
  
		a_obj = tv;

	}
    else if( attribute.data_type == ISO_TIME_RANGE) {
      
		char *sp = (char *)attribute.data;
		QdTimeInterval_var tiv = new QdTimeInterval();
  		tiv->setFromISOstring(sp);
		
		a_obj = tiv;

	}
    else if( attribute.data_type == CDF_EPOCH ){
      
		double *dp = (double *)attribute.data;
		QdTime_var tv = new QdTime();
		tv->setFromCDFepoch(*dp);
 
		a_obj = tv;

	}
    else if( attribute.data_type == CDF_EPOCH16 ){
      
		double *dp = (double *)attribute.data;
		QdTime_var tv = new QdTime();
		tv->setFromCDFepoch16(dp);
		a_obj = tv;

	}
	else{
		// cast all other data types to double matrix
		
		if(attribute.num_elems == 1)
		{
			// scalar
			void *p = attribute.data;

			switch( attribute.data_type )
			{
				case CDF_BYTE:
				case CDF_INT1:
				{
					QdRScalar_var so = new QdRScalar(*( static_cast<char*>(p) ) );
					a_obj = so;
					break;
				}
				case CDF_UINT1:
				{
					QdRScalar_var so = new QdRScalar(*( static_cast<unsigned char*>(p) ) );
					a_obj = so;
					break;
				}
				case CDF_INT2:
				{
					QdRScalar_var so = new QdRScalar( *( static_cast<short*>(p) ) );
					a_obj = so;
					break;
				}
				case CDF_UINT2:
				{
					QdRScalar_var so = new QdRScalar( *( static_cast<unsigned short*>(p) ) );
					a_obj = so;
					break;
				}
				case CDF_INT4:
				{
					QdRScalar_var so = new QdRScalar( *( static_cast<int*>(p) ) );
					a_obj = so;
					break;
				}
				case CDF_UINT4:
				{
					QdRScalar_var so = new QdRScalar( *( static_cast<unsigned int*>(p) ) );
					a_obj = so;
					break;
				}
				case CDF_REAL4:
				case CDF_FLOAT:
				{
					QdRScalar_var so = new QdRScalar( *( static_cast<float*>(p) ) );
					a_obj = so;
					break;
				}
				case CDF_REAL8:
				case CDF_DOUBLE:
				{
					QdRScalar_var so = new QdRScalar( *( static_cast<double*>(p) ) );
					a_obj = so;
					break;
				}
			}
		}
		else
		{
			// matrix
			int n_length = (int) attribute.num_elems;


			// for numeric types use element by element static casting
			//   and let = (double) do the conversion to double

			void *p = attribute.data;
			vector <double> d_elements(n_length);
			switch( attribute.data_type )
			{
				case CDF_BYTE:
				case CDF_INT1:
				{
					for(int i=0; i<n_length; i++)
						d_elements[i] = *( static_cast<char*>(p) + i );
					break;
				}
				case CDF_UINT1:
				{
					for(int i=0; i<n_length; i++)
						d_elements[i] = *( static_cast<unsigned char*>(p) + i );
					break;
				}
				case CDF_INT2:
				{
					for(int i=0; i<n_length; i++)
						d_elements[i] = *( static_cast<short*>(p) + i );
					break;
				}
				case CDF_UINT2:
				{
					for(int i=0; i<n_length; i++)
						d_elements[i] = *( static_cast<unsigned short*>(p) + i );
					break;
				}
				case CDF_INT4:
				{
					for(int i=0; i<n_length; i++)
						d_elements[i] = *( static_cast<int*>(p) + i );
					break;
				}
				case CDF_UINT4:
				{
					for(int i=0; i<n_length; i++)
						d_elements[i] = *( static_cast<unsigned int*>(p) + i );
					break;
				}
				case CDF_REAL4:
				case CDF_FLOAT:
				{
					for(int i=0; i<n_length; i++)
						d_elements[i] = *( static_cast<float*>(p) + i );
					break;
				}
				case CDF_REAL8:
				case CDF_DOUBLE:
				{
					for(int i=0; i<n_length; i++)
						d_elements[i] = *( static_cast<double*>(p) + i );
					break;
				}
			}
			QdRMatrix_var mo = new QdRMatrix(Dim(n_length), d_elements);
			a_obj = mo;
		}
		
	} // end else on string vs double

	dobj->change_xref(attribute.name, a_obj);

	return;
}


long  QdiFindVarNum(QiCDFContents *QieCont, const char *name ){
  for(int i=0; i< QieCont->n_vars; i++){
    if(QieCont->vardata[i] == NULL) continue;
    if( strcmp(QieCont->vardata[i]->name, name) == 0 ) return i;
  }
  return -1;
}

long  QdiFindGattrNum(QiCDFContents *QieCont, const char *name ){
  for(int i=0; i< QieCont->num_g_attrs; i++){
    if( strcmp(QieCont->g_attr[i]->name, name) == 0 ) return i;
  }
  return -1;
}

// -------------------------------------------------------------
// ---- QDOS 2 Native QIE interface ----------------------------
// -------------------------------------------------------------

// -----------------------------------------------------------------------


DirectAccessWindow::DirectAccessWindow( QWidget *parent, const char *name ) 
  : QMainWindow(parent)
{
	// Status bar
	
	msgBar = statusBar();
	
	// Menu
	
	menuBar()->addSeparator();   
	QMenu * help= menuBar()->addMenu( " Help");
	help->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->addSeparator();
	help->addAction("...Data Import", this, SLOT( launchManDirectImport() ));
	help->addSeparator();
	help->addAction("Module Version", this, SLOT( launchVersion() ));
	
	// Top widget
	
	QWidget * topWidget=new QWidget(this);
	QVBoxLayout * topLayout=new QVBoxLayout;
	
	// List panel
	
	QHBoxLayout * listLayout=new QHBoxLayout;
	
    QVBoxLayout* varLayout=new QVBoxLayout;
    QLabel* varLabel=new QLabel("Variables");
    varLabel->setAlignment(Qt::AlignCenter);
    varLayout->addWidget(varLabel);
    varList=new QListWidget;
    varList-> setSortingEnabled(false);
    connect( varList, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(setSliderForItem(QListWidgetItem*)) );
    connect( varList, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(clearGattrList()) );
    connect( varList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(varDoubleClick(QListWidgetItem*)) );
    activeList = varList;
    varLayout->addWidget(varList);
    listLayout->addLayout(varLayout);

    QVBoxLayout* GattrLayout=new QVBoxLayout;
    QLabel* GattrLabel=new QLabel("Global Attributes");
    GattrLabel->setAlignment(Qt::AlignCenter);
    GattrLayout->addWidget(GattrLabel);
    GattrList=new QListWidget;
    GattrList-> setSortingEnabled(true);
    connect( GattrList, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(clearVarList()) );
    connect( GattrList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(GattrDoubleClick(QListWidgetItem*)) );
    GattrLayout->addWidget(GattrList);
    listLayout->addLayout(GattrLayout);
	
	topLayout->addLayout(listLayout);
	
	// Subset panel
	
	subset_panel=new QGroupBox(topWidget);
	QVBoxLayout * subset_layout=new QVBoxLayout;
	
	useTcheck = new QCheckBox;
	useTcheck->setText("Get time for subsetting ");
	useTcheck->setDown(false);
	connect(useTcheck, SIGNAL(clicked( )), this, SLOT(tRangeToggle( )) );
	subset_layout->addWidget(useTcheck);
	
	fromSlider = new  RecordSlider("From  ", subset_panel, "fromSlider");
	subset_layout->addWidget(fromSlider);
	
	toSlider =   new  RecordSlider("To  ", subset_panel, "toSlider");
	subset_layout->addWidget(toSlider);
	
	subset_panel->setLayout(subset_layout);
	topLayout->addWidget(subset_panel);
	
	// Buttons
	
	QHBoxLayout * button_layout=new QHBoxLayout;
	
	getVarBtn = new QPushButton( "Get Data");
	getVarBtn->setMinimumSize(getVarBtn->sizeHint());
	getVarBtn->setDefault(TRUE);
	connect( getVarBtn, SIGNAL(clicked()), this, SLOT(getData()) );
	button_layout->addWidget(getVarBtn);

	browseVarBtn = new QPushButton( "Browse Data");
	browseVarBtn->setMinimumSize(browseVarBtn->sizeHint());
	connect( browseVarBtn, SIGNAL(clicked()), this, SLOT(browseData()) );
	button_layout->addWidget(browseVarBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	button_layout->addWidget(dismissBtn);
	
	topLayout->addLayout(button_layout);
	
	topWidget->setLayout(topLayout);
	setCentralWidget(topWidget);
	
	QieDataCont = new QieData();
	setAttribute(Qt::WA_DeleteOnClose);
	

}

// ------------------------------------------

void DirectAccessWindow::close(){

 if(QuSlotDebug())
 {
  printf("DirectAccesstWindow::close %ld\n", (long)this);
  fflush(stdout);
 }
 
 QieDataCont->inFile.closeFile();
 
  delete QieDataCont;
  
  QMainWindow::close();
}

// ------------------------------------------

void DirectAccessWindow::tRangeToggle(){ 

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::tRangeToggle %ld\n", (long)this);
  fflush(stdout);
 }
 
 
  QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
	
  // get time variables (note they are cached in earlier calls)
  for(int i=0; i<QieDataCont->QieVars.size(); i++){
	 if(QieDataCont->QieVars[i].datatype == "ISO_TIME")  QieDataCont->readVarData(i, 0, -1) ;
  }
	
  QApplication::restoreOverrideCursor();
  
  if(QieDataCont->d0_var_num != -1){
  	QdTimeSeq_var tt = QdTimeSeq_var::narrow(QieDataCont->QieVars[QieDataCont->d0_var_num].Vobj);
  	QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(QieDataCont->QieVars[QieDataCont->d0_var_num].Vobj);
	if(!tt.is_nil()){
	 	fromSlider->ttags = tt;
     	toSlider->ttags   = tt;
	}
	else if(!ss.is_nil()){
	 	fromSlider->stags = ss;
     	toSlider->stags   = ss;
	}
	
  }

  updateOnSelection();
  useTcheck->setEnabled(false); // prevent being unset
}

// ------------------------------------------

void DirectAccessWindow::launchManual(){ 

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::launchManual %ld\n", (long)this);
  fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("index.html");
}

// ------------------------------------------

void DirectAccessWindow::launchManDirectImport(){ 

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::launchManDirectImport %ld\n", (long)this);
  fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}

// ------------------------------------------

void DirectAccessWindow::launchVersion(){

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::launchVersion %ld\n", (long)this);
  fflush(stdout);
 }
 
  
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n Direct Access:  ";
  text += QString(QIE_VERSION);
  QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" );

}

// ------------------------------------------

int DirectAccessWindow::getFileContents(const char *file){

	int nRecs=0;
	
   // keep global copy of path to add as xref later 
   
	Selected_File_Path = QString(file);

	if(QuSlotDebug())  setDebug(true);
	
	try{
		if( QieDataCont->queryFile(file) ){
			// file pointer is at start of data, we need number of records for sliders
			// re initialise global block size
			nRecs = QieDataCont->readDataRecords();
		
			// Set up display if file read OK
		
			// Set up global list
			GattrList->clear();
			vector <string> sl;
			QieDataCont->GlobalList->list_names_r( sl );
			for(int i=0; i<sl.size(); i++){
				QListWidgetItem* newGattrItem=new QListWidgetItem(sl.at(i).c_str(), GattrList);
				QSize size=newGattrItem->sizeHint();
				size.setWidth(180);
				size.setHeight(20);
				newGattrItem->setSizeHint(size);
			}
		
			// set up variable list

			QLinkedList<QListWidgetItem*> metaList;
			QLinkedList<QListWidgetItem*> dataList;
			varList->clear();

			QListWidgetItem* topItem=new QListWidgetItem("Data", varList);
			setAsSeparator(topItem);

			for(int i=0; i<QieDataCont->QieVars.size(); i++){
				bool isData = true;
				if(QieDataCont->QieVars[i].nRecs == -1){
					if(QieDataCont->QieVars[i].recVary)
					{
					QieDataCont->QieVars[i].nRecs = nRecs; // set here as flat files don't know it
					}
					else
					{
					QieDataCont->QieVars[i].nRecs = 1;
					}
				}
				QdString_var typeXref;
				if(QieDataCont->QieVars[i].Vxref->xref_exists("PARAMETER_TYPE"))
					typeXref = QdString_var::narrow(QieDataCont->QieVars[i].Vxref->get_xref("PARAMETER_TYPE"));
				else if (QieDataCont->QieVars[i].Vxref->xref_exists("VAR_TYPE")) 
					typeXref = QdString_var::narrow(QieDataCont->QieVars[i].Vxref->get_xref("VAR_TYPE"));

				if ( !typeXref.is_nil() ){
					if ( *typeXref == QdString("Support_Data") ) isData = false;
					if ( *typeXref == QdString("support_data") ) isData = false;
					if ( *typeXref == QdString("metadata") ) isData = false;
				}

				QListWidgetItem* newItem=new QListWidgetItem(QieDataCont->QieVars[i].name.c_str(), NULL);
				QSize size=newItem->sizeHint();
				size.setWidth(180);
				size.setHeight(20);
				newItem->setSizeHint(size);
            
				if(isData){
					dataList.push_back(newItem);
				}
				else{
					metaList.push_back(newItem);
				}
			
			}

			foreach(QListWidgetItem* i, dataList)
			{
				varList->addItem(i);
			}

			QListWidgetItem* sepItem=new QListWidgetItem("Support Data", varList);
			setAsSeparator(sepItem);

			foreach(QListWidgetItem* i, metaList)
			{
				varList->addItem(i);
			}
		
		}
		else{
			getVarBtn->setEnabled(false);
			browseVarBtn->setEnabled(false);
		}
	
	}
	catch(Exception &e)
	{
		e.print_msg();
	}
	return nRecs;
}

// ------------------------------------------

void DirectAccessWindow::setSliders(int nRecs){
	// set initial values for sliders
	if( !(getVarBtn->isEnabled()) ) return; // not enabled if file read failed
	 
	if(nRecs == 0) {
		fromSlider->recBar->setRange(0, 0);
		toSlider->recBar->setRange(0, 0);
		getVarBtn->setEnabled(false);
		browseVarBtn->setEnabled(false);
		fromSlider->maxRecord = 0;
		toSlider->maxRecord = 0;
		msgBar->showMessage("WARNING: No records were read, corrupt file?" );
	}
	else{
		fromSlider->recBar->setRange(0, nRecs-1);
		toSlider->recBar->setRange(0, nRecs-1);
		fromSlider->maxRecord = nRecs-1;
		toSlider->maxRecord = nRecs-1;
		
		fromSlider->recBar->setValue(0);
		toSlider->recBar->setValue(toSlider->maxRecord);
	}
	//fromSlider->updateTitle();
	//toSlider->updateTitle();

}

void DirectAccessWindow::updateOnSelection(QieVariable *V){

	fromSlider->updateTitle(0);
	toSlider->updateTitleEnd();
	
}


// ------------------------------------------

void DirectAccessWindow::setSliderForItem(QListWidgetItem *item){
        
    int nRecs;
    int vnum;
    QieVariable *V;
    
    if(item == 0) return;

    // This slot is only used for data files with sparse records,
    // i.e. files where different variables may have different time tags
    
    bool intOK;
    int fromRec = fromSlider->recField->text().toInt(&intOK);
    if (!intOK) fromRec = 0;
    int toRec   = toSlider->recField->text().toInt(&intOK);
    if (!intOK) toRec = toSlider->maxRecord;

    if( activeList == varList) {
    
        if(item != 0){
            vnum = QieDataCont->findVarNumByName(item->text().toLatin1().data() );
            if( vnum == -1 ) return;
        }
        V = &(QieDataCont->QieVars[vnum]);
        nRecs = V->nRecs;
    }
    else return;
             
    if(nRecs == 0) {
        fromSlider->recBar->setRange(0, 0);
        toSlider->recBar->setRange(0, 0);
        getVarBtn->setEnabled(false);
        browseVarBtn->setEnabled(false);
        fromSlider->maxRecord = 0;
        toSlider->maxRecord = 0;
        msgBar->showMessage("WARNING: No records were read, corrupt file?" );
    }
    else if(nRecs == 1) {
        fromSlider->recBar->setRange(0, 0);
        toSlider->recBar->setRange(0, 0);
        getVarBtn->setEnabled(true);
        browseVarBtn->setEnabled(true);
        fromSlider->maxRecord = 0;
        toSlider->maxRecord = 0;
        msgBar->showMessage("WARNING: single entry for variable" );
    }
    else{
        fromSlider->maxRecord = nRecs-1;
        toSlider->maxRecord = nRecs-1;
        getVarBtn->setEnabled(true);
        browseVarBtn->setEnabled(true);
        fromSlider->recBar->setRange(0, nRecs-1);
        toSlider->recBar->setRange(0, nRecs-1);
        msgBar->showMessage(" " );

    }

	QdObject_var D0 = QieDataCont->getD0_obj(vnum);
	QdTimeSeq_var ttv = QdTimeSeq_var::narrow(D0);
	QdRScalarSeq_var ssv = QdRScalarSeq_var::narrow(D0);
	
	if(!ttv.is_nil()){
    	// attach correct tt for variable
    	fromSlider->ttags = ttv;
    	toSlider->ttags   = ttv;
	}
	else if(!ssv.is_nil()){
    	// attach correct tt for variable
    	fromSlider->stags = ssv;
    	toSlider->stags   = ssv;
	}
	
    // reset current slider positions to where they were if safe
    if(toRec == 0) toSlider->recBar->setValue(toSlider->maxRecord);
    else if(toRec > toSlider->maxRecord) toSlider->recBar->setValue(toSlider->maxRecord);
    else toSlider->recBar->setValue(toRec);
        
    if(fromRec > fromSlider->maxRecord) fromSlider->recBar->setValue(0);
    else fromSlider->recBar->setValue(fromRec);

    fromSlider->setEnabled(true);
    toSlider->setEnabled(true);

     // now ttags are set, add time feedback to title
    updateOnSelection(V);
}

// -------------------------------------------------------

void DirectAccessWindow::browseData()
{

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::browseData %ld\n", (long)this);
  fflush(stdout);
 }
 
  QdObject_var  dobj;
  QdObject_var  nil_obj;
  int vnum;
  DataBrowseWindow *QBROWSER_widget;

  // browse either list 
  
  if( activeList == varList) {
    
      QListWidgetItem* item=varList->currentItem();
     if(item == 0){
       msgBar->showMessage( "No variable selected");
       return;
     }   
    vnum = QieDataCont->findVarNumByName(item->text().toLatin1().data() );
  
    if( vnum == -1 ) {
      msgBar->showMessage( "No Data Object selected to browse");
      return;
    }

	// read data in from file 
	
	QieDataCont->readVarData(vnum, 0, toSlider->maxRecord); 

    dobj = QieDataCont->QieVars[vnum].Vobj;
     
	// Attach cleaned xrefs to data object (fixes ptr to var attributes)
	QieDataCont->attachXrefs(vnum, "");
	
    //    open browser
    if( !dobj.is_nil() ){
    
      QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, item->text().toLatin1().data(), dobj, "");
      if(QBROWSER_widget != 0) {
			QBROWSER_widget->disallowSaves = true;
			if(QBROWSER_widget->browser != 0) QBROWSER_widget->editMenu->setEnabled(false);
			QBROWSER_widget->browser->parentsBtn->setEnabled(false);
      }
    }
  }
  else if( activeList == GattrList){
      QListWidgetItem* item=GattrList->currentItem();
     if(item == 0){
       msgBar->showMessage( "No G Attr selected");
       return;
     }
    
     // open G attr
     dobj = QieDataCont->GlobalList->resolve(item->text().toLatin1().data());

     //    open browser
     if( !dobj.is_nil() ){
        QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, item->text().toLatin1().data(), dobj, "");
        if(QBROWSER_widget != 0) {
			QBROWSER_widget->disallowSaves = true;
			if(QBROWSER_widget->browser != 0) QBROWSER_widget->editMenu->setEnabled(false);
			QBROWSER_widget->browser->parentsBtn->setEnabled(false);
		}
     }
  }
  

}

// -------------------------------------------------------------

void DirectAccessWindow::clearVarList(){

    if(QuSlotDebug())
    {
        printf("DirectAccessWindow::clearVarList %ld\n", (long)this);
        fflush(stdout);
    }
 

//     Q_LIST_VIEW_ITEM * next = varList->firstChild();
//     while(next){
//         varList->setSelected(next, false);
//         next->listView()->update();
//         next = next->nextSibling();
//     }
  
    varList->clearSelection();
    activeList = GattrList;
}

// -----------------------------------------------------------


void DirectAccessWindow::clearGattrList(){

    if(QuSlotDebug())
    {
        printf("DirectAccessWindow::clearGattrList %ld\n", (long)this);
        fflush(stdout);
    }
 

//     Q_LIST_VIEW_ITEM * next = GattrList->firstChild();
//     while(next){
//         GattrList->setSelected(next, false);
//         next->listView()->update();
//         next = next->nextSibling();
//     }
  
    GattrList->clearSelection();
    activeList = varList;
}

// ----------------------------------------------

void DirectAccessWindow::keyPressEvent(QKeyEvent * e )
{  
  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if(   getVarBtn->isDefault()) {
		 getVarBtn->setDefault(false);
		 browseVarBtn->setDefault(true);
	  }
	  else if(browseVarBtn->isDefault()) {
		 browseVarBtn->setDefault(false);
		 dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault())  {
		 dismissBtn->setDefault(false);
		 getVarBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(getVarBtn->isDefault()) getData();
	 else if (browseVarBtn->isDefault()) browseData();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

// ---------------------------------------------

void DirectAccessWindow::varDoubleClick(QListWidgetItem* item){

    if(QuSlotDebug())
    {
        printf("DirectAccessWindow::varDoubleClick %ld, item %ld\n", (long)this, (long)item);
        fflush(stdout);
    }
 
    if( !(getVarBtn->isEnabled()) ) return; // get data disabled
    if(item == 0) return;
    activeList = varList;
    varList->setCurrentItem(item);
    getData();

}

// ---------------------------------------------

void DirectAccessWindow::GattrDoubleClick(QListWidgetItem* item){

    if(QuSlotDebug())
    {
        printf("DirectAccessWindow::GattrDoubleClick %ld, item %ld\n", (long)this, (long)item);
        fflush(stdout);
    }
 

    if(item == 0) return;
    activeList = GattrList;
    GattrList->setCurrentItem(item);
    browseData();

}

// ------------------------------------------------

void DirectAccessWindow::getData()
{

	if(QuSlotDebug())
	{
		printf("DirectAccessWindow::getData %ld\n", (long)this);
		fflush(stdout);
	}

	QdObject_var  dobj;
	int vnum;
	int anum;

	// import from either list 
  
	if( activeList == varList) {
  
        QListWidgetItem *item = varList->currentItem();
		if(item == 0) {
			msgBar->showMessage( "No variable selected");
			return;
		}
		bool intOK;
		int fromRec = fromSlider->recField->text().toInt(&intOK);
		if(!intOK) fromRec = 0;
  
		int toRec = toSlider->recField->text().toInt(&intOK);
		if(!intOK) toRec = toSlider->maxRecord;
		vnum = QieDataCont->findVarNumByName(item->text().toLatin1().data() );
		if(vnum == -1 ) {
			msgBar->showMessage( "No Data Object selected");
			return;
		}

		if( !(QieDataCont->QieVars[vnum].recVary) ){
			toRec = 0;
			fromRec = 0;
		}
		
		QieDataCont->readVarData(vnum, fromRec, toRec);


		dobj = QieDataCont->QieVars[vnum].Vobj;


		//    Add object name to WL
		if( !dobj.is_nil() ){

			QieDataCont->attachXrefs(vnum);
		
			// Add parents xref for QSAS browse window use
			QdStringSeq_var file_seq = new QdStringSeq(1);
			file_seq[0] = QdString(QieDataCont->inFile.name); 

			dobj->change_xref("Parents", (QdObject_var) file_seq); 
		
			QdTimeInterval_var ti = QdTimeInterval_var::narrow(dobj);
			QdTimeIntervalSeq_var tis = QdTimeIntervalSeq_var::narrow(dobj);

			if(ti.is_nil() && tis.is_nil()){
				string sanitizedVarName = QuSanitizeName(item->text().toLatin1().data());
				QwuiPutOnWL((QdObject_var) dobj, QString("Data/")+QString(sanitizedVarName.c_str()));
			}
			else{
				string sanitizedVarName = QuSanitizeName(item->text().toLatin1().data());
				QwuiPutOnWL((QdObject_var) dobj, QString("Time Intervals/")+QString(sanitizedVarName.c_str()));
			}
		}
	}
	else if(activeList == GattrList){

        QListWidgetItem* item=GattrList->currentItem();
		if(item == 0){
			msgBar->showMessage( "No G Attr selected");
			return;
		}
    
		// open G attr
		dobj = QieDataCont->GlobalList->resolve(item->text().toLatin1().data());

		//   put G attr on WL
		if( !dobj.is_nil() ){
			QdTimeInterval_var ti = QdTimeInterval_var::narrow(dobj);
			QdTimeIntervalSeq_var tis = QdTimeIntervalSeq_var::narrow(dobj);

			if(ti.is_nil() && tis.is_nil()){
				string sanitizedGattrName = QuSanitizeName(item->text().toLatin1().data());
				QwuiPutOnWL((QdObject_var) dobj, QString("Data/")+QString(sanitizedGattrName.c_str()));
			}
			else{
				string sanitizedGattrName = QuSanitizeName(item->text().toLatin1().data());
				QwuiPutOnWL((QdObject_var) dobj, QString("Time Intervals/")+QString(sanitizedGattrName.c_str()));
			}
		}

	}


}



// Utility call for qwui
//----------------------

 void QdaLaunchWindow(QString file, QWidget *parent)
{
  // direct access window
  
	DirectAccessWindow *QDA_widget = new DirectAccessWindow(parent, "import");
	QSize qsasGUISize = qsasWindow->frameSize();
	QDA_widget->move(qsasGUISize.width(),20);
	
	if ( !file.isNull() ){  // open file and get data
 
		QDA_widget->setWindowTitle(file);
		
		QDA_widget->msgBar->showMessage("PLEASE WAIT, Counting data records..." );
		
		// open file
		int nRecs = QDA_widget->getFileContents(file.toLatin1().data());

		QDA_widget->show();

		// set sliders
		
		QDA_widget->setSliders(nRecs);
		QDA_widget->msgBar->showMessage(" " );

	}
	else {
		QDA_widget->msgBar->showMessage("no file selected?");
		QDA_widget->show();
	}

  return;
}
