/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include <iostream>
#include <stdio.h>

#include "qt_version.h"
#include <qapplication.h>
#include <qpaintdevice.h>
#include <qtooltip.h> 
#include <qglobal.h>
#include <qfont.h>
#include <qsize.h>
#include <qmainwindow.h>
#include <qmenubar.h> 
#include <qwidget.h> 
#include <qstring.h> 
#include <qstatusbar.h>
#include <qlocale.h>
#include <locale.h>

#include "qie_qsas.h"
#include "qwui.h"
#include "qsasMenus.h"

#include "qprofile.h"
#include "qcalcWindow.h"

extern qwuiWindow* qsasWindow;

#ifdef CALC_PROTO
extern QList<VCalcItem*> VCalcItem::currentItems;
extern CalcWindow *QCALC_Proto_widget;
#endif

// *************************************************************************

int main( int argc, char **argv )
{
	setlocale(LC_ALL, "C");
    
#ifdef Q_OS_MACX
    
    freopen("/Applications/QSAS_2_4/tmp/qsas.log", "w", stdout);
    freopen("/Applications/QSAS_2_4/tmp/qsas_err.log", "w", stderr);
    
#if QT_VERSION > 0x050000
    
    QDir dir(argv[0]);  // e.g. QSAS.app/Contents/MacOS/QSAS
    dir.cdUp();
    QCoreApplication::setLibraryPaths(QStringList(dir.absolutePath()));
    
#endif
#endif
    
	QApplication a( argc, argv );
	QLocale::setDefault(QLocale::c());
	QSAS_profiles = new QProfilesList;
	QSAS_profiles->load();
	
	defaultFont=QFont();
	defaultFont.setPixelSize(DEFAULT_FONT_SIZE);
	
	if(QSAS_profiles->current()->fontSize>=5)
	{
		QFont f;
		f.setPixelSize(QSAS_profiles->current()->fontSize);
		a.setFont(f);
	}

	// set default decimal places for seconds in time fields
	QmjdTime t;
	t.setSecResolution(QSAS_profiles->current()->secDecimal);
	

	char * cef2Read = (char *) getenv ("CEF_READ_DEFAULT");
	if(cef2Read == NULL) 
		cefRead2(0);  // false:  default is read cef1 as cef2 can be autodetected
	else if(0 == strcmp(cef2Read,"CEF2")  ) 
		cefRead2(1);
	else
		cefRead2(0);  
  
	char * cef2Write = (char *) getenv ("CEF_WRITE_DEFAULT");
	if(cef2Write == NULL) 
		cefWrite2(1);  // default is write CAA syntax CEF2 files
	else if(0 == strcmp(cef2Write,"CEF2")  ) 
		cefWrite2(1);
	else
		cefWrite2(0);  
  
	// QSAS wide font settings
	QFont qsas_font = QFont(qApp->font());
	QApplication::setFont(qsas_font);

	static QColor col_White("#ffffff");
	static QColor col_Light_Grey("#eeeeee");
	static QColor col_Grey("#8c8c8c");
	static QColor col_Black("#000000");
	static QColor col_Green("#228b22");
	static QColor col_Red("#ff3333");
	static QColor col_Yellow("#ffff00");
	static QColor col_Blue("#000099");

	QPalette qsas_palette = QPalette(qApp->palette());
	
	qsas_palette.setColor(QPalette::Active, QPalette::Light, col_White);
	qsas_palette.setColor(QPalette::Active, QPalette::Midlight,col_Light_Grey);
	qsas_palette.setColor(QPalette::Active, QPalette::Mid,col_Grey);
	qsas_palette.setColor(QPalette::Active, QPalette::Dark,col_Black); 
	qsas_palette.setColor(QPalette::Active, QPalette::Foreground,col_Black);
	qsas_palette.setColor(QPalette::Active, QPalette::Base,col_White);
	qsas_palette.setColor(QPalette::Active, QPalette::Highlight,col_Grey);
	qsas_palette.setColor(QPalette::Active, QPalette::HighlightedText,col_White);
	qsas_palette.setColor(QPalette::Active, QPalette::Text,col_Black);
	qsas_palette.setColor(QPalette::Active, QPalette::Link,col_Red);
	qsas_palette.setColor(QPalette::Active, QPalette::LinkVisited,col_Yellow);
	qsas_palette.setColor(QPalette::Active, QPalette::Button,col_Light_Grey);
	qsas_palette.setColor(QPalette::Active, QPalette::ButtonText,col_Black);

	qsas_palette.setColor(QPalette::Inactive, QPalette::Light,col_White);
	qsas_palette.setColor(QPalette::Inactive, QPalette::Mid,col_Grey);
	qsas_palette.setColor(QPalette::Inactive, QPalette::Midlight,col_Light_Grey);
	qsas_palette.setColor(QPalette::Inactive, QPalette::Foreground,col_Grey);	  
	qsas_palette.setColor(QPalette::Inactive, QPalette::Base,col_Light_Grey);
	qsas_palette.setColor(QPalette::Inactive, QPalette::Highlight,col_Grey);
	qsas_palette.setColor(QPalette::Inactive, QPalette::HighlightedText,col_Light_Grey);
	qsas_palette.setColor(QPalette::Inactive, QPalette::Text,col_Grey);
	qsas_palette.setColor(QPalette::Inactive, QPalette::Button,col_Light_Grey);
	qsas_palette.setColor(QPalette::Inactive, QPalette::ButtonText,col_Grey);

	qsas_palette.setColor(QPalette::Disabled, QPalette::Light,col_White);
	qsas_palette.setColor(QPalette::Disabled, QPalette::Mid,col_Grey);
	qsas_palette.setColor(QPalette::Disabled, QPalette::Midlight,col_Light_Grey);
	qsas_palette.setColor(QPalette::Disabled, QPalette::Foreground,col_Grey);	  
	qsas_palette.setColor(QPalette::Disabled, QPalette::Base,col_Light_Grey);
	qsas_palette.setColor(QPalette::Disabled, QPalette::Highlight,col_Grey);
	qsas_palette.setColor(QPalette::Disabled, QPalette::HighlightedText,col_Light_Grey);
	qsas_palette.setColor(QPalette::Disabled, QPalette::Text,col_Grey);
	qsas_palette.setColor(QPalette::Disabled, QPalette::Button,col_Light_Grey);
	qsas_palette.setColor(QPalette::Disabled, QPalette::ButtonText,col_Grey);

	QApplication::setPalette(qsas_palette);
    
	qsasWindow = new qwuiWindow(0, "qsas");
    
	printf("QSAS version %s.\n", QSAS_VERSION_ID);
	qsasWindow->setWindowTitle( QSAS_VERSION_ID );

	qsasWindow->setFocusPolicy(Qt::WheelFocus);

	
	// create some GUIs now so they can receive restoreSession signal
	QEXPmakeWindow (qsasWindow, "Export");

	QJUImakeWindow (qsasWindow, "Join");

	QDSmakeWindow (qsasWindow, "Data Selection");

	QPUImakeWindow (qsasWindow, "Plot");
	// plot creates the colour tables available for all plot windows

	QPolarMakeWindow (qsasWindow, "Polar Plot");
	
	QSurfaceMakeWindow (qsasWindow, "Surface Plot");

	Q3dMakeWindow (qsasWindow, "3D View Plot");

	QCALCmakeWindow( qsasWindow, "Calculator (Old Interface)" );
	
	QCalcMakeWindowProto(qsasWindow, "Calculator");


	AddQwuiMenus();
    

    QString txt = "QSAS version ";
	txt += QSAS_VERSION_ID;
	qsasWindow->statusBar()->showMessage(txt);
	
	qsasWindow->show();

	qsasWindow->makeWindows();
	

	qsasWindow->move(0,25);     
	qsasWindow->bringToFront();
   
	return a.exec();

}

