//  qpui.h
//
//  Qsas - Plot Widgets Interface File
//
//  Janet Barnes
//  July 2002
//
//  Tony Allen
//  Dec 2007

#ifndef QSAS_PLOT_UI_H_
#define QSAS_PLOT_UI_H_

#include <qsplitter.h>
#include <qstatusbar.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qtabbar.h>
#include <qlayout.h>

#include <QAbstractItemModel>
#include <QApplication>
#include <QMimeData>
#include <QStyle>
#include <QTreeView>
#include <QItemDelegate>
#include <QHeaderView>
#include <QPainter>
#include <QScrollArea>

#include "qt_version.h"
#include "qslot.h"
#include "QbTimeEditors.h"
#include "IntervalDialog.h"

#include "qgu.h"
#include "floatspinbox.h"
#include "IntervalWidgets.h"
#include "ColourTable.h"

#include "PlotWindow.h"

#define QPUI_VERSION  " V_3.1.0 [ Feb 2013 ]"

#if QT_VERSION >= 0x040300
#define QPUI_CHAR_HEIGHT 130 // Truetype fonts handled by Qt
#else
#define QPUI_CHAR_HEIGHT 90 // Hershey fonts handled by PLplot itself
#endif

#define QPUI_PAGE_HGAP 0.05
#define QPUI_PAGE_VGAP 0.1
#define QPUI_PAGE_LMARGIN 0.1 //0.2
#define QPUI_PAGE_RMARGIN 0.1
#define QPUI_PAGE_TMARGIN 0.05 //0.03
#define QPUI_PAGE_BMARGIN 0.08 //0.05

#define QPUI_PANEL_LOFFSET 0.2
#define QPUI_PANEL_WIDTH 0.7
#define QPUI_PANEL_TOFFSET 0.05
#define QPUI_PANEL_HEIGHT 0.90

#define QPUI_LINE_WIDTH 1
#define QPUI_TICKS_LENGTH 0.7

#define QPUI_TITLE_CHAR_SIZE_ADD 0.3 //1.0
#define QPUI_TITLE_DISP 1.0 //0.7
#define QPUI_SUBTITLE_CHAR_SIZE_ADD 0.1
#define QPUI_SUBTITLE_DISP 2.0 //1.5
#define QPUI_LABELS_CHAR_SIZE_ADD 0.0
#define QPUI_LABELS_DISP 3.0 //2.0
#define QPUI_SYMBOL_SIZE 1.0
#define QPUI_LEGEND_CHAR_SIZE_ADD 0.0
#define QPUI_LEGEND_BOX_LINE_WIDTH 1
#define QPUI_LEGEND_LINE_LENGTH 0.2
#define QPUI_LEGEND_MAX_CHAR_LENGTH 25
#define QPUI_LEGEND_JUST 0.5
#define QPUI_LEGEND_HEIGHT 0.4
#define QPUI_LEGEND_WIDTH 0.25

#define QPUI_COLOUR_BAR_DISP 1.5
#define QPUI_COLOUR_BAR_WIDTH 2.5
#define QPUI_COLOUR_BAR_HEIGHT 1.0
#define QPUI_COLOUR_BAR_VOFFSET 0.0

#define QPUI_SKIP_LOG_VALUE -1.e31

#define QPUI_LOWEST_COLOUR_INDEX 16
#define QPUI_HIGHEST_COLOUR_INDEX 255
#define QPUI_NCOLOURS_MAX 256

#define QPUI_SPECTROGRAM_LOFFSET 0.1
#define QPUI_SPECTROGRAM_WIDTH 0.7
#define QPUI_SPECTROGRAM_TOFFSET 0.1
#define QPUI_SPECTROGRAM_HEIGHT 0.2

#define QP_LINEAR 0
#define QP_LOG 1

#define QP_FROM_META 0
#define QP_FROM_DATA 1
#define QP_FROM_USER 2
#define QP_AUTO_ALL_SELECTED 3

#define NAME_COL 0
#define LINK_COL 1

#define PATH_SEPARATOR "&&" // String for paths separator, one that will never be used by anyone

extern ColourTablePalette stdPalette;
extern ColourTablePalette profilePalette;
extern ColourTablePalette importedPalette;

extern ColourTablePalette savedPalette; // Used to reference and save used and non-standard colour tables
extern int importedPaletteOffset; // Used to offset the indexes, when merging a session with another one

class QpuiItem;

class RootItem;
class PageItem;
class FrameItem;
class PanelItem;
class PlotItem;
class TraceItem;
class SpectrumItem;
class EventItem;

class PlotModel;
class PlotView;

typedef vector <double> dblBins;


typedef enum QpItemType_e
{
	QP_TRACE=0,
	QP_SPECTROGRAM=1,
	QP_CONST_XT=2,
	QP_CONST_Y=3,
	QP_EVENT=4,
	QP_PANEL=5,
	QP_FRAME=6,
	QP_PAGE=7,
	QP_NONE=8,
	QP_REG_LINE=9
} QpItemType;

typedef enum QpFrameType_e
{
	QP_TS=0,
	QP_XY=1
	
} QpFrameType;

typedef enum QpTabType_e
{
	QP_DATA_TAB=0,
	QP_TRACE_TAB=1,
	QP_PLOT_TYPE_TAB=2,
	QP_VALUE_TAB=3,
	QP_COLOUR_TAB=4,
	QP_Y_TAB=5,
	QP_LEGEND_TAB=6,
	QP_XT_TAB=7,
	QP_POSITION_TAB=8,
	QP_FRAME_TAB=9,
	QP_TITLE_TAB=10,
	QP_PAGE_TAB=11,
	QP_LAYOUT_TAB=12,
	QP_EVENT_TAB=13,
	QP_IGNORE_TAB=14
  
} QpTabType;

typedef enum QpuiPageSize_e
{
	QPUI_A4=0,
	QPUI_CUSTOM=1,
	QPUI_EXECUTIVE=2,
	QPUI_LEGAL=3,
	QPUI_LETTER=4,
	QPUI_DEFAULT=5
} QpuiPageSize;

typedef enum QpuiPageDPI_e
{
	QPUI_72_DPI=0,
	QPUI_150_DPI=1,
	QPUI_300_DPI=2,
	QPUI_600_DPI=3,
	QPUI_1200_DPI=4
} QpuiPageDPI;

typedef enum QpuiUnits_e
{
	QPUI_PIXELS=0,
	QPUI_INCHES=1,
	QPUI_CM=2,
	QPUI_MM=3,
	QPUI_PSPOINTS=4
} QpuiUnits;

typedef enum QpuiPlotRatio_e
{
    QPUI_FREE=0,
    QPUI_FIXED=1,
    QPUI_SQUARE_ASPECT=2,
    QPUI_ISOTROPIC=3
} QpuiPlotRatio;

typedef enum QpuiReturnStatus_e
{
	QPUI_SUCCESS=0,
	QPUI_FAILURE=1
} QpuiReturnStatus;

typedef enum QpuiLinkStatus_e
{
	QPUI_UNLINKED=0,
	QPUI_LINKED=1
} QpuiLinkStatus;

typedef enum QpuiUpdateStatus_e
{
	QPUI_NO_UPDATES=0,
	QPUI_UPDATED=1,
	QPUI_INCOMPATIBLE=2
} QpuiUpdateStatus;

typedef enum QpuiPageOrientation_e
{
	QPUI_PORTRAIT=0,
	QPUI_LANDSCAPE
} QpuiPageOrientation;

typedef enum QpuiTickMarks_e
{
	QPUI_NOTICKS=0,
	QPUI_IN=1,
	QPUI_OUT=2,
	QPUI_GRID=3
} QpuiTickMarks;

typedef enum QpuiTickLabels_e
{
	QPUI_AUTOMATIC=0,
	QPUI_DECIMAL=1,
	QPUI_EXPONENTIAL=2,
	QPUI_TIME=3
} QpuiTickLabels;


typedef enum QpLineStyle_e
{
	
	QP_NOLINE=0,
	QP_SOLID=1,
	QP_DASHED=2,
	QP_DOTDASH=3,
	QP_DOTTED=4,
	QP_DASHDOTDOTDOT=5
} QpLineStyle;

typedef enum QpuiColours_e
{
	QPUI_WHITE=0,
	QPUI_BLACK=1,
	QPUI_RED=2,
	QPUI_GREEN=3,
	QPUI_BLUE=4,
	QPUI_CYAN=5,
	QPUI_MAGENTA=6,
	QPUI_YELLOW=7,
	QPUI_ORANGE=8,
	QPUI_YELLOWGREEN=9
} QpuiColours;

typedef enum QpuiColourTable_e
{
	QPUI_RAINBOW=0,
	QPUI_RAINBOW_EXT=1,
	QPUI_GRAY_WHITE_LOW=2,
	QPUI_GRAY_WHITE_HIGH=3,
	QPUI_RED_BLUE=4,
	QPUI_INV_RED_BLUE=5,
	QPUI_EARTH=6,
	QPUI_USER_TABLE=7
} QpuiColourTable;

typedef enum QpuiSymbolShape_e
{
	QPUI_NOSYMBOL=0,
	QPUI_POINT=1,
	QPUI_PLUS=2,
	QPUI_ASTERISK=3,
	QPUI_CIRCLE=23,
	QPUI_CROSS=5,
	QPUI_SQUARE=6,
	QPUI_TRIANGLE=7,
// 	QPUI_CIRCLED_CROSS=8,
	QPUI_CIRCLED_POINT=9,
	QPUI_LENS_FLARE=10,
	QPUI_DIAMOND=11,
	QPUI_STAR=12,
  //QPUI_FILLEDTRIANGLE=13, // Not in plplot: square
	QPUI_WIDECROSS=14,
  //QPUI_FILLEDASTERISK=15, // Not in plplot: david star
	QPUI_FILLEDSQUARE=16,
	QPUI_FILLEDCIRCLE=17,
	QPUI_FILLEDSTAR=18,
	QPUI_LEFT_ARROW=28,
	QPUI_RIGHT_ARROW=29,
	QPUI_UP_ARROW=30,
	QPUI_DOWN_ARROW=31,
	QPUI_CUSTOM_SHAPE=32
} QpuiSymbolShape;

typedef enum QpuiConnectStyle_e
{
	QPUI_STRAIGHT=0,
	QPUI_HIST=1,
	QPUI_NOCONNECT=2
} QpuiConnectStyle;

typedef enum QpuiObjectType_e
{
	QPUI_DATA_SERIES=0,
	QPUI_TIME_SERIES=1,
	QPUI_TIME_CONSTANT=2,
	QPUI_SCALAR_Y_CONSTANT=3,
	QPUI_SCALAR_X_CONSTANT=4
} QpuiObjectType;

typedef enum QpuiJustification_e
{
	QPUI_HCENTRE=0,
	QPUI_LEFT=1,
	QPUI_RIGHT=2
} QpuiJustification;

typedef enum QpuiLegendJust_e
{
	QPUI_VCENTRE=0,
	QPUI_TOP=1,
	QPUI_BOTTOM=2
} QpuiLegendJust;

typedef enum QpuiCharFont_e
{
	QPUI_NORMAL=0,
	QPUI_ROMAN=1,
	QPUI_ITALIC=2,
	QPUI_SCRIPT=3
} QpuiCharFont;

typedef enum QpuiVectorComp_e
{
	QPUI_VECTOR_X=0,
	QPUI_VECTOR_Y=1,
	QPUI_VECTOR_Z=2,
	QPUI_VECTOR_RMAG=3,
	QPUI_VECTOR_LAT_DEG=4,
	QPUI_VECTOR_THETA_DEG=5,
	QPUI_VECTOR_PHI_DEG=6,
	QPUI_VECTOR_LAT_RAD=7,
	QPUI_VECTOR_THETA_RAD=8,
	QPUI_VECTOR_PHI_RAD=9
} QpuiVectorComp;

typedef enum QpuiTimeIntervalComp_e
{
	QPUI_START_TIME=0,
	QPUI_END_TIME=1,
	QPUI_CENTRE_TIME=2,
	QPUI_START_AND_END_TIME=3
} QpuiTimeIntervalComp;

typedef enum QpuiLegendAlign_e
{
	QPUI_INSIDE_TOP_RIGHT=0,
	QPUI_INSIDE_TOP_LEFT=1,
	QPUI_INSIDE_BOTTOM_RIGHT=2,
	QPUI_INSIDE_BOTTOM_LEFT=3,
	QPUI_ABOVE_TOP_CENTRE=4,
	QPUI_OUTSIDE_TOP_RIGHT=5,
	QPUI_USER_DEFINED=6
} QpuiLegendAlign;

typedef enum QpuiLegendBorder_e
{
	QPUI_NO_BORDER=0,
	QPUI_LINE=1
} QpuiLegendBorder;

typedef enum QpuiLegendBackground_e
{
	QPUI_TRANSPARENT=0,
	QPUI_OPAQUE=1
} QpuiLegendBackground;

typedef enum QpuiPageFlds_e
{
	PAGE_SIZE=0,
	PAGE_UNITS=1,
	PAGE_WIDTH=2,
	PAGE_HEIGHT=3,
	PAGE_ORIENTATION=4,
	XWIN_NUM=5,
	IDENT=6,
	NUM_ROWS=7,
	NUM_COLS=8,
	HGAP=9,
	VGAP=10,
	LMARGIN=11,
	RMARGIN=12,
	TMARGIN=13,
	BMARGIN=14,
	AUTOLAYOUT=15,
	PAGE_DPI=16
} QpuiPageFlds;

const int QpuiPageNumFlds=17;

typedef enum QPanelFlds_e
{
	LOFFSET=0,
	WIDTH=1,
	TOFFSET=2,
	HEIGHT=3,
	ASPECT_RATIO=4,
	PANEL_LINE_WIDTH=5,
	TICKS_LENGTH=6,
	PANEL_LINE_COLOUR=7,
	TOP_AXIS=8,
	BOTTOM_AXIS=9,
	LEFT_AXIS=10,
	RIGHT_AXIS=11,
	TOP_TICKS_LABELS=12,
	BOTTOM_TICKS_LABELS=13,
	LEFT_TICKS_LABELS=14,
	RIGHT_TICKS_LABELS=15,
	TOP_LABELS=16,
	BOTTOM_LABELS=17,
	LEFT_LABELS=18,
	RIGHT_LABELS=19,
	X_IS_ZERO=20,
	Y_IS_ZERO=21,
	AUTOTITLE=22,
	TITLE=23,
	TITLE_CHAR_SIZE=24,
	TITLE_DISP=25,
	SUBTITLE=26,
	SUBTITLE_CHAR_SIZE=27,
	SUBTITLE_DISP=28,
	TITLE_JUST=29,
	TITLE_FONT_FAMILY=30,
	TITLE_COLOUR=31,
	LABELS_CHAR_SIZE=32,
	LABELS_DISP=33,
	LABELS_FONT=34,
	LABELS_COLOUR=35,
	LABELS_ROTATION=36,
	FRAME_TYPE=37,
	TICKS_CHAR_SIZE=38,
	LR_ALTERNATING_AXIS=39,
	LR_ALTERNATING_TICKS_LABELS=40,
	LR_ALTERNATING_LABELS=41,
	RL_ALTERNATING_AXIS=42,
	RL_ALTERNATING_TICKS_LABELS=43,
	RL_ALTERNATING_LABELS=44,
	TITLE_FONT_STYLE=45,
	TITLE_FONT_WEIGHT=46
} QpPanelFlds;

const int QpuiPanelNumFlds=47;

typedef enum QpAxes_e
{
	X_AXIS=0,
	Y_AXIS=1,
	T_AXIS=2,
	Z_AXIS=3,
	COL_AXIS=4
} QpAxes;

typedef enum QpuiAxisFlds_e
{
	AXIS_AUTOSCALE=0,

 AXIS_MIN=1,
 AXIS_MAX=2,
 AXIS_T_DATA=3,
 AXIS_LINLOG=4,
 AXIS_NUM_MAJOR_TICKS=5,
 AXIS_NUM_MINOR_TICKS=6,
 AXIS_MAJOR_TICKMARKS=7,
 AXIS_MINOR_TICKMARKS=8,
 AXIS_LABEL=9,
 AXIS_AUTOLABEL=10,
 AXIS_LABEL_CHAR_SIZE=11,
 AXIS_LABEL_DISP=13,
 AXIS_LABEL_ROTATION=14,
 AXIS_LABEL_FONT_FAMILY=15,
 AXIS_LABEL_COLOUR=16,
 AXIS_TICK_TIME_FORMAT=17,
 AXIS_TICK_TIME_FORMAT_AUTO=18,
 AXIS_TIME_LABELS_FORMAT=19,
 AXIS_TIME_LABELS_FORMAT_AUTO=20,
 AXIS_LABEL_FONT_STYLE=21,
 AXIS_LABEL_FONT_WEIGHT=22,
 AXIS_EXP_LABEL_Y=23,
 AXIS_EXP_LABEL_X=24

} QpuiAxisFlds;

const int QpuiAxisNumFlds=24;

typedef enum QpuiLegendFlds_e
{
	SHOW_LEGEND=0,
	LEGEND_BORDER=1,
	LEGEND_NUM_ROWS=4,
	LEGEND_NUM_COLS=5,
	LEGEND_CHAR_SIZE=6,
	LEGEND_MAX_CHAR_LENGTH=7,
	LEGEND_LINE_LENGTH=8,
	LEGEND_JUST=9,
	LEGEND_FONT_FAMILY=10,
	LEGEND_ALIGN=11,
	LEGEND_L_OFFSET=12,
	LEGEND_T_OFFSET=13,
	LEGEND_BACKGROUND=14,
	LEGEND_WIDTH=15,
	LEGEND_HEIGHT=16,
	LEGEND_FONT_STYLE=17,
	LEGEND_FONT_WEIGHT=18
} QpuiLegendFlds;

const int QpuiLegendNumFlds=19;

typedef enum QpuiEventFlds_e
{
	EVENT_LABEL=0,
	EVENT_LABEL_CHAR_SIZE=1,
	EVENT_LABEL_FONT_FAMILY=2,
	EVENT_LABEL_COLOUR=3,
	EVENT_X_DATA=4,
	EVENT_Y_DATA=5,
	EVENT_LABEL_FONT_STYLE=6,
	EVENT_LABEL_FONT_WEIGHT=7,
    EVENT_LABEL_JUST=9
} QpuiEventFlds;

const int QpuiEventNumFlds=9;


typedef enum QpuiTraceFlds_e
{
 TRACE_PLOT_TYPE=0,
 TRACE_YDATA=1,
 TRACE_XDATA=2,
 TRACE_YCOMP=3,
 TRACE_XCOMP=4,
 TRACE_CONNECT_STYLE=5,
 TRACE_LINE_STYLE=6,
 TRACE_LINE_WIDTH=7,
 TRACE_LINE_COLOUR=8,
 TRACE_SYMBOL_SHAPE=9,
 TRACE_SYMBOL_SIZE=10,
 TRACE_SYMBOL_COLOUR=11,
 TRACE_CONSTANT_DATA=12,
 TRACE_TCOMP=13,
 TRACE_LEGEND_TEXT=14,
 TRACE_SHOW_LEGEND=15,
 LEGEND_AUTO_TEXT=16,
 TRACE_GAP_DETECTION=17,
 TRACE_GAP_THRESHOLD=18,
 TRACE_GAP_THRESHOLD_TYPE=19,
 TRACE_LINE_SCALE=20,
 TRACE_CUSTOM_SYMBOL=21,
 TRACE_TIDATA=22,
 TRACE_TICOMP=23,
 TRACE_COL_COMP=24,
 TRACE_COL_DATA=25,
 TRACE_SHOW_COL_BAR=26,
 TRACE_EXP_COL_BAR_X=27,
 TRACE_EXP_COL_BAR_Y=28,
 TRACE_AUTO_COL_BAR=29,
 TRACE_LOG_COL_BAR=30,
 TRACE_COL_MIN=31,
 TRACE_COL_MAX=32,
 TRACE_SET_COL_TABLE=33,
 TRACE_USER_COL_LAB=34,
 TRACE_COL_BAR_W=35,
 TRACE_COL_BAR_H=36,
 TRACE_COL_BAR_HOFF=37,
 TRACE_COL_BAR_VOFF=38,
 TRACE_COL_BAR_TICK=39,
 TRACE_SET_C_COL=40,
 TRACE_SET_C_SIZE=41
} QpuiTraceFlds;

typedef enum QpuiDataFlds_e
{
	USE_TIME_AS_SEC=0
} QpuiDataFlds;

const int QpuiTraceNumFlds=21;

typedef enum QpuiSpectroGramFlds_e
{
	SPECTROGRAM_PLOT_TYPE=0,

 SPECTROGRAM_ZDATA=1,
 COLOUR_SCALE_AUTOSCALE=2,
 COLOUR_SCALE_MIN=3,
 COLOUR_SCALE_MAX=4,
 COLOUR_SCALE_LINLOG=5,
 COLOUR_SCALE_SHOW_COLOUR_BAR=6,
 COLOUR_SCALE_COLOUR_BAR_WIDTH=7,
//  COLOUR_SCALE_TABLE=8,
 COLOUR_SCALE_COLOUR_BAR_DISP=9,
 COLOUR_SCALE_LABEL=10,
 COLOUR_SCALE_AUTOLABEL=11,
 SPECTROGRAM_YDATA_START=12,
 SPECTROGRAM_XDATA_START=13,
 SPECTROGRAM_XCOMP_START=14,
 SPECTROGRAM_YDATA_END=15,
 SPECTROGRAM_XDATA_END=16,
 SPECTROGRAM_XCOMP_END=17,
 SPECTROGRAM_YCOMP_START=18,
 SPECTROGRAM_YCOMP_END=19,
 SPECTROGRAM_ZCOMP=20,
 COLOUR_SCALE_COLOUR_BAR_HEIGHT=21,
 COLOUR_SCALE_COLOUR_BAR_VOFFSET=22,
 COLOUR_SCALE_TICK_LENGTH=23,
 COLOUR_SCALE_TABLE_STD=24,
 COLOUR_SCALE_TABLE_PROFILE=25,
 COLOUR_SCALE_TABLE_SESSION=26,
 COLOUR_SCALE_AXIS_EXP_LABEL_X=27,
 COLOUR_SCALE_AXIS_EXP_LABEL_Y=28,
 SPECTROGRAM_XY_SWAP=29,
 SPECTROGRAM_TICOMP=30,
 SPECTROGRAM_TIDATA=31,
 SPECTROGRAM_INTERLACE=32,
 SPECTROGRAM_MIN_CELL_WIDTH=33,
 SPECTROGRAM_GAP=34
} QpuiSpectroGramFlds;

const int QpuiSpectroGramNumFlds=29;

int calc_tol_spacing(double& spacing, const QmjdTimeSeq& timeseq, double tol=1.5, double fract_reqd=0.01);

double getSpacingMedian(QdObject_var D0);

// int calc_tol_spacing(QmjdTime& half_spacing_t, const QmjdTimeSeq& timeseq);

class FontButton: public QPushButton
{
	Q_OBJECT
	
	public:
		FontButton(QWidget* parent=NULL, int family=PL_FCI_SANS, bool italic=false, bool bold=false);
		
		~FontButton();
		
		void getFontParams(PLINT& family, PLINT& style, PLINT& weight);
		void setFontParams(PLINT family, PLINT style, PLINT weight);

#if QT_VERSION < 0x040300
        int getFontIndex();
#endif
	
	signals:
		void familyChanged(int);
		void styleChanged(bool);
		void weightChanged(bool);
		
	public slots:

		void setSansFont();
		void setSerifFont();
		void setMonotypeFont();
		void setDecorativeFont();
		void setItalic(bool i);
		void checkItalic();
		void setBold(bool b);
		void checkBold();

        void setRomanFont();
        void setNormalFont();
        void setItalicFont();

		
	protected:
		
		void update();
		
		int family;
		bool italic;
		bool bold;
		
		QMenu* menu;

		QAction* sansAction;
		QAction* serifAction;
		QAction* monotypeAction;
		QAction* decorativeAction;
		QAction* italicAction;
		QAction* boldAction;
#if QT_VERSION < 0x040300
        QAction* normalFontAction;
        QAction* romanFontAction;
        QAction* italicFontAction;
#endif
};

class ColourTableButton: public QPushButton
{
    Q_OBJECT

    public:
        ColourTableButton(QWidget* parent=0);

        void setCurrentTable(const ColourTable&);

        ColourTable getCurrentTable() const;

        void setStdPalette(int i); // for compatibility with old combobox
        void setProfilePalette(int i);
        void setSessionPalette(int i);
        
    public slots:
        void updateMenuEntries();

        void selected(QAction*);

    signals:

        void selectedStd(int);
        void selectedProfile(int);
        void selectedSession(int);
        
    protected:
        QMenu* menu;
        ColourTable currentTable;
};

// QPUI version 3 classes - AJA Jan 2008

class QpWindow;

// QpTab top level class
// ----------------------
class QpTab:  public QWidget
{
	Q_OBJECT

	public:
		QpTab(QTabWidget *tabWidget,  const QString& name, QpTabType _tabType);
		~QpTab(){;}
		void setSlotColour(bool active);

		QPalette GUIpalette;
		QpTabType TabType;
	
};

// QpValueReturnTab Class
// ----------------------

class QpValueReturnTab:  public QpTab
{
	Q_OBJECT

	public:
		QpValueReturnTab(QTabWidget *tabWidget,  const QString& name, PageItem * page);
	
		QpValueReturnTab(QpValueReturnTab* tab, PageItem* _page); // Copy constructor for copy/paste functions

		~QpValueReturnTab(){ 
			killTimer(_timer);
		}

		PageItem*  _page_item;
		FrameItem* _frame_item;
		PanelItem* _panel_item;

		QPushButton *_rePlot;
		QPushButton *_reset;

		ScalarIntervalWidget* _xaxis_range_fld;
		TimeIntervalWidget * _taxis_range_fld;
		QCheckBox *_xaxis_rescale_fld;
		QCheckBox *_xaxis_swap_fld;


		ScalarIntervalWidget* _yaxis_range_fld;
		QCheckBox *_yaxis_rescale_fld;
		QCheckBox *_yaxis_swap_fld;
		
		QLabel *message_lbl;
		QCheckBox *_replot_fld;

		double _x_from;
		double _x_to;
		double _y_from;
		double _y_to;

		vector <double> _x_from_old;
		vector <double>  _x_to_old;
		vector <double>  _y_from_old;
		vector <double>  _y_to_old;
		vector <int>     num_old_frames;
		vector <int>     num_old_panels;
		vector <FrameItem*>  _frame_item_old;
		vector <PanelItem*>  _panel_item_old;
		vector <int>  _panel_autoscale_old;
		vector <int>  _frame_autoscale_old;
        
		int _timer;

	protected:
		virtual void timerEvent(QTimerEvent *e)
		{
			if(parentWidget()!=NULL)
			{
				if(!parentWidget()->isVisible())
					hide();
			}
		}

	public slots:
		void catchValueRet(int);
		void reject(){;}
		void resetValues();
		void rePlot();
		void getValues();
	protected:
		void create(); // common part for both constructors
};


// class QpAxisTab
//------------------

class QpAxisTab : public QpTab
{
	Q_OBJECT

	public:
		QpAxisTab(QTabWidget *tabWidget,  const QString& name, QpuiItem * item, QpTabType _tabType);
		QpAxisTab(QpAxisTab* tab, QpuiItem* item, QpTabType _tabType);
		~QpAxisTab(){;}
			// methods to be re-implemented for each axis type
		virtual void setAxisMinMax(){;}
		virtual void setAutoLabelText(){;}
		virtual void setTfromObj(){;}
		void refreshYZRanges();
		void handleAutoScale(int);
	
			QpuiItem * I_Item;
			QpAxes I_axis;
	
			PanelItem * I_Panel;
			QComboBox *_axis_autoscale_fld;
			QLabel* linlog_label;
			QComboBox *_axis_linlog_fld;
			QLabel* t_drop_label;
			QguWLinput *_t_data_fld;

			QCheckBox *_axis_y_nudge_fld;

	
			ScalarIntervalWidget* _axis_xy_fld;
			TimeIntervalWidget* _axis_time_fld;

			QSpinBox *_axis_major_ticks_num_fld;
			QComboBox *_axis_major_tickmarks_fld;
			QSpinBox *_axis_minor_ticks_num_fld;
			QComboBox *_axis_minor_tickmarks_fld;
			QGroupBox* _exp_label_params_groupbox; // hidden for T axes
			FloatSpinBox * _exp_label_offset_y_fld;
			FloatSpinBox * _exp_label_offset_x_fld;
			QLineEdit *_label_fld;
			FloatSpinBox *_label_char_size_fld;
			FloatSpinBox *_label_disp_fld;
			QCheckBox *_axis_autolabel_fld;
			FloatSpinBox *_label_rotation_fld;
			FontButton *_label_font_fld;
			QPushButton *_label_colour_fld;
			QPushButton *_event_label_fld;
			QGroupBox *frame_type_groupbox;
			QPushButton *_frame_type_btn;
	
			QLabel* tick_time_fmt_label;
			QLabel* time_labels_fmt_label;
			QLineEdit* tick_time_fmt_fld;
			QLineEdit* time_labels_fmt_fld;
			QCheckBox* tick_time_fmt_auto_fld;
			QCheckBox* time_labels_fmt_auto_fld;
	
			int axis_auto_last;
			QColor _label_colour;

			QDomElement toXMLElement(QDomDocument&) const;
			void fromXMLElement(QDomElement&);
	
	//IntervalEditorWindow* editorWindow;

	public slots:
		void setAxisAutoScale(int);
		void setAxisLinLog(int);
		void setAxisRange();
		void setAxisNumMajorTicks(int);
		void setAxisMajorTickMarks(int);
		void setAxisNumMinorTicks(int);
		void setAxisMinorTickMarks(int);
        void setAxisExpLabelYOffset(double);
        void setAxisExpLabelXOffset(double);
		void setAxisLabel();
		void setAxisLabelCharSize(double);
		void setAxisLabelDisp(double);
		void setAxisAutoLabel(bool);
		void setAxisLabelRotation(double);
		void setAxisLabelFamily(int);
		void setAxisLabelStyle(bool);
		void setAxisLabelWeight(bool);
		void setAxisLabelColour();
		void launchFrameTypeSelector();
	protected:
		void create();

};

// class QpXAxisTab
//------------------

class QpXAxisTab : public QpAxisTab
{
	Q_OBJECT

	public:
		QpXAxisTab(QTabWidget *tabWidget,  const QString& name, FrameItem * frame);
		QpXAxisTab(QpXAxisTab* tab, FrameItem* frame);
		~QpXAxisTab(){;}
		void setAxisMinMax();
		void setAutoLabelText();
	
		FrameItem * I_Frame;
	
	public slots:
    
		void interactiveRescale(double start, double end);

};


// class QpTAxisTab
//------------------

class QpTAxisTab : public QpAxisTab
{
	Q_OBJECT

	public:
		QpTAxisTab(QTabWidget *tabWidget,  const QString& name, FrameItem * frame);
		QpTAxisTab(QpTAxisTab* tab, FrameItem* frame);
		~QpTAxisTab(){;}
		void setAxisMinMax();
		void setAutoLabelText();
		void setTfromObj();
	
		FrameItem * I_Frame;
	
	public slots:
		void setTData();
    
		void setTickTimeFormat(const QString&);
		void setTickTimeFormatAuto(int);
		void setTimeLabelsFormat(const QString&);
		void setTimeLabelsFormatAuto(int);

		void interactiveRescale(const QmjdTimeInterval&);
};

// class QpYAxisTab
//------------------

class QpYAxisTab : public QpAxisTab
{
	Q_OBJECT

	public:
		QpYAxisTab(QTabWidget *tabWidget,  const QString& name, PanelItem * panel);
		QpYAxisTab(QpYAxisTab* tab, PanelItem* panel);
		~QpYAxisTab(){;}
		void setAxisMinMax();
		void setAutoLabelText();
	
		PanelItem * I_Panel;

	public slots:
		void updateNudge();
};

// class QpLayoutTab
//------------------

class QpLayoutTab : public QpTab
{
	Q_OBJECT
	public:
		QpLayoutTab(QTabWidget *tabWidget,  const QString& name, PageItem * _page);
		QpLayoutTab(QpLayoutTab* tab, PageItem* _page);
	
		~QpLayoutTab(){;}
		void updateRowCol(int numFrames);

		PageItem * I_Page;
		QSpinBox* _row_num_fld;
		QSpinBox* _col_num_fld;
		QCheckBox* _autolayout_fld;
		FloatSpinBox* _hgap_fld;
		FloatSpinBox* _vgap_fld;
		FloatSpinBox* _lmargin_fld;
		FloatSpinBox* _rmargin_fld;
		FloatSpinBox* _tmargin_fld;
		FloatSpinBox* _bmargin_fld;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
	public slots:
		void setNumRows(int);
		void setNumCols(int);
		void setHGap(double);
		void setVGap(double);
		void setLMargin(double);
		void setRMargin(double);
		void setTMargin(double);
		void setBMargin(double);
		void setAutoLayout(bool);
	protected:
		void create(); // common part for both constructors


};

// class QpPageTab
//------------------

class QpPageTab : public QpTab
{
	Q_OBJECT
	public:
		QpPageTab(QTabWidget *tabWidget,  const QString& name, PageItem * _page);
		QpPageTab(QpPageTab* tab, PageItem* _page);
		~QpPageTab(){;}

		PageItem * I_Page;
	
		QComboBox* _page_size_fld;
		QComboBox* _units_fld;
		QLineEdit* _width_fld;
		QLineEdit* _height_fld;
		QComboBox* _dpi_fld;
		QComboBox* _orientation_fld;
		QLineEdit* _xwin_num_fld;
		QCheckBox* _show_ident_fld;
		int oldUnits;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
	public slots:
		void setPageSize(int);
		void setUnits(int);
		void setDPI(int);
		void setWidth(const QString&);
		void setHeight(const QString&);
		void setOrientation(int);
		void setXwinNum(const QString&);
		void setIdent(bool);
	
	protected:
		void create(); // common part for both constructors
};

class QpEventTab: public QpTab
{
	Q_OBJECT

	public:
		QpEventTab( QTabWidget *tabWidget,  const QString& name, EventItem * _event);
		QpEventTab(QpEventTab* tab, EventItem* _event);
		~QpEventTab(){;}

		EventItem * I_event;
		QguWLinput* _event_label_fld;
		QLabel* _y_label_fld;
		QLabel* _x_label_fld;
		QguWLinput* _y_data_fld;
		QguWLinput* _x_data_fld;
		FloatSpinBox* _char_size_fld;
		FontButton* _event_label_font_fld;
		QPushButton* _event_label_colour_fld;
        FloatSpinBox* _event_label_just_fld;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);

	signals:

	public slots:
		void setEventLabel();
		void setEventLabelCharSize(double);
        void setEventLabelJust(double);
		void setEventLabelFamily(int);
		void setEventLabelStyle(bool);
		void setEventLabelWeight(bool);
		void setEventLabelColour();
		void setYData();
		void setXData();
		void getValues();
	protected:
		void create();

};



// class QpDisplayTab
//------------------

class QpDisplayTab : public QpTab
{
	Q_OBJECT
	public:
		QpDisplayTab(QTabWidget *tabWidget,  const QString& name, FrameItem * _frame);
		QpDisplayTab(QpDisplayTab* tab, FrameItem* _frame);
		~QpDisplayTab(){;}

		FrameItem* I_Frame;
  
		FloatSpinBox* _line_width_fld;
		QPushButton* _line_colour_fld;
		FloatSpinBox* _ticks_length_fld;
		FloatSpinBox* _ticks_char_size_fld;
		QCheckBox* _show_top_axis_fld;
		QCheckBox* _show_bottom_axis_fld;
		QCheckBox* _show_left_axis_fld;
		QCheckBox* _show_right_axis_fld;
		QCheckBox* _show_lr_alternating_axis_fld;
		QCheckBox* _show_rl_alternating_axis_fld;
		QCheckBox* _show_top_ticks_labels_fld;
		QCheckBox* _show_bottom_ticks_labels_fld;
		QCheckBox* _show_left_ticks_labels_fld;
		QCheckBox* _show_right_ticks_labels_fld;
		QCheckBox* _show_lr_alternating_ticks_labels_fld;
		QCheckBox* _show_rl_alternating_ticks_labels_fld;
		QCheckBox* _show_top_labels_fld;
		QCheckBox* _show_bottom_labels_fld;
		QCheckBox* _show_left_labels_fld;
		QCheckBox* _show_right_labels_fld;
		QCheckBox* _show_lr_alternating_labels_fld;
		QCheckBox* _show_rl_alternating_labels_fld;
		QCheckBox* _show_xiszero_fld;
		QCheckBox* _show_yiszero_fld;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
	public slots:
		void setLineWidth(double);
		void setLineColour();
		void setTicksLength(double);
		void setTicksCharSize(double);
		void setTopAxis(bool);
		void setBottomAxis(bool);
		void setLeftAxis(bool);
		void setRightAxis(bool);
		void setLRAlternatingAxis(bool);
		void setRLAlternatingAxis(bool);
		void setTopTicksLabels(bool);
		void setBottomTicksLabels(bool);
		void setLeftTicksLabels(bool);
		void setRightTicksLabels(bool);
		void setLRAlternatingTicksLabels(bool);
		void setRLAlternatingTicksLabels(bool);
		void setTopLabels(bool);
		void setBottomLabels(bool);
		void setLeftLabels(bool);
		void setRightLabels(bool);
		void setLRAlternatingLabels(bool);
		void setRLAlternatingLabels(bool);
		void setYIsZero(bool);
		void setXIsZero(bool);
	protected:
		void create();

};

// class QpPropertiesTab
// --------------------------

class QpPropertiesTab : public QpTab
{
	Q_OBJECT
	public:
		QpPropertiesTab(QTabWidget *tabWidget,  const QString& name, FrameItem * _frame);
		QpPropertiesTab(QpPropertiesTab* tab, FrameItem* _frame);
		~QpPropertiesTab(){;}
		double setIsotropicRatio();

		FrameItem* I_Frame;
		QLabel* _autolayout_lbl;
		FloatSpinBox* _loffset_fld;
		FloatSpinBox* _width_fld;
		FloatSpinBox* _toffset_fld;
		FloatSpinBox* _height_fld;
        QComboBox* _aspect_ratio_fld;
		QComboBox* _frame_type_fld;
		double _aspect_ratio;
		double _isotropic_y_range;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
		
	public slots:
		void setLOffset(double);
		void setTOffset(double);
		void setWidth(double);
		void setHeight(double);
		void setAspectRatio(int);
        
	protected:
		void create();

};

// class QpTitleTab
// -----------------

class QpTitleTab : public QpTab
{
	Q_OBJECT
	public:
		QpTitleTab(QTabWidget *tabWidget,  const QString& name, FrameItem * _frame);
		QpTitleTab(QpTitleTab* tab, FrameItem* _frame);
		~QpTitleTab(){;}

		FrameItem* I_Frame;
		QCheckBox* _autotitle_fld;
		QLineEdit* _title_fld;
		FloatSpinBox* _title_char_size_fld;
		FloatSpinBox* _title_disp_fld;
		QLineEdit* _subtitle_fld;
		FloatSpinBox* _subtitle_char_size_fld;
		FloatSpinBox* _subtitle_disp_fld;
		QComboBox* _title_just_fld;
		FontButton* _title_font_fld;
		QPushButton* _title_colour_fld;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
					
	public slots:
		void setAutoTitle(bool);
		void setTitle(const QString &);
		void setTitleCharSize(double);
		void setTitleDisp(double);
		void setSubTitle(const QString &);
		void setSubTitleCharSize(double);
		void setSubTitleDisp(double);
		void setTitleJust(int);
		void setTitleFamily(int);
		void setTitleStyle(bool);
		void setTitleWeight(bool);
		void setTitleColour();
	protected:
		void create();
	
};



// class QpDataTab
//------------------

class QpDataTab : public QpTab
{
	Q_OBJECT
	public:

		QpDataTab(QTabWidget *tabWidget,  const QString& name, PlotItem * _plot, QpItemType plotType);
		QpDataTab(QpDataTab* tab, PlotItem* _plot);
		~QpDataTab(){;}
		void setDataTypeTitle();
	
		PlotItem * I_Plot;
		QpItemType I_PlotType;
	
		QLabel* _y_label_fld;
		QguWLinput* _y_data_fld;
		QLabel* _x_label_fld;
		QguWLinput* _x_data_fld;
		QLineEdit *_legend_text_fld;
		QCheckBox* _show_legend_fld;
		QCheckBox* _auto_legend_fld;

		QLabel* _mask_label_fld;
		QguWLinput* _mask_data_fld;

        QWidget* _gap_params_container;
        QCheckBox* _gap_detection_fld;
        FloatSpinBox* _gap_threshold_fld;
        QComboBox* _gap_threshold_type_fld;
        double gap_threshold; // in seconds
		
		QGroupBox *meta_groupbox;
		QMenu *metaMenu;
        QLineEdit *metaText;
		vector <QAction*> metaActions;
        
	// for Spectrum
		QCheckBox* _XYtoggle;
		QguWLinput* _z_data_fld;
		QguWLinput* _y_start_data_fld;
		QLabel* _x_start_label_fld;
		QguWLinput* _x_start_data_fld;
		QCheckBox *gap_toggle;
		FloatSpinBox *minCellWidth;
		
		QCheckBox *interlace_toggle;
		QCheckBox *timeAsSec_toggle;
		QGroupBox *data_type_groupbox;
		QString old_BaseName;
		QString old_BaseTIName;
		QString old_BaseXName;
		QString old_XComp;
		QString old_YComp;

        QDomElement toXMLElement(QDomDocument&, QpItemType) const;
        void fromXMLElement(QDomElement&, QpItemType);

	public slots:
		void setYData();
		void setXData();
		void setTIData();
		void resetLinReg();
		void setTI_ZData();
		void setLegendText(const QString &);
		void setAutoLegend(bool);
		void setShowLegend();
		void useTimeAsSec(bool);
		void gap_toggle_changed(bool);
		void interlace_toggle_changed(bool);
		void propagateLink();
        void gap_detection_toggled(bool);
        void gap_threshold_changed(double);
        void gap_threshold_type_changed(int);
        void updateGapThreshold();
		void setMetaMenu();
		void setMetaText();
		void minCellWidth_changed(double value);
   
	// for Spectrum
		void setZData();
		void setXYswap();
		void setYStartData();
		void checkYStartData(const QString &);
		void setXStartData();
		void launchTypeSelector();
	protected:
		void create();
        
};


// class QpTraceTab
//------------------

class QpTraceTab : public QpTab
{
	Q_OBJECT
	public:
		QpTraceTab(QTabWidget *tabWidget,  const QString& name, TraceItem * _trace);
		QpTraceTab(QpTraceTab* tab, TraceItem* trace);
		~QpTraceTab(){;}
   
		TraceItem * I_Trace;
  
		QComboBox* _line_style_fld;
		FloatSpinBox* _line_width_fld;
		QComboBox* _connect_style_fld;
		FloatSpinBox* _line_scale_fld;
		QPushButton* _line_colour_fld;
		QComboBox* _symbol_shape_fld;
		FloatSpinBox* _symbol_size_fld;
		QLineEdit* _custom_symbol_fld;
		QPushButton* _symbol_colour_fld;
		void setLineColour(QColor c);
		void setSymbolColour(QColor c);
		QLabel *_description_fld;
		QguWLinput* _c_data_fld;
		QLabel *_c_label_fld;
		QLabel *maxLabel;
		QLabel *minLabel;
		QLabel* colour_table_label;
		QLabel * colLabLab;
		QCheckBox *c_range_auto;
		QCheckBox *c_range_log;
		QLineEdit *_c_min_fld;
		QLineEdit *_c_max_fld;
		ColourTableButton *colourTable;
		QCheckBox *_show_colour_bar;
		QLineEdit *colLabSlot;
		bool userColLab;
		FloatSpinBox *_exp_label_offset_x;
		FloatSpinBox *_exp_label_offset_y;
		FloatSpinBox *_colour_bar_width;
		FloatSpinBox *_colour_bar_height;
		FloatSpinBox *_colour_bar_disp;
		FloatSpinBox *_colour_bar_voffset;
		FloatSpinBox *_colourscale_ticklength;
		bool useColBarOn;
		int oldSymbolShape;
		QCheckBox *_colour_on;
		QCheckBox *_size_on;
		
        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
	public slots:
		void setLineStyle(int choice);
		void setLineWidth(double choice);
		void setConnectStyle(int choice);
		void setLineScale(double choice);
		void setLineColour();
		void setSymbolShape(int choice);
		void setSymbolSize(double choice);
		void setSymbolColour();
		void setCustomSymbol(const QString& text);
		void setColData();
		void setColDataOff();
		void setColUserMin(const QString &);
		void setColUserMax(const QString &);
		void setAutoColour(bool on);
		void setLogColour(bool on);
		void setUserColLab(const QString& );
		void setAxisExpLabelXOffset(double val);
		void setAxisExpLabelYOffset(double val);
		void showColourBar(bool);
		void setColourTable(int);
        void setColBarWidth(double);
        void setColBarHeight(double);
        void setColBarHoffset(double);
        void setColBarVoffset(double);
        void setColBarTickLen(double);
		void setColourOn(bool);
		void setSizeOn(bool);
	
	protected:
		void create();
		QString old_BaseName;
		QString old_Comp;


};


// class QpLegendTab
//------------------

class QpLegendTab : public QpTab
{
	Q_OBJECT
	public:
		QpLegendTab(QTabWidget *tabWidget,  const QString& name, PanelItem * panel);
		QpLegendTab(QpLegendTab* tab, PanelItem* panel);
		~QpLegendTab(){;}
	
		PanelItem * I_Panel;
		QCheckBox *_show_legend_fld;
		QComboBox *_legend_border_fld;
		QComboBox *_legend_background_fld;
		QSpinBox *_legend_num_rows_fld;
		QSpinBox *_legend_num_cols_fld;
		FloatSpinBox *_legend_width_fld;
		FloatSpinBox *_legend_height_fld;
		QSpinBox *_legend_max_char_length_fld;
		FloatSpinBox *_legend_line_length_fld;
		FloatSpinBox *_legend_char_size_fld;
		QComboBox *_legend_just_fld;
		FontButton *_legend_font_fld;
		QComboBox *_legend_align_fld;
		FloatSpinBox *_legend_loffset_fld;
		FloatSpinBox *_legend_toffset_fld;
		QGroupBox* legend_box_groupbox;
		QGroupBox* legend_text_groupbox;
		QGroupBox* legend_position_groupbox;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
	public slots:
		void setShowLegend(bool);
		void setLegendBorder(int);
		void setLegendBackground(int);
		void setLegendNumRows(int);
		void setLegendNumCols(int);
		void setLegendWidth(double);
		void setLegendHeight(double);
		void setLegendMaxCharLength(int);
		void setLegendLineLength(double);
		void setLegendCharSize(double);
		void setLegendJust(int);
// 	void setLegendFont(int);
		void setLegendFamily(int);
		void setLegendStyle(bool);
		void setLegendWeight(bool);
		void setLegendAlign(int);
		void setLegendLoffset(double);
		void setLegendToffset(double);
		void updateNumTraces();
	protected:
		void create();
	
};


class QpColourscaleTab : public QpTab
{
	Q_OBJECT
	public:
		QpColourscaleTab(QTabWidget *tabWidget,  const QString& name, SpectrumItem * _spectro);
		QpColourscaleTab(QpColourscaleTab* tab, SpectrumItem* _spectro);
		~QpColourscaleTab(){;}

		SpectrumItem * I_Spectrum;
  
		QComboBox* _colourscale_auto_fld;
		QComboBox* _colourscale_linlog_fld;
		QguWLinput* _colourscale_min_fld;
		QguWLinput* _colourscale_max_fld;
		QCheckBox* _colourscale_show_colour_bar_fld;
		FloatSpinBox* _colourscale_colour_bar_width_fld;
		FloatSpinBox* _colourscale_colour_bar_height_fld;
		FloatSpinBox* _colourscale_colour_bar_voffset_fld;
		FloatSpinBox* _colourscale_colour_bar_disp_fld;
		FloatSpinBox* _colourscale_ticklength_fld;
        FloatSpinBox * _exp_label_offset_y_fld;
        FloatSpinBox * _exp_label_offset_x_fld;
        ColourTableButton* _colourtable_fld;
		QLineEdit* _colourscale_label_fld;
		QCheckBox* _colourscale_autolabel_fld;
	
		int colour_auto_last;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);


	public slots:
		void setColourScaleMin();
		void setColourScaleMax();
		void setColourScaleLabel();
		void setColourscaleMinMax();
		void setColourScaleAutoLabel();
		void setAutoScale(int);
		void setLinLog(int);
		void setColourScaleAutoLabelToggle(bool On);
		void setColourbarWidth(double width);
		void setColourbarVoffset(double offset);
		void setColourbarHeight(double height);
		void setColourbarDisplacement(double disp);
        void setColourtableStd(int choice);
        void setColourtableProfile(int choice);
        void setColourtableSession(int choice);
		void setShowColourBar(bool On);
		void setTicksLength(double);
        void setAxisExpLabelXOffset(double);
        void setAxisExpLabelYOffset(double);
	protected:
		void create();
	
};

// Function generating the dash styles
// ----------------------
void setDashStyle(QpLineStyle style, double scale=1.);

// Function getting the character height in normalized device coordinates
// ----------------------
PLFLT getCharHeight();

int getPGPColourIndex(const QColor&, bool reset=false);
PLINT symbolIndexConversion(int index);
string SafePGPString(const string&);
float roundf(float,short);

void print_iden();

// Plot Utility calls
//------------------
QpItemType QpRemapPlotType(int _type);


class QpuiColourScale
{
	public:
		bool _autoscale;
		bool _recalc_range;
		double _min;
		double _max;
		double _missing_value;
		int _linlog;
		bool _show_colour_bar;
		float _colour_bar_width;
		int _colour_table;
		float _colour_bar_disp;
		string _label;
		bool _autolabel;
		float _colour_bar_height;
		float _colour_bar_voffset;

};

// class  QpWindow
// -----------------------

class QpWindow : public QguWindow
{
	Q_OBJECT
 
			QWidget*    _top_widget;
	QGridLayout* _top_gl;
	QSplitter * _top_splitter;
	QStatusBar* _status_bar;
	QMenuBar *_menu_bar;
	PlotView* plotView;

	public slots:
		void status(const QString& str) {_status_bar->showMessage(str,2000);};
		void doPlot();
		void deleteAllItems();
        void saveAsXML();
        void restoreXML(bool replace=true);

		void launchManPlot();
		void launchManual();
		void launchVersion();

	signals:
        void refreshColourTableButtonsSignal();

	public:
		QpWindow( QWidget *parent=0);
		~QpWindow(){;}
		void save(Vostream&);
		void restore(Vistream&,bool replace=true);
		void restoreV22(Vistream& v_is, PlotView* view, bool replace);
		void resetDefaultBtn();
        bool restoreXML(QString path, bool replace=true);
        void saveAsXML(QString path);

		QMenuBar* menu_bar(){ return _menu_bar;}

		int _releaseVersion;
		int _majorVersion;
		int _minorVersion;

        void refreshColourTableButtons();

	protected:
		void keyPressEvent(QKeyEvent * e );
		QPushButton *plotBtn;
		QPushButton *dismissBtn;
};



void QPUImakeWindow(QWidget*,const char*);
void QpuiShow();
extern QpWindow *QPUI_widget;
QdObject_var makeIndexObject(QdObject_var zdo, int indx);

Vostream& operator<<(Vostream&, QpAxisTab&);
Vistream& operator>>(Vistream&, QpAxisTab&);

// interface to old (V2.2 and earlier) savesets
void restoreModel(Vistream& v_is, PlotModel *model, bool replace);
void restoreItem(Vistream& v_is, QpuiItem* item);
void restorePageItem(Vistream& v_is, PageItem* page);
bool restoreTSFrameItem(Vistream& v_is, FrameItem *frame);
void restoreXYFrameItem(Vistream& v_is, FrameItem *frame);
void restorePanelItem(Vistream& v_is, FrameItem *frame, PanelItem *panel);
void restoreAxis(Vistream& v_is, QpAxisTab *axis, bool keep, bool isXY);
void restoreFrameTabs(Vistream& v_is, FrameItem *frame);
void restoreLegend(Vistream& v_is, PanelItem *panel);
void restoreTSstack(Vistream& v_is, FrameItem *frame, PanelItem *panel);
void restoreXYstack(Vistream& v_is, FrameItem *frame, PanelItem *panel);
void restoreTrace(Vistream& v_is, TraceItem *trace);
void restoreEvent(Vistream& v_is, EventItem *event);
void restoreSpectroGram(Vistream& v_is, SpectrumItem *spectrum);


//////////////// ALBAN ////////////////////


// Vostream& operator<<(Vostream&, QpuiItem&);
Vostream& save(Vostream&, QpuiItem&); // strange compilation issue if defined as operator<<
Vistream& operator>>(Vistream&, QpuiItem&);

class QpuiItem: public QObject
{
	Q_OBJECT
	
// 	friend Vostream& operator<<(Vostream&, PageItem&);
			Vostream& save(Vostream&, QpuiItem&);
	friend Vistream& operator>>(Vistream&, QpuiItem&);
	friend class RootItem;
	friend class PageItem;
	friend class FrameItem;
	friend class PanelItem;
	friend class PlotItem;
	
	public:
		QpuiItem(const QString& name="", QpuiItem* parent=NULL);
		~QpuiItem()
		{
			clear();
			_parent=NULL;
		}
		
		virtual bool setName(const QString& newName);
        
		inline QString getName() const {return _name;}
		inline QString getPath() const {return _path;}
		
		inline QpuiItem* parent() const {return _parent;}
		
		inline bool hasChildren() const {return !_children.isEmpty();}
		inline int numChildren() const {return _children.size();}
		int childNumber();
				
		virtual bool isParentOf(QpuiItem* other);
		virtual bool isDescendentOf(QpuiItem* other);
		bool ancestorInList(QList <QpuiItem*> &list);
		
		QpuiItem* getChildByName(const QString& name) const;
		QpuiItem* getChildByPath(const QString& path) const;
		QpuiItem* getChildByNumber(int i) const;
		
		virtual bool removeChild(QpuiItem* child, bool alsoDelete);
		virtual bool removeChild(int i, bool alsoDelete);
		
		virtual bool addChild(QpuiItem* /*newChild*/, int position=-1){return false;}
		virtual bool reparent(QpuiItem* /*newParent*/, int position=-1){return false;}
		
		void refreshPath();
		
		void clear();
		
		virtual RootItem* root(){return NULL;}
		virtual QpuiItem* copyTo(QpuiItem* newParent){return NULL;}
		
		virtual void showTabs(){;}
		virtual void setDefaultOptions(){;}
		virtual QpFrameType frameType(){return QP_TS;}
		virtual void updateAxes(){;}

        virtual void addCustomColourTableTo(ColourTablePalette&);
        
		QpItemType I_Type;
		QTabWidget *tabWidget;
		QList<QpuiItem*> _children;

        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
		
	protected:
		QString _name;
		QString _path;
		QpuiItem* _parent;
		
		bool nameLock;

};

class RootItem: public QpuiItem
{
	friend class PageItem;
	friend class FrameItem;
	friend class PanelItem;
	friend class PlotItem;
	
	public:
		RootItem(PlotView* plotView);
		virtual ~RootItem();
		
		virtual bool isDescendentOf(QpuiItem*){return false;}
		virtual bool addChild(QpuiItem* newChild, int position=-1);
		
		PlotView* I_PlotView;
		
		RootItem* root(){return this;}


};

Vostream& operator<<(Vostream&, PageItem&);
Vistream& operator>>(Vistream&, PageItem&);
class PageItem: public QpuiItem
{
	Q_OBJECT
			friend Vostream& operator<<(Vostream&, PageItem&);
	friend Vistream& operator>>(Vistream&, PageItem&);
	friend class RootItem;
	friend class FrameItem;
	friend class PanelItem;
	friend class PlotItem;

	public:
		
		PageItem(const QString& name="", QpuiItem* parent=NULL, int position=-1);
		PageItem(PageItem* page, QpuiItem* parent, int position=-1);
		virtual ~PageItem();
		
        virtual bool setName(const QString&);
		
		virtual bool addChild(QpuiItem* newChild, int position=-1);
		virtual bool reparent(QpuiItem* newParent, int position=-1);
		
		QpuiReturnStatus plot(PlotWindow* win);
		void set_colour_table(const QpuiColourTable& colour_table=QPUI_RAINBOW);
        void updateIsotropicFrames();
		void setLayout();
		void showTabs();
		void setFramesAuto(bool active);
		void setDefaultOptions();
		void setSmooth(int id, bool smooth);
		PlotWindow* getWindow(int id);

        double aspectRatio() const;
        
		PlotWindow* newWindow(double width, double height, int* id,  int resolution=72, QPrinter::PageSize pageSize=QPrinter::A4, QPrinter::Orientation orientation=QPrinter::Portrait);
		PlotWindow* focusWindow(int id, double width=-1., double height=-1., int resolution=72, QPrinter::PageSize pageSize=QPrinter::A4, QPrinter::Orientation orientation=QPrinter::Portrait);
		void clearWindows();
		int panelsInColumn(int colno);
		
		int _print_id;
// 		string _filename;

		bool containsSpectrum();
		double _conversion_factors[5];
		static double _width_in_inches[6];
		static double _height_in_inches[6];
		
		QpValueReturnTab* ValueReturnTab;
		QpPageTab* PageTab;
		QpLayoutTab* LayoutTab;
		
		QList<PlotWindow*> PlotWindows;
		
		RootItem* root()
		{
			return dynamic_cast<RootItem*>(_parent);
		}

        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
		
	public slots:
		void displayIntervalWidget(double x, double y, int pos_x, int pos_y);
};

Vostream& operator<<(Vostream&, FrameItem&);
Vistream& operator>>(Vistream&, FrameItem&);
class FrameItem: public QpuiItem
{
	friend Vostream& operator<<(Vostream&, FrameItem&);
	friend Vistream& operator>>(Vistream&, FrameItem&);
	friend class RootItem;
	friend class PageItem;
	friend class PanelItem;
	friend class PlotItem;

	public:
		
		FrameItem(const QString& name="", QpuiItem* parent=NULL, QpFrameType frame_type=QP_TS, int position=-1);
		FrameItem(FrameItem* frame, QpuiItem* parent, int position=-1);
		~FrameItem();
		
		bool addChild(QpuiItem* newChild, int position=-1);
		bool reparent(QpuiItem* newParent, int position=-1);
		
		string getTitle();
		void showTabs();
		bool getXmetaRange(double& min, double& max);
		bool getXRange(double& min, double& max);
		void getYObjects();
		int numberInStack();
		QpuiReturnStatus plot(PlotWindow* win);
		void postPlot();
		QpFrameType frameType(){return _frameType;}
		void setPanelProperties(double loffset, double width, double toffset, double height);
		int panelNum(PanelItem *panelItem);
		void changeFrame(int frameType);
		bool containsSpectrum();
		void updateAxes();
		void setDefaultOptions();
		void updateIsotropic(bool resize = true);
		
		QpPropertiesTab* PropertiesTab;
		QpDisplayTab*   DisplayTab;
		QpTitleTab*     TitleTab;
		QpAxisTab * xtAxisTab; // pointer to x or t tab
		QpXAxisTab * xAxisTab;
		QpTAxisTab * tAxisTab;
		
		QColor _line_colour;
		QColor _title_colour;
		QpFrameType _frameType;

        virtual bool removeChild(QpuiItem* child, bool alsoDelete);
        virtual bool removeChild(int i, bool alsoDelete);
		
		inline PageItem* page()
		{
			return dynamic_cast<PageItem*>(_parent);
		}
		
		inline RootItem* root()
		{
			PageItem* pageItem=page();
			if(pageItem!=NULL) return pageItem->root();
			return NULL;
		}

        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
		
		bool restoring;

};

Vostream& operator<<(Vostream&, PanelItem&);
Vistream& operator>>(Vistream&, PanelItem&);
class PanelItem: public QpuiItem
{
	friend Vostream& operator<<(Vostream&, PanelItem&);
	friend Vistream& operator>>(Vistream&, PanelItem&);

	public:
		
		PanelItem(const QString& name="", QpuiItem* parent=NULL, int position=-1);
		PanelItem(PanelItem* panel, QpuiItem* parent, int position=-1);
		~PanelItem();
		
		bool addChild(QpuiItem* newChild, int position=-1);
		bool reparent(QpuiItem* newParent, int position=-1);
		
		void showTabs();
		int  getStackNumber();
		bool getXmetaRange(double& panel_xmin, double& panel_xmax);
		int  getXmetaIsLog();
		bool getXRange(double& panel_xmin, double& panel_xmax);
		void getYObjects();
		bool getYmetaRange(double& ymin, double& ymax);
		bool getYRange(double& _ymin, double& _ymax);
		void getLabel(const QpAxes& axis, string& label);
		int numberTraces();
		int numberSpectra();
		int numberThisSpectrum(SpectrumItem *item);
		QpuiReturnStatus plot(double _height, double _width, double _toffset, double _loffset, PlotWindow* win);
		void postPlot(bool show_top_axis, bool show_bottom_axis);
		void prePlot();
		string getTitle();
		QpFrameType frameType(){return frame()->frameType();}
		void updateYZRange();
		void updateAxes();
		void setDefaultOptions();
		bool isInterlaced();
		
        virtual bool removeChild(QpuiItem* child, bool alsoDelete);
        virtual bool removeChild(int i, bool alsoDelete);

		QpLegendTab* LegendTab;
		QpYAxisTab * yAxisTab;
		
		int   _legend_entry;
		float _legend_cell_width;
		float _legend_cell_height;
		
		inline FrameItem* frame()
		{
			if(_parent) return dynamic_cast<FrameItem*>(_parent);
			return NULL;
		}
		
		inline PageItem* page()
		{
			FrameItem* frameItem=frame();
			if(frameItem!=NULL) return frameItem->page();
			return NULL;
		}
		
		inline RootItem* root()
		{
			FrameItem* frameItem=frame();
			if(frameItem!=NULL) return frameItem->root();
			return NULL;
		}

        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
};

// Vostream& operator<<(Vostream&, PlotItem&);
// Vistream& operator>>(Vistream&, PlotItem&);
class PlotItem: public QpuiItem
{
// 	friend Vostream& operator<<(Vostream&, PlotItem&);
// 	friend Vistream& operator>>(Vistream&, PlotItem&);

	public:
		
		PlotItem(const QString& name="", QpuiItem* parent=NULL, const QString& WLcontext="", int position=-1);
		PlotItem(PlotItem* plot, QpuiItem* parent, int position=-1);
		
		virtual ~PlotItem();
		
		bool addChild(QpuiItem*, int){return false;}
		bool reparent(QpuiItem* newParent, int position=-1);
		
		void setLinkText(const QString& linkText){this->linkText=linkText;}
		QString getLinkText() const {return linkText;}
		
		virtual void changeFrameType(int frameType){;}
		QpFrameType frameType();
		virtual	QpuiReturnStatus plot(){return QPUI_FAILURE;}
		virtual void replaceData(QString obj_name){;}
		virtual void replaceComponent(QString comp){;}
		virtual void replaceTIdata(QString obj_name){;}
		virtual void replaceTIcomp(QString obj_name){;}
		virtual void replaceXData(QString obj_name){;}
		virtual void replaceXComponent(QString comp){;}
		virtual void clearObj_id(){;}
		virtual int getXmetaIsLog(){return 0;}
		virtual int getYmetaIsLog(){return 0;}
		virtual bool getXmetaRange(double& min, double& max){return false;}
		virtual bool getYmetaRange(double&, double&){return false;}
		virtual bool getXRange(double& min, double& max){return false;}
		virtual bool getYRange(double& ymin, double& ymax){return false;}
		virtual void get_dobjs(){;}
		void setLinks(QList<PlotItem*>& linkList);
		void removeLink(PlotItem* plot_item);
		QString WLcontext;

		QpDataTab * DataTab;
			
		int with;
		double missing_value;
		QList<PlotItem*> linkedPlots;
		
		inline PanelItem* panel()
		{
			return dynamic_cast<PanelItem*>(_parent);
		}
		
		inline FrameItem* frame()
		{
			PanelItem* panelItem=panel();
			if(panelItem!=NULL) return panelItem->frame();
			return NULL;
		}
		
		inline PageItem* page()
		{
			PanelItem* panelItem=panel();
			if(panelItem!=NULL) return panelItem->page();
			return NULL;
		}
		
		inline RootItem* root()
		{
			PanelItem* panelItem=panel();
			if(panelItem!=NULL) return panelItem->root();
			return NULL;
		}
		
	protected:
		QString linkText;
};

Vostream& operator<<(Vostream&, TraceItem&);
Vistream& operator>>(Vistream&, TraceItem&);
class TraceItem : public PlotItem
{
	friend Vostream& operator<<(Vostream&, TraceItem&);
	friend Vistream& operator>>(Vistream&, TraceItem&);
	
	public:

		TraceItem(const QString& name="", QpuiItem* parent=NULL, const QString& WLcontext="", QpItemType plotType=QP_TRACE, int position=-1);
		
		TraceItem(TraceItem* trace, QpuiItem* parent, int position=-1);
		
		~TraceItem();

        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
		
		int getXmetaIsLog();
		int getYmetaIsLog();
		bool getXmetaRange(double& min, double& max);
		bool getYmetaRange(double& min, double& max);
		bool getXRange(double& min, double& max);
		bool getYRange(double& ymin, double& ymax);
		void getLabel(const QpAxes& axis, string& label);
		void getLegendText(string& legend_text);
		string getTitle();
		void getFrameName(string& name); // replace getName(string& name)
		void changeFrameType(int frameType);
		void showTabs();
		QpuiReturnStatus plotLegend();
		QpuiReturnStatus plot();
		void replaceData(QString obj_name);
		void replaceComponent(QString comp);
		void replaceTIdata(QString comp);
		void replaceTIcomp(QString comp);
		void replaceXData(QString obj_name);
		void replaceXComponent(QString comp);
		void refreshTabs();
		void clearObj_id();
		void updateAxes();
		void get_dobjs();
		void setDefaultOptions();
		void updateColourBar();
		void replace_C_Component(QString comp);
		void replaceCdata(QString base);


		QpTraceTab * TraceTab;

		QdObject_var _ydo;
		QdObject_var _xdo;
		QdObject_var _cdo; // for colour points by extra variable c
		QdRScalarSeq_var linFit_var;
		
		int _x_obj_id_getXRange;
		int _y_obj_id_getYRange;
		double _x_range_min; // min used last y range
		double _x_range_max;   // max used last y range
		double _x_from; // min from data for autoscale
		double _x_to;   // max from data for autoscale
		double _y_from; // min from data for autoscale
		double _y_to;   // max from data for autoscale
		
		QColor _line_colour;
		QColor C1_colour;
		QColor C2_colour;
		QColor C3_colour;
		QColor C4_colour;
		QColor R_colour;
		QColor X_colour;
		QColor Y_colour;
		QColor Z_colour;
		QColor T_colour;
		QColor P_colour;
		QColor L_colour;
		QColor _symbol_colour;
		bool userLineColour;
		bool userSymbolColour;
};

Vostream& operator<<(Vostream&, EventItem&);
Vistream& operator>>(Vistream&, EventItem&);
class EventItem : public PlotItem
{
	friend Vostream& operator<<(Vostream&, EventItem&);
	friend Vistream& operator>>(Vistream&, EventItem&);
	
	public:

		EventItem(const QString& name="", QpuiItem* parent=NULL, const QString& WLcontext="", int position=-1);
		
		EventItem(EventItem* item, QpuiItem* parent, int position=-1);
		
		~EventItem();
		
		QpuiReturnStatus plot();
		void showTabs();

        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
		QpEventTab *eventTab;
		QColor _event_label_colour;
};

Vostream& operator<<(Vostream&, SpectrumItem&);
Vistream& operator>>(Vistream&, SpectrumItem&);
class SpectrumItem : public PlotItem
{
	friend Vostream& operator<<(Vostream&, SpectrumItem&);
	friend Vistream& operator>>(Vistream&, SpectrumItem&);

	public:

		SpectrumItem(const QString& name="", QpuiItem* parent=NULL, const QString& WLcontext="", int position=-1);
		SpectrumItem(SpectrumItem* item, QpuiItem* parent, int position=-1);
		
		~SpectrumItem();
		
		void getLabel(const QpAxes axis, string& label);
		int  getXData(QdObject_var xdo, QdObject_var xdo_end);
		int  getXData(QdObject_var xdo, QdObject_var xdo_plus, QdObject_var xdo_minus);
		void refreshX();
		int  getXData(QdObject_var xdo);
		int  getXData(QdTimeSeq_var xdo, int nSpectra, int nItem);
		int  getYData(QdObject_var ydo, QdObject_var ydo_end);
		int  getYData(QdObject_var ydo, QdObject_var ydo_plus, QdObject_var ydo_minus);
		int  getYData(QdObject_var ydo);
		int  isZConformal(int nx_len, int ny_len);
		string getTitle();
		void getSpectrogramName(string& name); // replaces getName(string&)
		int  getXmetaIsLog();
		int  getYmetaIsLog();
		bool getXRange(double&, double&);
		bool getXmetaRange(double&, double&);
		bool getYmetaRange(double&, double&);
		bool getYRange(double& ymin, double& ymax);
		bool getZRange(double &zmin, double &zmax);
		bool getZmetaRange(double& zmin,double& zmax);
		void updateAxes();
		void get_dobjs();
		void setDefaultOptions();

		QdObject_var getZDO();
		int  fill_X_Data();
		int  fill_Y_Data();
		int  fill_Z_Data(int XtoY, bool inPlot=true);
		void changeFrameType(int frameType);
		void showTabs();
		QpuiReturnStatus plot();
		void replaceData(QString obj_name);
		void replaceComponent(QString comp);
		void replaceTIdata(QString comp);
		void replaceTIcomp(QString comp);
		void refreshTabs();
		void clearObj_id();

        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);

        virtual void addCustomColourTableTo(ColourTablePalette&);
        
		QpColourscaleTab* ColourscaleTab;

		QdObject_var _zdo;
		int _z_obj_id;
		vector <dblBins> _z_data_all;
		QdObject_var _ydo;
		int _y_start_obj_id;
		vector <dblBins> _y_start_all;
		int _y_end_obj_id;
		vector <dblBins> _y_end_all;
		QdObject_var _xdo;
		int _x_start_obj_id;
		vector <double> _x_start_all;
		int _x_end_obj_id;
		vector <double> _x_end_all;
		QString y_data_start_str;
		QString y_data_end_str;
		QString x_data_start_str;
		QString x_data_end_str;
		int _nx_data;
		int _ny_data;
		bool XYtoggle_in_fill_x;
		bool XYtoggle_in_fill_y;
		int zdo_id_getZRange;
		int ydo_id_getYRange;
		int xdo_id_getXRange;
		
		double x_range_min;
		double x_range_max;
		double x_range_min_z;
		double x_range_max_z;
		double y_range_min_z;
		double y_range_max_z;
		double x_min_data;
		double x_max_data;
		double y_min_data;
		double y_max_data;
		double z_min_data;
		double z_max_data;
};

Vostream& operator<<(Vostream&, PlotModel&);
Vistream& operator>>(Vistream&, PlotModel&);
class PlotModel : public QAbstractItemModel
{
	Q_OBJECT
	
	friend Vostream& operator<<(Vostream&, PlotModel&);
	friend Vistream& operator>>(Vistream&, PlotModel&);
	
	public:
		PlotModel(PlotView* parent=0);
		
		~PlotModel();
		
		// Minimum set of functions
		
		QModelIndex index (int row, int column, const QModelIndex& parent=QModelIndex()) const;
		 
		QModelIndex parent(const QModelIndex& index) const;
		
		int rowCount(const QModelIndex& parent=QModelIndex()) const;
		
		int columnCount(const QModelIndex& parent=QModelIndex()) const;
		
		QVariant data(const QModelIndex& index, int role=Qt::DisplayRole) const;
		
		QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;

		// Editable
		
		Qt::ItemFlags flags(const QModelIndex &index=QModelIndex()) const;
		
		bool setData ( const QModelIndex & index, const QVariant & value, int role = Qt::EditRole );
		
		// Add/Remove rows/cols
		
// 		bool insertRows(int position, int rows, const QModelIndex& parent= QModelIndex());
		
		bool removeRows(int position, int rows, const QModelIndex& parent= QModelIndex());
		
		bool insertColumns(int position, int columns, const QModelIndex& parent= QModelIndex());
		
		bool removeColumns(int position, int columns, const QModelIndex& parent= QModelIndex());
		
		// Drag and drop tools
		
		QStringList mimeTypes() const;
		
		QMimeData* mimeData(const QModelIndexList& indexes) const;
		
		bool dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex &parent);
		
		// Utils
		
		inline QpuiItem* itemFromIndex(const QModelIndex& index) const
		{
			if(index.isValid())
			{
				return static_cast<QpuiItem*>(index.internalPointer());
			}
			return NULL;
		}
		
		QModelIndex indexFromItem(QpuiItem* item);
				
		inline QpuiItem* itemFromPath(const QString& path) const
		{
			return _root->getChildByPath(path);
		}
		
		void BIR(QpuiItem* parent, int position, int rows) // to be able to call BeginInsertRows from outside
		{
			QModelIndex index;
			if(parent==_root)
				index=createIndex(0, 0, parent);
			else
				index=createIndex(parent->childNumber(), 0, parent);
			beginInsertRows(index, position, position+rows-1);
		}
		
		void EIR() // to be able to call EndInsertRows from outside
		{
			endInsertRows();
		}
		
		PageItem* makePage();
		
		inline void emitLayoutAboutToBeChanged(){emit layoutAboutToBeChanged();}
		
		inline void emitLayoutChanged(){emit layoutChanged();}
		
/*		inline void clear(QModelIndex& parent)
		{
			if(!parent.isValid())
				return;
			
			QpuiItem* item=itemFromIndex(parent);
			if(item!=NULL && item->numChildren()>0)
			{
				removeRows(0, item->numChildren(), parent);
			}
		}*/
		void clear();
		
		void makeMultiTraceItem(QpuiItem* parent, QVector <int> &nTraces, QVector <TraceItem *> &stack, QpFrameType frameType);
		
		TraceItem* makeVector_RXYZ_Item(QpuiItem* parent, QpFrameType frameType=QP_TS, int row=-1);
		
		TraceItem* makeVector_R_XYZ_Item(QpuiItem* parent, QpFrameType frameType=QP_TS, int row=-1);
		
		TraceItem* makeVector_XYZ_Item(QpuiItem* parent, QpFrameType frameType=QP_TS, int row=-1);
		
		TraceItem* makeVector_R_TP_Item(QpuiItem* parent, QpFrameType frameType=QP_TS, int row=-1);
		
		TraceItem* makeVector_R_LP_Item(QpuiItem* parent, QpFrameType frameType=QP_TS, int row=-1);
		
		TraceItem* makeVector_R_X_Y_Z_Item(QpuiItem* parent, QpFrameType frameType=QP_TS, int row=-1);
		
		TraceItem* makeVector_X_Y_Z_Item(QpuiItem* parent, QpFrameType frameType=QP_TS, int row=-1);
		
		TraceItem* makeVector_R_T_P_Item(QpuiItem* parent, QpFrameType frameType=QP_TS, int row=-1);
		
		TraceItem* makeVector_R_L_P_Item(QpuiItem* parent, QpFrameType frameType=QP_TS, int row=-1);
		
		TraceItem* makeC1_2_3_4_Item(QpuiItem* parent, int row=-1);
		
		TraceItem* makeC1234_Item(QpuiItem* parent, int row=-1);
		
		PlotItem* makePlotItem(QpuiItem* parent, const QString& obj_name, int row=-1);
		
		PlotItem* makePlot(QpuiItem * parent, QpItemType plotType, QpFrameType frameType, const QString& WL, int row=-1);
		
		TraceItem* makeConstTrace(QpuiItem* parent, const QString& WL, int row, QpItemType plotType, QpFrameType frameType);
		
		TraceItem* makeTrace(QpuiItem* parent, const QString& WL, int row=-1);
		
		SpectrumItem* makeSpectrum(QpuiItem* parent, const QString& WL, int row=-1);
		
		EventItem* makeEvent(QpuiItem* parent, int row=-1);
		
		TraceItem* make2Panel_Item(QpuiItem* parent, int row=-1);
		TraceItem* make3Panel_Item(QpuiItem* parent, int row=-1);
		TraceItem* make4Panel_Item(QpuiItem* parent, int row=-1);
		
		FrameItem* makeTSFrameItem(QpuiItem* parent, int row=-1);
		FrameItem* makeXYFrameItem(QpuiItem* parent, int row=-1);

		TraceItem* makeTraceItem(QpuiItem* parent, int row=-1);
		SpectrumItem* makeSpectrumItem(QpuiItem* parent, int row=-1);
		PanelItem* makePanelItem(QpuiItem* parent, int row=-1);
		
		QModelIndex moveUp(QModelIndex& index);
		QModelIndex moveDown(QModelIndex& index);
		
		bool deleteItem(QpuiItem* item);
		void getLinkFlags(QList<QString>& io_list, QpuiItem* parent);
		
		void getLinkedItems(QList<PlotItem*>& io_list, QpuiItem* current, const QString& flag);
		
		void linkPlots(QList<PlotItem*>& linkedItems);
		
		void changePlotType(QModelIndex& index, QpItemType plotType);

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
        
		RootItem* _root;
		bool _replace;

        void doReset() {;}
		
	protected:
		
		int LinkNumber;
		void linkTraceItems(QVector<TraceItem *>& stack, bool onceOnly);
		PlotView* _view;
		
	public slots:
		void setPageLayout(PageItem *pageItem);
		
	signals:
		void pageLayoutChanged(PageItem*);
};

class PlotEditor: public QItemDelegate
{
	Q_OBJECT
	
	public:
		PlotEditor(QObject *parent = 0);

		QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;

		void setEditorData(QWidget *editor, const QModelIndex &index) const;
		
		void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;

		void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;
};

// Vostream& operator<<(Vostream&, PlotView&);
Vistream& operator>>(Vistream&, PlotView&);

class PlotView:public QTreeView
{
	// friend Vostream& operator<<(Vostream&, PlotView&);
	friend Vistream& operator>>(Vistream&, PlotView&);
	
	Q_OBJECT
	public:
		PlotView(QWidget* parent=NULL);
		
		PlotModel* plotModel(){return (PlotModel*)model();}
		
		QTabWidget *tabWidget;

 		QDomDocument *copyDoc;
		
		void plot();
		
		PageItem* makePage();
		
		void setCurrentItem(QpuiItem *i);
		
		inline QpuiItem* currentItem()
		{
			QModelIndex i=currentIndex();
			if(i.isValid())
				return plotModel()->itemFromIndex(currentIndex());
			return NULL;
		}
		
		QpTabType I_LastTab;
	public slots:
		void plotSelectedItems();
		void setDefaultSelectedItems();
		void updateOnSelection();
		void cut();
		void copy();
		void paste();
		void moveItemUp();
		void moveItemDown();
		void deleteSelectedItems();
		void deleteAllItems();
		void currentChanged(const QModelIndex& current, const QModelIndex& previous);

		inline void makeVector_RXYZ_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeVector_RXYZ_Item(curr);
			}
		}
		
		inline void makeVector_XYZ_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeVector_XYZ_Item(curr);
			}
		}
		
		inline void makeVector_R_XYZ_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeVector_R_XYZ_Item(curr);
			}
		}
		
		inline void makeVector_R_LP_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeVector_R_LP_Item(curr);
			}
		}
		
		inline void makeVector_R_TP_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeVector_R_TP_Item(curr);
			}
		}
		
		inline void makeVector_R_X_Y_Z_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeVector_R_X_Y_Z_Item(curr);
			}
		}
		
		inline void makeVector_X_Y_Z_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeVector_X_Y_Z_Item(curr);
			}
		}
		
		inline void makeVector_R_L_P_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeVector_R_L_P_Item(curr);
			}
		}
		
		inline void makeVector_R_T_P_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeVector_R_T_P_Item(curr);
			}
		}
		
		inline void makeC1_2_3_4_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeC1_2_3_4_Item(curr);
			}
		}
		
		inline void makeC1234_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeC1234_Item(curr);;
			}
		}
		
		inline void makeEventItem()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makePlot(curr, QP_EVENT, curr->frameType(), ""/*, curr->childNumber()*/);
			}
		}
		
		inline void makeConstYTraceItem()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makePlot(curr, QP_CONST_Y, curr->frameType(), ""/* curr->childNumber()*/);
			}
		}
		
		inline void makeConstXTraceItem()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makePlot(curr, QP_CONST_XT, curr->frameType(), ""/*, curr->childNumber()*/);
			}
		}

		inline void makeRegLineTraceItem()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makePlot(curr, QP_REG_LINE, curr->frameType(), ""/*, curr->childNumber()*/);
			}
		}
		
		inline void make2Panel_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->make2Panel_Item(curr);
			}
		}
		
		inline void make3Panel_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->make3Panel_Item(curr);
			}
		}
		
		inline void make4Panel_Item()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->make4Panel_Item(curr);
			}
		}
		
		inline void makePageItem()
		{
			plotModel()->makePage();
		}
		
		inline void makeTSFrameItem()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeTSFrameItem(curr);
			}
		}
		
		inline void makeXYFrameItem()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeXYFrameItem(curr);
			}
		}
		
		
		inline void makeTraceItem()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeTraceItem(curr);
			}
		}
		
		inline void makeSpectrumItem()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makeSpectrumItem(curr);
			}
		}
		
		inline void makePanelItem()
		{
			QpuiItem* curr=currentItem();
			if(curr!=NULL)
			{
				plotModel()->makePanelItem(curr);
			}
		}
		
		void setPageOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		
		void setFrameOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		
		void setTraceOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		
		void setEventOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		
		void setSpectrumOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		
		void setLegendOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		
		void setAxisOption(int option, int axis=T_AXIS, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		
		void setYRangeAllSelected();

		void setPlotOption(int option,  bool bChoice =false);
		
		void renameSelectedItem();
		
		void linkPlots();
		
		void unlinkPlots();
		
		void displayMenu(const QPoint& p);
		void mousePressEvent(QMouseEvent* event);
		
		void setPlotType(QAction *);
		void setFrameType(QAction*);
		
		void openObjectEditor(const QModelIndex& index);
		
		void expandChildren(const QModelIndex& i);
		
		QList<QPersistentModelIndex> selectedIndexes();
		QList<QpuiItem*> selectedItems();
		
	signals:
		void raiseWindow();
		
	protected slots:
		void dragEnterEvent(QDragEnterEvent *event);
		void dragMoveEvent(QDragMoveEvent *event);
		
		
	public:
		QList<PageItem*> _plot_page_list;
		bool isLink;
		PlotEditor* delegate;
		QMenu * RButtonMenu;
		int dropRow;
		
};

/////////////////////////////////////////////////



# endif // QSAS_PLOT_UI_H_
