// qpui_frame_options.cc
//
// Plot Interface Implementation - Frame Options
//
// Tony Allen
// Apr 2008

#include "qpui.h"
#include "qdutil.h"

#include <qcolordialog.h>
#include <qtooltip.h>

extern QPalette QPUI_normal_palette;


//  QpPropertiesTab Class
// -----------------------

QpPropertiesTab::QpPropertiesTab(QTabWidget *tabWidget,  const QString& name, FrameItem * _frame):
		QpTab(tabWidget, name, QP_POSITION_TAB)
{
	// Constructor for the Properties Tab widget shown in the Panel Options

	I_Frame = _frame;

	create();
	
	connect( _loffset_fld, SIGNAL(valueChanged(double)), this, SLOT(setLOffset(double)) );
	connect( _width_fld, SIGNAL(valueChanged(double)), this, SLOT(setWidth(double)) );
	connect( _toffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setTOffset(double)) );
	connect( _height_fld, SIGNAL(valueChanged(double)), this, SLOT(setHeight(double)) );
    connect( _aspect_ratio_fld, SIGNAL(currentIndexChanged(int)), this, SLOT(setAspectRatio(int)) );

}

QpPropertiesTab::QpPropertiesTab(QpPropertiesTab* tab, FrameItem* _frame):
		QpTab((QTabWidget*)(tab->parent()), "Properties", QP_POSITION_TAB)
{
	I_Frame = _frame;

	create();
	
	_loffset_fld->setValue(tab->_loffset_fld->value());
	_width_fld->setValue(tab->_width_fld->value());
	_toffset_fld->setValue(tab->_toffset_fld->value());
	_height_fld->setValue(tab->_height_fld->value());
    _aspect_ratio_fld->setCurrentIndex(tab->_aspect_ratio_fld->currentIndex());
	
	connect( _loffset_fld, SIGNAL(valueChanged(double)), this, SLOT(setLOffset(double)) );
	connect( _width_fld, SIGNAL(valueChanged(double)), this, SLOT(setWidth(double)) );
	connect( _toffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setTOffset(double)) );
	connect( _height_fld, SIGNAL(valueChanged(double)), this, SLOT(setHeight(double)) );
    connect( _aspect_ratio_fld, SIGNAL(currentIndexChanged(int)), this, SLOT(setAspectRatio(int)) );
}

void QpPropertiesTab::create()
{
	QVBoxLayout *panel_properties_top_layout = new QVBoxLayout(this);
	panel_properties_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_properties_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	
	QGroupBox *panel_position_groupbox = new QGroupBox(this);

	panel_position_groupbox->setTitle("Frame Position");

// 	QWidget* panel_position_widget = new QWidget(panel_position_groupbox, "panel_position_widget");

	QGridLayout *panel_position_layout = new QGridLayout;
	panel_position_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_position_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_autolayout_lbl = new QLabel("Autoset by Page Layout ");
	panel_position_layout->addWidget(_autolayout_lbl, 1, 0, 1, 2);

	QLabel* loffset_label = new QLabel("Left offset: ");
	panel_position_layout->addWidget(loffset_label, 3, 0, Qt::AlignRight);
	_loffset_fld = new FloatSpinBox(0., 1., 2, .05, panel_position_groupbox, "loffset");
	_loffset_fld->setFixedWidth(100);
	_loffset_fld->setFocusPolicy(Qt::StrongFocus);
	panel_position_layout->addWidget(_loffset_fld, 3, 1, Qt::AlignLeft);
	_loffset_fld->setValue(QPUI_PANEL_LOFFSET);
	
	
	QLabel* width_label = new QLabel("Width: ");
	panel_position_layout->addWidget(width_label, 3, 2, Qt::AlignRight);
	_width_fld = new FloatSpinBox(0.05, 1., 2, .05, panel_position_groupbox, "width");
	_width_fld->setFixedWidth(100);
	_width_fld->setFocusPolicy(Qt::StrongFocus);
	panel_position_layout->addWidget(_width_fld, 3, 3, Qt::AlignLeft);
	_width_fld->setValue(QPUI_PANEL_WIDTH);

	QLabel* toffset_label = new QLabel("Top offset: ");
	panel_position_layout->addWidget(toffset_label, 5, 0, Qt::AlignRight);
	_toffset_fld = new FloatSpinBox(0., 1., 2,.05, panel_position_groupbox, "toffset");
	_toffset_fld->setFixedWidth(100);
	_toffset_fld->setFocusPolicy(Qt::StrongFocus);
	panel_position_layout->addWidget(_toffset_fld, 5, 1, Qt::AlignLeft);
	_toffset_fld->setValue(QPUI_PANEL_TOFFSET);

	QLabel* height_label = new QLabel("Height: ");
	panel_position_layout->addWidget(height_label, 5, 2, Qt::AlignRight);
	_height_fld = new FloatSpinBox(0.05, 1., 2, .05, panel_position_groupbox, "height");
	_height_fld->setFixedWidth(100);
	_height_fld->setFocusPolicy(Qt::StrongFocus);
	panel_position_layout->addWidget(_height_fld, 5, 3, Qt::AlignLeft);
	_height_fld->setValue(QPUI_PANEL_HEIGHT);

	_aspect_ratio = QPUI_PANEL_HEIGHT / QPUI_PANEL_WIDTH; // initialise aspect ratio
    panel_position_layout->addWidget(new QLabel("Aspect ratio:"), 7, 0, Qt::AlignRight);
    _aspect_ratio_fld=new QComboBox;
    _aspect_ratio_fld->addItem("Free");
    _aspect_ratio_fld->addItem("Fixed");
    _aspect_ratio_fld->addItem("Square");
    _aspect_ratio_fld->addItem("Isotropic");
	panel_position_layout->addWidget(_aspect_ratio_fld, 7, 1);

	if(I_Frame->page()->LayoutTab->_autolayout_fld->isChecked()) {
		_loffset_fld->setDisabled(true);
		_width_fld->setDisabled(true);
		_toffset_fld->setDisabled(true);
		_height_fld->setDisabled(true);
		_aspect_ratio_fld->setDisabled(true);
	}
	
	panel_position_groupbox->setLayout(panel_position_layout);

	panel_properties_top_layout->addWidget(panel_position_groupbox);

	panel_properties_top_layout->addSpacing(4);
	panel_properties_top_layout->addStretch(10);

// 	panel_position_widget->adjustSize();
}

void QpPropertiesTab::setLOffset(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(LOFFSET, 0, fChoice);
}

void QpPropertiesTab::setTOffset(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TOFFSET, 0, fChoice);
}

double QpPropertiesTab::setIsotropicRatio()
{
	// set default if no data available yet
	_aspect_ratio = I_Frame->page()->aspectRatio();
	
	// get y range for panels

	double y_range = 0.;
	PanelItem* panelItem;
	for(int i=0; i<I_Frame->numChildren(); ++i)
	{
		panelItem=dynamic_cast<PanelItem*>(I_Frame->getChildByNumber(i));
		double  item_from = panelItem->yAxisTab->_axis_xy_fld->getStart();
		double  item_to = panelItem->yAxisTab->_axis_xy_fld->getEnd();
		
		y_range += fabs(item_to - item_from);

	}
	_isotropic_y_range = y_range;
	if( y_range < 1.e-10 ) return _aspect_ratio;

	// get x range
	double xmin=1.e30;
	double xmax=-1.e30;
	if(I_Frame->frameType()==QP_TS)
	{	
		xmin = I_Frame->tAxisTab->_axis_time_fld->getStart().getEpoch2000();
		xmax = I_Frame->tAxisTab->_axis_time_fld->getEnd().getEpoch2000();
	}
	else if(I_Frame->frameType()==QP_XY)
	{
		xmin = I_Frame->xAxisTab->_axis_xy_fld->getStart();
		xmax = I_Frame->xAxisTab->_axis_xy_fld->getEnd();
	}
	if( xmax - xmin < 1.e-10 ) return _aspect_ratio;

	_aspect_ratio *= y_range / (xmax - xmin);
	return _aspect_ratio;
}


void QpPropertiesTab::setWidth(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   // Width is entered in field, adjust height accordingly
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(WIDTH, 0, fChoice);

    double page_ratio;
    switch(this->_aspect_ratio_fld->currentIndex())
    {
        case QPUI_FIXED:
            _height_fld->setValue(fChoice*_aspect_ratio);
            break;

        case QPUI_SQUARE_ASPECT:
		{
            page_ratio=I_Frame->page()->aspectRatio();
			int stackNum = I_Frame->numberInStack();
			double newHeight = fChoice*page_ratio*stackNum;
			double t_offset = _toffset_fld->value();
			if(newHeight + t_offset > 0.99)
			{
				newHeight = 0.99 - t_offset;
				double newWidth = newHeight / page_ratio / stackNum;
				_width_fld->blockSignals(true);
				_width_fld->setValue(newWidth);
				_width_fld->blockSignals(false);
			}
			_height_fld->setValue(newHeight);
            break;
		}
        case QPUI_ISOTROPIC:
		{
		    setIsotropicRatio();
			double newHeight = fChoice*_aspect_ratio;
			double t_offset = _toffset_fld->value();
			if(newHeight + t_offset > 0.99)
			{
				newHeight = 0.99 - t_offset;
				double newWidth = newHeight / _aspect_ratio;
				_width_fld->blockSignals(true);
				_width_fld->setValue(newWidth);
				_width_fld->blockSignals(false);
			}
            _height_fld->setValue(newHeight);
            break;
		}
        default:
            _aspect_ratio = _height_fld->value()/fChoice;
            break;
    }
//     if(this->_aspect_ratio_fld->isChecked()) _height_fld->setValue(fChoice*_aspect_ratio);
//     else _aspect_ratio = _height_fld->value()/fChoice;

}

void QpPropertiesTab::setHeight(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   // height is set in field, adjust width accordingly
   
    if( this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(HEIGHT, 0, fChoice);

    double page_ratio;
    switch(this->_aspect_ratio_fld->currentIndex())
    {
        case QPUI_FIXED:
            _width_fld->setValue(fChoice/_aspect_ratio);
            break;

        case QPUI_SQUARE_ASPECT:
		{
            page_ratio=I_Frame->page()->aspectRatio();
			int stackNum = I_Frame->numberInStack();
			double newWidth = fChoice/page_ratio/stackNum;
			double l_offset = _loffset_fld->value();
			if(newWidth + l_offset > 0.99)
			{
				newWidth = 0.99 - l_offset;
				double newHeight = newWidth * page_ratio * stackNum;
				_height_fld->blockSignals(true);
				_height_fld->setValue(newHeight);
				_height_fld->blockSignals(false);
			}
            _width_fld->setValue(newWidth);
            break;
		}
        case QPUI_ISOTROPIC:
		{
  			setIsotropicRatio();
			double newWidth = fChoice/_aspect_ratio;
			double l_offset = _loffset_fld->value();
			if(newWidth + l_offset > 0.99)
			{
				newWidth = 0.99 - l_offset;
				double newHeight = newWidth * _aspect_ratio;
				_height_fld->setValue(newHeight);
			}
            _width_fld->setValue(newWidth);
            break;
		}
        default:
            _aspect_ratio = fChoice/_width_fld->value();
                    break;
    }
}

void QpPropertiesTab::setAspectRatio(int iChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   // try to set height, keeping width. If too large, reset width as well.
   
    double page_ratio;
    if(this->isVisible() )
    {
        I_Frame->root()->I_PlotView->setFrameOption(ASPECT_RATIO, iChoice);
		
        if(iChoice==QPUI_SQUARE_ASPECT)
        {
            page_ratio=I_Frame->page()->aspectRatio();
			int stackNum = I_Frame->numberInStack();
			double newHeight = _width_fld->value()*page_ratio*stackNum;
			double t_offset = _toffset_fld->value();
			if(newHeight + t_offset > 0.99)
			{
				newHeight = 0.99 - t_offset;
				double newWidth = newHeight / page_ratio / stackNum;
				_width_fld->blockSignals(true);
				_width_fld->setValue(newWidth);
				_width_fld->blockSignals(false);
			}
			_height_fld->setValue(newHeight);

        }
        if(iChoice==QPUI_ISOTROPIC)
        {
		    setIsotropicRatio();
			double newHeight = _width_fld->value()*_aspect_ratio;
			double t_offset = _toffset_fld->value();
			if(newHeight + t_offset > 0.95)
			{
				newHeight = 0.95 - t_offset;
				double newWidth = newHeight / _aspect_ratio;
				_width_fld->blockSignals(true);
				_width_fld->setValue(newWidth);
				_width_fld->blockSignals(false);
			}
            _height_fld->setValue(newHeight);

        }
    }
    
}

QDomElement QpPropertiesTab::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Properties_Tab");
    xml.appendChild(createTextXMLNode(doc, "Aspect_Ratio", _aspect_ratio_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Left_Offset", _loffset_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Top_Offset", _toffset_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Width", _width_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Height", _height_fld->value()));

    return xml;
}

void QpPropertiesTab::fromXMLElement(QDomElement& el)
{
    QString s;
    if(getXMLNodeText(el, "Aspect_Ratio", s))
    {
        _aspect_ratio_fld->setCurrentIndex(s.toInt());
    }
    if(getXMLNodeText(el, "Left_Offset", s))
    {
        _loffset_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Top_Offset", s))
    {
        _toffset_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Width", s))
    {
        _width_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Height", s))
    {
        _height_fld->setValue(s.toDouble());
    }

}



// QpTitleTab class
// ----------------

QpTitleTab::QpTitleTab(QTabWidget *tabWidget,  const QString& name, FrameItem * _frame):
		QpTab(tabWidget, name, QP_TITLE_TAB)
{  
	// Constructor for the Title Tab widget shown in the Panel Options

	I_Frame = _frame;
	
	create();
	
	connect( _autotitle_fld, SIGNAL(toggled(bool)), this, SLOT(setAutoTitle(bool)) );
	connect( _title_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setTitle(const QString&)) );
	_title_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_TITLE_CHAR_SIZE_ADD);
	connect(_title_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setTitleCharSize(double)));
	connect(_title_disp_fld,SIGNAL(valueChanged(double)),this,SLOT(setTitleDisp(double)));
	connect( _subtitle_fld, SIGNAL(textChanged(const QString &)), this, SLOT(setSubTitle(const QString&)) );

	connect(_subtitle_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setSubTitleCharSize(double)));
	_subtitle_disp_fld->setValue(QPUI_SUBTITLE_DISP);
	connect(_subtitle_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setSubTitleDisp(double)));
	connect( _title_just_fld, SIGNAL(activated(int)),this, SLOT(setTitleJust(int)) );
	connect( _title_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setTitleFamily(int)) );
	connect( _title_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setTitleStyle(bool)) );
	connect( _title_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setTitleWeight(bool)) );
	connect(_title_colour_fld, SIGNAL(clicked()), this, SLOT(setTitleColour()));
}

QpTitleTab::QpTitleTab(QpTitleTab* tab, FrameItem* _frame):
		QpTab((QTabWidget*)(tab->parent()), "Display", QP_TITLE_TAB)
{
	I_Frame = _frame;
	
	create();
	
	_autotitle_fld->setChecked(tab->_autotitle_fld->isChecked());
	_title_fld->setText(tab->_title_fld->text());
	_title_char_size_fld->setValue(tab->_title_char_size_fld->value());
	_title_disp_fld->setValue(tab->_title_disp_fld->value());
	_subtitle_fld->setText(tab->_subtitle_fld->text());
	_subtitle_char_size_fld->setValue(tab->_subtitle_char_size_fld->value());
	_subtitle_disp_fld->setValue(tab->_subtitle_disp_fld->value());
	_title_just_fld->setCurrentIndex(tab->_title_just_fld->currentIndex());
	PLINT family, style, weight;
	tab->_title_font_fld->getFontParams(family, style, weight);
	_title_font_fld->setFontParams(family, style, weight);
	
	connect( _autotitle_fld, SIGNAL(toggled(bool)), this, SLOT(setAutoTitle(bool)) );
	connect( _title_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setTitle(const QString&)) );
	connect(_title_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setTitleCharSize(double)));
	connect(_title_disp_fld,SIGNAL(valueChanged(double)),this,SLOT(setTitleDisp(double)));
	connect( _subtitle_fld, SIGNAL(textChanged(const QString &)), this, SLOT(setSubTitle(const QString&)) );
	connect(_subtitle_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setSubTitleCharSize(double)));
	connect(_subtitle_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setSubTitleDisp(double)));
	connect( _title_just_fld, SIGNAL(activated(int)),this, SLOT(setTitleJust(int)) );

	connect( _title_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setTitleFamily(int)) );
	connect( _title_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setTitleStyle(bool)) );
	connect( _title_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setTitleWeight(bool)) );
	connect(_title_colour_fld, SIGNAL(clicked()), this, SLOT(setTitleColour()));
}

void QpTitleTab::create()
{
	QVBoxLayout *panel_title_top_layout=new QVBoxLayout(this);
	panel_title_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_title_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *panel_title_groupbox = new QGroupBox(this);

	panel_title_groupbox->setTitle("Title");

	QGridLayout *panel_title_layout = new QGridLayout;
	panel_title_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_title_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_autotitle_fld = new QCheckBox("Autotitle");
	_autotitle_fld->setChecked(true);
	panel_title_layout->addWidget(_autotitle_fld, 1, 0, Qt::AlignLeft);

	QLabel* text_label = new QLabel("Text: ");
	panel_title_layout->addWidget(text_label, 3, 0, Qt::AlignRight);
	_title_fld = new QLineEdit;
	_title_fld->setMinimumWidth(500);
	panel_title_layout->addWidget(_title_fld, 3, 1, 1, 5, Qt::AlignLeft);
	setSlotColour(false);
	_title_fld->setPalette(GUIpalette);

	QLabel* title_char_size_label = new QLabel("Character size: ");
	panel_title_layout->addWidget(title_char_size_label, 5, 0, Qt::AlignRight);
	_title_char_size_fld = new FloatSpinBox(0.01, 40., 1, .1, panel_title_groupbox, "title_char_size");
	_title_char_size_fld->setFixedWidth(100);
	panel_title_layout->addWidget(_title_char_size_fld, 5, 1, Qt::AlignLeft);
	_title_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_TITLE_CHAR_SIZE_ADD);

	QLabel* title_disp_label = new QLabel("Displacement: ");
	panel_title_layout->addWidget(title_disp_label, 5, 2, Qt::AlignRight);
	_title_disp_fld = new FloatSpinBox(0., 40., 1, .5, panel_title_groupbox, "title_disp");
	_title_disp_fld->setFixedWidth(100);
	panel_title_layout->addWidget(_title_disp_fld, 5, 3, Qt::AlignLeft);
	_title_disp_fld->setValue(QPUI_TITLE_DISP);

	QGroupBox *panel_subtitle_groupbox = new QGroupBox(panel_title_groupbox);

	panel_subtitle_groupbox->setTitle("Subtitle");

	QGridLayout *panel_subtitle_layout = new QGridLayout;
	panel_subtitle_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_subtitle_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* subtitle_text_label = new QLabel("Text: ");
	panel_subtitle_layout->addWidget(subtitle_text_label, 1, 0, Qt::AlignRight);
	_subtitle_fld = new QLineEdit;
	_subtitle_fld->setMinimumWidth(350);
	panel_subtitle_layout->addWidget(_subtitle_fld, 1, 1, 1, 4, Qt::AlignLeft);

	QLabel* subtitle_char_size_label = new QLabel("Character size: ");
	panel_subtitle_layout->addWidget(subtitle_char_size_label, 3, 0, Qt::AlignRight);
	_subtitle_char_size_fld = new FloatSpinBox(0.01, 40., 1, .1, panel_subtitle_groupbox, "subtitle_char_size");
	_subtitle_char_size_fld->setFixedWidth(80);
	panel_subtitle_layout->addWidget(_subtitle_char_size_fld, 3, 1, Qt::AlignLeft);
	_subtitle_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_SUBTITLE_CHAR_SIZE_ADD);
	
	QLabel* subtitle_disp_label = new QLabel("Displacement: ");
	panel_subtitle_layout->addWidget(subtitle_disp_label, 3, 2, Qt::AlignRight);
	_subtitle_disp_fld = new FloatSpinBox(-10., 40., 1, .2, panel_subtitle_groupbox, "subtitle_disp");
	_subtitle_disp_fld->setFixedWidth(80);
	panel_subtitle_layout->addWidget(_subtitle_disp_fld, 3, 3, Qt::AlignLeft);
	_subtitle_disp_fld->setValue(QPUI_SUBTITLE_DISP);
	
	panel_subtitle_groupbox->setLayout(panel_subtitle_layout);

	panel_title_layout->addWidget(panel_subtitle_groupbox, 7, 0, 5, 5);
	
	QLabel* justification_label = new QLabel("Justification: ");
	panel_title_layout->addWidget(justification_label, 13, 0, Qt::AlignRight);
	_title_just_fld = new QComboBox();
	_title_just_fld->insertItem(QPUI_HCENTRE, "centre");
	_title_just_fld->insertItem(QPUI_LEFT, "left");
	_title_just_fld->insertItem(QPUI_RIGHT, "right");
	QFontMetrics title_just_fm(_title_just_fld->font());
	int title_just_w = title_just_fm.width("centre");
	int title_just_h = title_just_fm.height();
	_title_just_fld->setMinimumSize(title_just_w, title_just_h);

	_title_just_fld->setToolTip("Select title justification");
	panel_title_layout->addWidget(_title_just_fld, 13, 1, Qt::AlignLeft);

	QLabel* font_label = new QLabel("Font: ");
	panel_title_layout->addWidget(font_label, 15, 0, Qt::AlignRight);
	_title_font_fld = new FontButton;

	_title_font_fld->setToolTip("Select title font");
	panel_title_layout->addWidget(_title_font_fld, 15, 1, Qt::AlignLeft);

	QLabel* colour_label = new QLabel("Colour: ");
	panel_title_layout->addWidget(colour_label, 15, 2, Qt::AlignRight);
	_title_colour_fld = new QPushButton("Colour...");
	_title_colour_fld->setFixedWidth(100);
	_title_colour_fld->setFixedHeight(30);
	QFont f = QFont(qApp->font());
	f.setBold(TRUE);
	_title_colour_fld->setFont(f);
	panel_title_layout->addWidget(_title_colour_fld, 15, 3, Qt::AlignLeft);

	panel_title_groupbox->setLayout(panel_title_layout);
	
	panel_title_top_layout->addWidget(panel_title_groupbox);

	panel_title_top_layout->addSpacing(4);
	panel_title_top_layout->addStretch(10);
// 	panel_title_widget->adjustSize();
}

void QpTitleTab::setAutoTitle(bool bChoice)
{
	if ( QuSlotDebug() )
	{
		printf( "QpTitleTab::setAutoTitle \n" );
		fflush(stdout);
	}

	if(bChoice) 
	{
		string title = I_Frame->getTitle();
		_title_fld->clearFocus(); // ensure field doesn't try to turn autotitle off
		_title_fld->setText(title.c_str());

	}
	setSlotColour(!bChoice);
	_title_fld->setPalette(GUIpalette);

	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(AUTOTITLE, 0, 0.0, "", bChoice);
	
}

void QpTitleTab::setTitle(const QString &title)
{
	if(_title_fld->hasFocus())
	{
		// only change if user input
		_autotitle_fld->setChecked(false);
		
		// only tell list view to make changes to selected panel items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
 		if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TITLE, 0, 0.0, title);

	}	
}

void QpTitleTab::setTitleCharSize(double fChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TITLE_CHAR_SIZE, 0, fChoice);

}

void QpTitleTab::setTitleDisp(double fChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TITLE_DISP, 0, fChoice);

}

void QpTitleTab::setSubTitle(const QString &subTitle)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TITLE_DISP, 0, 0.0, subTitle);

}

void QpTitleTab::setSubTitleCharSize(double fChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(SUBTITLE_CHAR_SIZE, 0, fChoice);

}

void QpTitleTab::setSubTitleDisp(double fChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(SUBTITLE_DISP, 0, fChoice);

}

void QpTitleTab::setTitleJust(int iChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TITLE_JUST, iChoice);

}

void QpTitleTab::setTitleFamily(int iChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)

	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TITLE_FONT_FAMILY, iChoice);
}

void QpTitleTab::setTitleStyle(bool bChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TITLE_FONT_STYLE, 0, 0., "", bChoice);
}

void QpTitleTab::setTitleWeight(bool bChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TITLE_FONT_WEIGHT, 0, 0., "", bChoice);
}

void QpTitleTab::setTitleColour()
{
	bool changed = false;
	// ask the user for colour choice
	QColor c = QuGetColour(I_Frame->_title_colour, &changed, this);
	if(changed)
	{
		QPalette line_palette = QPalette(QPUI_normal_palette);
        line_palette.setColor(QPalette::Active ,QPalette::ButtonText, c);
        line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, c);
        this->_title_colour_fld->setPalette(line_palette);
        I_Frame->_title_colour = c;
	}

	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TITLE_COLOUR, 0, 0., "", false, c);
}

QDomElement QpTitleTab::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Title_Tab");
    xml.appendChild(createTextXMLNode(doc, "AutoTitle", _autotitle_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Title", _title_fld->text()));
    xml.appendChild(createTextXMLNode(doc, "Title_Char_Size", _title_char_size_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Title_Displacement", _title_disp_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Subtitle", _subtitle_fld->text()));
    xml.appendChild(createTextXMLNode(doc, "Subtitle_Char_Size", _subtitle_char_size_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Subtitle_Displacement", _subtitle_disp_fld->value()));

    xml.appendChild(createTextXMLNode(doc, "Title_Justification", _title_just_fld->currentIndex()));
    
    PLINT family, style, weight;
    _title_font_fld->getFontParams(family, style, weight);
    xml.appendChild(createTextXMLNode(doc, "Title_Font_Family", family));
    xml.appendChild(createTextXMLNode(doc, "Title_Font_Style", style));
    xml.appendChild(createTextXMLNode(doc, "Title_Font_Weight", weight));

    return xml;
}

void QpTitleTab::fromXMLElement(QDomElement& el)
{
    QString s;
    if(getXMLNodeText(el, "AutoTitle", s))
    {
        _autotitle_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Title", s))
    {
        _title_fld->setText(s);
    }
    if(getXMLNodeText(el, "Title_Char_Size", s))
    {
        _title_char_size_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Title_Displacement", s))
    {
        _title_disp_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Subtitle", s))
    {
        _subtitle_fld->setText(s);
    }
    if(getXMLNodeText(el, "Subtitle_Char_Size", s))
    {
        _subtitle_char_size_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Subtitle_Displacement", s))
    {
        _subtitle_disp_fld->setValue(s.toDouble());
    }

    if(getXMLNodeText(el, "Title_Justification", s))
    {
        _title_just_fld->setCurrentIndex(s.toInt());
    }

    PLINT family(PL_FCI_SANS), style(0), weight(0);
    if(getXMLNodeText(el, "Title_Font_Family", s))
    {
        family=s.toInt();
    }
    if(getXMLNodeText(el, "Title_Font_Style", s))
    {
        style=s.toInt();
    }
    if(getXMLNodeText(el, "Title_Font_Weight", s))
    {
        weight=s.toInt();
    }
    switch(family)
    {
        case PL_FCI_SERIF: _title_font_fld->setSerifFont(); break;
        case PL_FCI_MONO: _title_font_fld->setMonotypeFont(); break;
        case PL_FCI_SCRIPT: _title_font_fld->setDecorativeFont(); break;
        default: _title_font_fld->setSansFont(); break;
    }
    _title_font_fld->setItalic(style==PL_FCI_ITALIC);
    _title_font_fld->setBold(weight==PL_FCI_BOLD);
}


//  QpDisplayTab Class
// ---------------------
QpDisplayTab::QpDisplayTab(QTabWidget *tabWidget,  const QString& name, FrameItem * _frame):
		QpTab(tabWidget, name, QP_FRAME_TAB)

{
	// Constructor for the Display Tab widget shown in the Panel Options

	I_Frame = _frame;

	create();
	
	connect(_line_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setLineWidth(double)));
	connect(_line_colour_fld, SIGNAL(clicked()), this, SLOT(setLineColour()));
	connect(_ticks_length_fld, SIGNAL(valueChanged(double)), this, SLOT(setTicksLength(double)));
	_ticks_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LABELS_CHAR_SIZE_ADD);
	connect(_ticks_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setTicksCharSize(double)));
	connect( _show_top_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setTopAxis(bool)) );
	connect( _show_bottom_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomAxis(bool)) );
	connect( _show_left_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftAxis(bool)) );
	connect(_show_right_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setRightAxis(bool)) );
	connect(_show_lr_alternating_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingAxis(bool)) );
	connect(_show_rl_alternating_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingAxis(bool)) );
	connect( _show_top_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setTopTicksLabels(bool)) );
	connect( _show_bottom_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomTicksLabels(bool)) );
	connect( _show_left_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftTicksLabels(bool)) );
	connect( _show_right_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRightTicksLabels(bool)) );
	connect( _show_lr_alternating_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingTicksLabels(bool)) );
	connect( _show_rl_alternating_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingTicksLabels(bool)) );
	connect( _show_top_labels_fld, SIGNAL(toggled(bool)), this,SLOT(setTopLabels(bool)) );
	connect( _show_bottom_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomLabels(bool)) );
	connect( _show_left_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftLabels(bool)) );
	connect( _show_right_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRightLabels(bool)) );
	connect( _show_lr_alternating_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingLabels(bool)) );
	connect( _show_rl_alternating_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingLabels(bool)) );
	connect( _show_xiszero_fld, SIGNAL(toggled(bool)), this, SLOT(setXIsZero(bool)) );
	connect( _show_yiszero_fld, SIGNAL(toggled(bool)), this, SLOT(setYIsZero(bool)) );

}

QpDisplayTab::QpDisplayTab(QpDisplayTab* tab, FrameItem* _frame):
		QpTab((QTabWidget*)(tab->parent()), "Display", QP_FRAME_TAB)
{
	I_Frame = _frame;

	create();
	
	_line_width_fld->setValue(tab->_line_width_fld->value());
	_ticks_length_fld->setValue(tab->_ticks_length_fld->value());
	_ticks_char_size_fld->setValue(tab->_ticks_char_size_fld->value());
	_show_top_axis_fld->setChecked(tab->_show_top_axis_fld->isChecked());
	_show_bottom_axis_fld->setChecked(tab->_show_bottom_axis_fld->isChecked());
	_show_left_axis_fld->setChecked(tab->_show_left_axis_fld->isChecked());
	_show_right_axis_fld->setChecked(tab->_show_right_axis_fld->isChecked());
	_show_lr_alternating_axis_fld->setChecked(tab->_show_lr_alternating_axis_fld->isChecked());
	_show_rl_alternating_axis_fld->setChecked(tab->_show_rl_alternating_axis_fld->isChecked());
	_show_top_ticks_labels_fld->setChecked(tab->_show_top_ticks_labels_fld->isChecked());
	_show_bottom_ticks_labels_fld->setChecked(tab->_show_bottom_ticks_labels_fld->isChecked());
	_show_left_ticks_labels_fld->setChecked(tab->_show_left_ticks_labels_fld->isChecked());
	_show_right_ticks_labels_fld->setChecked(tab->_show_right_ticks_labels_fld->isChecked());
	_show_lr_alternating_ticks_labels_fld->setChecked(tab->_show_lr_alternating_ticks_labels_fld->isChecked());
	_show_rl_alternating_ticks_labels_fld->setChecked(tab->_show_rl_alternating_ticks_labels_fld->isChecked());
	_show_top_labels_fld->setChecked(tab->_show_top_labels_fld->isChecked());
	_show_bottom_labels_fld->setChecked(tab->_show_bottom_labels_fld->isChecked());
	_show_left_labels_fld->setChecked(tab->_show_left_labels_fld->isChecked());
	_show_right_labels_fld->setChecked(tab->_show_right_labels_fld->isChecked());
	_show_lr_alternating_labels_fld->setChecked(tab->_show_lr_alternating_labels_fld->isChecked());
	_show_rl_alternating_labels_fld->setChecked(tab->_show_rl_alternating_labels_fld->isChecked());
	_show_xiszero_fld->setChecked(tab->_show_xiszero_fld->isChecked());
	_show_yiszero_fld->setChecked(tab->_show_yiszero_fld->isChecked());
	
	connect(_line_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setLineWidth(double)));
	connect(_line_colour_fld, SIGNAL(clicked()), this, SLOT(setLineColour()));
	connect(_ticks_length_fld, SIGNAL(valueChanged(double)), this, SLOT(setTicksLength(double)));
	_ticks_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LABELS_CHAR_SIZE_ADD);
	connect(_ticks_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setTicksCharSize(double)));
	connect( _show_top_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setTopAxis(bool)) );
	connect( _show_bottom_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomAxis(bool)) );
	connect( _show_left_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftAxis(bool)) );
	connect(_show_right_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setRightAxis(bool)) );
	connect(_show_lr_alternating_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingAxis(bool)) );
	connect(_show_rl_alternating_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingAxis(bool)) );
	connect( _show_top_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setTopTicksLabels(bool)) );
	connect( _show_bottom_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomTicksLabels(bool)) );
	connect( _show_left_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftTicksLabels(bool)) );
	connect( _show_right_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRightTicksLabels(bool)) );
	connect( _show_lr_alternating_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingTicksLabels(bool)) );
	connect( _show_rl_alternating_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingTicksLabels(bool)) );
	connect( _show_top_labels_fld, SIGNAL(toggled(bool)), this,SLOT(setTopLabels(bool)) );
	connect( _show_bottom_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomLabels(bool)) );
	connect( _show_left_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftLabels(bool)) );
	connect( _show_right_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRightLabels(bool)) );
	connect( _show_lr_alternating_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingLabels(bool)) );
	connect( _show_rl_alternating_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingLabels(bool)) );
	connect( _show_xiszero_fld, SIGNAL(toggled(bool)), this, SLOT(setXIsZero(bool)) );
	connect( _show_yiszero_fld, SIGNAL(toggled(bool)), this, SLOT(setYIsZero(bool)) );
}

void QpDisplayTab::create()
{
	QVBoxLayout *panel_display_top_layout = new QVBoxLayout(this);
	panel_display_top_layout->setSpacing(4);

	QGroupBox *panel_frame_groupbox = new QGroupBox(this);

	if(I_Frame->frameType() == QP_TS) panel_frame_groupbox->setTitle("TS Frame");
	else  panel_frame_groupbox->setTitle("XY Frame");


// 	QWidget* panel_frame_widget = new QWidget(panel_frame_groupbox, "panel_frame_widget");

	QGridLayout *panel_frame_layout = new QGridLayout;
	panel_frame_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_frame_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* line_width_label = new QLabel("Line width: ");
	panel_frame_layout->addWidget(line_width_label, 1, 0, Qt::AlignRight);

	_line_width_fld = new FloatSpinBox(0.1, 20., 1, 0.1, panel_frame_groupbox, "line_width");
	_line_width_fld->setValue(QSAS_profiles->current()->LineWidth);
	_line_width_fld->setFixedWidth(100);
	panel_frame_layout->addWidget(_line_width_fld, 1, 1, Qt::AlignLeft);

	QLabel* colour_label = new QLabel("Line colour: ");
	panel_frame_layout->addWidget(colour_label, 1, 2, Qt::AlignRight);
	_line_colour_fld = new QPushButton("Colour...");
	_line_colour_fld->setFixedWidth(100);
	_line_colour_fld->setFixedHeight(30);
	QFont f = _line_colour_fld->font();
	f.setBold(TRUE);
	_line_colour_fld->setFont(f);
	panel_frame_layout->addWidget(_line_colour_fld,1,3,Qt::AlignLeft);

	QLabel* ticks_length_label = new QLabel("Ticks length: ");
	panel_frame_layout->addWidget(ticks_length_label, 3, 0, Qt::AlignRight);
	_ticks_length_fld = new FloatSpinBox(0.01, 40., 1, .5, panel_frame_groupbox, "ticks_length");
	_ticks_length_fld->setFixedWidth(100);
	panel_frame_layout->addWidget(_ticks_length_fld, 3, 1, Qt::AlignLeft);
	_ticks_length_fld->setValue(QPUI_TICKS_LENGTH);
	
	QLabel* ticks_char_size_label = new QLabel("Ticks font size: ");
	panel_frame_layout->addWidget(ticks_char_size_label, 3, 2, Qt::AlignRight);
	_ticks_char_size_fld = new FloatSpinBox(0.01, 40., 1, .5, panel_frame_groupbox, "ticks_char_size");
	_ticks_char_size_fld->setFixedWidth(100);
	panel_frame_layout->addWidget(_ticks_char_size_fld, 3, 3, Qt::AlignLeft);
	_ticks_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LABELS_CHAR_SIZE_ADD);

	panel_frame_layout->setRowStretch(0,0);
	panel_frame_layout->setRowStretch(1,1);
	panel_frame_layout->setRowStretch(2,0);
	panel_frame_layout->setRowStretch(3,1);
	panel_frame_layout->setRowStretch(4,0);

	panel_frame_layout->setColumnStretch(0,0);
	panel_frame_layout->setColumnStretch(1,0);
	panel_frame_layout->setColumnStretch(2,0);
	panel_frame_layout->setColumnStretch(3,0);
	panel_frame_layout->setColumnStretch(4,1);
	
	panel_frame_groupbox->setLayout(panel_frame_layout);

	panel_display_top_layout->addWidget(panel_frame_groupbox);

	QGroupBox *panel_display_groupbox  =new QGroupBox(this);

	panel_display_groupbox->setTitle("Show");

	QGridLayout *panel_display_layout = new QGridLayout;
	panel_display_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_display_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* top_label = new QLabel("Top");
	panel_display_layout->addWidget(top_label, 1, 1, Qt::AlignCenter);
	QLabel* bottom_label = new QLabel("Bottom");
	panel_display_layout->addWidget(bottom_label, 1, 2, Qt::AlignCenter);
	QLabel* left_label = new QLabel("Left");
	panel_display_layout->addWidget(left_label, 1, 3, Qt::AlignCenter);
	QLabel* right_label = new QLabel("Right");
	panel_display_layout->addWidget(right_label, 1, 4, Qt::AlignCenter);
	QLabel* lr_alternating_label = new QLabel("L/R alternating");
	panel_display_layout->addWidget(lr_alternating_label, 1, 5, Qt::AlignCenter);
	QLabel* rl_alternating_label = new QLabel("R/L alternating");
	panel_display_layout->addWidget(rl_alternating_label, 1, 6, Qt::AlignCenter);

	QFrame *hline = new QFrame;
	hline->setFrameStyle(QFrame::HLine | QFrame::Raised);
	hline->setMinimumWidth(500);
	panel_display_layout->addWidget(hline, 2, 0, 1, 7, Qt::AlignCenter);

	QLabel* axis_label = new QLabel("Axis");
	panel_display_layout->addWidget(axis_label, 3, 0, Qt::AlignCenter);

	_show_top_axis_fld = new QCheckBox;
	_show_top_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_top_axis_fld, 3, 1, Qt::AlignCenter);

	_show_bottom_axis_fld = new QCheckBox;
	_show_bottom_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_bottom_axis_fld, 3, 2, Qt::AlignCenter);

	_show_left_axis_fld = new QCheckBox;
	_show_left_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_left_axis_fld, 3, 3, Qt::AlignCenter);

	_show_right_axis_fld = new QCheckBox;
	_show_right_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_right_axis_fld, 3, 4, Qt::AlignCenter);
	
	_show_lr_alternating_axis_fld = new QCheckBox;
	_show_lr_alternating_axis_fld->setChecked(false);
	panel_display_layout->addWidget(_show_lr_alternating_axis_fld, 3, 5, Qt::AlignCenter);
	
	_show_rl_alternating_axis_fld = new QCheckBox;
	_show_rl_alternating_axis_fld->setChecked(false);
	panel_display_layout->addWidget(_show_rl_alternating_axis_fld, 3, 6, Qt::AlignCenter);

	QLabel* ticks_labels_label = new QLabel("Ticks Labels");
	panel_display_layout->addWidget(ticks_labels_label, 5, 0, Qt::AlignCenter);

	_show_top_ticks_labels_fld = new QCheckBox;
	_show_top_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_top_ticks_labels_fld, 5, 1, Qt::AlignCenter);

	_show_bottom_ticks_labels_fld=new QCheckBox;
	_show_bottom_ticks_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_bottom_ticks_labels_fld, 5, 2, Qt::AlignCenter);

	_show_left_ticks_labels_fld = new QCheckBox;
	_show_left_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_left_ticks_labels_fld, 5, 3, Qt::AlignCenter);

	_show_right_ticks_labels_fld = new QCheckBox;
	_show_right_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_right_ticks_labels_fld, 5, 4, Qt::AlignCenter);
	
	_show_lr_alternating_ticks_labels_fld = new QCheckBox;
	_show_lr_alternating_ticks_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_lr_alternating_ticks_labels_fld, 5, 5, Qt::AlignCenter);
	
	_show_rl_alternating_ticks_labels_fld = new QCheckBox;
	_show_rl_alternating_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_rl_alternating_ticks_labels_fld, 5, 6, Qt::AlignCenter);

	QLabel* labels_label = new QLabel("Labels");
	panel_display_layout->addWidget(labels_label, 7, 0, Qt::AlignCenter);

	_show_top_labels_fld = new QCheckBox;
	_show_top_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_top_labels_fld, 7, 1, Qt::AlignCenter);

	_show_bottom_labels_fld = new QCheckBox;
	_show_bottom_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_bottom_labels_fld, 7, 2, Qt::AlignCenter);

	_show_left_labels_fld = new QCheckBox;
	_show_left_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_left_labels_fld, 7, 3, Qt::AlignCenter);

	_show_right_labels_fld = new QCheckBox;
	_show_right_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_right_labels_fld, 7, 4, Qt::AlignCenter);
	
	_show_lr_alternating_labels_fld = new QCheckBox;
	_show_lr_alternating_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_lr_alternating_labels_fld, 7, 5, Qt::AlignCenter);
	
	_show_rl_alternating_labels_fld = new QCheckBox;
	_show_rl_alternating_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_rl_alternating_labels_fld, 7, 6, Qt::AlignCenter);

	QFrame *hline2 = new QFrame;
	hline2->setFrameStyle(QFrame::HLine | QFrame::Raised);
	hline2->setMinimumWidth(500);
	panel_display_layout->addWidget(hline2, 8, 0, 1, 7, Qt::AlignCenter);

	QLabel* additional_lines_label = new QLabel("Additional Lines");
	panel_display_layout->addWidget(additional_lines_label, 9, 0, 1, 6, Qt::AlignCenter);

	QLabel* xiszero_label = new QLabel("X = 0");
	panel_display_layout->addWidget(xiszero_label, 10, 0, Qt::AlignRight);
	_show_xiszero_fld = new QCheckBox;
	_show_xiszero_fld->setChecked(false);
	panel_display_layout->addWidget(_show_xiszero_fld, 10, 1, Qt::AlignCenter);

	QLabel* yiszero_label = new QLabel("Y = 0");
	panel_display_layout->addWidget(yiszero_label, 10, 2, Qt::AlignRight);
	_show_yiszero_fld = new QCheckBox;
	_show_yiszero_fld->setChecked(false);
	panel_display_layout->addWidget(_show_yiszero_fld, 10, 3, Qt::AlignCenter);

	panel_display_layout->setRowStretch(0,0);
	panel_display_layout->setRowStretch(1,0);
	panel_display_layout->setRowStretch(2,0);
	panel_display_layout->setRowStretch(3,1);
	panel_display_layout->setRowStretch(4,0);
	panel_display_layout->setRowStretch(5,1);
	panel_display_layout->setRowStretch(6,0);
	panel_display_layout->setRowStretch(7,1);
	panel_display_layout->setRowStretch(8,0);
	panel_display_layout->setRowStretch(9,0);
	panel_display_layout->setRowStretch(10,1);
	panel_display_layout->setRowStretch(11,0);

	panel_display_layout->setColumnStretch(0,0);
	panel_display_layout->setColumnStretch(1,0);
	panel_display_layout->setColumnStretch(2,0);
	panel_display_layout->setColumnStretch(3,0);
	panel_display_layout->setColumnStretch(4,0);
	panel_display_layout->setColumnStretch(5,0);
	panel_display_layout->setColumnStretch(6,0);
	panel_display_layout->setColumnStretch(7,1);

	panel_display_groupbox->setLayout(panel_display_layout);
	
	panel_display_top_layout->addWidget(panel_display_groupbox);

	panel_display_top_layout->addSpacing(4);
	panel_display_top_layout->addStretch(10);
	
// 	panel_frame_widget->adjustSize();
// 	panel_display_widget->adjustSize();
}

void QpDisplayTab::setLineWidth(double fChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(PANEL_LINE_WIDTH, 0, (float)fChoice);

}

void QpDisplayTab::setLineColour()
{
	bool changed = false;
	// ask the user for colour choice
	QColor c = QuGetColour(I_Frame->_line_colour, &changed, this);
	if(changed)
	{
		QPalette line_palette = QPalette(QPUI_normal_palette);
		line_palette.setColor(QPalette::Active ,QPalette::ButtonText, c);
		line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, c);
		this->_line_colour_fld->setPalette(line_palette);
		I_Frame->_line_colour = c;
	}

	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(PANEL_LINE_COLOUR, 0, 0., "", false, c);
}

void QpDisplayTab::setTicksLength(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TICKS_LENGTH, 0, fChoice);
}

void QpDisplayTab::setTicksCharSize(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TICKS_CHAR_SIZE, 0, fChoice);
}

void QpDisplayTab::setTopAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TOP_AXIS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setBottomAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(BOTTOM_AXIS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLeftAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(LEFT_AXIS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRightAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(RIGHT_AXIS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLRAlternatingAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(LR_ALTERNATING_AXIS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRLAlternatingAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(RL_ALTERNATING_AXIS, 0, 0.0, "", bChoice);
}


void QpDisplayTab::setTopTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TOP_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setBottomTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(BOTTOM_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLeftTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(LEFT_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRightTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(RIGHT_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLRAlternatingTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(LR_ALTERNATING_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRLAlternatingTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(RL_ALTERNATING_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setTopLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(TOP_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setBottomLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(BOTTOM_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLeftLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(LEFT_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRightLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(RIGHT_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLRAlternatingLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(LR_ALTERNATING_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRLAlternatingLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(RL_ALTERNATING_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setYIsZero(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(Y_IS_ZERO, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setXIsZero(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Frame->root()->I_PlotView->setFrameOption(X_IS_ZERO, 0, 0.0, "", bChoice);
}

QDomElement QpDisplayTab::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Display_Tab");
    xml.appendChild(createTextXMLNode(doc, "Line_Width", _line_width_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Ticks_Length", _ticks_length_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Ticks_Char_Size", _ticks_char_size_fld->value()));

    xml.appendChild(createTextXMLNode(doc, "Show_Top_Axis", _show_top_axis_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_Bottom_Axis", _show_bottom_axis_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_Left_Axis", _show_left_axis_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_Right_Axis", _show_right_axis_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_LR_Axis", _show_lr_alternating_axis_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_RL_Axis", _show_rl_alternating_axis_fld->isChecked()));

    xml.appendChild(createTextXMLNode(doc, "Show_Top_Ticks_Labels", _show_top_ticks_labels_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_Bottom_Ticks_Labels", _show_bottom_ticks_labels_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_Left_Ticks_Labels", _show_left_ticks_labels_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_Right_Ticks_Labels", _show_right_ticks_labels_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_LR_Ticks_Labels", _show_lr_alternating_ticks_labels_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_RL_Ticks_Labels", _show_rl_alternating_ticks_labels_fld->isChecked()));

    xml.appendChild(createTextXMLNode(doc, "Show_Top_Labels", _show_top_labels_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_Bottom_Labels", _show_bottom_labels_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_Left_Labels", _show_left_labels_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_Right_Labels", _show_right_labels_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_LR_Labels", _show_lr_alternating_labels_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_RL_Labels", _show_rl_alternating_labels_fld->isChecked()));

    xml.appendChild(createTextXMLNode(doc, "Show_X_Zero", _show_xiszero_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Show_Y_Zero", _show_yiszero_fld->isChecked()));


    return xml;
}

void QpDisplayTab::fromXMLElement(QDomElement& el)
{
    QString s;
    if(getXMLNodeText(el, "Line_Width", s))
    {
        _line_width_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Ticks_Length", s))
    {
        _ticks_length_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Ticks_Char_Size", s))
    {
        _ticks_char_size_fld->setValue(s.toDouble());
    }

    if(getXMLNodeText(el, "Show_Top_Axis", s))
    {
        _show_top_axis_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_Bottom_Axis", s))
    {
        _show_bottom_axis_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_Left_Axis", s))
    {
        _show_left_axis_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_Right_Axis", s))
    {
        _show_right_axis_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_LR_Axis", s))
    {
        _show_lr_alternating_axis_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_RL_Axis", s))
    {
        _show_rl_alternating_axis_fld->setChecked((bool)s.toInt());
    }

    if(getXMLNodeText(el, "Show_Top_Ticks_Labels", s))
    {
        _show_top_ticks_labels_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_Bottom_Ticks_Labels", s))
    {
        _show_bottom_ticks_labels_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_Left_Ticks_Labels", s))
    {
        _show_left_ticks_labels_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_Right_Ticks_Labels", s))
    {
        _show_right_ticks_labels_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_LR_Ticks_Labels", s))
    {
        _show_lr_alternating_ticks_labels_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_RL_Ticks_Labels", s))
    {
        _show_rl_alternating_ticks_labels_fld->setChecked((bool)s.toInt());
    }

    if(getXMLNodeText(el, "Show_Top_Labels", s))
    {
        _show_top_labels_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_Bottom_Labels", s))
    {
        _show_bottom_labels_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_Left_Labels", s))
    {
        _show_left_labels_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_Right_Labels", s))
    {
        _show_right_labels_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_LR_Labels", s))
    {
        _show_lr_alternating_labels_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_RL_Labels", s))
    {
        _show_rl_alternating_labels_fld->setChecked((bool)s.toInt());
    }

    if(getXMLNodeText(el, "Show_X_Zero", s))
    {
        _show_xiszero_fld->setChecked((bool)s.toInt());
    }
    if(getXMLNodeText(el, "Show_Y_Zero", s))
    {
        _show_yiszero_fld->setChecked((bool)s.toInt());
    }

}


