// qpui_value_return.cc
//
// Plot Interface Implementation - Value Return 
//
// Janet Barnes
//
// July 2002
// Modified Tony Allen 2008


#include "qpui.h"
#include "qwui.h"


//     QpValueReturnTab CLASS
//     ---------------------------  

QpValueReturnTab::QpValueReturnTab(QTabWidget *tabwidget, const QString& name, PageItem *page):
		QpTab(tabwidget, name, QP_VALUE_TAB)
{
  // Constructor
     
	_page_item=page;
	
	create();
	
	connect(tabwidget, SIGNAL(currentChanged(int)), this, SLOT(catchValueRet(int)) );
}

QpValueReturnTab::QpValueReturnTab(QpValueReturnTab* tab, PageItem* _page):
		QpTab((QTabWidget*)(tab->parent()), "Value Return", QP_VALUE_TAB)
{
	_page_item=_page;
	
	create();
	
	connect(tab, SIGNAL(currentChanged(int)), this, SLOT(catchValueRet(int)) );
}

void QpValueReturnTab::create()
{
	QBoxLayout * value_return_layout=new QBoxLayout(QBoxLayout::TopToBottom, this);
	value_return_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	value_return_layout->setMargin(QSAS_profiles->current()->GUImargins);
	
	QGroupBox *getvalue_groupbox=new QGroupBox(this);
	getvalue_groupbox->setTitle("Get Values from Plot");
  
// 	QWidget* getvalue_widget=new QWidget(getvalue_groupbox, "getvalue_widget");

	QBoxLayout * getvalue_v_layout=new QBoxLayout(QBoxLayout::TopToBottom);
	
	getvalue_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	getvalue_v_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QBoxLayout * getvalue_h_layout_1=new QBoxLayout(QBoxLayout::LeftToRight);
	QPushButton *get_values_pb=new QPushButton("Get Values...");
	get_values_pb->setFixedWidth(300);
	getvalue_h_layout_1->addWidget(get_values_pb);
	getvalue_h_layout_1->setSpacing(QSAS_profiles->current()->GUImargins);
	getvalue_h_layout_1->setMargin(QSAS_profiles->current()->GUImargins);
  
	QBoxLayout * getvalue_h_layout_2=new QBoxLayout(QBoxLayout::LeftToRight);
  
	_xaxis_swap_fld=new QCheckBox("Preserve X,T direction");
	_xaxis_swap_fld->setChecked(true);
	getvalue_h_layout_2->addWidget(_xaxis_swap_fld);
  
	_yaxis_swap_fld=new QCheckBox("Preserve Y direction");
	_yaxis_swap_fld->setChecked(true);
	getvalue_h_layout_2->addWidget(_yaxis_swap_fld);
	getvalue_h_layout_2->setSpacing(QSAS_profiles->current()->GUImargins);
	getvalue_h_layout_2->setMargin(QSAS_profiles->current()->GUImargins);
  
	getvalue_v_layout->addLayout(getvalue_h_layout_1);
	getvalue_v_layout->addLayout(getvalue_h_layout_2);
    
	getvalue_groupbox->setLayout(getvalue_v_layout);
	
	value_return_layout->addWidget(getvalue_groupbox);
  
	QGroupBox *xaxis_groupbox=new QGroupBox(this);
	xaxis_groupbox->setTitle("Horizontal Axis");
  
	QBoxLayout *xaxis_v_layout=new QBoxLayout(QBoxLayout::TopToBottom);

	xaxis_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	xaxis_v_layout->setMargin(QSAS_profiles->current()->GUImargins);
	
	_xaxis_range_fld=new ScalarIntervalWidget(xaxis_groupbox);
	_xaxis_range_fld->setRangeName(QString("X_Range"));
	_taxis_range_fld=new TimeIntervalWidget(xaxis_groupbox);
	_taxis_range_fld->hide();
	
   
	xaxis_v_layout->addWidget(_xaxis_range_fld);
	xaxis_v_layout->addWidget(_taxis_range_fld);

	xaxis_groupbox->setLayout(xaxis_v_layout);
	
	value_return_layout->addWidget(xaxis_groupbox);

	QGroupBox *yaxis_groupbox=new QGroupBox(this);
	yaxis_groupbox->setTitle("Y Axis");
  
	QBoxLayout *yaxis_v_layout=new QBoxLayout(QBoxLayout::TopToBottom);

	yaxis_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	yaxis_v_layout->setMargin(QSAS_profiles->current()->GUImargins);
  
	_yaxis_range_fld=new ScalarIntervalWidget(yaxis_groupbox);
	_yaxis_range_fld->setRangeName(QString("Y_Range"));
	

	yaxis_v_layout->addWidget(/*yaxis_h_layout_1*/_yaxis_range_fld);

	yaxis_groupbox->setLayout(yaxis_v_layout);
	
	value_return_layout->addWidget(yaxis_groupbox);

	QGroupBox *replot_groupbox=new QGroupBox(this);
	replot_groupbox->setTitle("Apply to plot");
  
	QBoxLayout *replot_v_layout=new QBoxLayout(QBoxLayout::TopToBottom);

	replot_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	replot_v_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QBoxLayout *replot_h_layout_1=new QBoxLayout(QBoxLayout::LeftToRight);
	_xaxis_rescale_fld=new QCheckBox("Rescale X,T axis");
	_xaxis_rescale_fld->setChecked(true);
	replot_h_layout_1->addWidget(_xaxis_rescale_fld);
  
	_yaxis_rescale_fld=new QCheckBox("Rescale Y axis");
	_yaxis_rescale_fld->setChecked(true);
	replot_h_layout_1->addWidget(_yaxis_rescale_fld);
	replot_h_layout_1->setSpacing(QSAS_profiles->current()->GUImargins);
	replot_h_layout_1->setMargin(QSAS_profiles->current()->GUImargins);
  
	QBoxLayout *replot_h_layout_2=new QBoxLayout(QBoxLayout::LeftToRight);
	_rePlot = new QPushButton( "RePlot");
	connect( _rePlot, SIGNAL(clicked()), this, SLOT(rePlot()) );
	replot_h_layout_2->addWidget(_rePlot);

	_reset = new QPushButton( "Undo");
	connect( _reset, SIGNAL(clicked()), this, SLOT(resetValues()) );
	replot_h_layout_2->addWidget(_reset);
	replot_h_layout_2->setSpacing(QSAS_profiles->current()->GUImargins);
	replot_h_layout_2->setMargin(QSAS_profiles->current()->GUImargins);

	replot_v_layout->addLayout(replot_h_layout_1);
	replot_v_layout->addLayout(replot_h_layout_2);

	replot_groupbox->setLayout(replot_v_layout);
			
	value_return_layout->addWidget(replot_groupbox);

	message_lbl = new QLabel("", this);
	value_return_layout->addWidget(message_lbl);

	connect(get_values_pb, SIGNAL(clicked()), this, SLOT(getValues()));  


	_frame_item = 0;
	_panel_item = 0;
   
  // disable reset and apply push buttons
	_reset->setEnabled(false);
	_rePlot->setEnabled(false);

	
}

// ----------------------------------

void QpValueReturnTab::catchValueRet(int tabIdx)
{
	if( !_page_item || tabIdx < 0) return;

	if (tabIdx == _page_item->tabWidget->indexOf(this)) _page_item->root()->I_PlotView->clearSelection();
}

// ----------------------------------

// ----------------------------------

void QpValueReturnTab::rePlot()
{
	// Slot called when rePlot push button clicked
	// If items are selected after values returned, then apply propagates down to them.

	if(QuSlotDebug()) cout << "QpValueReturnTab::rePlot" << endl;
	
	if( !_page_item || !_page_item->root()->isParentOf(_frame_item) || !_page_item->root()->isParentOf(_panel_item))
	{
		message_lbl->setText("Warning Frame or Panel no longer exists! Select new values.");
		return;
	}
	message_lbl->setText("");
	
	// make frame and panel lists and include panel used in selection

	QVector<PanelItem*> panelList;
	QVector<FrameItem*> frameList;
	
	// add frame and panel used in selection
	if( _xaxis_rescale_fld->isChecked()  ) frameList.push_back(_frame_item);

	if( _yaxis_rescale_fld->isChecked() )  panelList.push_back(_panel_item);

	// update frames
	if( _xaxis_rescale_fld->isChecked() ) 
	{
		num_old_frames.push_back(frameList.size());
		
		for(int i=0; i<frameList.size(); i++)
		{
			FrameItem *frameItem = frameList[i];
			
			// store original values on end of undo stack
			_frame_item_old.push_back(frameItem);

			if(frameItem->frameType() == QP_XY)
			{
				// XY Frame
				_x_from_old.push_back(frameItem->xAxisTab->_axis_xy_fld->getStart());
				_x_to_old.push_back(frameItem->xAxisTab->_axis_xy_fld->getEnd());
				_frame_autoscale_old.push_back(frameItem->xAxisTab->_axis_autoscale_fld->currentIndex());
				
				// replace with new values
				frameItem->xAxisTab->_axis_xy_fld->setInterval(_xaxis_range_fld->getStart(), _xaxis_range_fld->getEnd());
				frameItem->xAxisTab->setAxisRange();
				
				frameItem->xAxisTab->_axis_autoscale_fld->setCurrentIndex(QP_FROM_USER);
			}
			else
			{
				// TS Frame
				_x_from_old.push_back(frameItem->tAxisTab->_axis_time_fld->getStart().getEpoch2000());
				_x_to_old.push_back(frameItem->tAxisTab->_axis_time_fld->getEnd().getEpoch2000());
				_frame_autoscale_old.push_back(frameItem->tAxisTab->_axis_autoscale_fld->currentIndex());
				
				// replace with new values
				frameItem->tAxisTab->_axis_time_fld->setStart(_taxis_range_fld->getStart());
				frameItem->tAxisTab->_axis_time_fld->setEnd(_taxis_range_fld->getEnd());
				frameItem->tAxisTab->setAxisRange();
				
				frameItem->tAxisTab->_axis_autoscale_fld->setCurrentIndex(QP_FROM_USER);

			}
		}
	}
	else 		num_old_frames.push_back(0);

	// update panels
	if( _yaxis_rescale_fld->isChecked() )
	{
		num_old_panels.push_back(panelList.size());

		for(int i=0; i<panelList.size(); i++)
		{
			PanelItem *panelItem = panelList[i];

			_y_from_old.push_back(panelItem->yAxisTab->_axis_xy_fld->getStart());
			_y_to_old.push_back(panelItem->yAxisTab->_axis_xy_fld->getEnd());
			_panel_item_old.push_back(panelItem);
			_panel_autoscale_old.push_back(panelItem->yAxisTab->_axis_autoscale_fld->currentIndex());
					
			// replace with new values
			panelItem->yAxisTab->_axis_xy_fld->setInterval(_yaxis_range_fld->getStart(), _yaxis_range_fld->getEnd());			
			panelItem->yAxisTab->setAxisRange();
			panelItem->yAxisTab->handleAutoScale(QP_FROM_USER);

		}
	}
	else 	num_old_panels.push_back(0);

   // enable reset push button
   _reset->setEnabled(true);

	_page_item->root()->I_PlotView->plot();
   
}

void QpValueReturnTab::getValues()
{
	if(QuSlotDebug()) cout << "QpValueReturnTab::getValues" << endl;

      // check a plot page is currently selected
    if(!_page_item)
    {
        (void)QMessageBox::warning(this,"Value Return","No page selected!");
        return;
    }
    
      // clear listview selection to avoid user accidentally applying changes
  // to the wrong panel. Selection is from the plot.
//     _page_item->I_ListView->localList.clear(); //TODO
//     _page_item->I_ListView->clearSelection();
    
      // check that the currently selected plot x window is open
    int plot_id=_page_item->PageTab->_xwin_num_fld->text().toInt();
    if(_page_item->PlotWindows.empty())
    {
// 		_page_item->I_ListView->setCurrentItem(_page_item);
// 	    _page_item->I_ListView->plotCurrentItem();
	    _page_item->root()->I_PlotView->plotSelectedItems();
		plot_id=_page_item->PageTab->_xwin_num_fld->text().toInt();
    }
    
    PlotWindow* win=_page_item->focusWindow(plot_id);
    if(win==NULL)
    {
	    (void)QMessageBox::warning( this,"Value Return", "Error! Window id not found.");
	    return;
    }

    double x_start, y_start, x_end, y_end;
       win->plot->captureMouse(&x_start, &y_start, &x_end, &y_end);

	y_start=1.-y_start;
	y_end=1.-y_end;
    
  // swap range ends?
	if(x_end < x_start && _xaxis_swap_fld->isChecked())
	{
		double x_tmp=x_start;
		x_start=x_end;
		x_end=x_tmp;
	}
  
	if(y_end < y_start && _yaxis_swap_fld->isChecked())
	{
		double y_tmp=y_start;
		y_start=y_end;
		y_end=y_tmp;
	}
  
  // descend through frames in selected page to find frames
  // lying within range selected interactively
   	
	FrameItem* frameItem;
	_panel_item = 0;
	int nPanel=0;
	for(int i=0; i<_page_item->numChildren(); ++i)
	{
		frameItem=dynamic_cast<FrameItem*>(_page_item->getChildByNumber(i));
		int numPanels = frameItem->numChildren();
		float panelHt = frameItem->PropertiesTab->_height_fld->value() / numPanels;
		int num = 0;
		bool hasSpectrum = _page_item->containsSpectrum();
		double factor=(hasSpectrum?0.9:1.);
		PanelItem * panelItem;
		for(int j=0; j<frameItem->numChildren(); ++j)
		{
			panelItem=dynamic_cast<PanelItem*>(frameItem->getChildByNumber(j));
		// check if selected start points lies within this panel
	    // if so this is selected panel so break out of loop
		// x-axis is on the frame and y-axis on the panel
			if(frameItem->PropertiesTab->_loffset_fld->value() <= x_start &&  
						x_start <= frameItem->PropertiesTab->_loffset_fld->value() + frameItem->PropertiesTab->_width_fld->value()*factor && 
						1. - frameItem->PropertiesTab->_toffset_fld->value() - num * panelHt >= y_start &&
						y_start >= 1. - frameItem->PropertiesTab->_toffset_fld->value() - (num+1) * panelHt)
			{
				_panel_item = panelItem;
				nPanel = num;
				break;
			}
			num++;
		}	    
		if(_panel_item) break; // panel has been found
	}
	
	_frame_item = frameItem;
  
  
  // Do we have a panel?
	if(!_panel_item)
	{
		(void)QMessageBox::warning(this, "Value Return", "Error! No panel selected");
		return;
	}
 
	bool hasSpectrum = _page_item->containsSpectrum();
	double factor=(hasSpectrum?0.9:1.);
  // check selected END points also lie within this frame 
	if(x_end > _frame_item->PropertiesTab->_loffset_fld->value() + _frame_item->PropertiesTab->_width_fld->value()*factor || 
		  x_end < _frame_item->PropertiesTab->_loffset_fld->value() || 
		  y_end > 1. - _frame_item->PropertiesTab->_toffset_fld->value() || 
		  y_end < 1. - _frame_item->PropertiesTab->_toffset_fld->value() - _frame_item->PropertiesTab->_height_fld->value())
	{
		_panel_item=0;
		_frame_item=0;
		(void)QMessageBox::warning(this, "Value Return", "Error! Start and finish in different frames");
		return;
	}
   

	double x_offset = _frame_item->PropertiesTab->_loffset_fld->value();
	double x_sz = _frame_item->PropertiesTab->_width_fld->value()*factor;
	double y_sz = _frame_item->PropertiesTab->_height_fld->value();
	y_sz /= _frame_item->numChildren();
	double y_offset = 1. - _frame_item->PropertiesTab->_toffset_fld->value() - ( nPanel + 1 ) * y_sz;

  
  // calculate new x axis ranges from plot selection coordinates 
	if(_frame_item->frameType() == QP_XY)
	{
	
		_xaxis_range_fld->show();
		_taxis_range_fld->hide();
		
		// Get boundaries of the frame item, to delimit the range of the time interval editor
		double from, to;
		bool found = _frame_item->getXmetaRange(from, to);

		double xmin = _frame_item->xAxisTab->_axis_xy_fld->getStart();
		double xmax = _frame_item->xAxisTab->_axis_xy_fld->getEnd();
	
    // if logarithmic axis, convert to logs first
		if(_frame_item->xAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG)
		{
			xmin = SafeLog10(xmin);
			xmax = SafeLog10(xmax);
		}
	
		double x_range = xmax - xmin;
		_x_from = xmin + x_range * (x_start-x_offset)/x_sz;
		_x_to = xmin + x_range * (x_end-x_offset)/x_sz;
		if(_frame_item->xAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG)
		{
			_x_from=pow(10., _x_from);
			_x_to=pow(10., _x_to);
		}
		_xaxis_range_fld->setAvailabilityRange(from, to);	   // also sets view range 
		_xaxis_range_fld->setInterval(_x_from, _x_to, false);

	}
	else if(_frame_item->frameType() == QP_TS)
	{
		_xaxis_range_fld->hide();
		_taxis_range_fld->show();
		
		// Get boundaries of the frame item, to delimit the range of the time interval editor
		double from, to;
		bool found = _frame_item->getXmetaRange(from, to);

		double tmin = _frame_item->tAxisTab->_axis_time_fld->getStart().getEpoch2000();
		double tmax = _frame_item->tAxisTab->_axis_time_fld->getEnd().getEpoch2000();
		double t_range = tmax - tmin;
		_x_from = tmin + t_range * (x_start-x_offset)/x_sz;
		_x_to = tmin + t_range * (x_end-x_offset)/x_sz;
   
   	
		QmjdTime tFrom(_x_from);
		QmjdTime tTo(_x_to);
		_taxis_range_fld->setViewRange(QmjdTimeInterval(QmjdTime(from), QmjdTime(to)));
		_taxis_range_fld->setAvailabilityRange(QmjdTime(from), QmjdTime(to));
		_taxis_range_fld->setInterval(tFrom, tTo, false);

	} 
  
  // calculate new y axis ranges
	if(_panel_item->yAxisTab)
	{
		double ymin = _panel_item->yAxisTab->_axis_xy_fld->getStart();
		double ymax = _panel_item->yAxisTab->_axis_xy_fld->getEnd();
    // if logarithmic axis convert to logs first
		if(_panel_item->yAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG)
		{
			ymin = SafeLog10(ymin);
			ymax = SafeLog10(ymax);
		}
		double y_range = ymax - ymin;
		_y_from = ymin + y_range*(y_start-y_offset)/y_sz;
		_y_to = ymin + y_range*(y_end-y_offset)/y_sz;
		if(_panel_item->yAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG)
		{
			_y_from = pow(10., _y_from);
			_y_to = pow(10., _y_to);
		}	    
		_yaxis_range_fld->setInterval(_y_from, _y_to);

	}
	
	// enable Apply push buttons
	_rePlot->setEnabled(true);

    
    return;
} 

void QpValueReturnTab::resetValues()
{
	// slot called when undo button pushed, resets ranges to their last values 

	if(QuSlotDebug())  cout << "QpValueReturnTab::resetValues" << endl;
    
	if( !_page_item) {
		message_lbl->setText("No page available");
		return;
	}
	
	message_lbl->setText("");

	// undo frame changes
	for(int i=0; i<num_old_frames.back(); i++)
	{
		FrameItem *frameItem = _frame_item_old.back();
		
		if( _page_item->root()->isParentOf(frameItem) )
		{
			if(frameItem->frameType() == QP_XY)
			{
				// restore values
				frameItem->xAxisTab->_axis_xy_fld->setInterval(_x_from_old.back(), _x_to_old.back());
				frameItem->xAxisTab->_axis_autoscale_fld->setCurrentIndex(_frame_autoscale_old.back());
			}
			else if(_frame_item && _frame_item->frameType() == QP_TS)
			{
				// restore values
				frameItem->tAxisTab->_axis_time_fld->setStart(QmjdTime(_x_from_old.back()));
				frameItem->tAxisTab->_axis_time_fld->setEnd(QmjdTime(_x_to_old.back()));
				frameItem->tAxisTab->_axis_autoscale_fld->setCurrentIndex(_frame_autoscale_old.back());
			}
		}
		else message_lbl->setText("Cannot Undo, frame deleted");

		_frame_item_old.pop_back();
		_x_from_old.pop_back();
		_x_to_old.pop_back();
		_frame_autoscale_old.pop_back();
	}
	num_old_frames.pop_back();

	// undo panel changes
	for(int i=0; i<num_old_panels.back(); i++)
	{
		PanelItem *panelItem = _panel_item_old.back();
		
		if( _page_item->root()->isParentOf(panelItem) )
		{
			// restore values
			panelItem->yAxisTab->_axis_xy_fld->setInterval(_y_from_old.back(), _y_to_old.back());
			panelItem->yAxisTab->_axis_autoscale_fld->setCurrentIndex(_panel_autoscale_old.back());
		
		}
		else 	message_lbl->setText("Cannot Undo, panel deleted");

		_panel_item_old.pop_back();
		_y_from_old.pop_back();
		_y_to_old.pop_back();
		_panel_autoscale_old.pop_back();
		
	}
	num_old_panels.pop_back();

	// plot
	_page_item->root()->I_PlotView->plot();
		

	// disable reset
	if (num_old_frames.size() <= 0) _reset->setEnabled(false);

}  

