//
//  Dim.cc
//
//  Qdos2 - methods for Dim Class
//
//  Janet Barnes
//  Sept 1999
//

#include "Dim.h"
namespace QDOS{

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ class Dim::Dim
// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// ------------------------------------------------------------------------- Dim
// Construct from a valarray<size_t> of dimensions
Dim::Dim(const valarray<size_t>& dms, DimStorageOption order):
        _rank(dms.size()),_order(order),_gsl(initgslice(dms,order))
{_size=product(dms);} 

// Construct from a vector<int> of dimensions
Dim::Dim(const vector<int>& dms, DimStorageOption order):
    _rank(dms.size()), _order(order), _gsl(), _size(0)
{ valarray<size_t> va_dms(dms.size());
  for(int i=0;i<_rank;i++) va_dms[i]=dms[i];
  _gsl = initgslice( va_dms,order );
  _size=product(_gsl.size());
} 
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++ class Dim::samesize
// ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// -------------------------------------------------------------------- samesize
bool Dim::samesize(const Dim& d) const
{
// check whether d and current Dim object are of same size and rank
    bool res;
    res=_rank==d._rank;
    int i=0;
    
    while (res && i<_rank)
    {
        res= res && dims()[i]==d.dims()[i];
        i++;
     }
     
    return res;
} 
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++ class Dim::get_dims
// ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// -------------------------------------------------------------------- get_dims
void Dim::get_dims(vector<int>& dims_out) const
{   
    valarray<size_t> dms=_gsl.size();
    size_t sz=dms.size();
    dims_out.resize(sz);
    copy(&dms[0],&dms[sz],dims_out.begin());
} 
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++ class Dim::get_strides
// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// ----------------------------------------------------------------- get_strides
void Dim::get_strides(vector<int>& str_out) const
{
    valarray<size_t> str=_gsl.stride();
    size_t sz=str.size();
    str_out.resize(sz);
    copy(&str[0],&str[sz],str_out.begin());
} 

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ class Dim::display
// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// --------------------------------------------------------------------- display
void Dim::display() const 
{
// --- Display function for Dim object
    int i;
    if (_order==ROW_MAJOR_ORDER)
        cout << "Order: Row-Major" << endl;
    else 
    {
        if (_order==COL_MAJOR_ORDER)
            cout << "Order: Col-Major" << endl;
        else
            cout << "Order: ????" << endl;
    }         

    cout << "Rank=" << _rank << endl;    
    cout << "Dimensions: ";
    if (_rank==0)
        cout << "0";        
    else
        cout << dims()[0];
    for (i=1;i<_rank;i++)
        cout << " x " << dims()[i];
    cout << endl;
    cout << "Strides: ";
    if (_rank==0)
        cout << "0";
    else
        cout << strides()[0];
    for (i=1;i<_rank;i++)
        cout << ", " << strides()[i];    
    cout << endl; 
}

ostream& operator<<(ostream& os, const Dim& d)
{
  if(d.rank()>0)
  {
    os << "[ " << d[0];
    for(int i=1;i<d.rank();i++)
      os << " x " << d[i];
    os << " ]" << endl;
  }
  return os;
}

}  // end namespace QDOS
