#include "MatrixSVD.h"
//#include <g2c.h>

namespace QDOS{

	inline double_complex sign_mod(double_complex v)
	{
		if(v!=0.)
		{
			return v/abs(v);
		}
		return 1.;
	}

	double_complex house(double_complex * x, int n, double_complex * v)
	{

		double max=0., alpha(0.);
		double_complex beta(0.);
		int i;

		memcpy(v, x, n*sizeof(double_complex));
	
		//std::cout << "House " << x[0] << " " << x[1] << " " << x[2] << " = " << std::endl;
	
		for(i=0; i<n;i++)
		{
			if(abs(v[i])>max)
			{
				max=abs(v[i]);
			}
		}
		
		//std::cout << "Max " << max << std::endl;
	
		if(max>0)
		{
			for(i=0; i<n; i++)
			{
				v[i]=v[i]/max;
			}
			for(i=0; i<n; i++)
			{
				alpha+=abs(v[i])*abs(v[i]);
			}
			alpha=std::sqrt(alpha);
		
			if(alpha>0.)
			{
				beta=1./(alpha*(alpha+abs(v[0])));
				v[0]=v[0]+alpha*sign_mod(v[0]);
			}
			else
			{
				beta=0.;
			}
			beta=beta*v[0]*conj(v[0]);

			for(i=1; i<n; i++)
			{
				v[i]=v[i]/v[0];
			}
			v[0]=1.;
		}
		else
		{
			beta=0.;
		}
		
		//std::cout << beta << std::endl;
		
		return beta;
	}


	double_complex right_hand_householder(double_complex * a, int m, int n, int row)
	{
		int i, j, k;
		double_complex beta;
		double_complex temp;
		double_complex * x=new double_complex[m];
		double_complex * v=new double_complex[m];
		double_complex * temp_arr=new double_complex[m*n];
	

		memset(x, 0x0, m*sizeof(double_complex));
		memset(v, 0x0, m*sizeof(double_complex));
		memset(temp_arr, 0x0, m*n*sizeof(double_complex));
		
		for(i=row+1; i<n; i++)
		{
			x[i-row-1]=a[row*n+i];
		}
			
		beta=house(x, n-row-1, v);

		memcpy(temp_arr, a, m*n*sizeof(double_complex));
	
		for(i=row; i<m; i++)
		{
			for(j=row+1; j<n; j++)
			{
				temp=0.;
				for(k=row+1; k<n; k++)
				{
					temp+=temp_arr[i*n+k]*conj(v[k-row-1]);
				}
				a[i*n+j]=temp_arr[i*n+j]-beta*temp*(v[j-row-1]);
			}
		}

		for(j=row+2; j<n; j++)
		{
			a[row*n+j]=v[j-row-1];
		}
		
		/*std::cout << "..." <<std::endl;
		for(i=0; i<m; ++i)
		{
			for(j=0; j<n; ++j) std::cout << a[i*n+j];
			std::cout << std::endl;
		}
		std::cout << "***** ...RHHouseHolder " << std::endl;*/
		
		delete[] temp_arr;
		delete[] v;
		delete[] x;

		return beta;
	}

	double_complex left_hand_householder(double_complex * a, int m, int n, int column)
	{
		int i, j, k;
		double_complex beta;
		double_complex temp;
		double_complex * x=new double_complex[m];
		double_complex * v=new double_complex[m];
		double_complex * temp_arr=new double_complex[m*n];
		
		memset(x, 0x0, m*sizeof(double_complex));
		memset(v, 0x0, m*sizeof(double_complex));
		memset(temp_arr, 0x0, m*n*sizeof(double_complex));
		
		/*for(i=0; i<m; ++i)
		{
			x[i]=0.;
			v[i]=0.;
			for(j=0; j<n; ++j) temp_arr[i*n+j]=0.;
		}*/
		
		memset(x, 0x0, m*sizeof(double_complex));
		memset(v, 0x0, m*sizeof(double_complex));
		memset(temp_arr, 0x0, m*n*sizeof(double_complex));
		
		/*std::cout << "***** LHHouseHolder... " << std::endl;
		for(i=0; i<m; ++i)
		{
			for(j=0; j<n; ++j) std::cout << a[i*n+j];
			std::cout << std::endl;
		}*/
		
		for(i=column; i<m; i++)
		{
			x[i-column]=a[i*n+column];
		}
		beta=house(x, m-column, v);

		for(i=0; i<m; i++)
		{
			for(j=0; j<n; j++)
			{
				temp_arr[i*n+j]=a[i*n+j];
			}
		}
		
		for(i=column; i<m; i++)
		{
			for(j=column; j<n; j++)
			{
				temp=0.;
				for(k=column; k<m; k++)
				{
					temp+=conj(v[k-column])*temp_arr[k*n+j];
				}
				a[i*n+j]=temp_arr[i*n+j]-beta*v[i-column]*temp;
			}
		}
	
		for(i=column+1; i<m; i++)
			a[i*n+column]=v[i-column];
		
		/*std::cout << "..." <<std::endl;
		for(i=0; i<m; ++i)
		{
			for(j=0; j<n; ++j) std::cout << a[i*n+j];
			std::cout << std::endl;
		}
		std::cout << "***** ...LHHouseHolder " << std::endl;*/

		
		delete[] temp_arr;
		delete[] v;
		delete[] x;

		return beta;

	}

	void bidiag(double_complex * a, int m, int n, double_complex * U, double_complex * V)
	{
		int i, j, k, iter;
		double_complex coeff;
		double_complex * beta_u=new double_complex[n];
		double_complex * beta_v=new double_complex[n-1];
		double_complex * temp_arr=new double_complex[m*m];
		double_complex * v=new double_complex[m];
		
		memset(beta_u, 0x0, n*sizeof(double_complex));
		memset(beta_v, 0x0, (n-1)*sizeof(double_complex));
		memset(temp_arr, 0x0, m*m*sizeof(double_complex));
		memset(v, 0x0, m*sizeof(double_complex));
		
		for(i=0; i<n; ++i)
		{
			if(i!=(n-1)) beta_v[i]=0.;
			beta_u[i]=0.;
		}
		
		for(i=0; i<m; ++i)
		{
			v[i]=0.;
			for(j=0; j<m; ++j)
			{
				temp_arr[i*m+j]=0.;
			}
		}
	
	//memset(U, 0x0, m*m*sizeof(double_complex));
	
		for(i=0; i<m; i++)
		{
			U[i*m+i]=1.;
			if(i<n) V[i*n+i]=1.;
		}
	
		for(iter=0; iter<n; iter++)
		{
			beta_u[iter]=left_hand_householder(a, m, n, iter);
			if(iter<n-2)
			{
				beta_v[iter]=right_hand_householder(a, m, n, iter);
			}
		}
	
	// At this point A is bidiagonal. Its non-bidiagonal elements as well as beta_u and beta_v contain the information to reconstruct U and V
	
	// Reconstruction of the U matrix: Un=Un-1*Un, U0=Im and the householder vectors are stored in subdiag(a) and beta_u - backward accumulation for efficiency

		for(iter=n-1; iter>=0; iter--)
		{
			for(i=iter+1; i<m; i++)
			{
				v[i-iter]=a[i*n+iter];
				a[i*n+iter]=0.;
			}
			v[0]=1.;
		
			memcpy(temp_arr, U, m*m*sizeof(double_complex));
		
			for(i=iter; i<m; i++)
			{
				for(j=iter; j<m; j++)
				{
					coeff=0.;
					for(k=iter; k<m; k++)
					{
						coeff+=conj(v[k-iter])*temp_arr[k*m+j];
					}
					U[i*m+j]=temp_arr[i*m+j]-beta_u[iter]*v[i-iter]*coeff;
				}
			}
		}
	
	// Same thing for V

		for(iter=n-3; iter>=0; iter--)
		{
			for(i=iter+2; i<n; i++)
			{
				v[i-iter-1]=a[iter*n+i];
				a[iter*n+i]=0.;
			}
			v[0]=1.;
		
			memcpy(temp_arr, V, n*n*sizeof(double_complex));
		
			for(i=iter; i<n-1; i++)
			{
				for(j=iter; j<n-1; j++)
				{
					coeff=0.;
					for(k=iter; k<n-1; k++)
					{
						coeff+=(v[k-iter])*temp_arr[(k+1)*n+(j+1)];
					}
					V[(i+1)*n+(j+1)]=temp_arr[(i+1)*n+(j+1)]-beta_v[iter]*conj(v[i-iter])*coeff;
				}
			}
		}
	
		delete[] beta_u;
		delete[] beta_v;
		delete[] temp_arr;
		delete[] v;
	}

	void givens(double_complex a, double_complex b, double_complex * c, double_complex * s)
	{
		if(b==0.)
		{
			*c=1.;
			*s=0.;
		}
		else
		{
			if(a==0.)
			{
				*c=0.;
				*s=conj(b)/(abs(b));
			}
			else
			{
				*c=conj(a)/std::sqrt(abs(a)*abs(a)+abs(b)*abs(b));
				*s=conj(b)/std::sqrt(abs(a)*abs(a)+abs(b)*abs(b));
			}
		}
	}

	void bidiag2diag_step(double_complex * a, double_complex * u, double_complex * v, int m, int n, int p, int q)
	{
		int k, j;
	// if T=a*.a (with a=a(p:q-1, p:q-1))
	// find the eigenvalues of the trailing 2x2 submatrix of t

		double_complex t11(0.), t12(0.), t22(0.), t21(0.);
		double_complex mu, lambda1, lambda2, y(0.), z(0.);
		double_complex c, s;

		for(k=p; k<q; k++)
		{
			t11+=conj(a[k*n+q-2])*a[k*n+q-2];
			t21+=conj(a[k*n+q-1])*a[k*n+q-2];
			t12+=conj(a[k*n+q-2])*a[k*n+q-1];
			t22+=conj(a[k*n+q-1])*a[k*n+q-1];
		}
	
		lambda1=(t11+t22+std::sqrt(4.*t12*t21+(t11-t22)*(t11-t22)))/2.;
		lambda2=(t11+t22-std::sqrt(4.*t12*t21+(t11-t22)*(t11-t22)))/2.;
	
		if(abs(lambda1-t22)>abs(lambda2-t22))
		{
			mu=lambda2;
		}
		else
		{
			mu=lambda1;
		}

		for(k=p; k<q; k++)
		{
			y+=conj(a[k*n+p])*a[k*n+p];
			z+=conj(a[k*n+p])*a[k*n+p+1];
		}
		y-=mu;
	
		for(k=p; k<q-1; k++)
		{
			givens(y, z, &c, &s);
		// Apply givens to the right
			for(j=0; j<m; j++)
			{
				t11=a[j*n+k];
				t22=a[j*n+k+1];
				a[j*n+k]=c*t11+s*t22;
				a[j*n+k+1]=-conj(s)*t11+conj(c)*t22;			
			}
		
			for(j=0; j<n; j++)
			{
				t11=v[j*n+k];
				t22=v[j*n+k+1];
				v[j*n+k]=c*t11+s*t22;
				v[j*n+k+1]=-conj(s)*t11+conj(c)*t22;
			}
		
			y=a[k*n+k];
			z=a[(k+1)*n+k];
		
			givens(y, z, &c, &s);
		// Apply givens to the left
			for(j=0; j<n; j++)
			{
				t11=a[k*n+j];
				t22=a[(k+1)*n+j];
				a[k*n+j]=c*t11+s*t22;
				a[(k+1)*n+j]=-conj(s)*t11+conj(c)*t22;
			}
			for(j=0; j<m; j++)
			{
				t11=u[j*m+k];
				t22=u[j*m+k+1];
				u[j*m+k]=conj(c)*t11+conj(s)*t22;
				u[j*m+k+1]=-s*t11+c*t22;
			}
		
			if(k<q-2)
			{
				y=a[k*n+k+1];
				z=a[k*n+k+2];
			}
		}
	
	}	

// the very specific case where a is zero except on the superdiagonal is not handled properly by the algorithm
//This allows the detection of this case, for proper handling
	bool is_superdiagonal(double_complex *a, int m, int n)
	{
		int i;
		bool superdiagonal=true;
	
		for(i=0; i<n-1 && superdiagonal; i++)
		{
			superdiagonal=superdiagonal && (a[i*n+i]==0. && a[i*n+i+1] != 0.);
		}
	
		return superdiagonal;
	}

	void bidiag2diag(double_complex * a, double_complex * u, double_complex *v, int m, int n)
	{
		int p, q, i, end_loop, iter;
		p=0;
		q=n-1;
		end_loop=0;
	
		if(!is_superdiagonal(a, m, n))
		{
			while(q>p)
			{
				for(i=0; i<n-1; i++)
				{
					if(std::abs(a[i*n+i+1])<=1e-12*(std::abs(a[i*n+i])+std::abs((i+1)*n+i+1)))
					{
						a[i*n+i+1]=0.;
					}
				}
			
				end_loop=0;
				iter=q-1;
				while(!end_loop && iter>0)
				{
					if(a[iter*n+iter+1]==0.)
					{
						q=iter;
						iter--;
					}
					else
					{
						end_loop=1;
					}
				}
			
				end_loop=0;
				iter=p;
				while(!end_loop && iter<n-1)
				{
					if(a[iter*n+iter+1]==0.)
					{
						p=++iter;
					}
					else
					{
						end_loop=1;
					}
				}
			
				if(q>p)
				{
					bidiag2diag_step(a, u, v, m, n, p, q+1);
				}
			}
		
			for(p=0; p<n; p++)
			{
				double_complex temp=a[p*n+p];
//                 std::cout << "temp " << temp << std::endl;
				if(std::abs(std::imag(temp))>1e-12) // If not real (no givens transform applied)
				{
					temp=conj(temp)/abs(temp);
					a[p*n+p]*=temp;
					for(iter=0; iter<n; iter++)
					{
						v[iter*n+p]*=temp;
					}
				}
                if(std::real(a[p*n+p])<0)
                {
                    a[p*n+p]=-a[p*n+p];
                    for(iter=0; iter<n; iter++)
                    {
                        v[iter*n+p]=-v[iter*n+p];
                    }
                }
			}
		}
		else
		{
			double_complex factor, temp;
		// Case of superdiagonal-only bidiagonal matrix
			for(i=0; i<n; i++)
			{
				if(i!=n-1)
				{
				// The SVs are the modules of the superdiagonal entries, which become diagonal
					a[i*n+i]=a[i*n+i+1];
					a[i*n+i+1]=0.;
				}

			// The columns of V are switched to compensate for the columns switches of the SVs
				for(p=0; p<n-1; p++)
				{
					temp=v[i*n+p];
					v[i*n+p]=v[i*n+p+1];
					v[i*n+p+1]=temp;
				}
			
			}
		
		// A complex factor is introduced to compensate for taking the norms for the SVs	
			for(i=0; i<n; i++)
			{
				if(a[i*n+i]!=0.)
				{
					factor=conj(a[i*n+i])/abs(a[i*n+i]);
					a[i*n+i]=abs(a[i*n+i]);

				}
				else
				{
					factor=1.;
				}
			
				for(p=0; p<n; p++)
				{
					v[p*n+i]*=factor;
				}
			}
		}
	
	}


	
	void sort(double * q, double * u, double * v, int rows, int cols)
	{
		double temp;

		for(int i=0; i<cols; i++)
		{
			for(int j=0; j<cols-i-1; j++)
			{
				if(q[j]<q[j+1])
				{
					temp=q[j+1];
					q[j+1]=q[j];
					q[j]=temp;
				
					for(int k=0; k<rows; k++)
					{
						temp=u[k*rows+j+1];
						u[k*rows+j+1]=u[k*rows+j];
						u[k*rows+j]=temp;
					}
					
					for(int k=0; k<cols; k++)
					{
						temp=v[k*cols+j+1];
						v[k*cols+j+1]=v[k*cols+j];
						v[k*cols+j]=temp;
					}
				
				}
			}
		}
	}
	
	void sort(double * q, double_complex * u, double_complex * v, int rows, int cols)
	{
		double temp;
		double_complex temp2;

		for(int i=0; i<cols; i++)
		{
			for(int j=0; j<cols-i-1; j++)
			{
				if(q[j]<q[j+1])
				{
					temp=q[j+1];
					q[j+1]=q[j];
					q[j]=temp;
				
					for(int k=0; k<rows; k++)
					{
						temp2=u[k*rows+j+1];
						u[k*rows+j+1]=u[k*rows+j];
						u[k*rows+j]=temp2;
					}
					
					for(int k=0; k<cols; k++)
					{
						temp2=v[k*cols+j+1];
						v[k*cols+j+1]=v[k*cols+j];
						v[k*cols+j]=temp2;
					}
				
				}
			}
		}
	}
	
	void svd_golub_reinsch(double_complex * a, int m, int n, double * q, double_complex * u, double_complex * v)
	{

		int i, j;

		memset(u, 0x0, m*m*sizeof(double_complex));
		memset(v, 0x0, n*n*sizeof(double_complex));
	
		bidiag(a, m, n, u, v);

//         std::cout << "bidiag U" << std::endl;
//         std::cout << u[0] << " "<< u[1] << " " << u[2] << std::endl;
//         std::cout << u[3] << " "<< u[4] << " " << u[5] << std::endl;
//         std::cout << u[6] << " "<< u[7] << " " << u[8] << std::endl;
//         std::cout << "bidiag V" << std::endl;
//         std::cout << v[0] << " "<< v[1] << " " << v[2] << std::endl;
//         std::cout << v[3] << " "<< v[4] << " " << v[5] << std::endl;
//         std::cout << v[6] << " "<< v[7] << " " << v[8] << std::endl;
        
		bidiag2diag(a, u, v, m, n);

		for(int i=0; i<n; i++)
		{
//             std::cout << a[i*n+i] << std::endl;
            q[i]=std::abs(a[i*n+i]);
		}
		
		sort(q, u, v, m, n);
	}
	
	void svd_golub_reinsch(double * a, int m, int n, double * q, double * u, double * v)
	{

            //Quick fix for the double svd
            double_complex *A, *U, *V;
            A=new double_complex[m*n];
            U=new double_complex[m*m];
            V=new double_complex[n*n];
            for(int I=0; I<m*n; I++) A[I]=(double_complex)a[I];

            svd_golub_reinsch(A, m, n, q, U, V);

            for(int I=0; I<m*m; I++) u[I]=real(U[I]);
            for(int I=0; I<n*n; I++) v[I]=real(V[I]);

            delete[] A;
            delete[] U;
            delete[] V;
//             std::cout << "Q" << std::endl;
//             std::cout << q[0] << " "<< q[1] << " " << q[2] << std::endl;
//             std::cout << "U" << std::endl;
//             std::cout << u[0] << " "<< u[1] << " " << u[2] << std::endl;
//             std::cout << u[3] << " "<< u[4] << " " << u[5] << std::endl;
//             std::cout << u[6] << " "<< u[7] << " " << u[8] << std::endl;
//             std::cout << "V" << std::endl;
//             std::cout << v[0] << " "<< v[1] << " " << v[2] << std::endl;
//             std::cout << v[3] << " "<< v[4] << " " << v[5] << std::endl;
//             std::cout << v[6] << " "<< v[7] << " " << v[8] << std::endl;
            return;
            /////////////// end quick fix
		int i, j, k, l, l1;
		double c, f, g, h, s, x, y, z;
		double * e=new double[n];

		for(i=0; i<m; i++) for(j=0; j<n; j++) u[i*n+j]=a[i*n+j];
		
	// Householder's reduction to bidiagonal form
		
		g=x=0.;
	
		for(i=0; i<n; i++)
		{
			e[i]=g;
			s=0.;
			l=i+1;
			for(j=i; j<m; j++) s+=u[j*n+i]*u[j*n+i];
			if(s<EPSILON)
			{
				g=0.;
			}
			else
			{
				f=u[i*n+i];
				g=(f<0. ? std::sqrt(s) : -std::sqrt(s));
				h=f*g-s;
				u[i*n+i]=f-g;
			
				for(j=l; j<n; j++)
				{
					s=0.;
					for(k=i; k<m; k++) s+=u[k*n+i]*u[k*n+j];
					f=s/h;
					for(k=i; k<m; k++) u[k*n+j]+=f*u[k*n+i];
				}
			}
		
		// page 411
		
			q[i]=g;
			s=0.;
		
			for(j=l; j<n; j++) s+=u[i*n+j]*u[i*n+j];
		
			if(s<EPSILON)
			{
				g=0.;
			}
			else
			{
				f=u[i*n+i+1];
				g=(f<0. ? std::sqrt(s) : -std::sqrt(s));
				h=f*g-s;
				u[i*n+i+1]=f-g;
				for(j=l; j<n; j++) e[j]=u[i*n+j]/h;
				for(j=l; j<m; j++)
				{
					s=0.;
					for(k=l; k<n; k++) s+=u[j*n+k]*u[i*n+k];
					for(k=l; k<n; k++) u[j*n+k]+=s*e[k];
				}
			}
		
			y=fabs(q[i])+fabs(e[i]);
			if(y>x) x=y;
		}

	//exit(0);
	// Accumulation of right-hand transformations
	
	//if withv
		for(i=n-1; i>=0; i--)
		{
			if(g!=0.)
			{
				h=u[i*n+i+1]*g;
				for(j=l; j<n; j++) v[j*n+i]=u[i*n+j]/h;
				for(j=l; j<n; j++)
				{
					s=0.;
					for(k=l; k<n; k++) s+=u[i*n+k]*v[k*n+j];
					for(k=l; k<n; k++) v[k*n+j]+=s*v[k*n+i];
				}
			}
			for(j=l; j<n; j++) v[i*n+j]=v[j*n+i]=0.;
			v[i*n+i]=1.;
			g=e[i];
			l=i;
		}
	
	//Accumulation of left-hand transformations

	//if withu
		for(i=n-1; i>=0; i--)
		{
			l=i+1;
			g=q[i];
			for(j=l; j<n; j++) u[i*n+j]=0.;
		
			if(g!=0.)
			{
				h=u[i*n+i]*g;
				for(j=l; j<n; j++)
				{
				// page 412
					s=0.;
					for(k=l; k<m; k++) s+=u[k*n+i]*u[k*n+j];
					f=s/h;
					for(k=i; k<m; k++) u[k*n+j]+=f*u[k*n+i];
				}
				for(j=i; j<m; j++) u[j*n+i]=u[j*n+i]/g;
			}
			else
			{
				for(j=i; j<m; j++) u[j*n+i]=0.;
			}
			u[i*n+i]+=1.;
		}

	// Diagonalization of the bidiagonal form
		double eps=0.0001;
		eps=eps*x;
		for(k=n; k>=1; k--)
		{
		test_f_splitting_ref:
		
				for(l=k; l>=1; l--)
				{
					if(fabs(e[l-1])<=EPSILON) goto test_f_convergence_ref;
					if(fabs(q[l-2])<=EPSILON) goto cancellation_ref;
				}
		
		// Cancellation of e[l] if l>1
		
		cancellation_ref:
		
				c=0.;
		s=1.;
		l1=l-1;
		
		for(i=l; i<=k; i++)
		{
			f=s*e[i-1];
			e[i-1]=c*e[i-1];
			if(fabs(f)<=EPSILON) goto test_f_convergence_ref;
			g=q[i-1];
			h=q[i-1]=std::sqrt(f*f+g*g);
			c=g/h;
			s=-f/h;
			//if withu
			for(j=1; j<=m; j++)
			{
				y=u[(j-1)*n+l1-1];
				z=u[(j-1)*n+i-1];
				u[(j-1)*n+l1-1]=y*c+z*s;
				u[(j-1)*n+i-1]=-y*s+z*c;
			}
		}
		
		test_f_convergence_ref:
		
				z=q[k-1];
		if(l==k) goto convergence_ref;
		
		// shift from bottom 2*2 minor
		
		x=q[l-1];
		y=q[k-2];
		g=e[k-2];
		h=e[k-1];
		f=((y-z)*(y+z)+(g-h)*(g+h))/(2*h*y);
		g=std::sqrt(f*f+1);
		f=((x-z)*(x*z)+h*(y/(f<0. ? f-g : f+g)-h))/x;
		
		// next QR transformation
		
		c=s=1.;
		for(i=l+1; i<=k; i++)
		{
			//page 413
			g=e[i-1];
			y=q[i-1];
			h=s*g;
			g=c*g;
			e[i-2]=z=std::sqrt(f*f+h*h);
			c=f/z;
			s=h/z;
			f=x*c+g*s;
			g=-x*s+g*c;
			h=y*s;
			y=y*c;
			
			//if withv
			for(j=1; j<=n; j++)
			{
				x=v[(j-1)*n+i-2];
				z=v[(j-1)*n+i-1];
				v[(j-1)*n+i-2]=x*c+z*s;
				v[(j-1)*n+i-1]=-x*s+z*c;
			}
			q[i-2]=z=std::sqrt(f*f+h*h);
			c=f/z;
			s=h/z;
			f=c*g+s*y;
			x=-s*g+c*y;
			//if withu
			for(j=1; j<=m; j++)
			{
				y=u[(j-1)*n+i-2];
				z=u[(j-1)*n+i-1];
				u[(j-1)*n+i-2]=y*c+z*s;
				u[(j-1)*n+i-1]=-y*s+z*c;
			}
		}
		
		e[l-1]=0.;
		e[k-1]=f;
		q[k-1]=x;
		goto test_f_splitting_ref;
		
		convergence_ref:
		
				if(z<0.)
				{
			// q[k] is made non-negative
					q[k-1]=-z;
			//if withv
					for(j=1; j<=n; j++) v[(j-1)*n+k-1]=-v[(j-1)*n+k-1];
				}
		}
		delete[] e;
		
		sort(q, u, v, m, n);


	}

}  // end namespace QDOS        


  
