#include "Numeric.h"
#include "Matrix.cc"
#include "QdTime.h"
#include "Viostream.h"

namespace QDOS{

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// QdRScalar Numeric Operators/Functions
//------------------------------------------------------------------------------
void QdRScalar::put(Vostream& v_os)
{ const RScalar& rscalar=*this; v_os<<rscalar;}

void QdRScalar::get(Vistream& v_is)
{ RScalar& rscalar=*this; v_is >> rscalar;}

QdNObject_var QdRScalar::negate() const
{ const RScalar& rscalar=*this; return QdRScalar_var(new QdRScalar(-rscalar));}

QdNObject_var QdRScalar::add(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalar_var(new QdRScalar(*this+*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this+*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrix_var(new QdRMatrix(*this+*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this+*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalar_var(new QdCScalar(*this+*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this+*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this+*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdRScalar::sub(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalar_var(new QdRScalar(*this-*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this-*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrix_var(new QdRMatrix(*this-*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this-*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalar_var(new QdCScalar(*this-*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this-*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this-*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdRScalar::mult(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalar_var(new QdRScalar(*this**rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this**rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrix_var(new QdRMatrix(*this**rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this**rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalar_var(new QdCScalar(*this**cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this**cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this**cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdRScalar::div(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalar_var(new QdRScalar(*this/(*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this/(*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRScalar_var(new QdRScalar(*this/(*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this/(*rmatrixseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalar_var(new QdCScalar(*this/(*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this/(*cscalarseq)));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCScalar_var(new QdCScalar(*this/(*cmatrix)));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdRScalar::abs() const
{ return QdRScalar_var(new QdRScalar(QDOS::abs(*this)));}

QdNObject_var QdRScalar::acos() const
{ return QdRScalar_var(new QdRScalar(QDOS::acos(*this)));}

QdNObject_var QdRScalar::asin() const
{ return QdRScalar_var(new QdRScalar(QDOS::asin(*this)));}

QdNObject_var QdRScalar::atan() const
{ return QdRScalar_var(new QdRScalar(QDOS::atan(*this)));}

QdNObject_var QdRScalar::atan2(QdNObject_var obj) const
{ 
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalar_var(new QdRScalar(QDOS::atan2(*this,*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(QDOS::atan2(*this,*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrix_var(new QdRMatrix(QDOS::atan2(*this,*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::atan2(*this,*rmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdRScalar::cos() const
{ return QdRScalar_var(new QdRScalar(QDOS::cos(*this)));}

QdNObject_var QdRScalar::cosh() const
{ return QdRScalar_var(new QdRScalar(QDOS::cosh(*this)));}

QdNObject_var QdRScalar::exp() const
{ return QdRScalar_var(new QdRScalar(QDOS::exp(*this)));}

QdNObject_var QdRScalar::invert() const
{ return QdRScalar_var(new QdRScalar(this->pinverse()));}

QdNObject_var QdRScalar::log() const
{ return QdRScalar_var(new QdRScalar(QDOS::log(*this)));}

QdNObject_var QdRScalar::log10() const
{ return QdRScalar_var(new QdRScalar(QDOS::log10(*this)));}

QdNObject_var QdRScalar::mod() const
{ return QdRScalar_var(new QdRScalar(QDOS::norm(*this)));}

QdNObject_var QdRScalar::pow(QdNObject_var obj) const
{ 
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalar_var(new QdRScalar(QDOS::pow(*this,*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(QDOS::pow(*this,*rscalarseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalar_var(new QdCScalar(QDOS::pow(*this,*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::pow(*this,*cscalarseq)));
     throw BadNArg();
}

QdNObject_var QdRScalar::sin() const
{ return QdRScalar_var(new QdRScalar(QDOS::sin(*this)));}

QdNObject_var QdRScalar::sinh() const
{ return QdRScalar_var(new QdRScalar(QDOS::sinh(*this)));}

QdNObject_var QdRScalar::sqrt() const
{ return QdRScalar_var(new QdRScalar(QDOS::sqrt(*this)));}

bool QdRScalar::LT(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj);
    if(!rscalar.is_nil()){ return (*this < *rscalar ? 1 : 0 );}   
    throw BadNArg();    
}
bool QdRScalar::LE(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj);
    if(!rscalar.is_nil()){ return (*this <= *rscalar ? 1 : 0 );}
   throw BadNArg();    
}
bool QdRScalar::EQ(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj);
    if(!rscalar.is_nil()){ return (*this == *rscalar ? 1 : 0 );}
    throw BadNArg();    
}
bool QdRScalar::GE(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj);
    if(!rscalar.is_nil()){ return (*this >= *rscalar ? 1 : 0 );} 
    throw BadNArg();    
}
bool QdRScalar::GT(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj);   
    if(!rscalar.is_nil()){ return (*this > *rscalar ? 1 : 0 );}
    throw BadNArg();    
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// QdCScalar Numeric Operators/Functions
//------------------------------------------------------------------------------
void QdCScalar::put(Vostream& v_os)
{ const CScalar& cscalar=*this; v_os << cscalar;}

void QdCScalar::get(Vistream& v_is)
{ CScalar& cscalar=*this; v_is >> cscalar;}

QdNObject_var QdCScalar::negate() const
{ const CScalar& cscalar=*this; return QdCScalar_var(new QdCScalar(-cscalar));}

QdNObject_var QdCScalar::add(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCScalar_var(new QdCScalar(*this+*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this+*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this+*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalar_var(new QdCScalar(*this+*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this+*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this+*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdCScalar::sub(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCScalar_var(new QdCScalar(*this-*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this-*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this-*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalar_var(new QdCScalar((*this)-(*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this-*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this-*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdCScalar::mult(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCScalar_var(new QdCScalar(*this**rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this**rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this**rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalar_var(new QdCScalar(*this**cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this**cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this**cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdCScalar::div(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCScalar_var(new QdCScalar(*this/(*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this/(*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCScalar_var(new QdCScalar(*this/(*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*rmatrixseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalar_var(new QdCScalar(*this/(*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this/(*cscalarseq)));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCScalar_var(new QdCScalar(*this/(*cmatrix)));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdCScalar::abs() const
{ return QdCScalar_var(new QdCScalar(QDOS::abs(*this)));}

QdNObject_var QdCScalar::cos() const
{ return QdCScalar_var(new QdCScalar(QDOS::cos(*this)));}

QdNObject_var QdCScalar::cosh() const
{ return QdCScalar_var(new QdCScalar(QDOS::cosh(*this)));}

QdNObject_var QdCScalar::exp() const
{ return QdCScalar_var(new QdCScalar(QDOS::exp(*this)));}

QdNObject_var QdCScalar::invert() const
{ return QdCScalar_var(new QdCScalar(this->pinverse()));}

QdNObject_var QdCScalar::log() const
{ return QdCScalar_var(new QdCScalar(QDOS::log(*this)));}

QdNObject_var QdCScalar::log10() const
{ return QdCScalar_var(new QdCScalar(QDOS::log10(*this)));}

QdNObject_var QdCScalar::mod() const
{ return QdCScalar_var(new QdCScalar(QDOS::norm(*this)));}

QdNObject_var QdCScalar::pow(QdNObject_var obj) const
{ 
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCScalar_var(new QdCScalar(QDOS::pow(*this,*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::pow(*this,*rscalarseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalar_var(new QdCScalar(QDOS::pow(*this,*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::pow(*this,*cscalarseq)));
    throw BadNArg();
}

QdNObject_var QdCScalar::sin() const
{ return QdCScalar_var(new QdCScalar(QDOS::sin(*this)));}

QdNObject_var QdCScalar::sinh() const
{ return QdCScalar_var(new QdCScalar(QDOS::sinh(*this)));}

QdNObject_var QdCScalar::sqrt() const
{ return QdCScalar_var(new QdCScalar(QDOS::sqrt(*this)));}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// QdSMatrix Operators/Functions
//------------------------------------------------------------------------------
void QdSMatrix::put(Vostream& v_os)
{ const SMatrix& smatrix=*this; v_os << smatrix;}

void QdSMatrix::get(Vistream& v_is)
{ SMatrix& smatrix=*this; v_is >> smatrix;}

QdNObject_var QdSMatrix::add(QdNObject_var obj) const
{
    QdSMatrix_var smatrix=QdSMatrix_var::narrow(obj);
    if(!smatrix.is_nil())
        return QdSMatrix_var(new QdSMatrix(*this+*smatrix));
    QdSMatrixSeq_var smatrixseq=QdSMatrixSeq_var::narrow(obj);
    if(!smatrixseq.is_nil())
    {
        QdSMatrixSeq_var smatrixseq2=new QdSMatrixSeq(*smatrixseq);
        smatrixseq2->operator+=(*this);
        return smatrixseq2;
    }
    throw BadNArg();
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// QdRMatrix Numeric Operators/Functions
//------------------------------------------------------------------------------
void QdRMatrix::put(Vostream& v_os)
{ const RMatrix& rmatrix=*this; v_os << rmatrix;}

void QdRMatrix::get(Vistream& v_is)
{ RMatrix& rmatrix=*this; v_is >> rmatrix;}

QdNObject_var QdRMatrix::negate() const
{ const RMatrix& rmatrix=*this; return QdRMatrix_var(new QdRMatrix(-rmatrix));}

QdNObject_var QdRMatrix::add(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRMatrix_var(new QdRMatrix(*this+*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this+*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrix_var(new QdRMatrix(*this+*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this+*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this+*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this+*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdRMatrix::sub(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRMatrix_var(new QdRMatrix(*this-*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this-*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrix_var(new QdRMatrix(*this-*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this-*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this-*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this-*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdRMatrix::mult(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRMatrix_var(new QdRMatrix(*this**rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this**rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
    { 
      QdRMatrix_var res=new QdRMatrix(*this**rmatrix);
      if(res->rank()==0)
        return new QdRScalar(*res);
      else  
        return res;
    }
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
    { 
      QdRMatrixSeq_var res=new QdRMatrixSeq(*this**rmatrixseq);
      if(res->is_scalar_seq())
        return new QdRScalarSeq(*res);
      else  
        return res;
    }
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this**cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
    { 
      QdCMatrix_var res=new QdCMatrix(*this**cmatrix);
      if(res->rank()==0)
        return new QdCScalar(*res);
      else  
        return res;
    }
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
    { 
      QdCMatrixSeq_var res=new QdCMatrixSeq(*this**cmatrixseq);
      if(res->is_scalar_seq())
        return new QdCScalarSeq(*res);
      else  
        return res;
    }
    throw BadNArg();
}

QdNObject_var QdRMatrix::div(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRMatrix_var(new QdRMatrix(*this/(*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this/(*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrix_var(new QdRMatrix(*this/(*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this/(*rmatrixseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this/(*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cscalarseq)));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this/(*cmatrix)));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdRMatrix::abs() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::abs(*this)));}

QdNObject_var QdRMatrix::acos() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::acos(*this)));}

QdNObject_var QdRMatrix::asin() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::asin(*this)));}

QdNObject_var QdRMatrix::atan() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::atan(*this)));}

QdNObject_var QdRMatrix::atan2(QdNObject_var obj) const
{ 
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRMatrix_var(new QdRMatrix(QDOS::atan2(*this,*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::atan2(*this,*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrix_var(new QdRMatrix(QDOS::atan2(*this,*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::atan2(*this,*rmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdRMatrix::cos() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::cos(*this)));}

QdNObject_var QdRMatrix::cosh() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::cosh(*this)));}

QdNObject_var QdRMatrix::exp() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::exp(*this)));}

QdNObject_var QdRMatrix::inner(QdNObject_var obj) const
{
    QdRMatrix_var rmatrix2=QdRMatrix_var::narrow(obj); 
    if(!rmatrix2.is_nil())
    {
        const RMatrix& rmatrix1=*this;
        return QdRScalar_var(new QdRScalar(rmatrix1.dot(*rmatrix2)));
    }
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(QDOS::dot(*this,*rmatrixseq)));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj); 
    if(!cmatrix.is_nil())
    {   
        const RMatrix& rmatrix1=*this;
        return QdCScalar_var(new QdCScalar(cmatrix->dot(rmatrix1)));
    }
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::dot(*this,*cmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdRMatrix::invert() const
{ 
    size_t ndims=rank();
    if(ndims==0 || ndims==1)
        return QdRScalar_var(new QdRScalar(this->pinverse()));
    else
        return QdRMatrix_var(new QdRMatrix(this->pinverse()));
}

QdNObject_var QdRMatrix::log() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::log(*this)));}

QdNObject_var QdRMatrix::log10() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::log10(*this)));}

QdNObject_var QdRMatrix::mod() const
{ return QdRScalar_var(new QdRScalar(QDOS::norm(*this)));}

QdNObject_var QdRMatrix::outer(QdNObject_var obj) const
{ 
    QdRMatrix_var rmatrix2=QdRMatrix_var::narrow(obj); 
    if(!rmatrix2.is_nil())
    {
        const RMatrix& rmatrix1=*this;
        return QdRMatrix_var(new QdRMatrix(rmatrix1.cross(*rmatrix2)));
    }
    QdRMatrixSeq_var rmatrixs2=QdRMatrixSeq_var::narrow(obj); 
    if(!rmatrixs2.is_nil())
    {
        const RMatrix& rmatrix1=*this;
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::cross(rmatrix1, *rmatrixs2)));
    }
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj); 
    if(!cmatrix.is_nil())
    {
        const RMatrix& rmatrix=*this;
        return QdCMatrix_var(new QdCMatrix(cmatrix->cross(rmatrix)));
    }
    throw BadNArg();
}

QdNObject_var QdRMatrix::pow(QdNObject_var obj) const
{ 
    bool scalar_result=(rank()==0 || rank()==1);
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
    {
        if(scalar_result)
            return QdRScalar_var(new QdRScalar(this->operator^(*rscalar)));
        else
            return QdRMatrix_var(new QdRMatrix(this->operator^(*rscalar)));
    }
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
    {
        if(scalar_result)
            return QdRScalarSeq_var(\
                new QdRScalarSeq(this->operator^(*rscalarseq)));
        else
            return QdRMatrixSeq_var(\
                new QdRMatrixSeq(this->operator^(*rscalarseq)));
    }
    throw BadNArg();
}

QdNObject_var QdRMatrix::sin() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::sin(*this)));}

QdNObject_var QdRMatrix::sinh() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::sinh(*this)));}

QdNObject_var QdRMatrix::sqrt() const
{ return QdRMatrix_var(new QdRMatrix(QDOS::sqrt(*this)));}

bool QdRMatrix::LT(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj); 
    QdRMatrix_var rmat = QdRMatrix_var::narrow(nobj); 
    if(rscalar.is_nil() && rmat.is_nil()){
       cout<<"Object is neither a scalar nor a matrix.\n";
       return 0;
    }
    QdRScalarSeq_var tmpseq1, tmpseq2;
    QdRMatrix_var mat = new QdRMatrix(*this); 
    if(!mat.is_nil()){
       if(!rscalar.is_nil()){           
           for(int i=0; i<(mat->dimdata())[0]; i++){
              tmpseq1 = new QdRScalarSeq((*mat)[i].matrix_to_seq());
	      for(int j=0; j<tmpseq1->sequence_size(); j++)	     
	          if(tmpseq1[j] >= *rscalar ) return 0;
	   }	    
       }  
       if(!rmat.is_nil()){    
          if(rmat->size() != mat->size() ){
	      cout<<"Error matrices of different size.\n";
              return 0;
          }    
          if((rmat->dims())[0] != (mat->dims())[0]){
	      cout<<"Matrices of different dimensions.\n";
              return 0;
          }    	   
          for(int i=0; i<(mat->dimdata())[0]; i++){
              tmpseq1 = new QdRScalarSeq((*mat)[i].matrix_to_seq());
	      tmpseq2 = new QdRScalarSeq((*rmat)[i].matrix_to_seq());     
              if( tmpseq1->GE(tmpseq2) )
	          return 0;
	   }	    
	}	
        return 1;
    }    	
    throw BadNArg();    
} 

bool QdRMatrix::LE(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj); 
    QdRMatrix_var rmat = QdRMatrix_var::narrow(nobj); 
    if(rscalar.is_nil() && rmat.is_nil()){
       cout<<"Object is neither a scalar nor a matrix.\n";
       return 0;
    }
    QdRScalarSeq_var tmpseq1, tmpseq2;
    QdRMatrix_var mat = new QdRMatrix(*this); 
    if(!mat.is_nil()){
       if(!rscalar.is_nil()){           
           for(int i=0; i<(mat->dimdata())[0]; i++){
              tmpseq1 = new QdRScalarSeq((*mat)[i].matrix_to_seq());
	      for(int j=0; j<tmpseq1->sequence_size(); j++)	     
	          if(tmpseq1[j] > *rscalar ) return 0;
	   }	    
       }  
       if(!rmat.is_nil()){    
          if(rmat->size() != mat->size() ){
	      cout<<"Error matrices of different size.\n";
              return 0;
          }    
          if((rmat->dims())[0] != (mat->dims())[0]){
	      cout<<"Matrices of different dimensions.\n";
              return 0;
          }    	   
          for(int i=0; i<(mat->dimdata())[0]; i++){
              tmpseq1 = new QdRScalarSeq((*mat)[i].matrix_to_seq());
	      tmpseq2 = new QdRScalarSeq((*rmat)[i].matrix_to_seq());     
              if( tmpseq1->GT(tmpseq2) )
	          return 0;
	   }	    
	}	
        return 1;
    }    	
    throw BadNArg();    
} 

bool QdRMatrix::EQ(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj); 
    QdRMatrix_var rmat = QdRMatrix_var::narrow(nobj); 
    if(rscalar.is_nil() && rmat.is_nil()){
       cout<<"Object is neither a scalar nor a matrix.\n";
       return 0;
    }
    QdRScalarSeq_var tmpseq1, tmpseq2;
    QdRMatrix_var mat = new QdRMatrix(*this); 
    if(!mat.is_nil()){
       if(!rscalar.is_nil()){           
           for(int i=0; i<(mat->dimdata())[0]; i++){
              tmpseq1 = new QdRScalarSeq((*mat)[i].matrix_to_seq());
	      for(int j=0; j<tmpseq1->sequence_size(); j++)	     
	          if(tmpseq1[j] != *rscalar ) return 0;
	   }	    
       }  
       if(!rmat.is_nil()){    
          if(rmat->size() != mat->size() ){
	      cout<<"Error matrices of different size.\n";
              return 0;
          }    
          if((rmat->dims())[0] != (mat->dims())[0]){
	      cout<<"Matrices of different dimensions.\n";
              return 0;
          }    	  
	  int i = 0;
	  while( i <(mat->dimdata())[0]){ 
              tmpseq1 = new QdRScalarSeq((*mat)[i].matrix_to_seq());
	      tmpseq2 = new QdRScalarSeq((*rmat)[i].matrix_to_seq());
              if(tmpseq1->EQ(tmpseq2) ) i++;
	      else return 0;
	  }
	}	    	
        return 1;
    }    	
    throw BadNArg();    
}
 
bool QdRMatrix::GE(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj); 
    QdRMatrix_var rmat = QdRMatrix_var::narrow(nobj); 
    if(rscalar.is_nil() && rmat.is_nil()){
       cout<<"Object is neither a scalar nor a matrix.\n";
       return 0;
    }
    QdRScalarSeq_var tmpseq1, tmpseq2;
    QdRMatrix_var mat = new QdRMatrix(*this); 
    if(!mat.is_nil()){
       if(!rscalar.is_nil()){           
           for(int i=0; i<(mat->dimdata())[0]; i++){
              tmpseq1 = new QdRScalarSeq((*mat)[i].matrix_to_seq());
	      for(int j=0; j<tmpseq1->sequence_size(); j++)	     
	          if(tmpseq1[j] < *rscalar ) return 0;
	   }	    
       }  
       if(!rmat.is_nil()){    
          if(rmat->size() != mat->size() ){
	      cout<<"Error matrices of different size.\n";
              return 0;
          }    
          if((rmat->dims())[0] != (mat->dims())[0]){
	      cout<<"Matrices of different dimensions.\n";
              return 0;
          }    	   
          for(int i=0; i<(mat->dimdata())[0]; i++){
              tmpseq1 = new QdRScalarSeq((*mat)[i].matrix_to_seq());
	      tmpseq2 = new QdRScalarSeq((*rmat)[i].matrix_to_seq());     
              if( tmpseq1->LT(tmpseq2) )
	          return 0;
	   }	    
	}	
        return 1;
    }    	
    throw BadNArg();    
} 
bool QdRMatrix::GT(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj); 
    QdRMatrix_var rmat = QdRMatrix_var::narrow(nobj); 
    if(rscalar.is_nil() && rmat.is_nil()){
       cout<<"Object is neither a scalar nor a matrix.\n";
       return 0;
    }
    QdRScalarSeq_var tmpseq1, tmpseq2;
    QdRMatrix_var mat = new QdRMatrix(*this); 
    if(!mat.is_nil()){
       if(!rscalar.is_nil()){           
           for(int i=0; i<(mat->dimdata())[0]; i++){
              tmpseq1 = new QdRScalarSeq((*mat)[i].matrix_to_seq());
	      for(int j=0; j<tmpseq1->sequence_size(); j++)	     
	          if(tmpseq1[j] <= *rscalar ) return 0;
	   }	    
       }  
       if(!rmat.is_nil()){    
          if(rmat->size() != mat->size() ){
	      cout<<"Error matrices of different size.\n";
              return 0;
          }    
          if((rmat->dims())[0] != (mat->dims())[0]){
	      cout<<"Matrices of different dimensions.\n";
              return 0;
          }    	   
          for(int i=0; i<(mat->dimdata())[0]; i++){
              tmpseq1 = new QdRScalarSeq((*mat)[i].matrix_to_seq());
	      tmpseq2 = new QdRScalarSeq((*rmat)[i].matrix_to_seq());     
              if( tmpseq1->LE(tmpseq2) )
	          return 0;
	   }	    
	}	
        return 1;
    }    	
    throw BadNArg();    
} 
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// QdCMatrix Numeric Operators/Functions
//------------------------------------------------------------------------------
void QdCMatrix::put(Vostream& v_os)
{ const CMatrix& cmatrix=*this; v_os << cmatrix;}

void QdCMatrix::get(Vistream& v_is)
{ CMatrix& cmatrix=*this; v_is >> cmatrix;}

QdNObject_var QdCMatrix::negate() const
{ const CMatrix& cmatrix=*this; return QdCMatrix_var(new QdCMatrix(-cmatrix));}

QdNObject_var QdCMatrix::add(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this+*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this+*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this+*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this+*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdCMatrix::sub(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this-*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this-*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this-*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this-*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdCMatrix::mult(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this**rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
    { 
      QdCMatrix_var res=new QdCMatrix(*this**rmatrix);
      if(res->rank()==0)
        return new QdCScalar(*res);
      else  
        return res;
    }
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
    { 
      QdCMatrixSeq_var res=new QdCMatrixSeq(*this**rmatrixseq);
      if(res->is_scalar_seq())
        return new QdCScalarSeq(*res);
      else  
        return res;
    }
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this**cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
    { 
      QdCMatrix_var res=new QdCMatrix(*this**cmatrix);
      if(res->rank()==0)
        return new QdCScalar(*res);
      else  
        return res;
    }
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
    { 
      QdCMatrixSeq_var res=new QdCMatrixSeq(*this**cmatrixseq);
      if(res->is_scalar_seq())
        return new QdCScalarSeq(*res);
      else  
        return res;
    }
    throw BadNArg();
}

QdNObject_var QdCMatrix::div(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this/(*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this/(*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*rmatrixseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this/(*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cscalarseq)));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrix_var(new QdCMatrix(*this/(*cmatrix)));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdCMatrix::abs() const
{ return QdCMatrix_var(new QdCMatrix(QDOS::abs(*this)));}

QdNObject_var QdCMatrix::cos() const
{ return QdCMatrix_var(new QdCMatrix(QDOS::cos(*this)));}

QdNObject_var QdCMatrix::cosh() const
{ return QdCMatrix_var(new QdCMatrix(QDOS::cosh(*this)));}

QdNObject_var QdCMatrix::exp() const
{ return QdCMatrix_var(new QdCMatrix(QDOS::exp(*this)));}

QdNObject_var QdCMatrix::inner(QdNObject_var obj) const
{
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj); 
    if(!rmatrix.is_nil())
    {   
         const QdCMatrix& cmatrix=*this;
         return QdCScalar_var(new QdCScalar(cmatrix.dot(*rmatrix)));
    }
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::dot(*this,*rmatrixseq)));
    QdCMatrix_var cmatrix2=QdCMatrix_var::narrow(obj); 
    if(!cmatrix2.is_nil())
    {
        const QdCMatrix& cmatrix1=*this;
        return QdCScalar_var(new QdCScalar(cmatrix1.dot(*cmatrix2)));
    }
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::dot(*this,*cmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdCMatrix::invert() const
{ 
    size_t ndims=rank();
    if(ndims==0 || ndims==1)
        return QdCScalar_var(new QdCScalar(this->pinverse()));
    else
        return QdCMatrix_var(new QdCMatrix(this->pinverse()));
}

QdNObject_var QdCMatrix::log() const
{ return QdCMatrix_var(new QdCMatrix(QDOS::log(*this)));}

QdNObject_var QdCMatrix::log10() const
{ return QdCMatrix_var(new QdCMatrix(QDOS::log10(*this)));}

QdNObject_var QdCMatrix::mod() const
{ return QdCScalar_var(new QdCScalar(QDOS::norm(*this)));}

QdNObject_var QdCMatrix::outer(QdNObject_var obj) const
{ 
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj); 
    if(!rmatrix.is_nil())
    {
        const QdCMatrix& cmatrix=*this;
        return QdCMatrix_var(new QdCMatrix(cmatrix.cross(*rmatrix)));
    }
    QdCMatrix_var cmatrix2=QdCMatrix_var::narrow(obj); 
    if(!cmatrix2.is_nil())
    {
        const QdCMatrix& cmatrix1=*this;
        return QdCMatrix_var(new QdCMatrix(cmatrix1.cross(*cmatrix2)));
    }
    throw BadNArg();
}

QdNObject_var QdCMatrix::pow(QdNObject_var obj) const
{
    bool scalar_result=(rank()==0 || rank()==1);
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
    {
        if(scalar_result)
            return QdCScalar_var(new QdCScalar(this->operator^(*rscalar)));
        else
            return QdCMatrix_var(new QdCMatrix(this->operator^(*rscalar)));
    }
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
    {
        if(scalar_result)
            return QdCScalarSeq_var(\
                new QdCScalarSeq(this->operator^(*rscalarseq)));
        else
            return QdCMatrixSeq_var(\
                new QdCMatrixSeq(this->operator^(*rscalarseq)));
    }
    throw BadNArg(); 
  
}

QdNObject_var QdCMatrix::sin() const
{ return QdCMatrix_var(new QdCMatrix(QDOS::sin(*this)));}

QdNObject_var QdCMatrix::sinh() const
{ return QdCMatrix_var(new QdCMatrix(QDOS::sinh(*this)));}

QdNObject_var QdCMatrix::sqrt() const
{ return QdCMatrix_var(new QdCMatrix(QDOS::sqrt(*this)));}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// QdRScalarSeq Numeric Operators/Functions
//------------------------------------------------------------------------------
void QdRScalarSeq::put(Vostream& v_os)
{ const vector<RScalar>& rscalar_vec=*this;v_os << rscalar_vec;}

void QdRScalarSeq::get(Vistream& v_is)
{ vector<RScalar>& rscalar_vec=*this; v_is >> rscalar_vec;}

QdNObject_var QdRScalarSeq::negate() const
{ const RScalarSeq& rscalarseq=*this;
  return QdRScalarSeq_var(new QdRScalarSeq(-rscalarseq));}

QdNObject_var QdRScalarSeq::add(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this+*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this+*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this+*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this+*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this+*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this+*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdRScalarSeq::sub(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this-*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this-*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this-*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this-*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this-*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this-*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdRScalarSeq::mult(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this**rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this**rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this**rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this**rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this**cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this**cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdRScalarSeq::div(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this/(*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(*this/(*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this/(*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this/(*rmatrixseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this/(*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this/(*cscalarseq)));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrix)));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdRScalarSeq::abs() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::abs(*this)));}

QdNObject_var QdRScalarSeq::acos() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::acos(*this)));}

QdNObject_var QdRScalarSeq::asin() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::asin(*this)));}

QdNObject_var QdRScalarSeq::atan() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::atan(*this)));}

QdNObject_var QdRScalarSeq::atan2(QdNObject_var obj) const
{ 
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(QDOS::atan2(*this,*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(QDOS::atan2(*this,*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::atan2(*this,*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::atan2(*this,*rmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdRScalarSeq::cos() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::cos(*this)));}

QdNObject_var QdRScalarSeq::cosh() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::cosh(*this)));}

QdNObject_var QdRScalarSeq::exp() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::exp(*this)));}

QdNObject_var QdRScalarSeq::invert() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::pinverse(*this)));}


QdNObject_var QdRScalarSeq::log() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::log(*this)));}

QdNObject_var QdRScalarSeq::log10() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::log10(*this)));}

QdNObject_var QdRScalarSeq::mod() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::norm(*this)));}

QdNObject_var QdRScalarSeq::pow(QdNObject_var obj) const
{ 
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(QDOS::pow(*this,*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(QDOS::pow(*this,*rscalarseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::pow(*this,*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::pow(*this,*cscalarseq)));
    throw BadNArg();
}

QdNObject_var QdRScalarSeq::sin() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::sin(*this)));}

QdNObject_var QdRScalarSeq::sinh() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::sinh(*this)));}

QdNObject_var QdRScalarSeq::sqrt() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::sqrt(*this)));}

bool QdRScalarSeq::LT(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj);   
    QdRScalarSeq_var rscalarseq = QdRScalarSeq_var::narrow(nobj); 
    if(rscalar.is_nil() && rscalarseq.is_nil()) return 0;
    QdRScalarSeq_var rseq = new QdRScalarSeq(*this); 
    if(!rseq.is_nil()){   
        if(!rscalar.is_nil()){     
          for(int i=0; i<rseq->sequence_size(); i++){
             if(rseq[i] >= *rscalar) return 0;
          }
	}
	if(!rscalarseq.is_nil()){     
	  if(rscalarseq->sequence_size() != rseq->sequence_size()){
	     cout <<"Error Objects of different size.\n";
	     return 0;
	  }
          for(int i=0; i<rseq->sequence_size(); i++){
             if(rseq[i] >= rscalarseq[i]) return 0;
          }
	}
       return 1;
    }	
    throw BadNArg();    
}
bool QdRScalarSeq::LE(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj);   
    QdRScalarSeq_var rscalarseq = QdRScalarSeq_var::narrow(nobj); 
    if(rscalar.is_nil() && rscalarseq.is_nil()) return 0;
    QdRScalarSeq_var rseq = new QdRScalarSeq(*this); 
    if(!rseq.is_nil()){   
        if(!rscalar.is_nil()){     
          for(int i=0; i<rseq->sequence_size(); i++){
             if(rseq[i] > *rscalar) return 0;
          }
	}
	if(!rscalarseq.is_nil()){     
	  if(rscalarseq->sequence_size() != rseq->sequence_size()){
	     cout <<"Error QdObjects of different size.\n";
	     return 0;
	  }
          for(int i=0; i<rseq->sequence_size(); i++){
             if(rseq[i] > rscalarseq[i]) return 0;
          }
	}
       return 1;
    }	
    throw BadNArg();    
}
bool QdRScalarSeq::EQ(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj);   
    QdRScalarSeq_var rscalarseq = QdRScalarSeq_var::narrow(nobj); 
    if(rscalar.is_nil() && rscalarseq.is_nil()) return 0;
    QdRScalarSeq_var rseq = new QdRScalarSeq(*this); 
    if(!rseq.is_nil()){   
        if(!rscalar.is_nil()){     
          for(int i=0; i<rseq->sequence_size(); i++){
             if(rseq[i] != *rscalar) return 0;
          }
	}
	if(!rscalarseq.is_nil()){     
	  if(rscalarseq->sequence_size() != rseq->sequence_size()){
	     cout <<"Error QdObjects of different size.\n";
	     return 0;
	  }
          for(int i=0; i<rseq->sequence_size(); i++){
             if(rseq[i] != rscalarseq[i]) return 0;
          }
	}
       return 1;
    }	
    throw BadNArg();    
}
bool QdRScalarSeq::GE(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj);   
    QdRScalarSeq_var rscalarseq = QdRScalarSeq_var::narrow(nobj); 
    if(rscalar.is_nil() && rscalarseq.is_nil()) return 0;
    QdRScalarSeq_var rseq = new QdRScalarSeq(*this); 
    if(!rseq.is_nil()){   
        if(!rscalar.is_nil()){     
          for(int i=0; i<rseq->sequence_size(); i++){
             if(rseq[i] < *rscalar) return 0;
          }
	}
	if(!rscalarseq.is_nil()){ 
	  if(rscalarseq->sequence_size() != rseq->sequence_size()){
	     cout <<"Error QdObjects of different size.\n";
	     return 0;
	  }
          for(int i=0; i<rseq->sequence_size(); i++){
             if(rseq[i] < rscalarseq[i]) return 0;
          }
	}
        return 1;
    }	
    throw BadNArg();    
}
bool QdRScalarSeq::GT(QdNObject_var nobj) const 
{ 
    QdRScalar_var rscalar = QdRScalar_var::narrow(nobj);   
    QdRScalarSeq_var rscalarseq = QdRScalarSeq_var::narrow(nobj); 
    if(rscalar.is_nil() && rscalarseq.is_nil()) return 0;
    QdRScalarSeq_var rseq = new QdRScalarSeq(*this); 
    if(!rseq.is_nil()){   
        if(!rscalar.is_nil()){     
          for(int i=0; i<rseq->sequence_size(); i++){
             if(rseq[i] <= *rscalar) return 0;
          }
	}
	if(!rscalarseq.is_nil()){     
	  if(rscalarseq->sequence_size() != rseq->sequence_size()){
	     cout <<"Error QdObjects of different size.\n";
	     return 0;
	  }
          for(int i=0; i<rseq->sequence_size(); i++){
             if(rseq[i] <= rscalarseq[i]) return 0;
          }
	}
       return 1;
    }	
    throw BadNArg();    
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// QdCScalarSeq Numeric Operators/Functions
//------------------------------------------------------------------------------
void QdCScalarSeq::put(Vostream& v_os)
{ const vector<CScalar>& cscalar_vec=*this;v_os << cscalar_vec;}

void QdCScalarSeq::get(Vistream& v_is)
{ vector<CScalar>& cscalar_vec=*this; v_is >> cscalar_vec;}

QdNObject_var QdCScalarSeq::negate() const
{ const CScalarSeq& cscalarseq=*this;
  return QdCScalarSeq_var(new QdCScalarSeq(-cscalarseq));}

QdNObject_var QdCScalarSeq::add(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this+*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this+*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this+*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this+*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdCScalarSeq::sub(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this-*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this-*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this-*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this-*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdCScalarSeq::mult(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this**rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this**rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this**cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this**cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdCScalarSeq::div(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this/(*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this/(*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*rmatrixseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this/(*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(*this/(*cscalarseq)));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrix)));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdCScalarSeq::abs() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::abs(*this)));}

QdNObject_var QdCScalarSeq::cos() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::cos(*this)));}

QdNObject_var QdCScalarSeq::cosh() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::cosh(*this)));}

QdNObject_var QdCScalarSeq::exp() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::exp(*this)));}

QdNObject_var QdCScalarSeq::invert() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::pinverse(*this)));}


QdNObject_var QdCScalarSeq::log() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::log(*this)));}

QdNObject_var QdCScalarSeq::log10() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::log10(*this)));}

QdNObject_var QdCScalarSeq::mod() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::norm(*this)));}

QdNObject_var QdCScalarSeq::pow(QdNObject_var obj) const
{ 
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::pow(*this,*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::pow(*this,*rscalarseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::pow(*this,*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::pow(*this,*cscalarseq)));
    throw BadNArg();
}

QdNObject_var QdCScalarSeq::sin() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::sin(*this)));}

QdNObject_var QdCScalarSeq::sinh() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::sinh(*this)));}

QdNObject_var QdCScalarSeq::sqrt() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::sqrt(*this)));}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// QdSMatrixSeq Operators/Functions
//------------------------------------------------------------------------------
void QdSMatrixSeq::put(Vostream& v_os)
{ const vector<SMatrix>& smatrix_vec=*this;v_os << smatrix_vec;}

void QdSMatrixSeq::get(Vistream& v_is)
{ vector<SMatrix>& smatrix_vec=*this; v_is >> smatrix_vec;}

QdNObject_var QdSMatrixSeq::add(QdNObject_var obj) const
{
    QdSMatrix_var smatrix=QdSMatrix_var::narrow(obj);
    if(!smatrix.is_nil())
    {
        QdSMatrixSeq_var smatrixseq=new QdSMatrixSeq(*this);
        smatrixseq->operator+=(*smatrix);
        return smatrixseq;
    }
    QdSMatrixSeq_var smatrixseq=QdSMatrixSeq_var::narrow(obj);
    if(!smatrixseq.is_nil())
    {
        QdSMatrixSeq_var smatrixseq2=new QdSMatrixSeq(*this);
        smatrixseq2->operator+=(*smatrixseq);
        return smatrixseq2;
    }
    throw BadNArg();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// QdRMatrixSeq Numeric Operators/Functions
//------------------------------------------------------------------------------
void QdRMatrixSeq::put(Vostream& v_os)
{ const vector<RMatrix>& rmatrix_vec=*this;v_os << rmatrix_vec;}

void QdRMatrixSeq::get(Vistream& v_is)
{ vector<RMatrix>& rmatrix_vec=*this; v_is >> rmatrix_vec;}

QdNObject_var QdRMatrixSeq::negate() const
{ const RMatrixSeq& rmatrixseq=*this;
  return QdRMatrixSeq_var(new QdRMatrixSeq(-rmatrixseq));}

QdNObject_var QdRMatrixSeq::add(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this+*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this+*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this+*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this+*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdRMatrixSeq::sub(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this-*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this-*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this-*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this-*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrixseq));
    throw BadNArg();
}


QdNObject_var QdRMatrixSeq::mult(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this**rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this**rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
    { 
      QdRMatrixSeq_var res=new QdRMatrixSeq(*this**rmatrix);
      if(res->is_scalar_seq())
        return new QdRScalarSeq(*res);
      else  
        return res;
    }
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
    {
      QdRMatrixSeq_var res=new QdRMatrixSeq(*this**rmatrixseq);
      if(res->is_scalar_seq())
        return new QdRScalarSeq(*res);
      else  
        return res;
    }
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
    { 
      QdCMatrixSeq_var res=new QdCMatrixSeq(*this**cmatrix);
      if(res->is_scalar_seq())
        return new QdCScalarSeq(*res);
      else  
        return res;
    }
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
    {
      QdCMatrixSeq_var res=new QdCMatrixSeq(*this**cmatrixseq);
      if(res->is_scalar_seq())
        return new QdCScalarSeq(*res);
      else  
        return res;
    }
    throw BadNArg();
}

QdNObject_var QdRMatrixSeq::div(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this/(*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this/(*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this/(*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(*this/(*rmatrixseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cscalarseq)));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrix)));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdRMatrixSeq::abs() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::abs(*this)));}

QdNObject_var QdRMatrixSeq::acos() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::acos(*this)));}

QdNObject_var QdRMatrixSeq::asin() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::asin(*this)));}

QdNObject_var QdRMatrixSeq::atan() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::atan(*this)));}

QdNObject_var QdRMatrixSeq::atan2(QdNObject_var obj) const
{ 
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::atan2(*this,*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::atan2(*this,*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::atan2(*this,*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::atan2(*this,*rmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdRMatrixSeq::cos() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::cos(*this)));}

QdNObject_var QdRMatrixSeq::cosh() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::cosh(*this)));}

QdNObject_var QdRMatrixSeq::exp() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::exp(*this)));}

QdNObject_var QdRMatrixSeq::inner(QdNObject_var obj) const
{
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(QDOS::dot(*this,*rmatrixseq)));
        
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRScalarSeq_var(new QdRScalarSeq(QDOS::dot(*this,*rmatrix)));
        
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::dot(*this,*cmatrixseq)));
     
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::dot(*this,*cmatrix)));
    
    throw BadNArg();
}

QdNObject_var QdRMatrixSeq::invert() const
{
    QdNObject_var obj_out;
    if(empty())
        return obj_out;
    else if(is_homogeneous_seq())
    {
        size_t ndims=elt(0).rank();
        if(ndims==0 || ndims==1)
            return QdRScalarSeq_var(new QdRScalarSeq(QDOS::pinverse(*this)));
        else
            return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::pinverse(*this)));
    }
    else       
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::pinverse(*this)));
}



QdNObject_var QdRMatrixSeq::log() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::log(*this)));}

QdNObject_var QdRMatrixSeq::log10() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::log10(*this)));}

QdNObject_var QdRMatrixSeq::mod() const
{ return QdRScalarSeq_var(new QdRScalarSeq(QDOS::norm(*this)));}

QdNObject_var QdRMatrixSeq::outer(QdNObject_var obj) const
{
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::cross(*this,*rmatrixseq)));
        
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::cross(*this,*rmatrix)));
        
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::cross(*this,*cmatrixseq)));
     
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::cross(*this,*cmatrix)));
    
    throw BadNArg();
}

QdNObject_var QdRMatrixSeq::pow(QdNObject_var obj) const
{ 
    QdNObject_var obj_out;
    if(empty())
      return obj_out;
    size_t ndims=elt(0).rank();
    bool scalar_result=\
        (is_homogeneous_seq() && (ndims==0 || ndims==1));
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
    {
        if(scalar_result)
            return QdRScalarSeq_var(\
                new QdRScalarSeq(this->operator^(*rscalar)));
        else
            return QdRMatrixSeq_var(\
                new QdRMatrixSeq(this->operator^(*rscalar)));
    }
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
    {
        if(scalar_result)
            return QdRScalarSeq_var(\
                new QdRScalarSeq(this->operator^(*rscalarseq)));
        else
            return QdRMatrixSeq_var(\
                new QdRMatrixSeq(this->operator^(*rscalarseq)));
    }
    throw BadNArg();
}

QdNObject_var QdRMatrixSeq::sin() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::sin(*this)));}

QdNObject_var QdRMatrixSeq::sinh() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::sinh(*this)));}

QdNObject_var QdRMatrixSeq::sqrt() const
{ return QdRMatrixSeq_var(new QdRMatrixSeq(QDOS::sqrt(*this)));}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// QdCMatrixSeq Numeric Operators/Functions
//------------------------------------------------------------------------------
void QdCMatrixSeq::put(Vostream& v_os)
{ const vector<CMatrix>& cmatrix_vec=*this;v_os << cmatrix_vec;}

void QdCMatrixSeq::get(Vistream& v_is)
{ vector<CMatrix>& cmatrix_vec=*this; v_is >> cmatrix_vec;}

QdNObject_var QdCMatrixSeq::negate() const
{ const CMatrixSeq& cmatrixseq=*this; 
  return QdCMatrixSeq_var(new QdCMatrixSeq(-cmatrixseq));}

QdNObject_var QdCMatrixSeq::add(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this+*cmatrixseq));
    throw BadNArg();
}

QdNObject_var QdCMatrixSeq::sub(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*rmatrix));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*rmatrixseq));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrix));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this-*cmatrixseq));
    throw BadNArg();
}


QdNObject_var QdCMatrixSeq::mult(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**rscalar));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**rscalarseq));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
    { 
      QdCMatrixSeq_var res=new QdCMatrixSeq(*this**rmatrix);
      if(res->is_scalar_seq())
        return new QdCScalarSeq(*res);
      else  
        return res;
    }
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
    {
      QdCMatrixSeq_var res=new QdCMatrixSeq(*this**rmatrixseq);
      if(res->is_scalar_seq())
        return new QdCScalarSeq(*res);
      else  
        return res;
    }
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cscalar));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this**cscalarseq));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
    { 
      QdCMatrixSeq_var res=new QdCMatrixSeq(*this**cmatrix);
      if(res->is_scalar_seq())
        return new QdCScalarSeq(*res);
      else  
        return res;
    }
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
    {
      QdCMatrixSeq_var res=new QdCMatrixSeq(*this**cmatrixseq);
      if(res->is_scalar_seq())
        return new QdCScalarSeq(*res);
      else  
        return res;
    }
    throw BadNArg();
}

QdNObject_var QdCMatrixSeq::div(QdNObject_var obj) const
{
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*rscalar)));
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*rscalarseq)));
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*rmatrix)));
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*rmatrixseq)));
    QdCScalar_var cscalar=QdCScalar_var::narrow(obj);
    if(!cscalar.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cscalar)));
    QdCScalarSeq_var cscalarseq=QdCScalarSeq_var::narrow(obj);
    if(!cscalarseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cscalarseq)));
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrix)));
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(*this/(*cmatrixseq)));
    throw BadNArg();
}

QdNObject_var QdCMatrixSeq::abs() const
{ return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::abs(*this)));}

QdNObject_var QdCMatrixSeq::cos() const
{ return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::cos(*this)));}

QdNObject_var QdCMatrixSeq::cosh() const
{ return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::cosh(*this)));}

QdNObject_var QdCMatrixSeq::exp() const
{ return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::exp(*this)));}

QdNObject_var QdCMatrixSeq::inner(QdNObject_var obj) const
{
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::dot(*this,*rmatrixseq)));
        
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::dot(*this,*rmatrix)));
        
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::dot(*this,*cmatrixseq)));
     
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCScalarSeq_var(new QdCScalarSeq(QDOS::dot(*this,*cmatrix)));
    
    throw BadNArg();
}

QdNObject_var QdCMatrixSeq::invert() const
{
    QdNObject_var obj_out;
    if(empty())
        return obj_out;
    else if(is_homogeneous_seq())
    {
        size_t ndims=elt(0).rank();
        if(ndims==0 || ndims==1)
            return QdCScalarSeq_var(new QdCScalarSeq(QDOS::pinverse(*this)));
        else
            return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::pinverse(*this)));
    }
    else       
        return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::pinverse(*this)));
}


QdNObject_var QdCMatrixSeq::log() const
{ return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::log(*this)));}

QdNObject_var QdCMatrixSeq::log10() const
{ return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::log10(*this)));}

QdNObject_var QdCMatrixSeq::mod() const
{ return QdCScalarSeq_var(new QdCScalarSeq(QDOS::norm(*this)));}

QdNObject_var QdCMatrixSeq::outer(QdNObject_var obj) const
{
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(!rmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::cross(*this,*rmatrixseq)));
        
    QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
    if(!rmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::cross(*this,*rmatrix)));
        
    QdCMatrixSeq_var cmatrixseq=QdCMatrixSeq_var::narrow(obj);
    if(!cmatrixseq.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::cross(*this,*cmatrixseq)));
     
    QdCMatrix_var cmatrix=QdCMatrix_var::narrow(obj);
    if(!cmatrix.is_nil())
        return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::cross(*this,*cmatrix)));
    
    throw BadNArg();
}


QdNObject_var QdCMatrixSeq::pow(QdNObject_var obj) const
{ 
    QdNObject_var obj_out;
    if(empty())
      return obj_out;
    size_t ndims=elt(0).rank();
    bool scalar_result= (is_homogeneous_seq() && (ndims==0 || ndims==1));
    QdRScalar_var rscalar=QdRScalar_var::narrow(obj);
    if(!rscalar.is_nil())
    {
        if(scalar_result)
            return QdCScalarSeq_var(new QdCScalarSeq(this->operator^(*rscalar)));
        else
            return QdCMatrixSeq_var( new QdCMatrixSeq(this->operator^(*rscalar)));
    }
    QdRScalarSeq_var rscalarseq=QdRScalarSeq_var::narrow(obj);
    if(!rscalarseq.is_nil())
    {
        if(scalar_result)
            return QdCScalarSeq_var(  new QdCScalarSeq(this->operator^(*rscalarseq)));
        else
            return QdCMatrixSeq_var(  new QdCMatrixSeq(this->operator^(*rscalarseq)));
    }
    throw BadNArg();
}

QdNObject_var QdCMatrixSeq::sin() const
{ return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::sin(*this)));}

QdNObject_var QdCMatrixSeq::sinh() const
{ return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::sinh(*this)));}

QdNObject_var QdCMatrixSeq::sqrt() const
{ return QdCMatrixSeq_var(new QdCMatrixSeq(QDOS::sqrt(*this)));}



}// end namespace QDOS
