#include "qprofile.h"
#include <iostream>

using namespace std;

QProfilesList * QSAS_profiles;

QString addTokens(const QString& in)
{
#ifndef WINDOWS
	QString home=QString(getenv("HOME"));
#else
	QString home=QString(getenv("USERPROFILE"));
#endif
	QString qsas_home=QString(getenv("QSAS_HOME"));
	
	QString res=in;
	res.replace(qsas_home, QString("${QSAS_HOME}"));
	res.replace(home, QString("${HOME}"));
	return res;
}

QString removeTokens(const QString& in)
{
#ifndef WINDOWS
	QString home=QString(getenv("HOME"));
#else
	QString home=QString(getenv("USERPROFILE"));
#endif
	QString qsas_home=QString(getenv("QSAS_HOME"));
	
	QString res=in;
	res.replace(QString("${QSAS_HOME}"), qsas_home);
	res.replace(QString("${HOME}"), home);
	return res;
}

QProfile::QProfile(QString name)
{
	this->name=name;
	
#ifndef WINDOWS
	char * home=getenv("HOME");
	char * qsas_home=getenv("QSAS_HOME");
	
	SaveRestorePath=QString(qsas_home).append("/tmp/SampleSession.qss");
	UserLibPath=QString(home).append("/MyQsas/lib");
	ConfigPath=QString(qsas_home).append("/examples");
	BlackBoxPath=QString(qsas_home).append("/black_boxes");
	DataPath=QString(home);
	FilterBankPath=QString(qsas_home).append("/examples/FilterBank");
#else
	char * windir=getenv("WINDIR");
	char * home=getenv("USERPROFILE");
	char * qsas_home=getenv("QSAS_HOME");
	
	SaveRestorePath=QString(qsas_home).append("\\tmp\\SampleSession.qss");
	UserLibPath=QString(home).append("\\MyQsas\\lib");
	ConfigPath=QString(qsas_home).append("\\examples");
	DataPath=QString(home);
	FilterBankPath=QString(qsas_home).append("\\examples\\FilterBank");
	BlackBoxPath=QString(qsas_home).append("\\black_boxes");
#endif
	
	Vector3D="R_XYZ";
	Array3Dto0D="SSS";
	Array3Dto1D="iSS";
	Array3Dto2D="iiS";
	PageSize="A4";
	PageOrientation="Landscape";
	PageDPI=150;
	PageWidth=8.; //inches
	PageHeight=11.694;
	TimeRep="HMS";
	UseKPrinter=0;
	SansFont="";
	SerifFont="";
	MonotypeFont="";
	DecorativeFont="";
	VectoriseSVGFonts=false;
// 	ColourTable="HSV(0,260,1,1)(.1,260,1,.5)(.9,0,1,.5)(1,0,1,0)";
	LineWidth=1.;
	BaseFontSize=0.5;
    gapTolerance=1.5;
    gapToleranceType=1;
    axisScaleType=0;
	
	Height=40;
	QpuiHeight=700;
	QpuiWidth=1050;
	fontSize=DEFAULT_FONT_SIZE;
	secDecimal=DEFAULT_DECIMAL_SEC;
	gregorian_start=DEFAULT_GREGORIAN_START;
	GUImargins=4; // Value for spacing and margins
	slotLR=0; // default, R
	timeIntervalButton=0;

    palette.clear();
    QVector<double> points;
    points << 0. << 0.1 << 0.9 << 1.;
    QVector<double> H;
    H << 260. << 260. << 0. << 0.;
    QVector<double> S;
    S << 1. << 1. << 1. << 1.;
    QVector<double> V;
    V << 1. << 0.5 << 0.5 << 0.;
    ColourTable table("Rainbow2", points, H, S, V, false);
    palette.push_back(table);
    
	Debug=0;
	QrtnDebug=0;
	dbgWL=0;
}

QProfile::QProfile(QProfile * p)
{
	this->name=p->name;
	
	SaveRestorePath=p->SaveRestorePath;
	UserLibPath=p->UserLibPath;
	ConfigPath=p->ConfigPath;
	DataPath=p->DataPath;
	FilterBankPath=p->FilterBankPath;
	BlackBoxPath=p->BlackBoxPath;
	
	Vector3D=p->Vector3D;
	Array3Dto0D=p->Array3Dto0D;
	Array3Dto1D=p->Array3Dto1D;
	Array3Dto2D=p->Array3Dto2D;
	PageSize=p->PageSize;
	PageDPI=p->PageDPI;
	PageOrientation=p->PageOrientation;
	PageWidth=p->PageWidth;
	PageHeight=p->PageHeight;
	TimeRep=p->TimeRep;
	UseKPrinter=p->UseKPrinter;
	SansFont=p->SansFont;
	SerifFont=p->SerifFont;
	MonotypeFont=p->MonotypeFont;
	DecorativeFont=p->DecorativeFont;
	VectoriseSVGFonts=p->VectoriseSVGFonts;
// 	ColourTable=p->ColourTable;
	LineWidth=p->LineWidth;
	BaseFontSize=p->BaseFontSize;
    gapTolerance=p->gapTolerance;
    gapToleranceType=p->gapToleranceType;
    axisScaleType=p->axisScaleType;
    palette.clear();
    foreach(TOTO table, p->palette)
    {
        palette.push_back(table);
    }
	
	Height=p->Height;
	QpuiHeight=p->QpuiHeight;
	QpuiWidth=p->QpuiWidth;
	fontSize=p->fontSize;
	secDecimal=p->secDecimal;
	gregorian_start=p->gregorian_start.toLatin1();
	GUImargins=p->GUImargins;
	slotLR=p->slotLR;
	timeIntervalButton=p->timeIntervalButton;
	
	Debug=p->Debug;
	QrtnDebug=p->QrtnDebug;
	dbgWL=p->dbgWL;
}

QProfilesList::QProfilesList()
{
	defaultProfile=new QProfile;
	currentName=defaultProfile->getName();
}

QProfilesList::~QProfilesList()
{
	for(int i=0; i<presets.size(); i++)
		delete presets[i];
	presets.clear();
	
	delete defaultProfile;
}

bool QProfilesList::load()
{

#ifndef WINDOWS
	char * home=getenv("HOME");
	string file_name=string(home).append("/.qsasrc");
#else
	char * home=getenv("USERPROFILE");
	string file_name=string(home).append("\\qsas.ini");
#endif
	ifstream file(file_name.c_str());
	
	QProfile * currentProfile=NULL;
	
	char tag[50];
	char myStr[500];
	QString myQStr;
	int myInt;
	float myFloat;
	double myDouble;
	bool myBool;
	while (file.good())
	{
		file >> tag;
		if(strcmp(tag, "CURRENT_PROFILE")==0)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentName=myQStr;
		}
		else if(strcmp(tag, "PROFILE")==0)
		{
			file.getline(myStr, 500);
			if(currentProfile!=NULL)
			{
				add(currentProfile);
			}
			myQStr=QString(myStr).trimmed();
			currentProfile=fromName(myQStr);
			if(currentProfile==NULL)
            {
				currentProfile=new QProfile(myQStr);
                currentProfile->palette.clear();
            }
		}
		else if(strcmp(tag, "3D_VECTOR")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->Vector3D=myQStr;
		}
		else if(strcmp(tag, "3D_ARRAY_TO_1D")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->Array3Dto1D=myQStr;
		}
		else if(strcmp(tag, "3D_ARRAY_TO_0D")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->Array3Dto0D=myQStr;
		}
		else if(strcmp(tag, "3D_ARRAY_TO_2D")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->Array3Dto2D=myQStr;
		}
		else if(strcmp(tag, "PAGE_SIZE")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->PageSize=myQStr;
		}
		else if(strcmp(tag, "PAGE_ORIENTATION")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->PageOrientation=myQStr;
		}
		else if(strcmp(tag, "PAGE_WIDTH")==0 && currentProfile!=NULL)
		{
			file >> myDouble;
			currentProfile->PageWidth=myDouble;
		}
		else if(strcmp(tag, "PAGE_HEIGHT")==0 && currentProfile!=NULL)
		{
			file >> myDouble;
			currentProfile->PageHeight=myDouble;
		}
		else if(strcmp(tag, "PAGE_DPI")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->PageDPI=myInt;
		}
		else if(strcmp(tag, "TIME_REPRESENTATION")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->TimeRep=myQStr;
		}
		else if(strcmp(tag, "USE_KPRINTER")==0 && currentProfile!=NULL)
		{
			file >> myBool;
			currentProfile->UseKPrinter=myBool;
		}
		else if(strcmp(tag, "SANS_FONT")==0)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			if(myQStr==QString("_NONE_")) myQStr="";
			currentProfile->SansFont=myQStr;
		}
		else if(strcmp(tag, "SERIF_FONT")==0)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			if(myQStr==QString("_NONE_")) myQStr="";
			currentProfile->SerifFont=myQStr;
		}
		else if(strcmp(tag, "MONOTYPE_FONT")==0)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			if(myQStr==QString("_NONE_")) myQStr="";
			currentProfile->MonotypeFont=myQStr;
		}
		else if(strcmp(tag, "DECORATIVE_FONT")==0)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			if(myQStr==QString("_NONE_")) myQStr="";
			currentProfile->DecorativeFont=myQStr;
		}
		else if(strcmp(tag, "VECTORISE_SVG_FONTS")==0 && currentProfile!=NULL)
		{
			file >> myBool;
			currentProfile->VectoriseSVGFonts=myBool;
		}
		else if(strcmp(tag, "COLOUR_TABLE")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			if(myQStr!=QString("_NONE_"))
            {
                ColourTable table;
                table.fromString(myQStr);
                currentProfile->palette.push_back(table);
            }
	}
		else if(strcmp(tag, "LINE_WIDTH")==0 && currentProfile!=NULL)
		{
			file >> myDouble;
			currentProfile->LineWidth=myDouble;
		}
		else if(strcmp(tag, "BASE_FONT_SIZE")==0 && currentProfile!=NULL)
		{
			file >> myDouble;
			currentProfile->BaseFontSize=myDouble;
		}
        else if(strcmp(tag, "GAP_TOLERANCE")==0 && currentProfile!=NULL)
        {
            file >> myDouble;
            currentProfile->gapTolerance=myDouble;
        }
        else if(strcmp(tag, "GAP_TOLERANCE_TYPE")==0 && currentProfile!=NULL)
        {
            file >> myInt;
            currentProfile->gapToleranceType=myInt;
        }
		else if(strcmp(tag, "AXIS_SCALE_TYPE")==0 && currentProfile!=NULL)
        {
            file >> myInt;
            currentProfile->axisScaleType=myInt;
        }
		else if(strcmp(tag, "SAVE_RESTORE")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->SaveRestorePath=removeTokens(myQStr);
		}
		else if(strcmp(tag, "USER_LIB")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->UserLibPath=removeTokens(myQStr);
		}
		else if(strcmp(tag, "CONFIG")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->ConfigPath=removeTokens(myQStr);
		}
		else if(strcmp(tag, "DATA")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->DataPath=removeTokens(myQStr);
		}
		else if(strcmp(tag, "FILTER_BANK")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->FilterBankPath=removeTokens(myQStr);
		}
		else if(strcmp(tag, "BLACK_BOXES")==0 && currentProfile!=NULL)
		{
			file.getline(myStr, 500);
			myQStr=QString(myStr).trimmed();
			currentProfile->BlackBoxPath=removeTokens(myQStr);
		}
		else if(strcmp(tag, "HEIGHT")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->Height=myInt;
		}
		else if(strcmp(tag, "PLOT_UI_HEIGHT")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->QpuiHeight=myInt;
		}
		else if(strcmp(tag, "PLOT_UI_WIDTH")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->QpuiWidth=myInt;
		}
		else if(strcmp(tag, "FONT_SIZE")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->fontSize=myInt;
		}
		else if(strcmp(tag, "GUI_MARGINS")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->GUImargins=myInt;
		}
		else if(strcmp(tag, "SLOT_LR")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->slotLR=myInt;
		}
		else if(strcmp(tag, "TIME_INTERVAL_BUTTON")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->timeIntervalButton=myInt;
		}
		else if(strcmp(tag, "DEBUG")==0 && currentProfile!=NULL)
		{
			file >> myBool;
			currentProfile->Debug=myBool;
		}
		else if(strcmp(tag, "QRTN_DEBUG")==0 && currentProfile!=NULL)
		{
			file >> myBool;
			currentProfile->QrtnDebug=myBool;
		}
		else if(strcmp(tag, "DEBUG_SHOW_TEMP")==0 && currentProfile!=NULL)
		{
			file >> myBool;
			currentProfile->dbgWL=myBool;
		}
		else if(strcmp(tag, "SEC_DECIMAL")==0 && currentProfile!=NULL)
		{
			file >> myInt;
			currentProfile->secDecimal=myInt;
		}
		else if(strcmp(tag, "GREGORIAN_START")==0 && currentProfile!=NULL)
		{
			file >> myStr;
			currentProfile->gregorian_start=myStr;
		}
		else
		{
			file.getline(myStr, 500);
		}
		tag[0] = '\0'; // empty to avoid replacing last entry with blank

	}
	
	if(currentProfile!=NULL) add(currentProfile);

	file.close();
	QProfile* test=current();
	if(test==NULL) currentName=defaultProfile->getName();

	return true;
}

bool QProfilesList::save()
{
#ifndef WINDOWS
	char * home=getenv("HOME");
	string file_name=string(home).append("/.qsasrc");
#else
	char * home=getenv("USERPROFILE");
	string file_name=string(home).append("\\qsas.ini");
#endif
	ofstream file(file_name.c_str());

	if(!file.good()) return false;
	
	file << "CURRENT_PROFILE " << currentName.toLatin1().data() << endl;
	
	file << endl;

	for(int i=0; i<presets.size(); ++i) // we do not save default
	{
		QProfile * prof;
		if(i<presets.size())
		{
			prof=presets[i];
		}
		
		if(prof!=NULL)
		{
			file << "PROFILE " << prof->getName().toLatin1().data() << endl;
			file << "3D_VECTOR " << prof->Vector3D.toLatin1().data() << endl;
			file << "3D_ARRAY_TO_1D " << prof->Array3Dto1D.toLatin1().data() << endl;
			file << "3D_ARRAY_TO_0D " << prof->Array3Dto0D.toLatin1().data() << endl;
			file << "3D_ARRAY_TO_2D " << prof->Array3Dto2D.toLatin1().data() << endl;
			file << "PAGE_SIZE " << prof->PageSize.toLatin1().data() << endl;
			file << "PAGE_DPI " << prof->PageDPI << endl;
			file << "PAGE_ORIENTATION " << prof->PageOrientation.toLatin1().data() << endl;
			file << "PAGE_WIDTH " << prof->PageWidth << endl;
			file << "PAGE_HEIGHT " << prof->PageHeight << endl;
			file << "TIME_REPRESENTATION " << prof->TimeRep.toLatin1().data() << endl;
			file << "USE_KPRINTER " << prof->UseKPrinter << endl;
			if(!prof->SansFont.isEmpty())
			{
				file << "SANS_FONT "<< prof->SansFont.toLatin1().data() << endl;
			}
			else
			{
				file << "SANS_FONT _NONE_" << endl;
			}
			if(!prof->SerifFont.isEmpty())
			{
				file << "SERIF_FONT "<< prof->SerifFont.toLatin1().data() << endl;
			}
			else
			{
				file << "SERIF_FONT _NONE_" << endl;
			}
			if(!prof->MonotypeFont.isEmpty())
			{
				file << "MONOTYPE_FONT "<< prof->MonotypeFont.toLatin1().data() << endl;
			}
			else
			{
				file << "MONOTYPE_FONT _NONE_" << endl;
			}
			if(!prof->DecorativeFont.isEmpty())
			{
				file << "DECORATIVE_FONT "<< prof->DecorativeFont.toLatin1().data() << endl;
			}
			else
			{
				file << "DECORATIVE_FONT _NONE_" << endl;
			}
			file << "VECTORISE_SVG_FONTS " << prof->VectoriseSVGFonts << endl;
// 			if(!prof->ColourTable.isEmpty())
// 			{
// 				file << "COLOUR_TABLE "<< prof->ColourTable.toLatin1().data() << endl;
// 			}
// 			else
// 			{
// 				file << "COLOUR_TABLE _NONE_" << endl;
// 			}
            foreach(ColourTable table, prof->palette)
            {
                file << "COLOUR_TABLE "<< table.toString().toLatin1().data() << endl;
            }
			file << "LINE_WIDTH " << prof->LineWidth << endl;
			file << "BASE_FONT_SIZE " << prof->BaseFontSize << endl;
            file << "GAP_TOLERANCE " << prof->gapTolerance << endl;
            file << "GAP_TOLERANCE_TYPE " << prof->gapToleranceType << endl;
            file << "AXIS_SCALE_TYPE " << prof->axisScaleType << endl;
			file << "SAVE_RESTORE " << addTokens(prof->SaveRestorePath).toLatin1().data() << endl;
			file << "USER_LIB " << addTokens(prof->UserLibPath).toLatin1().data() << endl;
			file << "CONFIG " << addTokens(prof->ConfigPath).toLatin1().data() << endl;
			file << "FILTER_BANK " << addTokens(prof->FilterBankPath).toLatin1().data() << endl;
			file << "BLACK_BOXES " << addTokens(prof->BlackBoxPath).toLatin1().data() << endl;
			file << "HEIGHT " << prof->Height << endl;
			file << "PLOT_UI_HEIGHT " << prof->QpuiHeight << endl;
			file << "PLOT_UI_WIDTH " << prof->QpuiWidth << endl;
			file << "FONT_SIZE " << prof->fontSize << endl;
			file << "SEC_DECIMAL " << prof->secDecimal << endl;
			file << "GREGORIAN_START " << prof->gregorian_start.toLatin1().data() << endl;
			file << "GUI_MARGINS " << prof->GUImargins << endl;
			file << "SLOT_LR " << prof->slotLR << endl;
			file << "TIME_INTERVAL_BUTTON " << prof->timeIntervalButton << endl;
			file << "DEBUG " << prof->Debug << endl;
			file << "QRTN_DEBUG " << prof->QrtnDebug << endl;
			file << "DEBUG_SHOW_TEMP " << prof->dbgWL << endl;
			file << "DATA " << addTokens(prof->DataPath).toLatin1().data() << endl;
			file << endl;
		}
	}
	file.close();

	
	return true;
}

QProfile * QProfilesList::current()
{
	QProfile* current=fromName(currentName);
	if(current==NULL)
	{
		return defaultProfile;
	}
	return current;
}

QProfile * QProfilesList::fromName(const QString& name)
{
	if(name==defaultProfile->getName())
	{
		return defaultProfile;
	}
	
	QProfile * res=NULL;
	for(int i=0; i<presets.size() && res==NULL; ++i)
	{
		if(presets[i]->getName()==name) res=presets[i];
	}
	return res;
}

bool QProfilesList::setCurrent(const QString& name)
{
	if(name==defaultProfile->getName())
	{
		currentName=name;
		return true;
	}
	
	for(int i=0; i<presets.size(); ++i)
	{
		if(presets[i]->getName()==name)
		{
			currentName=name;
			return true;
		}
	}
	return false;
}

void QProfilesList::names(vector<QString>& names)
{
	names.clear();
	names.push_back(defaultProfile->getName());
	for(int i=0; i<presets.size(); ++i)
	{
		names.push_back(presets[i]->getName());
	}
}

bool QProfilesList::add(QProfile* prof)
{
	QProfile* res=fromName(prof->getName());
	
	if(res)
	{
		return false;
	}
	
	presets.push_back(prof);
	
	return true;
}


bool QProfilesList::remove(const QString& name)
{
	bool found=false;
	int i, index;
	
	for(i=0; i<presets.size() && !found; ++i)
	{
		found=(presets[i]->getName()==name);
		index=i;
	}


	if(found)
	{
		vector<QProfile*>::iterator iter=presets.begin()+index;
		delete presets[index];
		presets.erase(iter);
		if(name==currentName) currentName=defaultProfile->getName();
	}
	
	return found;
}



