// PURE Qt4, no compatibility layer

#include "qsw_helpwindow.h"
#include "qsas_version.h"

#include <QToolBar>
#include <QStyle>
#include <QMenuBar>
#include <QMessageBox>
#include <cstdio>

using namespace std;

QswHelpWindow::QswHelpWindow( QWidget *parent, const char *name )
    : QMainWindow(parent)
{
    //Set up the menu
    QMenuBar* qmb_menu_bar = this->menuBar();

    QMenu * file_menu=qmb_menu_bar->addMenu(tr("&File"));
    QAction * qact_close=new QAction(tr("&Close"), this);
    connect(qact_close, SIGNAL(triggered()), this, SLOT(hide()));
    file_menu->addAction(qact_close);

    QMenu* help_menu=qmb_menu_bar->addMenu(tr("&Help"));
    QAction * qact_help=new QAction(tr("&About"), this);
    connect(qact_help, SIGNAL(triggered()), this, SLOT(about()));
    help_menu->addAction(qact_help);

    //Set up the toolbar
    
    QToolBar * tool_bar = new QToolBar(this);
    
    QAction * qact_back=new QAction(style()->standardIcon(QStyle::SP_ArrowBack, 0, this), "Back", this);
    connect(qact_back, SIGNAL(triggered()), this, SLOT(back()));
    tool_bar->addAction(qact_back);
    
    tool_bar->addSeparator();
    
    QAction * qact_index=new QAction(style()->standardIcon(QStyle::SP_ArrowUp, 0, this), "Index", this);
    connect(qact_index, SIGNAL(triggered()), this, SLOT(goIndex()));
    tool_bar->addAction(qact_index);
    
    addToolBar(tool_bar);
   
    // Text browser for simple html help files

    help_browser = new QTextBrowser(this);

    // also root directory stuff
    const char * home = getenv("QSAS_HOME");
#ifdef WINDOWS
	if(home==NULL) help_root_dir = "C:/Program Files/doc/QSAShelp/"; // Safe: QTextBrowser warns user
    else help_root_dir = string(home)+"/doc/QSAShelp/";

	// change Windows '\' in QSAS_HOME env to C '/' 
	size_t backslash;
	while( (backslash=help_root_dir.find("\\")) != string::npos){
		help_root_dir.replace(backslash, 1, 1, '/');
	}
	// now replace ' ' with '\ '
	if( (backslash=help_root_dir.find("\ ")) != string::npos){
		help_root_dir.replace(backslash, 1, 1, '\\ ');
	}
#else
	if(home==NULL) help_root_dir = "doc/QSAShelp/"; // Safe: QTextBrowser warns user
    else help_root_dir = string(home)+"/doc/QSAShelp/";
#endif   

	QStringList searchPaths;
	searchPaths.append(help_root_dir.c_str());
	help_browser->setSearchPaths(searchPaths);
	
    this->setCentralWidget( help_browser );
    
    resize( 800, 600 );
}

void QswHelpWindow::about()
{
    string about_txt("About QSAS version " + string(QSAS_VERSION_ID));
    about_txt+="\n\nThe QM Space Analysis System delivers a "
                "flexible data selection, manipulation and display \n"
                "environment tailored to the needs of space physics "
                "in general and Cluster in particular.\n\n"
                "The QSAS team is now at Imperial College, \n"
                "use csc-support-dll@imperial.ac.uk";
    QMessageBox::about(this,"About QSAS",about_txt.c_str());  
}

void QswHelpWindow::goIndex()
{ 
    manEntry("index.html");
}

void QswHelpWindow::manEntry(const string &entry, const string &anchor)
{ 
    FILE *fp;
    fp = fopen((help_root_dir+entry).c_str(), "r");
    if ( fp == NULL ){  
        QMessageBox::information( this, "Documentation not available", QString("Cannot find: ")+QString((help_root_dir+entry).c_str()), "OK" );    
    }
    else{
        fclose (fp);
        help_browser->setSource( QUrl((entry+anchor).c_str()) ); // ::fromLocalFile  removed
        this->show();
    }

}


void QswHelpWindow::back()
{
    help_browser->backward();
}

